/*
 * bspline3.h
 * 
 * Copyright 2013 Vova <vova@lexx>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 * 
 */

#ifndef BSPLINE3_H
#define BSPLINE3_H

#include <vector>

/*	here is an example how to do draw basis functions it in CINT:
	Bspline3 *bs = new Bspline3(0., 10., 10);  // create the user function class
	TF1 * f = new TF1("f", bs, &Bspline3::Basis, 0. ,10. , 1 , "Bspline3", "Basis");
	// now we want to plot 6-th basis function
	f->SetParameter(0, 6);	
	f->Draw();
*/

// coefficients for a uniform 3rd degree B-spline
// (divide by 6 to get right normalization)
//const double BSM[4][4] = {1., 4., 1., 0.,
//                        -3., 0., 3., 0.,
//                        3., -6., 3., 0.,
//                        -1., 3., -3., 1.};

// (GCC tries to be more Catholic than the Pope here
// and wants inner braces not required by the standard)

const double BSM[4][4] = {{1., 4., 1., 0.},
                          {-3., 0., 3., 0.},
                          {3., -6., 3., 0.},
                          {-1., 3., -3., 1.}};

class Bspline3 {
	public:
		Bspline3(double xmin, double xmax, int n_int);
        ~Bspline3();
		void SetRange(double xmin, double xmax) {xl = xmin; xr = xmax; dx = xr-xl;}
		void GetRange(double *xmin, double *xmax) {*xmin = xl;	*xmax = xr;}
        double GetXmin() {return xl;}
        double GetXmax() {return xr;}
		int GetNint() {return nint;}
		int GetNbas() {return nbas;}
// get coeff. at x^pow for interval i		
		double BasisPol(int i, int pow) {return BSM[pow][i]/6.;}
		double Basis(double x, int n);
		double Basis(double *x, double *p); // insertable into a ROOT function
		double BasisDrv(double x, int n);		
		double BasisDrv(double *x, double *p); // insertable into a ROOT function
		double Eval_slow(double x);
        double Eval_greedy(double x);
        double Eval_x_phobic(double x);
		double Eval(double x);
		double Eval(double *x, double *p);	// insertable into a ROOT function
        double EvalDrv(double x);
        double EvalDrv(double *x, double *p);	// insertable into a ROOT function
		void SetCoef(double *c);
		void GetCoef(double *c);
        void SetCoef(std::vector <double> c);
        const std::vector <double> GetCoef() const;
		double GetCoef(int i) {return coef[i];}
        std::vector <std::vector <double> > GetPoly();

	private:
        std::vector <double> coef;
        std::vector <double*> poly;
		double xl;
		double xr;
		double dx;
		int nint;	// number of intervals
		int nbas;	// number of basis splines		
};

#endif /* BSPLINE3_H */ 
