/*
 * tpspline3.h
 * 
 * Copyright 2013 Vova <vova@lexx>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 * 
 */

#ifndef TPSPLINE3_H
#define TPSPLINE3_H

#include "bspline3.h"

class TPspline3
{
	public:
		TPspline3(double xmin, double xmax, int n_intx, double ymin, double ymax, int n_inty);
        ~TPspline3();
        void SetRangeX(double xmin, double xmax);
		void SetRangeY(double ymin, double ymax);
		void GetRangeX(double *xmin, double *xmax) {*xmin = xl;	*xmax = xr;}
        void GetRangeY(double *ymin, double *ymax) {*ymin = yl;	*ymax = yr;}
        double GetXmin() {return xl;}
        double GetXmax() {return xr;}
        double GetYmin() {return yl;}
        double GetYmax() {return yr;}
		int GetNintX() {return nintx;}
		int GetNintY() {return ninty;}
		int GetNbas() {return nbas;}	
		double Basis(double x, double y, int n);
        double BasisDrvX(double x, double y, int n);
        double BasisDrvY(double x, double y, int n);
		double Basis(double *x, double *p);  // insertable into ROOT function
		double Eval_slow(double x, double y);
		double Eval(double x, double y);
        double Eval_greedy(double x, double y);
        double EvalDrvX(double x, double y);
        double EvalDrvY(double x, double y);
		double Eval(double *x, double *p);	// insertable into ROOT function
        double EvalDrvX(double *x, double *p);	// insertable into ROOT function
        double EvalDrvY(double *x, double *p);	// insertable into ROOT function
		void SetCoef(double *c);
		void GetCoef(double *c);
		double GetCoef(int i) {return coef[i];}
        void SetCoef(std::vector <double> c);
        std::vector <double> GetCoef();
        Bspline3 *GetBSX() {return bsx;}
        Bspline3 *GetBSY() {return bsy;}
        void SetWrapY(bool w) {wrapy = w;}
        bool GetWrapY() {return wrapy;}
        int GetWrapped(int n);
			
	private:
		Bspline3 *bsx, *bsy;
        std::vector <double> coef;
        std::vector <double*> poly;
//		double *coef;
//		double **poly;
        double xl, yl;
        double xr, yr;
		double dx, dy;
		int nintx, ninty;
		int nrec;	// number of rectangles
		int nbas;	// number of basis splines
		int nbasx;  // columns in spline grid
        bool wrapy;
};

#endif /* TPSPLINE3_H */ 
