#ifndef PMLOCALCACHE_H
#define PMLOCALCACHE_H

#include <math.h>
#include <vector>

class PMsensor;
class LRF2;
class LRFaxial;
class LRFcAxial;
class LRFxy;
class LRFcomposite;
class LRFaxial3d;
class LRFsliced3D;
struct ReconstructionStruct;
struct ScanStructure;
template<typename T> class QVector;

class SensorLocalCache
{
public:
    SensorLocalCache(int numGoodEvents, bool fDataRecon, bool fScaleByEnergy, const QVector<ReconstructionStruct*> reconData,
                     const QVector<ScanStructure*> *scan, const QVector< QVector <float> > *events);

    ~SensorLocalCache();

    void uncacheGroup();
    bool cacheGroup(const std::vector<PMsensor> *sensors, bool adjustGains, int igrp_);
    void calcRelativeGains2D(int ngrid, unsigned int pmsCount);

    LRF2 *fitLRF(LRF2 *lrf) const;
    LRFaxial *mkLRFaxial(int nodes, double *compr) const;
    LRFxy *mkLRFxy(int nodesx, int nodesy) const;
    LRFcomposite *mkLRFcomposite(int nodesr, int nodesxy, double *compr) const;
    LRFaxial3d *mkLRFaxial3d(int nodesr, int nodesz) const;
    LRFsliced3D *mkLRFsliced3D(int nodesr, int nodesz) const;

    void expandDomain(double fraction);

    const double *getGains() const { return gains; }

    bool fUseGrid;
    bool fFitError;

private:
    int igrp; // Current Group ID  - needed for debugging (VS 20/10/14)
    int numGoodEvents, dataSize;
    const QVector<float> * const *goodEvents;
    const double * const *r;
    const double *factors;
    double *xx, minx, maxx;
    double *yy, miny, maxy;
    double *zz, minz, maxz;
    double *sigsig;
    double *gains;
    double maxr, maxr2;
};

#endif // PMLOCALCACHE_H
