#--------------ANTS2--------------

ANTS2_MAJOR = 3
ANTS2_MINOR = 0
DEFINES += ANTS2_MINOR=\"$$ANTS2_MINOR\"
DEFINES += ANTS2_MAJOR=\"$$ANTS2_MAJOR\"

ANTS2_VERSION = 1380
DEFINES += ANTS2_VERSION=\"$$ANTS2_VERSION\"

#CONFIG += ants2_cuda        #enable CUDA support - need NVIDIA GPU and drivers installed!
#CONFIG += ants2_flann       #enable FLANN (fast neighbour search) library
#CONFIG += ants2_fann        #enables FANN (fast neural network) library

#DEFINES += advanced_photon_tracing #enable to account for photon flight time and collect statistics on photon terminations

#win32:CONFIG   += console   #enable to add standalone console for Windows
win32:DEFINES  +=_CRT_SECURE_NO_WARNINGS  #disable microsoft spam
#DEFINES += WINDOWSBIN       #enable for Windows binary-only

RC_FILE = myapp.rc
win32:QT += winextras  #used in windownavigator only

QT += core gui
QT += widgets
QT += script #scripts support

#TARGET = ants2_v$$ANTS2_VERSION
TARGET = ants2
TEMPLATE = app

win32 {
DEFINES += BUILDTIME=\\\"$$system('echo %time%')\\\"
DEFINES += BUILDDATE=\\\"$$system('echo %date%')\\\"
} else {
DEFINES += BUILDTIME=\\\"$$system(date '+%H:%M.%s')\\\"
DEFINES += BUILDDATE=\\\"$$system(date '+%d/%m/%y')\\\"
}

#---copy EXAMPLES directory from source to working directory---
win32 {
        #todir = $${OUT_PWD}/../EXAMPLES
        todir = $${OUT_PWD}/EXAMPLES
        fromdir = $${PWD}/EXAMPLES
        todir ~= s,/,\\,g
        fromdir ~= s,/,\\,g       
        QMAKE_POST_LINK = $$quote(cmd /c xcopy \"$${fromdir}\" \"$${todir}\" /y /e /i$$escape_expand(\n\t))
      }

linux-g++ {  
   #todir = $${OUT_PWD}/..
   todir = $${OUT_PWD}
   fromdir = $${PWD}/EXAMPLES  
   QMAKE_POST_LINK = $$quote(cp -rf \"$${fromdir}\" \"$${todir}\"$$escape_expand(\n\t))
}

unix {
   #todir = $${OUT_PWD}/..
   todir = $${OUT_PWD}
   fromdir = $${PWD}/EXAMPLES  
   QMAKE_POST_LINK = $$quote(cp -rf \"$${fromdir}\" \"$${todir}\"$$escape_expand(\n\t))
}
#------------

#---CERN ROOT---
win32 {
        INCLUDEPATH += c:/root/include
        LIBS += -Lc:/root/lib/ -llibCore -llibCint -llibRIO -llibNet -llibHist -llibGraf -llibGraf3d -llibGpad -llibTree -llibRint -llibPostscript -llibMatrix -llibPhysics -llibRint -llibMathCore -llibGeom -llibGeomPainter -llibGeomBuilder -llibMathMore -llibMinuit2 -llibThread
}
linux-g++ {
        INCLUDEPATH += $$system(root-config --incdir)                                        #raimundo was here
        LIBS += $$system(root-config --libs) -lGeom -lGeomPainter -lGeomBuilder -lMathMore -lMinuit2   #raimundo was also here
        #QMAKE_CXXFLAGS_RELEASE *= -msse2 -march=core2
}
unix {
        INCLUDEPATH += $$system(root-config --incdir) /usr/local/root/math/minuit2/inc       #raimundo also creeped here
        LIBS += $$system(root-config --libs) -lGeom -lGeomPainter -lGeomBuilder -lMathMore -lMinuit2   #raimundo was everywhere
}
#-----------

#---FLANN---
ants2_flann {
     DEFINES += ANTS_FLANN

     win32 {
        LIBS += -LC:/FLANN/lib -lflann
        INCLUDEPATH += C:/FLANN/include
     }
}
#----------

#---FANN---
ants2_fann {
     DEFINES += ANTS_FANN
     win32 {
        DEFINES += NOMINMAX
        LIBS += -Lc:/FANN-2.2.0-Source/bin/ -lfannfloat
        INCLUDEPATH += c:/FANN-2.2.0-Source/src/include
     }
     unix {
        LIBS += -lfann
     }
     linux-g++ {
        LIBS += -lfann
     }
}
#---------

#---CUDA---
ants2_cuda {
    DEFINES += __USE_ANTS_CUDA__
    CUDA_SOURCES += CUDA/cudaANTS.cu
    OTHER_FILES += CUDA/cudaANTS.cu
 #   OTHER_FILES +=
    CUDA_OBJECTS_DIR = release/cuda
    INCLUDEPATH += CUDA
    NVCC_OPTIONS = --use_fast_math

    LIBS += -lcuda -lcudart #-lcusolver -lcublas -lcublas_device -lcudadevrt -lcufft -lcufftw -lcurand -lcusparse -lnppc -lnppi -lnpps -lnvcuvenc -lnvcuvid

    unix {
        CUDA_DIR = $$system(which nvcc | sed 's,/bin/nvcc$,,')    # Path to cuda toolkit install
        NVCC_BIN = nvcc
        LIBS += -L$$CUDA_DIR/lib64                                # Library path (remove 64 on 32bit)
        debug:NVCC_OPTIONS += -D_DEBUG
        NVCC_COMPILER_FLAGS = -Xcompiler $$QMAKE_CXXFLAGS

        # Configuration of the Cuda compiler
        cuda.input = CUDA_SOURCES
        cuda.output = $$CUDA_OBJECTS_DIR/${QMAKE_FILE_BASE}_cuda.o
        cuda.commands = $$NVCC_BIN $$NVCC_OPTIONS $$NVCC_COMPILER_FLAGS -c -o ${QMAKE_FILE_OUT} ${QMAKE_FILE_NAME}
        cuda.dependency_type = TYPE_C
        QMAKE_EXTRA_COMPILERS += cuda
    }
    win32 {
        CUDA_DIR = "C:/cuda/toolkit"        # Path to cuda toolkit - !!!AVOID SPACES IN THE NAME!!! - when choosing path during installation
        #NVCC_BIN = $$CUDA_DIR/bin/nvcc.exe  # Location of nvcc
#        CUDA_ARCH = sm_13                   # Type of CUDA architecture, for example 'compute_10', 'compute_11', 'sm_10'
        SYSTEM_NAME = Win32                 # Depending on your system either 'Win32' or 'x64'
        SYSTEM_TYPE = 32                    # '32' or '64'; can be '32' on 64bit system
        #debug:NVCC_OPTIONS += -D_DEBUG
        #NVCC_COMPILER_FLAGS += -Xcompiler -MD --machine $$SYSTEM_TYPE -arch=$$CUDA_ARCH
        LIBS += -Lc:/cuda/toolkit/lib/Win32

        #INCLUDEPATH += C:/cuda/toolkit/include

        CONFIG(debug, debug|release){
         # Debug mode
         cuda_d.input = CUDA_SOURCES
         cuda_d.output = $$CUDA_OBJECTS_DIR/${QMAKE_FILE_BASE}_cuda.o
         cuda_d.commands = $$CUDA_DIR/bin/nvcc.exe -D_DEBUG -Xcompiler -MD $$NVCC_OPTIONS $$CUDA_INC $$LIBS --machine $$SYSTEM_TYPE -c -o ${QMAKE_FILE_OUT} ${QMAKE_FILE_NAME}
         cuda_d.dependency_type = TYPE_C
         QMAKE_EXTRA_COMPILERS += cuda_d
        }
        else{
         # Release mode
         cuda.input = CUDA_SOURCES
         cuda.output = $$CUDA_OBJECTS_DIR/${QMAKE_FILE_BASE}_cuda.o  #cuda.output = release/cuda/${QMAKE_FILE_BASE}_cuda.o
         #cuda.commands = $$CUDA_DIR/bin/nvcc.exe -Xcompiler -MD $$NVCC_OPTIONS $$CUDA_INC $$LIBS --machine $$SYSTEM_TYPE -arch=compute_35 -c -o ${QMAKE_FILE_OUT} ${QMAKE_FILE_NAME}
         cuda.commands = $$CUDA_DIR/bin/nvcc.exe -Xcompiler -MD $$NVCC_OPTIONS $$CUDA_INC $$LIBS --machine $$SYSTEM_TYPE -c -o ${QMAKE_FILE_OUT} ${QMAKE_FILE_NAME}
                                                                                                                         #-arch=sm20
         cuda.dependency_type = TYPE_C
         QMAKE_EXTRA_COMPILERS += cuda
        }
     }
}
#----------

SOURCES += main.cpp\
        gui/mainwindow.cpp \
    modules/primaryparticletracker.cpp \
    modules/s1_generator.cpp \
    modules/photon_generator.cpp \
    modules/photon_tracker.cpp \
    modules/s2_generator.cpp \
    common/functions.cpp \
    gui/materialinspectorwindow.cpp \
    gui/outputwindow.cpp \
    gui/guiutils.cpp \
    modules/pms.cpp \
    gui/lrfwindow.cpp \
    gui/reconstructionwindow.cpp \
    LRF/tpspline3.cpp \
    LRF/spline.cpp \
    LRF/bspline.cpp \
    LRF/bspline3.cpp \
#    LRF/bspline3nu.cpp \
    gui/windownavigatorclass.cpp \
    gui/MainWindowTools/MainWindowDiskIO.cpp \
    gui/MainWindowTools/MainWindowPhotonSource.cpp \
    gui/MainWindowTools/MainWindowTracks.cpp \
    gui/neuralnetworkswindow.cpp \
    modules/neuralnetworksmodule.cpp \
    gui/MainWindowTools/MainWindowParticleSimulation.cpp \
    gui/MainWindowTools/MainWindowDetectorConstructor.cpp \
    gui/MainWindowTools/MainWindowMenu.cpp \
    gui/MainWindowTools/MainWindowTests.cpp \
    gui/MainWindowTools/MainWindowInits.cpp \
    gui/ReconstructionWindowTools/Reconstruction_Tracks.cpp \
    gui/exampleswindow.cpp \
    gui/detectoraddonswindow.cpp \   
    gui/checkupwindowclass.cpp \
    gui/gainevaluatorwindowclass.cpp \
    gui/RasterWindow/rasterwindowbaseclass.cpp \
    gui/graphwindowclass.cpp \
    gui/geometrywindowclass.cpp \
    gui/RasterWindow/rasterwindowgraphclass.cpp \
    gui/ReconstructionWindowTools/reconstruction_diskio.cpp \
    gui/ReconstructionWindowTools/Reconstruction_Init.cpp \
    gui/ReconstructionWindowTools/Reconstruction_ResolutionAnalysis.cpp\
    common/jsonparser.cpp \
    CUDA/cudamanagerclass.cpp \
    gui/MainWindowTools/MainWindowParticles.cpp \
    modules/materialcollectionclass.cpp \
    modules/particlesourcesclass.cpp \
    gui/ReconstructionWindowTools/Reconstruction_NN.cpp \
    modules/nnmoduleclass.cpp \
    gui/viewer2darrayobject.cpp \
    modules/flatfield.cpp \
    gui/highlighterclass.cpp \
    gui/shapeablerectitem.cpp \
    gui/genericscriptwindowclass.cpp \
    gui/graphicsruler.cpp \
    LRF2/pmsensor.cpp \
    LRF2/lrf2.cpp \
    modules/sensorlrfs.cpp \
    LRF2/lrfcaxial.cpp \
    LRF2/lrfaxial.cpp \
    LRF2/pmsensorgroup.cpp \
    LRF2/lrfxy.cpp \
    LRF2/lrfcomposite.cpp \
    LRF2/lrfsliced3d.cpp \
    LRF2/lrfaxial3d.cpp \
    LRF2/lrf3d.cpp \
    LRF2/sensorlocalcache.cpp \
    gui/credits.cpp \
    LRF2/lrffactory.cpp \
    modules/geneticreconstruction.cpp \
    modules/oneeventclass.cpp \
    modules/detectionstatisticsclass.cpp \
    modules/eventsdataclass.cpp \
    modules/detectorclass.cpp \
    modules/simulationmanager.cpp \
    common/CorrelationFilters.cpp \
    modules/detectorsandwichclass.cpp \
    common/reconstructionsettings.cpp \
    common/generalsimsettings.cpp \
    common/pmtypeclass.cpp \
    modules/dynamicpassiveshandler.cpp \
    modules/reconstructionmanagerclass.cpp \
    modules/processorclass.cpp \
    common/globalsettingsclass.cpp \
    gui/globalsettingswindowclass.cpp \
    scriptmode/scriptmode.cpp \
    scriptmode/mainscriptinterface.cpp \
    scriptmode/configscriptinterface.cpp \
    scriptmode/scriptfunctions.cpp \
    modules/manifesthandling.cpp \
    modules/scriptfilterclass.cpp \
    gui/ReconstructionWindowTools/reconstruction_scriptfilter.cpp \
    gui/MainWindowTools/interfacetocheckerscript.cpp \
    common/completingtexteditclass.cpp \
    modules/phscatclaudiomodel.cpp \
    common/opticaloverrideclass.cpp \
    gui/MainWindowTools/mainwindowscatterfit.cpp \
    modules/scatteronmetal.cpp \
    gui/MainWindowTools/globalscript.cpp \
    gui/MainWindowTools/interfacetoglobscript.cpp \
    scriptmode/scriptminimizer.cpp \
    modules/scriptinterfaces.cpp \
    common/tmpobjhubclass.cpp

HEADERS  += gui/mainwindow.h \
    modules/primaryparticletracker.h \
    modules/s1_generator.h \
    modules/photon_generator.h \
    modules/photon_tracker.h \
    modules/s2_generator.h \
    common/classes.h \
    common/functions.h \
    gui/materialinspectorwindow.h \
    gui/outputwindow.h \
    gui/guiutils.h \
    modules/pms.h \
    gui/lrfwindow.h \
    gui/reconstructionwindow.h \
    LRF/tpspline3.h \
    LRF/spline.h \
    LRF/bspline.h \
    LRF/bspline3.h \
#    LRF/bspline3nu.h \
    gui/windownavigatorclass.h \
    gui/neuralnetworkswindow.h \
    modules/neuralnetworksmodule.h \
    gui/exampleswindow.h \
    gui/detectoraddonswindow.h \  
    gui/checkupwindowclass.h \
    gui/gainevaluatorwindowclass.h \
    gui/RasterWindow/rasterwindowbaseclass.h \
    gui/graphwindowclass.h \
    gui/geometrywindowclass.h \
    gui/RasterWindow/rasterwindowgraphclass.h \
    common/CorrelationFilters.h\
    common/jsonparser.h \
    CUDA/cudamanagerclass.h \
    modules/materialcollectionclass.h \
    modules/particlesourcesclass.h \
    modules/nnmoduleclass.h \
    gui/viewer2darrayobject.h \
    modules/flatfield.h \
    gui/highlighterclass.h \
    gui/shapeablerectitem.h \
    gui/genericscriptwindowclass.h \
    gui/graphicsruler.h \
    LRF2/pmsensor.h \
    LRF2/lrf2.h \
    modules/sensorlrfs.h \
    LRF2/lrfaxial.h \
    LRF2/lrfcaxial.h \
    LRF2/pmsensorgroup.h \
    LRF2/lrfxy.h \
    LRF2/lrfcomposite.h \
    LRF2/lrfsliced3d.h \
    LRF2/lrfaxial3d.h \
    LRF2/lrf3d.h \
    LRF2/sensorlocalcache.h \
    gui/credits.h \
    LRF2/lrffactory.h \
    modules/geneticreconstruction.h \
    modules/oneeventclass.h \
    modules/detectionstatisticsclass.h \
    modules/eventsdataclass.h \
    modules/detectorclass.h \
    modules/simulationmanager.h \
    modules/detectorsandwichclass.h \
    common/reconstructionsettings.h \
    common/generalsimsettings.h \
    common/pmtypeclass.h \
    modules/dynamicpassiveshandler.h \
    modules/reconstructionmanagerclass.h \
    modules/processorclass.h \
    common/globalsettingsclass.h \
    gui/globalsettingswindowclass.h \
    scriptmode/scriptmode.h \
    scriptmode/mainscriptinterface.h \
    scriptmode/configscriptinterface.h \
    scriptmode/scriptfunctions.h \
    modules/manifesthandling.h \
    modules/scriptfilterclass.h \
    gui/MainWindowTools/interfacetocheckerscript.h \
    common/completingtexteditclass.h \
    modules/phscatclaudiomodel.h \
    common/opticaloverrideclass.h \
    modules/scatteronmetal.h \
    gui/MainWindowTools/interfacetoglobscript.h \
    scriptmode/scriptminimizer.h \
    modules/scriptinterfaces.h \
    common/tmpobjhubclass.h

FORMS    += gui/mainwindow.ui \
    gui/materialinspectorwindow.ui \
    gui/outputwindow.ui \
    gui/lrfwindow.ui \
    gui/reconstructionwindow.ui \
    gui/windownavigatorclass.ui \
    gui/neuralnetworkswindow.ui \
    gui/exampleswindow.ui \
    gui/detectoraddonswindow.ui \
    gui/checkupwindowclass.ui \
    gui/gainevaluatorwindowclass.ui \
    gui/graphwindowclass.ui \
    gui/geometrywindowclass.ui \
    gui/genericscriptwindowclass.ui \
    gui/credits.ui \
    gui/globalsettingswindowclass.ui

INCLUDEPATH += gui
INCLUDEPATH += gui/MainWindowTools
INCLUDEPATH += gui/RasterWindow
INCLUDEPATH += gui/ReconstructionWindowTools
INCLUDEPATH += modules
INCLUDEPATH += LRF
INCLUDEPATH += LRF2
INCLUDEPATH += common
INCLUDEPATH += scriptmode

RESOURCES += \
    Resources.qrc

