#include "CorrelationFilters.h"

void CorrelationFilterStructure::copyFrom(CorrelationFilterStructure *source)
{
  Active = source->Active;
  AutoBin = source->AutoBin;
  BinX = source->BinX;
  BinY = source->BinY;
  AutoSize = source->AutoSize;
  minX = source->minX;
  maxX = source->maxX;
  minY = source->minY;
  maxY = source->maxY;

  setCorrelationUnitX(source->getCorrelationUnitX()->getClone());
  setCorrelationUnitY(source->getCorrelationUnitY()->getClone());
  setCut(source->getCut()->getClone());
}

void CorrelationFilterStructure::saveJSON(QJsonObject &json) const
{
  json["Active"] = Active;
  json["AutoBin"] = AutoBin;
  json["BinX"] = BinX;
  json["BinY"] = BinY;
  json["AutoSize"] = AutoSize;
  json["minX"] = minX;
  json["maxX"] = maxX;
  json["minY"] = minY;
  json["maxY"] = maxY;

  QJsonArray jsonArray;
  QJsonObject tmpX, tmpY;
  X->CorrelationUnitGenericClass::saveJSON(tmpX);
  tmpX["Type"] = X->getType();
  jsonArray.append(tmpX);
  Y->CorrelationUnitGenericClass::saveJSON(tmpY);
  tmpY["Type"] = Y->getType();
  jsonArray.append(tmpY);
  json["XY"] = jsonArray;
  QJsonObject cut;
  Cut->CorrelationCutGenericClass::saveJSON(cut);
  cut["Type"] = Cut->getType();
  json["Cut"] = cut;
}

CorrelationFilterStructure *CorrelationFilterStructure::createFromJson(QJsonObject &json, DetectorClass *detector, EventsDataClass *eventsDataHub)
{
  CorrelationFilterStructure* tmp = new CorrelationFilterStructure();
  parseJson(json, "Active", tmp->Active);
  parseJson(json, "AutoBin", tmp->AutoBin);
  parseJson(json, "BinX", tmp->BinX);
  parseJson(json, "BinY", tmp->BinY);
  parseJson(json, "AutoSize", tmp->AutoSize);
  parseJson(json, "minX", tmp->minX);
  parseJson(json, "maxX", tmp->maxX);
  parseJson(json, "minY", tmp->minY);
  parseJson(json, "maxY", tmp->maxY);

  QJsonObject cutJSON = json["Cut"].toObject();
  CorrelationCutGenericClass* cut;
  QString CutType = cutJSON["Type"].toString();
  if (CutType == "line") cut = new Cut_Line();
  else if (CutType == "ellipse") cut = new Cut_Ellipse();
  else if (CutType == "polygon") cut = new Cut_Polygon();
  else
    {
      qWarning("Unknown cut type!");
      return 0;
    }
  int CutOption;
  parseJson(cutJSON, "CutOption", CutOption);
  cut->CutOption = CutOption;
  QJsonArray jsonArray = cutJSON["Data"].toArray();
  QList<double> Data;
  for (int id=0; id<jsonArray.size(); id++) Data.append(jsonArray[id].toDouble());
  cut->Data = Data;
  // qDebug()<<"Cut:"<<CutType<<CutOption<<Data;
  tmp->setCut(cut);

  QJsonArray jsonXY = json["XY"].toArray();
  QJsonObject jsonX = jsonXY[0].toObject();
  QString type;
  parseJson(jsonX, "Type", type);
  CorrelationUnitGenericClass* X = CorrUnitCreator(type);
  jsonArray = jsonX["Array"].toArray();
  QList<int> Channels;
  for (int i=0; i<jsonArray.size(); i++) Channels.append(jsonArray[i].toInt());
  X->Channels = Channels;
  X->EventsDataHub = eventsDataHub;
  X->Detector = detector;
  // qDebug()<<"X:"<<type<<Channels;

  QJsonObject jsonY = jsonXY[1].toObject();
  parseJson(jsonY, "Type", type);
  CorrelationUnitGenericClass* Y = CorrUnitCreator(type);
  jsonArray = jsonY["Array"].toArray();
  Channels.clear();
  for (int i=0; i<jsonArray.size(); i++) Channels.append(jsonArray[i].toInt());
  Y->Channels = Channels;
  Y->EventsDataHub = eventsDataHub;
  Y->Detector = detector;
  // qDebug()<<"Y:"<<type<<Channels;

  tmp->setCorrelationUnitX(X);
  tmp->setCorrelationUnitY(Y);
  return tmp;
}

CorrelationUnitGenericClass* CorrelationFilterStructure::CorrUnitCreator(QString Type)
{
  if (Type == "SingleChannel") return new CU_SingleChannel();
  if (Type == "SumAllChannels") return new CU_SumAllChannels();
  if (Type == "SumChannels") return new CU_SumChannels();
  if (Type == "LoadedEnergy") return new CU_LoadedEnergy();
  if (Type == "RecE") return new CU_RecE();
  if (Type == "Chi2") return new CU_Chi2();
  if (Type == "RecX") return new CU_RecX();
  if (Type == "RecY") return new CU_RecY();
  if (Type == "RecZ") return new CU_RecZ();

  //if unknown type, returns 0
  return 0;
}
