#ifndef CORRELATIONFILTERS_H
#define CORRELATIONFILTERS_H

#include "classes.h"
#include "functions.h"
#include "pms.h"
#include "detectorclass.h"
#include "eventsdataclass.h"

//=================Correlation filter=======================


class CorrelationUnitGenericClass
{
  public:
    virtual ~CorrelationUnitGenericClass(){}

    virtual const QString getType() {return "undefined";}  //type
    virtual const TString getAxisTitle() {return "undefined";}  //axe title used on correlation histogram
    virtual int getCOBindex() const {return 0;} //index used in ui
    QList<int> Channels; //additional data, e.g. channel number

    ///QVector< QVector< float> >* *Events; //double pointer!
       //QVector<double>* Gains; - cannot be used anymore after Raimundo's changes in PMs module
    ///pms* *PMs;//double pointer!
    ///QVector<ReconstructionStruct*>* ReconstructionData;//pointer to vector of pointers!
    DetectorClass *Detector;
    EventsDataClass *EventsDataHub;

    virtual double getValue(int) {return 0;} //gives value for the event number
    virtual bool isRequireReconstruction() {return true;} //does require recon data to present?

    virtual CorrelationUnitGenericClass* getClone() {return new CorrelationUnitGenericClass(); } //never used not overloaded

    virtual void saveJSON(QJsonObject &json) const
      {
        QJsonArray jsonArray;
        for (int i=0; i<Channels.size(); i++) jsonArray.append(Channels[i]);
        json["Array"] = jsonArray;
      }
};

class CorrelationCutGenericClass
{
  public:
    virtual ~CorrelationCutGenericClass(){}

    virtual const QString getType() {return "undefined";}  //type
    virtual int getCOBindex() const {return 0;} //index used in ui

    int CutOption; //e.g. "left_above" for line or "inside" for ellipse
    QList<double> Data; //e.g. A B C for line or ellipse data or X Y for polygon nodes

    virtual bool filter(double , double ) {return false;} //true - passes

    virtual CorrelationCutGenericClass* getClone() {return new CorrelationCutGenericClass();} //never used not overloaded

    virtual void saveJSON(QJsonObject &json) const
      {
        json["CutOption"] = CutOption;
        QJsonArray jsonArray;
        for (int i=0; i<Data.size(); i++) jsonArray.append(Data[i]);
        json["Data"] = jsonArray;
      }
};

struct CorrelationFilterStructure
{
   //public
     bool Active;

     //indication
     bool AutoBin;
     int BinX;
     int BinY;
     bool AutoSize;
     double minX;
     double maxX;
     double minY;
     double maxY;

   private:
     //correlation units
     CorrelationUnitGenericClass* X;
     CorrelationUnitGenericClass* Y;

     //cut
     CorrelationCutGenericClass* Cut;

   public:
     CorrelationFilterStructure() {X = new CorrelationUnitGenericClass(); Y = new CorrelationUnitGenericClass(); Cut = new CorrelationCutGenericClass(); }
     CorrelationFilterStructure(CorrelationFilterStructure *from) { copyFrom(from); }
     ~CorrelationFilterStructure() {delete X; delete Y; delete Cut;}

     void setCorrelationUnitX(CorrelationUnitGenericClass* unit) { delete X; X = unit; }
     void setCorrelationUnitY(CorrelationUnitGenericClass* unit) { delete Y; Y = unit; }
     CorrelationUnitGenericClass* getCorrelationUnitX() {return X;}
     CorrelationUnitGenericClass* getCorrelationUnitY() {return Y;}

     void setCut(CorrelationCutGenericClass* cut) {delete Cut; Cut = cut; }
     CorrelationCutGenericClass* getCut() {return Cut;}

     void copyFrom(CorrelationFilterStructure* source);

     void saveJSON(QJsonObject &json) const;

     static CorrelationFilterStructure* createFromJson(QJsonObject &json, DetectorClass* detector, EventsDataClass* eventsDataHub);
     static CorrelationUnitGenericClass *CorrUnitCreator(QString Type);
};

//======= subtypes: corr units ==========

class CU_SingleChannel : public CorrelationUnitGenericClass
{
  public:
    const QString getType() { return "SingleChannel";}
    int getCOBindex() const {return 0;}
    virtual const TString getAxisTitle() { TString tmp = "Ch"; tmp += Channels[0]; return tmp;}

    //double getValue(int iev) {return (*Events)->at(iev)[Channels[0]] * Gains->at(Channels[0]);}
    //double getValue(int iev) {return (*Events)->at(iev)[Channels[0]] * (*PMs)->getGain(Channels[0]);}
    double getValue(int iev) {return EventsDataHub->Events.at(iev)[Channels[0]] * Detector->PMs->getGain(Channels[0]);}
    bool isRequireReconstruction() {return false;}

    CU_SingleChannel* getClone()
      {
        CU_SingleChannel* copy = new CU_SingleChannel();
        //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
        //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
        copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
        return copy;
      }
};

class CU_SumAllChannels : public CorrelationUnitGenericClass
{
  public:
    const QString getType() { return "SumAllChannels";}
    int getCOBindex() const {return 1;}
    virtual const TString getAxisTitle() { return "Sum all ch";}

    double getValue(int iev)
      {
         double sum = 0;
         for (int i=0; i<Channels.size(); i++)
           {
              int thisPM = Channels[i];
              //sum += (*Events)->at(iev)[thisPM] * Gains->at(thisPM);
              sum += EventsDataHub->Events.at(iev)[thisPM] * Detector->PMs->getGain(thisPM);
           }
        return sum;
      }
    bool isRequireReconstruction() {return false;}

    CU_SumAllChannels* getClone()
      {
        CU_SumAllChannels* copy = new CU_SumAllChannels();
        //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
        //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
        copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
        return copy;
      }
};

class CU_SumChannels : public CorrelationUnitGenericClass
{
  public:
    const QString getType() { return "SumChannels";}
    int getCOBindex() const {return 2;}
    virtual const TString getAxisTitle()
      {
        TString tmp = "Ch";
        for (int i=0; i<Channels.size(); i++)
          {
            tmp += Channels[i];
            tmp += "+";
          }
        if (tmp.Length()>0) tmp.Resize(tmp.Length()-1);
        return tmp;
      }

    double getValue(int iev)
      {
         double sum = 0;
         for (int i=0; i<Channels.size(); i++)
           {
              int thisPM = Channels[i];
              //sum += (*Events)->at(iev)[thisPM] * Gains->at(thisPM);
              sum += EventsDataHub->Events.at(iev)[thisPM] * Detector->PMs->getGain(thisPM);
           }
        return sum;
      }
    bool isRequireReconstruction() {return false;}

    CU_SumChannels* getClone()
      {
        CU_SumChannels* copy = new CU_SumChannels();
        //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
        //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
        copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
        return copy;
      }
};

class CU_LoadedEnergy : public CorrelationUnitGenericClass
{
  public:
    const QString getType() { return "LoadedEnergy";}
    int getCOBindex() const {return 3;}
    virtual const TString getAxisTitle() { return "Loaded energy";}

    double getValue(int iev) {return EventsDataHub->Events.at(iev).last();}
    bool isRequireReconstruction() {return false;}

    CU_LoadedEnergy* getClone()
      {
        CU_LoadedEnergy* copy = new CU_LoadedEnergy();
        //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
        //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
        copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
        return copy;
      }
};

class CU_RecE : public CorrelationUnitGenericClass
{
  public:
     const QString getType() { return "RecE";}
     int getCOBindex() const {return 4;}
     virtual const TString getAxisTitle() { return "Rec energy";}
     double getValue(int iev) {return EventsDataHub->ReconstructionData.at(iev)->Points[0].energy;}
     CU_RecE* getClone()
       {
         CU_RecE* copy = new CU_RecE();
         //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
         //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
         copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
         return copy;
       }
};

class CU_Chi2 : public CorrelationUnitGenericClass
{
  public:
     const QString getType() { return "Chi2";}
     int getCOBindex() const {return 5;}
     virtual const TString getAxisTitle() { return "Chi2";}
     double getValue(int iev) {return EventsDataHub->ReconstructionData.at(iev)->chi2;}
     CU_Chi2* getClone()
       {
         CU_Chi2* copy = new CU_Chi2();
         //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
         //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
         copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
         return copy;
       }
};

class CU_RecX : public CorrelationUnitGenericClass
{
  public:
     const QString getType() { return "RecX";}
     int getCOBindex() const {return 6;}
     virtual const TString getAxisTitle() { return "Rec X";}
     double getValue(int iev) {return EventsDataHub->ReconstructionData.at(iev)->Points[0].r[0];}
     CU_RecX* getClone()
       {
         CU_RecX* copy = new CU_RecX();
         //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
         //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
         copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
         return copy;
       }
};
class CU_RecY : public CorrelationUnitGenericClass
{
  public:
     const QString getType() { return "RecY";}
     int getCOBindex() const {return 7;}
     virtual const TString getAxisTitle() { return "Rec Y";}
     double getValue(int iev) {return EventsDataHub->ReconstructionData.at(iev)->Points[0].r[1];}
     CU_RecY* getClone()
       {
         CU_RecY* copy = new CU_RecY();
         //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
         //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
         copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
         return copy;
       }
};
class CU_RecZ : public CorrelationUnitGenericClass
{
  public:
     const QString getType() { return "RecZ";}
     int getCOBindex() const {return 8;}
     virtual const TString getAxisTitle() { return "Rec Z";}
     double getValue(int iev) {return EventsDataHub->ReconstructionData.at(iev)->Points[0].r[2];}
     CU_RecZ* getClone()
       {
         CU_RecZ* copy = new CU_RecZ();
         //copy->Channels = Channels; copy->Events = Events; copy->Gains = Gains; copy->ReconstructionData = ReconstructionData;
         //copy->Channels = Channels; copy->Events = Events; copy->PMs = PMs; copy->ReconstructionData = ReconstructionData;
         copy->Channels = Channels; copy->Detector = Detector; copy->EventsDataHub = EventsDataHub;
         return copy;
       }
};

/*
//-------------- creator of correlation unit by type string --------------------------
CorrelationUnitGenericClass* CorrUnitCreator(QString Type)
{
  if (Type == "SingleChannel") return new CU_SingleChannel();
  if (Type == "SumAllChannels") return new CU_SumAllChannels();
  if (Type == "SumChannels") return new CU_SumChannels();
  if (Type == "LoadedEnergy") return new CU_LoadedEnergy();
  if (Type == "RecE") return new CU_RecE();
  if (Type == "Chi2") return new CU_Chi2();
  if (Type == "RecX") return new CU_RecX();
  if (Type == "RecY") return new CU_RecY();
  if (Type == "RecZ") return new CU_RecZ();

  //if unknown type, returns 0
  return 0;
}
*/

//======================= subtypes: cuts ===========================
class Cut_Line : public CorrelationCutGenericClass
{
  public:
    const QString getType() {return "line";}
    int getCOBindex() const {return 0;}

    bool filter(double val1, double val2)
      {
        if (CutOption == 1)
          {
            if ( (Data[0]*val1 + Data[1]*val2) > Data[2] ) return false; //cuts above
            else return true;
          }
        else
          {
            if ( (Data[0]*val1 + Data[1]*val2) > Data[2] ) return true; //cuts below
            else return false;
          }
      }

    Cut_Line* getClone()
      {
        Cut_Line* copy = new Cut_Line();
        copy->CutOption = CutOption; copy->Data = Data;
        return copy;
      }
};

class Cut_Ellipse : public CorrelationCutGenericClass
{
  public:
    const QString getType() {return "ellipse";}
    int getCOBindex() const {return 1;}

    bool filter(double val1, double val2)
    {
       // 0-x 1-y 2-r1 3-r2 4-(-angle)
       bool outside = false;
       if ( (Data[2]<1.0e-10 && Data[2]>-1.0e-10) || (Data[3]<1.0e-10 && Data[3]>-1.0e-10) ) outside = true;
       else
         {
           double sinA = sin(-Data[4]*0.017453292519);
           double cosA = cos(-Data[4]*0.017453292519);
           double dx = val1 - Data[0];
           double dy = val2 - Data[1];

           double tmp1 = cosA*dx - sinA*dy;
           double tmp2 = sinA*dx + cosA*dy;
           double rad = tmp1*tmp1/Data[2]/Data[2] + tmp2*tmp2/Data[3]/Data[3];

           if (rad > 1) outside = true;
         }

       if (CutOption == 0) return !outside; //cut outside
       else return outside;
    }

    Cut_Ellipse* getClone()
      {
        Cut_Ellipse* copy = new Cut_Ellipse();
        copy->CutOption = CutOption; copy->Data = Data;
        return copy;
      }
};

class Cut_Polygon : public CorrelationCutGenericClass
{
  public:
    const QString getType() {return "polygon";}
    int getCOBindex() const {return 2;}

    bool filter(double val1, double val2)
      {
        QVector<QPointF> tmp(0);
        for (int i=0; i<Data.size()/2; i++) tmp.append(QPointF(Data[i*2], Data[i*2+1]));
        QPolygonF poly(tmp);

        bool inside = poly.containsPoint(QPointF(val1, val2), Qt::OddEvenFill);

        if (CutOption == 0) return inside;
        else return !inside;
      }

    Cut_Polygon* getClone()
      {
        Cut_Polygon* copy = new Cut_Polygon();
        copy->CutOption = CutOption; copy->Data = Data;
        return copy;
      }
};


#endif // CORRELATIONFILTERS_H
