#include "functions.h"
#include "classes.h"

#include <QtGui>
#include <QVector>
#include <QMessageBox>
#include <QTableWidget>
#include <QCheckBox>
#include <QSpinBox>
#include <QLineEdit>
#include <QComboBox>

#include "TMath.h"
#include "TRandom2.h"


bool parseJson(const QJsonObject &json, const QString &key, bool &var)
{
  if (json.contains(key))
    {
      var = json[key].toBool();
      return true;
    }
  else return false;
}
bool parseJson(const QJsonObject &json, const QString &key, int &var)
{
  if (json.contains(key))
    {
      var = json[key].toInt();
      return true;
    }
  else return false;
}
bool parseJson(const QJsonObject &json, const QString &key, double &var)
{
  if (json.contains(key))
    {
      var = json[key].toDouble();
      return true;
    }
  else return false;
}
bool parseJson(const QJsonObject &json, const QString &key, float &var)
{
  if (json.contains(key))
    {
      var = json[key].toDouble();
      return true;
    }
  else return false;
}
bool parseJson(const QJsonObject &json, const QString &key, QString &var)
{
  if (json.contains(key))
    {
      var = json[key].toString();
      return true;
    }
  else return false;
}

bool parseJson(const QJsonObject &json, const QString &key, QJsonArray &var)
{
  if (json.contains(key))
    {
      var = json[key].toArray();
      return true;
    }
  else return false;
}

void JsonToCheckbox(QJsonObject &json, QString key, QCheckBox *cb)
{
  if (json.contains(key))
    cb->setChecked(json[key].toBool());
}

void JsonToSpinBox(QJsonObject &json, QString key, QSpinBox *sb)
{
  if (json.contains(key))
    sb->setValue(json[key].toInt());
}

void JsonToLineEdit(QJsonObject &json, QString key, QLineEdit *le)
{
  if (json.contains(key))
    le->setText( QString::number(json[key].toDouble()) );
}

void JsonToComboBox(QJsonObject &json, QString key, QComboBox *qb)
{
  if (json.contains(key))
    {
      int index = json[key].toInt();
      if (index >= qb->count()) index = -1;
      qb->setCurrentIndex(index);
    }
}


bool MyNormalizeVector(double *arr)
{
    double Norm =0;
    for (int i=0;i<3;i++) Norm += arr[i]*arr[i];
    Norm = TMath::Sqrt(Norm);
    if (Norm < 1e-20)
      {
 //       qDebug()<<"fail norm";
        message("Error normalizing to unit vector!");
        return false;
      }

    for (int i=0; i<3; i++) arr[i] = arr[i]/Norm;
//    qDebug()<<arr[0]<<arr[1]<<arr[2]<<Norm;
    return true;
}

double Length(double *arr)
{
  return sqrt(arr[0]*arr[0] + arr[1]*arr[1] + arr[2]*arr[2]);
}

double InteractionValue(double energy, QVector<double>* X, QVector<double>* F, bool LogLog)
{
///  qDebug()<<"data point in arrays X and F:"<<X->size()<<F->size()<<"Min X:"<<X->first()<<"Max X:"<<X->last();
  //if (degree == 1)   {

    //linear interpolation of IterationData for current energy
    QVector<double>::iterator it;
    it = qLowerBound(X->begin(), X->end(), energy);
    int index = X->indexOf(*it);
///    qDebug()<<"energy:"<<energy<<"index"<<index;//<<*it;
    if (index < 1)
    {
        qWarning()<<"interpolation: value out (or on the border) of the interaction data range!";
        return F->at(index);
    }

    //interpolating:
    double Less = F->at(index-1);
    double More = F->at(index);
    //qDebug()<<" Less/More"<<Less<<More;
    double EnergyLess = X->at(index-1);
    double EnergyMore = X->at(index);
    // qDebug()<<" Energy Less/More"<<EnergyLess<<EnergyMore;
    double InteractionValue;
    if (EnergyLess == EnergyMore) {InteractionValue = More;}   //*** ==  To change to safer
    else
      {
        if (LogLog)
          {
            double LogLess = log(Less);
            double LogMore = log(More);
            double LogEnergyLess = log(EnergyLess);
            double LogEnergyMore = log(EnergyMore);
            InteractionValue = LogLess + (LogMore-LogLess)*(log(energy)-LogEnergyLess)/(LogEnergyMore-LogEnergyLess);
            InteractionValue = exp(InteractionValue);
          }
        else InteractionValue = Less + (More-Less)*(energy-EnergyLess)/(EnergyMore-EnergyLess);
      }
///     qDebug()<<"energy / interValue"<<energy<<InteractionValue;
    return InteractionValue;
// }

  /*
  else
   {
     return PolyInterpolation(energy, X, F, degree+1);
   }
   */
}

/*
double PolyInterpolation(double x, QVector<double>* xi, QVector<double>* yi, int order)  //with degenerate nodes
{
  qDebug()<<"-----------";
  int isize = xi->size();

  QVector<double> lambda;
  lambda.resize(isize);
  double y;
  int j, is, il;

  // check order of interpolation
  if (order > isize) order = isize;

  // if x is ouside the xi[] interval
  if (x <= xi->at(0)) return yi->at(0);
  if (x >= xi->at(isize-1)) return yi->at(isize-1);

  // loop to find j so that x[j-1] < x < x[j]
  j = 0;
  while (j <= isize-1)
    {
      if (xi->at(j) >= x) break;
      j++;
    }
  qDebug()<<"x="<<x<<"start from"<<j;
  qDebug()<<"set size"<<isize<<"order"<<order;

//  // shift j to correspond to (order-1)th interpolation
//  j = j - order/2;
//  // if j is ouside of the range [0, ... isize-1]
//  if (j < 0) j=0;
//  if (j+order-1 > isize-1 ) j = isize-order;

    //tring to go left
  int leftBoundLimited = false;
  int leftToGo = order/2;
  int leftBound = j;
  for (int itmp=1; itmp<leftToGo+1; itmp++)
    {
      if (j-itmp<0)
        {
          leftBoundLimited = true;
          break;
        }
      if (xi->at(j-itmp) == xi->at(j-itmp+1))
        {
          //dgenerate node found
          leftBoundLimited = true;
          break;
        }
      leftBound--;
    }
   qDebug()<<"left bound"<<leftBound<<leftBoundLimited;
     //trying to go right
   int rightBoundLimited = false;
   int rightToGo = order-1;
   int rightBound = leftBound;
   for (int itmp=1; itmp<rightToGo+1; itmp++)
     {
       if (rightBound+1 > isize-1)
         {
           rightBoundLimited = true;
           break;
         }
       if (xi->at(rightBound+1) == xi->at(rightBound))
         {
           //degenerate node found
           rightBoundLimited = true;
           break;
         }
       rightBound++;
     }
   qDebug()<<"right bound"<<rightBound<<rightBoundLimited;
     //if rightBound and not leftBound, attempt to extend left
   if (rightBoundLimited && !leftBoundLimited)
     {
       qDebug()<< " attempt to correct left bound";
       leftToGo = order-1 - (rightBound-leftBound);
       qDebug()<<"  want to make"<<leftToGo<<"extra steps left";
       for (int itmp = 1; itmp<leftToGo+1; itmp++)
         {
           if (leftBound-1<0)
             {
               leftBoundLimited = true;
               break;
             }
           if (xi->at(leftBound-1) == xi->at(leftBound))
             {
               //dgenerate node found
               leftBoundLimited = true;
               break;
             }
           leftBound--;
         }
       qDebug()<<"left bound corrected"<<leftBound<<leftBoundLimited;
     }

   //check if have to adjust degree for this iteration
   if (order > rightBound-leftBound+1)
     {
       order = rightBound-leftBound+1;
       qDebug()<<"corrected order"<<order;
     }

//  y = 0.0;
//  for (is = j; is <= j+order-1; is = is+1)
//   {
//     lambda[is] = 1.0;
//     for (il = j; il <= j+ order-1; il++)
//        if(il != is)
//             lambda[is] = lambda[is]*(x - xi->at(il))/(xi->at(is) - xi->at(il));

//     y = y + yi->at(is)*lambda[is];
//   }

   y = 0.0;
   for (is = leftBound; is <= rightBound; is = is+1)
    {
      lambda[is] = 1.0;
      for (il = leftBound; il <= rightBound; il++)
         if(il != is)
              lambda[is] = lambda[is]*(x - xi->at(il))/(xi->at(is) - xi->at(il));

      y = y + yi->at(is)*lambda[is];
    }
  return y;
}
*/

void ConvertToStandardWavelengthes(QVector<double>* sp_x, QVector<double>* sp_y, double WaveFrom, double WaveStep, int WaveNodes, QVector<double>* y)
{
  y->resize(0);

  //qDebug()<<"Data range:"<<sp_x->at(0)<<sp_x->at(sp_x->size()-1);
  double xx, yy;
  int points = WaveNodes;
  for (int i=0; i<points; i++)
    {
      xx = WaveFrom + WaveStep*i;
      if (xx <= sp_x->at(0)) yy = sp_y->at(0);
      else
        {
          if (xx >= sp_x->at(sp_x->size()-1)) yy = sp_y->at(sp_x->size()-1);
          else
            {
              //general case
              yy = InteractionValue(xx, sp_x, sp_y); //reusing interpolation function from functions.h
            }
        }
//      qDebug()<<xx<<yy;
      y->append(yy);
    }
}

GammaStructure Compton(GammaStructure *InGamma, TRandom2 *RandGen)
{
  //http://geant4.web.cern.ch/geant4/UserDocumentation/Doxygen/examples_doc/html_fanoCavity/html/MyKleinNishinaCompton_8cc_source.html
  double gamEnergy0 = InGamma->energy;
  double electron_mass_c2 = 510.998928; //keV
  double E0_m = gamEnergy0 / electron_mass_c2 ;
  TVector3 gamDirection0 = InGamma->direction;
  double epsilon, epsilonsq, onecost, sint2, greject;
  double eps0   = 1./(1. + 2.*E0_m);
  double eps0sq = eps0*eps0;
  double alpha1 = - log(eps0);
  double alpha2 = 0.5*(1.- eps0sq);

  do
    {
      if ( alpha1/(alpha1+alpha2) > RandGen->Rndm() )
        {
          epsilon   = exp(-alpha1*RandGen->Rndm());
          epsilonsq = epsilon*epsilon;
        }
      else
        {
          epsilonsq = eps0sq + (1.- eps0sq)*RandGen->Rndm();
          epsilon   = sqrt(epsilonsq);
        };
      onecost = (1.- epsilon)/(epsilon*E0_m);
      sint2   = onecost*(2.-onecost);
      greject = 1. - epsilon*sint2/(1.+ epsilonsq);
    } while (greject < RandGen->Rndm());

  double cosTeta = 1. - onecost;
  double sinTeta = sqrt (sint2);
  double Phi     = 2.*3.14159 * RandGen->Rndm();
  double dirx = sinTeta*cos(Phi), diry = sinTeta*sin(Phi), dirz = cosTeta;
  double gamEnergy1 = epsilon*gamEnergy0;

  TVector3 gamDirection1 ( dirx,diry,dirz );
  gamDirection1.RotateUz(gamDirection0);

  GammaStructure G1;
  G1.energy = gamEnergy1;
  G1.direction = gamDirection1;
  return G1;
}

int LoadDoubleVectorsFromFile(QString FileName, QVector<double>* x)
{
  if (FileName.isEmpty())
      {
          message("Error: empty name was given to file loader!");
          return 1;
      }

  QFile file(FileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: "+FileName);
      return 2;
    }

  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
  x->resize(0);
  while(!in.atEnd())
       {
          QString line = in.readLine();
          QStringList fields = line.split(rx, QString::SkipEmptyParts);

          bool ok1= false;
          double xx;
          if (fields.size()>0) xx = fields[0].toDouble(&ok1);  //*** potential problem with decimal separator!

          if (ok1)
            {
              x->append(xx);
            }
        }
   file.close();

   if (x->isEmpty())
   {
       message("Error: Wrong format - nothing was red: "+FileName);
       return 3;
   }

   return 0;
}


int LoadDoubleVectorsFromFile(QString FileName, QVector<double>* x, QVector<double>* y)
{
  if (FileName.isEmpty())
      {
          message("Error: empty name was given to file loader!");
          return 1;
      }

  QFile file(FileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: "+FileName);
      return 2;
    }

  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
  x->resize(0);
  y->resize(0);
  while(!in.atEnd())
       {
          QString line = in.readLine();
          QStringList fields = line.split(rx, QString::SkipEmptyParts);

          bool ok1=false, ok2;
          double xx, yy;
          if (fields.size() > 1 )
            {
              xx = fields[0].toDouble(&ok1);  //*** potential problem with decimal separator!
              yy = fields[1].toDouble(&ok2);
            }
          if (ok1 && ok2)
            {
              x->append(xx);
              y->append(yy);
            }
        }
   file.close();

   if (x->isEmpty())
   {
       message("Error: Wrong format - nothing was red: "+FileName);
       return 3;
   }

   return 0;
}

int LoadDoubleVectorsFromFile(QString FileName, QVector<double>* x, QVector<double>* y, QVector<double>* z)
{
  if (FileName.isEmpty())
  {
      message("Error: empty name was given to file loader!");
      return 1;
  }

  QFile file(FileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: "+FileName);
      return 2;
    }

  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
  x->resize(0);
  y->resize(0);
  z->resize(0);
  while(!in.atEnd())
       {
          QString line = in.readLine();
          QStringList fields = line.split(rx, QString::SkipEmptyParts);

          bool ok1=false, ok2, ok3;
          double xx, yy, zz;
          if (fields.size() > 2 )
            {
               xx = fields[0].toDouble(&ok1);  //*** potential problem with decimal separator!
               yy = fields[1].toDouble(&ok2);
               zz = fields[2].toDouble(&ok3);
            }
          if (ok1 && ok2 && ok3)
            {
              x->append(xx);
              y->append(yy);
              z->append(zz);
            }
        }
   file.close();

   if (x->isEmpty())
   {
       message("Error: Wrong format - nothing was red: "+FileName);
       return 3;
   }

  return 0;
}

int SaveDoubleVectorsToFile(QString FileName, const QVector<double>* x, int count)
{
  if (count == -1) count = x->size();
  QFile outFile( FileName );
  outFile.open(QIODevice::WriteOnly);
  if(!outFile.isOpen())
    {
      qDebug() << "- Error, unable to open" << FileName << "for output";
      QMessageBox mb;
      mb.setText("Unable to open file " +FileName+ " for writing!");
      mb.exec();
      return 1;
    }
  QTextStream outStream(&outFile);

  for (int i=0; i<count; i++)
    outStream << x->at(i) <<"\r\n";
  outFile.close();
  return 0;
}


int SaveDoubleVectorsToFile(QString FileName, const QVector<double> *x, const QVector<double> *y, int count)
{
  if (count == -1) count = x->size();  
  QFile outFile( FileName );
  outFile.open(QIODevice::WriteOnly);
  if(!outFile.isOpen())
    {
      qDebug() << "- Error, unable to open" << FileName << "for output";
      QMessageBox mb;
      mb.setText("Unable to open file " +FileName+ " for writing!");
      mb.exec();
      return 1;
    }
  QTextStream outStream(&outFile);

  for (int i=0; i<count; i++)
    outStream << x->at(i) << " " << y->at(i) <<"\r\n";
  outFile.close();
  return 0;
}

int SaveDoubleVectorsToFile(QString FileName, const QVector<double> *x, const QVector<double> *y, const QVector<double> *z, int count)
{
  if (count == -1) count = x->size();
  QFile outFile( FileName );
  outFile.open(QIODevice::WriteOnly);
  if(!outFile.isOpen())
    {
      qDebug() << "- Error, unable to open" << FileName << "for output";
      QMessageBox mb;
      mb.setText("Unable to open file " +FileName+ " for writing!");
      mb.exec();
      return 1;
    }
  QTextStream outStream(&outFile);

  for (int i=0; i<count; i++)
    outStream << x->at(i) << " " << y->at(i) << " " << z->at(i) << "\r\n";
  outFile.close();
  return 0;
}

int LoadIntVectorsFromFile(QString FileName, QVector<int>* x)
{
  if (FileName.isEmpty())
      {
          message("Empty file: "+FileName);
          return 1;
      }

  QFile file(FileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: "+FileName);
      return 2;
    }

  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
  x->resize(0);
  while(!in.atEnd())
       {
          QString line = in.readLine();
          QStringList fields = line.split(rx);

          bool ok1;
          int xx = fields[0].toInt(&ok1);
          if (ok1)
            {
              x->append(xx);
            }
        }
   file.close();

   if (x->isEmpty())
   {
       message("Error: Wrong format - nothing was red: "+FileName);
       return 3;
   }

   return 0;
}

int LoadIntVectorsFromFile(QString FileName, QVector<int> *x, QVector<int> *y)
{
  if (FileName.isEmpty())
      {
          message("Empty file: "+FileName);
          return 1;
      }

  QFile file(FileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: "+FileName);
      return 2;
    }

  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
  x->resize(0);
  y->resize(0);
  while(!in.atEnd())
       {
          QString line = in.readLine();
          QStringList fields = line.split(rx);

          bool ok1, ok2;
          int xx = fields[0].toInt(&ok1);
          int yy = fields[1].toInt(&ok2);
          if (ok1 && ok2)
            {
              x->append(xx);
              y->append(yy);
            }
        }
   file.close();

   if (x->isEmpty())
   {
       message("Error: Wrong format - nothing was red: "+FileName);
       return 3;
   }

   return 0;
}

void message(QString text, QWidget* parent)
{
  QMessageBox mb(parent);
  mb.setWindowFlags(mb.windowFlags() | Qt::WindowStaysOnTopHint);
  mb.setText(text);
  if (!parent) mb.move(200,200);
  mb.exec();
}

bool ExtractNumbersFromQString(QString input, QList<int> *ToAdd)
{
  ToAdd->clear();

  QRegExp rx("(\\,|\\-)"); //RegEx for ' ' and '-'

  QStringList fields = input.split(rx, QString::SkipEmptyParts);

  if (fields.size() == 0 )
    {
      //message("Nothing to add!");
      return false;
    }

  fields = input.split(",", QString::SkipEmptyParts);
  //  qDebug()<<"found "<<fields.size()<<" records";

  for (int i=0; i<fields.size(); i++)
    {
      QString thisField = fields[i];

      //are there "-" separated fields?
      QStringList subFields = thisField.split("-", QString::SkipEmptyParts);
      //bool error = false;

      if (subFields.size() > 2 || subFields.size() == 0) return false;
      else if (subFields.size() == 1)
        {
          //just one number
          bool ok;
          int pm;
          pm = subFields[0].toInt(&ok);

          if (ok) ToAdd->append(pm);
          else return false;
        }
      else //range - two subFields
        {
          bool ok1, ok2;
          int pm1, pm2;
          pm1 = subFields[0].toInt(&ok1);
          pm2 = subFields[1].toInt(&ok2);
          if (ok1 && ok2)
            {
               if (pm2<pm1) return false;//error = true;
               else
                 {
                   for (int j=pm1; j<=pm2; j++) ToAdd->append(j);
                 }
            }
          else return false;//error = true;

        }
  /*
      if (error)
        {
          message("Input error!");
          return;
        }
 */
    }

  return true;
}

bool IntersectionOfTwoLines(double A1, double B1, double C1, double A2, double B2, double C2, double* result)
{
  double det = A1*B2 - A2*B1;

  if (fabs(det) < 1.0e-10)
    {
       //lines are ~parallel
      return false;
    }
  else
    {
      result[0] = (B2*C1 - B1*C2)/det;
      result[1] = (A1*C2 - A2*C1)/det;
    }

  return true;
}

int SaveIntVectorsToFile(QString FileName, const QVector<int> *x, int count)
{
  if (count == -1) count = x->size();
  QFile outFile( FileName );
  outFile.open(QIODevice::WriteOnly);
  if(!outFile.isOpen())
    {
      qDebug() << "- Error, unable to open" << FileName << "for output";
      QMessageBox mb;
      mb.setText("Unable to open file " +FileName+ " for writing!");
      mb.exec();
      return 1;
    }
  QTextStream outStream(&outFile);

  for (int i=0; i<count; i++)
    outStream << x->at(i) <<"\r\n";
  outFile.close();
  return 0;
}


int SaveIntVectorsToFile(QString FileName, const QVector<int> *x, const QVector<int> *y, int count)
{
  if (count == -1) count = x->size();
  QFile outFile( FileName );
  outFile.open(QIODevice::WriteOnly);
  if(!outFile.isOpen())
    {
      qDebug() << "- Error, unable to open" << FileName << "for output";
      QMessageBox mb;
      mb.setText("Unable to open file " +FileName+ " for writing!");
      mb.exec();
      return 1;
    }
  QTextStream outStream(&outFile);

  for (int i=0; i<count; i++)
    outStream << x->at(i) << " " << y->at(i) <<"\r\n";
  outFile.close();
  return 0;
}

bool LoadJsonFromFile(QJsonObject &json, QString fileName)
{
    QFile loadFile(fileName);
    if (loadFile.open(QIODevice::ReadOnly))
      {
        QByteArray saveData = loadFile.readAll();
        QJsonDocument loadDoc(QJsonDocument::fromJson(saveData));
        json = loadDoc.object();
        loadFile.close();
        //qDebug()<<"  Loaded Json from file:"<<fileName;
      }
    else
      {
        message("Cannot open file: "+fileName);
        return false;
      }
    return true;
}

bool SaveJsonToFile(QJsonObject &json, QString fileName)
{
  QJsonDocument saveDoc(json);

  QFile saveFile(fileName);
  if (saveFile.open(QIODevice::WriteOnly))
    {
      saveFile.write(saveDoc.toJson());
      saveFile.close();
      //  qDebug()<<"  Saved Json to file:"<<fileName;
    }
  else
    {
      message("Couldn't save json to file: "+fileName);
      return false;
    }

  return true;
}

bool writeTH1ItoJsonArr(TH1I* hist, QJsonArray &ja)
{  
  if (hist)
    {
      for (int i=1; i<hist->GetSize(); i++)
        {
          QJsonArray el;
          el.append(hist->GetBinLowEdge(i));
          el.append(hist->GetBinContent(i));
          ja.append(el);
        }
      return true;
    }
  else return false;
}

bool writeTH1DtoJsonArr(TH1D* hist, QJsonArray &ja)
{ 
  if (hist)
    {
      for (int i=1; i<hist->GetSize(); i++)
        {
          QJsonArray el;
          el.append(hist->GetBinLowEdge(i));
          el.append(hist->GetBinContent(i));
          ja.append(el);
        }
      return true;
    }
  else return false;
}

/*
void jsonArrToTH1I(QJsonArray &ja, TH1I *hist)
{
  int size = ja.size();
  double* xx = new double[size];
  int* yy    = new int[size];
  for (int i=0; i<size; i++)
      {
        xx[i] = ja[i].toArray()[0].toDouble();
        yy[i] = ja[i].toArray()[1].toInt();
      }
  for (int i = 1; i<size+1; i++)  hist->SetBinContent(i, yy[i-1]);
  delete[] xx;
  delete[] yy;
}

void jsonArrToTH1D(QJsonArray &ja, TH1D *hist)
{
  int size = ja.size();
  double* xx = new double[size];
  int* yy    = new int[size];
  for (int i=0; i<size; i++)
      {
        xx[i] = ja[i].toArray()[0].toDouble();
        yy[i] = ja[i].toArray()[1].toDouble();
      }
  for (int i = 1; i<size+1; i++)  hist->SetBinContent(i, yy[i-1]);
  delete[] xx;
  delete[] yy;
}
*/


bool writeTwoQVectorsToJArray(QVector<double> &x, QVector<double> &y, QJsonArray &ar)
{
  if (x.size() != y.size())
    {
      qWarning() << "Vectors mismatch!";
      return false;
    }

  for (int i=0; i<x.size(); i++)
    {
      QJsonArray el;
      el.append(x[i]);
      el.append(y[i]);
      ar.append(el);
    }
  return true;
}

void readTwoQVectorsFromJArray(QJsonArray &ar, QVector<double> &x, QVector<double> &y)
{
  for (int i=0; i<ar.size(); i++)
    {
      double X = ar[i].toArray()[0].toDouble();
      x.append(X);
      double Y = ar[i].toArray()[1].toDouble();
      y.append(Y);
    }
}

bool write2DQVectorToJArray(QVector<QVector<double> > &xy, QJsonArray &ar)
{
  for (int i1=0; i1<xy.size(); i1++)
    {
      QJsonArray el;
      for (int i2=0; i2<xy[i1].size(); i2++) el.append(xy[i1][i2]);
      ar.append(el);
    }
  return true;
}

void read2DQVectorFromJArray(QJsonArray &ar, QVector<QVector<double> > &xy)
{
  xy.resize(ar.size());
  for (int i1=0; i1<xy.size(); i1++)
    {
      QJsonArray el = ar[i1].toArray();
      for (int i2=0; i2<el.size(); i2++)
        xy[i1].append( el[i2].toDouble());
    }
}
