#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#include <QVector>
//#include <QTableWidget>

#include <QJsonObject>
#include <QJsonArray>

struct GammaStructure;
class TRandom2;
class QWidget;
class TH1I;
class TH1D;
class QCheckBox;
class QSpinBox;
class QLineEdit;
class QComboBox;

bool parseJson(const QJsonObject &json, const QString &key, bool &var);
bool parseJson(const QJsonObject &json, const QString &key, int &var);
bool parseJson(const QJsonObject &json, const QString &key, double &var);
bool parseJson(const QJsonObject &json, const QString &key, float &var);
bool parseJson(const QJsonObject &json, const QString &key, QString &var);
bool parseJson(const QJsonObject &json, const QString &key, QJsonArray &var);

void JsonToCheckbox(QJsonObject& json, QString key, QCheckBox* cb);
void JsonToSpinBox(QJsonObject& json, QString key, QSpinBox* sb);
void JsonToLineEdit(QJsonObject& json, QString key, QLineEdit* le);
void JsonToComboBox(QJsonObject& json, QString key, QComboBox* qb);

bool writeTwoQVectorsToJArray(QVector<double> &x, QVector<double> &y, QJsonArray &ar);
void readTwoQVectorsFromJArray(QJsonArray &ar, QVector<double> &x, QVector<double> &y);
bool write2DQVectorToJArray(QVector< QVector<double> > &xy, QJsonArray &ar);
void read2DQVectorFromJArray(QJsonArray &ar, QVector<QVector<double> > &xy);

bool LoadJsonFromFile(QJsonObject &json, QString fileName);
bool SaveJsonToFile(QJsonObject &json, QString fileName);

bool MyNormalizeVector(double *arr);
double Length(double *arr);

inline void updateMax(double &max, double value) {if (value>max) max=value;} //assume positive max and value!

double InteractionValue(double energy, QVector<double> *X, QVector<double> *F, bool LogLog = false);

//double PolyInterpolation(double x, QVector<double> *xi, QVector<double> *yi, int order);

void ConvertToStandardWavelengthes(QVector<double>* sp_x, QVector<double>* sp_y, double WaveFrom, double WaveStep, int WaveNodes, QVector<double>* y);

GammaStructure Compton(GammaStructure *InGamma, TRandom2 *RandGen);

bool writeTH1ItoJsonArr(TH1I *hist, QJsonArray &ja);
bool writeTH1DtoJsonArr(TH1D* hist, QJsonArray &ja);
//void jsonArrToTH1I(QJsonArray &ja, TH1I *hist); bad idea
//void jsonArrToTH1D(QJsonArray &ja, TH1D *hist);

int LoadDoubleVectorsFromFile(QString FileName, QVector<double>* x);  //cleans previous data
int LoadDoubleVectorsFromFile(QString FileName, QVector<double>* x, QVector<double>* y);  //cleans previous data
int LoadDoubleVectorsFromFile(QString FileName, QVector<double>* x, QVector<double>* y, QVector<double>* z);  //cleans previous data

int SaveDoubleVectorsToFile(QString FileName, const QVector<double>* x, int count = -1);
int SaveDoubleVectorsToFile(QString FileName, const QVector<double>* x, const QVector<double>* y, int count = -1);
int SaveDoubleVectorsToFile(QString FileName, const QVector<double>* x, const QVector<double>* y, const QVector<double>* z, int count = -1);

int SaveIntVectorsToFile(QString FileName, const QVector<int>* x, int count = -1);
int SaveIntVectorsToFile(QString FileName, const QVector<int>* x, const QVector<int>* y, int count = -1);

int LoadIntVectorsFromFile(QString FileName, QVector<int>* x);
int LoadIntVectorsFromFile(QString FileName, QVector<int>* x, QVector<int>* y);

void message(QString text, QWidget* parent = 0);

bool ExtractNumbersFromQString(QString input, QList<int> *ToAdd);  //parse number string e.g. like that: "2, 4-8, 7, 25-65"

bool IntersectionOfTwoLines(double A1, double B1, double C1, double A2, double B2, double C2, double* result);



#endif // FUNCTIONS_H
