#ifndef OPTICALOVERRIDECLASS_H
#define OPTICALOVERRIDECLASS_H

#include "materialcollectionclass.h"

//Qt
#include <QJsonObject>

//Root
#include "TRandom2.h"

class OpticalOverrideClass
{
public:
  //the status for photon tracing:
  enum OpticalOverrideResultEnum {NotTriggered, Absorbed, Forward, Back, _Error_};
  //detailed status for statistics only:
  enum ScatterStatusEnum {SpikeReflection, LobeReflection, LambertianReflection, Absorption, Transmission, ErrorDetected};


  OpticalOverrideClass(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : MatCollection(MatCollection), MatFrom(MatFrom), MatTo(MatTo) {}
  virtual ~OpticalOverrideClass() {}

  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave) = 0; //unitary vectors! iWave = -1 if not wavelength-resolved

  virtual void printConfiguration(int iWave) = 0;
  virtual QString getType() const = 0;
  virtual QString getReportLine() = 0; // for GUI: reports override status "to material blabla (#id): properies"

  // save/load config
  virtual bool writeToJson(QJsonObject &json) = 0;
  virtual bool readFromJson(QJsonObject &json) = 0;

  void updateMatIndices(int iMatFrom, int iMatTo) {MatFrom = iMatFrom; MatTo = iMatTo;}

  // read-out variables for standalone checker only (not multithreaded)
  ScatterStatusEnum Status;               // type of interaction which happened
  TString LastError;                      // last detected error

protected:  
  MaterialCollectionClass* MatCollection;
  int MatFrom, MatTo;   // material index of material before(from) and after(to) the optical interface
};

class BasicOpticalOverride : public OpticalOverrideClass
{
public:
  BasicOpticalOverride(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo, double probLoss, double probRef, double probDiff, int scatterModel)
    : OpticalOverrideClass(MatCollection, MatFrom, MatTo),
      probLoss(probLoss), probRef(probRef), probDiff(probDiff), scatterModel(scatterModel) {}
  BasicOpticalOverride(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : OpticalOverrideClass(MatCollection, MatFrom, MatTo) {}
  virtual ~BasicOpticalOverride() {}

  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave); //unitary vectors! iWave = -1 if not wavelength-resolved

  virtual void printConfiguration(int iWave);
  virtual QString getType() const {return "Simplistic_model";}
  virtual QString getReportLine();

  // save/load config is not used for this type!
  virtual bool writeToJson(QJsonObject &json);
  virtual bool readFromJson(QJsonObject &json);

  //-- parameters --
  double probLoss; //probability of absorption
  double probRef;  //probability of specular reflection
  double probDiff; //probability of scattering
  int    scatterModel; //0 - 4Pi, 1 - 2Pi back, 2 - 2Pi forward

private:
  void RandomDir(TRandom2* RandGen, double* PhotonDir);
};

OpticalOverrideClass* OpticalOverrideFactory(QString model, MaterialCollectionClass* MatCollection, int MatFrom, int MatTo);

#endif // OPTICALOVERRIDECLASS_H
