#include "reconstructionsettings.h"

bool ReconstructionSettings::readFromJson(QJsonObject &Json)
{
  if (!Json.contains("ReconstructionOptions"))
    {
      ErrorString = "Json sent to reconstructor does not contain reconstruction config data!";
      return false;
    }
  ErrorString = "";
  QJsonObject RecJson = Json["ReconstructionOptions"].toObject();
  //general
  QJsonObject gjson = RecJson["General"].toObject();
  fReconstructEnergy = gjson["ReconstructEnergy"].toBool();
  SuggestedEnergy = gjson["InitialEnergy"].toDouble();
  fReconstructZ = gjson["ReconstructZ"].toBool();
  SuggestedZ = gjson["InitialZ"].toDouble();
  Zstrategy = gjson["Zstrategy"].toInt();
  fIncludePassive = gjson["IncludePassives"].toBool();
  fWeightedChi2calculation = gjson["WeightedChi2"].toBool();
  NumThreads = gjson["NumThreads"].toInt();

  //Algotithm
  QJsonObject ajson = RecJson["AlgorithmOptions"].toObject();
  ReconstructionAlgorithm = ajson["Algorithm"].toInt();
  //cog
  QJsonObject cogjson = ajson["CoGoptions"].toObject();
  fCogForceFixedZ = cogjson["ForceFixedZ"].toBool();
  fCoGStretch = cogjson["DoStretch"].toBool();
  CoGStretchX = cogjson["StretchX"].toDouble();
  CoGStretchY = cogjson["StretchY"].toDouble();
  CoGStretchZ = cogjson["StretchZ"].toDouble();
  fCoGIgnoreLow = cogjson["IgnoreLow"].toBool();
  CoGIgnoreThreshold = cogjson["IgnoreThreshold"].toDouble();
  fCoGIgnoreFar = false; //compatibility
  fCoGIgnoreFar = cogjson["IgnoreFar"].toBool();
  if (fCoGIgnoreFar)
    {
      CoGIgnoreDistance = cogjson["IgnoreDistance"].toDouble();
      CoGIgnoreDistance2 = CoGIgnoreDistance*CoGIgnoreDistance;
    }
  if (ReconstructionAlgorithm == 0) fUseDynamicPassives = fCoGIgnoreLow || fCoGIgnoreFar;

  //MGonCPU
  QJsonObject gcpuJson = ajson["CPUgridsOptions"].toObject();
  CGstartOption = gcpuJson["StartOption"].toInt();
  CGoptimizeWhat = gcpuJson["OptimizeWhat"].toInt();
  CGnodesXY = gcpuJson["NodesXY"].toInt();
  CGiterations = gcpuJson["Iterations"].toInt();
  CGinitialStep = gcpuJson["InitialStep"].toDouble();
  CGreduction = gcpuJson["Reduction"].toDouble();
  if (ReconstructionAlgorithm == 1)
    {
      fUseDynamicPassives = gcpuJson["DynamicPassives"].toBool();
      DynamicPassiveType = gcpuJson["PassiveType"].toInt();
    }

  //RootMini
  QJsonObject rootJson = ajson["RootMinimizerOptions"].toObject();
  RMstartOption = rootJson["StartOption"].toInt();
  RMminuitOption = rootJson["Minuit2Option"].toInt();
  fRMuseML = rootJson["LSorLikelihood"].toInt();
  RMstepX = rootJson["StartStepX"].toDouble();
  RMstepY = rootJson["StartStepY"].toDouble();
  RMstepZ = rootJson["StartStepZ"].toDouble();
  RMstepEnergy = rootJson["StartStepEnergy"].toDouble();
  RMmaxCalls = rootJson["MaxCalls"].toInt();
  fRMsuppressConsole = rootJson["LSsuppressConsole"].toBool();
  if (ReconstructionAlgorithm == 2)
    {
      fUseDynamicPassives = rootJson["DynamicPassives"].toBool();
      DynamicPassiveType = rootJson["PassiveType"].toInt();
    }
  //ANN
  ANNsettings = ajson["FANNsettings"].toObject();
  if (ReconstructionAlgorithm == 3) fUseDynamicPassives = false;

  //CUDA
  CGonCUDAsettings = ajson["CGonCUDAsettings"].toObject();

  //kNNreconstruct
  kNNrecSet = ajson["kNNrecSet"].toObject();

  if (ReconstructionAlgorithm == 4)
    {
      fUseDynamicPassives = CGonCUDAsettings["Passives"].toBool();
      DynamicPassiveType = CGonCUDAsettings["PassiveOption"].toInt();
    }

  //Multiples
  QJsonObject multJson = RecJson["MultipleEvents"].toObject();
  MultipleEventOption = multJson["Option"].toInt();

  //Limit nodes during reconstruction
  fLimitNodes = false;
  QJsonObject lnJson = RecJson["LimitNodes"].toObject();
  if (!lnJson.isEmpty())
    {
      fLimitNodes = lnJson["Active"].toBool();
      LimitNodesShape = lnJson["Shape"].toInt();
      LimitNodesSize1 = lnJson["Size1"].toDouble();
      LimitNodesSize2 = lnJson["Size2"].toDouble();
    }

  return true;
}
