//ANTS2 modules and windows
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "detectoraddonswindow.h"
#include "pms.h"
#include "outputwindow.h"
#include "geometrywindowclass.h"
#include "reconstructionwindow.h"
#include "photon_tracker.h"
#include "primaryparticletracker.h"
#include "s2_generator.h"
#include "sensorlrfs.h"
#include "gainevaluatorwindowclass.h"
#include "detectorclass.h"
#include "simulationmanager.h"
#include "guiutils.h"
#include "highlighterclass.h"
#include "genericscriptwindowclass.h"
#include "globalsettingsclass.h"
#include "materialinspectorwindow.h"
#include "checkupwindowclass.h"

//Qt
#include <QtGui>
#include <QtScript>
#include <QDialog>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPlainTextEdit>

void MainWindow::ReconstructDetector(bool fKeepData)
{
  //qDebug() << ">>>> Generate sandwich detector triggered <<<<";
  QJsonObject json;
  writeDetectorToJson(json);
  Detector->makeDetector(json);

  if (!fKeepData)
    {
      MainWindow::ClearData();
      MainWindow::PMnumberMightHaveChanged();
    }
  if (GeometryWindow->isVisible()) MainWindow::ShowGeometry(false);
  MainWindow::ShowPMcount();
  MainWindow::CheckPresenseOfSecScintillator(); //checks if SecScint present, and update selectors of primary/secondary scintillation
  DAwindow->UpdateObjectNames();  //update list of volumes which can be containers for custon objects
  Owindow->RefreshData();
  Owindow->ResetViewport();  // *** !!! extend to other windows too!
  onGDMLstatusChage(!Detector->isGDMLempty());

  if (!ui->cbFixedTopSize->isChecked())
    {
      ui->ledTopSizeXY->setText( QString::number(Detector->Sandwich.WorldSizeXY) );
      ui->ledTopSizeZ->setText( QString::number(Detector->Sandwich.WorldSizeZ) );
    }
  MainWindow::on_pbIndPMshowInfo_clicked(); //all indication in PM explorer
  MainWindow::on_pbElUpdateIndication_clicked(); //electronics
  MIwindow->UpdateSelection();
}

void MainWindow::CheckPresenseOfSecScintillator()
{
  Detector->checkSecScintPresent();

  if (!Detector->fSecScintPresent)
    {
      ui->cobScintTypePointSource->setCurrentIndex(0);
      ui->cbGunDoS2->setChecked(false);
      ui->cbDoS2tester->setChecked(false);
    }
  ui->cobScintTypePointSource->setEnabled(Detector->fSecScintPresent);
  ui->cbGunDoS2->setEnabled(Detector->fSecScintPresent);
  ui->cbDoS2tester->setEnabled(Detector->fSecScintPresent);
}

void MainWindow::PMnumberMightHaveChanged()
{  
  int NumPMs = PMs->count();

  //LRFmodule  
  SensLRF->clear(NumPMs); //geometry changed - old data is not relevent

  //this window
  if (ui->sbIndPMnumber->value() > NumPMs-1) ui->sbIndPMnumber->setValue(0);
  if (ui->sbElPMnumber->value() > NumPMs-1) ui->sbElPMnumber->setValue(0);
  if (ui->sbPreprocessigPMnumber->value() > NumPMs-1) ui->sbPreprocessigPMnumber->setValue(0);

  if (Rwindow)
    {
      Rwindow->PMnumChanged();
      Rwindow->UpdatePMgroupIndication();
    }
  if (Owindow)
   {    
     Owindow->RefreshData();
     Owindow->PMnumChanged();
   }
  if (GainWindow) GainWindow->Reset();

  ui->sbLoadedEnergyChannelNumber->setValue(NumPMs);  
  ui->fReloadRequired->setVisible(false); 
}

void MainWindow::on_pbCheckChangeSizes_clicked()
{
  QDialog dialog(this);
  dialog.resize(650,380);
  dialog.move(this->x()+100, this->y()+100);
  dialog.setWindowTitle("Custom sizes");
  QPushButton *okButton = new QPushButton("Accept");
  connect(okButton,SIGNAL(clicked()),&dialog,SLOT(accept()));
  QPushButton *cancelButton = new QPushButton("Cancel and exit");
  connect(cancelButton,SIGNAL(clicked()),&dialog,SLOT(reject()));
  QHBoxLayout *buttonsLayout = new QHBoxLayout;
  buttonsLayout->addStretch(1);
  buttonsLayout->addWidget(okButton);
  buttonsLayout->addStretch(1);
  buttonsLayout->addWidget(cancelButton);
  buttonsLayout->addStretch(1);

  QTableWidget *tw = new QTableWidget();
  tw->clearContents();

  int rows = 0;
  for (int i=0; i<SandwichLayers.size(); i++)
    if (SandwichLayers.at(i).fActive) rows++;
  tw->setRowCount(rows);

  int columns = 5;
  tw->setColumnCount(columns);
  tw->setHorizontalHeaderItem(0, new QTableWidgetItem("Rectangle = 0\nCylinder = 1\nPolygon = 2"));
  tw->setHorizontalHeaderItem(1, new QTableWidgetItem("--\n--\nSides"));
  tw->setHorizontalHeaderItem(2, new QTableWidgetItem("Size1, mm\nDiameter, mm\nSide, mm"));
  tw->setHorizontalHeaderItem(3, new QTableWidgetItem("Size2, mm\n--\n--"));
  tw->setHorizontalHeaderItem(4, new QTableWidgetItem("Angle, deg"));
  for (int i=0; i<columns; i++) tw->horizontalHeaderItem(i)->setTextAlignment(Qt::AlignHCenter);

  int index = 0;
  for (int i=0; i<SandwichLayers.size(); i++)
    if (SandwichLayers.at(i).fActive)
      {
        tw->setVerticalHeaderItem(index, new QTableWidgetItem(SandwichLayers.at(i).Name));
        tw->setItem(index, 0, new QTableWidgetItem(QString::number(SandwichLayers.at(i).Shape)));
        tw->setItem(index, 1, new QTableWidgetItem(QString::number(SandwichLayers.at(i).Sides)));
        tw->setItem(index, 2, new QTableWidgetItem(QString::number(SandwichLayers.at(i).Size1)));
        tw->setItem(index, 3, new QTableWidgetItem(QString::number(SandwichLayers.at(i).Size2)));
        tw->setItem(index, 4, new QTableWidgetItem(QString::number(SandwichLayers.at(i).Angle)));
        for (int j=0; j<columns; j++) tw->item(index, j)->setTextAlignment(Qt::AlignCenter);
        index++;
      }
  tw->setItemDelegate(new TableDoubleDelegateClass(tw)); //accept only doubles

  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(tw);
  mainLayout->addLayout(buttonsLayout);
  dialog.setLayout(mainLayout);

  do {
      bool fError;
      int result;
      do
        {
          result = dialog.exec();
          if (result == 0)
            { //cancel
              return;
            }
          //accept pressed, validating data
          fError = false;
          for (int i=0; i<rows; i++)
            {
              int Shape = tw->item(i, 0)->text().toInt();
              if (Shape<0 || Shape>2)
                {
                  message("Shape can be only 0 (rectangular), 1 (cylinder) or 2 (polygon)", this);
                  fError = true;
                  break;
                }
              int Sides = tw->item(i, 1)->text().toInt();
              if (Sides<0 || Sides>100)
                {
                  message("Sides can be only positive and are limited now to 100", this);
                  fError = true;
                  break;
                }
            }
        }
      while (fError);

      //updating data
      index = 0;
      for (int i=0; i<SandwichLayers.size(); i++)
        if (SandwichLayers[i].fActive)
          {
            SandwichLayers[i].Shape = tw->item(index, 0)->text().toInt();
            SandwichLayers[i].Sides = tw->item(index, 1)->text().toInt();
            SandwichLayers[i].Size1 = tw->item(index, 2)->text().toDouble();
            SandwichLayers[i].Size2 = tw->item(index, 3)->text().toDouble();
            SandwichLayers[i].Angle = tw->item(index, 4)->text().toDouble();
            index++;
          }
      MainWindow::ReconstructDetector();

    }
  while (true);
}

void MainWindow::on_pbPositionScript_clicked()
{
  extractGeometryOfScriptWindow();
  if (GenScriptWindow) delete GenScriptWindow;
  GenScriptWindow = new GenericScriptWindowClass();
  recallGeometryOfScriptWindow();
  int ul = ui->cobUpperLowerPMs->currentIndex();

  //configure the script window and engine
  PMscriptInterface = new InterfaceToPMscript(); //deleted by the GenScriptWindow
  GenScriptWindow->SetInterfaceObject(PMscriptInterface);
  QStringList coms;
  coms << "PM"<< "pm";
  GenScriptWindow->SetCustomCommands(coms);
  QString HelpText = "  Available commands:\n\n"
                     " pm(x0, y0)\n"
                     " PM(x0, y0, z0, type)\n"
                     " PM(x0, y0, z0, type, phi, theta, psi)\n";
  GenScriptWindow->SetShowEvaluationResult(false); //do not show "undefined"
  GenScriptWindow->SetExample("for (var i=0; i<5; i++) PM(i*10, (i-2)*20, 0, 0)");

  GenScriptWindow->SetTitle("Position PMs "+ (ul==0)?"- upper array":"- lower array");

  GenScriptWindow->SetScript(&Detector->PMarrays[ul].PositioningScript);

  GenScriptWindow->SetStarterDir(GlobSet->LibScripts);

  //define what to do on evaluation success
  connect(GenScriptWindow, SIGNAL(success(QString)), this, SLOT(PMscriptSuccess()));
  //if needed. connect signals of the interface object with the required slots of any ANTS2 objects
  GenScriptWindow->show();  
}

void MainWindow::PMscriptSuccess()
{
  QJsonArray &arr = PMscriptInterface->arr;
  if (arr.isEmpty())
    {
      GenScriptWindow->ShowText("No PMs were added!");
      return;
    }
  int ul = ui->cobUpperLowerPMs->currentIndex();
  PMarrayDataClass *PMarr = &Detector->PMarrays[ul];
  int reg = PMarr->Regularity;
  //checking data
  for (int ipm=0; ipm<arr.size(); ipm++)
    {
      QJsonArray el = arr[ipm].toArray();
      int size = el.size();
      if (reg == 2)
        {
          if (size < 4)
            {
              GenScriptWindow->ShowText("Cannot use 'pm' command with fully custom arrays - need Z coordinate!");
              return;
            }
        }
      if (el.size()>3)
        {
          int PMtype = el[3].toInt();
          if (PMtype > Detector->PMs->countPMtypes()-1)
            {
              GenScriptWindow->ShowText("Non-existent PM type!");
              return;
            }          
        }
      if (el.size()>4)
        {
           double phi = el[4].toDouble();
           double theta = el[5].toDouble();
//           double psi = el[6].toDouble();
//           if (reg == 0 && (phi!=0 || theta!=0 || psi!=0) )
//             {
//               GenScriptWindow->ShowText("Cannot assign angles for fully regular arrays!");
//               return;
//             }
           if (reg == 1 && (phi!=0 || theta!=0) )
             {
               //GenScriptWindow->ShowText("Cannot assign phi or theta angles - change to fully custom array geometry!");
               //return;
               qDebug() << "Changing regularity to fully custom";
               GenScriptWindow->ShowText("Changed array regularity to fully custom!");
               PMarr->Regularity = 2;
               reg = 2;
             }
        }
    }
  //copy data
  PMarr->PositionsAnglesTypes.clear();
  for (int ipm=0; ipm<arr.size(); ipm++)
      {
        QJsonArray el = arr[ipm].toArray();
        int size = el.size();
        //qDebug() << "Element to process:"<<el;
        double x = el[0].toDouble();
        double y = el[1].toDouble();
        if (size < 3)
          {
            PMarr->PositionsAnglesTypes.append(PosAngTypeStructure(x, y, 0, 0,0,0, 0));
            continue;
          }
        double z=0, phi=0, theta=0, psi=0;
        int type=0;
        if (size>2)
          {
            z = el[2].toDouble();
            type = el[3].toInt();
          }
        if (size>4)
          {
            phi = el[4].toDouble();
            theta = el[5].toDouble();
            psi = el[6].toDouble();
          }
        PMarr->PositionsAnglesTypes.append(PosAngTypeStructure(x, y, z, phi,theta,psi, type));
      }
  //clean array in case the script will be called again
  PMscriptInterface->arr = QJsonArray();
  ReconstructDetector();
  updatePMArrayDataIndication();

  int nooverlaps = CheckUpWindow->CheckGeoOverlaps();
  if (nooverlaps != 0) CheckUpWindow->show();
}

void MainWindow::writeWorldFixedToJson(QJsonObject &json)
{
  QJsonObject js;
  bool fOn = ui->cbFixedTopSize->isChecked();
  js["WorldSizeFixed"] = fOn;
  if (fOn)
    {
      js["XY"] = ui->ledTopSizeXY->text().toDouble();
      js["Z"] = ui->ledTopSizeZ->text().toDouble();
    }
  json["FixedWorldSizes"] = js;
}

void SandwichLayerGUIclass::writeToJson(QJsonObject &json)
{
  QJsonObject js;
  js["Active"] = fActive;
  if (fActive)
    {
      js["Height"] = Height;
      js["Material"] = Material;
      js["Size1"] = Size1;
      js["Size2"] = Size2;
      js["Shape"] = Shape;
      js["Sides"] = Sides;
      js["Angle"] = Angle;
    }
  json = js;
}

void MainWindow::writeDetectorLayersToJson(QJsonObject &json)
{
   QJsonObject sandwichj;

   QJsonArray lar;
   for (int i=0; i<SandwichLayers.size(); i++)
     {
       QJsonObject js;
       SandwichLayers[i].writeToJson(js);
       lar.append(js);
     }
   sandwichj["Layers"] = lar;
   bool fUseDefSizes = ui->cbUseSameSizes->isChecked();
   sandwichj["UseDefaultSizes"] = fUseDefSizes;   
   sandwichj["DefaultSize1"] = ui->ledSizeDiameter->text().toDouble();
   sandwichj["DefaultSize2"] = ui->ledSizeDiameter_2->text().toDouble();

   int Shape = ui->cobTubeBox->currentIndex();
   sandwichj["Shape"] = Shape;
   if (Shape == 2)
     {
       sandwichj["Sides"] = ui->sbPolygonSides->value();
       sandwichj["Angle"] = ui->ledPolygonAngle->text().toDouble();
     }
   sandwichj["TopMaterial"] = ui->cobTOP->currentIndex();

   json["SandwichSettings"] = sandwichj;
}

void MainWindow::initDetectorSandwich()
{
  SandwichLayers << SandwichLayerGUIclass("Spacer1",        ui->cbS1, ui->ledS1, ui->cobS1)
                 << SandwichLayerGUIclass("Upper window",   ui->cbUW, ui->ledUW, ui->cobUW)
                 << SandwichLayerGUIclass("Spacer2",        ui->cbS2, ui->ledS2, ui->cobS2)
                 << SandwichLayerGUIclass("Sec.Scint.",     ui->cbSS, ui->ledSS, ui->cobSS)
                 << SandwichLayerGUIclass("Spacer3",        ui->cbS3, ui->ledS3, ui->cobS3)
                 << SandwichLayerGUIclass("Primary Scint.", 0,        ui->ledPS, ui->cobPS)
                 << SandwichLayerGUIclass("Spacer4",        ui->cbS4, ui->ledS4, ui->cobS4)
                 << SandwichLayerGUIclass("Lower window",   ui->cbLW, ui->ledLW, ui->cobLW)
                 << SandwichLayerGUIclass("Spacer5",        ui->cbS5, ui->ledS5, ui->cobS5);

  double DetSize1 = ui->ledSizeDiameter->text().toDouble();
  double DetSize2 = ui->ledSizeDiameter_2->text().toDouble();
  for (int i=0; i<9;i++)
    {
      SandwichLayers[i].Size1 = DetSize1;
      SandwichLayers[i].Size2 = DetSize2;
    }
}

void SandwichLayerGUIclass::updateData()
{
  if (CBactive) fActive = CBactive->isChecked(); //Prim Scint has 0 pointer
  else fActive = true;
  Height = LEheight->text().toDouble();
  Material = COBmaterial->currentIndex();

  LEheight->setVisible(fActive);
  COBmaterial->setVisible(fActive);
}

void SandwichLayerGUIclass::updateGui()
{
  if (CBactive) CBactive->setChecked(fActive);
  LEheight->setVisible(fActive);
  COBmaterial->setVisible(fActive);
  if (fActive)
    {
      LEheight->setText( QString::number(Height));
      COBmaterial->setCurrentIndex(Material);
    }
}

void MainWindow::on_pbUpdateSandwich_clicked()
{  
  //qDebug() << "Sandwich update triggered";//<<fSimDataNotSaved;
  for (int i=0; i<SandwichLayers.size(); i++)
    SandwichLayers[i].updateData(); 

  if (ui->cbUseSameSizes->isChecked())
    for (int i=0; i<SandwichLayers.size(); i++)
      {
        SandwichLayers[i].Size1 = ui->ledSizeDiameter->text().toDouble();
        SandwichLayers[i].Size2 = ui->ledSizeDiameter_2->text().toDouble();
        SandwichLayers[i].Shape = ui->cobTubeBox->currentIndex();
        SandwichLayers[i].Sides = ui->sbPolygonSides->value();
        SandwichLayers[i].Angle = ui->ledPolygonAngle->text().toDouble();
      }
  MainWindow::ReconstructDetector();  
}
