//ANTS2 modules and windows
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "pms.h"
#include "materialinspectorwindow.h"
#include "reconstructionwindow.h"
#include "outputwindow.h"
#include "windownavigatorclass.h"
#include "primaryparticletracker.h"
#include "exampleswindow.h"
#include "detectoraddonswindow.h"
#include "photon_generator.h"
#include "classes.h"
#include "jsonparser.h"
#include "detectorclass.h"
#include "detectorsandwichclass.h"
#include "pmtypeclass.h"
#include "geometrywindowclass.h"
#include "globalsettingsclass.h"
//#include "simulationmanager.h"

//Qt
#include <QMessageBox>
#include <QtGui>
#include <QFileDialog>

//Root
#include "TTree.h"
#include "TFile.h"

void MainWindow::writeDetectorToJson(QJsonObject &json)
{
  QJsonObject js;
  Detector->writeParticleCollectionToJson(js);      //Particle collection
  Detector->MaterialCollection->writeToJson(js);    //Materials collection including overrides
  writeWorldFixedToJson(js);                        //Fixed world size - if activated
  writeDetectorLayersToJson(js);                    //Detector layers
  PMs->writePMtypesToJson(js);                      //PM array types
  Detector->writePMarraysToJson(js);                //PM arrays
  if (Detector->PMdummies.size()>0)
    Detector->writeDummyPMsToJson(js);              //dummy PMs
  if (Detector->Lightguides[0].fActive ||
      Detector->Lightguides[1].fActive)
    Detector->writeLightguidesToJson(js);           //Lightguides
  if (Detector->AdditionalGeoObjects.size()>0)
    Detector->writeAddGeoObjectsToJson(js);         //AddGeoObjects
//  if (Detector->PrScintCont.fActive)
//    Detector->PrScintCont.writeToJson(js);          //PrScintCont
  js["PrScintCont"] = Detector->fPrScintCont;       //PrScint to container
  if (!Detector->Masks.isEmpty())
    Detector->Masks.writeToJson(js);                //Masks
  if (Detector->Grids.size()>0)
    Detector->writeGridsToJson(js);                 //Optical grids
  Detector->PMs->writeInividualOverridesToJson(js); //Overrides for individual PMs if defined
  Detector->PMs->writeElectronicsToJson(js);        //Electronics
  if (!Detector->isGDMLempty())
    Detector->writeGDMLtoJson(js);                  //GDML
  if (ui->cbPMsignalPreProcessing->isChecked())
    writeLoadExpDataConfigToJson(js);               //LoadData preprocessing settings
  json["DetectorConfig"] = js;

  QJsonObject js1;
  js1["DetectorConfig"] = js;
  SaveJsonToFile(js1, "DetectorConfig.json");
}

bool MainWindow::readDetectorFromJson(QJsonObject &json)
{  
  GeometryWindow->fRecallWindow = false;
  bool fOK = Detector->makeDetector(json);
  if (!fOK)
    {
      qCritical() << "Error while loading detector config";
      return false;
    }

  DoNotUpdateGeometry = true;
  //Particles
  MainWindow::ListActiveParticles();
  //Materials
  MainWindow::on_pbRefreshMaterials_clicked();
  MIwindow->UpdateActiveMaterials(); //refresh indication on Material Inspector window
  MIwindow->SetMaterialSelection(0);
  MIwindow->setLogLog(Detector->MaterialCollection->fLogLogInterpolation);
  //Material overrides
  int iFrom, iTo;
  Detector->MaterialCollection->getFirstOverridenMaterial(iFrom, iTo);
  ui->cobMaterialForOverrides->setCurrentIndex(iFrom);
  ui->cobMaterialTo->setCurrentIndex(iTo);
  //MainWindow::UpdateOverridePercetageIndication();
  MainWindow::on_pbRefreshOverrides_clicked();
  //PM types
  ui->sbPMtype->setValue(0); //protection: could be less types than before
  MainWindow::updateCOBsWithPMtypeNames();
  MainWindow::on_pbRefreshPMproperties_clicked();
  //PM arrays
  ui->cbUPM->setChecked(Detector->PMarrays[0].fActive);
  ui->cbLPM->setChecked(Detector->PMarrays[1].fActive);
  MainWindow::on_pbShowPMsArrayRegularData_clicked(); //update regular array fields
  //Detector addon window updates
  DAwindow->UpdateGUI();
  if (Detector->PMdummies.size()>0) DAwindow->UpdateDummyPMindication();
  //world
  ui->ledTopSizeXY->setText( QString::number(Detector->Sandwich.WorldSizeXY) );
  ui->ledTopSizeZ->setText( QString::number(Detector->Sandwich.WorldSizeZ) );
  ui->cbFixedTopSize->setChecked( Detector->Sandwich.fWorldSizeFixed );
  //Sandwich general
  ui->ledSizeDiameter->setText( QString::number(Detector->Sandwich.DefaultSizeX) );
  ui->ledSizeDiameter_2->setText( QString::number(Detector->Sandwich.DefaultSizeY) );
  ui->cbUseSameSizes->setChecked( Detector->Sandwich.fUseDefaultSizes );
  ui->cobTubeBox->setCurrentIndex( Detector->Sandwich.Shape );
  ui->sbPolygonSides->setValue( Detector->Sandwich.Sides );
  ui->ledPolygonAngle->setText( QString::number(Detector->Sandwich.Angle ) );
  ui->cobTOP->setCurrentIndex( Detector->Sandwich.TopMaterial );
  //Sandwich layers
  for (int i=0; i<Detector->Sandwich.Layers.size(); i++)
    {
      SandwichLayerGUIclass &g = SandwichLayers[i];
      SandwichLayerClass &m = Detector->Sandwich.Layers[i];
      g.fActive = m.fActive;
      if (g.fActive) (BaseLayerClass&)g = (BaseLayerClass&)m;
      g.updateGui();
    }
  //PM explore
  MainWindow::on_pbIndPMshowInfo_clicked();
  //Electronics
  MainWindow::on_pbElUpdateIndication_clicked();
  //GDML?
  onGDMLstatusChage(!Detector->isGDMLempty());

  Detector->checkSecScintPresent();
  ui->fSecondaryLightGenType->setEnabled(Detector->fSecScintPresent);
  readLoadExpDataConfigFromJson(json); //preprocessing GUI settings

  DoNotUpdateGeometry = false;
  return true;
}

void MainWindow::writeExtraGuiToJson(QJsonObject &json)
{
  QJsonObject js;
  js["Particle"] = ui->cobParticleToStack->currentIndex();
  js["Copies"] = ui->sbNumCopies->value();
  js["X"] = ui->ledParticleStackX->text().toDouble();
  js["Y"] = ui->ledParticleStackY->text().toDouble();
  js["Z"] = ui->ledParticleStackZ->text().toDouble();
  js["dX"] = ui->ledParticleStackVx->text().toDouble();
  js["dY"] = ui->ledParticleStackVy->text().toDouble();
  js["dZ"] = ui->ledParticleStackVz->text().toDouble();
  js["Energy"] = ui->ledParticleStackEnergy->text().toDouble();
  js["Time"] = ui->ledParticleStackTime->text().toDouble();
  js["ScriptEV"] = CheckerScript;
  json["PartcleStackChecker"] = js;

  QJsonObject jeom;
  jeom["ZoomLevel"] = GeometryWindow->ZoomLevel;
  json["GeometryWindow"] = jeom;
}

void MainWindow::readExtraGuiFromJson(QJsonObject &json)
{
  QJsonObject js = json["PartcleStackChecker"].toObject();
  if (!js.isEmpty())
    {
      JsonToComboBox(js, "Particle", ui->cobParticleToStack);
      JsonToSpinBox(js, "Copies", ui->sbNumCopies);
      JsonToLineEdit(js, "X", ui->ledParticleStackX);
      JsonToLineEdit(js, "Y", ui->ledParticleStackY);
      JsonToLineEdit(js, "Z", ui->ledParticleStackZ);
      JsonToLineEdit(js, "dX", ui->ledParticleStackVx);
      JsonToLineEdit(js, "dY", ui->ledParticleStackVy);
      JsonToLineEdit(js, "dZ", ui->ledParticleStackVz);
      JsonToLineEdit(js, "Energy", ui->ledParticleStackEnergy);
      JsonToLineEdit(js, "Time", ui->ledParticleStackTime);
      parseJson(js, "ScriptEV", CheckerScript);
    }

  js = json["GeometryWindow"].toObject();
  if (js.contains("ZoomLevel"))
    GeometryWindow->ZoomLevel = js["ZoomLevel"].toInt();
}

void MainWindow::writeSimSettingsToJson(QJsonObject &json, bool fVerbose)
{
  QJsonObject js;
  SimGeneralConfigToJson(js);         //general sim settings
  if (ui->twSourcePhotonsParticles->currentIndex() == 0)
    { //point source sim
      js["Mode"] = "PointSim";
      SimPointSourcesConfigToJson(js, fVerbose);
      if (fVerbose) SimParticleSourcesConfigToJson(js);
    }
  else
    { //particle source sim
      js["Mode"] = "SourceSim";
      SimParticleSourcesConfigToJson(js);
      if (fVerbose) SimPointSourcesConfigToJson(js, fVerbose);
    }
  js["DoGuiUpdate"] = true;           //batcher have to set it to false!
  json["SimulationConfig"] = js;

  QJsonObject js1;
  js1["SimulationConfig"] = js;
  SaveJsonToFile(js1, "SimConfig.json");
}

bool MainWindow::readSimSettingsFromJson(QJsonObject &json)
{
  if (!json.contains("SimulationConfig"))
    {
      qWarning() << "Json does not contain sim settings!";
      return false;
    }

  //cleanup
  MainWindow::ClearData();
  if (histScan) delete histScan;
  histScan = 0;
  if (histSecScint) delete histSecScint;
  histSecScint = 0;
  ui->pbScanDistrShow->setEnabled(false);
  ui->pbScanDistrDelete->setEnabled(false);
  populateTable = true;
  if (ParticleStack.size()>0)
    {
      for (int i=0; i<ParticleStack.size(); i++) delete ParticleStack[i];
      ParticleStack.resize(0);
    }

  //cleaning particle sources
  ParticleSources.clear();
  ParticleSources.append(new ParticleSourceStructure());
  MainWindow::on_pbUpdateSourcesIndication_clicked(); //to clean indication
  ParticleSources.clear();
  MainWindow::on_pbUpdateSourcesIndication_clicked();

  DoNotUpdateGeometry = true;
  BulkUpdate = true;

  QJsonObject js = json["SimulationConfig"].toObject();  

  //GENERAL SETTINGS
  QJsonObject gjs = js["GeneralSimConfig"].toObject();  
  // wave
  QJsonObject wj = gjs["WaveConfig"].toObject();
  JsonToCheckbox(wj, "WaveResolved", ui->cbWaveResolved);
  JsonToLineEdit(wj, "WaveFrom", ui->ledWaveFrom);
  JsonToLineEdit(wj, "WaveTo", ui->ledWaveTo);
  JsonToLineEdit(wj, "WaveStep", ui->ledWaveStep);
  // time
  QJsonObject tj = gjs["TimeConfig"].toObject();
  JsonToCheckbox(tj, "TimeResolved", ui->cbTimeResolved);
  JsonToLineEdit(tj, "TimeFrom", ui->ledTimeFrom);
  JsonToLineEdit(tj, "TimeTo", ui->ledTimeTo);
  JsonToSpinBox (tj, "TimeBins", ui->sbTimeBins);
    //JsonToLineEdit(tj, "TimeWindow", ui->ledTimeOfOneMeasurement); //moved to detector config
  // Angular
  QJsonObject aj = gjs["AngleConfig"].toObject();
  JsonToCheckbox(aj, "AngResolved", ui->cbAngularSensitive);
  JsonToSpinBox (aj, "NumBins", ui->sbCosBins);
  // Area
  QJsonObject arj = gjs["AreaConfig"].toObject();
  JsonToCheckbox(arj, "AreaResolved", ui->cbAreaSensitive);
  //Electronics
  QJsonObject ej = gjs["ElectronicsConfig"].toObject();
  JsonToCheckbox(ej, "SPS", ui->cbEnableSPePHS);
  JsonToCheckbox(ej, "Noise", ui->cbEnableElNoise);
  JsonToCheckbox(ej, "ADC", ui->cbEnableADC);
  //Tracking options
  QJsonObject trj = gjs["TrackingConfig"].toObject();
  JsonToLineEdit(trj, "MinStep", ui->ledMinStep);
  JsonToLineEdit(trj, "MaxStep", ui->ledMaxStep);
  JsonToLineEdit(trj, "dE", ui->ledDE);
  JsonToLineEdit(trj, "MinEnergy", ui->ledMinEnergy);
  JsonToLineEdit(trj, "Safety", ui->ledSafety);
  JsonToSpinBox(trj, "TrackColorAdd", ui->sbParticleTrackColorIndexAdd);
  //Accelerators
  QJsonObject acj = gjs["AcceleratorConfig"].toObject();
  JsonToSpinBox (acj, "MaxNumTransitions", ui->sbMaxNumbPhTransitions);
  JsonToCheckbox(acj, "CheckBeforeTrack", ui->cbRndCheckBeforeTrack);
  JsonToCheckbox(acj, "OnlyTracksOnPMs", ui->cbOnlyBuildTracksOnPMs);
  JsonToCheckbox(acj, "LogsStatistics", ui->cbDoLogsAndStatistics);
  JsonToSpinBox(acj, "NumberThreads", ui->sbNumberThreads);
  //Sec scint
  QJsonObject scj = gjs["SecScintConfig"].toObject();
  JsonToComboBox(scj, "Type", ui->cobSecScintillationGenType);
  /*
if (scj.contains("CustomDistrib"))
  {
    QJsonArray ja = scj["CustomDistrib"].toArray();
    int size = ja.size();
    if (size>0)
      {
        double* xx = new double[size];
        double* yy = new double[size];
        for (int i=0; i<size; i++)
            {
              xx[i] = ja[i].toArray()[0].toDouble();
              yy[i] = ja[i].toArray()[1].toDouble();
            }
        histSecScint = new TH1D("SecScintTheta","SecScint: Theta", size-1, xx);
        for (int i = 1; i<size+1; i++)  histSecScint->SetBinContent(i, yy[i-1]);
        delete[] xx;
        delete[] yy;
      }
  }
*/
  JsonToCheckbox(gjs, "BuildPhotonTracks", ui->cbPointSourceBuildTracks); //general now

  //POINT SOURCES
  QJsonObject pojs = js["PointSourcesConfig"].toObject();
  //control
  QJsonObject pcj = pojs["ControlOptions"].toObject();
  int SimMode = pcj["Single_Scan_Flood"].toInt();
  if (SimMode < ui->twSingleScan->count()) ui->twSingleScan->setCurrentIndex(SimMode);
  JsonToCheckbox(pcj, "UseLRFs", ui->cbLRFs); // *** update GUI ?
  JsonToSpinBox (pcj, "NumPhotsLRFunity", ui->sbLRFphotons);
  JsonToComboBox(pcj, "Primary_Secondary", ui->cobScintTypePointSource);
  JsonToCheckbox(pcj, "BuildTracks", ui->cbPointSourceBuildTracks);
  JsonToCheckbox(pcj, "MultipleRuns", ui->cbNumberOfRuns);
  JsonToSpinBox (pcj, "MultipleRunsNumber", ui->sbNumberOfRuns);
  JsonToCheckbox(pcj, "GenerateOnlyInPrimary", ui->cbScanFloodLimitToPrimaryScint);
  //potons per node;
  QJsonObject ppj = pojs["PhotPerNodeOptions"].toObject();
  JsonToComboBox(ppj, "PhotPerNodeMode", ui->cobScanNumPhotonsMode);
  JsonToSpinBox (ppj, "PhotPerNodeConstant", ui->sbScanNumPhotons);
  JsonToSpinBox (ppj, "PhotPerNodeUniMin", ui->sbScanNumMin);
  JsonToSpinBox (ppj, "PhotPerNodeUniMax", ui->sbScanNumMax);
  JsonToSpinBox (ppj, "PhotPerNodeGaussMean", ui->sbScanGaussMean);
  JsonToSpinBox (ppj, "PhotPerNodeGaussSigma", ui->sbScanGaussSigma);
  if (ppj.contains("PhotPerNodeCustom"))
    {
      QJsonArray ja = ppj["PhotPerNodeCustom"].toArray();
      int size = ja.size();
      if (size > 0)
        {
          double* xx = new double[size];
          int* yy    = new int[size];
          for (int i=0; i<size; i++)
            {
              xx[i] = ja[i].toArray()[0].toDouble();
              yy[i] = ja[i].toArray()[1].toInt();
            }
          histScan = new TH1I("histPhotDistr","Photon distribution", size-1, xx);
          histScan->GetIntegral(); //to make it thread safe
          histScan->SetXTitle("Number of generated photons");
          histScan->SetYTitle("Relative probability");
          for (int i = 1; i<size+1; i++) histScan->SetBinContent(i, yy[i-1]);
          delete[] xx;
          delete[] yy;
          ui->pbScanDistrShow->setEnabled(true);
          ui->pbScanDistrDelete->setEnabled(true);
        }
    }
  //Wavelength/decay options
  QJsonObject wdj = pojs["WaveTimeOptions"].toObject();
  JsonToComboBox(wdj, "Direct_Material", ui->cobDirectlyOrFromMaterial);
  JsonToSpinBox (wdj, "WaveIndex", ui->sbWaveIndexPointSource);
  JsonToLineEdit(wdj, "DecayTime", ui->ledDecayTime);
  JsonToComboBox(wdj, "Material", ui->cobMatPointSource);
  //Photon direction options
  QJsonObject pdj = pojs["PhotonDirectionOptions"].toObject();
  JsonToLineEdit(pdj, "FixedX", ui->ledSingleDX);
  JsonToLineEdit(pdj, "FixedY", ui->ledSingleDY);
  JsonToLineEdit(pdj, "FixedZ", ui->ledSingleDZ);
  JsonToCheckbox(pdj, "Random", ui->cbRandomDir);
  //bad event config
  QJsonObject bej = pojs["BadEventOptions"].toObject();
  JsonToCheckbox(bej, "BadEvents", ui->cbScanFloodAddNoise);
  JsonToLineEdit(bej, "Probability", ui->leoScanFloodNoiseProbability);
  JsonToLineEdit(bej, "SigmaDouble", ui->ledScanFloodNoiseOffset);
  if (bej.contains("NoiseArray"))
    {
      QJsonArray ar = bej["NoiseArray"].toArray();
      if (ar.size() == 6)
        {
          for (int ic=0; ic<6; ic++)
            {
              QJsonObject js = ar[ic].toObject();
              ScanFloodNoise[ic].Active = js["Active"].toBool();
              //ScanFloodNoise[ic].Description = js["Description"].toString(); //keep standart description!
              ScanFloodNoise[ic].Probability = js["Probability"].toDouble();
              ScanFloodNoise[ic].AverageValue = js["AverageValue"].toDouble();
              ScanFloodNoise[ic].Spread = js["Spread"].toDouble();
            }
        }
      MainWindow::PointSource_UpdateTabWidget();
    }
  //Single position options
  QJsonObject spj = pojs["SinglePositionOptions"].toObject();
  JsonToLineEdit(spj, "SingleX", ui->ledSingleX);
  JsonToLineEdit(spj, "SingleY", ui->ledSingleY);
  JsonToLineEdit(spj, "SingleZ", ui->ledSingleZ);
  //Regular scan options
  QJsonObject rsj = pojs["RegularScanOptions"].toObject();
  JsonToLineEdit(rsj, "ScanX0", ui->ledOriginX);
  JsonToLineEdit(rsj, "ScanY0", ui->ledOriginY);
  JsonToLineEdit(rsj, "ScanZ0", ui->ledOriginZ);
  ui->cbSecondAxis->setChecked(false);
  ui->cbThirdAxis->setChecked(false);
  if (rsj.contains("AxesData"))
    {
      QJsonArray ar = rsj["AxesData"].toArray();
      if (ar.size()>0)
        {
          QJsonObject js = ar[0].toObject();
          JsonToLineEdit(js, "dX", ui->led0X);
          JsonToLineEdit(js, "dY", ui->led0Y);
          JsonToLineEdit(js, "dZ", ui->led0Z);
          JsonToSpinBox (js, "Nodes", ui->sb0nodes);
          JsonToComboBox(js, "Option", ui->cob0dir);
        }
      if (ar.size()>1)
        {
          QJsonObject js = ar[1].toObject();
          JsonToLineEdit(js, "dX", ui->led1X);
          JsonToLineEdit(js, "dY", ui->led1Y);
          JsonToLineEdit(js, "dZ", ui->led1Z);
          JsonToSpinBox (js, "Nodes", ui->sb1nodes);
          JsonToComboBox(js, "Option", ui->cob1dir);
          ui->cbSecondAxis->setChecked(true);
        }
      if (ar.size()>2)
        {
          QJsonObject js = ar[2].toObject();
          JsonToLineEdit(js, "dX", ui->led2X);
          JsonToLineEdit(js, "dY", ui->led2Y);
          JsonToLineEdit(js, "dZ", ui->led2Z);
          JsonToSpinBox (js, "Nodes", ui->sb2nodes);
          JsonToComboBox(js, "Option", ui->cob2dir);
          ui->cbThirdAxis->setChecked(true);
        }
    }
  //Flood options
  QJsonObject fj = pojs["FloodOptions"].toObject();
  JsonToSpinBox (fj, "Nodes", ui->sbScanFloodNodes);
  JsonToComboBox(fj, "Shape", ui->cobScanFloodShape);
  JsonToLineEdit(fj, "Xfrom", ui->ledScanFloodXfrom);
  JsonToLineEdit(fj, "Xto", ui->ledScanFloodXto);
  JsonToLineEdit(fj, "Yfrom", ui->ledScanFloodYfrom);
  JsonToLineEdit(fj, "Yto", ui->ledScanFloodYto);
  JsonToLineEdit(fj, "CenterX", ui->ledScanFlood_CenterX);
  JsonToLineEdit(fj, "CenterY", ui->ledScanFlood_CenterY);
  JsonToLineEdit(fj, "DiameterOut", ui->ledScanFlood_Radius);
  JsonToLineEdit(fj, "DiameterIn", ui->ledScanFlood_Radius0);
  JsonToComboBox(fj, "Zoption", ui->cobScanFloodZtype);
  JsonToLineEdit(fj, "Zfixed", ui->ledScanFloodZ);
  JsonToLineEdit(fj, "Zfrom", ui->ledScanFloodZfrom);
  JsonToLineEdit(fj, "Zto", ui->ledScanFloodZto);
  //Custom nodes
  QJsonObject njson = pojs["CustomNodesOptions"].toObject();
  NodesScript.clear();
  parseJson(njson, "Script", NodesScript);
  QJsonArray arr;
  parseJson(njson, "Nodes", arr);
  CustomScanNodes.clear();
  for (int i=0; i<arr.size(); i++)
    {
      QJsonArray el = arr[i].toArray();
      if (el.size()<3)
        {
          qWarning() << "Error in custom nodes array";
          break;
        }
      CustomScanNodes.append( QVector3D(el[0].toDouble(), el[1].toDouble(), el[2].toDouble()));
    }
  ui->lScriptNodes->setText( QString::number(CustomScanNodes.size()) );

  //PARTICLE SOURCES
  QJsonObject psjs = js["ParticleSourcesConfig"].toObject();
  //full sources mode
  QJsonObject csjs = psjs["SourceControlOptions"].toObject();
  JsonToSpinBox (csjs, "EventsToDo", ui->sbGunEvents);
  JsonToCheckbox(csjs, "AllowMultipleParticles", ui->cbGunAllowMultipleEvents);
  JsonToLineEdit(csjs, "AverageParticlesPerEvent", ui->ledGunAverageNumPartperEvent);
  JsonToCheckbox(csjs, "DoS1", ui->cbGunDoS1);
  JsonToCheckbox(csjs, "DoS1", ui->cbDoS1tester);
  JsonToCheckbox(csjs, "DoS2", ui->cbGunDoS2);
  JsonToCheckbox(csjs, "DoS2", ui->cbDoS2tester);
  JsonToCheckbox(csjs, "ParticleTracks", ui->cbGunParticleTracks);
  JsonToCheckbox(csjs, "ParticleTracks", ui->cbBuildParticleTrackstester);
  JsonToCheckbox(csjs, "PhotonTracks", ui->cbGunPhotonTracks);
  JsonToCheckbox(csjs, "PhotonTracks", ui->cbBuilPhotonTrackstester);
  ui->cbIgnoreEventsWithNoHits->setChecked(false);//compatibility
  JsonToCheckbox(csjs, "IgnoreNoHitsEvents", ui->cbIgnoreEventsWithNoHits);
  //particle sources
  ParticleSources.readFromJson(psjs);

  //Window CONTROL
  if (js.contains("Mode"))
    {
      QString Mode = json["Mode"].toString();
      if (Mode == "PointSim") ui->twSourcePhotonsParticles->setCurrentIndex(0);
      else if (Mode =="StackSim" || Mode =="SourceSim") ui->twSourcePhotonsParticles->setCurrentIndex(1);
    }

  DoNotUpdateGeometry = false;
  BulkUpdate = false;

  //updating global parameters
  MainWindow::on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked()); //update on toggle
  MainWindow::on_cbAngularSensitive_toggled(ui->cbAngularSensitive->isChecked());
  MainWindow::on_cbTimeResolved_toggled(ui->cbTimeResolved->isChecked());

  //update indication
  MainWindow::on_pbRefreshStack_clicked();

  //particle sources indication
  MainWindow::on_pbUpdateSourcesIndication_clicked();
  if (ParticleSources.getTotalActivity() > 0)
    {  //show the first source with non-zero activity
      int i;
      for (i=0; i<ParticleSources.size(); i++)
        if (ParticleSources[i].Activity>0) break;
      ui->cobParticleSource->setCurrentIndex(i);
      MainWindow::on_pbUpdateSourcesIndication_clicked();
    }
  MainWindow::on_pbGunRefreshparticles_clicked();

  return true;
}

void MainWindow::SavePhElToSignalData(QString fileName)
{
  QFile outputFile(fileName);
  outputFile.open(QIODevice::WriteOnly);
  if(!outputFile.isOpen())
    {
          message("Unable to open file " +fileName+ " for writing!", this);
          return;
     }

  QTextStream outStream(&outputFile);
  QFileInfo QF(fileName);

  for (int ipm = 0; ipm<PMs->count(); ipm++)
    {
     // outStream<<ipm<<" ";
      int mode = PMs->getSPePHSmode(ipm);
      outStream<<mode<<" ";
      switch (mode)
        {
        case 0:
          outStream<<PMs->getAverageSignalPerPhotoelectron(ipm);
          break;
        case 1:
          outStream<<PMs->getAverageSignalPerPhotoelectron(ipm)<<" ";
          outStream<<PMs->getSPePHSsigma(ipm);
          break;
        case 2:
          outStream<<PMs->getAverageSignalPerPhotoelectron(ipm)<<" ";
          outStream<<PMs->getSPePHSshape(ipm);
          break;
        case 3:
          {
            QString str;
            str.setNum(ipm);
            outStream<<"SPePHS_"+str+".dat";
            SaveDoubleVectorsToFile(QF.path()+"/"+"SPePHS_"+str+".dat", PMs->getSPePHS_x(ipm), PMs->getSPePHS(ipm));
          }
        }
      outStream<<"\n";
    }
  outputFile.close();
}

void MainWindow::LoadPhElToSignalData(QString fileName)
{
  QFile file(fileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: "+fileName, this);
      return;
    }

  QFileInfo QF(fileName);
  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\t)"); //separators: ' ' or ',' or '\t'

  int ipm = -1;
  while(!in.atEnd())
       {
          QString line = in.readLine();
          QStringList fields = line.split(rx, QString::SkipEmptyParts);

 //         qDebug()<<"line: "<<line;
 //         qDebug()<<"fields="<<fields.count();
          if (fields.count()<2) continue;

          int mode = fields[0].toInt();
          if ( (mode==1 || mode==2) && fields.count()<3) continue;
 //         qDebug()<<"adding these data";
          ipm++;
          if (ipm>PMs->count()) continue; //protection, not using break to compare with actual numPMs below

          PMs->setSPePHSmode(ipm, mode);
          switch (mode)
            {
            case 0:
              {
              double mean = fields[1].toDouble();
              PMs->setAverageSignalPerPhotoelectron(ipm, mean);
              break;
              }
            case 1:
              {
              double mean = fields[1].toDouble();
              PMs->setAverageSignalPerPhotoelectron(ipm, mean);
              double sigma = fields[2].toDouble();
              PMs->setSPePHSsigma(ipm, sigma);
              break;
              }
            case 2:
              {
              double mean = fields[1].toDouble();
              PMs->setAverageSignalPerPhotoelectron(ipm, mean);
              double shape = fields[2].toDouble();
              PMs->setSPePHSshape(ipm, shape);
              break;
              }
            case 3:
              {
                QString name = fields[1];
                QVector<double> x, y;
                int error = LoadDoubleVectorsFromFile(QF.path()+"/"+name, &x, &y);
                if (error == 0)
                  {
                    PMs->setElChanSPePHS(ipm, &x, &y);
                  }
              }
            }
        }
   file.close();

   ipm++; //since started from 0
   if (ipm != PMs->count())
   {
       QString str;
       str.setNum(ipm);
       message("Warning: wrong number of PMs (" +str+ ") in the file:"+fileName, this);
   }
}

int MainWindow::LoadAreaResponse(QString fileName, QVector<QVector<double> >* tmp, double* xStep, double* yStep)
{
  QFile file(fileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("failed to open file "+fileName, this);
      return 1;
    }

  QTextStream in(&file);
  QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'

  QString line = in.readLine();
  QStringList fields = line.split(rx);

  if (fields[0].compare("PM-AreaResponse"))
    {
      message("Unrecognized format!", this);
      return 2;
    }
  bool ok1, ok2, ok3, ok4;
  int xNum = fields[1].toInt(&ok1);
  *xStep = fields[2].toDouble(&ok2);  //*** potential problem with decimal separator!
  int yNum = fields[3].toInt(&ok3);
  *yStep = fields[4].toDouble(&ok4);
  if (!ok1 || !ok2 || !ok3 || !ok4)
    {
      message("Unrecognized format!", this);
      return 2;
    }

  qDebug()<<xNum<<*xStep<<yNum<<*yStep;
  //QVector< QVector <double>> tmp;
  tmp->resize(xNum);
  for (int i=0; i<xNum; i++) (*tmp)[i].resize(yNum);

  for (int iY = 0; iY<yNum; iY++)
       {
          if (in.atEnd())
            {
              message("Unexpected end of file reached!", this);
              file.close();
              return 100;
            }
          line = in.readLine();
          fields = line.split(rx, QString::SkipEmptyParts);
          if (fields.size() != xNum)
            {
              message("Unexpected end of line reached!", this);
              file.close();
              return 110;
            }

          for (int iX=0; iX < xNum; iX++)
            {
              double val = fields[iX].toDouble();
              if (val<0 || val>1.0)
                {
                  message("response values should be in range from 0 to 1.0!", this);
                  file.close();
                  return 120;
                }
              (*tmp)[iX][yNum-1-iY] = val; //x: -to+ y: +to-
            }
        }

   file.close();
   return 0;
}

int MainWindow::LoadSPePHSfile(QString fileName, QVector<double>* SPePHS_x, QVector<double>* SPePHS) //0 - no errors
{
  int error = LoadDoubleVectorsFromFile(fileName, SPePHS_x, SPePHS);

  if (error != 0) return error; //message is in the loader function

  double sum = 0;
  for (int i=0; i<SPePHS->size(); i++) sum+= SPePHS->at(i);

  if (TMath::Abs(sum) < 0.0000001)
    {
      message("Sum of all probabilities cannot be zero!", this);
      return 10;
    }

  return error;
}

int MainWindow::LoadPMsignals(QString fileName)
{ 
  qDebug()<<"Loading event data from file: "<<fileName;
  QJsonObject json;
  writeLoadExpDataConfigToJson(json);
  //if (ui->cbPrePro_BacktrackElectronics->isChecked()) Detector->PMs->writeElectronicsToJson(json);
  QJsonObject js;
  js["DetectorConfig"] = json;
  WindowNavigator->BusyOn();
  ui->prbarLoadData->setValue(0);
  ui->fLoadProgress->setVisible(true);
  SaveJsonToFile(js, "Preprocessing.json");
  int loaded = EventsDataHub->loadEventsFromTxtFile(fileName, js, Detector->PMs);
  ui->fLoadProgress->setVisible(false);
  WindowNavigator->BusyOff();
  if (loaded<0) message(EventsDataHub->ErrorString, this);
  //qDebug() << "Load reported:"<<loaded;
  return loaded;
}

void MainWindow::updateLoaded(int events, int progress)
{
  ui->leoLoadedEvents->setText(QString::number(events));
  ui->prbarLoadData->setValue(progress);
}

void MainWindow::LoadSimulationDataFromTree(QString fileName)
{
  int numEv = EventsDataHub->loadSimulatedEventsFromTree(fileName, Detector->PMs);

  ui->leoTotalLoadedEvents->setText(QString::number(EventsDataHub->Events.size()));
  if (numEv != -1)
    {
      ui->lwLoadedSims->addItem(fileName);
      ui->leoLoadedEvents->setText(QString::number(numEv));
    }
  else
    {
      message(EventsDataHub->ErrorString, this);
      ui->leoLoadedEvents->setText("0");
    }
}

void MainWindow::LoadScanPhotonDistribution(QString fileName)
{
  QVector<double> x, y;
  int error = LoadDoubleVectorsFromFile(fileName, &x, &y);
  if (error>0) return;

  if (histScan) delete histScan;
  int size = x.size();
  double* xx = new double [size];
  for (int i = 0; i<size; i++) xx[i]=x[i];
  histScan = new TH1I("histPhotDistr","", size-1, xx);
//  histScan = new TH1I("histScan","", size, x[0], x[size-1]);
  for (int j = 1; j<size+1; j++)  histScan->SetBinContent(j, y[j-1]);
  histScan->GetIntegral(); //to make thread safe
  histScan->SetXTitle("Number of generated photons");
  histScan->SetYTitle("Probability");

  ui->pbScanDistrShow->setEnabled(true);
  ui->pbScanDistrDelete->setEnabled(true);
}

void MainWindow::ExportDeposition(QFile &outputFile)
{
  if(!outputFile.isOpen() || !outputFile.isWritable())
          return;

  QTextStream outStream(&outputFile);

  outStream << ">Materials:\n";
  for (int i=0; i<MaterialCollection->size(); i++) outStream << "> " << i << " " << (*MaterialCollection)[i]->name << "\n";
  outStream << ">Particles:\n";
  for (int i=0; i<Detector->ParticleCollection.size(); i++) outStream << "> " << i << " " << Detector->ParticleCollection[i]->ParticleName << "\n";
  outStream << ">\n";
  outStream << ">Event# particle# particleId MaterialId x[mm] y[mm] z[mm] CellLength[mm] time dE[keV]\n";
  /*
  double r[3];  // x,y,z -coordinates of the cell center
  double cellLength; //cell length for charged particles
  double time;
  double dE; //energy deposited in this cell
  int ParticleId;          // index of the particle form the particle list
  int MaterialId;         //  index of the material
  int index;  //particle number - need for visualization if tracking several particles with same id - index continuosly increases throughout the EnergyVector
  int eventId; //for EnergyVector import mode (to be done) - event number
  */

  for (int i=0; i<EnergyVector.size(); i++)
    {
      outStream << EnergyVector[i]->eventId
                << " " << EnergyVector[i]->index
                << " " << EnergyVector[i]->ParticleId
                << " " << EnergyVector[i]->MaterialId
                << " " << EnergyVector[i]->r[0]
                << " " << EnergyVector[i]->r[1]
                << " " << EnergyVector[i]->r[2]
                << " " << EnergyVector[i]->cellLength
                << " " << EnergyVector[i]->time
                << " " << EnergyVector[i]->dE
                <<"\n";
    }
}

void MainWindow::ImportDeposition(QFile &file)
{
  QTextStream in(&file);

  MainWindow::ClearData();

  QRegExp rx("(\\ |\\,|\\:|\\t)"); //RegEx for ' ' or ',' or ':' or '\t' //separators
  while(!in.atEnd())
      {
        QString line = in.readLine();
        QStringList fields = line.split(rx, QString::SkipEmptyParts);
        if (fields.size() !=10) continue;

        EnergyDepositionCell* cell = new EnergyDepositionCell();

        bool ok, flagOK = true;
        int maxMatIndex = MaterialCollection->size()-1;
        cell->eventId = fields[0].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->index = fields[1].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->ParticleId = fields[2].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->MaterialId = fields[3].toDouble(&ok);
        if (!ok) flagOK = false;
        if (cell->MaterialId > maxMatIndex)
          {
            message("Material index is out of range. Retry with 'Reassign material index' option", this);
            delete cell;
            MainWindow::ClearData();
            return;
          }
        cell->r[0] = fields[4].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->r[1] = fields[5].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->r[2] = fields[6].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->cellLength = fields[7].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->time = fields[8].toDouble(&ok);
        if (!ok) flagOK = false;
        cell->dE = fields[9].toDouble(&ok);
        if (!ok) flagOK = false;

        if (ui->cbReassignMatIndex->isChecked())
          {
            Detector->GeoManager->SetCurrentPoint(cell->r);
            Detector->GeoManager->FindNode();
            cell->MaterialId = Detector->GeoManager->GetCurrentVolume()->GetMaterial()->GetIndex();
          }

        if (flagOK) EnergyVector.append(cell);
        else delete cell;
    }

  if (EnergyVector.isEmpty()) message("Error: no data were loaded!", this);
  else ui->pbGenerateLight->setEnabled(true);
}

void MainWindow::LoadDummyPMs(QString DFile)
{
  QFile file(DFile);

  if(!file.open(QIODevice::ReadOnly | QFile::Text)) message("Error while opening Dummy PMs file!"+file.fileName()+"\n"+file.errorString(), this);
  else
    {
      //loading
       QTextStream in(&file);
       QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
       while(!in.atEnd())
         {
              QString line = in.readLine();
              QStringList fields = line.split(rx, QString::SkipEmptyParts);            
              if (fields.size() == 8)
                {
                  PMdummyStructure dpm;
                  dpm.PMtype = fields[0].toInt();
                  dpm.UpperLower = fields[1].toInt();
                  dpm.r[0] = fields[2].toDouble();
                  dpm.r[1] = fields[3].toDouble();
                  dpm.r[2] = fields[4].toDouble();
                  dpm.Angle[0] = fields[5].toDouble();
                  dpm.Angle[1] = fields[6].toDouble();
                  dpm.Angle[2] = fields[7].toDouble();
                  Detector->PMdummies.append(dpm);
                }
         }
       file.close();      
    }
}

void MainWindow::LoadAdditionalObjects(QString File)
{
  QFile file(File);

  if(!file.open(QIODevice::ReadOnly | QFile::Text)) message("Error while opening file with additional geo objects!"+file.fileName()+"\n"+file.errorString(), this);
  else
    {
      //loading
       QTextStream in(&file);
       QRegExp rx("(\\ |\\,|\\:|\\t)"); //separators: ' ' or ',' or ':' or '\t'
       while(!in.atEnd())
         {
              QString line = in.readLine();
              QStringList fields = line.split(rx, QString::SkipEmptyParts);

              //fix
              if (fields.size()>0)
                  if (fields[0] == "+TGeoArb8data")
                   {
                      //data for arb8 shape, adding info for the last object
                      if (Detector->AdditionalGeoObjects.size() == 0)
                      {
                          message("Unexpected Arb8 data found!");
                          continue;
                      }

                      if (fields.size() == 18)
                      {
                          for (int i=0; i<8; i++)
                          {
                              Detector->AdditionalGeoObjects.last().Vertices[i][0] = fields[2 + i*2].toDouble();
                              Detector->AdditionalGeoObjects.last().Vertices[i][1] = fields[2 + i*2 +1].toDouble();
                          }

                          Detector->AdditionalGeoObjects.last().Arb8height = fields[1].toDouble();
                      }
                      else
                      {
                          message("Mismatch in Abs8 data format!");
                          continue;
                      }
                   }


              if (fields.size() == 13)
                {
                  AdditionalGeoObjectClass o;

                  o.Name = fields[0];
                  o.Container = fields[1];
                  o.Material = fields[2].toInt();
                  o.Shape = fields[3].toInt();
                  o.Size[0] = fields[4].toDouble(); o.Size[1] = fields[5].toDouble(); o.Size[2] = fields[6].toDouble();
                  o.R[0] = fields[7].toDouble(); o.R[1] = fields[8].toDouble(); o.R[2] = fields[9].toDouble();
                  o.Angles[0] = fields[10].toDouble(); o.Angles[1] = fields[11].toDouble(); o.Angles[2] = fields[12].toDouble();

                  o.fActivated = true;

                  Detector->AdditionalGeoObjects.append(o);
                }

         }
       file.close();
    }
}

void MainWindow::SaveGains(QString fileName)
{
  QFile outputFile(fileName);
  outputFile.open(QIODevice::WriteOnly);
  if(!outputFile.isOpen())
   {
      message("Unable to open file " +fileName+ " for writing!", this);
      return;
    }

  int numPMs = PMs->count();
  QVector<double> X(0);
  QVector<double> Y(0);
  for (int ipm=0; ipm<numPMs; ipm++)
    {
      X.append(ipm);
      Y.append(PMs->at(ipm).relGain);
    }

  SaveDoubleVectorsToFile(fileName, &X, &Y);
}

bool MainWindow::LoadGains(QString fileName)
{
  QVector<double> num, gain;
  LoadDoubleVectorsFromFile(fileName, &num, &gain);  //cleans previous data

  int numPMs = PMs->count();
  QVector<bool> Present(numPMs, false);
  QWidget* par = this;
  if (Rwindow)
    if (Rwindow->isVisible()) par = Rwindow;

  for (int i=0; i<num.size(); i++)
    {
      int ipm = num[i];
      if (ipm > numPMs-1 || ipm<0)
        {
          message("Error: PM number out of range detected!", Rwindow);
          return false;
        }
      Present[ipm] = true;
    }

  for (int i=0; i<numPMs; i++)
    if (!Present[i])
        {
           message("Error: Data is not complete, info for some PMs is missing", Rwindow);
           return false;
        }

  for (int i=0; i<num.size(); i++)
    PMs->at(i).relGain = gain[i];

  return true;
}

void MainWindow::on_pbLoadPMcenters_clicked()
{
  int reg = ui->cobPMarrayRegularity->currentIndex();

  QString str;
  int ul = ui->cobUpperLowerPMs->currentIndex();
  if (ul == 0) str = "upper PMs"; else str = "lower PMs";

  if (reg == 1) str = "Load PM center positions (X,Y) for "+str;
  else str = "Load PM center positions (X,Y,Z) for "+str;

  QString fileName = QFileDialog::getOpenFileName(this, str, GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  PMarrayDataClass *PMar = &Detector->PMarrays[ul];
  PMar->PositionsAnglesTypes.clear();
  if (reg == 1)
    { //2D - autoz
      QVector<double> x, y;
      LoadDoubleVectorsFromFile(fileName, &x, &y);
      for (int i=0; i<x.size(); i++)
        PMar->PositionsAnglesTypes.append(PosAngTypeStructure(x[i], y[i],0, 0,0,0, 0));
    }
   else //reg = 2
    { //3D - custom
      QVector<double> x, y, z;
      LoadDoubleVectorsFromFile(fileName, &x, &y, &z);
      for (int i=0; i<x.size(); i++)
        PMar->PositionsAnglesTypes.append(PosAngTypeStructure(x[i], y[i], z[i], 0,0,0, 0));
    }
  MainWindow::ReconstructDetector();
  MainWindow::ShowPMcount();
  MainWindow::ClearData();
  Owindow->RefreshData();
}

void MainWindow::on_pbSavePMcenters_clicked()
{
    int A_Reg = ui->cobPMarrayRegularity->currentIndex();

    QString str;
    int ul = ui->cobUpperLowerPMs->currentIndex();
    str = ul == 0 ? "upper PMs" : "lower PMs";

    if (A_Reg == 1) str = "Save PM center positions (X,Y) for "+str;
    else str = "Save PM center positions (X,Y,Z) for "+str;

    QString fileName = QFileDialog::getSaveFileName(this, str, GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*.*)");
    if (fileName.isEmpty()) return;

    GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
    QFileInfo file(fileName);
    if(file.suffix().isEmpty()) fileName += ".dat";

    QFile outFile(fileName);
    outFile.open(QIODevice::WriteOnly);
    if(!outFile.isOpen())
    {
        QMessageBox::information(this, "I/O Error", "Unable to open file " +fileName+ " for writing!");
        return;
    }
    QTextStream outStream(&outFile);

    if (A_Reg == 1)
    {
        //2D
        for (int i=0; i<PMs->count(); i++)
            if (PMs->at(i).upperLower == ul)
                PMs->at(i).saveCoords2D(outStream);
    }
    else if (A_Reg == 2)
    {
        //3D
        for (int i=0; i<PMs->count(); i++)
            if (PMs->at(i).upperLower == ul)
                PMs->at(i).saveCoords(outStream);
    }
}

//void MainWindow::SavePreprocessingSettings(QString fileName)
//{
//  QFile saveFile(fileName);

//  if (!saveFile.open(QIODevice::WriteOnly))
//    {
//          qWarning("Failed to save preprocessing settings!");
//          message("Cannot open file "+fileName + "in order to save preprocessing settings!", this);
//          return;
//    }

//  QFileInfo QF(fileName);
//  qDebug()<<QF.path();

//  QJsonObject json;

//  QString addMultFile = "AddMultiply.dat";
//  json["AddMultFile"] = addMultFile;
//  MainWindow::SavePreprocessingAddMulti(QF.path() + "/" + addMultFile);

//  bool energy = ui->cbLoadedDataHasEnergy->isChecked();
//  json["Energy"] = energy;
//  if (energy) json["EnergyChannel"] = ui->sbLoadedEnergyChannelNumber->value();

//  bool ignoreLess = ui->cbIgnoreEvent->isChecked();
//  json["IgnoreLess"] = ignoreLess;
//  if (ignoreLess)
//    {
//      json["IgnoreThresholdMin"] = ui->ledIgnoreThresholdMin->text();
//      json["IgnoreThresholdMax"] = ui->ledIgnoreThresholdMax->text();
//    }

//  //json["NegativePolicy"] = ui->cobNegativeSignalStrategy->currentIndex();

//  json["Electronics"] = ui->cbPrePro_BacktrackElectronics->isChecked();

//  bool useFirstOnly = ui->cbLimitNumberEvents->isChecked();
//  json["useFirstOnly"] = useFirstOnly;
//  if (useFirstOnly) json["FirstEvents"] = ui->sbFirstNevents->value();

//  QJsonDocument saveDoc(json);
//  saveFile.write(saveDoc.toJson());
//}

void MainWindow::SavePreprocessingAddMulti(QString fileName)
{
  QVector<double> ii, PreprocessingAdd, PreprocessingMultiply;
  for (int i=0; i<PMs->count(); i++)
    {
      ii.append(i);
      PreprocessingAdd.append(PMs->at(i).PreprocessingAdd);
      PreprocessingMultiply.append(PMs->at(i).PreprocessingMultiply);
    }
  ii.append(PMs->count());
  PreprocessingAdd.append(EventsDataHub->PreprocessingSettings.LoadEnAdd);
  PreprocessingMultiply.append(EventsDataHub->PreprocessingSettings.LoadEnMulti);
  SaveDoubleVectorsToFile(fileName, &ii, &PreprocessingAdd, &PreprocessingMultiply);
}

//void MainWindow::LoadPreprocessingSettings(QString fileName)
//{
//  QFile loadFile(fileName);

//  if (!loadFile.open(QIODevice::ReadOnly))
//    {
//          qWarning("Failed to load preprocessing settings!");
//          message("Cannot open file "+fileName+" with preprocessing settings!", this);
//          return;
//     }

//  QFileInfo QF(fileName);
//  QByteArray saveData = loadFile.readAll();

//  QJsonDocument loadDoc(QJsonDocument::fromJson(saveData));
//  QJsonObject json = loadDoc.object();

//  JsonParser jp(json);

//  bool tmpBool;
//  jp.ParseObject("Energy", tmpBool);
//  ui->cbLoadedDataHasEnergy->setChecked(tmpBool);

//  int tmpInt;
//  if (jp.ParseObject("EnergyChannel", tmpInt)) ui->sbLoadedEnergyChannelNumber->setValue(tmpInt);

//  jp.ParseObject("IgnoreLess", tmpBool);
//  ui->cbIgnoreEvent->setChecked(tmpBool);

//  QString tmpStr;
//  if (jp.ParseObject("IgnoreThreshold", tmpStr)) ui->ledIgnoreThresholdMin->setText(tmpStr); //compatibility
//  if (jp.ParseObject("IgnoreThresholdMin", tmpStr)) ui->ledIgnoreThresholdMin->setText(tmpStr);
//  if (jp.ParseObject("IgnoreThresholdMax", tmpStr)) ui->ledIgnoreThresholdMax->setText(tmpStr);

//  jp.ParseObject("Electronics", tmpBool);
//  ui->cbPrePro_BacktrackElectronics->setChecked(tmpBool);

//  jp.ParseObject("useFirstOnly", tmpBool);
//  ui->cbLimitNumberEvents->setChecked(tmpBool);
//  if (jp.ParseObject("FirstEvents", tmpInt)) ui->sbFirstNevents->setValue(tmpInt);

//  if (json.contains("ManifestFile")) ui->leManifestFile->setText(json["ManifestFile"].toString());
//  else ui->leManifestFile->setText("");

//  //load add multiv AFTER all other settings!
//  QString addMultFile;
//  if (jp.ParseObject("AddMultFile", addMultFile))
//    {
//      addMultFile = QF.path() + "/" + addMultFile;
//      MainWindow::LoadPreprocessingAddMulti(addMultFile); //takes care of indication
//    }

//  ui->cbPMsignalPreProcessing->setChecked(true);
//}

void MainWindow::LoadPreprocessingAddMulti(QString filename)
{
  QVector<double> ii, add, multiply;
  LoadDoubleVectorsFromFile(filename, &ii, &add, &multiply);

  if (add.size() < PMs->count())
    {
      message("Error: data do not exist for all PMs!", this);
      return;
    }

  if (add.size() < PMs->count()+1  && EventsDataHub->fLoadedEventsHaveEnergyInfo)
    {
      message("Loaded data do not have energy channel, extending with add:0, multiply:1.0!", this);
      add.append(0);
      multiply.append(1.0);
    }

  for (int i=0; i<PMs->count(); i++)
    {
      PMs->at(i).PreprocessingAdd = add[i];
      PMs->at(i).PreprocessingMultiply = multiply[i];
    }
  if (EventsDataHub->fLoadedEventsHaveEnergyInfo)
    {
      EventsDataHub->PreprocessingSettings.LoadEnAdd = add.last();
      EventsDataHub->PreprocessingSettings.LoadEnMulti = multiply.last();
    }
  MainWindow::on_sbPreprocessigPMnumber_valueChanged(ui->sbPreprocessigPMnumber->value());
}
