//ANTS2 modules and windows
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "pms.h"
#include "materialinspectorwindow.h"
#include "reconstructionwindow.h"
#include "outputwindow.h"
#include "windownavigatorclass.h"
#include "exampleswindow.h"
#include "graphwindowclass.h"
#include "geometrywindowclass.h"
#include "lrfwindow.h"
#include "genericscriptwindowclass.h"
#include "globalsettingsclass.h"

//Qt
#include <QFileDialog>
#include <QInputDialog>

void MainWindow::on_actionWindow_navigator_triggered()
{
  if (WindowNavigator)
    {
      WindowNavigator->showNormal();
      WindowNavigator->activateWindow();
    }
  else message("error - winnavigator was deleted", this);
}

void MainWindow::on_actionReconstructor_triggered()
{
  Rwindow->showNormal();
  Rwindow->raise();
  Rwindow->activateWindow();
}

void MainWindow::on_actionOutput_triggered()
{
  Owindow->showNormal();
  Owindow->raise();
  Owindow->activateWindow();
}

void MainWindow::on_actionLRF_triggered()
{
  lrfwindow->showNormal();
  lrfwindow->raise();
  lrfwindow->activateWindow();
}

void MainWindow::on_actionMaterial_inspector_window_triggered()
{
  MIwindow->showNormal();
  MIwindow->raise();
  MIwindow->activateWindow();
}

void MainWindow::on_actionGeometry_triggered()
{
  GeometryWindow->showNormal();
  GeometryWindow->raise();
  GeometryWindow->activateWindow();
}

void MainWindow::on_actionGraph_triggered()
{
  if (!GraphWindow) return;
  GraphWindow->showNormal();
  GraphWindow->raise();
  GraphWindow->activateWindow();
}

void MainWindow::on_actionExamples_triggered()
{
  ELwindow->showNormal();
  ELwindow->raise();
  ELwindow->activateWindow();
}

void MainWindow::on_actionGain_evaluation_triggered()
{
  Rwindow->ShowGainWindow();
}

void MainWindow::on_actionReset_position_of_windows_triggered()
{
   QList<QMainWindow*> list;
   list << this<<Rwindow<<Owindow<<lrfwindow<<MIwindow<<WindowNavigator<<ELwindow;
   foreach(QMainWindow *w, list)
     {
       w->move(20,20);
       w->showNormal();
       w->raise();
     }
   Owindow->resize(524, 385);

   if (GeometryWindow)
     {
       GeometryWindow->setGeometry(20, 20, 600, 600);
       GeometryWindow->showNormal();
       GeometryWindow->raise();
     }

   if (GraphWindow)
     {
       GraphWindow->setGeometry(20, 20, 700, 500);
       GraphWindow->showNormal();
       GraphWindow->raise();
     }
}

void addWindow(QString name, QMainWindow* w, QJsonObject &json)
{
  QJsonObject js;
  js["x"] = w->x();
  js["y"] = w->y();
  js["w"] = w->width();
  js["h"] = w->height();
  js["vis"] = w->isVisible();
  json[name] = js;
}

void MainWindow::on_actionSave_position_and_stratus_of_all_windows_triggered()
{
  GraphWindow->switchOffBasket();

  QJsonObject json;
  addWindow("Main", this, json);
  addWindow("Recon", Rwindow, json);
  addWindow("Out", Owindow, json);
  addWindow("LRF", lrfwindow, json);
  addWindow("Navi", WindowNavigator, json);
  addWindow("Material", MIwindow, json);
  addWindow("ExLoad", ELwindow, json);
  addWindow("Geom", GeometryWindow, json);
  addWindow("Graph", GraphWindow, json);

  if (GenScriptWindow)
    {
      ScriptWinX = GenScriptWindow->x();
      ScriptWinY = GenScriptWindow->y();
      ScriptWinW = GenScriptWindow->width();
      ScriptWinH = GenScriptWindow->height();
    }
  QJsonObject js;
  js["x"] = ScriptWinX;
  js["y"] = ScriptWinY;
  js["w"] = ScriptWinW;
  js["h"] = ScriptWinH;
  json["Script"] = js;

  //QString configDir = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation)+"/ants2";
  //if (!QDir(configDir).exists()) QDir().mkdir(configDir);
  QString fileName = GlobSet->ConfigDir + "/WindowConfig.ini";
  SaveJsonToFile(json, fileName);
}

int MainWindow::getNormalWindowStatus(QMainWindow* win)
{
  if (win->isMinimized()) return 1;
  if (win->isMaximized()) return 2;
  if (win->isHidden()) return 3;
  return 0;
}

void readXYwindow(QString key, QMainWindow* w, bool fWH, QJsonObject &json)
{
  QJsonObject js = json[key].toObject();
  if (js.isEmpty())
    {
      qWarning() << "No json found for key:"<<key;
      return;
    }
  w->move(js["x"].toInt(), js["y"].toInt());
  if (fWH) w->resize(js["w"].toInt(), js["h"].toInt());
  if (js["vis"].toBool()) w->showNormal(); //raise
  else w->hide();
}

void MainWindow::on_actionLoad_positions_and_status_of_all_windows_triggered()
{

  QString fileName = GlobSet->ConfigDir + "/WindowConfig.ini";
  if (QFile(fileName).exists())
    {
      QJsonObject json;
      bool ok = LoadJsonFromFile(json, fileName);
      if (!ok)
        {
          qDebug() << "Config of window positions/sizes: empty json";
          return;
        }
      readXYwindow("Main", this, false, json);
      readXYwindow("Recon", Rwindow, false, json);
      readXYwindow("Out", Owindow, true, json);
      readXYwindow("LRF", lrfwindow, false, json);
      readXYwindow("Navi", WindowNavigator, false, json);
      readXYwindow("Material", MIwindow, false, json);
      readXYwindow("ExLoad", ELwindow, false, json);
      readXYwindow("Geom", GeometryWindow, true, json);
      readXYwindow("Graph", GraphWindow, true, json);

      if (json.contains("Script"))
        {
          QJsonObject js = json["Script"].toObject();
          parseJson(js, "x", ScriptWinX);
          parseJson(js, "y", ScriptWinY);
          parseJson(js, "w", ScriptWinW);
          parseJson(js, "h", ScriptWinH);
          if (GenScriptWindow) recallGeometryOfScriptWindow();
        }
    }
}

void MainWindow::setNormalWindowStatus(QMainWindow* win, int status)
{
  //0-normal,1-minimized,2-maximized,3-hidden
  switch (status)
    {
    case 0:
      win->showNormal();
      win->raise();
      break;
    case 1:
      win->setWindowState(Qt::WindowMinimized);
      break;
    case 2:
      win->setWindowState(Qt::WindowMaximized);
      break;
    case 3:
      win->hide();
      break;
    }
  return;
}

void MainWindow::setQWindowStatus(QWindow* win, int status)
{
  //0-normal,1-minimized,2-maximized,3-hidden
  switch (status)
    {
    case 0:
      win->showNormal();
      win->raise();
      break;
    case 1:
      win->setWindowState(Qt::WindowMinimized);
      break;
    case 2:
      win->setWindowState(Qt::WindowMaximized);
      break;
    case 3:
      win->hide();
      break;
    }
  return;
}

void MainWindow::SetProgress(int val)
{
  ui->prScan->setValue(val);
}

void MainWindow::SetMultipliersUsingGains(bool fIncludeChPerPhEl, QVector<double>* ChPerPhEl )
{
  if (fIncludeChPerPhEl && ChPerPhEl->size()<Detector->PMs->count())
    {
      fIncludeChPerPhEl = false;
      message("Channels per photoelectron data is not defined, switching off correction", this);
    }
  for (int ipm=0; ipm<PMs->count(); ipm++)
    {
      const double gain = Detector->PMs->at(ipm).relGain;
      if (fabs(gain)>1.0e-20) PMs->at(ipm).PreprocessingMultiply = 1.0/gain;
      if (fIncludeChPerPhEl) PMs->at(ipm).PreprocessingMultiply /= (*ChPerPhEl)[ipm];
    }
  int ipm = ui->sbPreprocessigPMnumber->value();
  ui->lePreprocessingMultiply->setText( QString::number( PMs->at(ipm).PreprocessingMultiply) );
}

void MainWindow::on_actionSave_configuration_triggered()
{
  QFileDialog fileDialog;
  QString fileName = fileDialog.getSaveFileName(this, "Save configuration", GlobSet->LastOpenDir, "Json files (*.json)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  QFileInfo file(fileName);
  if(file.suffix().isEmpty()) fileName += ".json";
  ELwindow->SaveAllConfig(fileName);
}

void MainWindow::on_actionLoad_configuration_triggered()
{
  QFileDialog fileDialog;
  QString fileName = fileDialog.getOpenFileName(this,"Load configuration", GlobSet->LastOpenDir, "All files (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  ELwindow->LoadAllConfig(fileName);
  if (GeometryWindow->isVisible()) ShowGeometry();
}

void MainWindow::on_actionSave_detector_constructor_triggered()
{
   QFileDialog fileDialog;
   QString fileName = fileDialog.getSaveFileName(this, "Save detector constructor", GlobSet->LastOpenDir, "Json files (*.json)");
   if (fileName.isEmpty()) return;
   GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
   QFileInfo file(fileName);
   if(file.suffix().isEmpty()) fileName += ".json";
   ELwindow->SaveAllConfig(fileName, true, false, false);
}

void MainWindow::on_actionLoad_detector_settings_triggered()
{   
   QFileDialog fileDialog;
   QString fileName = fileDialog.getOpenFileName(this, "Load detector constructor file", GlobSet->LastOpenDir, "All files (*.*)");
   if (fileName.isEmpty()) return;
   GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
   ELwindow->LoadAllConfig(fileName, true, false, false);
   if (GeometryWindow->isVisible()) ShowGeometry();
}

void MainWindow::on_actionSave_simulation_settings_triggered()
{
  QFileDialog fileDialog;
  QString fileName = fileDialog.getSaveFileName(this, "Save simulation config", GlobSet->LastOpenDir, "Json files (*.json)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  QFileInfo file(fileName);
  if(file.suffix().isEmpty()) fileName += ".json";
  ELwindow->SaveAllConfig(fileName, false, true, false);
}

void MainWindow::on_actionLoad_simulation_settings_triggered()
{
  QFileDialog fileDialog;
  QString fileName = fileDialog.getOpenFileName(this, "Load simulation config file", GlobSet->LastOpenDir, "All files (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  ELwindow->LoadAllConfig(fileName, false, true, false);
}

void MainWindow::on_actionQuicksave_triggered()
{
   ELwindow->QuickSave();
}

void MainWindow::on_actionQuickload_triggered()
{
   ELwindow->QuickLoad();
}

void MainWindow::setFontSizeAllWindows(int size)
{
  QFont font = this->font();
  font.setPointSize(size);

  QList<QMainWindow*> wins;
  wins << (QMainWindow*)this << (QMainWindow*)Rwindow << (QMainWindow*)Owindow << (QMainWindow*)MIwindow << (QMainWindow*)lrfwindow << (QMainWindow*)WindowNavigator << (QMainWindow*)GeometryWindow << (QMainWindow*)GraphWindow << (QMainWindow*)ELwindow << (QMainWindow*)DAwindow << (QMainWindow*)GlobSetWindow << (QMainWindow*)CheckUpWindow;
  if (GainWindow) wins << (QMainWindow*)GainWindow;

  for (int i=0; i<wins.size(); i++) wins[i]->setFont(font);
}
