//ANTS2
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "outputwindow.h"
#include "pms.h"
#include "reconstructionwindow.h"
#include "windownavigatorclass.h"
#include "geometrywindowclass.h"
#include "particlesourcesclass.h"
#include "graphwindowclass.h"
#include "detectorclass.h"
#include "checkupwindowclass.h"
#include "globalsettingsclass.h"

//Qt
#include <QtGui>
#include <QFileDialog>
#include <QMessageBox>

//Root
#include "TVirtualGeoTrack.h"

void MainWindow::SimParticleSourcesConfigToJson(QJsonObject &json)
{  
  QJsonObject masterjs;
    // control options
  QJsonObject cjs;
  cjs["EventsToDo"] = ui->sbGunEvents->value();
  bool fAllowMulti = ui->cbGunAllowMultipleEvents->isChecked();
  cjs["AllowMultipleParticles"] = fAllowMulti;
  if (fAllowMulti) cjs["AverageParticlesPerEvent"] = ui->ledGunAverageNumPartperEvent->text().toDouble();
  cjs["DoS1"] = ui->cbGunDoS1->isChecked();
  cjs["DoS2"] = ui->cbGunDoS2->isChecked();
  cjs["ParticleTracks"] = ui->cbGunParticleTracks->isChecked();
  cjs["IgnoreNoHitsEvents"] = ui->cbIgnoreEventsWithNoHits->isChecked();
  masterjs["SourceControlOptions"] = cjs;
    //particle sources
  ParticleSources.writeToJson(masterjs);

  json["ParticleSourcesConfig"] = masterjs;
}

void MainWindow::ShowGeneratedPhotonsLog()
{
  if (EventsDataHub->GeneratedPhotonsHistory.isEmpty()) return;

  Owindow->OutText("----photon log----");
  for (int i=0; i<EventsDataHub->GeneratedPhotonsHistory.size(); i++)
        {
          QString out, str;

          str.setNum(EventsDataHub->GeneratedPhotonsHistory[i].event);
          out = str + "/";
          str.setNum(EventsDataHub->GeneratedPhotonsHistory[i].index);
          out += str + "> ";

          out += Detector->ParticleCollection[EventsDataHub->GeneratedPhotonsHistory[i].ParticleId]->ParticleName;
          out += " in " + (*MaterialCollection)[EventsDataHub->GeneratedPhotonsHistory[i].MaterialId]->name;

          str.setNum(EventsDataHub->GeneratedPhotonsHistory[i].Energy);
          out += "  deposited: " + str + " keV";
      //    if (ui->cbDoS1->isChecked())
            {
              out += "  Primary photons: ";
              str.setNum(EventsDataHub->GeneratedPhotonsHistory[i].PrimaryPhotons);
              out += str;
            }
      //    if (ui->cbDoS2->isChecked())
            {
              out += "  Secondary photons: ";
              str.setNum(EventsDataHub->GeneratedPhotonsHistory[i].SecondaryPhotons);
              out += str;
            }
          //out += "\n";
          Owindow->OutText(out);
        }
}

void MainWindow::ShowSource(int isource, bool clear)
{
  if (BulkUpdate) return;

  int index = ParticleSources[isource].index;
  double X0 = ParticleSources[isource].X0;
  double Y0 = ParticleSources[isource].Y0;
  double Z0 = ParticleSources[isource].Z0;
  double Phi = ParticleSources[isource].Phi*3.1415926535/180.0;
  double Theta = ParticleSources[isource].Theta*3.1415926535/180.0;
  double Psi = ParticleSources[isource].Psi*3.1415926535/180.0;
  double size1 = ParticleSources[isource].size1;
  double size2 = ParticleSources[isource].size2;
  double size3 = ParticleSources[isource].size3;
  double CollPhi = ParticleSources[isource].CollPhi*3.1415926535/180.0;
  double CollTheta = ParticleSources[isource].CollTheta*3.1415926535/180.0;
  double Spread = ParticleSources[isource].Spread*3.1415926535/180.0;

  //calculating unit vector along 1D direction
  TVector3 VV(sin(Theta)*sin(Phi), sin(Theta)*cos(Phi), cos(Theta));
  //qDebug()<<VV[0]<<VV[1]<<VV[2];
  if (clear) Detector->GeoManager->ClearTracks();

  TVector3 V[3];
  V[0].SetXYZ(size1, 0, 0);
  V[1].SetXYZ(0, size2, 0);
  V[2].SetXYZ(0, 0, size3);
  for (int i=0; i<3; i++)
   {
    V[i].RotateX(Phi);
    V[i].RotateY(Theta);
    V[i].RotateZ(Psi);
   }
  switch (index)
    {
     case (1):
      { //linear source
        Int_t track_index = Detector->GeoManager->AddTrack(1,22);
        TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
        track->AddPoint(X0+VV[0]*size1, Y0+VV[1]*size1, Z0+VV[2]*size1, 0);
        track->AddPoint(X0-VV[0]*size1, Y0-VV[1]*size1, Z0-VV[2]*size1, 0);
        track->SetLineWidth(3);
        track->SetLineColor(9);
        break;
      }
     case (2):
      { //area source - square
        Int_t track_index = Detector->GeoManager->AddTrack(1,22);
        TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
        track->AddPoint(X0-V[0][0]-V[1][0], Y0-V[0][1]-V[1][1], Z0-V[0][2]-V[1][2], 0);
        track->AddPoint(X0+V[0][0]-V[1][0], Y0+V[0][1]-V[1][1], Z0+V[0][2]-V[1][2], 0);
        track->AddPoint(X0+V[0][0]+V[1][0], Y0+V[0][1]+V[1][1], Z0+V[0][2]+V[1][2], 0);
        track->AddPoint(X0-V[0][0]+V[1][0], Y0-V[0][1]+V[1][1], Z0-V[0][2]+V[1][2], 0);
        track->AddPoint(X0-V[0][0]-V[1][0], Y0-V[0][1]-V[1][1], Z0-V[0][2]-V[1][2], 0);
        track->SetLineWidth(3);
        track->SetLineColor(9);
        break;
      }
     case (3):
      { //area source - round
        Int_t track_index = Detector->GeoManager->AddTrack(1,22);
        TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
        TVector3 Circ;
        for (int i=0; i<51; i++)
        {
            double x = size1*cos(3.1415926535/25.0*i);
            double y = size1*sin(3.1415926535/25.0*i);
            Circ.SetXYZ(x,y,0);
            Circ.RotateX(Phi);
            Circ.RotateY(Theta);
            Circ.RotateZ(Psi);
            track->AddPoint(X0+Circ[0], Y0+Circ[1], Z0+Circ[2], 0);
        }
        track->SetLineWidth(3);
        track->SetLineColor(9);
        break;
      }

    case (4):
      { //volume source - box
        for (int i=0; i<3; i++)
         for (int j=0; j<3; j++)
           {
             if (j==i) continue;
             //third k
             int k;
             for (k=0; k<3; k++) if (k!=i && k!=j) break;
             for (int s=-1; s<2; s+=2)
               {
                //qDebug()<<"i j k shift"<<i<<j<<k<<s;
                Int_t track_index = Detector->GeoManager->AddTrack(1,22);
                TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
                track->AddPoint(X0-V[i][0]-V[j][0]+V[k][0]*s, Y0-V[i][1]-V[j][1]+V[k][1]*s, Z0-V[i][2]-V[j][2]+V[k][2]*s, 0);
                track->AddPoint(X0+V[i][0]-V[j][0]+V[k][0]*s, Y0+V[i][1]-V[j][1]+V[k][1]*s, Z0+V[i][2]-V[j][2]+V[k][2]*s, 0);
                track->AddPoint(X0+V[i][0]+V[j][0]+V[k][0]*s, Y0+V[i][1]+V[j][1]+V[k][1]*s, Z0+V[i][2]+V[j][2]+V[k][2]*s, 0);
                track->AddPoint(X0-V[i][0]+V[j][0]+V[k][0]*s, Y0-V[i][1]+V[j][1]+V[k][1]*s, Z0-V[i][2]+V[j][2]+V[k][2]*s, 0);
                track->AddPoint(X0-V[i][0]-V[j][0]+V[k][0]*s, Y0-V[i][1]-V[j][1]+V[k][1]*s, Z0-V[i][2]-V[j][2]+V[k][2]*s, 0);
                track->SetLineWidth(3);
                track->SetLineColor(9);
               }
           }
        break;
      }
    case(5):
      { //volume source - cylinder
      TVector3 Circ;
      Int_t track_index = Detector->GeoManager->AddTrack(1,22);
      TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
      double z = size3;
      for (int i=0; i<51; i++)
      {
          double x = size1*cos(3.1415926535/25.0*i);
          double y = size1*sin(3.1415926535/25.0*i);
          Circ.SetXYZ(x,y,z);
          Circ.RotateX(Phi);
          Circ.RotateY(Theta);
          Circ.RotateZ(Psi);
          track->AddPoint(X0+Circ[0], Y0+Circ[1], Z0+Circ[2], 0);
      }
      track->SetLineWidth(3);
      track->SetLineColor(9);
      track_index = Detector->GeoManager->AddTrack(1,22);
      track = Detector->GeoManager->GetTrack(track_index);
      z = -z;
      for (int i=0; i<51; i++)
            {
                double x = size1*cos(3.1415926535/25.0*i);
                double y = size1*sin(3.1415926535/25.0*i);
                Circ.SetXYZ(x,y,z);
                Circ.RotateX(Phi);
                Circ.RotateY(Theta);
                Circ.RotateZ(Psi);
                track->AddPoint(X0+Circ[0], Y0+Circ[1], Z0+Circ[2], 0);
            }
      track->SetLineWidth(3);
      track->SetLineColor(9);
      break;
      }
    }

  TVector3 K(sin(CollTheta)*sin(CollPhi), sin(CollTheta)*cos(CollPhi), cos(CollTheta)); //collimation direction
  Int_t track_index = Detector->GeoManager->AddTrack(1,22);
  TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
  double Klength = 20.0;

  track->AddPoint(X0, Y0, Z0, 0);
  track->AddPoint(X0+K[0]*Klength, Y0+K[1]*Klength, Z0+K[2]*Klength, 0);
  track->SetLineWidth(2);
  track->SetLineColor(9);

  TVector3 Knorm = K.Orthogonal();
  TVector3 K1(K);
  K1.Rotate(Spread, Knorm);
  for (int i=0; i<8; i++)  //drawing spread
  {
      Int_t track_index = Detector->GeoManager->AddTrack(1,22);
      TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);

      track->AddPoint(X0, Y0, Z0, 0);
      track->AddPoint(X0+K1[0]*Klength, Y0+K1[1]*Klength, Z0+K1[2]*Klength, 0);
      K1.Rotate(3.1415926535/4.0, K);

      track->SetLineWidth(1);
      track->SetLineColor(9);
  }

  MainWindow::ShowTracks();
  Detector->GeoManager->SetCurrentPoint(X0,Y0,Z0);
  Detector->GeoManager->DrawCurrentPoint(9);
  GeometryWindow->UpdateRootCanvas();
}

void MainWindow::on_pbGunTest_clicked()
{
  MainWindow::on_pbGunShowSource_clicked();

  ParticleSources.Init();

  double R[3], K[3];
  int Points = ui->sbGunTestEvents->value();
  for (int i=0; i<Points; i++)
    {      
      QVector<GeneratedParticleStructure>* GP = ParticleSources.GenerateEvent();
      //qDebug()<<"particles:"<<GP->size();
      for (int iP = 0; iP<GP->size(); iP++)
        {
          R[0] = (*GP)[iP].Position[0];
          R[1] = (*GP)[iP].Position[1];
          R[2] = (*GP)[iP].Position[2];
          K[0] = (*GP)[iP].Direction[0];
          K[1] = (*GP)[iP].Direction[1];
          K[2] = (*GP)[iP].Direction[2];

          Int_t track_index = Detector->GeoManager->AddTrack(1,22);
          TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);
          track->AddPoint(R[0], R[1], R[2], 0);
          track->AddPoint(R[0]+K[0]*20., R[1]+K[1]*20., R[2]+K[2]*20., 0);
          track->SetLineWidth(1);
          track->SetLineColor(1+(*GP)[iP].ParticleId);
        }
      //clear ad delete QVector with generated event
      GP->clear();
      delete GP;
    }
   MainWindow::ShowTracks();
}

void MainWindow::on_pbGunRefreshparticles_clicked()
{
    ui->lwGunParticles->clear(); //Cleaning ListView

    if (ParticleSources.size() == 0) return;
    int isource = ui->cobParticleSource->currentIndex();
    if (isource > ParticleSources.size()-1) return; //protection

    //Populating ListView
    int DefinedSourceParticles = ParticleSources[isource].GunParticles.size();
    //qDebug() << "Defined particles for this source:"<< DefinedSourceParticles;

    for (int i=0; i<DefinedSourceParticles; i++)
      {
        bool Individual = ParticleSources[isource].GunParticles[i]->Individual;

        QString str, str1;
        if (Individual) str = "";
        else str = ">";
        str1.setNum(i);
        str += str1 + "> ";
        str += Detector->ParticleCollection[ParticleSources[isource].GunParticles[i]->ParticleId]->ParticleName;
        if (ParticleSources[isource].GunParticles[i]->spectrum) str += " E=spec";
        else
          {
            str1.setNum(ParticleSources[isource].GunParticles[i]->energy);
            str += " E=" + str1;
          }

        if (Individual)
          {
            str += " W=";
            str1.setNum(ParticleSources[isource].GunParticles[i]->StatWeight);
            str += str1;
          }
        else
          {
            str += " Link:";
            str += QString::number(ParticleSources[isource].GunParticles[i]->LinkedTo);
            str += " P=";
            str += QString::number(ParticleSources[isource].GunParticles[i]->LinkingProbability);
          }
        ui->lwGunParticles->addItem(str);
        //qDebug() << str;
      }
  MainWindow::SourceUpdateThisParticleIndication();
}

void MainWindow::SourceUpdateThisParticleIndication()
{
  //showing selected row and updating data indication
  int isource = ui->cobParticleSource->currentIndex();
  if (isource<0) return;
  int row = ui->lwGunParticles->currentRow();
  int DefinedSourceParticles = ParticleSources[isource].GunParticles.size();
  if (DefinedSourceParticles > 0 && row>-1)
    {
      ui->fGunParticle->setEnabled(true);
      int part = ParticleSources[isource].GunParticles[row]->ParticleId;
      ui->cobGunParticle->setCurrentIndex(part);

      QString str;
      str.setNum(ParticleSources[isource].GunParticles[row]->StatWeight);
      ui->ledGunParticleWeight->setText(str);
      str.setNum(ParticleSources[isource].GunParticles[row]->energy);
      ui->ledGunEnergy->setText(str);
      ui->cbIndividualParticle->setChecked(ParticleSources[isource].GunParticles[row]->Individual);
      ui->leiParticleLinkedTo->setText( QString::number(ParticleSources[isource].GunParticles[row]->LinkedTo) );
      str.setNum(ParticleSources[isource].GunParticles[row]->LinkingProbability);
      ui->ledLinkingProbability->setText(str);
      //Depending on the presence of spectrum data, modifing visiblility and enable/disable
      if (ParticleSources[isource].GunParticles[row]->spectrum == 0)
        {
          ui->pbGunShowSpectrum->setEnabled(false);
          ui->pbGunDeleteSpectrum->setEnabled(false);
          ui->ledGunEnergy->setEnabled(true);
          ui->fPartEnergy->setVisible(true);
        }
      else
        {
          ui->pbGunShowSpectrum->setEnabled(true);
          ui->pbGunDeleteSpectrum->setEnabled(true);
          ui->ledGunEnergy->setEnabled(false);
          ui->fPartEnergy->setVisible(false);
        }
    }
  else ui->fGunParticle->setEnabled(false);
}

void MainWindow::on_pbGunAddNew_clicked()
{
    int isource = ui->cobParticleSource->currentIndex();

    GunParticleStruct* tmp = new GunParticleStruct();
    tmp->ParticleId = ui->cobGunParticle->currentIndex();
    tmp->StatWeight = ui->ledGunParticleWeight->text().toDouble();
    tmp->energy = ui->ledGunEnergy->text().toDouble();
    tmp->spectrum = 0;
    ParticleSources[isource].GunParticles.append(tmp);

    MainWindow::on_pbGunRefreshparticles_clicked();
    ui->lwGunParticles->setCurrentRow( ParticleSources[isource].GunParticles.size()-1 );
}

void MainWindow::on_pbGunRemove_clicked()
{
  int isource = ui->cobParticleSource->currentIndex();
  if (ParticleSources[isource].GunParticles.size() == 0) return;

  int iparticle = ui->lwGunParticles->currentRow();
  if (iparticle == -1)
    {
      message("Select a particle to remove!", this);
      return;
    }
  delete ParticleSources[isource].GunParticles[iparticle];
  ParticleSources[isource].GunParticles.remove(iparticle);  
  MainWindow::on_pbGunRefreshparticles_clicked();
  int errCode = ParticleSources.CheckSource(isource);
  if (errCode>0)
    {
      QString err = ParticleSources.getErrorString(errCode);
      message("Source checker reports an error: "+err, this);
    }
}

void MainWindow::on_cobGunSourceType_currentIndexChanged(int index)
{
  QList <QString> s;
  switch (index)
    {
    default:
    case 0: s <<""<<""<<"";
      break;
    case 1: s <<"Size"<<""<<"";
      break;
    case 2: s <<"SizeX"<<"SizeY"<<"";
      break;
    case 3: s <<"Radius"<<""<<"";
      break;
    case 4: s <<"SizeX"<<"SizeY"<<"SizeZ";
      break;
    case 5: s <<"Radius"<<""<<"Height";
    }
  ui->lGun1DSize->setText(s[0]);
  ui->lGun2DSize->setText(s[1]);
  ui->lGun3DSize->setText(s[2]);
  ui->fGun1D->setDisabled(s[0].isEmpty());
  ui->fGun2D->setDisabled(s[1].isEmpty());
  ui->fGun3D->setDisabled(s[2].isEmpty());

  ui->fGunPhi->setEnabled(index != 0);
  ui->fGunTheta->setEnabled(index != 0);
  ui->fGunPsi->setEnabled(index > 1);

  if (index == 1)
  {
      ui->labGunPhi->setText("Phi");
      ui->labGunTheta->setText("Theta");
  }
  else
  {
      ui->labGunPhi->setText("around X");
      ui->labGunTheta->setText("around Y");
  }
}

void MainWindow::on_cobGunParticle_activated(int index)
{
     //next 4 obsolete?
   if (ParticleSources.size() == 0) return;
   int isource = ui->cobParticleSource->currentIndex();
   if (ParticleSources[isource].GunParticles.isEmpty()) return;
   if (isource > ParticleSources.size() - 1) return;

   int particle = ui->lwGunParticles->currentRow();
   if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
   ParticleSources[isource].GunParticles[particle]->ParticleId = index;
   MainWindow::on_pbGunRefreshparticles_clicked();
   ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_ledGunEnergy_editingFinished()
{
  //if (BulkUpdate) return;
  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;

  ParticleSources[isource].GunParticles[particle]->energy = ui->ledGunEnergy->text().toDouble();
  MainWindow::on_pbGunRefreshparticles_clicked();
  ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_ledGunParticleWeight_editingFinished()
{
  //if (BulkUpdate) return;
  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
  ParticleSources[isource].GunParticles[particle]->StatWeight = ui->ledGunParticleWeight->text().toDouble();
  MainWindow::on_pbGunRefreshparticles_clicked();
  ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_cbIndividualParticle_clicked(bool checked)
{
  //if (BulkUpdate) return;
  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;

  if (particle == 0 && !checked)
    {
      ui->cbIndividualParticle->setChecked(true);
      message("First particle should be 'Individual'", this);
      return;//update on_toggle
    }
  ParticleSources[isource].GunParticles[particle]->Individual = checked;
  if (!checked) ParticleSources[isource].GunParticles[particle]->LinkedTo = ui->leiParticleLinkedTo->text().toInt();
  MainWindow::on_pbGunRefreshparticles_clicked();
  ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_leiParticleLinkedTo_editingFinished()
{
  //if (BulkUpdate) return;
  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
  int iLinkedTo = ui->leiParticleLinkedTo->text().toInt();
  if (iLinkedTo == particle)
    {
      ui->leiParticleLinkedTo->setText("0");
      message("Particle cannot be linked to itself!", this);
      MainWindow::on_leiParticleLinkedTo_editingFinished();
      return;
    }

  int TotParticles = ParticleSources[isource].GunParticles.size();
  if (iLinkedTo>TotParticles-1 || iLinkedTo<0)
    {
      if (iLinkedTo == 0) return; //protection - no particles are defined
      ui->leiParticleLinkedTo->setText("0");
      message("Particle index is out of range: "+QString::number(TotParticles)+" particle are defined", this);
      MainWindow::on_leiParticleLinkedTo_editingFinished();
      return;
    }

  ParticleSources[isource].GunParticles[particle]->LinkedTo = iLinkedTo;
  MainWindow::on_pbGunRefreshparticles_clicked();
  ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_ledLinkingProbability_editingFinished()
{
  //if (BulkUpdate) return;
  double val = ui->ledLinkingProbability->text().toDouble();
  if (val<0 || val>1)
    {
      ui->ledLinkingProbability->setText("0");
      message("Error in probability value: should be between 0 and 1",this);
      val = 0;
    }

  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
  ParticleSources[isource].GunParticles[particle]->LinkingProbability = val;
  MainWindow::on_pbGunRefreshparticles_clicked();
  ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_pbGunLoadSpectrum_clicked()
{
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load energy spectrum", GlobSet->LastOpenDir, "Data files (*.dat);;All files (*.*)");
  qDebug()<<fileName;
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
  ParticleSources.LoadGunEnergySpectrum(isource, particle, fileName);
  MainWindow::on_pbGunRefreshparticles_clicked();
  ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_pbGunShowSpectrum_clicked()
{
  int isource = ui->cobParticleSource->currentIndex();
  int particle = ui->lwGunParticles->currentRow();
  if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
  GraphWindow->Draw(ParticleSources[isource].GunParticles[particle]->spectrum);
}

void MainWindow::on_pbGunDeleteSpectrum_clicked()
{
   int isource = ui->cobParticleSource->currentIndex();
   int particle = ui->lwGunParticles->currentRow();
   if (particle<0 || particle > ParticleSources[isource].GunParticles.size()-1) return;
   if (ParticleSources[isource].GunParticles[particle]->spectrum)
     {
       delete ParticleSources[isource].GunParticles[particle]->spectrum;
       ParticleSources[isource].GunParticles[particle]->spectrum = 0;
     }
   ui->pbGunShowSpectrum->setEnabled(false);
   ui->pbGunDeleteSpectrum->setEnabled(false);
   MainWindow::on_pbGunRefreshparticles_clicked();
   ui->lwGunParticles->setCurrentRow(particle);
}

void MainWindow::on_ledGunAverageNumPartperEvent_editingFinished()
{
   double val = ui->ledGunAverageNumPartperEvent->text().toDouble();
   if (val<0)
     {
       message("Average number of particles per event should be more than 0", this);
       ui->ledGunAverageNumPartperEvent->setText("0.01");
     }
}

void MainWindow::on_pbRemoveSource_clicked()
{
  if (ParticleSources.size() == 0) return;

  int isource = ui->cobParticleSource->currentIndex();
  ParticleSources.remove(isource);
  ui->cobParticleSource->removeItem(isource);
  ui->cobParticleSource->setCurrentIndex(isource-1);
  if (ParticleSources.size() == 0)
    {
      //No sources left!
      ParticleSources.append(new ParticleSourceStructure());
      MainWindow::on_pbUpdateSourcesIndication_clicked(); //to clean indication
      ParticleSources.remove(0);
      ui->cobParticleSource->removeItem(0);
      MainWindow::on_pbUpdateSourcesIndication_clicked();
      ui->fParticleSources->setEnabled(false);
    }
  else MainWindow::on_pbUpdateSourcesIndication_clicked();
}

void MainWindow::on_pbAddSource_clicked()
{
  ParticleSources.append(new ParticleSourceStructure());
  ///ui->sbSourceNumber->setValue(ParticleSources.size()-1);
  ui->cobParticleSource->addItem(ParticleSources.last().name);
  ui->cobParticleSource->setCurrentIndex(ParticleSources.size()-1);
  ///MainWindow::on_pbUpdateSourcesIndication_clicked();
}

void MainWindow::on_pbUpdateSources_clicked()
{
  if (BulkUpdate) return;
  if (DoNotUpdateGeometry) return;
  if (ShutDown) return;
  if (ParticleSources.size() == 0)
    {
      qWarning()<<"Attempt to update particle source while there are no defined ones!";
      return;
    }
  qDebug()<<"Update sources";
  int isource = ui->cobParticleSource->currentIndex();
  if (isource < 0 || isource > ParticleSources.size()-1)
    {
      message("Error: Attempting to update particle source parameters, but source number is out of bounds!",this);
      return;
    }
  ParticleSources[isource].Activity = ui->ledSourceActivity->text().toDouble();
  //ParticleSources[isource].name = ui->leSourceName->text();
  ParticleSources[isource].index = ui->cobGunSourceType->currentIndex();
  ParticleSources[isource].X0 = ui->ledGunOriginX->text().toDouble();
  ParticleSources[isource].Y0 = ui->ledGunOriginY->text().toDouble();
  ParticleSources[isource].Z0 = ui->ledGunOriginZ->text().toDouble();
  ParticleSources[isource].Phi = ui->ledGunPhi->text().toDouble();
  ParticleSources[isource].Theta = ui->ledGunTheta->text().toDouble();
  ParticleSources[isource].Psi = ui->ledGunPsi->text().toDouble();
  ParticleSources[isource].size1 = ui->ledGun1DSize->text().toDouble();
  ParticleSources[isource].size2 = ui->ledGun2DSize->text().toDouble();
  ParticleSources[isource].size3 = ui->ledGun3DSize->text().toDouble();
  ParticleSources[isource].CollPhi = ui->ledGunCollPhi->text().toDouble();
  ParticleSources[isource].CollTheta = ui->ledGunCollTheta->text().toDouble();
  ParticleSources[isource].Spread = ui->ledGunSpread->text().toDouble();

  //if (!GeoManagerImported)
  if (Detector->isGDMLempty())
    { //check world size
      double XYm = 0;
      double  Zm = 0;
      for (int isource = 0; isource < ParticleSources.size(); isource++)
        {
          double msize =   ParticleSources[isource].size1;
          updateMax(msize, ParticleSources[isource].size2);
          updateMax(msize, ParticleSources[isource].size3);

          updateMax(XYm, fabs(ParticleSources[isource].X0)+msize);
          updateMax(XYm, fabs(ParticleSources[isource].Y0)+msize);
          updateMax(Zm,  fabs(ParticleSources[isource].Z0)+msize);
        }

      double currXYm = Detector->Sandwich.WorldSizeXY;
      double  currZm = Detector->Sandwich.WorldSizeZ;
      if (XYm>currXYm || Zm>currZm)
        {
          //need to override
          ui->cbFixedTopSize->setChecked(true);
          ui->ledTopSizeXY->setText( QString::number( (XYm>currXYm)?XYm:currXYm) );
          ui->ledTopSizeZ->setText(  QString::number( (Zm>currZm)?Zm:currZm) );
          MainWindow::ReconstructDetector();
        }
    }
  MainWindow::on_pbGunShowSource_clicked();
}

void MainWindow::on_cobParticleSource_currentIndexChanged(int index)
{
  //qDebug() << "Attempt to set cob to"<<index;
  if (index == -1) return;
  if (ParticleSources.size() == 0) return;
  int numSources = ParticleSources.size();
  if (index > numSources-1)
    {

      qWarning() << "||| Attempt to set non-existent particle source in cob!";
      return;
    }

  ui->fGunParticle->setEnabled(ParticleSources[index].GunParticles.size() != 0);
  MainWindow::on_pbUpdateSourcesIndication_clicked();
  //PartSource_CurrentlySelectedParticle = 0;
  MainWindow::on_pbGunRefreshparticles_clicked();
}

void MainWindow::on_pbUpdateSourcesIndication_clicked()
{
  //qDebug() << "______ Update sources triggered";
  int numSources = ParticleSources.size();
  ui->labPartSourcesDefined->setText(QString::number(numSources));
  int isource = ui->cobParticleSource->currentIndex();
  //qDebug() << "Def sources:"<<numSources<<"cobSize"<<ui->cobParticleSource->count()<<"cob index:"<<isource;

  if (isource == -1 && numSources>0)
    {
      //updating cobParticleSources content
      if (ui->cobParticleSource->count() != numSources)
        {
          ui->cobParticleSource->clear();
          for (int i=0; i<numSources; i++) ui->cobParticleSource->addItem(ParticleSources[i].name);
        }
      else
        {
          for (int i=0; i<numSources; i++) ui->cobParticleSource->setItemText(i, ParticleSources[i].name);
        }
      ui->cobParticleSource->setCurrentIndex(0);
      return;
    }

  if (isource == -1) return;
  if (numSources == 0)
    {
      ui->fParticleSources->setEnabled(false);
      ///if (isource != 0)  ui->sbSourceNumber->setValue(0);
      ui->cobParticleSource->setCurrentIndex(-1);
      return;
    }
  else ui->fParticleSources->setEnabled(true);

    if (isource > numSources-1)
      {
        ///ui->sbSourceNumber->setValue(0);
        ui->cobParticleSource->setCurrentIndex(0);
        return;
      }

    bool BulkUpdateCopy = BulkUpdate; //it could be set outside to true already, do not want to reselt to false on exit
    BulkUpdate = true;

    double activity = ParticleSources[isource].Activity;
    ui->ledSourceActivity->setText(QString::number(activity));
    double fraction = 0;
    double TotalActivity = ParticleSources.getTotalActivity();
    if (TotalActivity != 0) fraction = activity / TotalActivity * 100.0;
    ui->labOfTotal->setText(QString::number(fraction, 'g', 3)+"%");

    ///ui->leSourceName->setText(ParticleSources[isource].name);
    ui->cobGunSourceType->setCurrentIndex(ParticleSources[isource].index);
    ui->ledGunOriginX->setText(QString::number(ParticleSources[isource].X0));
    ui->ledGunOriginY->setText(QString::number(ParticleSources[isource].Y0));
    ui->ledGunOriginZ->setText(QString::number(ParticleSources[isource].Z0));
    ui->ledGunPhi->setText(QString::number(ParticleSources[isource].Phi));
    ui->ledGunTheta->setText(QString::number(ParticleSources[isource].Theta));
    ui->ledGunPsi->setText(QString::number(ParticleSources[isource].Psi));
    ui->ledGun1DSize->setText(QString::number(ParticleSources[isource].size1));
    ui->ledGun2DSize->setText(QString::number(ParticleSources[isource].size2));
    ui->ledGun3DSize->setText(QString::number(ParticleSources[isource].size3));
    ui->ledGunCollPhi->setText(QString::number(ParticleSources[isource].CollPhi));
    ui->ledGunCollTheta->setText(QString::number(ParticleSources[isource].CollTheta));
    ui->ledGunSpread->setText(QString::number(ParticleSources[isource].Spread));

    BulkUpdate = BulkUpdateCopy;
    //update particle indication
    MainWindow::on_pbGunRefreshparticles_clicked(); //refresh indication
}

void MainWindow::on_pbGunShowSource_clicked()
{
   if (BulkUpdate) return;
   int isource = ui->cobParticleSource->currentIndex();
   if (isource < 0) return;
   if (isource > ParticleSources.size()-1)
     {
       message("Source number is out of bounds!",this);
       return;
     }
   MainWindow::ShowSource(isource, true);
}

void MainWindow::on_ledSourceActivity_editingFinished()
{
    MainWindow::on_pbUpdateSourcesIndication_clicked();
}

void MainWindow::on_lwGunParticles_currentRowChanged(int /*currentRow*/)
{
  MainWindow::SourceUpdateThisParticleIndication();
}

void MainWindow::on_pbSaveParticleSource_clicked()
{
  QString starter = (GlobSet->LibParticleSources.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibParticleSources;
  QString fileName = QFileDialog::getSaveFileName(this, "Export particle source", starter, "Json files (*.json)");
  if (fileName.isEmpty()) return;
  QFileInfo file(fileName);
  if (file.suffix().isEmpty()) fileName += ".json";
  QJsonObject json, js;
  ParticleSources.writeSourceToJson(ui->cobParticleSource->currentIndex(), json);
  js["ParticleSource"] = json;
  SaveJsonToFile(js, fileName);
}

void MainWindow::on_pbLoadParticleSource_clicked()
{
  QMessageBox msgBox(this);
  msgBox.setText("You are about to replace this source with a source from a file.");
  msgBox.setInformativeText("Proceed?");
  msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
  msgBox.setDefaultButton(QMessageBox::Cancel);
  int ret = msgBox.exec();
  if (ret == QMessageBox::Cancel) return;

  QString starter = (GlobSet->LibParticleSources.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibParticleSources;
  QString fileName = QFileDialog::getOpenFileName(this, "Import particle source", starter, "json files (*.json)");
  if (fileName.isEmpty()) return;
  int iSource = ui->cobParticleSource->currentIndex();
  QJsonObject json, js;
  bool ok = LoadJsonFromFile(json, fileName);
  if (!ok) return;
  if (!json.contains("ParticleSource"))
    {
      message("Json file format error", this);
      return;
    }
  int oldPartCollSize = Detector->ParticleCollection.size();
  js = json["ParticleSource"].toObject();
  ParticleSources.readSourceFromJson(iSource, js);
  ui->cobParticleSource->setItemText(iSource, ParticleSources[iSource].name);

  MainWindow::on_pbUpdateSourcesIndication_clicked();
  int newPartCollSize = Detector->ParticleCollection.size();
  if (oldPartCollSize != newPartCollSize)
    {
      //qDebug() << oldPartCollSize <<"->"<<newPartCollSize;
      QString str = "Warning!\nThe following particle";
      if (newPartCollSize-oldPartCollSize>1) str += "s were";
      else str += " was";
      str += " added:\n";
      for (int i=oldPartCollSize; i<newPartCollSize; i++)
          str += Detector->ParticleCollection[i]->ParticleName + "\n";
      str += "\nSet the interaction data for the detector materials!";
      message(str,this);
    }

  MainWindow::ListActiveParticles(); //also updates all cobs with particle names!
}

void MainWindow::on_pbSingleSourceShow_clicked()
{
    double r[3];
    r[0] = ui->ledSingleX->text().toDouble();
    r[1] = ui->ledSingleY->text().toDouble();
    r[2] = ui->ledSingleZ->text().toDouble();

    clearGeoMarkers();
    GeoMarkerClass* marks = new GeoMarkerClass("Source", 3, 10, kBlack);
    marks->SetNextPoint(r[0], r[1], r[2]);   
    GeoMarkers.append(marks);
    GeoMarkerClass* marks1 = new GeoMarkerClass("Source", 4, 3, kRed);
    marks1->SetNextPoint(r[0], r[1], r[2]);    
    GeoMarkers.append(marks1);

    ShowGeometry(false);
}

void MainWindow::on_ledGunSpread_editingFinished()
{
    if (ui->ledGunSpread->text().toDouble() < 1.0)
        message("Warning: simulation with sources containing linking particles is slow for angles <1 degree!", this);
}

void MainWindow::on_cobParticleSource_editTextChanged(const QString &arg1)
{
    //qDebug() << "Text edit:"<<arg1;
    int isource = ui->cobParticleSource->currentIndex();
    //qDebug() << "current source:"<<isource;
    if (isource<0 || isource>ParticleSources.size()-1) return;
    ParticleSources[isource].name = arg1;
    ui->cobParticleSource->setItemText(isource, arg1);
}

void MainWindow::on_pbAddParticleToStack_clicked()
{
    //check is this particle's energy in the defined range?
    double energy = ui->ledParticleStackEnergy->text().toDouble();
    int iparticle = ui->cobParticleToStack->currentIndex();
    QList<QString> mats;

    for (int imat=0; imat<MaterialCollection->size(); imat++)
      {
//        qDebug()<<imat;
//        qDebug()<<" mat:"<<(*MaterialCollection)[imat]->name;
        if ( !(*MaterialCollection)[imat]->MatParticle[iparticle].TrackingAllowed )
          {
//            qDebug()<<"  tracking not allowed";
            continue;
          }
        if ( (*MaterialCollection)[imat]->MatParticle[iparticle].MaterialIsTransparent )
          {
//            qDebug()<<"  user set as transparent for this particle!";
            continue;
          }

//        qDebug()<<" getting energy range...";
        if ((*MaterialCollection)[imat]->MatParticle[iparticle].InteractionDataX.isEmpty())
          {
//           qDebug()<<"  Interaction data are not loaded!";
           mats << (*MaterialCollection)[imat]->name;
           continue;
          }
        double minE = (*MaterialCollection)[imat]->MatParticle[iparticle].InteractionDataX.first();
//        qDebug()<<"  min energy:"<<minE;
        double maxE = (*MaterialCollection)[imat]->MatParticle[iparticle].InteractionDataX.last();
//        qDebug()<<"  max energy:"<<maxE;

        if (energy<minE || energy>maxE) mats << (*MaterialCollection)[imat]->name;
//        qDebug()<<"->"<<mats;
      }
//    qDebug()<<"reporting...";

    if (mats.size()>0)
      {
        QString str = Detector->ParticleCollection[iparticle]->ParticleName;
        str += " energy (" + ui->ledParticleStackEnergy->text() + " keV) ";
        str += "is conflicting with the defined interaction energy range of material";
        if (mats.size()>1) str += "s";
        str += ": ";
        for (int i=0; i<mats.size(); i++)
            str += mats[i]+", ";

        str.chop(2);
        message(str,this);
      }

    WindowNavigator->BusyOn();
    int numCopies = ui->sbNumCopies->value();
    ParticleStack.reserve(ParticleStack.size() + numCopies);
    for (int i=0; i<numCopies; i++)
    {
       ParticleOnStack *tmp = new ParticleOnStack(ui->cobParticleToStack->currentIndex(),
                                                  ui->ledParticleStackX->text().toDouble(), ui->ledParticleStackY->text().toDouble(), ui->ledParticleStackZ->text().toDouble(),
                                                  ui->ledParticleStackVx->text().toDouble(), ui->ledParticleStackVy->text().toDouble(), ui->ledParticleStackVz->text().toDouble(),
                                                  ui->ledParticleStackTime->text().toDouble(), ui->ledParticleStackEnergy->text().toDouble());
       ParticleStack.append(tmp);
    }
    MainWindow::on_pbRefreshStack_clicked();
    WindowNavigator->BusyOff(false);
}

void MainWindow::on_pbRefreshStack_clicked()
{
  ui->teParticleStack->clear();
  int elements = ParticleStack.size();


  if (ui->cbHideStackText->isChecked())
  {
     QString str;
     str.setNum(elements);
     str = "Stack contains "+str;
     if (elements == 1) str += " particle"; else str += " particles";
     ui->teParticleStack->append(str);
  }
  else
  {
    QString record, lastRecord;
    int counter = 0; //0 since it will inc on i==0
    int first = 0;
    for (int i=0; i<elements; i++)
      {
        record = Detector->ParticleCollection[ParticleStack[i]->Id]->ParticleName + "   ";
        record += "x,y,z: (" + QString::number(ParticleStack[i]->r[0]) + ","
                             + QString::number(ParticleStack[i]->r[1]) + ","
                             + QString::number(ParticleStack[i]->r[2]) + ")   ";
        record += "i,j,k: (" + QString::number(ParticleStack[i]->v[0]) + ","
                             + QString::number(ParticleStack[i]->v[1]) + ","
                             + QString::number(ParticleStack[i]->v[2]) + ")   ";
        record += "e: " + QString::number(ParticleStack[i]->Energy) + " keV   ";
        record += "t: " + QString::number(ParticleStack[i]->t);

        if (i == 0) lastRecord = record;

        if (record == lastRecord) counter++;
        else
          {
            QString out = QString::number(first);
            if (counter > 1) out += "-" + QString::number(first + counter - 1);
            out += "> " + lastRecord;
            ui->teParticleStack->append(out);
            counter = 1;
            first = i;
            lastRecord = record;
          }
      }

    if (!record.isEmpty())
      {
        QString out = QString::number(first);
        if (counter > 1) out += "-" + QString::number(first + counter - 1);
        out += "> " + lastRecord;
        ui->teParticleStack->append(out);
      }
  }

  ui->sbStackElement->setValue(elements-1);
  ui->pbTrackStack->setEnabled(elements);
}

void MainWindow::on_pbRemoveFromStack_clicked()
{
    int element = ui->sbStackElement->value();
    int StackSize = ParticleStack.size();
    if (element > StackSize-1) return;
    delete ParticleStack[element];
    ParticleStack.remove(element);
    MainWindow::on_pbRefreshStack_clicked();
}

void MainWindow::on_pbClearAllStack_clicked()
{
  int StackSize = ParticleStack.size();
  if (StackSize == 0) return;

  //if(QMessageBox::question(this, "", "Clear all stack?", QMessageBox::Yes, QMessageBox::Cancel, QMessageBox::NoButton) == QMessageBox::Yes)
    {
      for (int i=StackSize-1; i>-1; i--)
        {
          delete ParticleStack[i];
          ParticleStack.remove(i);
        }
      MainWindow::on_pbRefreshStack_clicked();
    }
}
