//ANTS2
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "materialinspectorwindow.h"
#include "outputwindow.h"
#include "detectorclass.h"

//Qt
#include <QMessageBox>

//Root
#include "TColor.h"
#include "TROOT.h"

void MainWindow::ListActiveParticles()
{
    //visualization
    ui->lwParticles->clear();
    int NumParticles = Detector->ParticleCollection.size();
    QString tmpStr, tmpStr2;
    for (int i=0; i<NumParticles; i++){
        //tmpStr2.setNum(ParticleCollection[i]->ParticleId);
        tmpStr2.setNum(i);
        tmpStr=tmpStr2+">  "+Detector->ParticleCollection[i]->ParticleName;

        QListWidgetItem* pItem =new QListWidgetItem(tmpStr);
        ui->lwParticles->addItem(pItem);
    }    

    MainWindow::on_pbRefreshParticles_clicked();
}

void MainWindow::on_pbShowColorCoding_pressed()
{
  for (int i=0; i<ui->lwParticles->count(); i++)
    {
      TColor* rc = gROOT->GetColor(i +1 +ui->sbParticleTrackColorIndexAdd->value());
      QColor qc = QColor(255*rc->GetRed(), 255*rc->GetGreen(), 255*rc->GetBlue(), 255*rc->GetAlpha());

      QListWidgetItem* pItem = ui->lwParticles->item(i);
      pItem->setTextColor(qc);
    }
}

void MainWindow::on_pbShowColorCoding_released()
{
  for (int i=0; i<ui->lwParticles->count(); i++)
    {
      QListWidgetItem* pItem = ui->lwParticles->item(i);
      pItem->setTextColor(Qt::black);
    }
}


void MainWindow::on_pbRefreshParticles_clicked()
{
    ui->cobParticleToInspect->clear();
    ui->cobParticleToStack->clear();
    ui->cobGunParticle->clear();
    ui->cobShowParticle->clear();

    for (int i=0; i<Detector->ParticleCollection.size();i++)
    {
        ui->cobParticleToInspect->addItem(Detector->ParticleCollection[i]->ParticleName);
        ui->cobParticleToStack->addItem(Detector->ParticleCollection[i]->ParticleName);
        ui->cobGunParticle->addItem(Detector->ParticleCollection[i]->ParticleName);
        ui->cobShowParticle->addItem(Detector->ParticleCollection[i]->ParticleName);
    }
}

void MainWindow::on_lwParticles_currentRowChanged(int currentRow)
{
   if (currentRow<0) return;

   ui->cobParticleToInspect->setCurrentIndex(currentRow);
   MainWindow::on_cobParticleToInspect_currentIndexChanged(ui->cobParticleToInspect->currentIndex()); //refresh indication
}

void MainWindow::on_leParticleName_editingFinished()
{
    ui->cobParticleToInspect->setCurrentIndex(-1);
}

void MainWindow::on_pbAddparticleToActive_clicked()
{
    QString name = ui->leParticleName->text();
    int charge = ui->ledParticleCharge->text().toInt();
    double mass = ui->ledMass->text().toDouble();
    ParticleType type;

    if (!name.compare("gamma",Qt::CaseSensitive) || !name.compare("neutron",Qt::CaseSensitive) )
     {
         message("Cannot update neutral particles", this);
         return;
     }

    if (charge == 0)
      {
        if (name != "gamma" && name != "neutron")
          {
            message("Only gamma and neutron can be neutral!", this);
            ui->ledParticleCharge->setText( "1" );
            return;
          }
      }

    if (name == "gamma") type = _gamma_;
    else if (name == "neutron") type = _neutron_;
    else type = _charged_;

    int size = Detector->ParticleCollection.size();
    bool flagFound = false;
    int index;
    for (index = 0; index < size; index++)
        if (!name.compare(Detector->ParticleCollection[index]->ParticleName ,Qt::CaseSensitive))
          {
            flagFound = true;
            break;
          }

    if (flagFound)
      {
        qDebug()<<"This name already in use! index = "<<index;
        switch( QMessageBox::information( this, "What to do?",
                                                  "Particle with this name alredy added!",
                                                  "Overwrite", "Cancel",
                                                  0, 1 ) )
          {
                case 0:
//                 qDebug()<<"Overwrite";
                   Detector->ParticleCollection[index]->ParticleName = name;
                   Detector->ParticleCollection[index]->type = type;
                   Detector->ParticleCollection[index]->charge = charge;
                   Detector->ParticleCollection[index]->mass = mass;
                   ui->cobParticleToInspect->setCurrentIndex(index);
                   ReconstructDetector();
                   return;
                //case 1:
               default:
//                 qDebug()<<"cancel";
                   return;
         }
    }

    qDebug()<<"new name!"<<name;
    ParticleDescriptor* pd = new ParticleDescriptor(name, type, charge, mass);
    Detector->ParticleCollection.append(pd);
    MaterialCollection->RegisterNewParticle(); //resize particle-indexed properties of materials
    index = Detector->ParticleCollection.size()-1;

    qDebug()<<"added!";
    MainWindow::ListActiveParticles(); //also updates and resets cob for active particle inspector    
    MIwindow->VisualizeTmpMaterial();
    ui->cobParticleToInspect->setCurrentIndex(index);
    ReconstructDetector();
}

void MainWindow::on_cobParticleToInspect_currentIndexChanged(int index)
{
    if (index<0) return;

    //loading to the temporary particle container.
    tmpParticle.ParticleName =  Detector->ParticleCollection[index]->ParticleName;
    tmpParticle.charge = Detector->ParticleCollection[index]->charge;
    tmpParticle.mass = Detector->ParticleCollection[index]->mass;

    MainWindow::VisualizeTmpParticle();
}

void MainWindow::VisualizeTmpParticle()
{
    //filling the fields
    ui->leParticleName->setText(tmpParticle.ParticleName);
    //if (tmpParticle.charged) ui->leCharged->setText("charged");
    //else ui->leCharged->setText("neutral");
    ui->ledParticleCharge->setText(QString::number(tmpParticle.charge));
    ui->ledMass->setText(QString::number(tmpParticle.mass));
}

void MainWindow::on_pbRemoveParticle_clicked()
{
    int Id = ui->lwParticles->currentRow();
    MainWindow::RemoveParticle(Id);
}

//--------------remove particle----------------
void MainWindow::RemoveParticle(int Id)
{
  switch( QMessageBox::information( this, "Are you sure?",
                                    "Really remove particle " + Detector->ParticleCollection[Id]->ParticleName + " ?",
                                        "Yes", "Cancel",
                                        0, 1 ) )
    {
      case 0: break;  //OK received;
      default: return;
    }

  //checking is it one of the secondary particles
  bool flagFound = false;
  QString tmpStr;
  for (int m=0; m<MaterialCollection->size();m++)
      for (int p=0; p<Detector->ParticleCollection.size(); p++)
      {
          if ( (*MaterialCollection)[m]->MatParticle.size() == 0) continue;
          for (int s=0; s<(*MaterialCollection)[m]->MatParticle[p].Terminators.size();s++)
              for (int v=0; v<(*MaterialCollection)[m]->MatParticle[p].Terminators[s].GeneratedParticles.size();v++)
                  if (Id == (*MaterialCollection)[m]->MatParticle[p].Terminators[s].GeneratedParticles[v])
                  {
                      flagFound = true;
                      tmpStr += (*MaterialCollection)[m]->name + "\n";
                  }
      }

  if (flagFound)
  {
      message("This particle is a secondary particle defined in neutron capture\nIt appears in the following materials:\n"+tmpStr, this);
      return;
  }

  //checking if this particle is used in a particle stack or defined on the stack
  for (int i=0; i<ParticleStack.size(); i++)
    {
      if (ParticleStack[i]->Id == Id)
        {
          message("This particle is currently on the simulation stack!", this);
          return;
        }
    }
  for (int isource=0; isource<ParticleSources.size(); isource++ )
    {
      for (int ip = 0; ip<ParticleSources[isource].GunParticles.size(); ip++)
        {
          if (ParticleSources[isource].GunParticles[ip]->ParticleId == Id)
            {
              message("This particle is currently in use in the particle source "+ParticleSources[isource].name+ "(index "+QString::number(isource)+")", this);
              return;
            }
        }
    }

  //shifting all Id down so it is possible to remove this (Id) particle
  for (int m=0; m<MaterialCollection->size(); m++)
  {
    for (int p=0; p<Detector->ParticleCollection.size(); p++)
      {
          //replacing generated particles
          for (int s=0; s<(*MaterialCollection)[m]->MatParticle[p].Terminators.size();s++)
              for (int v=0; v<(*MaterialCollection)[m]->MatParticle[p].Terminators[s].GeneratedParticles.size();v++)
              {
                  int thisParticle = (*MaterialCollection)[m]->MatParticle[p].Terminators[s].GeneratedParticles[v];
                  if (thisParticle > Id ) (*MaterialCollection)[m]->MatParticle[p].Terminators[s].GeneratedParticles[v] = thisParticle-1;
              }

          //shifting down all particles above Id
          if (p > Id) (*MaterialCollection)[m]->MatParticle[p-1] = (*MaterialCollection)[m]->MatParticle[p];
      }
    (*MaterialCollection)[m]->MatParticle.resize(Detector->ParticleCollection.size()-1);
  }
  //same with the tmpMaterial
  for (int p=0; p<Detector->ParticleCollection.size(); p++)
    {
        //replacing generated particles
        for (int s=0; s<MaterialCollection->tmpMaterial.MatParticle[p].Terminators.size();s++)
            for (int v=0; v<MaterialCollection->tmpMaterial.MatParticle[p].Terminators[s].GeneratedParticles.size();v++)
            {
                int thisParticle = MaterialCollection->tmpMaterial.MatParticle[p].Terminators[s].GeneratedParticles[v];
                if (thisParticle > Id ) MaterialCollection->tmpMaterial.MatParticle[p].Terminators[s].GeneratedParticles[v] = thisParticle-1;
            }

        //shifting down all particles above Id
        if (p > Id) MaterialCollection->tmpMaterial.MatParticle[p-1] = MaterialCollection->tmpMaterial.MatParticle[p];
    }

  //shifting all Ids in the particle stack and particle sources
  for (int i=0; i<ParticleStack.size(); i++)
    if (ParticleStack[i]->Id > Id) ParticleStack[i]->Id--;
  for (int isource=0; isource<ParticleSources.size(); isource++ )
    {
      for (int ip = 0; ip<ParticleSources[isource].GunParticles.size(); ip++)
          if (ParticleSources[isource].GunParticles[ip]->ParticleId > Id) ParticleSources[isource].GunParticles[ip]->ParticleId--;
    }

  Detector->ParticleCollection.remove(Id);//removing
  MainWindow::ListActiveParticles();
  MIwindow->VisualizeTmpMaterial();

  ReconstructDetector();
}
