//ANTS2 modules and windows
//#include "TCanvas.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "checkupwindowclass.h"
#include "reconstructionwindow.h"
#include "windownavigatorclass.h"
#include "pms.h"
#include "outputwindow.h"
#include "sensorlrfs.h"
#include "detectorclass.h"
#include "graphwindowclass.h"
#include "TApplication.h"
#include "globalsettingsclass.h"
#include "genericscriptwindowclass.h"
#include "interfacetocheckerscript.h"
#include "scatteronmetal.h"
#include "interfacetoglobscript.h"

#include "TPrincipal.h"
#include "TBrowser.h"

#include "geometrywindowclass.h"
#include "TVirtualGeoTrack.h"
#include "phscatclaudiomodel.h"
//#include "materialcollectionclass.h"

#include "TMultiGraph.h"
#include "TLegend.h"

//Qt
#include <QtGui>
#include <QObject>
#include <QFileDialog>

static QVector<TrackHolderClass> tracks;
static TVector3 NormViz;

void MainWindow::on_pbST_AngleCos_clicked()
{
  //vector photon dir
  TVector3 ph(ui->ledST_i->text().toDouble(), ui->ledST_j->text().toDouble(), ui->ledST_k->text().toDouble());
  ph = ph.Unit();

  //vector surface normal
  TVector3 surf(-ui->ledST_si->text().toDouble(), -ui->ledST_sj->text().toDouble(), -ui->ledST_sk->text().toDouble());
  surf = surf.Unit();

  double cos = ph*surf;
  double ang = 180.0/3.1415926*acos(cos);

  ui->pbST_AngleCos->setText("Theta="+QString::number(ang, 'g', 3)+"  cos="+QString::number(cos, 'g', 3));
}

OpticalOverrideClass* InitOverride(Ui::MainWindow *ui, MaterialCollectionClass& MatCol, DetectorClass* Detector)
{
  MatCol.SetParticleCollection(&Detector->ParticleCollection); //need to function
  MatCol.AddNewMaterial();
  MatCol.AddNewMaterial();
  MatCol[0]->n = ui->ledST_Ref1->text().toDouble();
  MatCol[1]->n = ui->ledST_Ref2->text().toDouble();
  double wave = ui->ledST_wave->text().toDouble();
  MatCol.SetWave(false, wave, wave, 0, 1); //conversion of iWave=-1 to wavelength will give wave

  if (ui->cobOverrideTesterModel->currentIndex() == 0)
    {
      //Claudio's model
      PhScatClaudioModel* sc;
      switch (ui->cobST_ClaudioModelVersion->currentIndex())
        {
        case 0:
          sc = new PhScatClaudioModelV2d2(&MatCol, 0, 1);
          break;
        case 1:
          sc = new PhScatClaudioModelV2d1(&MatCol, 0, 1);
          break;
        case 2:
          sc = new PhScatClaudioModelV2(&MatCol, 0, 1);
          break;
        case 3:
          sc = new PhScatClaudioModelV1(&MatCol, 0, 1);
          break;
        default:
          message("Unexpected scattering model index");
          exit(666);
        }

      sc->sigma_alpha = ui->ledST_SigmaAlpha->text().toDouble();
      sc->sigma_h = ui->ledST_SigmaSpike->text().toDouble();
      sc->albedo = ui->ledST_albedo->text().toDouble();

      switch(ui->conST_heightModel->currentIndex())
        {
          case 0: sc->HeightDistribution = empirical; break;
          case 1: sc->HeightDistribution = gaussian; break;
          case 2: sc->HeightDistribution = exponential; break;
        default: exit(-123);
        }
      switch(ui->cobST_slopeModel->currentIndex())
        {
          case 0: sc->SlopeDistribution = trowbridgereitz; break;
          case 1: sc->SlopeDistribution = cooktorrance; break;
          case 2: sc->SlopeDistribution = bivariatecauchy; break;
        default: exit(-123);
        }
      return sc;
    }
  else
    {
      //DielectricToMetal model
      ScatterOnMetal* sm = new ScatterOnMetal(&MatCol, 0, 1);
      sm->RealN = ui->ledST_Ref2->text().toDouble();
      sm->ImaginaryN = ui->ledImNAfter->text().toDouble();
      return sm;
    }
}

void MainWindow::on_pbCSMtestmany_clicked()
{
  // Have to create a fake material collection with two materials
  // to provide refractive indices and iwave->wavelength conversion to the override class
  MaterialCollectionClass MatCol;
  //creating and filling override object and this fake material collection
  OpticalOverrideClass* ov = InitOverride(ui, MatCol, Detector);

  //surface normal and photon direction
  TVector3 SurfNorm(ui->ledST_si->text().toDouble(),
                    ui->ledST_sj->text().toDouble(),
                    ui->ledST_sk->text().toDouble());
  SurfNorm = SurfNorm.Unit();
  NormViz = SurfNorm; //visualization of tracks
  SurfNorm = - SurfNorm; //In ANTS2 navigator provides normal in the opposite direction
  double N[3];
  N[0] = SurfNorm.X();
  N[1] = SurfNorm.Y();
  N[2] = SurfNorm.Z();
  TVector3 PhotDir(ui->ledST_i->text().toDouble(),
                   ui->ledST_j->text().toDouble(),
                   ui->ledST_k->text().toDouble());
  PhotDir = PhotDir.Unit();
  double K[3];

  //report configuration
  ov->printConfiguration(-1);

  //other inits
  tracks.clear();
  double d = 0.5; //offset - for drawing only
  tracks.append(TrackHolderClass(10, kRed)); //incoming photon track
  tracks.last().Nodes.append(TrackNodeStruct(d, d ,d, 0));
  tracks.last().Nodes.append(TrackNodeStruct(d-PhotDir.X(), d-PhotDir.Y(), d-PhotDir.Z(), 0));

  //preparing and running cycle with photons
  int abs, spike, lobe, lamb;
  abs = spike = lobe = lamb = 0;
  if (hist1) delete hist1;
  bool fDegrees = ui->cbST_cosToDegrees->isChecked();
  if (fDegrees) hist1 = new TH1D("statScatter", "Angle_scattered", 100, 0, 0);
  else          hist1 = new TH1D("statScatter", "Cos_scattered", 100, 0, 0);
  for (int i=0; i<ui->sbST_number->value(); i++)
    {
      K[0] = PhotDir.X(); //old is output dir after full cycle!
      K[1] = PhotDir.Y();
      K[2] = PhotDir.Z();

      ov->calculate(Detector->RandGen, K, N, -1);

      Color_t col;
      Int_t type;
      if (ov->Status == OpticalOverrideClass::Absorption)
        {
          abs++;
          continue;
        }
      else if (ov->Status == OpticalOverrideClass::SpikeReflection)
        {
          spike++;
          type = 0;
          col = 6; //0,magenta for Spike
        }
      else if (ov->Status == OpticalOverrideClass::LobeReflection)
        {
          lobe++;
          type = 1;
          col = 7; //1,teal for Lobe
        }
      else if (ov->Status == OpticalOverrideClass::LambertianReflection)
        {
          lamb++;
          type = 2;
          col = 3; //2,grean for lambert
        }
      else
        {
          type = 666;
          col = kBlue; //blue for error
        }

      tracks.append(TrackHolderClass(type, col));
      tracks.last().Nodes.append(TrackNodeStruct(d,d,d, 0));
      tracks.last().Nodes.append(TrackNodeStruct(d+K[0], d+K[1], d+K[2], 0));

      double costr = -SurfNorm[0]*K[0] -SurfNorm[1]*K[1] -SurfNorm[2]*K[2];

      if (fDegrees) hist1->Fill(180.0/3.1415926535*acos(costr));
      else          hist1->Fill(costr);
    }
  //show cos angle hist
  GraphWindow->Draw(hist1);
  //show tracks
  MainWindow::on_pbST_showTracks_clicked();
  //show stat of processes
  int sum = abs + spike + lobe + lamb;
  QString str = QString::number(abs) + "/" +
      QString::number(spike) + "/" +
      QString::number(lobe) + "/" +
      QString::number(lamb);
  if (sum>0)
    {
      str += "   (" + QString::number(1.0*abs/sum, 'g', 3) + "/" +
            QString::number(1.0*spike/sum, 'g', 3) + "/" +
            QString::number(1.0*lobe/sum, 'g', 3) + "/" +
            QString::number(1.0*lamb/sum, 'g', 3) + ")";
    }
  ui->leST_out->setText(str);
  delete ov;
}

void MainWindow::on_pbST_uniform_clicked()
{
  //   Have to create a fake material collection with two materials
  // to provide refractive indices and iwave->wavelength conversion to the override class
  MaterialCollectionClass MatCol;
  //creating and filling override object and this fake material collection
  OpticalOverrideClass* ov = InitOverride(ui, MatCol, Detector);

  TVector3 SurfNorm(ui->ledST_si->text().toDouble(),
                    ui->ledST_sj->text().toDouble(),
                    ui->ledST_sk->text().toDouble());
  SurfNorm = SurfNorm.Unit();
  SurfNorm = - SurfNorm; //In ANTS2 navigator provides normal in the opposite direction
  double N[3];
  N[0] = SurfNorm.X();
  N[1] = SurfNorm.Y();
  N[2] = SurfNorm.Z();

  //TVector3 PhotDir(ui->ledST_i->text().toDouble(),
  //                 ui->ledST_j->text().toDouble(),
  //                 ui->ledST_k->text().toDouble());
  //PhotDir = PhotDir.Unit();
  double K[3];

  ov->printConfiguration(-1);

  int abs, spike, lobe, lamb;
  abs = spike = lobe = lamb = 0;
  if (hist1) delete hist1;
  bool fDegrees = ui->cbST_cosToDegrees->isChecked();
  if (fDegrees) hist1 = new TH1D("statScatter", "Angle_scattered", 100, 0, 0);
  else          hist1 = new TH1D("statScatter", "Cos_scattered", 100, 0, 0);

  int num = ui->sbST_number->value();
  for (int i=0; i<num; i++)
    {
//      //Sphere function of Root - uniform distributed photon dir in 4Pi
//      double a=0, b=0, r2=1;
//      while (r2 > 0.25)
//        {
//          a  = Detector->RandGen->Rndm() - 0.5;
//          b  = Detector->RandGen->Rndm() - 0.5;
//          r2 =  a*a + b*b;
//        }
//      K[2] = ( -1.0 + 8.0 * r2 );
//      double scale = 8.0 * TMath::Sqrt(0.25 - r2);
//      K[0] = a*scale;
//      K[1] = b*scale;
//      //accept only in the same direction (2Pi) as the normal of the surface (normal is already inverted - it points to the second material)
//      if (K[0]*N[0]+K[1]*N[1]+K[2]*N[2]<0)
//        {
//          K[0]=-K[0];
//          K[1]=-K[1];
//          K[2]=-K[2];
//        }

      //diffuse illumination - lambertian is used
      double sin2angle = Detector->RandGen->Rndm();
      double angle = asin(sqrt(sin2angle));
      double yOff = cos(angle), zOff = -sin(angle);
      N[0] = 0; N[1] = 0; N[2] = -1;   // convention of photon tracer - normal is med1->med2
      K[0] = 0; K[1] = yOff; K[2] = zOff;  // -z direction on xy plane (incidence angle from 90 to 0)

      ov->calculate(Detector->RandGen, K, N, -1);

      if (ov->Status == OpticalOverrideClass::Absorption)
        {
          abs++;
          continue;
        }
      else if (ov->Status == OpticalOverrideClass::SpikeReflection)
          spike++;
      else if (ov->Status == OpticalOverrideClass::LobeReflection)
          lobe++;
      else if (ov->Status == OpticalOverrideClass::LambertianReflection)
          lamb++;
      else
        {
          qCritical()<<"Unknown process!";
          exit(666);
        }

      double costr = -N[0]*K[0] -N[1]*K[1] -N[2]*K[2];  // after scatter, K will be in positive Z direction
      if (fDegrees) hist1->Fill(180.0/3.1415926535*acos(costr));
      else          hist1->Fill(costr);
    }

  //show cos angle hist
  GraphWindow->Draw(hist1);

  //show stat of processes
  int sum = abs + spike + lobe + lamb;
  QString str = "Scat probability: "+QString::number(1.0*(sum-abs)/num, 'g', 4) + "  Processes:"+
      QString::number(abs) + "/" +
      QString::number(spike) + "/" +
      QString::number(lobe) + "/" +
      QString::number(lamb);
  if (sum>0)
    {
      str += "   (" + QString::number(1.0*abs/sum, 'g', 3) + "/" +
            QString::number(1.0*spike/sum, 'g', 3) + "/" +
            QString::number(1.0*lobe/sum, 'g', 3) + "/" +
            QString::number(1.0*lamb/sum, 'g', 3) + ")";
    }
  ui->leST_out->setText(str);
  delete ov;
}

void MainWindow::on_pbST_RvsAngle_clicked()
{
  // Have to create a fake material collection with two materials
  // to provide refractive indices and iwave->wavelength conversion to the override class
  MaterialCollectionClass MatCol;
  //creating and filling override object and this fake material collection
  OpticalOverrideClass* ov = InitOverride(ui, MatCol, Detector);

  int num = ui->sbST_number->value();
  QVector<double> Spike(91, 0), Lobe(91, 0), Diffuse(91, 0), Total(91, 0), Angle;
  double N[3], K[3];
  N[0] = 0;
  N[1] = 0;
  N[2] = 1.0;

  for (int iA=0; iA<91; iA++) //cycle by angle of incidence
    {
      double angle = iA;
      if (angle == 90) angle = 89.9;
      Angle.append(angle);
      //angle->photon direction
      double cosA = cos(3.1415926535*angle/180.0);
      double sinA = sin(3.1415926535*angle/180.0);
      for (int i=0; i<num; i++) //cycle by photons
        {
          //have to reset since K is modified by the override object
          K[0] = sinA;
          K[1] = 0;
          K[2] = cosA;

          ov->calculate(Detector->RandGen, K, N, -1);

          switch (ov->Status)
            {
            case OpticalOverrideClass::Absorption: continue; break;
            case OpticalOverrideClass::SpikeReflection: Spike[iA]++; break;
            case OpticalOverrideClass::LobeReflection: Lobe[iA]++; break;
            case OpticalOverrideClass::LambertianReflection: Diffuse[iA]++; break;
            default:
              qCritical()<<"Unknown process!";
              exit(666);
            }
        }
      Spike[iA] /= num;
      Lobe[iA] /= num;
      Diffuse[iA] /= num;
      Total[iA] = Spike[iA]+Lobe[iA]+Diffuse[iA];
    }

  // multigraph missbehave with "Show ontop"
//  //preparing multigraph
//  TGraph *gS, *gL, *gD, *gT;
//  gS = GraphWindow->ShowGraph(&Angle, &Spike,   1, "Angle", "", 0, 1, 1, 1, "", true);
//  gL = GraphWindow->ShowGraph(&Angle, &Lobe,    3, "Angle", "", 0, 1, 1, 1, "", true);
//  gD = GraphWindow->ShowGraph(&Angle, &Diffuse, 4, "Angle", "", 0, 1, 1, 1, "", true);
//  gT = GraphWindow->ShowGraph(&Angle, &Total,   2, "Angle", "", 0, 1, 1, 2, "", true);
//  TMultiGraph* multigrEE = new TMultiGraph();
//  multigrEE->Add(gS, "L"); //on delete, multigraph automatically deletes member graphs
//  multigrEE->Add(gL, "L");
//  multigrEE->Add(gD, "L");
//  multigrEE->Add(gT, "L");
//  multigrEE->SetTitle("Scatter vs Angle;Angle of incidence, degrees;Reflected fraction");
//  GraphWindow->Draw(multigrEE, "A");

  TGraph *gS, *gL, *gD, *gT;
  gT = GraphWindow->ShowGraph(&Angle, &Total,   2, "Angle", "", 0, 1, 1, 2, "", true);
  gT->SetMinimum(0);
  gS = GraphWindow->ShowGraph(&Angle, &Spike,   1, "Angle", "", 0, 1, 1, 1, "", true);
  gL = GraphWindow->ShowGraph(&Angle, &Lobe,    3, "Angle", "", 0, 1, 1, 1, "", true);
  gD = GraphWindow->ShowGraph(&Angle, &Diffuse, 4, "Angle", "", 0, 1, 1, 1, "", true);

  if (TMPmultiGraph) delete TMPmultiGraph; //to delete old only - we will show graphs separately
  TMPmultiGraph = new TMultiGraph;
  TMPmultiGraph->Add(gS, "L"); //on delete, multigraph automatically deletes member graphs
  TMPmultiGraph->Add(gL, "L");
  TMPmultiGraph->Add(gD, "L");
  TMPmultiGraph->Add(gT, "L");
  //TMPmultiGraph->SetTitle("Scatter vs Angle;Angle of incidence, degrees;Reflected fraction");
  GraphWindow->Draw(gT, "AL");
  GraphWindow->Draw(gS, "Lsame");
  GraphWindow->Draw(gL, "Lsame");
  GraphWindow->Draw(gD, "Lsame");

  TLegend *leg = new TLegend(0.1, 0.7, 0.3, 0.9);
  leg->SetFillColor(0);
  //leg->SetHeader("test legend");
  leg->AddEntry(gT, "Total", "lp");
  leg->AddEntry(gS, "Spike", "lp");
  leg->AddEntry(gL, "Lobe", "lp");
  leg->AddEntry(gD, "Diffuse", "lp");
  GraphWindow->Draw(leg, "same");
}

static QVector<double> vParam, vTot, vSpike, vLobe, vDiff, vDir;
static QString parName;
void MainWindow::on_pbST_ReflectionVsParameter_clicked()
{
  //input string processing
  QString inS = ui->leST_Parameter->text().simplified();
  QStringList inL = inS.split(",", QString::SkipEmptyParts);
  qDebug() << inL.size()<<inL;
  if (inL.size()!=4)
    {
      message("Input line format: Parameter, From, To, Step", this);
      return;
    }
  QList<QString> parameters;
  //============== list of parameter names ==============
  parameters << "n1" << "n2" << "alb" << "sigA" << "sigS" << "k2";
  parName = inL[0];
  if (!parameters.contains(parName))
    {
      QString str;
      for (int i=0; i<parameters.size(); i++) str += parameters[i]+"\n";
      message("Unknown parameter! Defined parameters are:\n"+str, this);
      return;
    }
  bool ok;
  double from, to, step;
  from = inL[1].toDouble(&ok);
  if (!ok)
    {
      message("Format error of the \"From\" value", this);
      return;
    }
  to = inL[2].toDouble(&ok);
  if (!ok)
    {
      message("Format error of the \"To\" value", this);
      return;
    }
  step = inL[3].toDouble(&ok);
  if (!ok)
    {
      message("Format error of the \"Step\" value", this);
      return;
    }
  if (from>to)
    {
      message("From is larger than To!", this);
      return;
    }
  qDebug() << from << to << step;
  int steps = (to-from)/step;
  qDebug() << "Steps:"<<steps;

  // Have to create a fake material collection with two materials
  // to provide refractive indices and iwave->wavelength conversion to the override class
  MaterialCollectionClass MatCol;
  //creating and filling override object and this fake material collection
  OpticalOverrideClass* ov = InitOverride(ui, MatCol, Detector);

  PhScatClaudioModel* ClaudMod = dynamic_cast<PhScatClaudioModel*>(ov);
  ScatterOnMetal* ScatOnMet = dynamic_cast<ScatterOnMetal*>(ov);

  TVector3 SurfNorm(ui->ledST_si->text().toDouble(),
                    ui->ledST_sj->text().toDouble(),
                    ui->ledST_sk->text().toDouble());
  SurfNorm = SurfNorm.Unit();
  SurfNorm = - SurfNorm; //In ANTS2 navigator provides normal in the opposite direction
  double N[3];
  N[0] = SurfNorm.X();
  N[1] = SurfNorm.Y();
  N[2] = SurfNorm.Z();
  TVector3 PhotDir(ui->ledST_i->text().toDouble(),
                   ui->ledST_j->text().toDouble(),
                   ui->ledST_k->text().toDouble());
  PhotDir = PhotDir.Unit();
  double K[3];

  int num = ui->sbST_number->value();
  vParam.clear();
  vSpike.clear();
  vLobe.clear();
  vTot.clear();
  vDiff.clear();
  vDir.clear();
  for (int iS = 0; iS<steps; iS++)
    { //cycle by parameter steps
      double parVal = from + iS*step;
      vParam.append(parVal);
      vSpike.append(0);
      vLobe.append(0);
      vTot.append(0);
      vDiff.append(0);
      vDir.append(0);
      if (parName == "n1")
         MatCol[0]->n = parVal;
      else if (parName == "n2")
        {
         MatCol[1]->n = parVal;
         if (ScatOnMet) ScatOnMet->RealN = parVal;
        }
      else if (parName == "alb")
        {
         if (ClaudMod) ClaudMod->albedo = parVal;
        }
      else if (parName == "sigA")
        {
         if (ClaudMod) ClaudMod->sigma_alpha = parVal;
        }
      else if (parName == "sigS")
        {
         if (ClaudMod) ClaudMod->sigma_h = parVal;
        }
      else if (parName == "k2")
        {
         if (ScatOnMet) ScatOnMet->ImaginaryN = parVal;
        }

      for (int iphot=0; iphot<num; iphot++)
        { //cycle by photons
          switch ( ui->cobST_PhotDirIso->currentIndex() )
            {
            case 0: //given direction
              K[0] = PhotDir.X();
              K[1] = PhotDir.Y();
              K[2] = PhotDir.Z();
              break;
            case 1: //uniform distr
              {
                double a=0, b=0, r2=1;
                while (r2 > 0.25)
                  {
                    a  = Detector->RandGen->Rndm() - 0.5;
                    b  = Detector->RandGen->Rndm() - 0.5;
                    r2 =  a*a + b*b;
                  }
                K[2] = ( -1.0 + 8.0 * r2 );
                double scale = 8.0 * TMath::Sqrt(0.25 - r2);
                K[0] = a*scale;
                K[1] = b*scale;
                //accept only in the same direction (2Pi) as the normal of the surface (normal is already inverted - it points to the second material)
                if (K[0]*N[0]+K[1]*N[1]+K[2]*N[2]<0)
                  {
                    K[0]=-K[0];
                    K[1]=-K[1];
                    K[2]=-K[2];
                  }
                break;
              }
            case 2: //diffuse (lambertian) distr
              {
                double sin2angle = Detector->RandGen->Rndm();
                double angle = asin(sqrt(sin2angle));
                double yOff = cos(angle), zOff = -sin(angle);
                N[0] = 0; N[1] = 0; N[2] = -1;   // convention of photon tracer - normal is med1->med2
                K[0] = 0; K[1] = yOff; K[2] = zOff;  // -z direction on xy plane (incidence angle from 90 to 0)
                break;
              }
            default:
              message("Unknown type of photon irradiation!", this);
              return;
            }

          ov->calculate(Detector->RandGen, K, N, -1);

          switch (ov->Status)
            {
            case OpticalOverrideClass::Absorption: continue;
            case OpticalOverrideClass::SpikeReflection: vSpike[iS]++; vDir[iS]++; break;
            case OpticalOverrideClass::LobeReflection: vLobe[iS]++; vDir[iS]++; break;
            case OpticalOverrideClass::LambertianReflection: vDiff[iS]++; break;
            default:
              qCritical()<<"Unknown process!";
              exit(666);
            }
          vTot[iS]++;
        } //end cycle by photons
      vSpike.last() /= num;
      vDir.last() /= num;
      vLobe.last() /= num;
      vDiff.last() /= num;
      vTot.last() /= num;
    } //end cycle by parameter steps

  on_cobST_ShowWhatRef_activated(ui->cobST_ShowWhatRef->currentIndex());
}

void MainWindow::on_pbST_VsParameterHelp_clicked()
{
  QString str =
      "Format: Parameter_name, StartValue, EndValue, Step\n"
      "Parameter names:\n"
      "sigA - SigmaAlpha\n"
      "sigS - SigmaSpike\n"
      "alb  - albedo\n"
      "n1   - Refractive index 1\n"
      "n2   - Refractive index 2\n"
      "k2   - Imaginary part of refractive index 2\n";
  message(str, this);
}

void MainWindow::on_cobST_ShowWhatRef_activated(int index)
{
  switch (index)
    {
    case 0:
      GraphWindow->ShowGraph(&vParam, &vTot, 4, parName.toLocal8Bit().data(), "Sum reflectivity", 20, 1, 1, 1);
      break;
    case 1:
      GraphWindow->ShowGraph(&vParam, &vDiff, 4, parName.toLocal8Bit().data(), "Diffuse reflectivity", 20, 1, 1, 1);
      break;
    case 2:
      GraphWindow->ShowGraph(&vParam, &vDir, 4, parName.toLocal8Bit().data(), "Directed reflectivity", 20, 1, 1, 1);
      break;
    case 3:
      GraphWindow->ShowGraph(&vParam, &vSpike, 4, parName.toLocal8Bit().data(), "Spike reflectivity", 20, 1, 1, 1);
      break;
    case 4:
      GraphWindow->ShowGraph(&vParam, &vLobe, 4, parName.toLocal8Bit().data(), "Lobe reflectivity", 20, 1, 1, 1);
      break;
    default:;
    }
}

void MainWindow::on_pbST_showTracks_clicked()
{
  if (tracks.isEmpty()) return;
  int selector = ui->cobST_trackType->currentIndex() - 1;
  if (selector == 3) return; //do not show any tracks

  Detector->GeoManager->ClearTracks();
  GeometryWindow->ClearRootCanvas();
  //showing surface
  Int_t track_index = Detector->GeoManager->AddTrack(1,22);
  TVirtualGeoTrack* track = Detector->GeoManager->GetTrack(track_index);  
  double d = 0.5;  
  double f = 0.5;

  //surface normal
  track->AddPoint(d, d, d, 0);
  track->AddPoint(d+NormViz.X(), d+NormViz.Y(), d+NormViz.Z(), 0);
  track->SetLineWidth(3);
  track->SetLineColor(1);
  //surf
  track_index = Detector->GeoManager->AddTrack(1,22);
  track = Detector->GeoManager->GetTrack(track_index);
  TVector3 perp = NormViz.Orthogonal();
  perp.Rotate(0.25*3.1415926535, NormViz);
  for (int i=0; i<5; i++)
    {
      track->AddPoint(d+f*perp.X(), d+f*perp.Y(), d+f*perp.Z(), 0);
      perp.Rotate(0.5*3.1415926535, NormViz);
    }
  track->SetLineWidth(2);
  track->SetLineColor(1);

  //photon tracks  
  int numTracks = 0;
  for(int i = 1; i<tracks.count() && numTracks<10000; i++)
    {
      const TrackHolderClass* th = &tracks.at(i);
      //filter
      if (selector>-1)  //-1 - show all
        if (selector != th->UserIndex) continue;

      track_index = Detector->GeoManager->AddTrack(1,22);
      track = Detector->GeoManager->GetTrack(track_index);
      track->SetLineColor(th->Color);
      track->SetLineWidth(1);
      for (int iNode=0; iNode<th->Nodes.size(); iNode++)
        track->AddPoint(th->Nodes[iNode].R[0], th->Nodes[iNode].R[1], th->Nodes[iNode].R[2], th->Nodes[iNode].Time);
    }
  //intitial photon track
  track_index = Detector->GeoManager->AddTrack(1,22);
  track = Detector->GeoManager->GetTrack(track_index);
  TrackHolderClass* th = &tracks.first();
  track->SetLineColor(kRed);
  track->SetLineWidth(2);
  for (int iNode=0; iNode<th->Nodes.size(); iNode++)
    track->AddPoint(th->Nodes[iNode].R[0], th->Nodes[iNode].R[1], th->Nodes[iNode].R[2], th->Nodes[iNode].Time);

  ShowTracks();
}

void MainWindow::on_pobTest_clicked()
{
  Rwindow->DotActualPositions();
  ShowGeometry(false, false);
  return;


  //qDebug() << "\n N A V I G A T O R S: "<< Detector->GeoManager->GetListOfNavigators()->GetEntries();

  //cusolverDnHandle_t my_handle;
  //cusolverDnCreate(&my_handle);

  if (!EventsDataHub->isTimed())
    {
      message("There are no timed data!", this);
      return;
    }
  ///QVector< QVector < QVector <float> > > TimedEvents; //event timebin pm

  // we are looking for correlation between different PMs! //not compatible with loaded energy files
  int m = EventsDataHub->TimedEvents[0].size();       //number of data points = time bins
  int n = EventsDataHub->TimedEvents[0][0].size();    //number of variables = number of PMs
  qDebug() << "Number of variables (PMs):"<<n<<"  Number of data points (time bins):"<<m;

  //TPrincipal* principal = new TPrincipal(n,"ND");
  TPrincipal* principal = new TPrincipal(n,"D");

  Double_t* data = new Double_t[n];

  TMatrixD Input(m, n); ///Andr    //(Int_t nrows, Int_t ncols); |m  -n
  for (int i = 0; i < m; i++)  //cycle on points (i.e. over time bins)
    {
      QString test = "Time bin# " + QString::number(i)+"> ";
      for (int j = 0; j < n; j++) //cycle on variables (i.e. PMs)
        {
          data[j] = EventsDataHub->TimedEvents[0][i][j];
          Input[i][j] = data[j]; ///Andr
          test += QString::number(data[j]) + " ";
        }
      qDebug() << test;

      principal->AddRow(data);
    }
  delete [] data;

    // Do the actual analysis
    principal->MakePrincipals();

    // Print out the result on
    principal->Print();

    // Test the PCA
    principal->Test();

/*
    // Make some histograms of the orginal, principal, residue, etc data
    principal->MakeHistograms("name","XPE");

    // Start a browser, so that we may browse the histograms generated above
    TBrowser* b = new TBrowser("principalBrowser", principal);
*/

    const TMatrixD* EigenVectors = principal->GetEigenVectors();
    //qDebug() << "Eigen Vectors"<< (*EigenVectors)[0][0] << (*EigenVectors)[0][1] << (*EigenVectors)[0][2];
    //qDebug() << "Eigen Vectors"<< (*EigenVectors)[1][0] << (*EigenVectors)[1][1] << (*EigenVectors)[1][2];
    //qDebug() << "Eigen Vectors"<< (*EigenVectors)[2][0] << (*EigenVectors)[2][1] << (*EigenVectors)[2][2];

    const TVectorD * MeanValues = principal->GetMeanValues();
    //qDebug() << "Mean values:"<< (*MeanValues)[0]<<(*MeanValues)[1]<<(*MeanValues)[2];
    for (int i=0; i<n; i++)
      {
        for (int j=0; j<m; j++)
          Input[j][i] -= (*MeanValues)[i];
      }

    Input *= *EigenVectors;

    for (int iprin=0; iprin<n; iprin++)
      {
        if (iprin>5) break; //too much on the same graph
        QVector<double> x, y;
        for (int i=0; i<m; i++)
          {
            x.append(i);
            y.append(Input[i][iprin]);
          }
        if (iprin==0) GraphWindow->ShowGraph(&x, &y, 1, "Time", "", 2);
        else GraphWindow->ShowGraph(&x, &y, 1+iprin, "", "", 2, 1, 1, 2, "same");
      }

}

void MainWindow::on_leRootIn_editingFinished()
{
  //stopRootUpdate();
  //RootApp->StartIdleing();
  QString fileName =GlobSet->TmpDir + "/root.log";
  if (!QFileInfo(fileName).exists())
    {
      QFile file(fileName);
      if (!file.open(QIODevice::ReadWrite))
        {
           message("Cannot create exchange file: "+fileName, this);
           return;
        }
      file.close();
    }
  GraphWindow->SetAsActiveRootWindow();

  QString inputStr = ui->leRootIn->text();
  //inputStr += "; >& " + fileName;
  TString input = inputStr.toLocal8Bit().data();
  qDebug() << input;
  if (inputStr.contains("Draw(")) GraphWindow->show();
  RootApp->ProcessLine(input);
  //qDebug() << "returned!";

  ui->leRootIn->selectAll();

  ui->pteRootOut->clear();
  QFile file(fileName);
  if (!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Cannot read root echange file: "+fileName , this);
      return;
    }
  QTextStream ts(&file);
  QString output = ts.readAll();
  ui->pteRootOut->appendPlainText(output);
  file.close();

  GraphWindow->UpdateRootCanvas();

  //startRootUpdate();
  //RootApp->StopIdleing();
}

void MainWindow::on_pbCheckDerivatives_clicked()
{
  if (!SensLRF->isAllLRFsDefined())
    {
      message("Not all LRFs are defined!", this);
      return;
    }

  int ipm = ui->sbPMnumForDerivatives->value();
  if (ipm>PMs->count()-1)
    {
      message("Wrong PM number!", this);
      ui->sbPMnumForDerivatives->setValue(0);
      return;
    }

  double x = ui->ledDertestX->text().toDouble();
  double y = ui->ledDertestY->text().toDouble();
  double z = ui->ledDertestZ->text().toDouble();
  double offset = ui->ledDerOffset->text().toDouble();

  QString LRFtype = (*SensLRF)[ipm]->type();
  qDebug()<<"PM#:"<<ipm<<" LRF type:"<<LRFtype;
  qDebug()<<"x,y"<<x<<y;


        double lrfXm = SensLRF->getLRF(ipm,x-offset,y,z);
        double lrfXp = SensLRF->getLRF(ipm,x+offset,y,z);
        double lrfYm = SensLRF->getLRF(ipm,x,y-offset,z);
        double lrfYp = SensLRF->getLRF(ipm,x,y+offset,z);
        qDebug()<<"LRF xy cross (x-, x+, -y, y+)"<<lrfXm<<lrfXp<<lrfYm<<lrfYp;

        if (lrfXm == 0 || lrfXp == 0 || lrfYm == 0 || lrfYp == 0)
          {
            qDebug()<<"-->undefined LRF found!";
            return;
          }

        double twoDelta = 2.0*offset;
        double numDerX = (lrfXp - lrfXm)/twoDelta;
        double numDerY = (lrfYp - lrfYm)/twoDelta;
        qDebug()<<"numeric derivatives (x,y):"<<numDerX<<numDerY;

        double anDerX = SensLRF->getLRFDrvX(ipm,x, y, z);
        double anDerY = SensLRF->getLRFDrvY(ipm,x, y, z);
        qDebug()<<"module gives derivatives (x,y):"<<anDerX<<anDerY;
}


void MainWindow::TestCompton()
{
  int Maxi = ui->sbTmp->value();
  GammaStructure G0, G1;

  G0.energy = ui->dsbTmp->value();
  TVector3 gdir ( 0, 1./1.4142135623730950488016887242097, 1./1.4142135623730950488016887242097 );
  G0.direction = gdir;

  if (hist1) delete hist1;
  if (hist2) delete hist2;
  hist1 = new TH1D("ComptonTestha","Angle of scattered gamma",181,0,180);
  hist1->GetXaxis()->SetTitle("Angle, degrees");
  hist2 = new TH1D("ComptonTesthf","Energy of scattered gamma",101,0,G0.energy);
  hist2->GetXaxis()->SetTitle("Energy, keV");

  for (int i=0;i<Maxi;i++) {
      G1 = Compton(&G0, Detector->RandGen);

      double angle = G1.direction.Angle(G0.direction);
   //   qDebug() << "angle" << angle;

      //hist1->Fill(angle*180./3.14159, 1./sin(angle));
      hist1->Fill(angle*180./3.14159);

      double energy = G1.energy;
  //    qDebug()<<"energy:" << energy;
      hist2->Fill(energy);
  }
}

void MainWindow::on_pbShowCheckUpWindow_clicked()
{
  if (!CheckUpWindow) return;
  CheckUpWindow->show();
}

void MainWindow::on_pbShowEnergyDeposition_clicked()
{
  if (EventsDataHub->EventHistory.isEmpty())
    {
      message("No data available!", this);
      return; //nothing to show
    }
  int index = ui->sbParticleIndexForShowDepEnergy->value();
  int i=-1;
  int Number;
  do
    {
      i++;
      if (i>EnergyVector.size()-1)
        {
          message("Deposition information not found for this particle", this);
          return;
        }
      Number = EnergyVector.at(i)->index;
    }
  while(Number != index);
  //on exit: i - first record with user-defined particle#

  QVector<double> x,y;
  x.resize(0);
  y.resize(0);

  double Length = 0;

  //info on this particle starts at i
  for (i; i<EnergyVector.size();i++)
    {
      if (EnergyVector[i]->index != index) break; //another particle starts here

      double DeltaLength = EnergyVector[i]->cellLength;
      Length += DeltaLength;
      x.append(Length);
      if (DeltaLength == 0) DeltaLength = 1; //0 protection for point deposition events
      y.append(EnergyVector[i]->dE/DeltaLength);
    }

  if (x.size() == 0 )
    {
      message("This particle did not deposit any energy", this);
      GraphWindow->hide();
      return;
   }

  GraphWindow->ShowGraph(&x, &y, kBlue, "Distance, mm", "Deposited energy linear density, keV/mm");
}

void MainWindow::on_pbShowSelected_clicked()
{
  if (EventsDataHub->EventHistory.isEmpty())
    {
      message("No data available!", this);
      return;
    }

  int particleId = ui->cobShowParticle->currentIndex();
  int materialId = ui->cobShowMaterial->currentIndex();
  bool fSelectiveParticleType = ui->cobShowParticleType->currentIndex() == 0 ? false : true;
  int particleType = ui->cobShowParticleType->currentIndex() - 1;  // 0-primary, 1-secondary
  ui->pteOut->clear();

  int counter=0;
  switch (ui->cobWhatToShow->currentIndex())
  {
  case (0):
    {
      //Statistics: energy deposition
      if (hist1) delete hist1;
      hist1 = new TH1D("DepEnergyHist","Deposited energy",ui->sbShowBins->value(),0,0);
      hist1->GetXaxis()->SetTitle("Deposited energy, keV");

      for (int i=0; i<EventsDataHub->EventHistory.size(); i++)
      {          
          if (ui->cbCheckerParticle->isChecked() && particleId != EventsDataHub->EventHistory[i]->ParticleId) continue;  //wrong particle

          if (Detector->ParticleCollection.at( EventsDataHub->EventHistory[i]->ParticleId )->type == _neutron_ ) continue; //to prevent overflow, we know neutrons cannot deposit energy anyway

          if (fSelectiveParticleType) if (particleType != EventsDataHub->EventHistory[i]->flagSecondary) continue; //wrong type
          for (int m=0; m < EventsDataHub->EventHistory[i]->Deposition.size(); m++)
          {
              if (ui->cbCheckerMaterial->isChecked() && materialId != EventsDataHub->EventHistory[i]->Deposition[m].MaterialId) continue; //wrong material
              //all filters OK
              if (EventsDataHub->EventHistory[i]->Deposition[m].DepositedEnergy>0) hist1->Fill(EventsDataHub->EventHistory[i]->Deposition[m].DepositedEnergy);
              counter++;
          }
      }
      break; //draw below
    }

  case (1):
    {
      //Statistics: travelled distance
      if (hist1) delete hist1;
      hist1 = new TH1D("TravDistHist","Travelled distance",ui->sbShowBins->value(),0,0);
      hist1->GetXaxis()->SetTitle("Travelled distance, mm");

      for (int i=0; i < EventsDataHub->EventHistory.size(); i++)
      {
          if (ui->cbCheckerParticle->isChecked() && particleId != EventsDataHub->EventHistory[i]->ParticleId) continue;  //wrong particle
          if (fSelectiveParticleType) if (particleType != EventsDataHub->EventHistory[i]->flagSecondary) continue; //wrong type
          for (int m=0; m < EventsDataHub->EventHistory[i]->Deposition.size(); m++)
          {
              if (ui->cbCheckerMaterial->isChecked() && materialId != EventsDataHub->EventHistory[i]->Deposition[m].MaterialId) continue; //wrong material
              //all filters OK
              hist1->Fill(EventsDataHub->EventHistory[i]->Deposition[m].Distance);
              counter++;
          }
      }
      break; //draw below
    } 
  case (2):
      {
        //Statistics: angle vs Z axis
        if (hist1) delete hist1;
        hist1 = new TH1D("AnglZHist","Angle distribution",ui->sbShowBins->value(),0,0);
        hist1->GetXaxis()->SetTitle("Angle with Z, degrees");

        //qDebug() << EnergyVector.size() << EventsDataHub->EventHistory.size();
        for (int i=0; i < EventsDataHub->EventHistory.size(); i++)
        {
            if (ui->cbCheckerParticle->isChecked() && particleId != EventsDataHub->EventHistory[i]->ParticleId) continue;  //wrong particle
            if (fSelectiveParticleType) if (particleType != EventsDataHub->EventHistory[i]->flagSecondary) continue; //wrong type
            for (int m=0; m < EventsDataHub->EventHistory[i]->Deposition.size(); m++)
            {
                if (ui->cbCheckerMaterial->isChecked() && materialId != EventsDataHub->EventHistory[i]->Deposition[m].MaterialId) continue; //wrong material
                //all filters OK
                float& dx = EventsDataHub->EventHistory[i]->dx;
                float& dy = EventsDataHub->EventHistory[i]->dy;
                float& dz = EventsDataHub->EventHistory[i]->dz;
                float trans2 = dx*dx + dy*dy;
                float angle =  ( fabs(dz) < 1e-10 ? 0 : atan( sqrt(trans2)/dz )*180.0/3.1415926 );

                hist1->Fill(angle);
                counter++;
            }
        }
        break; //draw below
      }
  case (3):
    {
      //interaction probability in this material for this particle
      int total = 0;
      int interactions = 0;

      for (int i=0; i<EventsDataHub->EventHistory.size(); i++)
      {
          if (ui->cbCheckerParticle->isChecked() && particleId != EventsDataHub->EventHistory[i]->ParticleId) continue;  //wrong particle
          if (fSelectiveParticleType) if (particleType != EventsDataHub->EventHistory[i]->flagSecondary) continue; //wrong type

          bool flagFound = false;
          bool flagInteraction = false;
          for (int m=0; m < EventsDataHub->EventHistory[i]->Deposition.size(); m++)
          {
              //could be several layers with the same material, so make a flag
              if (ui->cbCheckerMaterial->isChecked() && materialId != EventsDataHub->EventHistory[i]->Deposition[m].MaterialId) continue; //wrong material
              //all filters OK
              flagFound = true;
              if (EventsDataHub->EventHistory[i]->Deposition[m].DepositedEnergy > 0) flagInteraction = true; //takes care of all but neutron capture - they do not deposit energy directly
          }
          if (flagFound) total++; //this particle was in this material
          if (flagInteraction) interactions++; //this particle had intreaction inside this material

          if (Detector->ParticleCollection.at( EventsDataHub->EventHistory[i]->ParticleId )->type == _neutron_ )
          {
              if (EventsDataHub->EventHistory[i]->Termination != 5) continue;
              //last medium?
              if (EventsDataHub->EventHistory[i]->Deposition[EventsDataHub->EventHistory[i]->Deposition.size()-1].MaterialId != materialId) continue;
              interactions++;
          }
      }

      double InterProbability = 0;
      if (total != 0 ) InterProbability = 1.0*interactions / total;
      QString str, str1;
      str = "Total: ";
      str1.setNum(total);
      str += str1 + " Interacted: ";
      str1.setNum(interactions);
      str += str1;      
      ui->pteOut->appendPlainText(str);
      str.setNum(InterProbability*100.0, 'g', 4);
      ui->pteOut->appendPlainText("Interaction probability: "+str+ " %");
      return;
      //break;
    }
    case (4):
      {
        //deposited energy in this material for this particle more than a certain value
        int total = 0;
        int interactions = 0;
        double threshold = ui->ledShowThreshold->text().toDouble();

        for (int i=0; i < EventsDataHub->EventHistory.size(); i++)
          {
            if (ui->cbCheckerParticle->isChecked() && particleId != EventsDataHub->EventHistory[i]->ParticleId) continue;  //wrong particle
            if (fSelectiveParticleType) if (particleType != EventsDataHub->EventHistory[i]->flagSecondary) continue; //wrong type

            bool flagFound = false;
            bool flagInteraction = false;
            for (int m=0; m < EventsDataHub->EventHistory[i]->Deposition.size(); m++)
              {
                //could be several layers with the same material, so make a flag
                if (ui->cbCheckerMaterial->isChecked() && materialId != EventsDataHub->EventHistory[i]->Deposition[m].MaterialId) continue; //wrong material
                //all filters OK
                flagFound = true;
                if (EventsDataHub->EventHistory[i]->Deposition[m].DepositedEnergy > threshold) flagInteraction = true; //neutrons do not deposit directly anyway
              }
            if (flagFound) total++; //this particle was in this material
            if (flagInteraction) interactions++; //this particle had interaction inside this material
          }

        double Fraction = 0;
        if (total != 0 ) Fraction = 1.0*interactions / total;
        QString str, str1;
        str = "Total: ";
        str1.setNum(total);
        str += str1 + " Deposited above threshold (" + QString::number(threshold) + "): ";
        str1.setNum(interactions);
        str += str1;        
        ui->pteOut->appendPlainText(str);
        str.setNum(Fraction*100.0, 'g', 4);
        ui->pteOut->appendPlainText("Fraction: "+str+ " %");
        return;
      }
  }


  if (counter == 0)
  {
      message("No data was found for the given selection", this);
      return;
  }
  //drawing histogram:
  GraphWindow->Draw(hist1);
}

void MainWindow::on_cobWhatToShow_currentIndexChanged(int index)
{
  switch (index)
    {
    case 0:
    case 1:      
    case 2:
      ui->swShowPrPData->setCurrentIndex(0);
      break;
    case 3:
      ui->swShowPrPData->setCurrentIndex(1);
    break;
    case 4:
      ui->swShowPrPData->setCurrentIndex(2);
    }
}

void MainWindow::on_pbShowDetailedLog_clicked()
{
    Owindow->showNormal();
    Owindow->raise();
    Owindow->activateWindow();
    Owindow->LogEventHistory();
    Owindow->SetTab(0);
}

void MainWindow::on_pbCheckerScript_clicked()
{
  extractGeometryOfScriptWindow();
  if (GenScriptWindow) delete GenScriptWindow;
  GenScriptWindow = new GenericScriptWindowClass();
  recallGeometryOfScriptWindow();

  InterfaceToCheckerScript* interfaceObject = new InterfaceToCheckerScript(&EnergyVector, &EventsDataHub->EventHistory); //deleted by the GenScriptWindow
  GenScriptWindow->SetInterfaceObject(interfaceObject);

  InterfaceToGraphs* graph = new InterfaceToGraphs(this);
  GenScriptWindow->SetInterfaceObject(graph, "graph");

  InterfaceToHistD* hist = new InterfaceToHistD(this);
  GenScriptWindow->SetInterfaceObject(hist, "hist");

  GenScriptWindow->SetExample(
        "var GasMatId = 0\n"
        "var thresholdE = 100\n"
        " \n"
        "var good = 0\n"
        "var total = 0\n"
        "hist.NewHist(\"h\", 100, 0, 0 )\n"
        "hist.SetTitles(\"h\", \"R in gas\", \"\")\n"
        " \n"
        " var indexEV = 0\n"
        " do\n"
        " {\n"
        "    indexEV = pr_init(indexEV) \n"
        "    total++ \n"
        "   \n"
        "    for (var imat=0; imat<pr_matCount(); imat++)\n"
        "    {\n"
        "       if ( pr_mr_mId( imat ) != GasMatId) continue  //wrong material\n"
        " \n"
        "       var edep = 0 \n"
        "       for (var i=pr_firstEV(imat); i<=pr_lastEV(imat); i++)   edep +=  ev_dE(i)  \n"
        "       if (edep > thresholdE)  good++  //too low energy deposition\n"
        " \n"
        "       var dx = pr_dX()\n"
        "       var dy = pr_dY()\n"
        "       var dz = pr_dZ()\n"
        "       var ij = math.sqrt( dx*dx + dy*dy  )\n"
        "       var z = ev_Z( pr_lastEV(imat)  ) -  pr_mr_z0(imat) \n"
        "       var r =   ij * z / dz      \n"
        "       hist.Fill(\"h\", r, 1)\n"
        "    }\n"
        "}\n"
        "while (indexEV < ev_count())\n"
        "\n"
        "if (good>0) hist.Draw(\"h\", \"\")\n"
        "\"detected events: \"+good+\"  total pairs:\"+ total/2\n"
        );
  GenScriptWindow->SetShowEvaluationResult(true);
  GenScriptWindow->SetTitle("Energy deposition analysis");
  GenScriptWindow->SetScript(&CheckerScript);
  GenScriptWindow->SetStarterDir(GlobSet->LibScripts);

  connect(GenScriptWindow, SIGNAL(RequestDraw(TObject*,QString,bool)), GraphWindow, SLOT(DrawStrOpt(TObject*,QString,bool)));

  GenScriptWindow->show();
}
