//ANTS2
#include "reconstructionwindow.h"
#include "ui_reconstructionwindow.h"
#include "mainwindow.h"
#include "guiutils.h"

#ifdef ANTS_FLANN
#include "reconstructionmanagerclass.h"
#include "nnmoduleclass.h"
#endif

//Qt
#include <QtGui>

//Root
#include "TLine.h"
#include "TEllipse.h"
#include "TPolyLine.h"

ReconstructionWindow::ReconstructionWindow(QWidget *parent, MainWindow *mw, EventsDataClass* eventsDataHub) :
  QMainWindow(parent),
  ui(new Ui::ReconstructionWindow)
{
  MW = mw;
  EventsDataHub = eventsDataHub;
  TableLocked = true; //will be unlocked after table is filled

  ui->setupUi(this);
  this->setFixedSize(this->size());

  Qt::WindowFlags windowFlags = (Qt::Window | Qt::CustomizeWindowHint);
  windowFlags |= Qt::WindowCloseButtonHint;
  this->setWindowFlags( windowFlags );

  ForbidUpdate = false;  
  tmp1Dhist = 0;
  tmpHist = 0;
  centHist = 0;
  multigrEE = 0;
  ShiftZoldValue = 0;
  LastDistributionShown = "";
  modelSpF_TV = 0;
  StopRecon = false;
  polygon.clear();
  histCorr = 0;
  CorrelationFilters.resize(0);
  TMPignore = false;
  vo = 0;
  dialog = 0; 

  CorrCutLine = new TLine(0,0,0,0); //invisible
  CorrCutEllipse = new TEllipse(0, 0, 0, 0, 0, 360, 0); //invisible
  CorrCutPolygon = new TPolyLine(0);

  funcParams[0] = 0;
  funcParams[1] = 0;
  funcParams[2] = 1.0;
  funcParams[3] = 1.0;

  RedIcon = createColorCircleIcon(ui->twOptions->iconSize(), Qt::red);
  ui->labDataIsMissing->setPixmap(RedIcon.pixmap(16,16));
  ui->fNoData->setVisible(true);
  YellowIcon = createColorCircleIcon(ui->twData->iconSize(), Qt::yellow);

  QString style = "QLabel { background-color: #E0E0E0; }";  //setting background color for the "Configuration" tab
  ui->laBackground->setStyleSheet(style);
  ui->twData->setCurrentIndex(0);
  ui->twOptions->setCurrentIndex(0);
  ui->bsAnalyzeScan->setCurrentIndex(0);
  ui->cbRecType->setCurrentIndex(-1);
  ui->cbRecType->setCurrentIndex(0);

  ui->pbRefreshDataIndication->setVisible(false);
  ui->pbRefreshDataIndication->setVisible(false);
  ui->fFilterSumSignal->setEnabled(false);
  ui->fFilterIndividualSignal->setEnabled(false);
  ui->fEnergyFilter->setEnabled(false);
  ui->fChi2Filter->setEnabled(false);
  //ui->fAverageZ->setEnabled(false);
  ui->fCustomSpatFilter->setEnabled(ui->cbSpFcustom->isChecked());
  ui->fSpFz->setEnabled(!ui->cbSpFallZ->isChecked());
  ui->pbUpdateFilters->setVisible(false); 
  ui->pbUpdateStatusAllEvents->setVisible(false);
  ui->pbShowStatistics->setVisible(false);
  ui->pbSpF_UpdateTable->setVisible(false);  
  ui->pbUpdatePMgroupIndication->setVisible(false);
  ui->pbUpdateGainsIndication->setVisible(false);
  ui->leoTimeBins->setEnabled(false);
  ui->labTimeBins->setEnabled(false);
  ui->fCorrCut->setEnabled(false);
  ui->fCorrAddRemoe->setEnabled(false);
  ui->fCorrSize->setEnabled(false);
  ui->cbCorrShowCut->setEnabled(false);
  ui->pbCorrUpdateTMP->setVisible(false);
  ui->dLoadedEnergy->setEnabled(false);
  ui->fLoadedEnergyFilter->setEnabled(false);
  ui->fDynPassive->setVisible(false);
  ui->cbForceCoGgiveZof->setEnabled(false);
  ui->fCustomBins->setEnabled(false);
  ui->fCustomRanges->setEnabled(false);
  ui->fShowXYPmnumber->setVisible(false);
  ui->pbKNNupdate->setVisible(false);
  ui->fInRecFilter->setEnabled(false);

#ifdef ANTS_FLANN
  ui->fNNfilter->setVisible(true);
  ui->labFLANNenabled->setVisible(false);

  connect(&MW->ReconstructionManager->KNNmodule->Reconstructor, SIGNAL(readyXchanged(bool, int)), this, SLOT(onKNNreadyXchanged(bool, int)));
  connect(&MW->ReconstructionManager->KNNmodule->Reconstructor, SIGNAL(readyYchanged(bool, int)), this, SLOT(onKNNreadyYchanged(bool, int)));
#else
  ui->fNNfilter->setVisible(false);
  ui->labFLANNenabled->setVisible(true);
#endif

  //setting validator for lineEdit boxes
    //double
  QDoubleValidator* dv = new QDoubleValidator(this);
  dv->setNotation(QDoubleValidator::ScientificNotation);
  QList<QLineEdit*> list = this->findChildren<QLineEdit *>();
  foreach(QLineEdit *w, list) if (w->objectName().startsWith("led")) w->setValidator(dv);
    //int
  QIntValidator* iv  = new QIntValidator(this);
  iv->setBottom(0);  
  foreach(QLineEdit *w, list) if (w->objectName().startsWith("lei")) w->setValidator(iv);

  ui->cbDynamicPassives->setChecked(false); //has to be false on start!!!

  ui->twOptions->tabBar()->setTabIcon(1, RedIcon);
  //ui->twData->tabBar()->setTabIcon(0, RedIcon);

  ui->tabwidCorrPolygon->resizeColumnsToContents();
  ui->tabwidCorrPolygon->resizeRowsToContents();

  //Initialization of CorrelationFilters and tmpCorrFilter is done in ReconstructionWindow::ConfigureReconstructor() - need
  //  Reconstructor module to be created (it is not onthe moment of ReconstructionWindow creation)

  ui->tabWidget->setCurrentIndex(1); //reconstruction options tab
  ui->bsAnalyzeScan->setCurrentIndex(0); //reconstruct all tab
  ui->cobCUDAoffsetOption->setCurrentIndex(1);
  ui->cobCUDAoffsetOption->setCurrentIndex(0);

  //handling of NN module reconstruction options.
  connect(ui->cbReconstructEnergy,SIGNAL(clicked(bool)),this,SIGNAL(cbReconstructEnergyChanged(bool)));
  connect(ui->cb3Dreconstruction,SIGNAL(clicked(bool)),this,SIGNAL(cb3DreconstructionChanged(bool)));

  //ui->cobReconstructionAlgorithm->setCurrentIndex(0); //update indication
  ReconstructionWindow::on_cobReconstructionAlgorithm_currentIndexChanged(ui->cobReconstructionAlgorithm->currentIndex());
   // qDebug()<<"  Reconstruction Window created";

  on_cbLimitNumberEvents_toggled(ui->cbLimitNumberEvents->isChecked());
  on_cobHowToAverageZ_currentIndexChanged(ui->cobHowToAverageZ->currentIndex());
}

