#include "reconstructionwindow.h"
#include "ui_reconstructionwindow.h"
#include "pms.h"
#include "sensorlrfs.h"
#include "neuralnetworkswindow.h"
#include "functions.h"
#include "mainwindow.h"
#include "graphwindowclass.h"
#include "jsonparser.h"
#include "lrfwindow.h"
#include "genericscriptwindowclass.h"
#include "eventsdataclass.h"
#include "functions.h"
#include "windownavigatorclass.h"
#include "globalsettingsclass.h"

#include <QtGui>
#include <QFileDialog>

#include "TTree.h"
#include "TFile.h"
#include "TH2D.h"

void ReconstructionWindow::writeMiscGUIsettingsToJson(QJsonObject &json)
{
  QJsonObject js;

  //adaptive script
  js["ScriptAdaptive"] = CurrentScriptText;

  //Plot XY settings
  QJsonObject pj;
  pj["PlotXYfromX"] = ui->ledXfrom->text().toDouble();
  pj["PlotXYfromY"] = ui->ledYfrom->text().toDouble();
  pj["PlotXYtoX"] = ui->ledXto->text().toDouble();
  pj["PlotXYtoY"] = ui->ledYto->text().toDouble();
  pj["PlotXYbinsX"] = ui->sbXbins->value();
  pj["PlotXYbinsY"] = ui->sbYbins->value();
  pj["PlotXYoptionZ"] = ui->cobHowToAverageZ->currentIndex();
  pj["PlotXYfromZ"] = ui->ledZfrom->text().toDouble();
  pj["PlotXYtoZ"] = ui->ledZto->text().toDouble();
  js["PlotXYsettings"] = pj;

  json["GuiMisc"] = js;
}

void ReconstructionWindow::readMiscGUIsettingsFromJson(QJsonObject &json)
{
   QJsonObject js1 = json["GuiMisc"].toObject();

   //adaptive script
   parseJson(js1, "ScriptAdaptive", MW->Rwindow->CurrentScriptText);

   //PlotXY settings
   QJsonObject js = js1["PlotXYsettings"].toObject();
   JsonToLineEdit(js, "PlotXYfromX", ui->ledXfrom);
   JsonToLineEdit(js, "PlotXYfromY", ui->ledYfrom);
   JsonToLineEdit(js, "PlotXYtoX", ui->ledXto);
   JsonToLineEdit(js, "PlotXYtoY", ui->ledYto);
   JsonToLineEdit(js, "PlotXYfromZ", ui->ledZfrom);
   JsonToLineEdit(js, "PlotXYtoZ", ui->ledZto);
   JsonToSpinBox(js, "PlotXYbinsX", ui->sbXbins);
   JsonToSpinBox(js, "PlotXYbinsY", ui->sbYbins);
   JsonToComboBox(js, "PlotXYoptionZ", ui->cobHowToAverageZ);
}

void ReconstructionWindow::on_pbSaveScanTree_clicked()
{
  if (EventsDataHub->isScanEmpty())
    {
      message("Scan data are not ready!", this);
      return;
    }
  if ( !EventsDataHub->isReconstructionDataReady() )
    {
      message("Need to run reconstruction first!", this);
      ui->bsAnalyzeScan->setCurrentIndex(0);
      return;
    }

  QString fileName = QFileDialog::getSaveFileName(this, "Save sigma/distortion information", MW->GlobSet->LastOpenDir, "Root files (*.root)");
  if (fileName.isEmpty()) return;
  if (QFileInfo(fileName).suffix().isEmpty()) fileName += ".root";
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QByteArray ba = fileName.toLocal8Bit();
  const char *c_str = ba.data();
  TFile f(c_str,"recreate");
  ReconstructionWindow::BuildResolutionTree();
  if (!EventsDataHub->ResolutionTree)
    {
      message("Failed to create Tree!", this);
      f.Close();
      return;
    }
  EventsDataHub->ResolutionTree->Write();
  f.Close();
  EventsDataHub->ResolutionTree= 0;  //tree does not exist after save
}

void ReconstructionWindow::on_pbSaveReconstructionAsText_clicked()
{
  //in this mode only time-integrated data are considered!
  if (EventsDataHub->isEmpty()) return;
  int size = EventsDataHub->ReconstructionData.size();
  if (size == 0)
    {
      message("There are no reconstruction data to save!", this);
      return;
    }

  QString fileName = QFileDialog::getSaveFileName(this, "Save reconstruction data", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*.*)");
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  if(QFileInfo(fileName).suffix().isEmpty()) fileName += ".dat";

  bool ok = EventsDataHub->saveReconstructionAsText(fileName);
  if (!ok) message("Error writing the file!", this);
}

void ReconstructionWindow::on_pbSaveReconstructionAsRootTree_clicked()
{
  //in this mode only time-integrated data are considered!
  if (EventsDataHub->isEmpty())
    {
      message("There are no events!", this);
      return;
    }
  if (EventsDataHub->ReconstructionData.isEmpty())
    {
      message("There are no reconstruction data to save!", this);
      return;
    }
  QString fileName = QFileDialog::getSaveFileName(this, "Save reconsruction data as Root Tree", MW->GlobSet->LastOpenDir, "Root files (*.root)");
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  if ( QFileInfo(fileName).suffix().isEmpty()) fileName += ".root";

  MW->WindowNavigator->BusyOn();
  qApp->processEvents();
  bool ok = EventsDataHub->saveReconstructionAsTree(fileName, MW->Detector->PMs);
  if (!ok) message("Error writing to file!", this);
  MW->WindowNavigator->BusyOff();
}
