#include "detectoraddonswindow.h"
#include "ui_detectoraddonswindow.h"
#include "mainwindow.h"
#include "functions.h"
#include "classes.h"
#include "pms.h"
#include "checkupwindowclass.h"
#include "geometrywindowclass.h"
#include "guiutils.h"
#include "genericscriptwindowclass.h"
#include "detectorclass.h"
#include "simulationmanager.h" //for GeoManager update
#include "globalsettingsclass.h"
#include "scriptinterfaces.h"

#include <QtGui>
#include <QFileDialog>
#include <QMessageBox>

DetectorAddOnsWindow::DetectorAddOnsWindow(MainWindow *parent, DetectorClass *detector) :
  QMainWindow(parent),
  ui(new Ui::DetectorAddOnsWindow)
{
  MW = parent;
  Detector = detector;
  ui->setupUi(this);
  this->setFixedSize(this->size());
  ForbidUpdateValues = false;
  TMPignore = false;
  AddedButNotConfirmed = false;
  siHoles = 0;

  Qt::WindowFlags windowFlags = (Qt::Window | Qt::CustomizeWindowHint);
  windowFlags |= Qt::WindowCloseButtonHint;
  windowFlags |= Qt::WindowStaysOnTopHint;
  this->setWindowFlags( windowFlags );

  QIcon RedIcon = createColorCircleIcon(ui->twObjects->iconSize(), Qt::red);
  ui->labDataMissing->setPixmap(RedIcon.pixmap(16,16));  
  ui->labDataMissing->setVisible(false);
  ui->labGridUpdateRequired->setPixmap(RedIcon.pixmap(16,16));
  ui->labGridUpdateRequired->setVisible(false);
  ui->labLightguideConfirmRequired->setPixmap(RedIcon.pixmap(16,16));
  ui->labLightguideConfirmRequired->setVisible(false);
  ui->labLightguideOverlapsDetected->setVisible(false);
  ui->fLightguide->setEnabled(false);
  ui->labMaskConfirmRequired->setPixmap(RedIcon.pixmap(16,16));
  ui->labMaskConfirmRequired->setVisible(false);
  ui->labMaskOverlapsDetected->setVisible(false);
  ui->fGDML->setEnabled(false);
  ui->fSubPrScint->setEnabled(false);
  ui->lGridOverlaps->setVisible(false);
  ui->fPolygon->setVisible(false);

  Detector->Lightguides.resize(0);
  LightguidePropertiesStructure tmp = LightguidePropertiesStructure();
  Detector->Lightguides.append(tmp);
  Detector->Lightguides.append(tmp);

  DetectorAddOnsWindow::UpdateGUI();

  ui->pbReconstructDetector->setVisible(false);
  ui->pbUpdateWaveguideIndication->setVisible(false);
  ui->fWaveCone->setVisible(false);
  ui->fAngle->setVisible(false);
  ui->pbUpdateObjectCollection->setVisible(false);
  ui->pbUpdateObjectsIndication->setVisible(false);
  ui->fObjects->setEnabled(false);
  ui->pbGridUpdateIndication->setVisible(false);

  QDoubleValidator* dv = new QDoubleValidator(this);
  dv->setNotation(QDoubleValidator::ScientificNotation);
  QList<QLineEdit*> list = this->findChildren<QLineEdit *>();
  foreach(QLineEdit *w, list) if (w->objectName().startsWith("led")) w->setValidator(dv);

  DetectorAddOnsWindow::EditFinish();
  Detector->fAdditionalGeoObjectsEnabled = !ui->cbDisableAllAddedObjects->isChecked();

  ui->tableWidget->resizeColumnsToContents();
  ui->tableWidget->resizeRowsToContents();
  DetectorAddOnsWindow::on_pbGridUpdateIndication_clicked();
  DetectorAddOnsWindow::on_cbLightguideAutoshape_toggled(ui->cbLightguideAutoshape->isChecked());
}

DetectorAddOnsWindow::~DetectorAddOnsWindow()
{
  delete ui;  
}

void DetectorAddOnsWindow::on_pbReconstructDetector_clicked()
{ 
  if (ForbidUpdateValues) return; //to avoid loop
  if (MW->DoNotUpdateGeometry) return; //if bulk update in progress
  MW->ReconstructDetector();
}

void DetectorAddOnsWindow::UpdateGUI()
{
  //PrScintCont
  ui->cbPrScint->setChecked(Detector->fPrScintCont);

  //mask
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1 && im != 0)
    {
      im = 0;
      ui->sbMask->setValue(0);
      return; //update on_change
    }
  bool fValid = ( im < Detector->Masks.size() );
  //qDebug() << "---masks:" << im << Detector->Masks.size();
  //qDebug() << "Valid?" << fValid;
  ui->fMask->setEnabled(fValid);
  ui->cbMask->setEnabled(fValid);

  if (fValid)
    {
      MaskClass& m = Detector->Masks[im];
      //ForbidUpdateValues = true;  //forbid update!
      ui->cbMask->setChecked(m.fActive);
      ui->fMask->setEnabled(m.fActive);
      ui->cobMaskTubeBox->setCurrentIndex(m.Shape);
      QString str;
      str.setNum(m.Size);
      ui->ledMaskSizeDiameter->setText(str);
      str.setNum(m.Thickness);
      ui->ledMaskThickness->setText(str);
      ui->leoMaskContainer->setText(m.Container);
      str.setNum(m.OriginX);
      ui->ledMaskX->setText(str);
      str.setNum(m.OriginY);
      ui->ledMaskY->setText(str);
      str.setNum(m.OriginZ);
      ui->ledMaskZ->setText(str);
      str.setNum(m.Phi);
      ui->ledMaskPhi->setText(str);
      str.setNum(m.Theta);
      ui->ledMaskTheta->setText(str);
      str.setNum(m.Psi);
      ui->ledMaskPsi->setText(str);
      ui->cobMaskMaterialBulk->setCurrentIndex(m.MaterialBulk);
      ui->cobMaskMaterialHoles->setCurrentIndex(m.MaterialHoles);
      //ForbidUpdateValues = false; //enable back
    }

  //waveguide
  DetectorAddOnsWindow::on_pbUpdateWaveguideIndication_clicked();  

  //additional objects
  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();

  //Grids
  DetectorAddOnsWindow::on_pbGridUpdateIndication_clicked();

  //GDML
  ui->fGDML->setEnabled(!Detector->isGDMLempty());
}

void DetectorAddOnsWindow::ClearDAMaterialCOBs()
{
  ui->cobMaskMaterialBulk->clear();
  ui->cobMaskMaterialHoles->clear();
  ui->cobWaveguideBulkMaterial->clear();
  ui->cobWaveguideOpeningsMaterial->clear();
  ui->cobObjectMaterial->clear();
}

void DetectorAddOnsWindow::AddDAMaterialToCOBs(QString str)
{
  ui->cobMaskMaterialBulk->addItem(str);
  ui->cobMaskMaterialHoles->addItem(str);
  ui->cobWaveguideBulkMaterial->addItem(str);
  ui->cobWaveguideOpeningsMaterial->addItem(str);
  ui->cobObjectMaterial->addItem(str);
}

void DetectorAddOnsWindow::RemoveMaterial(int imat)
{
  ForbidUpdateValues = true;
  ui->cobMaskMaterialBulk->removeItem(imat);
  ui->cobMaskMaterialHoles->removeItem(imat);
  ui->cobWaveguideBulkMaterial->removeItem(imat);
  ui->cobWaveguideOpeningsMaterial->removeItem(imat);
  ui->cobObjectMaterial->removeItem(imat);
  ForbidUpdateValues = false;

  DetectorAddOnsWindow::on_pbReconstructDetector_clicked();
}

void DetectorAddOnsWindow::UpdateMaterialInCobs(int imat)
{
  QString name = (*Detector->MaterialCollection)[imat]->name;
  ui->cobMaskMaterialBulk->setItemText(imat, name);
  ui->cobMaskMaterialHoles->setItemText(imat, name);
  ui->cobWaveguideBulkMaterial->setItemText(imat, name);
  ui->cobWaveguideOpeningsMaterial->setItemText(imat, name);
  ui->cobObjectMaterial->setItemText(imat, name);
}

void DetectorAddOnsWindow::SetTab(int tab)
{
  if (tab<0) return;
  if (tab>ui->tabwidAddOns->count()-1) return;
  ui->tabwidAddOns->setCurrentIndex(tab);
}

void DetectorAddOnsWindow::UpdateObjectNames()
{
  ui->cobObjectContainer->clear();
  ui->cobGridContainer->clear();
  ui->cobMaskContainer->clear();

  int Volumes = Detector->GeoManager->GetListOfVolumes()->GetEntries();
//  qDebug()<<"Volumes:"<<Volumes;
  for (int iv=0; iv<Volumes; iv++)
    {
      TGeoVolume* thisVolume = (TGeoVolume*)Detector->GeoManager->GetListOfVolumes()->At(iv);
      QString VolName = thisVolume->GetName();
//      qDebug()<<VolName;
      ui->cobObjectContainer->addItem(VolName);
      ui->cobGridContainer->addItem(VolName);
      ui->cobMaskContainer->addItem(VolName);
    }
}

void DetectorAddOnsWindow::on_pbMaskLoad_clicked()
{
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load mask file", MW->GlobSet->LastOpenDir, "Text files (*.txt);;Data files (*.dat);; All files (*.*)");
  //qDebug()<<fileName;
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  DetectorAddOnsWindow::LoadMaskHoles(fileName);

  //ui->labMaskConfirmRequired->setVisible(true);
  //on_pbReconstructDetector_clicked();
  DetectorAddOnsWindow::on_pbMaskConfirmChanges_clicked();
  //ui->labMaskConfirmRequired->hide();
}

void DetectorAddOnsWindow::LoadMaskHoles(const QString fileName)
{
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1)
    {
      ui->sbMask->setValue(0);
      UpdateGUI();
      return;
    }
  MaskClass& m = Detector->Masks[im];

  QFile file(fileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Cannot open file: "+fileName, this);
      return;
    }

  QTextStream in(&file);
  m.MaskElements.resize(0);
  MaskElementStructure tmp;
  bool error = false;
  QRegExp rx("(\\ |\\,|\\:|\\t)");
  while(!in.atEnd())
       {
          QString line = in.readLine();
          //qDebug()<<"Line:" <<line;
          QStringList fields = line.split(rx, QString::SkipEmptyParts);
          //qDebug()<<"fields:"<<fields.size();
          if (fields.size() < 4) continue;

          bool ok, ok1, ok2, ok3, ok4, ok5;
          int type = fields[0].toInt(&ok);
          if (ok)
            {
              if (type == 0)
                {
                  //square
                  if (fields.size()<5)
                    {
                       error = true;
                       break;
                    }

                  if (fields.size() == 5)
                    {
                      //no angle, setting it to 0
                      tmp.Type = 0;
                      tmp.DX = fields[1].toDouble(&ok1);
                      tmp.DY = fields[2].toDouble(&ok2);
                      tmp.SizeX = fields[3].toDouble(&ok3);
                      tmp.SizeY = fields[4].toDouble(&ok4);
                      tmp.Angle = 0;

                      if (!ok1 || !ok2 || !ok3 || !ok4 )
                        {
                          error = true;
                          break;
                        }
                    }
                  else
                    {
                      tmp.Type = 0;
                      tmp.DX = fields[1].toDouble(&ok1);
                      tmp.DY = fields[2].toDouble(&ok2);
                      tmp.SizeX = fields[3].toDouble(&ok3);
                      tmp.SizeY = fields[4].toDouble(&ok4);
                      tmp.Angle = fields[5].toDouble(&ok5);

                      if (!ok1 || !ok2 || !ok3 || !ok4 || !ok5)
                        {
                          error = true;
                          break;
                        }
                    }

                   //qDebug()<< "rectangle with parameters x, y, sizeX, sizeY, angle: " <<tmp.dX<<tmp.dY<<tmp.sizeX <<tmp.sizeY<< tmp.angle;
                   m.MaskElements.append(tmp);
                }
              else
                {
                  //tube
                  if (fields.size()<4)
                    {
                       error = true;
                       break;
                    }
                   tmp.Type = 1;
                   tmp.DX = fields[1].toDouble(&ok1);
                   tmp.DY = fields[2].toDouble(&ok2);
                   tmp.Radius = fields[3].toDouble(&ok3);
                   if (!ok1 || !ok2 || !ok3)
                     {
                       error = true;
                       break;
                     }
                   //qDebug()<< "round with parameters x, y, radius: " <<tmp.dX<<tmp.dY<<tmp.radius;
                   m.MaskElements.append(tmp);
                }
            }
        }

   file.close();
   if (error)
     {
       message("I/O error!", this);
       m.MaskElements.resize(0);
     }

   //DetectorAddOnsWindow::on_pbReconstructDetector_clicked();
   ui->labMaskConfirmRequired->showNormal();
}

void DetectorAddOnsWindow::on_pbMaskDelete_clicked()
{
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1)
    {
      ui->sbMask->setValue(0);
      UpdateGUI();
      return;
    }
  MaskClass& m = Detector->Masks[im];
  m.MaskElements.resize(0);
  DetectorAddOnsWindow::on_pbMaskConfirmChanges_clicked();
}

void DetectorAddOnsWindow::on_pbUpdateWaveguideIndication_clicked()
{
    //protection
    for (int i=0; i<2; i++)
      {
         if (Detector->Lightguides[i].BulkMaterial == -1) Detector->Lightguides[i].BulkMaterial = 0;
         if (Detector->Lightguides[i].OpeningsMaterial == -1) Detector->Lightguides[i].OpeningsMaterial = 0;
      }

    int index = ui->tabwidWaveguideUpperLower->currentIndex();

    ForbidUpdateValues = true;

    const LightguidePropertiesStructure& lg = Detector->Lightguides[index];

    ui->cbWaveguideActivate->setChecked(lg.fActive);
    ui->cbLightguideAutoshape->setChecked( lg.fAutoshape );
    if (!lg.fAutoshape)
      {
        ui->cobLightguideShape->setCurrentIndex(lg.Shape);
        if (lg.Shape==2) ui->sbLightguideSides->setValue( lg.Sides );
        if (lg.Shape!=1) ui->ledLightguidePhi->setText( QString::number( lg.Phi ) );
        ui->ledLightguideSizeX->setText( QString::number( lg.SizeX ) );
        if (lg.Shape==0) ui->ledLightguideSizeY->setText( QString::number( lg.SizeY ) );
      }

    QString str;
    str.setNum(Detector->Lightguides[index].Thickness);
    ui->ledWaveguideThickness->setText(str);
    ui->cobWaveguideOpeningsShape->setCurrentIndex(Detector->Lightguides[index].OpeningsShape);
    str.setNum(Detector->Lightguides[index].OpeningsSize);
    ui->ledWaveguideOpeningsSize->setText(str);
    str.setNum(Detector->Lightguides[index].Angle);
    ui->ledWaveguideAngle->setText(str);
    ui->cobWaveguideBulkMaterial->setCurrentIndex(Detector->Lightguides[index].BulkMaterial);
    ui->cobWaveguideOpeningsMaterial->setCurrentIndex(Detector->Lightguides[index].OpeningsMaterial);
    str.setNum(Detector->Lightguides[index].ConeFactor);
    ui->ledWaveguideConeFactor->setText(str);
    ui->cbWaveguideCone->setChecked(Detector->Lightguides[index].fCone);

    ForbidUpdateValues = false;
}

void DetectorAddOnsWindow::on_tabwidWaveguideUpperLower_currentChanged(int)
{
    //ui->tabwidWaveguideUpperLower->setFocus();  //focus comes too late :(
    DetectorAddOnsWindow::on_pbUpdateWaveguideIndication_clicked();
}

void DetectorAddOnsWindow::on_cobWaveguideOpeningsShape_currentIndexChanged(int index)
{
  if (index == 0) ui->fAngle->setVisible(true);
  else ui->fAngle->setVisible(false);
}

void DetectorAddOnsWindow::on_pbConvertToDummies_clicked()
{
  QList<int> ToAdd;
  bool ok = ExtractNumbersFromQString(ui->lePMlist->text(), &ToAdd);
  if (!ok)
    {
      message("Input error!", this);
      return;
    }

  int iMaxPM = MW->PMs->count()-1;
  for (int i=0; i<ToAdd.size(); i++)
    if (ToAdd[i] >iMaxPM )
      {
        message("Range error!", this);
        return;
      }

  //---converting PMs to dummies---
  bool SawLower = false;
  bool SawUpper = false;
  qSort(ToAdd.begin(), ToAdd.end()); //sorted - starts from smallest
  for (int iadd=ToAdd.size()-1; iadd>-1; iadd--)
    {
       //adding dummy
       PMdummyStructure dpm;

       int ipm = ToAdd[iadd];
//       qDebug()<<"PM->dummy  ToAddindex="<<iadd<<"pm number="<<ipm;

       const pm &PM = MW->PMs->at(ipm);
       dpm.r[0] = PM.x;
       dpm.r[1] = PM.y;
       dpm.r[2] = PM.z;

       dpm.Angle[0] = PM.phi;
       dpm.Angle[1] = PM.theta;
       dpm.Angle[2] = PM.psi;

       dpm.PMtype = PM.type;
       dpm.UpperLower = PM.upperLower;

       Detector->PMdummies.append(dpm);
//       qDebug()<<"dummy added";

       //deleting PM
          //MW->PMs->remove(ipm);
       int ul, index;
       Detector->findPM(ipm, ul, index);
       Detector->PMarrays[ul].PositionsAnglesTypes.remove(index);
//       qDebug()<<"PM removed";
       if (ul == 0) SawUpper = true;
       if (ul == 1) SawLower = true;
    }

//  qDebug()<<"All list done!";

  //updating array type
  //if (MW->PMArrayType() == 0) MW->SetPMarrayType(1); //cannot be regular array!
  if (SawUpper)
    if (MW->PMArrayType(0) == 0) MW->SetPMarrayType(0, 1);
  if (SawLower)
    if (MW->PMArrayType(1) == 0) MW->SetPMarrayType(1, 1);

  MW->updatePMArrayDataIndication();
  MW->PMnumberMightHaveChanged();  
//  qDebug()<<"environment updated";
   DetectorAddOnsWindow::UpdateDummyPMindication();
   //MW->RebuildPmArray();
   MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_sbDummyPMindex_valueChanged(int arg1)
{
  if (arg1 > Detector->PMdummies.count()-1 )
    {
      ui->sbDummyPMindex->setValue(0);
      if (Detector->PMdummies.count() == 0) return;
    }
  DetectorAddOnsWindow::UpdateDummyPMindication();
}

void DetectorAddOnsWindow::on_pbDeleteDummy_clicked()
{
  int idpm = ui->sbDummyPMindex->value();
  if (idpm < Detector->PMdummies.size()) Detector->PMdummies.remove(idpm);
  else
    {
      message("Error: index is out of bounds!", this);
      return;
    }
  if (idpm > Detector->PMdummies.size()-1)
    if (idpm != 0) ui->sbDummyPMindex->setValue(Detector->PMdummies.size()-1);
  DetectorAddOnsWindow::UpdateDummyPMindication();
  MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_pbConvertDummy_clicked()
{
  int idpm = ui->sbDummyPMindex->value();
  if (idpm >= Detector->PMdummies.size())
    {
      message("Error: index is out of bounds!", this);
      return;
    }

  DetectorAddOnsWindow::ConvertDummyToPM(idpm);

  if (idpm > Detector->PMdummies.size()-1)
    if (idpm != 0) ui->sbDummyPMindex->setValue(Detector->PMdummies.size()-1);
  DetectorAddOnsWindow::UpdateDummyPMindication();
  MW->ReconstructDetector();
}

void DetectorAddOnsWindow::ConvertDummyToPM(int idpm)
{
  PMdummyStructure* dum = &Detector->PMdummies[idpm];
  //adding PM
    //MW->PMs->add(Detector->PMdummies[idpm].UpperLower, Detector->PMdummies[idpm].r[0], Detector->PMdummies[idpm].r[1], Detector->PMdummies[idpm].r[2], Detector->PMdummies[idpm].Angle[2], Detector->PMdummies[idpm].PMtype);
    //MW->PMs->at(MW->PMs->count()-1).phi = Detector->PMdummies[idpm].Angle[0];
    //MW->PMs->at(MW->PMs->count()-1).theta = Detector->PMdummies[idpm].Angle[2];
  int ul = dum->UpperLower;
  Detector->PMarrays[ul].PositionsAnglesTypes.append(PosAngTypeStructure(dum->r[0], dum->r[1], dum->r[2],
                                                                         dum->Angle[0], dum->Angle[1], dum->Angle[2],
                                                                         dum->PMtype ));
  //removing dummy
  Detector->PMdummies.remove(idpm);
}

void DetectorAddOnsWindow::on_pbConvertAllToPMs_clicked()
{
   for (int idpm = Detector->PMdummies.size()-1; idpm>-1; idpm--)
     DetectorAddOnsWindow::ConvertDummyToPM(idpm);
   ui->sbDummyPMindex->setValue(Detector->PMdummies.size()-1);
   MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_pbUpdateDummy_clicked()
{
  int idpm = ui->sbDummyPMindex->value();
  if (idpm >= Detector->PMdummies.size())
    {
      message("Error: index is out of bounds!", this);
      return;
    }

  int type = ui->sbDummyType->value();
  if (type >= MW->PMs->countPMtypes())
    {
      message("Error: PM type is out of bounds!", this);
      return;
    }

  PMdummyStructure dpm;
  dpm.PMtype = type;
  dpm.UpperLower = ui->cobDummyUpperLower->currentIndex();
  dpm.r[0] = ui->ledDummyX->text().toDouble();
  dpm.r[1] = ui->ledDummyY->text().toDouble();
  dpm.r[2] = ui->ledDummyZ->text().toDouble();
  dpm.Angle[0] = ui->ledDummyPhi->text().toDouble();
  dpm.Angle[1] = ui->ledDummyTheta->text().toDouble();
  dpm.Angle[2] = ui->ledDummyPsi->text().toDouble();
  Detector->PMdummies[idpm] = dpm;

  DetectorAddOnsWindow::UpdateDummyPMindication();
  MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_pbCreateNewDummy_clicked()
{
  int type = ui->sbDummyType->value();
  if (type >= MW->PMs->countPMtypes())
    {
      message("Error: PM type is out of bounds!", this);
      return;
    }

  PMdummyStructure dpm;
  dpm.PMtype = type;
  dpm.UpperLower = ui->cobDummyUpperLower->currentIndex();
  dpm.r[0] = ui->ledDummyX->text().toDouble();
  dpm.r[1] = ui->ledDummyY->text().toDouble();
  dpm.r[2] = ui->ledDummyZ->text().toDouble();
  dpm.Angle[0] = ui->ledDummyPhi->text().toDouble();
  dpm.Angle[1] = ui->ledDummyTheta->text().toDouble();
  dpm.Angle[2] = ui->ledDummyPsi->text().toDouble();
  Detector->PMdummies.append(dpm);

  DetectorAddOnsWindow::UpdateDummyPMindication();
  ui->sbDummyPMindex->setValue(Detector->PMdummies.size()-1);
  MW->ReconstructDetector();
}

void DetectorAddOnsWindow::UpdateDummyPMindication()
{
  bool on;
  if (Detector->PMdummies.size() == 0) on = false;
  else on = true;

  ui->pbDeleteDummy->setEnabled(on);
  ui->pbUpdateDummy->setEnabled(on);
  ui->pbConvertDummy->setEnabled(on);

  int idpm = ui->sbDummyPMindex->value();
  if (idpm > Detector->PMdummies.count() ) return;

  ui->sbDummyType->setValue(Detector->PMdummies[idpm].PMtype);
  ui->leoDummyType->setText(MW->PMs->getType(Detector->PMdummies[idpm].PMtype)->name);
  ui->cobDummyUpperLower->setCurrentIndex(Detector->PMdummies[idpm].UpperLower);
  ui->ledDummyX->setText(QString::number(Detector->PMdummies[idpm].r[0]));
  ui->ledDummyY->setText(QString::number(Detector->PMdummies[idpm].r[1]));
  ui->ledDummyZ->setText(QString::number(Detector->PMdummies[idpm].r[2]));
  ui->ledDummyPhi->setText(QString::number(Detector->PMdummies[idpm].Angle[0]));
  ui->ledDummyTheta->setText(QString::number(Detector->PMdummies[idpm].Angle[1]));
  ui->ledDummyPsi->setText(QString::number(Detector->PMdummies[idpm].Angle[2]));
}

void DetectorAddOnsWindow::setDisbaleAllAdditionalObjects(bool flag)
{
  ui->cbDisableAllAddedObjects->setChecked(flag);
  Detector->fAdditionalGeoObjectsEnabled = !flag;
}

void DetectorAddOnsWindow::on_sbDummyType_valueChanged(int arg1)
{
  if (arg1 >= MW->PMs->countPMtypes())
    {
      message("Error: PM type is out of bounds!", this);
      ui->sbDummyType->setValue(0);
      return;
    }
}

void DetectorAddOnsWindow::on_pbLoadDummyPMs_clicked()
{
    QString fileName;
    fileName = QFileDialog::getOpenFileName(this, "Load file with dummy PMs", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);; All files (*.*)");
    //qDebug()<<fileName;
    if (fileName.isEmpty()) return;
    MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
    Detector->PMdummies.resize(0);
    MW->LoadDummyPMs(fileName);
    ui->sbDummyPMindex->setValue(0);
    DetectorAddOnsWindow::UpdateDummyPMindication();
    MW->ReconstructDetector();
}


//------------//-----------------//----------------------------//

void DetectorAddOnsWindow::on_pbRemoveObject_clicked()
{
  int size = Detector->AdditionalGeoObjects.size();
  int iobj = ui->sbObjectNumber->value();
  if (iobj>size-1)
    {
      if (iobj == 0) return;
      ui->sbObjectNumber->setValue(0);
      //message("Object number is out of range!", this);
      return;
    }
  Detector->AdditionalGeoObjects.remove(iobj);

  iobj--;
  if (iobj<0) iobj=0;
  ui->sbObjectNumber->setValue(iobj);
  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();
  MW->ReconstructDetector();
  ui->pbAddNewObject->setEnabled(true);
}

void DetectorAddOnsWindow::on_pbAddNewObject_clicked()
{
  int size = Detector->AdditionalGeoObjects.size();
  QString name = "GeoObject" + QString::number(size);
  while (DetectorAddOnsWindow::isNameInUse(name)) name += "_1";

  Detector->AdditionalGeoObjects.append(AdditionalGeoObjectClass(name));
  ui->sbObjectNumber->setValue(size);

  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();
  AddedButNotConfirmed = true;
  ui->lwAddGeoObjects->setEnabled(false);
  ui->pbCopyObject->setEnabled(false);
  ui->pbAddNewObject->setEnabled(false);
  ui->labDataMissing->setVisible(true);
  DetectorAddOnsWindow::ShowObject();
}

void DetectorAddOnsWindow::on_pbUpdateObjectCollection_clicked()
{
  int size = Detector->AdditionalGeoObjects.size();
  int iobj = ui->sbObjectNumber->value();
  if (iobj>size-1)
    {
      if (iobj == 0) return;
      ui->sbObjectNumber->setValue(0);
      //message("Object number is out of range!", this);
      return;
    }

  AdditionalGeoObjectClass* o = &Detector->AdditionalGeoObjects[iobj];

  o->Name = ui->leObjectName->text();
  o->Container = ui->leoObjectContainer->text();
  o->Material = ui->cobObjectMaterial->currentIndex();
  o->Shape = ui->cobObjectShape->currentIndex();
  o->R[0] = ui->ledObjectX->text().toDouble();
  o->R[1] = ui->ledObjectY->text().toDouble();
  o->R[2] = ui->ledObjectZ->text().toDouble();
  o->Size[0] = ui->ledSizeX->text().toDouble();
  o->Size[1] = ui->ledSizeY->text().toDouble();
  o->Size[2] = ui->ledSizeZ->text().toDouble();
  o->Angles[0] = ui->ledObjectPhi->text().toDouble();
  o->Angles[1] = ui->ledObjectTheta->text().toDouble();
  o->Angles[2] = ui->ledObjectPsi->text().toDouble();

  o->Arb8height = ui->ledArb8Height->text().toDouble();
  for (int i=0; i<8; i++)
    {
      double X = ui->tableWidget->item(i,0)->text().toDouble();
      double Y = ui->tableWidget->item(i,1)->text().toDouble();

      o->Vertices[i][0] = X;
      o->Vertices[i][1] = Y;
 //     qDebug()<<X<<Y;
    }

  o->Sides = ui->sbPolygonSides->value();
}

void DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked()
{
  int size = Detector->AdditionalGeoObjects.size();

  bool flag;
  if (size>0) flag = true; else flag = false;
  ui->pbRemoveObject->setEnabled(flag);
  ui->pbShowObject->setEnabled(flag);
  ui->fObjects->setEnabled(flag);
  ui->pbCopyObject->setEnabled(flag);

  //updating list
  ui->lwAddGeoObjects->clear();

  for (int i=0; i<size; i++)
    {
      QString name = Detector->AdditionalGeoObjects[i].Name;
      ui->lwAddGeoObjects->addItem(name);
    }

  //updating object info

  int iobj = ui->sbObjectNumber->value();
  if (iobj>size-1)
    {      
      if (iobj == 0) return;
      ui->sbObjectNumber->setValue(0);
      //message("Object number is out of range!", this);
      return;
    }
  AdditionalGeoObjectClass* o = &Detector->AdditionalGeoObjects[iobj];

  ui->leObjectName->setText(o->Name); 
  ui->leoObjectContainer->setText(o->Container);
  ui->cobObjectMaterial->setCurrentIndex(o->Material);
  ui->cobObjectShape->setCurrentIndex(o->Shape);
  ui->ledObjectX->setText(QString::number(o->R[0]));
  ui->ledObjectY->setText(QString::number(o->R[1]));
  ui->ledObjectZ->setText(QString::number(o->R[2]));
  ui->ledSizeX->setText(QString::number(o->Size[0]));
  ui->ledSizeY->setText(QString::number(o->Size[1]));
  ui->ledSizeZ->setText(QString::number(o->Size[2]));
  ui->ledObjectPhi->setText(QString::number(o->Angles[0]));
  ui->ledObjectTheta->setText(QString::number(o->Angles[1]));
  ui->ledObjectPsi->setText(QString::number(o->Angles[2]));

  if (o->Shape == 5) ui->sbPolygonSides->setValue(o->Sides);

  if (o->Shape == 4)
  {
      ForbidUpdateValues = true; //--//

      ui->swSizes->setCurrentIndex(1);
      ui->ledArb8Height->setText(QString::number(o->Arb8height));
      for (int i=0; i<8; i++)
        {
          //suspect a bug in Qt 5.3.0 related to assertion of item(i,j)
//          qDebug()<<"----index:"<<i;
          double v0 = o->Vertices[i][0];
          double v1 = o->Vertices[i][1];
//          qDebug()<<"vertices to assign:"<<v0<<v1;
//          qDebug()<<"current text:"<<ui->tableWidget->item(i,0)->text()<<ui->tableWidget->item(i,1)->text();

          ui->tableWidget->item(i,0)->setText( QString::number(v0) );
          ui->tableWidget->item(i,1)->setText( QString::number(v1) );
//          qDebug()<<"done!";          
        }

      ui->tableWidget->resizeColumnsToContents();
      ui->tableWidget->resizeRowsToContents();

      ForbidUpdateValues = false; //--//
  }
  else ui->swSizes->setCurrentIndex(0);  
}

void DetectorAddOnsWindow::on_sbObjectNumber_valueChanged(int arg1)
{
  int size = Detector->AdditionalGeoObjects.size();
  if (arg1 == 0 && size == 0) return;
  if (arg1 > size-1) ui->sbObjectNumber->setValue(0);

  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();
  ui->lwAddGeoObjects->setCurrentRow(arg1);
}

void DetectorAddOnsWindow::EditFinish()
{ 
  int size = Detector->AdditionalGeoObjects.size();
  ui->sbObjectNumber->setEnabled(true);
  if (size>0) ui->pbRemoveObject->setEnabled(true);
  ui->pbAddNewObject->setEnabled(true);
  ui->lwAddGeoObjects->setEnabled(true);
}

void DetectorAddOnsWindow::on_lwAddGeoObjects_clicked(const QModelIndex &)
{
  ui->sbObjectNumber->setValue(ui->lwAddGeoObjects->currentRow());
  DetectorAddOnsWindow::ShowObject();
}

void DetectorAddOnsWindow::on_pbObjectConfirm_clicked()
{    
  int size = Detector->AdditionalGeoObjects.size();
  int iobj = ui->sbObjectNumber->value();
  if (iobj>size-1)
    {
      if (iobj == 0) return;
      ui->sbObjectNumber->setValue(0);
      //message("Object number is out of range!", this);
      return;
    }
  //activating
  ui->lwAddGeoObjects->setEnabled(true);
  ui->pbAddNewObject->setEnabled(true);
  ui->pbCopyObject->setEnabled(true);
  DetectorAddOnsWindow::on_pbUpdateObjectCollection_clicked();
  DetectorAddOnsWindow::EditFinish();
  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();

  Detector->AdditionalGeoObjects[iobj].fActivated = true;
  MW->ReconstructDetector();

  //checking overlaps  
  int nooverlaps = MW->CheckUpWindow->CheckGeoOverlaps();
//  qDebug()<<"nooverlap report:"<<nooverlaps;
  if (nooverlaps == 0)    
    {
      ui->labOverlaps->setText("Not detected");
      ui->labDataMissing->setVisible(false);
    }
  else
    {
      ui->labOverlaps->setText("Detected!");
      ui->labDataMissing->setVisible(true);
      MW->CheckUpWindow->show();
    }

  DetectorAddOnsWindow::ShowObject();  
}

void DetectorAddOnsWindow::on_cbDisableAllAddedObjects_toggled(bool checked)
{ 
    Detector->fAdditionalGeoObjectsEnabled = !checked;
    if (!MW->DoNotUpdateGeometry)
      MW->ReconstructDetector(); //due to disable, cannot be triggered when GDML imported
}

void DetectorAddOnsWindow::on_cobObjectContainer_activated(int index)
{
  if (index == -1) return; //protection
  ui->leoObjectContainer->setText(ui->cobObjectContainer->currentText());
}

void DetectorAddOnsWindow::on_cobObjectShape_currentIndexChanged(int index)
{
   //default:
   ui->swSizes->setCurrentIndex(0);
   ui->fPolygon->setVisible(index == 5);

   switch (index)
     {
     case 0:
       //box
       ui->labX->setText("Length");
       ui->labY->setText("Width");
       ui->ledSizeY->setEnabled(true);
       ui->labZ->setText("Height");
       ui->ledSizeZ->setEnabled(true);
       break;
     case 1:
       //cylinder
       ui->labX->setText("Diameter");
       ui->labY->setText("");
       ui->ledSizeY->setEnabled(false);
       ui->labZ->setText("Height");
       ui->ledSizeZ->setEnabled(true);
       break;
     case 2:
       //cone
       ui->labX->setText("Diameter(top)");
       ui->labY->setText("Diameter(bottom)");
       ui->ledSizeY->setEnabled(true);
       ui->labZ->setText("Height");
       ui->ledSizeZ->setEnabled(true);
       break;
     case 3:
       //sphere
       ui->labX->setText("Diameter");
       ui->labY->setText("");
       ui->ledSizeY->setEnabled(false);
       ui->labZ->setText("");
       ui->ledSizeZ->setEnabled(false);
       break;
     case 4:
       //arbitrary 8 vertices shape
       ui->swSizes->setCurrentIndex(1);
       break;
     case 5:
       //TGeoPgone
       ui->labX->setText("Size(top)");
       ui->labY->setText("Size(bottom)");
       ui->ledSizeY->setEnabled(true);
       ui->labZ->setText("Height");
       ui->ledSizeZ->setEnabled(true);
       break;

     default:;
     }
}

void DetectorAddOnsWindow::on_pbShowObject_clicked()
{
   MW->GeometryWindow->ShowAndFocus();
   DetectorAddOnsWindow::ShowObject();
}

void DetectorAddOnsWindow::ShowObject()
{
  QString VolName = ui->leObjectName->text(); 
  DetectorAddOnsWindow::HighlightVolume(VolName);
  MW->GeometryWindow->SetAsActiveRootWindow();
  Detector->GeoManager->ClearTracks();
  Detector->top->Draw();
  MW->GeometryWindow->PostDraw();
  MW->GeometryWindow->UpdateRootCanvas();
}

void DetectorAddOnsWindow::HighlightVolume(QString VolName)
{
  TObjArray* list = Detector->GeoManager->GetListOfVolumes();
  int size = list->GetEntries();

  for (int i=0; i<size; i++)
    {
      TGeoVolume* vol = (TGeoVolume*)list->At(i);
      if (!vol) break;
      QString name = vol->GetName();
//      qDebug()<<"name: "<<name;
      if (name == VolName) vol->SetLineColor(kRed);
      else vol->SetLineColor(kGray);
    }
}

void DetectorAddOnsWindow::on_leObjectName_editingFinished()
{
  if (TMPignore) return;
    //shoul be unique name!
  int iobj = ui->sbObjectNumber->value();
  //object with this name already defined in GeoManager?
  QString name = ui->leObjectName->text();

  if (name != Detector->AdditionalGeoObjects[iobj].Name) //else name has not been changed
    {
      if ( DetectorAddOnsWindow::isNameInUse(name) )
        {
          TMPignore = true;
          message("An object with this name is already defined in GeoManager!", this);
          TMPignore = false;
          ui->leObjectName->setText(Detector->AdditionalGeoObjects[iobj].Name);
          return;
        }
    }
  //Ok name!
  ui->labDataMissing->setVisible(true);
}

bool DetectorAddOnsWindow::isNameInUse(QString name)
{
 // qDebug()<<"Exists? ->"<<name;
  int Volumes = Detector->GeoManager->GetListOfVolumes()->GetEntries();
  for (int iv=0; iv<Volumes; iv++)
    {
      TGeoVolume* thisVolume = (TGeoVolume*)Detector->GeoManager->GetListOfVolumes()->At(iv);
      QString VolName = thisVolume->GetName();
      if (name == VolName) return true;
    }
  return false;
}

void DetectorAddOnsWindow::on_pbCopyObject_clicked()
{
  int copyFrom = ui->sbObjectNumber->value();

  DetectorAddOnsWindow::on_pbAddNewObject_clicked();
  QString name = Detector->AdditionalGeoObjects[Detector->AdditionalGeoObjects.size()-1].Name;
  Detector->AdditionalGeoObjects[Detector->AdditionalGeoObjects.size()-1] = Detector->AdditionalGeoObjects[copyFrom];
  Detector->AdditionalGeoObjects[Detector->AdditionalGeoObjects.size()-1].Name = name;
 // MW->AdditionalGeoObjects[MW->AdditionalGeoObjects.size()-1].r[0] += 10.0;

  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();
  ui->pbCopyObject->setEnabled(false);
  DetectorAddOnsWindow::ShowObject();
}

void DetectorAddOnsWindow::on_pbWaveguideConfirmChanges_clicked()
{  
  //qDebug() << "-------------------"<< ForbidUpdateValues << MW->DoNotUpdateGeometry;
  if (ForbidUpdateValues) return; //to avoid loop

  int index = ui->tabwidWaveguideUpperLower->currentIndex();
  LightguidePropertiesStructure& lg = Detector->Lightguides[index];
  lg.fActive = ui->cbWaveguideActivate->isChecked();
  if (lg.fActive)
    {
      // bulk (container)
      lg.BulkMaterial = ui->cobWaveguideBulkMaterial->currentIndex();
      lg.fAutoshape = ui->cbLightguideAutoshape->isChecked();
      if (!lg.fAutoshape)
        {
          lg.Shape = ui->cobLightguideShape->currentIndex();
          lg.Sides = ui->sbLightguideSides->value();
          lg.Phi = ui->ledLightguidePhi->text().toDouble();
          lg.SizeX = ui->ledLightguideSizeX->text().toDouble();
          lg.SizeY = ui->ledLightguideSizeY->text().toDouble();
        }
      lg.Thickness = ui->ledWaveguideThickness->text().toDouble();

      // openings
      lg.fCone = ui->cbWaveguideCone->isChecked();
      if (lg.fCone) lg.ConeFactor = ui->ledWaveguideConeFactor->text().toDouble();
      lg.OpeningsMaterial = ui->cobWaveguideOpeningsMaterial->currentIndex();
      lg.OpeningsSize = ui->ledWaveguideOpeningsSize->text().toDouble();
      lg.OpeningsShape = ui->cobWaveguideOpeningsShape->currentIndex();
      lg.Angle = ui->ledWaveguideAngle->text().toDouble();
    }

  DetectorAddOnsWindow::on_pbReconstructDetector_clicked();

  int nooverlaps = MW->CheckUpWindow->CheckGeoOverlaps();
  if (nooverlaps == 0) ui->labLightguideOverlapsDetected->setVisible(false);
  else
    {
      ui->labLightguideOverlapsDetected->setVisible(true);
      MW->CheckUpWindow->show();
    }
  ui->labLightguideConfirmRequired->setVisible(false);
}

void DetectorAddOnsWindow::on_pbMaskConfirmChanges_clicked()
{
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1)
    {
      ui->sbMask->setValue(0);
      UpdateGUI();
      return;
    }
  MaskClass& m = Detector->Masks[im];

  m.fActive = ui->cbMask->isChecked();
  m.Shape = ui->cobMaskTubeBox->currentIndex();
  m.Size = ui->ledMaskSizeDiameter->text().toDouble();
  m.Container = ui->leoMaskContainer->text();
  m.Thickness = ui->ledMaskThickness->text().toDouble();
  m.OriginX = ui->ledMaskX->text().toDouble();
  m.OriginY = ui->ledMaskY->text().toDouble();
  m.OriginZ = ui->ledMaskZ->text().toDouble();
  m.Phi = ui->ledMaskPhi->text().toDouble();
  m.Theta = ui->ledMaskTheta->text().toDouble();
  m.Psi = ui->ledMaskPsi->text().toDouble();
  m.MaterialBulk = ui->cobMaskMaterialBulk->currentIndex();
  m.MaterialHoles = ui->cobMaskMaterialHoles->currentIndex();

  on_pbReconstructDetector_clicked();

  int nooverlaps = MW->CheckUpWindow->CheckGeoOverlaps();
  //qDebug()<<"overlaps:"<<nooverlaps;

  if (nooverlaps == 0) ui->labMaskOverlapsDetected->setVisible(false);
  else
    {
      ui->labMaskOverlapsDetected->setVisible(true);
      MW->CheckUpWindow->show();
    }

  ui->labMaskConfirmRequired->setVisible(false);
}

void DetectorAddOnsWindow::on_tableWidget_cellChanged(int row, int column)
{
    if (TMPignore) return; //do not update during filling with data!

    //catching syntax errors
    int iobj = ui->sbObjectNumber->value();
    AdditionalGeoObjectClass* o = &Detector->AdditionalGeoObjects[iobj];
    bool ok;
    QString str = ui->tableWidget->item(row,column)->text();
    str.toDouble(&ok);
    if (!ok)
      {
        message("Input error!", this);
        ui->tableWidget->item(row, column)->setText( QString::number(o->Vertices[row][column]) );
        return;
      }

    ui->tableWidget->resizeColumnsToContents();
    ui->tableWidget->resizeRowsToContents();
}

void DetectorAddOnsWindow::on_pbHolesScript_clicked()
{
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1)
    {
      ui->sbMask->setValue(0);
      UpdateGUI();
      return;
    }
  MaskClass& m = Detector->Masks[im];

    MW->extractGeometryOfScriptWindow();
    if (MW->GenScriptWindow) delete MW->GenScriptWindow;
    MW->GenScriptWindow = new GenericScriptWindowClass();
    MW->recallGeometryOfScriptWindow();

    //configure the script window and engine
    siHoles = new InterfaceToHolesScript; //deleted by the GenScriptWindow
    MW->GenScriptWindow->SetInterfaceObject(siHoles);    
    QStringList coms;
    coms << "Tube" << "Box";
    MW->GenScriptWindow->SetCustomCommands(coms);
    QString HelpText = "  Available commands:\n\n"
                       " Tube(x0, y0, diameter)\n"
                       " Box(x0, y0, sizeX, sizeY, angle)\n";
    MW->GenScriptWindow->SetExample("for (var i=0; i<10; i++)\n{\n  var diam = 0.1*(10-i)+1\n  Tube( 5.0*(i-5), i-5, diam)\n}");
    MW->GenScriptWindow->SetShowEvaluationResult(false); //do not show "undefined"
    MW->GenScriptWindow->SetTitle("Add phantom holes");
    MW->GenScriptWindow->SetScript(&m.MaskScript);
    MW->GenScriptWindow->SetStarterDir(MW->GlobSet->LibScripts);

    //define what to do on evaluation success
    connect(MW->GenScriptWindow, SIGNAL(success(QString)), this, SLOT(HolesScriptSuccess()));
    //if needed. connect signals of the interface object with the required slots of any ANTS2 objects

    MW->GenScriptWindow->show();
}

void DetectorAddOnsWindow::HolesScriptSuccess()
{
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1)
    {
      ui->sbMask->setValue(0);
      UpdateGUI();
      return;
    }
  MaskClass& m = Detector->Masks[im];

   //script engine reported success
   m.MaskElements = siHoles->Holes;
   siHoles->Holes.clear();

   //ui->labMaskConfirmRequired->showNormal();
   //on_pbReconstructDetector_clicked();
   //DetectorAddOnsWindow::on_pbMaskConfirmChanges_clicked();
   //ui->labMaskConfirmRequired->hide();
   DetectorAddOnsWindow::on_pbMaskConfirmChanges_clicked();
}

void DetectorAddOnsWindow::on_pbGridUpdateIndication_clicked()
{
  bool fNoGrids = Detector->Grids.isEmpty();

  ui->fGridThis->setVisible(!fNoGrids);
  ui->lGrids->setVisible(!fNoGrids);
  ui->pbRemoveGrid->setEnabled(!fNoGrids);

  if (fNoGrids) return;  
  //updating indication
  int ig = ui->sbGrid->value();
  if (ig > Detector->Grids.size()-1)
    {
      ui->sbGrid->setValue(0); //update on_change
      return;
    }

  OpticalGridStructure* g = &Detector->Grids[ig];
  ui->fGridEnabled->setEnabled(g->fActive);
  ui->cbGridEnabled->setChecked(g->fActive);
  ui->cobGridType->setCurrentIndex(g->Type);
  ui->ledGridDiameter->setText(QString::number(g->Diameter));
  ui->ledGridPitch->setText(QString::number(g->Pitch));
  ui->ledGridAngle->setText(QString::number(g->WireAngle));

  ui->ledGridAbsProb->setText(QString::number(g->Abs));
  ui->leoGridContainer->setText(g->Container);
  ui->ledGridZ->setText(QString::number(g->Z));
  int Shape = g->Shape;
  ui->cobGridShape->setCurrentIndex(Shape);
  ui->ledGridSizeX->setText(QString::number(g->SizeX));
  if (Shape==0) ui->ledGridSizeY->setText(QString::number(g->SizeY));
  if (Shape!=1) ui->ledGridPhi->setText(QString::number(g->Phi));
  if (Shape==2) ui->sbGridSides->setValue(g->Sides);

  ui->labGridUpdateRequired->setVisible(false);
}

void DetectorAddOnsWindow::on_sbGrid_valueChanged(int arg1)
{
   if (Detector->Grids.isEmpty())
     {
       ui->sbGrid->setValue(0);
     }
   else
     {
       int numGrids = Detector->Grids.size();
       if (arg1 > numGrids-1)
         {
           ui->sbGrid->setValue(numGrids-1);
           return; //update on_change
         }
     }

   DetectorAddOnsWindow::on_pbGridUpdateIndication_clicked();
}

void DetectorAddOnsWindow::on_pbRemoveGrid_clicked()
{
   int grid = ui->sbGrid->value();
   if (grid>Detector->Grids.size()-1)
     {
       message("Wrong grid number!", this);
       DetectorAddOnsWindow::on_sbGrid_valueChanged(0);
       return;
     }

   Detector->Grids.remove(grid);
   ui->sbGrid->setValue(grid-1);
   DetectorAddOnsWindow::on_pbGridUpdateIndication_clicked();
   MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_pbAddGrid_clicked()
{
    OpticalGridStructure g(1, 0, 0.1, 10, 1.0); //int Type, double WireAngle, double Diameter, double Pitch, double Abs
    int shape = Detector->Sandwich.Shape;
    double sizeX = Detector->Sandwich.DefaultSizeX;
    double sizeY = Detector->Sandwich.DefaultSizeY;
    double phi = Detector->Sandwich.Angle;
    g.PositionGrid("Top", 0, shape, sizeX, sizeY, phi); //QString container, double z, int shape, double sizeX, double sizeY, double phi
    g.fActive = true;
    Detector->Grids.append(g);

    ui->sbGrid->setValue(Detector->Grids.size()-1);
    DetectorAddOnsWindow::on_sbGrid_valueChanged(ui->sbGrid->value()); //update indication
    ui->labGridUpdateRequired->showNormal();
}

void DetectorAddOnsWindow::on_pbGridsAllDisable_clicked()
{
    for (int i=0; i<Detector->Grids.size();i++) Detector->Grids[i].fActive = false;
    DetectorAddOnsWindow::on_pbGridUpdateIndication_clicked();
    MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_cbGridEnabled_clicked()
{
  int grid = ui->sbGrid->value();
  if (grid>Detector->Grids.size()-1)
    {
      message("Wrong grid number!", this);
      DetectorAddOnsWindow::on_sbGrid_valueChanged(0);
      return;
    }

  Detector->Grids[grid].fActive = ui->cbGridEnabled->isChecked();
  MW->ReconstructDetector();
}

void DetectorAddOnsWindow::on_pbGridConfirm_clicked()
{
  int grid = ui->sbGrid->value();
  if (grid > Detector->Grids.size()-1)
    {
      message("Wrong grid number!", this);
      DetectorAddOnsWindow::on_sbGrid_valueChanged(0);
      return;
    }

  //updating grid object
  OpticalGridStructure* g = &Detector->Grids[grid]; //protected
  g->Type = ui->cobGridType->currentIndex();
  g->Diameter = ui->ledGridDiameter->text().toDouble();
  g->Pitch = ui->ledGridPitch->text().toDouble();
  g->WireAngle = ui->ledGridAngle->text().toDouble();
  g->Abs = ui->ledGridAbsProb->text().toDouble();

  g->Container = ui->leoGridContainer->text();
  g->Z = ui->ledGridZ->text().toDouble();
  int shape = ui->cobGridShape->currentIndex();
  g->Shape = shape;
  g->SizeX = ui->ledGridSizeX->text().toDouble();
  if (shape==0) g->SizeY = ui->ledGridSizeY->text().toDouble();
  if (shape!=1) g->Phi = ui->ledGridPhi->text().toDouble();
  if (shape==2) g->Sides = ui->sbGridSides->value();

  ui->labGridUpdateRequired->setVisible(false);
  MW->ReconstructDetector();

  //checking overlaps
  int overlaps = MW->CheckUpWindow->CheckGeoOverlaps();
  if (overlaps == 0)
      ui->lGridOverlaps->setVisible(false);
  else
    {
      ui->lGridOverlaps->setVisible(true);
      MW->CheckUpWindow->show();
    }
}

void DetectorAddOnsWindow::on_ledGridAbsProb_editingFinished()
{
   double absProb = ui->ledGridAbsProb->text().toDouble();
   if (absProb<0 || absProb>1.0)
     {
       message("Absorption probability should be in the range from 0 to 1", this);
       ui->ledGridAbsProb->setText("1.0");
     }
}

void DetectorAddOnsWindow::on_cobGridContainer_activated(int index)
{
  if (index == -1) return; //protection
  ui->leoGridContainer->setText(ui->cobGridContainer->currentText());
}

void DetectorAddOnsWindow::on_pbUseScriptToAddObj_clicked()
{
  MW->extractGeometryOfScriptWindow();
  if (MW->GenScriptWindow) delete MW->GenScriptWindow;  
  MW->GenScriptWindow = new GenericScriptWindowClass();
  MW->recallGeometryOfScriptWindow();

  AddObjScriptInterface = new InterfaceToAddObjScript(MW); //deleted by the GenScriptWindow
  MW->GenScriptWindow->SetInterfaceObject(AddObjScriptInterface);
  MW->GenScriptWindow->SetShowEvaluationResult(false); //do not show "undefined"
  MW->GenScriptWindow->SetExample("ClearAll()\nfor (var i=0; i<3; i++)\n Box('Test'+i, 10,5,2, 0, 'PrScint', (i-1)*20,i*2,-i*5,  0,0,0)");
  MW->GenScriptWindow->SetTitle("Add custom objects");
  MW->GenScriptWindow->SetScript(&Detector->AddObjPositioningScript);
  MW->GenScriptWindow->SetStarterDir(MW->GlobSet->LibScripts);
  connect(MW->GenScriptWindow, SIGNAL(success(QString)), this, SLOT(AddObjScriptSuccess()));

  AddObjScriptInterface->GeoObjects.clear();
  MW->GenScriptWindow->show();
}

void DetectorAddOnsWindow::AddObjScriptSuccess()
{
  AddObjScriptInterface->UpdateGeometry(true);
  // this will call the next function to update indication
}

void DetectorAddOnsWindow::UpdateAddObjIndicationAfterScript(int res)
//0-no overlaps, 1-overlaps detected, 2-overlaps not checked
{
  ui->sbObjectNumber->setValue(Detector->AdditionalGeoObjects.size()-1);
  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();

  if (res == 1)
    {
      ui->labOverlaps->setText("Detected!");
      ui->labDataMissing->setVisible(true);
      MW->CheckUpWindow->show();
    }
  else if (res == 0)
    {
      ui->labOverlaps->setText("Not detected");
      ui->labDataMissing->setVisible(false);
    }
  else
    {
      ui->labOverlaps->setText("Not checked");
      ui->labDataMissing->setVisible(false);
    }
}

void DetectorAddOnsWindow::on_pbRemoveAllObjects_clicked()
{
  int ret = QMessageBox::question(this, "Confirmation", "Remove all objects?",
                                 QMessageBox::Yes | QMessageBox::Cancel);
  if (ret == QMessageBox::Cancel) return;
  Detector->AdditionalGeoObjects.clear();
  ui->sbObjectNumber->setValue(0);
  DetectorAddOnsWindow::on_pbUpdateObjectsIndication_clicked();
  MW->ReconstructDetector();
  ui->pbAddNewObject->setEnabled(true);
}

void DetectorAddOnsWindow::on_pbCopyThisObjToClipboard_clicked()
{
  QString name = ui->leObjectName->text();
  QString text;
  switch (ui->cobObjectShape->currentIndex())
    {
    case 0: //box
      text = "Box(\""+name+"\", "+ui->ledSizeX->text()+","+ui->ledSizeY->text()+","+ui->ledSizeZ->text()+", ";
      break;
    case 1:
      text = "Cylinder(\""+name+"\", "+ui->ledSizeX->text()+","+ui->ledSizeZ->text()+", ";
      break;
    case 2:
      text = "Cone(\""+name+"\", "+ui->ledSizeX->text()+","+ui->ledSizeY->text()+","+ui->ledSizeZ->text()+", ";
      break;
    case 3:
      text= "Sphere(\""+name+"\", "+ui->ledSizeX->text()+", ";
      break;
    default:
      message("Arb8 is unsupported!", this);
      return;
    }
  text += QString::number(ui->cobObjectMaterial->currentIndex()) + ", ";
  text += "\""+ui->leoObjectContainer->text() + "\", ";
  text += ui->ledObjectX->text()+","+ui->ledObjectY->text()+","+ui->ledObjectZ->text()+", ";
  text += ui->ledObjectPhi->text()+","+ui->ledObjectTheta->text()+","+ui->ledObjectPsi->text()+")";

  QClipboard *clipboard = QApplication::clipboard();// QString originalText = clipboard->text();
  clipboard->setText(text);
}

void DetectorAddOnsWindow::on_pbSaveTGeo_clicked()
{
  QString starter = MW->GlobSet->LastOpenDir;
  QFileDialog *fileDialog = new QFileDialog;
  fileDialog->setDefaultSuffix("gdml");
  QString fileName = fileDialog->getSaveFileName(this, "Export detector geometry", starter, "GDML files (*.gdml);;Root files (*.root)");
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QFileInfo fi(fileName);
  if (fi.suffix().isEmpty()) fileName += ".gdml";
  if (fi.suffix().compare("root") && fi.suffix().compare("gdml"))
    {
      message("Only root and gdml files are accepted!", this);
      return;
    }

  QByteArray ba = fileName.toLocal8Bit();
  const char *c_str = ba.data();
  Detector->GeoManager->Export(c_str);
}

void DetectorAddOnsWindow::on_pbLoadTGeo_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load TGeoManager geometry file", MW->GlobSet->LastOpenDir, "GDML files (*.gdml);;Root files (*.root)");
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QFileInfo fi(fileName);
  if (fi.suffix() != "root" && fi.suffix() != "gdml")
    {
      message("Only root and gdml files are accepted!", this);
      return;
    }
  QFile f(fileName);
  if (!f.open(QFile::ReadOnly | QFile::Text))
    {
      message("Cannot open file!", this);
      return;
    }
  QTextStream in(&f);
  QString gdml = in.readAll();

  //attempting to load and validity check
  bool fOK = Detector->importGDML(gdml);

  if (fOK)
    {
      //qDebug() << "--> GDML successfully registered";
      MW->PMnumberMightHaveChanged();
      MW->ShowGeometry();
    }
  else
    {
      message(Detector->ErrorString, this);
    }
  MW->onGDMLstatusChage(fOK); //update MW GUI
  ui->fGDML->setEnabled(fOK);

  //MW->Owindow->ResetViewport();  *** !!! need to reset all view ports
}

void DetectorAddOnsWindow::on_pbBackToSandwich_clicked()
{
  Detector->clearGDML();
  MW->onGDMLstatusChage(false); //update MW GUI
  MW->ReconstructDetector();
  ui->fGDML->setEnabled(false);
}

void DetectorAddOnsWindow::on_cobGridShape_currentIndexChanged(int index)
{
  ui->labGridSizeX->setText( index==0 ? "Size X:" : "Size:" );
  ui->fGridSizeY->setEnabled( index==0 );
  ui->fGridAngle->setEnabled( index!=1 );
  ui->fGridSides->setEnabled( index==2 );
}

void DetectorAddOnsWindow::on_pbRootWeb_clicked()
{
  QDesktopServices::openUrl( QUrl("ftp://root.cern.ch/root/doc/18Geometry.pdf") );
}

void DetectorAddOnsWindow::on_cbLightguideAutoshape_toggled(bool checked)
{
  ui->fLightguideShapeSize->setEnabled(!checked);
}

void DetectorAddOnsWindow::on_cobLightguideShape_currentIndexChanged(int index)
{
   ui->lLightguideSizeX->setText( index==0 ? "Size X:" : "Size:" );
   ui->fLightguideAngle->setEnabled( index!=1 );
   ui->fLightguideSizeY->setEnabled( index==0 );
}

void DetectorAddOnsWindow::on_pbHelpAddObjects_clicked()
{
  QString str = "'Top'' is the world container of the geometry.\n"
      "If the object has to be positioned inside the primary scintillator,\n"
      "  choose 'PrScint', not 'Top'!\n"
      "\nObjects are positioned using local coordinates of the container:\n"
      "(0,0,0) is the center of the container!\n"
      "The orientation angles are also given in respect to the container.\n"
      "\nThe object name has to be unique!\n"
      "\nIf the container has multiple instances (like PMs),\n"
      "  the object is positioned inside each container instance.";
  message(str, this);
}

void DetectorAddOnsWindow::on_cbPrScint_clicked()
{
  Detector->fPrScintCont = ui->cbPrScint->isChecked();
  on_pbReconstructDetector_clicked();
}

void DetectorAddOnsWindow::on_cbPrScint_toggled(bool checked)
{
    MW->setPrScintLabelText( checked ? "PrScint container" : "Primary scintillator");
}

void DetectorAddOnsWindow::on_pbAddMask_clicked()
{
   MaskClass m;
   Detector->Masks.append(m);
   ui->sbMask->setValue( Detector->Masks.size()-1 );
   UpdateGUI();
}

void DetectorAddOnsWindow::on_pbRemoveMask_clicked()
{
  int im = ui->sbMask->value();
  if (im > Detector->Masks.size()-1)
    {
      ui->sbMask->setValue(0);
      UpdateGUI();
      return;
    }

  Detector->Masks.remove(im);
  int i = im - 1;
  if (i < 0) i = 0;
  ui->sbMask->setValue( i );
  UpdateGUI();
  DetectorAddOnsWindow::on_pbMaskConfirmChanges_clicked();
}

void DetectorAddOnsWindow::on_sbMask_valueChanged(int arg1)
{
    if (arg1 > Detector->Masks.size()-1 && arg1 != 0)
      {
        ui->sbMask->setValue( Detector->Masks.size()-1 );
        return; //update on_change
      }

    UpdateGUI();
}

void DetectorAddOnsWindow::on_cobMaskContainer_activated(const QString &arg1)
{
  ui->leoMaskContainer->setText(arg1);
}
