#ifndef DETECTORADDONSWINDOW_H
#define DETECTORADDONSWINDOW_H

#include <QMainWindow>
#include "classes.h"
#include "detectorclass.h"

class MainWindow;
struct LightguidePropertiesStructure;
class InterfaceToHolesScript;
class InterfaceToAddObjScript;

namespace Ui {
  class DetectorAddOnsWindow;
}

class DetectorAddOnsWindow : public QMainWindow
{
  Q_OBJECT
  
public:
  explicit DetectorAddOnsWindow(MainWindow *parent, DetectorClass* detector);
  ~DetectorAddOnsWindow();

  void UpdateGUI(); //update gui controls
  void ClearDAMaterialCOBs();
  void AddDAMaterialToCOBs(QString str);
  void RemoveMaterial(int imat);
  void SetTab(int tab);

  void UpdateObjectNames();

  //Script interface object for MaskHoles
  InterfaceToHolesScript *siHoles;
  InterfaceToAddObjScript *AddObjScriptInterface;

  void LoadMaskHoles(const QString fileName);
  void setForbidUpdateValues(bool flag) {ForbidUpdateValues = flag;}

  void UpdateDummyPMindication();

  void setDisbaleAllAdditionalObjects(bool flag);

  void HighlightVolume(QString VolName);
  void ShowObject();
  bool isNameInUse(QString name);
  void UpdateMaterialInCobs(int imat);
private slots:
  void on_pbMaskLoad_clicked();

  void on_pbReconstructDetector_clicked();

  void on_pbMaskDelete_clicked();

  void on_pbUpdateWaveguideIndication_clicked();

  void on_tabwidWaveguideUpperLower_currentChanged(int index);

  void on_cobWaveguideOpeningsShape_currentIndexChanged(int index);

  void on_pbConvertToDummies_clicked();

  void on_sbDummyPMindex_valueChanged(int arg1);

  void on_pbDeleteDummy_clicked();

  void on_pbConvertDummy_clicked();

  void on_pbUpdateDummy_clicked();

  void on_pbCreateNewDummy_clicked();

  void on_sbDummyType_valueChanged(int arg1);

  void on_pbLoadDummyPMs_clicked();

  void on_pbObjectConfirm_clicked();



  void on_pbRemoveObject_clicked();

  void on_pbAddNewObject_clicked();

  void on_pbUpdateObjectCollection_clicked();

  void on_pbUpdateObjectsIndication_clicked();

  void on_sbObjectNumber_valueChanged(int arg1);

  void on_lwAddGeoObjects_clicked(const QModelIndex &index);

  void on_cbDisableAllAddedObjects_toggled(bool checked);

  void on_pbConvertAllToPMs_clicked();

  void on_cobObjectContainer_activated(int index);

  void on_cobObjectShape_currentIndexChanged(int index);

  void on_pbShowObject_clicked();

  void on_leObjectName_editingFinished();

  void on_pbCopyObject_clicked();

  void on_pbWaveguideConfirmChanges_clicked();

  void on_pbMaskConfirmChanges_clicked();

  void on_tableWidget_cellChanged(int row, int column);

  void on_pbHolesScript_clicked();

  void on_pbGridUpdateIndication_clicked();

  void on_sbGrid_valueChanged(int arg1);

  void on_pbRemoveGrid_clicked();

  void on_pbAddGrid_clicked();

  void on_pbGridsAllDisable_clicked();

  void on_cbGridEnabled_clicked();

  void on_pbGridConfirm_clicked();

  void on_ledGridAbsProb_editingFinished();

  void on_cobGridContainer_activated(int index);

  void on_pbUseScriptToAddObj_clicked();

  void on_pbRemoveAllObjects_clicked();

  void on_pbCopyThisObjToClipboard_clicked();

  void on_pbSaveTGeo_clicked();

  void on_pbLoadTGeo_clicked();

  void on_pbBackToSandwich_clicked();

  void on_cobGridShape_currentIndexChanged(int index);

  void on_pbRootWeb_clicked();

  void on_cbLightguideAutoshape_toggled(bool checked);

  void on_cobLightguideShape_currentIndexChanged(int index);

  void on_pbHelpAddObjects_clicked();

  void on_cbPrScint_clicked();

  void on_cbPrScint_toggled(bool checked);

  void on_pbAddMask_clicked();

  void on_pbRemoveMask_clicked();

  void on_sbMask_valueChanged(int arg1);

  void on_cobMaskContainer_activated(const QString &arg1);

private:
  Ui::DetectorAddOnsWindow *ui;
  MainWindow* MW;
  DetectorClass* Detector;

  bool ForbidUpdateValues;
  bool TMPignore;
  bool AddedButNotConfirmed;

  void EditFinish();
  void ConvertDummyToPM(int idpm);

public slots:
  void HolesScriptSuccess();
  void AddObjScriptSuccess();
  void UpdateAddObjIndicationAfterScript(int res); //0-no overlaps, 1-overlaps detected, 2-overlaps not checked
};

#endif // DETECTORADDONSWINDOW_H
