#include "exampleswindow.h"
#include "ui_exampleswindow.h"
#include "mainwindow.h"
#include "windownavigatorclass.h"
#include "geometrywindowclass.h"
#include "outputwindow.h"
#include "reconstructionwindow.h"
#include "sensorlrfs.h"
#include "materialinspectorwindow.h"
#include "detectoraddonswindow.h"
#include "globalsettingsclass.h"
//Qt
#include <QtGui>
#include <QFileDialog>
#include <QDialog>

ExamplesWindow::ExamplesWindow(QWidget *parent, MainWindow *mw) :
  QMainWindow(parent),
  ui(new Ui::ExamplesWindow)
{  
  MW = mw;
  ui->setupUi(this);
  this->setFixedSize(this->size());

  Qt::WindowFlags windowFlags = (Qt::Window | Qt::CustomizeWindowHint);
  windowFlags |= Qt::WindowCloseButtonHint;
  this->setWindowFlags( windowFlags );

  MainPointer = -1;
  SubPointer = -1;
  ExamplePointer = -1;
  MainTypeNames.resize(0);
  SubTypeNames.resize(0);
  Examples.resize(0);
  ListedExamples.resize(0);

  ExamplesWindow::BuildExampleRecord();
  //ExamplesWindow::PrepareLoadLastSettings();
}

ExamplesWindow::~ExamplesWindow()
{  
  //qDebug()<<"destructor of ExamplesWindow started";
  delete ui;
}

void ExamplesWindow::setShowExamplesOnStartStatus(bool show)
{
  ui->cbDoNotShowExamplesOnStart->setChecked(!show);
}

void ExamplesWindow::on_cbDoNotShowExamplesOnStart_toggled(bool checked)
{
  MW->ShowExamplesOnStart = !checked;
}

bool ExamplesWindow::event(QEvent *event)
{
  if (!MW->WindowNavigator) return QMainWindow::event(event);

  if (event->type() == QEvent::Hide)
    {
      MW->WindowNavigator->HideWindowTriggered("examples");
    }
  if (event->type() == QEvent::Show)
    {
      MW->WindowNavigator->ShowWindowTriggered("examples");
    }

  return QMainWindow::event(event);
}

void ExamplesWindow::BuildExampleRecord()
{
//  qDebug()<<"    Reading examples tree";
//  if (!QDir("EXAMPLES").exists())
//    {
//      qDebug()<<"!--Examples dir not found!";
//      return;
//    }

  QString fileName = MW->GlobSet->ExamplesDir + "/ExamplesTree.txt";
  QFile file(fileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
      {
          qDebug("!--Cannot find or open ExamplesTree.txt");
          return;
      }

  QTextStream in(&file);

  while(!in.atEnd())
      {
        QString line = in.readLine();
        QStringList fields = line.split(":", QString::SkipEmptyParts);
        if (fields.size()<2) continue;

        if (fields[0] == "define")
          {
            if (fields.size() == 2)
              {
                //defining main type
                ExamplesWindow::AddNewMainType(fields[1].simplified());
              }
            else if (fields.size() == 3)
              {
                ExamplesWindow::AddNewSubType(fields[1].simplified(), fields[2].simplified());
              }
            else
              {
                //wrong number of fields!
                qDebug()<<"!--Wrong number of fields found while processing ExamplesTree.txt"<<line;
              }
          }
        else if (fields[0] == "add")
          {
            if (fields.size() != 5 )
              {
                //wrong number of fields!
                qDebug()<<"!--Wrong number of fields found while processing ExamplesTree.txt: "<<line;
              }
            else
              {
                ExamplesWindow::AddNewExample(fields[1].simplified(), fields[2].simplified(), fields[3], fields[4]);
              }

          }
    }

  file.close();

  //filling ui data
  for (int i=0; i<MainTypeNames.size(); i++) ui->lwMainType->addItem(MainTypeNames[i]);
}

void ExamplesWindow::AddNewMainType(QString name)
{
  for (int i=0; i<MainTypeNames.size(); i++)
     if (name == MainTypeNames[i])
       {
          qDebug()<<"!--Attempt to register already existent MainTypeName during ExampleTree build";
          return;
       }
  MainTypeNames.append(name);
  SubTypeNames.resize(SubTypeNames.size()+1);
  SubTypeNames.last().resize(0);
}

void ExamplesWindow::AddNewSubType(QString maintype, QString subtype)
{
  for (int i=0; i<MainTypeNames.size(); i++)
     if (maintype == MainTypeNames[i])
       {
         //it is indeed a registered maintype
         //qDebug()<<maintype<<"->"<<i;
         for (int j=0; j<SubTypeNames[i].size(); j++)
           if (subtype == SubTypeNames[i][j])
             {
               qDebug()<<"Attempt to register already existent SubTypeName during ExampleTree build:"<<subtype;
               return;
             }

         SubTypeNames[i].append(subtype);
         //qDebug()<<"adding"<<subtype<<"for"<<maintype<<"for index="<<i;
         return;
       }
  qDebug()<<"!--Attempt to register subtype for a non-existent MainType during ExampleTree build:"<<maintype;
  return;
}

void ExamplesWindow::AddNewExample(QString maintype, QString subtype, QString filename, QString description)
{
  for (int i=0; i<MainTypeNames.size(); i++)
     if (maintype == MainTypeNames[i])
       {
         //it is indeed a registered maintype
//         qDebug()<<maintype<<"->"<<i;
         for (int j=0; j<SubTypeNames[i].size(); j++)
           if (subtype == SubTypeNames[i][j])
             {
               //it is indeed a registered subtype
//               qDebug()<<maintype<<subtype<<j;

               Examples.append(DetectorExampleStructure(i, j, filename, description));
               return;
             }

         qDebug()<<"!--Attempt to register example for a non-existent SubType during ExampleTree build:"<<subtype;
         return;
       }
  qDebug()<<"!--Attempt to register example for a non-existent MainType during ExampleTree build:"<<maintype;
  return;
}

void ExamplesWindow::on_lwMainType_currentRowChanged(int currentRow)
{
   if (currentRow == -1) return;

   MainPointer = currentRow;
   //qDebug()<<currentRow<<SubTypeNames[MainPointer].size();

   SubPointer = -1;
   ExamplePointer = -1;
   ui->lwSubType->clear();
   ui->lwExample->clear();
   ui->teExampleDescription->clear();
   for (int i=0; i<SubTypeNames[MainPointer].size(); i++) ui->lwSubType->addItem(SubTypeNames[MainPointer][i]);

   if (SubTypeNames[MainPointer].size() == 1) ui->lwSubType->setCurrentRow(0);
}

void ExamplesWindow::on_lwSubType_currentRowChanged(int currentRow)
{
   if (currentRow == -1) return;

   SubPointer = currentRow;

   ExamplePointer = -1;
   ui->lwExample->clear();
   ui->teExampleDescription->clear();

   //building list of examples and showing them
   ListedExamples.clear();
   for (int i=0; i<Examples.size(); i++)
     {
       if (Examples[i].MainType != MainPointer) continue;
       if (Examples[i].SubType != SubPointer) continue;

       //adding this one
       ListedExamples.append(i);
       ui->lwExample->addItem(Examples[i].filename);
     }

   if (ListedExamples.size() == 1) ui->lwExample->setCurrentRow(0);
}

void ExamplesWindow::on_lwExample_currentRowChanged(int currentRow)
{
  if (currentRow == -1)
    {
      ui->pbLoadExample->setEnabled(false);
      return;
    }

  ui->pbLoadExample->setEnabled(true);
  ExamplePointer = currentRow;
  ui->teExampleDescription->setText(Examples[ListedExamples[currentRow]].Description);
}

void ExamplesWindow::on_pbLoadExample_clicked()
{
  if (ExamplePointer == -1)
    {
      message("Example is not selected!", this);
      return;
    }
  this->close();
  MW->GeometryDrawDisabled = true;

  QString filename = MW->GlobSet->ExamplesDir + "/" + Examples[ListedExamples[ExamplePointer]].filename;

  LoadAllConfig(filename);
  MW->show();
  MW->raise();
  MW->GeometryWindow->show();
  MW->GeometryWindow->raise();

  MW->GeometryDrawDisabled = false;
  MW->ShowGeometry();
}

void ExamplesWindow::on_lwExample_doubleClicked(const QModelIndex &index)
{
  //ui->lwExample->setCurrentRow(index);
  //qDebug()<<"index:"<<ui->lwExample->currentRow();
  ui->lwExample->setCurrentRow(index.row());

  ExamplesWindow::on_pbLoadExample_clicked();
}

//-----------------------------------------------------------------------------------------

//Save / Load tools

bool ExamplesWindow::SaveAllConfig(QString fileName, bool DetConstructor, bool SimSettings, bool ReconstrSettings)
{
  QJsonObject json;
  SaveAllConfig(json, DetConstructor, SimSettings, ReconstrSettings);

  SaveJsonToFile(json, fileName);
  return true;
}

bool ExamplesWindow::SaveAllConfig(QJsonObject &json, bool DetConstructor, bool SimSettings, bool ReconstrSettings)
{
  if (DetConstructor) MW->writeDetectorToJson(json);
  if (SimSettings) MW->writeSimSettingsToJson(json, true); //true - write both point and particle jsons
  if (ReconstrSettings) MW->Rwindow->writeToJson(json, true); //true - include LRFs, gui and options for all algorithms

  MW->writeExtraGuiToJson(json);

  int versionNumber = ANTS2_VERSION;
  json["ANTS2build"] = versionNumber;
  return true;
}

bool ExamplesWindow::LoadAllConfig(QString fileName, bool DetConstructor, bool SimSettings, bool ReconstrSettings, QJsonObject *jsout)
{
  QFile loadFile(fileName);
  if (!loadFile.open(QIODevice::ReadOnly))
    {
      message("Error: Cannot open file "+fileName, this);
      return false;
    }
  QByteArray saveData = loadFile.readAll();
  QJsonDocument loadDoc(QJsonDocument::fromJson(saveData));
  QJsonObject json = loadDoc.object();

  if (json.isEmpty())
    {
      message("File does not contain json object:\nWrong file or unsupported old format.", this);
      return false;
    }
  if (jsout) *jsout = json; //for global script mode

  return LoadAllConfig(json, DetConstructor, SimSettings, ReconstrSettings);
}

bool ExamplesWindow::LoadAllConfig(QJsonObject &json, bool DetConstructor, bool SimSettings, bool ReconstrSettings, bool fKeepData)
{
  if (json.isEmpty())
    {
      message("Json is empty!", this);
      return false;
    }

  if (!fKeepData) MW->ClearData();

  //int build = 0; parseJson(json, "ANTS2build", build); if (build <= 886) old
  if (DetConstructor)
    {
      if (json.contains("DetectorConfig"))
        {
          //qDebug() << ">>> Loading detector settings...";
          MW->readDetectorFromJson(json);
        }
      else qWarning() << "||| Json does not contain detector settings!";
    }
  if (SimSettings)
    {
      if (json.contains("SimulationConfig"))
        {
          //qDebug() << ">>> Loading simulation settings...";
          MW->readSimSettingsFromJson(json);
        }
      else qWarning() << "||| Json does not contain simulation settings!";
    }
  MW->SensLRF->clear(MW->PMs->count()); //
  if (ReconstrSettings)
    {
      if (json.contains("ReconstructionConfig"))
        {
          //qDebug() << ">>> Loading reconstruction settings...";
          MW->Rwindow->readFromJson(json);
        }
      else qWarning() << "||| Json does not contain reconstruction settings!";
    }

  MW->DAwindow->UpdateObjectNames();
  MW->CheckPresenseOfSecScintillator();
  MW->MIwindow->SetMaterialSelection(0); //refresh indication
  MW->Owindow->ResetViewport();

  MW->readExtraGuiFromJson(json);

  //qDebug() << ">>> Load done";
  return true;
}

void ExamplesWindow::on_pbSaveSessings_clicked()
{
  QString fileName = QFileDialog::getSaveFileName(this,"Save configuration", MW->GlobSet->LastOpenDir, "Json files (*.json)");
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  QFileInfo file(fileName);
  if(file.suffix().isEmpty()) fileName += ".json";
  ExamplesWindow::SaveAllConfig(fileName, !ui->cbSkipDetectorConstructor->isChecked(), !ui->cbSkipSimConfig->isChecked(), !ui->cbSkipRecConfig->isChecked());
}

void ExamplesWindow::on_pbQuickSave_clicked()
{
  ExamplesWindow::QuickSave();
}

void ExamplesWindow::QuickSave()
{  
  // ExamplesWindow::SaveAllConfig("LAST/QuickSave.json");

  //QString configDir = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation)+"/ants2";
  //if (!QDir(configDir).exists()) QDir().mkdir(configDir);
  QString fileName = MW->GlobSet->QuicksaveDir + "/QuickSave.json";
  ExamplesWindow::SaveAllConfig(fileName);
}

void ExamplesWindow::on_pbLoadSettings_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load configuration", MW->GlobSet->LastOpenDir, "All files (*.*)");
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  MW->GeometryDrawDisabled = true;
  ExamplesWindow::LoadAllConfig(fileName, !ui->cbSkipDetectorConstructor->isChecked(), !ui->cbSkipSimConfig->isChecked(), !ui->cbSkipRecConfig->isChecked());
  MW->GeometryDrawDisabled = false;
  this->close();
  if (MW->GeometryWindow->isVisible()) MW->ShowGeometry(); 
}

void ExamplesWindow::on_pbQuickLoad_clicked()
{
  ExamplesWindow::QuickLoad();
}

void ExamplesWindow::QuickLoad()
{
  //QString configDir = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation)+"/ants2";
  QString fileName = MW->GlobSet->QuicksaveDir + "/QuickSave.json";

  MW->GeometryDrawDisabled = true;  
  ExamplesWindow::LoadAllConfig(fileName);
  MW->GeometryDrawDisabled = false;

  this->close();
  if (MW->GeometryWindow->isVisible()) MW->ShowGeometry();
}
