#include "genericscriptwindowclass.h"
#include "ui_genericscriptwindowclass.h"
#include "highlighterclass.h"
#include "completingtexteditclass.h"
#include "scriptinterfaces.h"
#include "interfacetoglobscript.h"
#include "functions.h"


#include "QTextStream"
#include <QSplitter>
#include <QFileDialog>
#include <QDebug>
#include <QMetaMethod>
#include <QPainter>
#include <QPlainTextEdit>
#include <QStringListModel>
#include <QShortcut>
#include <QScriptValueIterator>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QLineEdit>

QStringList getCustomCommandsOfObject(QObject *obj, QString ObjName, bool fWithArguments = false)
{  
  QStringList commands;
  int methods = obj->metaObject()->methodCount();
  for (int i=0; i<methods; i++)
    {
      const QMetaMethod &m = obj->metaObject()->method(i);
      bool fSlot   = (m.methodType() == QMetaMethod::Slot);
      bool fPublic = (m.access() == QMetaMethod::Public);
      QString commCandidate;
      if (fSlot && fPublic)
        {
          if (m.name() == "deleteLater") continue;
          if (m.name() == "help") continue;
          if (ObjName.isEmpty()) commCandidate = m.name();
          else commCandidate = ObjName + "." + m.name();

          if (fWithArguments)
            {              
              commCandidate += "(";
              int args = m.parameterCount();
              for (int i=0; i<args; i++)
                {
                  commCandidate += m.parameterTypes().at(i) + " " + m.parameterNames().at(i);
                  if (i!=args-1) commCandidate += ", ";
                }
              commCandidate += ")";
            }          
          if (commands.isEmpty() || commands.last() != commCandidate)
             commands << commCandidate;
        }
    }
  return commands;
}

GenericScriptWindowClass::GenericScriptWindowClass(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::GenericScriptWindowClass)
{
    tmpIgnore = false;
    ShowEvalResult = true;
    ui->setupUi(this);
    ui->pbStop->setVisible(false);
    LocalScript = "//no external script provided!";    
    //interfacer = 0;
    Example = "";
    ExamplesDir = "";
    ScriptHistory.append("");
    HistoryPosition = 0;

    engine.setProcessEventsInterval(200);
    fEngineIsRunning = false;

    QPixmap rm(16, 16);
    rm.fill(Qt::transparent);
    QPainter b(&rm);
    b.setBrush(QBrush(Qt::red));
    b.drawEllipse(0, 0, 14, 14);
    RedIcon = new QIcon(rm);

    //creating text edit with completition for the script window
    completitionModel = 0;
    cteScript = new CompletingTextEditClass(this);
    cteScript->setLineWrapMode(QTextEdit::NoWrap);
    QObject::connect(cteScript, SIGNAL(requestHelp(QString)), this, SLOT(onF1pressed(QString)));
    completer = new QCompleter(this);
    QStringList words;
    completer->setModel(createCompletitionModel(words));
    completer->setCompletionMode(QCompleter::UnfilteredPopupCompletion);
    //completer->setCompletionMode(QCompleter::PopupCompletion);
    completer->setFilterMode(Qt::MatchContains);
    completer->setModelSorting(QCompleter::CaseSensitivelySortedModel);
    completer->setCaseSensitivity(Qt::CaseSensitive);
    completer->setWrapAround(false);
    cteScript->setCompleter(completer);
    SetScript(&LocalScript); //to be reset from external
    highlighter = new HighlighterClass(cteScript->document());

    QSplitter* sp = new QSplitter();
    sp->setOrientation(Qt::Vertical);
    sp->setChildrenCollapsible(false);
      //
    QSplitter* hor = new QSplitter();
    cteScript->setMinimumHeight(25);
    hor->addWidget(cteScript); //already defined
    splHelp = new QSplitter();
    splHelp->setOrientation(Qt::Vertical);
    splHelp->setChildrenCollapsible(false);

    trwHelp = new QTreeWidget();
    //trwHelp->setVisible(false);
    trwHelp->setColumnCount(1);
    trwHelp->setHeaderLabel("Unit.Function");
    QObject::connect(ui->pbHelp, SIGNAL(toggled(bool)), splHelp, SLOT(setVisible(bool)));
    QObject::connect(trwHelp, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(onFunctionDoubleClicked(QTreeWidgetItem*,int)));
    QObject::connect(trwHelp, SIGNAL(itemClicked(QTreeWidgetItem*,int)), this, SLOT(onFunctionClicked(QTreeWidgetItem*,int)));
    splHelp->addWidget(trwHelp);

    pteHelp = new QPlainTextEdit();
    pteHelp->setReadOnly(true);
    pteHelp->setMinimumHeight(20);
    //pteHelp->setMaximumHeight(50);
    splHelp->addWidget(pteHelp);

    trwJson = new QTreeWidget();
    trwJson->setVisible(false);
    trwJson->setColumnCount(2);
    trwJson->setMinimumHeight(30);
    QStringList strl;
    strl << "Key" << "Type";
    trwJson->setHeaderLabels(strl);
    QObject::connect(trwJson, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(onKeyDoubleClicked(QTreeWidgetItem*,int)));
    QObject::connect(trwJson, SIGNAL(itemClicked(QTreeWidgetItem*,int)), this, SLOT(onKeyClicked(QTreeWidgetItem*,int)));
    splHelp->addWidget(trwJson);

    leFind = new QLineEdit("Find");
    splHelp->addWidget(leFind);
    leFind->setMinimumHeight(20);
    leFind->setMaximumHeight(20);
    QObject::connect(leFind, SIGNAL(textChanged(QString)), this, SLOT(onFindTextChanged(QString)));

    QList<int> sizesHelp;
    sizesHelp << 500 << 100 << 300 << 25;
    splHelp->setSizes(sizesHelp);
    splHelp->setVisible(false);

    hor->addWidget(splHelp);

    sp->addWidget(hor);
      //
    pteOut = new QPlainTextEdit();
    pteOut->setMinimumHeight(25);
    pteOut->setReadOnly(true);
    QPalette p = pteOut->palette();
     p.setColor(QPalette::Active, QPalette::Base, QColor(240,240,240));
     p.setColor(QPalette::Inactive, QPalette::Base, QColor(240,240,240));
    pteOut->setPalette(p);
    pteHelp->setPalette(p);
    QList<int> sizes;
    sizes << 800 << 75;
    sp->setSizes(sizes);

    sp->addWidget(pteOut);
    ui->centralwidget->layout()->removeItem(ui->horizontalLayout);
    ui->centralwidget->layout()->addWidget(sp);
    ui->centralwidget->layout()->addItem(ui->horizontalLayout);

    trwJson->header()->resizeSection(0, 200);

    connect(cteScript, SIGNAL(textChanged()), this, SLOT(cteScript_textChanged()));

    //shortcuts
    QShortcut* Run = new QShortcut(QKeySequence("Ctrl+Return"), this);
    connect(Run, SIGNAL(activated()), this, SLOT(on_pbRunScript_clicked()));

    QShortcut* HistBefore = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_PageUp), this);
    connect(HistBefore, SIGNAL(activated()), this, SLOT(onRequestHistoryBefore()));

    QShortcut* HistAfter = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_PageDown), this);
    connect(HistAfter, SIGNAL(activated()), this, SLOT(onRequestHistoryAfter()));
}

GenericScriptWindowClass::~GenericScriptWindowClass()
{
  //qDebug() << "Destructor of script window called";  
  tmpIgnore = true;
  delete ui;
  //qDebug() << "ui deleted, deleting interface object";

  //qDebug() << interfacer;  
  //if (interfacer) delete interfacer;
  for (int i=0; i<interfaces.size(); i++)
    delete interfaces[i];
  interfaces.clear();

  delete RedIcon;
  //qDebug() << "interfacer deleted";
}

void GenericScriptWindowClass::SetInterfaceObject(QObject *interfaceObject, QString name)
{
  //registering global object
      //QScriptValue math = engine.globalObject().property("Math"); //replaced with own math module
  QScriptValue obj = engine.newQObject(interfaceObject, QScriptEngine::QtOwnership);//, opts);
  if(name.isEmpty())
    { // empty name means the main module
      engine.setGlobalObject(obj);      
          // registering math module
          //engine.globalObject().setProperty("Math", math); //system module, does not need to be deleted
      // registering service object
      QObject* coreObj = new CoreInterfaceClass(this);
      QScriptValue coreVal = engine.newQObject(coreObj, QScriptEngine::QtOwnership);
      QString coreName = "core";
      engine.globalObject().setProperty(coreName, coreVal);
      interfaces.append(coreObj);  //CORE OBJECT IS FIRST in interfaces!
      interfaceNames.append(coreName);
      //registering math module
      QObject* mathObj = new MathInterfaceClass();
      QScriptValue mathVal = engine.newQObject(mathObj, QScriptEngine::QtOwnership);
      QString mathName = "math";
      engine.globalObject().setProperty(mathName, mathVal);
      interfaces.append(mathObj);  //SERVICE OBJECT IS FIRST in interfaces!
      interfaceNames.append(mathName);

      // populating help for main, math and core units
      trwHelp->clear();
      fillHelper(interfaceObject, "", "Global object functions");
          //fillHelper(math, "Math", "Math functions (arguments are not shown!) and constants");
      fillHelper(coreObj, "core", "Core object functions");
      fillHelper(mathObj, "math", "Basic mathematics: wrapper for std double functions");
      trwHelp->expandItem(trwHelp->itemAt(0,0));      
    }
  else
    { // name is not empty - this is one of the secondary modules
      engine.globalObject().setProperty(name, obj);
      fillHelper(interfaceObject, name, "");

      //special "needs" of particular interface objects
      if ( dynamic_cast<InterfaceToHistD*>(interfaceObject) || dynamic_cast<InterfaceToGraphs*>(interfaceObject)) //"graph" or "hist"
        QObject::connect(interfaceObject, SIGNAL(RequestDraw(TObject*,QString,bool)), this, SLOT(onRequestDraw(TObject*,QString,bool)));
    }

  interfaces.append(interfaceObject);
  interfaceNames.append(name);

  //connecting abort request from main interface to serviceObj
  int index = interfaceObject->metaObject()->indexOfSignal("AbortScriptEvaluation(QString)");
  if (index != -1)
    //qDebug() << interfaceObject->metaObject()->method(index).name();
    //QObject::connect(interfaceObject, "2AbortScriptEvaluation(QString)", interfaces.first(), SLOT(abort(QString)));  //1-slot, 2-signal
    QObject::connect(interfaceObject, "2AbortScriptEvaluation(QString)", this, SLOT(abortEvaluation(QString)));  //1-slot, 2-signal


  //auto-read list of public slots for highlighter
  QStringList functions, constants;
  for (int i=0; i<interfaces.size(); i++)
    functions << getCustomCommandsOfObject(interfaces[i], interfaceNames[i], false);

//  QScriptValueIterator it(math);
//  while (it.hasNext())
//    {
//      it.next();
//      QString toAdd = "Math."+it.name();
//      if (it.value().isFunction()) functions << toAdd;
//      else constants << toAdd;
//    }

  highlighter->setCustomCommands(functions, constants);

  for (int i=0; i<functions.size(); i++) functions[i] += "()";
  completer->setModel(createCompletitionModel(functions+constants));
}

void GenericScriptWindowClass::SetScript(QString* text)
{
    Script = text;

    tmpIgnore = true;
      cteScript->clear();
      cteScript->append(*text);
    tmpIgnore = false;
}

void GenericScriptWindowClass::SetCustomCommands(QStringList functions, QStringList constants)
{
    highlighter->setCustomCommands(functions, constants);
}

void GenericScriptWindowClass::SetTitle(QString title)
{
    this->setWindowTitle(title);
}

void GenericScriptWindowClass::SetExample(QString example)
{
  if (example.startsWith("#Load "))
    {
      //Load example from file
      example.remove("#Load ");
      QString fileName = ExamplesDir + "/" + example;
      QFile file(fileName);
      if (!file.open(QIODevice::ReadOnly))
        {
          message("Failed to open example file:\n"+example, this);
          return;
        }
      QTextStream in(&file);
      Example = in.readAll();
    }
  else Example = example;
}

void GenericScriptWindowClass::SetStarterDir(QString starter)
{
  LibScripts = starter;

  QString fileName = LibScripts + "/ScriptHistory.json";
  QFile file(fileName);
  if (file.open(QIODevice::ReadOnly))
    {
      QJsonParseError err;
      QJsonDocument loadDoc(QJsonDocument::fromJson(file.readAll(), &err));
      if (err.error == QJsonParseError::NoError)
        {
            QJsonObject json = loadDoc.object();
            QJsonArray ar =  json["ScriptHistory"].toArray();
            QStringList strl;
            for (int i=0; i<ar.size(); i++) strl<<ar[i].toString();

            if (strl.last().isEmpty()) strl.removeLast();
            if (!strl.isEmpty())
            {
                ScriptHistory = strl;
                HistoryPosition = ScriptHistory.size()-1;
            }
        }
    }
  ScriptHistory.append(*Script);
}

void GenericScriptWindowClass::SetStarterDir(QString ScriptDir, QString WorkingDir, QString ExamplesDir)
{
  LastOpenDir = WorkingDir;
  this->ExamplesDir = ExamplesDir;
  SetStarterDir(ScriptDir);
}

void GenericScriptWindowClass::ReportError(QString error, int line)
{   
  //pteOut->appendHtml(error);
   error = "<font color=\"red\">Error:</font><br>" + error;
   pteOut->appendHtml( error );
   //pteOut->moveCursor(QTextCursor::Start);
   HighlightErrorLine(line);
}

void GenericScriptWindowClass::HighlightErrorLine(int line)
{
  //highlight line with error
  QTextBlock block = cteScript->document()->findBlockByLineNumber(line-1);
  int loc = block.position();
  QTextCursor cur = cteScript->textCursor();
  cur.setPosition(loc);
  int length = block.text().split("\n").at(0).length();
  cur.movePosition(cur.Right, cur.KeepAnchor, length);

  QTextCharFormat tf = block.charFormat();
  tf.setBackground(QBrush(Qt::yellow));
  QTextEdit::ExtraSelection es;
  es.cursor = cur;
  es.format = tf;

  QList<QTextEdit::ExtraSelection> esList;
  esList << es;
  cteScript->setExtraSelections(esList);
}

void GenericScriptWindowClass::ShowText(QString text)
{   
  pteOut->appendHtml(text);
  qApp->processEvents();
}

void GenericScriptWindowClass::ClearText()
{
  pteOut->clear();
  qApp->processEvents();
}

void GenericScriptWindowClass::on_pbRunScript_clicked()
{   
   fAborted = false;

   QString corrected = cteScript->document()->toPlainText().replace("\\","/");
   *Script = ScriptHistory.last() = corrected; //override in case one of the history items is selected and run button is clicked

   //running InitOnRun method (if defined) for all defined interfaces
   for (int i=0; i<interfaces.size(); i++)
     {
//       bool result;
//       QObject* obj = interfaces[i];
//       qDebug() << obj->metaObject()->indexOfMethod("InitOnRun()");
//       if (obj->metaObject()->indexOfMethod("InitOnRun()") != -1)
//         {
//           qDebug() << "On run activated";
//           //bool test
//           QMetaObject::invokeMethod(obj, "InitOnRun", Qt::DirectConnection,
//                               Q_RETURN_ARG(bool, result)//,
//                               //Q_ARG(bool, test)
//                               );
//         }
         BaseInterfaceClass* bi = dynamic_cast<BaseInterfaceClass*>(interfaces[i]);
         if (bi)
           {
             if (!bi->InitOnRun())
               {
                 message("Init failed for unit: "+interfaceNames.at(i), this);
                 return;
               }
           }
     }

   //qDebug() << "Init on Start done";
   pteOut->clear();
   GenericScriptWindowClass::ShowText("Processing script");

   //syntax check
   QScriptSyntaxCheckResult check = QScriptEngine::checkSyntax(*Script);
   if(check.state() != QScriptSyntaxCheckResult::Valid)
     {
       int lineNum = check.errorLineNumber();
       GenericScriptWindowClass::ReportError("Syntax error!", lineNum);       
       return;
     }

   //script history handling
   saveScriptHistory();
   ScriptHistory.append(ScriptHistory.last());  //this will be the next "current" item
   if (ScriptHistory.size()>50) ScriptHistory.removeFirst();
   HistoryPosition = ScriptHistory.size()-1;


   ui->pbStop->setVisible(true);
   ui->pbRunScript->setVisible(false);
   qApp->processEvents();

   fEngineIsRunning = true;
   QScriptValue scriptreturn = engine.evaluate(*Script);
   fEngineIsRunning = false;

   ui->pbStop->setVisible(false);
   ui->pbRunScript->setVisible(true);

   if(engine.hasUncaughtException())
   {
       //Script has uncaught exception
       int lineNum = engine.uncaughtExceptionLineNumber();
       QString message = engine.uncaughtException().toString();
       //qDebug() << "Error message:" << message;
       //QString backtrace = engine.uncaughtExceptionBacktrace().join('\n');
       //qDebug() << "backtrace:" << backtrace;
       GenericScriptWindowClass::ReportError("Script error: "+message, lineNum);
   }
   else
   {
       //success
       QString result = scriptreturn.toString();
       //qDebug() << "Script returned:" << result;
       if (!fAborted)
         {
            if (ShowEvalResult) ShowText("Script evaluation result:\n"+result);
            else ShowText("Script evaluation: success");
         }
       else
         {
           //ShowText("Aborted!");
         }
       emit success(result); //bad name here :) could be not successful at all, but still want to trigger
       ui->pbRunScript->setIcon(QIcon()); //clear red icon
     }

   engine.collectGarbage();
}

void GenericScriptWindowClass::abortEvaluation(QString message)
{
  if (fAborted || !fEngineIsRunning) return;
  fAborted = true;
  emit onAbort();
  engine.abortEvaluation();  
  message = "<font color=\"red\">"+ message +"</font><br>";
  ShowText(message);

  // going through registered units and requesting abort
  for (int i=0; i<interfaces.size(); i++)
    {
      BaseInterfaceClass* bi = dynamic_cast<BaseInterfaceClass*>(interfaces[i]);
      if (bi) bi->ForceStop();
    }
}

void GenericScriptWindowClass::onF1pressed(QString text)
{
  //qDebug() << "F1 requested for:"<<text;
  ui->pbHelp->setChecked(true);

  trwHelp->collapseAll();
  trwHelp->clearSelection();

  QList<QTreeWidgetItem*> list;
  list = trwHelp->findItems(text, Qt::MatchContains | Qt::MatchRecursive, 0);

  for (int i=0; i<list.size(); i++)
    {
      QTreeWidgetItem* item = list[i];
      do
        {
          trwHelp->expandItem(item);
          item = item->parent();
        }
      while (item);
      trwHelp->setCurrentItem(list[i], 0, QItemSelectionModel::Select);
      trwHelp->setCurrentItem(list[i], 1, QItemSelectionModel::Select);
    }

  if (list.size() == 1)
    emit trwHelp->itemClicked(list.first(), 0);
}

void GenericScriptWindowClass::on_pbStop_clicked()
{
  if (fEngineIsRunning)
    {
      qDebug() << "Stop button pressed!";
      ShowText("Sending stop signal...");
      abortEvaluation("Aborted by user!");
      fEngineIsRunning = false;
      qApp->processEvents();
    }
}

void GenericScriptWindowClass::on_pbHelp_clicked()
{
    //message(HelpText, this);
}

void GenericScriptWindowClass::on_pbLoad_clicked()
{   
  QString starter = (LibScripts.isEmpty()) ? LastOpenDir : LibScripts;
  QString fileName = QFileDialog::getOpenFileName(this, "Load script", starter, "Text files (*.txt);;All files (*.*)"); //""
  if (fileName.isEmpty()) return;

  QFile file(fileName);
  if(!file.open(QIODevice::ReadOnly | QFile::Text))
    {
      message("Could not open: " + fileName, this);
      return;
    }

  QTextStream in(&file);
  *Script = in.readAll();
  file.close();

  tmpIgnore = true;
  cteScript->clear();
  cteScript->append(*Script);
  tmpIgnore = false;

  ui->pbRunScript->setIcon(*RedIcon);
}

void GenericScriptWindowClass::on_pbSave_clicked()
{
  QString starter = (LibScripts.isEmpty()) ? LastOpenDir : LibScripts;
  QString fileName = QFileDialog::getSaveFileName(this,"Save script", starter, "Text files (*.txt);;All files (*.*)");
  if (fileName.isEmpty()) return;

  QFileInfo fileInfo(fileName);
  if(fileInfo.suffix().isEmpty()) fileName += ".txt";

  QFile outputFile(fileName);
  outputFile.open(QIODevice::WriteOnly);

  if(!outputFile.isOpen())
    {
      message("Unable to open file " +fileName+ " for writing!", this);
      return;
    }

  QTextStream outStream(&outputFile);
  outStream << *Script;
  outputFile.close();
}

void GenericScriptWindowClass::cteScript_textChanged()
{
    if (tmpIgnore) return;
    *Script = cteScript->document()->toPlainText();
    ScriptHistory.last() = *Script;

    HighlightErrorLine(0);

    ui->pbRunScript->setIcon(*RedIcon);
}

void GenericScriptWindowClass::on_pbExample_clicked()
{
  tmpIgnore = true;
  *Script = Example;
  cteScript->clear();
  cteScript->append(Example);
  tmpIgnore = false;

  ui->pbRunScript->setIcon(*RedIcon);
}

void GenericScriptWindowClass::fillHelper(QObject* obj, QString module, QString helpText)
{
  QStringList functions = getCustomCommandsOfObject(obj, module, true);
  functions.sort();
  //qDebug() << functions;
  if (module != "Math") cteScript->functionList << functions;

  QTreeWidgetItem *objItem = new QTreeWidgetItem(trwHelp);
  objItem->setText(0, module);
  QFont f = objItem->font(0);
  f.setBold(true);
  objItem->setFont(0, f);
  //objItem->setBackgroundColor(QColor(0, 0, 255, 80));
  objItem->setToolTip(0, helpText);
  for (int i=0; i<functions.size(); i++)
  {
      QTreeWidgetItem *fItem = new QTreeWidgetItem(objItem);
      fItem->setText(0, functions[i]);

      QString retVal = "Help not provided";
      QString funcNoArgs = functions[i].remove(QRegExp("\\((.*)\\)"));
      if (!module.isEmpty()) funcNoArgs.remove(0, module.length()+1); //remove name.
      if (obj->metaObject()->indexOfMethod("help(QString)") != -1)
        {
          QMetaObject::invokeMethod(obj, "help", Qt::DirectConnection,
                              Q_RETURN_ARG(QString, retVal),
                              Q_ARG(QString, funcNoArgs)
                              );
        }
      fItem->setToolTip(0, retVal);
  }
}

//void GenericScriptWindowClass::fillHelper(QScriptValue &val, QString module, QString helpText)
//{
//  QStringList functions, constants;
//  QScriptValueIterator it(val);
//  while (it.hasNext())
//    {
//      it.next();
//      QString toAdd = "Math."+it.name();
//      if (it.value().isFunction()) functions << toAdd;
//      else constants << toAdd;
//    }

//  functions.sort();
//  constants.sort();
//  QTreeWidgetItem *objItem = new QTreeWidgetItem(trwHelp);
//  objItem->setText(0, module);
//  QFont f = objItem->font(0);
//  f.setBold(true);
//  objItem->setFont(0, f);
//  objItem->setToolTip(0, helpText);
//  for (int i=0; i<functions.size(); i++)
//    {
//      QTreeWidgetItem *fItem = new QTreeWidgetItem(objItem);
//      fItem->setText(0, functions[i]+"()");
//    }
//  for (int i=0; i<constants.size(); i++)
//    {
//      QTreeWidgetItem *fItem = new QTreeWidgetItem(objItem);
//      fItem->setText(0, constants[i]);
//    }
//}

QAbstractItemModel *GenericScriptWindowClass::createCompletitionModel(QStringList words)
{
  if (completitionModel) delete completitionModel;
  completitionModel = new QStringListModel(words, completer);
  return completitionModel;
}

void GenericScriptWindowClass::updateJsonTree()
{
  trwJson->clear();

  for (int i=0; i<interfaces.size(); i++)
    {
      InterfaceToConfig* inter = dynamic_cast<InterfaceToConfig*>(interfaces[i]);
      if (!inter) continue;

      QJsonObject json = inter->Config;
      QJsonObject::const_iterator it;
      for (it = json.begin(); it != json.end(); it++)
        {
           QString key = it.key();
           QTreeWidgetItem *TopKey = new QTreeWidgetItem(trwJson);
           TopKey->setText(0, key);

           const QJsonValue &value = it.value();
           TopKey->setText(1, getDesc(value));

           if (value.isObject())
             fillSubObject(TopKey, value.toObject());
           else if (value.isArray())
             fillSubArray(TopKey, value.toArray());
        }
    }
}

void GenericScriptWindowClass::fillSubObject(QTreeWidgetItem *parent, const QJsonObject &obj)
{
  QJsonObject::const_iterator it;
  for (it = obj.begin(); it != obj.end(); it++)
    {
      QTreeWidgetItem *item = new QTreeWidgetItem(parent);
      item->setText(0, it.key());
      QJsonValue value = it.value();
      item->setText(1, getDesc(value));

      if (value.isObject())
        fillSubObject(item, value.toObject());
      else if (value.isArray())
        fillSubArray(item, value.toArray());
    }
}

void GenericScriptWindowClass::fillSubArray(QTreeWidgetItem *parent, const QJsonArray &arr)
{
  for (int i=0; i<arr.size(); i++)
    {
      QTreeWidgetItem *item = new QTreeWidgetItem(parent);
      QString str = "[" + QString::number(i)+"]";
      item->setText(0, str);
      QJsonValue value = arr.at(i);
      item->setText(1, getDesc(value));

      if (value.isObject())
        fillSubObject(item, value.toObject());
      else if (value.isArray())
        fillSubArray(item, value.toArray());
    }
}

QString GenericScriptWindowClass::getDesc(const QJsonValue &ref)
{
  if (ref.isBool()) return "bool";
  if (ref.isDouble()) return "number";
  if (ref.isString()) return "string";
  if (ref.isObject()) return "obj";
  if (ref.isArray())
    {
      int size = ref.toArray().size();
      QString ret = "array[" + QString::number(size)+"]";
      return ret;
    }
  return "undefined";
}

void GenericScriptWindowClass::onFunctionDoubleClicked(QTreeWidgetItem *item, int /*column*/)
{
  QString text = item->text(0);
//  int args;
//  if (text.endsWith("()")) args = 0;
//  else args = text.count(",") + 1;
//  text.remove(QRegExp("\\(.*\\)"));
//  QString addon = "";
//  if (args>0) addon += " __ ";
//  if (args>1)
//    for (int i=0; i<args-1; i++) addon += ", __ ";
//  text += "(" + addon + ")\n";
  cteScript->insertPlainText(text);
}

void GenericScriptWindowClass::onFunctionClicked(QTreeWidgetItem *item, int /*column*/)
{
  pteHelp->clear();
  pteHelp->appendPlainText("-->"+item->text(0)+":");
  pteHelp->appendPlainText(item->toolTip(0));

  if (item->text(0) == "config.Replace(QString Key, QVariant val)")
    {
      trwJson->setVisible(true);
      updateJsonTree();
    }
  else
    {
      trwJson->setVisible(false);
    }
}

void GenericScriptWindowClass::onKeyDoubleClicked(QTreeWidgetItem *item, int /*column*/)
{
  QString str = item->text(0);
  while (item->parent())
    {
      item = item->parent();
      QString keyName = item->text(0);
      if (str.startsWith("[")) str = keyName + str;
      else str = keyName + "." + str;
    }

  str = "\""+str+"\"";
  cteScript->insertPlainText(str);
}

void GenericScriptWindowClass::onKeyClicked(QTreeWidgetItem* /*item*/, int /*column*/)
{
  //trwJson->resizeColumnToContents(column);
}

void GenericScriptWindowClass::onFindTextChanged(const QString &arg1)
{
  QTreeWidget* tw = trwHelp;
  if (trwJson->isVisible())
    tw = trwJson;

  tw->collapseAll();
  tw->clearSelection();
  if (arg1.length()<3) return;

  QList<QTreeWidgetItem*> list;
  list = tw->findItems(arg1, Qt::MatchContains | Qt::MatchRecursive, 0);

  for (int i=0; i<list.size(); i++)
    {
      QTreeWidgetItem* item = list[i];
      do
        {
          tw->expandItem(item);
          item = item->parent();
        }
      while (item);
      tw->setCurrentItem(list[i], 0, QItemSelectionModel::Select);
      tw->setCurrentItem(list[i], 1, QItemSelectionModel::Select);
    }
}

void GenericScriptWindowClass::onRequestHistoryBefore()
{
  if (HistoryPosition == 0) return;

  HistoryPosition--;
  tmpIgnore = true;
  cteScript->clear();
  cteScript->setText(ScriptHistory[HistoryPosition]);
  tmpIgnore = false;
}

void GenericScriptWindowClass::onRequestHistoryAfter()
{
  if (HistoryPosition == ScriptHistory.size()-1) return;

  HistoryPosition++;
  tmpIgnore = true;
  cteScript->clear();
  cteScript->setText(ScriptHistory[HistoryPosition]);
  tmpIgnore = false;
}

void GenericScriptWindowClass::closeEvent(QCloseEvent *e)
{
  if (fEngineIsRunning)
    {
      e->ignore();
      return;
    }
  saveScriptHistory();
}

void GenericScriptWindowClass::saveScriptHistory()
{
  QString fileName = LibScripts + "/ScriptHistory.json";
  //qDebug() << "-->Saving script history:"<<fileName;
  QFile file(fileName);
  if (file.open(QIODevice::WriteOnly))
  {
      for (int i=0; i<ScriptHistory.size()-1; i++)
        if (ScriptHistory[i] == ScriptHistory.last())
          {
            ScriptHistory.removeAt(i);
            break;
          }
      QJsonArray ar = QJsonArray::fromStringList(ScriptHistory);
      QJsonObject json;
      json["ScriptHistory"] = ar;

      QJsonDocument saveDoc(json);
      file.write(saveDoc.toJson());
  }
}
