#ifndef GENERICSCRIPTWINDOWCLASS_H
#define GENERICSCRIPTWINDOWCLASS_H

#include <QMainWindow>
#include <QScriptEngine>
#include <QThread>

class HighlighterClass;
class QAbstractItemModel;
class QCompleter;
class QStringListModel;
class CompletingTextEditClass;
class QPlainTextEdit;
class QTreeWidget;
class QTreeWidgetItem;
class QSplitter;
class QLineEdit;
class TObject;

namespace Ui {
class GenericScriptWindowClass;
}

class GenericScriptWindowClass : public QMainWindow
{
    Q_OBJECT

public:
    explicit GenericScriptWindowClass(QWidget *parent = 0);
    ~GenericScriptWindowClass();

     //configuration - can skip any of them!
    void SetInterfaceObject(QObject* interfaceObject, QString name = "");
    void SetScript(QString *text);
    void SetCustomCommands(QStringList functions, QStringList constants = QStringList());
    void SetLastOpenDir(QString lastOpenDir) {LastOpenDir = lastOpenDir;}
    void SetShowEvaluationResult(bool flag) {ShowEvalResult = flag;} //if false, window only reports "success", ptherwise eval result is shown
    void SetTitle(QString title);
    void SetExample(QString example);

    void SetStarterDir(QString starter);
    void SetStarterDir(QString ScriptDir, QString WorkingDir, QString ExamplesDir); //more advanced version, used by core in global script

      //can be used from outside to access output text edit and highlight a specific line
    void ReportError(QString error, int line = 0);   //0 - no line is highligted   
    void HighlightErrorLine(int line);    


    QScriptEngine engine;
    HighlighterClass* highlighter;

    QStringList ScriptHistory;
    int HistoryPosition;

    // specific directories
    QString LastOpenDir;
    QString LibScripts;
    QString ExamplesDir;

public slots:
    void ShowText(QString text); //shows text in the output box
    void ClearText(); //clears text in the output box
    void on_pbRunScript_clicked();    
    void abortEvaluation(QString message = "Aborted!");
    void onF1pressed(QString text);
    void onRequestHistoryBefore();
    void onRequestHistoryAfter();

private slots:    
    void on_pbStop_clicked();
    void on_pbHelp_clicked();
    void on_pbLoad_clicked();
    void on_pbSave_clicked();
    void on_pbExample_clicked();
    void cteScript_textChanged();        
    void onRequestDraw(TObject* obj, QString options, bool fFocus) {emit RequestDraw(obj, options, fFocus);}
    void onFunctionDoubleClicked(QTreeWidgetItem* item, int column);
    void onFunctionClicked(QTreeWidgetItem* item, int column);
    void onKeyDoubleClicked(QTreeWidgetItem* item, int column);
    void onKeyClicked(QTreeWidgetItem* item, int column);
    void onFindTextChanged(const QString &arg1);

private:
    Ui::GenericScriptWindowClass *ui;    
    QCompleter* completer;
    QStringListModel* completitionModel;
    QThread* EngineThread;

    QSplitter* splHelp;
    CompletingTextEditClass* cteScript;
    QPlainTextEdit* pteOut;
    QTreeWidget* trwHelp;
    QTreeWidget* trwJson;
    QPlainTextEdit* pteHelp;
    QLineEdit* leFind;
    QIcon* RedIcon;

    QString* Script;     //pointer to external script
    QString LocalScript; //if external not provided, this is the default script
    QString Example;

    QVector<QObject*> interfaces;
    QVector<QString> interfaceNames;

    bool tmpIgnore;
    bool ShowEvalResult;
    bool fEngineIsRunning;
    bool fAborted;

    QAbstractItemModel* createCompletitionModel(QStringList words);
    void updateJsonTree();
    void fillSubObject(QTreeWidgetItem* parent, const QJsonObject& obj);
    void fillSubArray(QTreeWidgetItem* parent, const QJsonArray& arr);
    QString getDesc(const QJsonValue &ref);
    void saveScriptHistory();
    void fillHelper(QObject* obj, QString module, QString helpText = "");  //fill help TreeWidget according to the data in the obj
    //void fillHelper(QScriptValue &val, QString module, QString helpText);

protected:
  virtual void closeEvent(QCloseEvent *e);

signals:
    void success(QString eval);
    void onStart();
    void onAbort();
    void RequestDraw(TObject* obj, QString options, bool fFocus);
};

#endif // GENERICSCRIPTWINDOWCLASS_H
