#include "globalsettingswindowclass.h"
#include "ui_globalsettingswindowclass.h"
#include "mainwindow.h"
#include "genericscriptwindowclass.h"
#include "globalsettingsclass.h"
#include "detectorclass.h"
#include "detectorsandwichclass.h"

//Qt
#include <QFileDialog>

GlobalSettingsWindowClass::GlobalSettingsWindowClass(MainWindow *parent) :
  QMainWindow(parent),
  ui(new Ui::GlobalSettingsWindowClass)
{
  MW = parent;
  GlobSet = MW->GlobSet;
  ui->setupUi(this);

  Qt::WindowFlags windowFlags = (Qt::Window | Qt::CustomizeWindowHint);
  windowFlags |= Qt::WindowCloseButtonHint;
  windowFlags |= Qt::WindowStaysOnTopHint;
  this->setWindowFlags( windowFlags );

  updateGUI();
}

GlobalSettingsWindowClass::~GlobalSettingsWindowClass()
{
  delete ui;
}

void GlobalSettingsWindowClass::updateGUI()
{
  ui->leLibPMtypes->setText(GlobSet->LibPMtypes);
  ui->leLibMaterials->setText(GlobSet->LibMaterials);
  ui->leLibParticleSources->setText(GlobSet->LibParticleSources);
  ui->leLibScripts->setText(GlobSet->LibScripts);

  ui->leWorkingDir->setText(GlobSet->TmpDir);

  ui->sbFontSize->setValue(GlobSet->FontSize);

  ui->rbNever->setChecked(GlobSet->NeverSaveOnExit);
  ui->rbAlways->setChecked(GlobSet->AlwaysSaveOnExit);
  ui->rbAskMe->setChecked(!GlobSet->NeverSaveOnExit && !GlobSet->AlwaysSaveOnExit);

  ui->cbOpenImageExternalEditor->setChecked(GlobSet->fOpenImageExternalEditor);

  ui->sbLogPrecision->setValue(GlobSet->TextLogPrecision);

  ui->sbNumBinsHistogramsX->setValue(GlobSet->BinsX);
  ui->sbNumBinsHistogramsY->setValue(GlobSet->BinsY);
  ui->sbNumBinsHistogramsZ->setValue(GlobSet->BinsZ);
}

void GlobalSettingsWindowClass::on_pbgStyleScript_clicked()
{
  MW->extractGeometryOfScriptWindow();
  if (MW->GenScriptWindow) delete MW->GenScriptWindow;
  MW->GenScriptWindow = new GenericScriptWindowClass();
  MW->recallGeometryOfScriptWindow();

  //configure the script window and engine
  GStyleInterface  = new  InterfaceToGStyleScript() ; //deleted by the GenScriptWindow
  MW->GenScriptWindow->SetInterfaceObject(GStyleInterface);
  //QStringList coms;
  //coms << "Tube" << "Box";
  //MW->GenScriptWindow->SetCustomCommands(coms);
  QString HelpText = "  Avalable commands: \nsee https://root.cern.ch/root/html/TStyle.html - all commands starting from \"Set\"\n";
  MW->GenScriptWindow->SetExample("");
  MW->GenScriptWindow->SetShowEvaluationResult(false); //do not show "undefined"
  MW->GenScriptWindow->SetTitle("Script to set ROOT's gStyle");
  MW->GenScriptWindow->SetScript(&GlobSet->RootStyleScript);
  MW->GenScriptWindow->SetStarterDir(MW->GlobSet->LibScripts);

  //define what to do on evaluation success
  //connect(MW->GenScriptWindow, SIGNAL(success(QString)), this, SLOT(HolesScriptSuccess()));
  //if needed. connect signals of the interface object with the required slots of any ANTS2 objects
  MW->GenScriptWindow->show();
}

void GlobalSettingsWindowClass::on_pbChoosePMtypeLib_clicked()
{
  QString starter = (GlobSet->LibPMtypes.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibPMtypes;
  QString dir = QFileDialog::getExistingDirectory(this, "Select directory for PM types",starter,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
  if (dir.isEmpty()) return;
  GlobSet->LibPMtypes = dir;
  ui->leLibPMtypes->setText(dir);
}
void GlobalSettingsWindowClass::on_leLibPMtypes_editingFinished()
{
  GlobSet->LibPMtypes = ui->leLibPMtypes->text();
}

void GlobalSettingsWindowClass::on_pbChooseMaterialLib_clicked()
{
  QString starter = (GlobSet->LibMaterials.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibMaterials;
  QString dir = QFileDialog::getExistingDirectory(this, "Select directory for materials",starter,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
  if (dir.isEmpty()) return;
  GlobSet->LibMaterials = dir;
  ui->leLibMaterials->setText(dir);
}
void GlobalSettingsWindowClass::on_leLibMaterials_editingFinished()
{
  GlobSet->LibMaterials = ui->leLibMaterials->text();
}

void GlobalSettingsWindowClass::on_pbChooseParticleSourcesLib_clicked()
{
  QString starter = (GlobSet->LibParticleSources.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibParticleSources;
  QString dir = QFileDialog::getExistingDirectory(this, "Select directory for particle sources",starter,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
  if (dir.isEmpty()) return;
  GlobSet->LibParticleSources = dir;
  ui->leLibParticleSources->setText(dir);
}
void GlobalSettingsWindowClass::on_leLibParticleSources_editingFinished()
{
  GlobSet->LibParticleSources = ui->leLibParticleSources->text();
}

void GlobalSettingsWindowClass::on_pbChooseScriptsLib_clicked()
{
  QString starter = (GlobSet->LibScripts.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibScripts;
  QString dir = QFileDialog::getExistingDirectory(this, "Select directory for scripts",starter,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
  if (dir.isEmpty()) return;
  GlobSet->LibScripts = dir;
  ui->leLibScripts->setText(dir);
}
void GlobalSettingsWindowClass::on_leLibScripts_editingFinished()
{
  GlobSet->LibScripts = ui->leLibScripts->text();
}

void GlobalSettingsWindowClass::on_pbChangeWorkingDir_clicked()
{
  QString starter = GlobSet->TmpDir;
  QString dir = QFileDialog::getExistingDirectory(this, "Select working directory (temporary files are saved here)",starter,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
  if (dir.isEmpty()) return;
  GlobSet->TmpDir = dir;
  QDir::setCurrent(GlobSet->TmpDir);
  ui->leWorkingDir->setText(dir);
}
void GlobalSettingsWindowClass::on_leWorkingDir_editingFinished()
{
   QString dir = ui->leWorkingDir->text();
   if ( QDir(dir).exists() )
     {
       GlobSet->TmpDir = dir;
       QDir::setCurrent(GlobSet->TmpDir);
       ui->leWorkingDir->setText(dir);
     }
   else
     {
       ui->leWorkingDir->setText(GlobSet->TmpDir);
       message("Dir does not exists!", this);
     }
}

void GlobalSettingsWindowClass::on_sbFontSize_editingFinished()
{
    GlobSet->FontSize = ui->sbFontSize->value();
    MW->setFontSizeAllWindows(GlobSet->FontSize);
}

void GlobalSettingsWindowClass::on_rbAlways_toggled(bool checked)
{
  GlobSet->AlwaysSaveOnExit = checked;
}

void GlobalSettingsWindowClass::on_rbNever_toggled(bool checked)
{
  GlobSet->NeverSaveOnExit = checked;
}


void GlobalSettingsWindowClass::on_cbOpenImageExternalEditor_clicked(bool checked)
{
  GlobSet->fOpenImageExternalEditor = checked;
}

void GlobalSettingsWindowClass::on_sbLogPrecision_editingFinished()
{
  GlobSet->TextLogPrecision = ui->sbLogPrecision->value();
}

void GlobalSettingsWindowClass::on_sbNumBinsHistogramsX_editingFinished()
{
  GlobSet->BinsX = ui->sbNumBinsHistogramsX->value();
}

void GlobalSettingsWindowClass::on_sbNumBinsHistogramsY_editingFinished()
{
  GlobSet->BinsY = ui->sbNumBinsHistogramsY->value();
}

void GlobalSettingsWindowClass::on_sbNumBinsHistogramsZ_editingFinished()
{
  GlobSet->BinsZ = ui->sbNumBinsHistogramsZ->value();
}

void GlobalSettingsWindowClass::on_ledPM_thickness_editingFinished()
{
    if (!MW->Detector->isGDMLempty()) return;

    MW->Detector->Sandwich.widthPMs = ui->ledPM_thickness->text().toDouble();
    MW->ReconstructDetector();
}

void GlobalSettingsWindowClass::on_ledPMdummy_thickness_editingFinished()
{
    if (!MW->Detector->isGDMLempty()) return;
    if (MW->Detector->PMdummies.isEmpty()) return;

    MW->Detector->Sandwich.widthDummyPMs = ui->ledPMdummy_thickness->text().toDouble();
    MW->ReconstructDetector();
}

void GlobalSettingsWindowClass::on_pbChoosePMtypeLib_customContextMenuRequested(const QPoint &/*pos*/)
{
  if (ui->leLibPMtypes->text().isEmpty()) return;
  QDesktopServices::openUrl(QUrl("file///:"+ui->leLibPMtypes->text(), QUrl::TolerantMode));
}

void GlobalSettingsWindowClass::on_pbChooseMaterialLib_customContextMenuRequested(const QPoint &/*pos*/)
{
  if (ui->leLibMaterials->text().isEmpty()) return;
  QDesktopServices::openUrl(QUrl("file:///"+ui->leLibMaterials->text(), QUrl::TolerantMode));
}

void GlobalSettingsWindowClass::on_pbChooseParticleSourcesLib_customContextMenuRequested(const QPoint &/*pos*/)
{
  if (ui->leLibParticleSources->text().isEmpty()) return;
  QDesktopServices::openUrl(QUrl("file:///"+ui->leLibParticleSources->text(), QUrl::TolerantMode));
}

void GlobalSettingsWindowClass::on_pbChooseScriptsLib_customContextMenuRequested(const QPoint &/*pos*/)
{
  if (ui->leLibScripts->text().isEmpty()) return;
  QDesktopServices::openUrl(QUrl("file:///"+ui->leLibScripts->text(), QUrl::TolerantMode));
}

void GlobalSettingsWindowClass::on_pbOpen_clicked()
{
  QString what;
  switch (ui->comboBox->currentIndex())
    {
    case 0:
      what = GlobSet->AntsBaseDir;
      break;
    case 1:
      what = GlobSet->ConfigDir;
      break;
    case 2:
      what = GlobSet->LibScripts;
      break;
    case 3:
      what = GlobSet->LastOpenDir;
      break;
    case 4:
      what = GlobSet->ExamplesDir;
      break;
    }
  QDesktopServices::openUrl(QUrl("file:///"+what, QUrl::TolerantMode));
}
