#ifndef LRFWINDOW_H
#define LRFWINDOW_H

#include <QMainWindow>
#include <QVector>
#include <QTimer>

class MainWindow;
class LRF2;
class PMsensorGroup;
class SensorLocalCache;
class QListWidgetItem;
class Viewer2DarrayObject;
class myQGraphicsView;

class TF2;
class TGraph2D;
class TGraph;
class TF1;
class TMultiGraph;
class TH2D;
class EventsDataClass;

namespace Ui {
  class LRFwindow;
}

class LRFwindow : public QMainWindow
{
  Q_OBJECT
  
public:
    explicit LRFwindow(QWidget* parent, MainWindow *mw, EventsDataClass *eventsDataHub);
    ~LRFwindow();

    void SetProgress(int val);

    void onBusyOn();
    void onBusyOff();
    void SaveLRFDialog(QWidget *wid);
    void LoadLRFDialog(QWidget *wid);
    void showLRF();

    void writeToJson(QJsonObject &json) const;
    void loadJSON(QJsonObject &json);

    void updateLRFinfo(QJsonObject &json);
    void updateGroupsInfo();

    void updateIterationIndication();
    void SaveIteration(int iter, QString fileName);

    QString LastError;

public slots:
  void LRF_ModuleReadySlot(bool ready);
  void on_pbMakeLRF_clicked();
  QString MakeLRFs(); //returns error string if error
  void DoMakeGroupLRF(int igroup);
  void paintLRFonDialog(QPointF *pos);  
  void onProgressReportReceived(int progress);

private slots:
  void contextMenuHistory(const QPoint &pos);

  void historyClickedTimeout();

  void on_pbStopLRFmake_clicked();

  void on_pbUpdateGUI_clicked();

  void on_pbSaveLRFs_clicked();

  void on_pbLoadLRFs_clicked();

  void on_pbShow2D_clicked();

  void on_sbPMnoButons_editingFinished();

  void on_sbPMnumberSpinButtons_valueChanged(int arg1);

  void on_sbPMnoButons_valueChanged(int arg1);

  void on_cb_data_2_clicked();

  void on_cb_diff_2_clicked();

  void on_pbShow1Ddistr_clicked();

  void on_pbShowSensorGroups_clicked();

  void on_pbUpdateHistory_clicked();

  void on_leCurrentLRFs_editingFinished();

  void on_pbClearHistory_clicked();

  void on_lwLRFhistory_itemClicked(QListWidgetItem *item);

  void on_pushButton_clicked();

  void on_pushButton_2_clicked();

  void on_pbClearTmpHistory_clicked();

  void on_pbShowLRFMouse_clicked();

  void on_pbSaveIterations_clicked();

  void on_pbExpand_clicked();

  void on_pbShrink_clicked();

  void on_pbRadialToText_clicked();

  void on_pbShowSensorGains_clicked();

  void on_pbAdjustOnlyGains_clicked();

  void on_pbCompareRadials_clicked();

  void on_led_compression_k_editingFinished();

  void on_cbUseGroupping_toggled(bool checked);

protected:
    bool event(QEvent *event);

private:
    Ui::LRFwindow *ui;
    MainWindow *MW;
    EventsDataClass *EventsDataHub;
    QTimer timer;

    bool StopSignal;  
    bool tmpIgnore; //temporary ignore update flag
    int secondIter; //second item in history selected - it will be shown on draw lrfs as blue
    bool fOneClick;
    QString lastPlotted;

    TGraph2D *d2t;
    TF2 *f2d, *ff2d;
    TGraph2D *d2f;   
    TF1 *f1d, *ff1d;
    TMultiGraph* MGraph;
    TH2D* h2d;

    QDialog *LRFdialog;
    Viewer2DarrayObject *LRFviewObj;
    myQGraphicsView* gv;

    void drawRadial();

    bool checkWatchdogs();
};

#endif // LRFWINDOW_H
