//ANTS2 modules
#include "geometrywindowclass.h"
#include "graphwindowclass.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "lrfwindow.h"
#include "reconstructionwindow.h"
#include "sensorlrfs.h"
#include "pms.h"
#include "oneeventclass.h"
#include "materialinspectorwindow.h"
#include "outputwindow.h"
#include "guiutils.h"
#include "windownavigatorclass.h"
#include "exampleswindow.h"
#include "detectoraddonswindow.h"
#include "checkupwindowclass.h"
#include "gainevaluatorwindowclass.h"
#include "genericscriptwindowclass.h"
#include "credits.h"
#include "detectorclass.h"
#include "simulationmanager.h"
#include "pmtypeclass.h"
#include "reconstructionmanagerclass.h"
#include "globalsettingswindowclass.h"
#include "globalsettingsclass.h"
#include "opticaloverrideclass.h"
#include "phscatclaudiomodel.h"
#include "scatteronmetal.h"

//Qt
#include <QtGui>
#include <QMessageBox>
#include <QTableWidget>
#include <QFileDialog>
#include <QTimer>
#include <QtWidgets>
#include <QDate>

//Root
#include "TApplication.h"
#include "TGeoMatrix.h"
#include "TSystem.h"
#include "TVirtualGeoTrack.h"
#include "TH1D.h"
#include "TH1I.h"
#include "TH2D.h"
#include "TGraph2D.h"
#include "TMultiGraph.h"
#include "TColor.h"
#include "TROOT.h"
#include "TRandom2.h"
#include "TGeoTrack.h"

#ifdef ANTS_FANN
#include "neuralnetworkswindow.h"
#endif

#ifdef ANTS_FLANN
#include "nnmoduleclass.h"
#endif

// See MainWindowInits.cpp for the constructor!

MainWindow::~MainWindow()
{
    //qDebug()<<"<-Staring destructor for MainWindow...";  
    if (hist1) delete hist1;
    if (hist2) delete hist2;
    if (hist2D) delete hist2D;
    if (histSecScint) delete histSecScint;
    if (histScan) delete histScan;
    if (TMPmultiGraph) delete TMPmultiGraph;
    //qDebug()<< "  Tmp hists deleted";

    //if (RandGen) delete RandGen;
    //qDebug() << "  Random generator deleted";
    //if (ReconstructionManager) delete ReconstructionManager;
    //qDebug() << "  Reconstruction manager deleted";
    //if (Detector) delete Detector;
    //qDebug() << "  Detector object deleted";
    if (SimManager) delete SimManager;
    //qDebug() << "  Simulation manager deleted";
    clearGeoMarkers();
    if (EnergyVector.size()>0)
     {
       for (int i=0; i<EnergyVector.size(); i++) delete EnergyVector[i];
       EnergyVector.resize(0);
     }

    //qDebug()<<"  GeoMarkers cleared";
    delete GlobSet;
    //qDebug()<<"  Global settings object deleted";
    delete ui;
    //qDebug()<<"  <-User interface deleted";
}

void MainWindow::onBusyOn()
{
  ui->tabConfig->setEnabled(false);
  ui->tabExpdata->setEnabled(false);
  ui->tabTests->setEnabled(false);
  ui->twOption->setEnabled(false);
  ui->twSourcePhotonsParticles->setEnabled(false);
  ui->fExportImport->setEnabled(false);
  ui->fSaveExport->setEnabled(false);
}

void MainWindow::onBusyOff()
{
  ui->tabConfig->setEnabled(true);
  ui->tabExpdata->setEnabled(true);
  ui->tabTests->setEnabled(true);
  ui->twOption->setEnabled(true);
  ui->twSourcePhotonsParticles->setEnabled(true); 
  ui->fExportImport->setEnabled(true);
  ui->fSaveExport->setEnabled(true);
  ui->pbStopScan->setChecked(false); //in case stop was triggered
  ui->pbStopScan->setEnabled(false);
}

void MainWindow::startRootUpdate()
{
  RootUpdateTimer->start();
}

void MainWindow::stopRootUpdate()
{
  RootUpdateTimer->stop();
}

void MainWindow::ClearData()
{   
   //   qDebug() << "---Before clear";
   if (msBox) return; //not yet confirmed clear status - this was called due to focus out.
   if (fSimDataNotSaved)
     {
       msBox = new QMessageBox();
       msBox->setText("Potential data loss!");
       msBox->setInformativeText("Unsaved simulation data are about to be deleted!\nSave simulation data?");
       msBox->setStandardButtons(QMessageBox::Save | QMessageBox::Discard);
       msBox->setDefaultButton(QMessageBox::Save);
       int ret = msBox->exec();
       delete msBox;
       msBox = 0;

       fSimDataNotSaved = false;
       if (ret == QMessageBox::Save) SaveSimulationDataTree();
     }
   fSimDataNotSaved = false;

   EventsDataHub->clear();
   if (EnergyVector.size()>0)
    {
      for (int i=0; i<EnergyVector.size(); i++) delete EnergyVector[i];
      EnergyVector.resize(0);
    }

#ifdef ANTS_FLANN
   ReconstructionManager->KNNmodule->Filter.clear();
#endif

   ui->leoLoadedEvents->setText("");
   ui->leoTotalLoadedEvents->setText("");
   ui->lwLoadedSims->clear();

   clearGeoMarkers();

   if (!EventsDataHub->isEmpty()) Rwindow->CheckAvailableEventData();

   Owindow->SiPMpixels.clear();
   Owindow->RefreshData();

   WindowNavigator->ResetAllProgressBars();
   ui->pbStopScan->setChecked(false);
   ui->pbStopScan->setEnabled(false);
   ui->pbExportDeposition->setEnabled(false);
   ui->pbGenerateLight->setEnabled(false);

   DeleteLoadedEvents(false); //to clear loaded exper/sim data
   //   qDebug()  << "Clear done";
}

void MainWindow::closeEvent(QCloseEvent *)
{   
   ShutDown = true;
   ui->pbAddparticleToActive->setFocus(); //to finish editing whatever QLineEdit the user can be in - they call on_editing_finish
   //qDebug() << "<-MainWindow close event received";
   GraphWindow->close();
   GraphWindow->ClearDrawObjects(); //to avoid any attempts to redraw deleted objects
   //saving ANTS master-configuration file
   //qDebug()<<"--Saving ANTS configuration";
   GlobSet->SaveANTSconfiguration();

   bool DoSaveConfiguration = !GlobSet->NeverSaveOnExit;  //if force skip, do not do it

   if (DoSaveConfiguration)
     {
       if (!GlobSet->AlwaysSaveOnExit) //then give the choice
         {
           QMessageBox *msgBox = new QMessageBox( this );
           msgBox->setWindowTitle("Session settings");
           msgBox->setText("Remember this session settings?");
           msgBox->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
           msgBox->setDefaultButton(QMessageBox::Yes);

           if (msgBox->exec() == QMessageBox::No) DoSaveConfiguration = false;
           if (msgBox) delete msgBox;
         }
     }
   if (DoSaveConfiguration) ELwindow->QuickSave();

   //if checked, save windows' status
   if (ui->actionSave_Load_windows_status_on_Exit_Init->isChecked())
     {
       //qDebug()<<"--Saving position/status of all windows";
       MainWindow::on_actionSave_position_and_stratus_of_all_windows_triggered();
     }
   //qDebug()<<"  <Stopping Root update timer-based cycle";
   RootUpdateTimer->stop();
   disconnect(RootUpdateTimer, SIGNAL(timeout()), this, SLOT(timerTimeout()));
   QThread::msleep(110);
   delete RootUpdateTimer;
   //qDebug()<<"        timer stopped and deleted";

   if (GenScriptWindow)
     {
       GenScriptWindow->close();
       //qDebug() << " <Deleting general script window";
       delete GenScriptWindow;
     }

#ifdef ANTS_FANN
   if (NNwindow)
     {
       //qDebug()<<"  <Deleting NeuralNetworksWindow";
       delete NNwindow->parent();
       NNwindow = 0;
     }
#endif

   if (lrfwindow) {
       //qDebug()<<"  <Deleting  lrf Window";
       delete lrfwindow->parent();
       lrfwindow = 0;
   }
   if (WindowNavigator) {
       //qDebug()<<"  <Deleting WindowNavigator";
       delete WindowNavigator->parent();
       WindowNavigator = 0;
   }
   if (GeometryWindow)
     {
       //qDebug()<<"  <Deleting Geometry Window";
       GeometryWindow->hide();
       delete GeometryWindow->parent();
       GeometryWindow = 0;
     }
   if (GraphWindow)
     {
       //qDebug()<<"  <Deleting Graph Window";
       GraphWindow->hide();
       delete GraphWindow->parent();
       GraphWindow = 0;
     }   
   if (GlobSetWindow)
     {
       //qDebug()<<"  <Deleting Settings Window";
       GlobSetWindow->hide();
       delete GlobSetWindow;
       GlobSetWindow = 0;
     }
   if (Rwindow)
     {
       //qDebug() << "  <Deleting Reconstruction window";
       Rwindow->close();
       delete Rwindow->parent();
       Rwindow = 0;
     }
   if (Owindow)
     {
       //qDebug() << "  <Deleting Output window";
       Owindow->close();
       delete Owindow->parent();
       Owindow = 0;
     }
   if (MIwindow)
     {
       //qDebug()<<"  <Deleting material inspector window";
       delete MIwindow->parent();
       MIwindow = 0;
     }
   if (ELwindow)
     {
       //qDebug()<<"  <Deleting examples window";
       ELwindow->close();
       delete ELwindow->parent();
       ELwindow = 0;
     }
   if (DAwindow)
     {
       //qDebug()<<"  <Deleting detector addon window";
       delete DAwindow;
       DAwindow = 0;
       //qDebug() << "   done";
     }
   if (CheckUpWindow)
     {
       //qDebug()<<"  <Deleting check up window";
       delete CheckUpWindow;
       CheckUpWindow = 0;
     }
   //Gain evaluation window is deleted in ReconstructionWindow destructor!
   //qDebug() << "  <-MainWindow close event: all done";
}

bool MainWindow::event(QEvent *event)
{
   if (!WindowNavigator) return QMainWindow::event(event);

   if (event->type() == QEvent::Hide)
     {
//       qDebug()<<"Main window HIDE event";
       WindowNavigator->HideWindowTriggered("main");
       //MainWindow::on_actionWindow_navigator_triggered();
       return true; //stop hadling
     }

   if (event->type() == QEvent::WindowStateChange)
     {
//       qDebug()<<"Main window STATECHANGE event";
//       qDebug()<<"Was requested by navigator:"<<WindowNavigator->isMainChangeExplicitlyRequested();
//       qDebug()<<"Is window currently minimized:"<<this->isMinimized();

       //note: after "minimize" is requested by any source, the WindowStateChange triggered AFTER the minimization

       if (WindowNavigator->isMainChangeExplicitlyRequested())
         {

           if (this->isMinimized()) WindowNavigator->HideWindowTriggered("main");
           else WindowNavigator->ShowWindowTriggered("main");
           return true;
         }
       else
         {
            if (this->isMinimized())
              {
                WindowNavigator->TriggerHideButton();
               // WindowNavigator->HideWindowTriggered("main");
              }
            else
              {
                WindowNavigator->TriggerShowButton();
               // WindowNavigator->ShowWindowTriggered("main");
                if (this->isVisible()) this->activateWindow();
              }
            return true;
         }
     }

   if (event->type() == QEvent::Show)
     {      
//       qDebug()<<"Main window SHOW event";
       WindowNavigator->ShowWindowTriggered("main");
       return true;
     }

   return QMainWindow::event(event);
}

void MainWindow::ShowGeometry(bool ActivateWindow, bool SAME, bool ColorUpdateAllowed)
// default:  ActivateWindow = true,  SAME = true,  ColorUpdateAllowed = true
{
    //qDebug()<<"  ----Showing geometry----"<<GeometryDrawDisabled;
    if (GeometryDrawDisabled) return;

    //setting this window as active pad in root
    //with or without activation (focussing) of this window
    if (ActivateWindow) GeometryWindow->ShowAndFocus(); //window is activated (focused)
    else GeometryWindow->SetAsActiveRootWindow(); //no activation in this mode

    //coloring volumes
    if (ColorUpdateAllowed)
      {
        if (ColorByMaterial) Detector->colorVolumes(1);
        else Detector->colorVolumes(0);
      }

    //top volume visibility
    if (ShowTop) Detector->GeoManager->SetTopVisible(true); // the TOP is generally invisible
    else Detector->GeoManager->SetTopVisible(false);

    //transparency setup    
    int totNodes = Detector->top->GetNdaughters();
    for (int i=0; i<totNodes; i++)
      {
        TGeoNode* thisNode = (TGeoNode*)Detector->top->GetNodes()->At(i);
        thisNode->GetVolume()->SetTransparency(0);
      }

    //making contaners visible
    Detector->top->SetVisContainers(true);

    //DRAW
    GeometryWindow->fNeedZoom = true;
    GeometryWindow->setHideUpdate(true);
    GeometryWindow->ClearRootCanvas();
    if (SAME)
      {
 //       qDebug()<<"keeping";
        Detector->top->Draw("SAME");
      }
    else
      {
 //       qDebug()<<"new";
        //GeometryWindow->ResetView();
        Detector->top->Draw("");
      }    
    GeometryWindow->PostDraw();

    //drawing dots
    MainWindow::ShowGeoMarkers();
    GeometryWindow->UpdateRootCanvas();
}

void MainWindow::clearGeoMarkers()
{
  for (int i=0; i<GeoMarkers.size(); i++) delete GeoMarkers[i];
  GeoMarkers.clear();
}

void MainWindow::ShowGeoMarkers()
{
  if (!GeoMarkers.isEmpty())
    {
      GeometryWindow->SetAsActiveRootWindow();

      for (int i=0; i<GeoMarkers.size(); i++)
        {
          GeoMarkerClass* gm = GeoMarkers[i];

          //overrides
          if (gm->Type == "Recon" || gm->Type == "Scan" || gm->Type == "Nodes")
            {
              gm->SetMarkerStyle(GeometryWindow->GeoMarkerStyle);
              gm->SetMarkerSize(GeometryWindow->GeoMarkerSize);
            }

          gm->Draw("same");
        }

      GeometryWindow->UpdateRootCanvas();
    }
}

void MainWindow::ExportGeometryImage(QString filename)
{  
   //root is focused on this window's canvas; canvas is cleaned too  
   GeometryWindow->SetAsActiveRootWindow();
   GeometryWindow->ClearRootCanvas();

   if (ShowTop) Detector->GeoManager->SetTopVisible(true); // the TOP is generally invisible
   else Detector->GeoManager->SetTopVisible(false);

   //setting colors
   if (ColorByMaterial) Detector->colorVolumes(1);
   else Detector->colorVolumes(0);

   //removing transparency
   int totNodes = Detector->top->GetNdaughters();
   for (int i=0; i<totNodes; i++)
     {
       TGeoNode* thisNode = (TGeoNode*)Detector->top->GetNodes()->At(i);
       thisNode->GetVolume()->SetTransparency(0);
     }
   Detector->top->Draw();

   //drawing tracks
   //GeoManager->DrawTracks();
   MainWindow::ShowTracks();

   //drawing dots
//   int dots = DotsTGeo.size();
//   if (dots>0)
//     {
//       for (int i=0; i<dots; i++)
//         {
//           Detector->GeoManager->SetCurrentPoint(DotsTGeo[i].r);
//           Detector->GeoManager->DrawCurrentPoint(DotsTGeo[i].color);
//         }
//     }
    MainWindow::ShowGeoMarkers();

    GeometryWindow->UpdateRootCanvas();

    GeometryWindow->SaveAs(filename);
}

void MainWindow::ShowGraphWindow()
{
  GraphWindow->ShowAndFocus();  
}

void MainWindow::ShowTracks()
{   
  if (GeometryDrawDisabled) return;

  GeometryWindow->SetAsActiveRootWindow();
  Detector->GeoManager->DrawTracks();
  GeometryWindow->UpdateRootCanvas();
}

void MainWindow::on_sbTimeBins_valueChanged(int /*arg1*/)
{   
    MainWindow::ClearData();
}

void MainWindow::on_ledTimeFrom_editingFinished()
{
  MainWindow::ClearData();
}

void MainWindow::on_ledTimeTo_editingFinished()
{
  MainWindow::ClearData();
}

void MainWindow::on_pbRebuildDetector_clicked()
{
  if (ui->cobTubeBox->currentIndex() < 2) ui->ledPolygonAngle->setText("0");
  if (DoNotUpdateGeometry) return; //if bulk update in progress

  //if sandwich uses default size, updating GUI sandwich layers
  if (ui->cbUseSameSizes->isChecked())
    for (int i=0; i<SandwichLayers.size(); i++)
      {
        SandwichLayers[i].Size1 = ui->ledSizeDiameter->text().toDouble();
        SandwichLayers[i].Size2 = ui->ledSizeDiameter_2->text().toDouble();
        SandwichLayers[i].Shape = ui->cobTubeBox->currentIndex();
        SandwichLayers[i].Sides = ui->sbPolygonSides->value();
        SandwichLayers[i].Angle = ui->ledPolygonAngle->text().toDouble();
      }
  //qDebug()<<"  <--Buiding detector";
  MainWindow::ReconstructDetector();
}

// -------materials: visualization--------------
void MainWindow::on_pbRefreshMaterials_clicked()
{
    MainWindow::UpdateMaterialListEdit();      
    bool tmpBool = DoNotUpdateGeometry;
    DoNotUpdateGeometry = true;
    //updating material selectors on the Detector tab
    ui->cobS1->clear();
    ui->cobS2->clear();
    ui->cobS3->clear();
    ui->cobS4->clear();
    ui->cobS5->clear();   
    ui->cobUW->clear();
    ui->cobLW->clear();
    ui->cobPS->clear();
    ui->cobSS->clear();
    ui->cobTOP->clear();
    ui->cobMatPM->clear();
    ui->cobMaterialForOverrides->clear();
    ui->cobMaterialTo->clear();
    ui->cobMaterialForWaveTests->clear();
    ui->cobMatPointSource->clear();
    ui->cobShowMaterial->clear();

    DAwindow->ClearDAMaterialCOBs();
    for (int i=0;i<MaterialCollection->size();i++) MainWindow::AddMaterialToCOBs(  (*MaterialCollection)[i]->name );
    MIwindow->UpdateActiveMaterials();
    DoNotUpdateGeometry = tmpBool; 
}

void MainWindow::AddMaterialToCOBs(QString s)
{
    ui->cobS1->addItem(s);
    ui->cobS2->addItem(s);
    ui->cobS3->addItem(s);
    ui->cobS4->addItem(s);
    ui->cobS5->addItem(s);
    ui->cobUW->addItem(s);
    ui->cobLW->addItem(s);
    ui->cobPS->addItem(s);
    ui->cobSS->addItem(s);
    ui->cobTOP->addItem(s);
    ui->cobMatPM->addItem(s);
    ui->cobMaterialForOverrides->addItem(s);
    ui->cobMaterialTo->addItem(s);
    ui->cobMaterialForWaveTests->addItem(s);
    ui->cobMatPointSource->addItem(s);
    ui->cobShowMaterial->addItem(s);

    DAwindow->AddDAMaterialToCOBs(s);    
    MIwindow->AddMatToCobs(s);
}

void MainWindow::RemoveMaterialFromCobs(int imat)
{
  bool tmpBool = DoNotUpdateGeometry;
  DoNotUpdateGeometry = true;

  ui->cobS1->removeItem(imat);
  ui->cobS2->removeItem(imat);
  ui->cobS3->removeItem(imat);
  ui->cobS4->removeItem(imat);
  ui->cobS5->removeItem(imat);
  ui->cobUW->removeItem(imat);
  ui->cobLW->removeItem(imat);
  ui->cobPS->removeItem(imat);
  ui->cobSS->removeItem(imat);
  ui->cobTOP->removeItem(imat);
  ui->cobMatPM->removeItem(imat);
  ui->cobMaterialForOverrides->removeItem(imat);
  ui->cobMaterialTo->removeItem(imat);
  ui->cobMaterialForWaveTests->removeItem(imat);
  ui->cobMatPointSource->removeItem(imat);
  ui->cobShowMaterial->removeItem(imat);

  MIwindow->RemoveMaterial(imat);
  DAwindow->RemoveMaterial(imat);

  DoNotUpdateGeometry = tmpBool;

  ///MainWindow::ReconstructDetector();
  MainWindow::on_pbUpdateSandwich_clicked();
}

void MainWindow::UpdateMaterialNameInCobs(int imat)
{
  QString name = (*MaterialCollection)[imat]->name;
  ui->cobS1->setItemText(imat, name);
  ui->cobS2->setItemText(imat, name);
  ui->cobS3->setItemText(imat, name);
  ui->cobS4->setItemText(imat, name);
  ui->cobS5->setItemText(imat, name);
  ui->cobUW->setItemText(imat, name);
  ui->cobLW->setItemText(imat, name);
  ui->cobPS->setItemText(imat, name);
  ui->cobSS->setItemText(imat, name);
  ui->cobTOP->setItemText(imat, name);
  ui->cobMatPM->setItemText(imat, name);
  ui->cobMaterialForOverrides->setItemText(imat, name);
  ui->cobMaterialTo->setItemText(imat, name);
  ui->cobMaterialForWaveTests->setItemText(imat, name);
  ui->cobMatPointSource->setItemText(imat, name);  
  ui->cobShowMaterial->setItemText(imat, name);

  DAwindow->UpdateMaterialInCobs(imat); 
  MIwindow->UpdateMaterialInCobs(imat);
}

void MainWindow::UpdateMaterialListEdit()
{
  ui->lwMaterials->clear();
  int NumMaterials = MaterialCollection->size();

  QString tmpStr, tmpStr2;
  for (int i=0; i<NumMaterials; i++)
   {
    tmpStr2.setNum(i);
    tmpStr=tmpStr2+">  "+(*MaterialCollection)[i]->name;

    //if there are overrides, add (*)
    bool fFound = false;
    for (int p=0; p<NumMaterials; p++)      
      //  if ( (*MaterialCollection)[i]->LossOverride[p]>0 || (*MaterialCollection)[i]->ReflectionOverride[p]>0 || (*MaterialCollection)[i]->ScatterOverride[p]>0)
      if ( (*MaterialCollection)[i]->OpticalOverrides[p] )
        {
          fFound = true;
          break;
        }

    if (fFound) tmpStr += "  (override)";

    QListWidgetItem* pItem =new QListWidgetItem(tmpStr);
    if (ColorByMaterial)
      {
        TColor* rc = gROOT->GetColor(i+1);
        QColor qc = QColor(255*rc->GetRed(), 255*rc->GetGreen(), 255*rc->GetBlue(), 255*rc->GetAlpha());
        pItem->setForeground(qc);
      }
    else pItem->setForeground(Qt::black);   
    ui->lwMaterials->addItem(pItem);
  }
}

void MainWindow::ShowPMcount()
{
  QString str;
  str.setNum(PMs->count());
  ui->labTotPMs->setText(str);
}

void MainWindow::PopulatePMarray(int ul, double z, int istart) // ul= 0 upper, 1 = lower; z - z coordinate; istart - first PM fill be added at this index
{  
  bool hexagonal;
  if (Detector->PMarrays[ul].Packing == 0) hexagonal = false; else hexagonal = true;
  int iX = Detector->PMarrays[ul].NumX;
  int iY = Detector->PMarrays[ul].NumY;
  double CtC = Detector->PMarrays[ul].CenterToCenter;
  double CtCbis = CtC*cos(30.*3.14159/180.);
  int rings = Detector->PMarrays[ul].NumRings;
  bool XbyYarray = !Detector->PMarrays[ul].fUseRings;
  double Psi = 0;

  double x, y;
  double typ = Detector->PMarrays[ul].PMtype;

  int ipos = istart; //where (ipm) insert the next PM

  //populating PM array
  if (XbyYarray)
  { //X by Y type of array  
    if (hexagonal)
     {
      for (int j=iY-1; j>-1; j--)
          for (int i=0; i<iX; i++)
            {
              x = CtC*( -0.5*(iX-1) +i  + 0.5*(j%2));
              y = CtCbis*(-0.5*(iY-1) +j);
              PMs->insert(ipos, ul,x,y,z,Psi,typ);
              ipos++;
            }
     }
    else
     {
       for (int j=iY-1; j>-1; j--)
           for (int i=0; i<iX; i++)
             {
               x = CtC*(-0.5*(iX-1) +i);
               y = CtC*(-0.5*(iY-1) +j);
               PMs->insert(ipos, ul,x,y,z,Psi,typ);
               ipos++;
             }
      }
  }
  else
    { //rings array
      if (hexagonal) {
     //   int nPMs = 1; //0 rings = 1 PM
     //   for (int i=1; i<rings+1; i++) nPMs += 6*i; //every new ring adds 6i PMs

        //qDebug()<<nPMs<<"for rings:"<<rings;
        PMs->insert(ipos, ul,0,0,z, Psi, typ);
        ipos++;

        ///int index=PMs->count();
        //int index = ipos;
        for (int i=1; i<rings+1; i++)
          {
            x = i*CtC;
            y = 0;
            PMs->insert(ipos, ul,x,y,z,Psi,typ);
            ipos++;
            //qDebug()<<"-*"<<PMx[index]<<PMy[index];
            //index++;

            for (int j=1; j<i+1; j++) {  //   /
                x = PMs->X(ipos-1) - 0.5*CtC; //was index
                y = PMs->Y(ipos-1) - CtCbis;
                PMs->insert(ipos, ul,x,y,z,Psi,typ);
                ipos++;
                // qDebug()<<"/"<<PMx[index]<<PMy[index];
                //index++;
            }
            for (int j=1; j<i+1; j++) {   // -
                x = PMs->X(ipos-1) - CtC;
                y = PMs->Y(ipos-1);
                PMs->insert(ipos, ul,x,y,z,Psi,typ);
                ipos++;
                // qDebug()<<"-"<<PMx[index]<<PMy[index];
                //index++;
            }
            for (int j=1; j<i+1; j++) {  // \
            //    qDebug()<<"previous x: "<<PMx[index-1];
                x = PMs->X(ipos-1) - 0.5*CtC;
                y = PMs->Y(ipos-1) + CtCbis;
                PMs->insert(ipos, ul,x,y,z,Psi,typ);
                ipos++;
                // qDebug()<<"\\"<<PMx[index]<<PMy[index];
                //index++;
            }
            for (int j=1; j<i+1; j++) {  // /
                x = PMs->X(ipos-1) + 0.5*CtC;
                y = PMs->Y(ipos-1) + CtCbis;
                PMs->insert(ipos, ul,x,y,z,Psi,typ);
                ipos++;
                //qDebug()<<"/"<<PMx[index]<<PMy[index];
                //index++;
            }
            for (int j=1; j<i+1; j++) {   // -
                x = PMs->X(ipos-1) + CtC;
                y = PMs->Y(ipos-1);
                PMs->insert(ipos, ul,x,y,z,Psi,typ);
                ipos++;
                //qDebug()<<"-"<<PMx[index]<<PMy[index];
                //index++;
            }
            for (int j=1; j<i; j++) {  // \       //dont do the last step - already positioned PM
                x = PMs->X(ipos-1) + 0.5*CtC;
                y = PMs->Y(ipos-1) - CtCbis;
                PMs->insert(ipos, ul,x,y,z,Psi,typ);
                ipos++;
                //qDebug()<<"\\"<<PMx[index]<<PMy[index];
                //index++;
            }
        }
      }
      else {
         //using the same algorithm as X * Y
         iX = rings*2 + 1;
         iY = rings*2 + 1;

         for (int j=0; j<iY; j++)
             for (int i=0; i<iX; i++) {
                 x = CtC*(-0.5*(iX-1) +i);
                 y = CtC*(-0.5*(iY-1) +j);
                 PMs->insert(ipos, ul,x,y,z,Psi,typ);
                 ipos++;
             }
      }
    }
}

void MainWindow::on_cbXbyYarray_stateChanged(int arg1)
{
        ui->sbNumX->setEnabled(arg1);
        ui->sbNumY->setEnabled(arg1);
}

void MainWindow::on_cbRingsArray_stateChanged(int arg1)
{
     ui->sbPMrings->setEnabled(arg1);
}

void MainWindow::on_pbOverride_clicked()
{
    //qDebug() << "->Registering new override parameters";
    int From = ui->cobMaterialForOverrides->currentIndex();
    int To =   ui->cobMaterialTo->currentIndex();

    OpticalOverrideClass* ov = (*MaterialCollection)[From]->OpticalOverrides[To];
    if (ov)
      {
        delete ov;
        ov = NULL;
      }

    int Model = ui->cobOpticalOverrideModel->currentIndex();
    switch (Model)
      {
      case 0: //Override not defined
        break;
      case 1: //Simplistic model
        {
        double abs = 0.01* ui->dsbNewProbLoss->value();
        double spec = 0.01* ui->dsbNewProbReflection->value();
        double scat = 0.01* ui->dsbNewProbDiffuse->value();
        int scatModel = ui->cobScatteringModel->currentIndex();
        ov = new BasicOpticalOverride(Detector->MaterialCollection, From, To, abs, spec, scat, scatModel);
        break;
        }
      case 2: //Claudio's model - using version 2.2
        {
        PhScatClaudioModelV2d2* cs = new PhScatClaudioModelV2d2(Detector->MaterialCollection, From, To);
        cs->sigma_alpha = ui->ledSigmaAlpha->text().toDouble();
        cs->sigma_h = ui->ledSigmaSpike->text().toDouble();
        cs->albedo = ui->ledAlbedo->text().toDouble();
        switch(ui->conHeightModel->currentIndex())
          {
            case 0: cs->HeightDistribution = empirical; break;
            case 1: cs->HeightDistribution = gaussian; break;
            case 2: cs->HeightDistribution = exponential; break;
            default:;
          }
        switch(ui->cobSlopeModel->currentIndex())
          {
            case 0: cs->SlopeDistribution = trowbridgereitz; break;
            case 1: cs->SlopeDistribution = cooktorrance; break;
            case 2: cs->SlopeDistribution = bivariatecauchy; break;
            default:;
          }
        ov = cs;
        break;
        }
      case 3: //Dielectric/Metal model
        {
          ScatterOnMetal* sm = new ScatterOnMetal(Detector->MaterialCollection, From, To);
          sm->RealN = ui->ledRealN->text().toDouble();
          sm->ImaginaryN = ui->ledImaginaryN->text().toDouble();
          ov = sm;
          break;
        }
      }

//    if (ov)
//      {
//        qDebug() << "Override type selected:" <<ov->getType();
//        ov->printConfiguration(-1);
//      }
//    else qDebug() << "No override selected";

    (*MaterialCollection)[From]->OpticalOverrides[To] = ov;

//    (*MaterialCollection)[From]->ReflectionOverride[To] = 0.01* ui->dsbNewProbReflection->value();
//    (*MaterialCollection)[From]->ScatterOverride[To] = 0.01* ui->dsbNewProbDiffuse->value();
//    (*MaterialCollection)[From]->ScatterModel[To] = ui->cobScatteringModel->currentIndex();
//    (*MaterialCollection)[From]->LossOverride[To] = 0.01* ui->dsbNewProbLoss->value();

    ReconstructDetector(true);

    MainWindow::on_pbRefreshOverrides_clicked();
    int i = ui->lwMaterials->currentRow();
    MainWindow::UpdateMaterialListEdit(); //to update (*) status
    ui->lwMaterials->setCurrentRow(i);
}

void MainWindow::on_pbRefreshOverrides_clicked()
{
    //qDebug() << "->Updating overrides indication";
    ui->lwOverrides->clear();
    int size = MaterialCollection->size();

    int MatFrom = ui->cobMaterialForOverrides->currentIndex();
    int MatTo   = ui->cobMaterialTo->currentIndex();

    //updating text field
    QString s;
    bool flagEmpty = true;
    for (int i=0; i<size; i++)
      {
        flagEmpty = false;
        if ( !(*MaterialCollection)[MatFrom]->OpticalOverrides.at(i) ) continue;
        s = (*MaterialCollection)[MatFrom]->OpticalOverrides.at(i)->getReportLine();
        //qDebug() << s;
        ui->lwOverrides->addItem(s);
      }
    if (flagEmpty) ui->lwOverrides->addItem("Not defined");

    //updating parameters area
    if ( !(*MaterialCollection)[MatFrom]->OpticalOverrides.at(MatTo) )
      ui->cobOpticalOverrideModel->setCurrentIndex(0);
    else
      {
        QString model = (*MaterialCollection)[MatFrom]->OpticalOverrides.at(MatTo)->getType();
        if (model == "Simplistic_model")
          {
            ui->cobOpticalOverrideModel->setCurrentIndex(1);
            BasicOpticalOverride* ov = dynamic_cast<BasicOpticalOverride*>( (*MaterialCollection)[MatFrom]->OpticalOverrides.at(MatTo) );
            ui->dsbNewProbLoss->setValue(ov->probLoss*100.0);
            ui->dsbNewProbReflection->setValue(ov->probRef*100.0);
            ui->dsbNewProbDiffuse->setValue(ov->probDiff*100.0);
            ui->cobScatteringModel->setCurrentIndex(ov->scatterModel);
          }
        else if (model.startsWith("Claudio_Model"))
          {
            ui->cobOpticalOverrideModel->setCurrentIndex(2);
            PhScatClaudioModel* ov = dynamic_cast<PhScatClaudioModel*>( (*MaterialCollection)[MatFrom]->OpticalOverrides.at(MatTo) );
            ui->ledSigmaAlpha->setText(QString::number(ov->sigma_alpha));
            ui->ledSigmaSpike->setText(QString::number(ov->sigma_h));
            ui->ledAlbedo->setText(QString::number(ov->albedo));
            ui->conHeightModel->setCurrentIndex( (int)ov->HeightDistribution );
            ui->cobSlopeModel->setCurrentIndex( (int)ov->SlopeDistribution );
          }
        else if (model == "DielectricToMetal")
          {
            ui->cobOpticalOverrideModel->setCurrentIndex(3);
            ScatterOnMetal* ov = dynamic_cast<ScatterOnMetal*>( (*MaterialCollection)[MatFrom]->OpticalOverrides.at(MatTo) );
            ui->ledRealN->setText(QString::number(ov->RealN));
            ui->ledImaginaryN->setText(QString::number(ov->ImaginaryN));
          }
        else
          {
             message("Unknown override model: "+model, this);
             ui->cobOpticalOverrideModel->setCurrentIndex(0);
          }
      }
}

void MainWindow::on_pbStartMaterialInspector_clicked()
{
    MIwindow->showNormal();
    MIwindow->raise();
    MIwindow->activateWindow();   
}

void MainWindow::on_pushButton_4_clicked()
{ 
  if (hist1) GraphWindow->Draw(hist1);
}

void MainWindow::on_pushButton_5_clicked()
{ 
  if (hist2) GraphWindow->Draw(hist2);
}

void MainWindow::on_pushButton_7_clicked()
{
    if (hist1) delete hist1;
    hist1 = new TH1D("Testrndm","uniform",100,0,1.);

    for (int i=0; i<100000; i++) hist1->Fill(Detector->RandGen->Rndm());
    GraphWindow->Draw(hist1);
}

void MainWindow::on_cbSecondAxis_toggled(bool checked)
{
    ui->fScanSecond->setEnabled(checked);
}

void MainWindow::on_cbThirdAxis_toggled(bool checked)
{
    ui->fScanThird->setEnabled(checked);
}

void MainWindow::on_cobPMdeviceType_currentIndexChanged(int index)
{ //on user click, see method on_cobPMdeviceType_activated(const QString &arg1)
  ui->swPMTvsSiPM->setCurrentIndex(index);

  if (index == 0)
    {
      ui->cobPMshape->setEnabled(true);

      ui->labPDE->setText("QE vs. wavelength:");
      ui->labPDEnwr->setText("QE:");
      ui->twPMtypeProperties->setTabText(0, "Quantum Efficiency");
    }
  else
    {
      ui->cobPMshape->setCurrentIndex(0);
      ui->cobPMshape->setEnabled(false);

      ui->labPDE->setText("PDE vs. wavelength:");
      ui->labPDEnwr->setText("PDE:");
      ui->twPMtypeProperties->setTabText(0, "Photon Detection Efficiency");
    }  
}

void MainWindow::on_cobPMdeviceType_activated(const QString &/*arg1*/)
{  //see also method on_cobPMdeviceType_currentIndexChanged(int index)
    MainWindow::on_pbUpdatePMproperties_clicked();
}

void MainWindow::CheckSetMaterial(const QString name, QComboBox* cob, QVector<QString>* vec)
{
    for (int i=0; i<MaterialCollection->size(); i++)
    {
        if (!(*MaterialCollection)[i]->name.compare(name,Qt::CaseSensitive))
        {
            cob->setCurrentIndex(i);
            return;
        }
    }
    //not found
   vec->append(name);
}

void MainWindow::on_cbSS_toggled(bool checked)
{
    ui->fSecondaryLightGenType->setEnabled(checked);
}

void MainWindow::on_cbUPM_toggled(bool checked)
{    
  Detector->PMarrays[0].fActive = checked;
  ToggleUpperLowerPMs();
}

void MainWindow::on_cbLPM_toggled(bool checked)
{
  Detector->PMarrays[1].fActive = checked;
  ToggleUpperLowerPMs();
}

void MainWindow::ToggleUpperLowerPMs()
{  
  //upper-lower selector in arrays is visible only if both lower and upper are enabled
  if (!ui->cbUPM->isChecked() || !ui->cbLPM->isChecked()) ui->fUpperLowerArrays->setVisible(false);
  else ui->fUpperLowerArrays->setVisible(true);

  //autoselect array  
  if (ui->cbUPM->isChecked() && !ui->cbLPM->isChecked()) ui->cobUpperLowerPMs->setCurrentIndex(0);
  else if (!ui->cbUPM->isChecked() && ui->cbLPM->isChecked()) ui->cobUpperLowerPMs->setCurrentIndex(1);

  //force-trigger visualization  
  MainWindow::on_cobUpperLowerPMs_currentIndexChanged(ui->cobUpperLowerPMs->currentIndex());

  if (DoNotUpdateGeometry) return; //if it is triggered during load/init hase
  MainWindow::on_pbRebuildDetector_clicked();  //rebuild detector
}

void MainWindow::on_pbRefreshPMArrayData_clicked()
{  
  if (DoNotUpdateGeometry) return;

  int ul = ui->cobUpperLowerPMs->currentIndex();
//  qApp->processEvents();
  if (ui->cobPMarrayRegularity->currentIndex() == 0)
    {
      //Updating regular properties
      Detector->PMarrays[ul].PMtype = ui->sbPMtypeForGroup->value();
      Detector->PMarrays[ul].Packing = ui->cobPacking->currentIndex();
      Detector->PMarrays[ul].CenterToCenter = ui->ledCtC->text().toDouble();
      Detector->PMarrays[ul].NumX = ui->sbNumX->value();
      Detector->PMarrays[ul].NumY = ui->sbNumY->value();
      Detector->PMarrays[ul].NumRings = ui->sbPMrings->value();
      Detector->PMarrays[ul].fUseRings = ui->cbRingsArray->isChecked();
    }  
  Detector->PMarrays[ul].Regularity = ui->cobPMarrayRegularity->currentIndex();

  //if (ui->cobPMarrayRegularity->currentIndex() < 2) MainWindow::RebuildPmArray();
  if (ui->cobPMarrayRegularity->currentIndex() == 1)
    {
      //there will be no automatic update of the base type in the array, have to do it:
      for (int i=0; i<PMs->count(); i++)
        if (PMs->at(i).upperLower == ul) PMs->at(i).type = ui->sbPMtypeForGroup->value();
    }

  //if (geometryRW) MainWindow::on_pbCreateCustomConfiguration_clicked();
  MainWindow::on_pbRebuildDetector_clicked();

  MainWindow::ClearData(); 
  Owindow->RefreshData();
}

void MainWindow::on_pbShowPMsArrayRegularData_clicked()
{
   updatePMArrayDataIndication();
}

void MainWindow::updatePMArrayDataIndication()
{
  int i = ui->cobUpperLowerPMs->currentIndex();

  bool tmpBool = DoNotUpdateGeometry;
  DoNotUpdateGeometry = true;

  ui->sbPMtypeForGroup->setValue(Detector->PMarrays[i].PMtype);
  ui->cobPMtypeInArrays->setCurrentIndex(Detector->PMarrays[i].PMtype);
  ui->cobPMarrayRegularity->setCurrentIndex(Detector->PMarrays[i].Regularity);
  if (!BulkUpdate) MainWindow::on_cobPMarrayRegularity_currentIndexChanged(Detector->PMarrays[i].Regularity); //force update
  ui->cobPacking->setCurrentIndex(Detector->PMarrays[i].Packing);
  QString str;
  str.setNum(Detector->PMarrays[i].CenterToCenter, 'g', 4);
  ui->ledCtC->setText(str);
  ui->sbNumX->setValue(Detector->PMarrays[i].NumX);
  ui->sbNumY->setValue(Detector->PMarrays[i].NumY);
  ui->sbPMrings->setValue(Detector->PMarrays[i].NumRings);
  ui->cbRingsArray->setChecked(Detector->PMarrays[i].fUseRings);
  ui->cbXbyYarray->setChecked(!Detector->PMarrays[i].fUseRings);

  DoNotUpdateGeometry = tmpBool;
  MainWindow::ShowPMcount();
}

void MainWindow::on_pbRefreshPMproperties_clicked()
{
    //refresh indication of PM model properties
    int index = ui->sbPMtype->value();
    PMtypeClass *type = PMs->getType(index);

    bool tmpBool = DoNotUpdateGeometry;
    DoNotUpdateGeometry = true;
    ui->lePMtypeName->setText(type->name);
    ui->cobPMdeviceType->setCurrentIndex(type->SiPM);
    ui->cobMatPM->setCurrentIndex(type->MaterialIndex);
    ui->cobPMshape->setCurrentIndex(type->Shape);
    QString str;

    str.setNum(type->SizeX, 'g', 4);
    ui->ledSizeX->setText(str);
    str.setNum(type->SizeY, 'g', 4);
    ui->ledSizeY->setText(str);
    ui->sbSiPMnumx->setValue(type->PixelsX);
    ui->sbSiPMnumy->setValue(type->PixelsY);
    str.setNum(type->PixelsX * type->PixelsY);
    ui->labTotalPixels->setText("("+str+")");
    str.setNum(type->DarkCountRate, 'g', 4);
    ui->ledSiPMdarCountRate->setText(str);
    str.setNum(type->RecoveryTime, 'g', 4);
    ui->ledSiPMrecoveryTime->setText(str);

    str.setNum(type->effectivePDE, 'g', 4);
    ui->ledPDE->setText(str);
    ui->pbShowPDE->setEnabled(type->PDE_lambda.size());
    ui->pbShowPDEbinned->setEnabled(ui->cbWaveResolved->isChecked() && type->PDE_lambda.size());
    ui->pbDeletePDE->setEnabled(type->PDE_lambda.size());
    MainWindow::RefreshAngularButtons();
    MainWindow::RefreshAreaButtons();
    str.setNum(type->n1, 'g', 4);
    ui->ledMediumRefrIndex->setText(str);

    //enable/disable control
    int numModels = PMs->countPMtypes();
    bool SeveralModels = (numModels > 1);
    ui->fSelectPMmodel->setVisible(SeveralModels);
    ui->pbRemoveThisPMtype->setVisible(SeveralModels);

    //update cob
    ui->cobPMtypes->clear();
    for (int i=0; i<numModels; i++) ui->cobPMtypes->addItem(PMs->getType(i)->name);
    ui->cobPMtypes->setCurrentIndex(index);

    DoNotUpdateGeometry = tmpBool;
}

void MainWindow::on_pbUpdatePMproperties_clicked()
{  
   if (DoNotUpdateGeometry) return;

   int index = ui->sbPMtype->value();
   PMtypeClass *type = PMs->getType(index);
   type->name = ui->lePMtypeName->text();

   type->SiPM = ui->cobPMdeviceType->currentIndex();
   type->MaterialIndex = ui->cobMatPM->currentIndex();
   type->Shape = ui->cobPMshape->currentIndex();
   type->SizeX = ui->ledSizeX->text().toDouble();
   type->SizeY = ui->ledSizeY->text().toDouble();
   type->PixelsX = ui->sbSiPMnumx->value();
   type->PixelsY = ui->sbSiPMnumy->value();
   type->DarkCountRate = ui->ledSiPMdarCountRate->text().toDouble();
   type->RecoveryTime = ui->ledSiPMrecoveryTime->text().toDouble();
   if (type->SiPM) type->Shape = 0; //SiPM - always rectangular
   type->effectivePDE = ui->ledPDE->text().toDouble();
   //PMs->setTypePropertiesScalar(index, &tp);
   QString str;
   str.setNum(type->PixelsX * type->PixelsY);
   ui->labTotalPixels->setText("("+str+")");

   //wave-, angular- and area-resolved data are updated independently

   MainWindow::updateCOBsWithPMtypeNames();
   MainWindow::on_pbShowPMsArrayRegularData_clicked(); //in case type name was changed
   MainWindow::ReconstructDetector();//MainWindow::on_pbCreateCustomConfiguration_clicked();

   //for indication, update PMs binned properties
   on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked());
   on_cbAngularSensitive_toggled(ui->cbAngularSensitive->isChecked());
}

void MainWindow::on_sbPMtype_valueChanged(int arg1)
{
  if (DoNotUpdateGeometry) return;
  if (arg1 > PMs->countPMtypes()-1) ui->sbPMtype->setValue(PMs->countPMtypes()-1);
  MainWindow::on_pbRefreshPMproperties_clicked();
}

void MainWindow::on_sbPMtypeForGroup_valueChanged(int arg1)
{
  if (arg1 > PMs->countPMtypes()-1)
    {
      ui->sbPMtypeForGroup->setValue(PMs->countPMtypes()-1);
      return;
    }
  ui->cobPMtypeInArrays->setCurrentIndex(arg1); //save for cross-call
  //ui->leoPMtypeInArrayTab->setText(PMs->getTypeProperties(arg1)->name);
  MainWindow::on_pbRefreshPMArrayData_clicked();
}

void MainWindow::on_cbXbyYarray_clicked()
{
    ui->cbRingsArray->toggle();
    MainWindow::on_pbRefreshPMArrayData_clicked();
}

void MainWindow::on_cbRingsArray_clicked()
{
    ui->cbXbyYarray->toggle();
    MainWindow::on_pbRefreshPMArrayData_clicked();
}

void MainWindow::on_cobUpperLowerPMs_currentIndexChanged(int index)
{
  if (index < 0) return;

  bool upper;
  if (index == 0) upper = true; else upper = false;
  ui->lineUpperPMs->setVisible(upper);
  ui->lineLowerPMs->setVisible(!upper);
  if (Detector->PMarrays[index].fActive) MainWindow::on_pbShowPMsArrayRegularData_clicked();
}

void MainWindow::on_pbRemoveThisPMtype_clicked()
{
    int itype = ui->sbPMtype->value();
    int numTypes = PMs->countPMtypes();
    //qDebug() << numTypes;
    if (numTypes < 2)
      {        
        message("Cannot delete the last type", this);
        return;
      }

    bool fOK = PMs->removePMtype(itype);
    if (!fOK)
      {        
        message("Type in use! Cannot delete", this);
        return;
      }
    //tmpPMtype = PMs->getType(itype-1);
    MainWindow::on_pbShowPMsArrayRegularData_clicked(); //refresh indication
    ui->sbPMtype->setValue(itype-1);
    //updating all comboboxes with PM type names
    MainWindow::updateCOBsWithPMtypeNames();
}

void MainWindow::on_pbAddNewPMtype_clicked()
{
  MainWindow::AddDefaultPMtype();
  ui->sbPMtype->setValue(PMs->countPMtypes()-1); //update of indication is in on_change
}

void MainWindow::AddDefaultPMtype()
{
  //creating a unique name
  QString name = "Type" + QString::number(PMs->countPMtypes());
  bool found;
  do
    {
      found = false;
      for (int i=0; i<PMs->countPMtypes(); i++)
        if (name == PMs->getType(i)->name)
          {
            found = true;
            break;
          }
      if (found) name += "_1";
    }
  while (found);
  PMtypeClass *type = new PMtypeClass(name);
  PMs->appendNewPMtype(type);

  //updating all comboboxes with PM type names
  MainWindow::updateCOBsWithPMtypeNames();
}

void MainWindow::updateCOBsWithPMtypeNames()
{
  if (PMs->count() == 0) return;

  int defTypes = PMs->countPMtypes();
  if (defTypes == 0) return; //protection

  bool tmpBool = DoNotUpdateGeometry;
  DoNotUpdateGeometry = true;

  ui->cobPMtypes->clear();
  ui->cobPMtypeInArrays->clear();
  ui->cobPMtypeInExplorers->clear();
  ui->cobPMtypeInArraysForCustom->clear();

  for (int i=0; i<defTypes; i++)
    {
      QString name = PMs->getType(i)->name;
      ui->cobPMtypes->addItem(name);
      ui->cobPMtypeInArrays->addItem(name);
      ui->cobPMtypeInExplorers->addItem(name);
      ui->cobPMtypeInArraysForCustom->addItem(name);
    }
  ui->cobPMtypes->setCurrentIndex(ui->sbPMtype->value());  
  ui->cobPMtypeInArrays->setCurrentIndex(Detector->PMarrays[ui->cobUpperLowerPMs->currentIndex()].PMtype); //MainWindow::on_pbShowPMsArrayRegularData_clicked();
  ui->cobPMtypeInExplorers->setCurrentIndex( PMs->at(ui->sbIndPMnumber->value()).type );

  DoNotUpdateGeometry = tmpBool;
}

void MainWindow::on_cbAreaSensitive_toggled(bool checked)
{
  PMs->setAreaResolved(checked);
  if (checked) ui->twOption->setTabText(3, "Area:On");
  else ui->twOption->setTabText(3, "Area:Off");  
}

void MainWindow::on_cbAngularSensitive_toggled(bool checked)
{
   if (BulkUpdate) return;
  //triggered also after load sim settings to update
    PMs->setAngularResolved(checked);
    int CosBins = ui->sbCosBins->value();
    PMs->setCosBins(CosBins);
    PMs->RecalculateAngular();

    MainWindow::RefreshAngularButtons();
    if (checked) ui->twOption->setTabText(2, "Angular:On");
    else ui->twOption->setTabText(2, "Angular:Off");
    ui->fAngular->setEnabled(checked);
    MainWindow::on_pbIndPMshowInfo_clicked(); //to refresh the binned button
}

void MainWindow::on_cbTimeResolved_toggled(bool checked)
{
   if (BulkUpdate) return;
    ui->fTime->setEnabled(checked);
    if (checked) ui->twOption->setTabText(0, "Time:On");
    else ui->twOption->setTabText(0, "Time:Off");
    MainWindow::ClearData();

    ui->fPointSource_Time->setEnabled(checked);
    ui->fDirectOrmat->setEnabled(ui->cbWaveResolved->isChecked() || ui->cbTimeResolved->isChecked());
}

void MainWindow::on_pbPMtypeShowCurrent_clicked()
{
  ui->tabWidget->setCurrentIndex(1);
  ui->sbPMtype->setValue(ui->sbPMtypeForGroup->value());
}

bool MainWindow::isWavelengthResolved() const
{
  return ui->cbWaveResolved->isChecked();
}

void MainWindow::recallGeometryOfScriptWindow()
{
  if (!GenScriptWindow) return;
    //GenScriptWindow->setGeometry(ScriptWinX, ScriptWinY, ScriptWinW, ScriptWinH);

  GenScriptWindow->move(ScriptWinX, ScriptWinY);
  GenScriptWindow->resize(ScriptWinW, ScriptWinH);
}

void MainWindow::extractGeometryOfScriptWindow()
{
  if (GenScriptWindow)
    {
      ScriptWinX = GenScriptWindow->x();
      ScriptWinY = GenScriptWindow->y();
      ScriptWinW = GenScriptWindow->width();
      ScriptWinH = GenScriptWindow->height();
    }
}

void MainWindow::on_cbWaveResolved_toggled(bool checked)
{
    if (BulkUpdate) return;
   //triggered as update after load sim config too!
   //this is toggled on editing_finished of WaveFrom Waveto and Wavestep
    //qDebug()<<"---toggle---";  
    //WavelengthResolved = checked;
    WaveFrom = ui->ledWaveFrom->text().toDouble();
    WaveStep = ui->ledWaveStep->text().toDouble();
    MainWindow::CorrectWaveTo(); //WaveTo and WaveNode are set here

    //update materialCollection info -rebinning, hists    
    MaterialCollection->SetWave(checked, WaveFrom, WaveTo, WaveStep, WaveNodes);
    for (int i=0; i<MaterialCollection->size(); i++) MaterialCollection->UpdateWaveResolvedProperties(i);
    //updating PMs
    PMs->SetWave(checked, WaveFrom, WaveStep, WaveNodes);
    PMs->RebinPDEs(); //update PMs info -rebinning, hists
    UpdateTestWavelengthProperties();

    ui->fWaveTests->setEnabled(checked); //tests tab: wavelength resolved properties
    ui->fWaveOptions->setEnabled(checked);
    if (checked) ui->twOption->setTabText(1, " Wave:On");
    else ui->twOption->setTabText(1, " Wave:Off");

    ui->pbShowPDEbinned->setEnabled(checked && PMs->getType(ui->sbPMtype->value())->PDE_lambda.size());
    MainWindow::on_pbIndPMshowInfo_clicked(); //to refresh the binned button

    ui->fPointSource_Wave->setEnabled(checked);
    ui->fDirectOrmat->setEnabled(checked || ui->cbTimeResolved->isChecked());
}

void MainWindow::on_ledWaveFrom_editingFinished()
{
  double newValue = ui->ledWaveFrom->text().toDouble();
  if (newValue<=0  || newValue > ui->ledWaveTo->text().toDouble())
    {
      QString str;
      str.setNum(WaveFrom,'g',5);
      ui->ledWaveFrom->setText(str);      
      message("Error in the starting wavelength value, resetted", this);
      return;
    }
  MainWindow::CorrectWaveTo();
  WaveFrom = ui->ledWaveFrom->text().toDouble();
  PMs->setWaveFrom(WaveFrom);

  MainWindow::on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked());
}

void MainWindow::on_ledWaveTo_editingFinished()
{
  double newValue = ui->ledWaveTo->text().toDouble();
  if (newValue<=0  || newValue < ui->ledWaveFrom->text().toDouble())
    {
      QString str;
      str.setNum(WaveTo,'g',5);
      ui->ledWaveTo->setText(str);      
      message("Error in the ending wavelength value, resetted", this);
      return;
    }
   MainWindow::CorrectWaveTo();

   MainWindow::on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked());
}

void MainWindow::on_ledWaveStep_editingFinished()
{
   double newValue = ui->ledWaveStep->text().toDouble();
   if (newValue < 1e-10)
    {
      QString str;
      str.setNum(WaveStep,'g',5);
      ui->ledWaveStep->setText(str);     
      message("Error in the step value, resetted", this);
      return;
    }
   MainWindow::CorrectWaveTo();
   WaveStep = ui->ledWaveStep->text().toDouble();
   PMs->setWaveStep(WaveStep);
   //PhotonGenerator->setWaveStep(WaveStep);

   MainWindow::on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked());
}

void MainWindow::CorrectWaveTo()
{
  int steps = (ui->ledWaveTo->text().toDouble() - ui->ledWaveFrom->text().toDouble()) / ui->ledWaveStep->text().toDouble();
  WaveNodes = steps+1;
  QString str;
  str.setNum(WaveNodes);
  ui->labWaveOptionNodes->setText(str);
  PMs->setWaveNodes(WaveNodes);  
  double to = ui->ledWaveFrom->text().toDouble() + ui->ledWaveStep->text().toDouble()*steps;
  WaveTo = to;   
  str.setNum(to,'g',5);
  ui->ledWaveTo->setText(str);
  if (ui->sbWaveIndexPointSource->value() > WaveNodes-1) ui->sbWaveIndexPointSource->setValue(0);
}

void MainWindow::on_cobMaterialForWaveTests_currentIndexChanged(int index)
{
  if (index<0) return;
  if (DoNotUpdateGeometry) return;
  UpdateTestWavelengthProperties();
}

void MainWindow::on_pbUpdateTestWavelengthProperties_clicked()
{
  UpdateTestWavelengthProperties();
}

void MainWindow::UpdateTestWavelengthProperties()
{
  if (!ui->cbWaveResolved->isChecked()) return;
 // if (graphRW) graphRW->hide();
  int matId = ui->cobMaterialForWaveTests->currentIndex();

  ui->pbTestShowPrimary->setEnabled((*MaterialCollection)[matId]->PrimarySpectrumHist);
  ui->pbTestShowSecondary->setEnabled((*MaterialCollection)[matId]->SecondarySpectrumHist);
  ui->pbTestGeneratorPrimary->setEnabled((*MaterialCollection)[matId]->PrimarySpectrumHist);
  ui->pbTestGeneratorSecondary->setEnabled((*MaterialCollection)[matId]->SecondarySpectrumHist);

  ui->pbTestShowRefrIndex->setEnabled((*MaterialCollection)[matId]->nWaveBinned.size());
  ui->pbTestShowAbs->setEnabled((*MaterialCollection)[matId]->absWaveBinned.size());
}


void MainWindow::on_pbTestShowPrimary_clicked()
{ 
  int matId = ui->cobMaterialForWaveTests->currentIndex();
  GraphWindow->Draw((*MaterialCollection)[matId]->PrimarySpectrumHist);
}

void MainWindow::on_pbTestShowSecondary_clicked()
{  
  int matId = ui->cobMaterialForWaveTests->currentIndex();
  GraphWindow->Draw((*MaterialCollection)[matId]->SecondarySpectrumHist);
}

void MainWindow::on_pbTestGeneratorPrimary_clicked()
{  
  int matId = ui->cobMaterialForWaveTests->currentIndex();

  if (!(*MaterialCollection)[matId]->PrimarySpectrumHist) return;

  if (hist1) delete hist1;
  hist1 = new TH1D("PrimaryTest", "Primary scintillation", WaveNodes, WaveFrom, WaveTo);

  for (int i=0; i<ui->sbWaveTestsNumPhotons->value(); i++) hist1->Fill( (*MaterialCollection)[matId]->PrimarySpectrumHist->GetRandom() );
  GraphWindow->Draw(hist1);
}

void MainWindow::on_pbTestGeneratorSecondary_clicked()
{  
  int matId = ui->cobMaterialForWaveTests->currentIndex();

  if (!(*MaterialCollection)[matId]->SecondarySpectrumHist) return;

  if (hist1) delete hist1;
  hist1 = new TH1D("SecondaryTest", "Secondary scintillation", WaveNodes, WaveFrom, WaveTo);

  for (int i=0; i<ui->sbWaveTestsNumPhotons->value(); i++) hist1->Fill((*MaterialCollection)[matId]->SecondarySpectrumHist->GetRandom());

  GraphWindow->Draw(hist1);
}

void MainWindow::on_pbTestShowRefrIndex_clicked()
{  
  int matId = ui->cobMaterialForWaveTests->currentIndex();

  QVector<double> x;
  x.resize(0);
  for (int i=0; i<WaveNodes; i++) x.append(WaveFrom + WaveStep*i);
  //ShowGraphWindow();
  GraphWindow->ShowGraph(&x, &(*MaterialCollection)[matId]->nWaveBinned, kRed, "Wavelength, nm", "Refractive index");
}

void MainWindow::on_pbTestShowAbs_clicked()
{  
  int matId = ui->cobMaterialForWaveTests->currentIndex();

  QVector<double> x;
  x.resize(0);
  for (int i=0; i<WaveNodes; i++) x.append(WaveFrom + WaveStep*i);
  //ShowGraphWindow();
  GraphWindow->ShowGraph(&x, &(*MaterialCollection)[matId]->absWaveBinned, kRed, "Wavelength, nm", "Attenuation coefficient, mm-1");
}

void MainWindow::on_pbShowThisMatInfo_clicked()
{
    MIwindow->show();
    MIwindow->raise();
    MIwindow->activateWindow();
    MIwindow->SetMaterial(ui->cobMatPointSource->currentIndex());
}

void MainWindow::on_sbWaveIndexPointSource_valueChanged(int arg1)
{
  if (arg1 > WaveNodes-1)
    {
      ui->sbWaveIndexPointSource->setValue(WaveNodes-1);
      return;
    }

  QString str;
  if (arg1 == -1)
    {
      str = "--";
    }
  else
    {
      double wave = WaveFrom + WaveStep * arg1;
      str.setNum(wave, 'g', 5);
      str += " nm";
    }
  ui->labWavelengthPointSource->setText(str);
}

void MainWindow::on_pbShowPDE_clicked()
{
   int typ = ui->sbPMtype->value();
   //ShowGraphWindow();
   if (ui->cobPMdeviceType->currentIndex() == 0)
     GraphWindow->ShowGraph(&PMs->getType(typ)->PDE_lambda, &PMs->getType(typ)->PDE, kRed, "Wavelength, nm", "Quantum Efficiency");
   else
     GraphWindow->ShowGraph(&PMs->getType(typ)->PDE_lambda, &PMs->getType(typ)->PDE, kRed, "Wavelength, nm", "Photon Detection Efficiency");
}

void MainWindow::on_pbLoadPDE_clicked()
{
  QString fileName;
  if (ui->cobPMdeviceType->currentIndex() == 0)
     fileName = QFileDialog::getOpenFileName(this, "Load quantum efficiency vs wavelength", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  else
    fileName = QFileDialog::getOpenFileName(this, "Load photon detection efficiency vs wavelength", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  qDebug()<<fileName;

  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector<double> x,y;
  LoadDoubleVectorsFromFile(fileName, &x, &y);

  for (int i=0; i<y.size(); i++)
    if (y[i]<0 || y[i]>1)
      {        
        message("Quantum Efficiency and Photon Detection Probability should be in the range from 0 to 1", this);
        return;
      }

  PMs->updateTypePDE(ui->sbPMtype->value(), &x, &y);
  //PMs->RebinPDEsForType(ui->sbPMtype->value());
  ReconstructDetector(true);

  MainWindow::on_pbRefreshPMproperties_clicked();
}

void MainWindow::on_pbDeletePDE_clicked()
{
  QVector<double> x,y;
  x.resize(0);
  y.resize(0);

  PMs->updateTypePDE(ui->sbPMtype->value(), &x, &y);
  //PMs->RebinPDEsForType(ui->sbPMtype->value());
  ReconstructDetector(true);

  MainWindow::on_pbRefreshPMproperties_clicked();
}

void MainWindow::on_pbScalePDE_clicked()
{
  int type = ui->sbPMtype->value();
  int numTypes = Detector->PMs->countPMtypes();
  if (type>numTypes-1) return;

  bool ok;
  double val = QInputDialog::getDouble(this, "PDE scaling",
                                             "Scale PDE data for PM type "+Detector->PMs->getType(type)->name+"\n"
                                             "Scaling factor:", 1.0, -1e10, 1e10, 5, &ok);
  if (!ok) return;

  Detector->PMs->scaleTypePDE(type, val);
  //PMs->RebinPDEsForType(type);
  ReconstructDetector(true);

  MainWindow::on_pbRefreshPMproperties_clicked();
}

void MainWindow::on_pbShowPDEbinned_clicked()
{
  if (!ui->cbWaveResolved->isChecked())
    {      
      message("First activate wavelength resolved simulation option", this);
      return;
    }

  //force rebinning
  on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked());

  QVector<double> x;
  x.resize(0);
  for (int i=0; i<WaveNodes; i++) x.append(WaveFrom + WaveStep*i);
  //MainWindow::ShowGraphWindow();
  GraphWindow->ShowGraph(&x, &PMs->getType(ui->sbPMtype->value())->PDEbinned, kRed, "Wavelength, nm", "PDE");
}

void MainWindow::on_pbPMtypeLoadAngular_clicked()
{
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load angular response (0 - 90 degrees)", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector<double> x,y;
  LoadDoubleVectorsFromFile(fileName, &x, &y);
  if (x[0] !=0 || x[x.size()-1] != 90)
      {
        message("Data should start at 0 and end at 90 degrees!", this);
        return;
      }
  for (int i=0; i<x.size(); i++)
    if (y[i] < 0.0)
       {
         message("Data should be positive!", this);
         return;
       }
  if (y[0] <1.0e-10)
    {
      message("Response for normal incidence cannot be 0!", this);
      return;
    }
  double norm = 1.0/y[0];
  if (norm != 1.0)
    {
      for (int i=0; i<y.size(); i++) y[i] = y[i]*norm;
      message("Data were scaled to have response of unity at normal incidence", this);
    }
  int typ = ui->cobPMtypes->currentIndex();
  PMs->updateTypeAngular(typ, &x, &y);
  PMs->updateTypeAngularN1(typ, ui->ledMediumRefrIndex->text().toDouble());
  PMs->RecalculateAngularForType(typ);
  ReconstructDetector(true);
  MainWindow::RefreshAngularButtons();
}

void MainWindow::RefreshAngularButtons()
{
  ui->pbPMtypeShowAngular->setEnabled(PMs->getType(ui->sbPMtype->value())->AngularSensitivity_lambda.size());
  ui->pbPMtypeShowEffectiveAngular->setEnabled(ui->cbAngularSensitive->isChecked() && PMs->getType(ui->sbPMtype->value())->AngularSensitivity_lambda.size());
  ui->pbPMtypeDeleteAngular->setEnabled(PMs->getType(ui->sbPMtype->value())->AngularSensitivity_lambda.size());
}

void MainWindow::on_pbPMtypeShowAngular_clicked()
{  
  int typ = ui->cobPMtypes->currentIndex();
  GraphWindow->ShowGraph(&PMs->getType(typ)->AngularSensitivity_lambda, &PMs->getType(typ)->AngularSensitivity, kRed, "Angle of incidence, degrees", "Response");
}

void MainWindow::on_pbPMtypeDeleteAngular_clicked()
{
  int typ = ui->cobPMtypes->currentIndex();
  QVector<double> x;
  x.resize(0);
  PMs->updateTypeAngular(typ, &x, &x);
  PMs->RecalculateAngularForType(typ);
  MainWindow::RefreshAngularButtons();
  ReconstructDetector(true);
}

void MainWindow::on_pbPMtypeShowEffectiveAngular_clicked()
{  
  int typ = ui->cobPMtypes->currentIndex();

  //force rebinning
  on_cbAngularSensitive_toggled(ui->cbAngularSensitive->isChecked());

  QVector<double> x;
  x.resize(0);
  int CosBins = ui->sbCosBins->value();
  for (int i=0; i<CosBins; i++) x.append(1.0/(CosBins-1)*i);  
  GraphWindow->ShowGraph(&x, &PMs->getType(typ)->AngularSensitivityCosRefracted, kRed, "Cosine of refracted beam", "Response");
}

void MainWindow::on_sbCosBins_valueChanged(int arg1)
{
    PMs->setCosBins(arg1);
    PMs->RecalculateAngular();
}

void MainWindow::onGDMLstatusChage(bool fGDMLactivated)
{
    ui->fDisableTab1->setEnabled(!fGDMLactivated);
    ui->tabArray->setEnabled(!fGDMLactivated);
    ui->fPMtypeDisable1->setEnabled(!fGDMLactivated);
    ui->ledSizeX->setEnabled(!fGDMLactivated);
    ui->ledSizeY->setEnabled(!fGDMLactivated);
    ui->pbAddNewPMtype->setEnabled(!fGDMLactivated);
    ui->pbRemoveThisPMtype->setEnabled(!fGDMLactivated);
    ui->fDisableIndividual->setEnabled(!fGDMLactivated);
    ui->pbIndPmRemove->setEnabled(!fGDMLactivated);
    ui->pbRemoveMaterial->setEnabled(!fGDMLactivated); //cannot remove materials if GDML is loaded

    ui->pbGDML->setVisible(fGDMLactivated);   
}

void MainWindow::on_ledMediumRefrIndex_editingFinished()
{
    PMs->updateTypeAngularN1(ui->sbPMtype->value(), ui->ledMediumRefrIndex->text().toDouble());
    PMs->RecalculateAngularForType(ui->sbPMtype->value());
    ReconstructDetector(true);
}

void MainWindow::on_cobTubeBox_currentIndexChanged(int index)
{
  if (index == 2) ui->fPolygon->setVisible(true);
  else ui->fPolygon->setVisible(false);

  if (index == 1) ui->labDetSizeDiameter->setText("Diameter");
  else ui->labDetSizeDiameter->setText("Size");

  if (index == 0) ui->fRectang->setVisible(true);
  else ui->fRectang->setVisible(false);
}

void MainWindow::on_pbPMtypeLoadArea_clicked()
{
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load area response", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector< QVector <double> > tmp;
  double xStep, yStep;
  int error = MainWindow::LoadAreaResponse(fileName, &tmp, &xStep, &yStep);

  if (error == 0)
    {
      PMs->updateTypeArea(ui->sbPMtype->value(), &tmp, xStep, yStep);
      MainWindow::RefreshAreaButtons();
    }
  ReconstructDetector(true);
}

void MainWindow::RefreshAreaButtons()
{
  ui->pbPMtypeShowArea->setEnabled(PMs->getType(ui->sbPMtype->value())->AreaSensitivity.size());
  ui->pbPMtypeDeleteArea->setEnabled(PMs->getType(ui->sbPMtype->value())->AreaSensitivity.size());
}

void MainWindow::on_pbPMtypeShowArea_clicked()
{
  int typ = ui->sbPMtype->value();

  //PMpropertiesStruct tp = *PMs->getTypeProperties(typ);

  if (PMs->getType(typ)->AreaSensitivity.isEmpty()) return;
  int xNum = PMs->getType(typ)->AreaSensitivity.size();
  int yNum = PMs->getType(typ)->AreaSensitivity[0].size();
  double xStep=PMs->getType(typ)->AreaStepX;
  double yStep=PMs->getType(typ)->AreaStepY;

//  qDebug()<<xNum<<xStep<<yNum<<yStep;

  if (hist2D) delete hist2D;
  hist2D = new TH2D("T-AreaSens","Response, %",xNum, -0.5*xNum*xStep, +0.5*xNum*xStep, yNum, -0.5*yNum*yStep, +0.5*yNum*yStep);

  for (int iX=0; iX<xNum; iX++)
   for (int iY=0; iY<yNum; iY++)
    {
       hist2D->SetCellContent(iX+1, iY+1, PMs->getType(typ)->AreaSensitivity[iX][iY]*100.0);
    }

  hist2D->SetXTitle("X, mm");
  hist2D->SetYTitle("Y, mm");
  GraphWindow->Draw(hist2D, "colz");
}

void MainWindow::on_pbPMtypeDeleteArea_clicked()
{
   PMs->clearTypeArea(ui->sbPMtype->value());
   MainWindow::RefreshAreaButtons();
   ReconstructDetector(true);
}

void MainWindow::on_cobPMarrayRegularity_currentIndexChanged(int index)
{

  //variable set
  int ul = ui->cobUpperLowerPMs->currentIndex();
  Detector->PMarrays[ul].Regularity = index;

  //UI update
  //if (index < 2) ui->fRegularAndAutoz->setVisible(true);
  //else ui->fRegularAndAutoz->setVisible(false);

  if (index == 0) ui->swPMarray->setCurrentIndex(0);
  else ui->swPMarray->setCurrentIndex(1);

  if (index == 1)
    {
      ui->pbLoadPMcenters->setText("Load PM positions (X,Y)");
      ui->pbSavePMcenters->setText("Save center positions (X,Y)");
    }
  else if (index == 2)
    {
      ui->pbLoadPMcenters->setText("Load PM positions (X,Y,Z)");
      ui->pbSavePMcenters->setText("Save center positions (X,Y,Z)");
    }

  if (index == 0) if (Detector->PMdummies.size()>0) Detector->PMdummies.resize(0); //dummies are not allowed :)

  if (index < 2)
    {
      /*
      //remove all PMs which do not belong to upper or lower  ///obsolete?
      for (int i=PMs->count()-1; i>-1; i--)
        if (PMs->upperLower(i) == -1) PMs->remove(i);
        */

      //on changed from irregular to any regular - has to rebuild the detector
      MainWindow::on_pbRefreshPMArrayData_clicked();
    }
}

int MainWindow::PMArrayType(int ul)
{
  //return ui->cobPMarrayRegularity->currentIndex();
  return Detector->PMarrays[ul].Regularity;
}

void MainWindow::SetPMarrayType(int ul, int itype)
{
  //ui->cobPMarrayRegularity->setCurrentIndex(itype);
  Detector->PMarrays[ul].Regularity = itype;
}

void MainWindow::on_sbIndPMnumber_valueChanged(int arg1)
{
  if (arg1 == 0   &&   PMs->count() ==0) return;
  if (arg1 > PMs->count()-1)
    {
      if (PMs->count() == 0)
        {
          ui->fIndPM->setEnabled(false);
          ui->sbIndPMnumber->setValue(0);
          return;
        }
      ui->sbIndPMnumber->setValue(PMs->count()-1);
      return;
    }

  ui->fIndPM->setEnabled(true);
  MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndPMshowInfo_clicked()
{
    //qDebug() << "--PM explorer indication--";
    if (PMs->count() == 0) return;
    int ipm = ui->sbIndPMnumber->value();
    int ul, index;
    Detector->findPM(ipm, ul, index);
    if (index<0) return;
    PosAngTypeStructure *p = &Detector->PMarrays[ul].PositionsAnglesTypes[index];
    pm *PM = &PMs->at(ipm);

    //format should be the same in MainWindow::on_pbUpdateToFixedZ_clicked() and MainWindow::on_pbUpdateToFullCustom_clicked()
    ui->ledIndPMx->setText(QString::number(PM->x, 'g', 4));
    ui->ledIndPMy->setText(QString::number(PM->y, 'g', 4));
    ui->ledIndPMz->setText(QString::number(PM->z, 'g', 4));

    ui->ledIndPMphi->setText(QString::number(PM->phi, 'g', 4));
    ui->ledIndPMtheta->setText(QString::number(PM->theta, 'g', 4));
    ui->ledIndPMpsi->setText(QString::number(PM->psi, 'g', 4));
    ui->cobPMtypeInExplorers->setCurrentIndex(PM->type);

    //-- overrides --
    //effective DE    
    double eDE = PMs->getPDEeffective(ipm);
    QString str;
    PMtypeClass* type = PMs->getType(p->type);
    if (eDE == -1)
      {
        str.setNum(type->effectivePDE);
        ui->labIndDEStatus->setText("Inherited:");
        ui->pbIndRestoreEffectiveDE->setEnabled(false);
      }
    else
      {
        str.setNum(eDE, 'g', 4);
        ui->labIndDEStatus->setText("<b>Override:</b>");
        ui->pbIndRestoreEffectiveDE->setEnabled(true);
      }
    ui->ledIndEffectiveDE->setText(str);
    //wave-resolved DE
    if (PMs->isPDEwaveOverriden(ipm))
      {
        ui->labIndDEwaveStatus->setText("<b>Override:</b>");
        ui->pbIndRestoreDE->setEnabled(true);
        ui->pbIndShowDE->setEnabled(true);
        if (ui->cbWaveResolved->isChecked()) ui->pbIndShowDEbinned->setEnabled(true);
        else ui->pbIndShowDEbinned->setEnabled(false);
      }
    else
      {       
       ui->pbIndRestoreDE->setEnabled(false);
       if (type->PDE.size()>0)
         {
          ui->labIndDEwaveStatus->setText("Inherited:");
          ui->pbIndShowDE->setEnabled(true);          
          if (ui->cbWaveResolved->isChecked()) ui->pbIndShowDEbinned->setEnabled(true);
          else ui->pbIndShowDEbinned->setEnabled(false);
         }
       else
         {
          ui->labIndDEwaveStatus->setText("Not defined");
          ui->pbIndShowDE->setEnabled(false);
          ui->pbIndShowDEbinned->setEnabled(false);
         }
    }
    //Angular
    if (PMs->isAngularOverriden(ipm))
      {
        ui->labIndAngularStatus->setText("<b>Override:</b>");
        ui->pbIndRestoreAngular->setEnabled(true);
        ui->pbIndShowAngular->setEnabled(true);
        if (ui->cbAngularSensitive->isChecked()) ui->pbIndShowEffectiveAngular->setEnabled(true);
        else ui->pbIndShowEffectiveAngular->setEnabled(false);
      }
    else
      {
        ui->pbIndRestoreAngular->setEnabled(false);
        if (type->AngularSensitivity.size()>0)
          {
           ui->labIndAngularStatus->setText("Inherited:");
           ui->pbIndShowAngular->setEnabled(true);
           if (ui->cbAngularSensitive->isChecked()) ui->pbIndShowEffectiveAngular->setEnabled(true);
           else ui->pbIndShowEffectiveAngular->setEnabled(false);
          }
        else
          {
           ui->labIndAngularStatus->setText("Not defined");
           ui->pbIndShowAngular->setEnabled(false);
           ui->pbIndShowEffectiveAngular->setEnabled(false);
          }
      }
    //Area
    if (PMs->isAreaOverriden(ipm))
      {
         ui->labIndAreaStatus->setText("<b>Override:</b>");
         ui->pbIndRestoreArea->setEnabled(true);
         ui->pbIndShowArea->setEnabled(true);
      }
    else
      {         
         ui->pbIndRestoreArea->setEnabled(false);
         if (type->AreaSensitivity.size()>0)
           {
             ui->labIndAreaStatus->setText("Inherited:");
             ui->pbIndShowArea->setEnabled(true);
           }
         else
           {
             ui->labIndAreaStatus->setText("Not defined");
             ui->pbIndShowArea->setEnabled(false);
           }
      }

    //show this PM
    if (ui->tabWidget->currentIndex()==3 && ui->tabwidMain->currentIndex()==0)
      {        
        Detector->GeoManager->ClearTracks();      
        double length = type->SizeX*0.5;
        Int_t track_index = Detector->GeoManager->AddTrack(1,22);
        TVirtualGeoTrack *track = Detector->GeoManager->GetTrack(track_index);      
        track->AddPoint(PM->x-length, PM->y, PM->z, 0);
        track->AddPoint(PM->x+length, PM->y, PM->z, 0);
        track->AddPoint(PM->x, PM->y, PM->z, 0);
        track->AddPoint(PM->x, PM->y+length, PM->z, 0);
        track->AddPoint(PM->x, PM->y-length, PM->z, 0);
        track->AddPoint(PM->x, PM->y, PM->z, 0);
        track->AddPoint(PM->x, PM->y, PM->z+length, 0);
        track->AddPoint(PM->x, PM->y, PM->z-length, 0);
        track->SetLineWidth(4);
        track->SetLineColor(kBlack);

        //show orientation
        double l[3] = {0,0,length}; //loacl coordinates (with PM)
        double m[3]; //master coordinates (world)
        TGeoRotation Rot = TGeoRotation("Rot", PM->phi, PM->theta, PM->psi);
        Rot.LocalToMaster(l, m);
        track_index = Detector->GeoManager->AddTrack(1,22);
        track = Detector->GeoManager->GetTrack(track_index);
        track->AddPoint(PM->x, PM->y, PM->z, 0);
        track->AddPoint(PM->x+m[0], PM->y+m[1], PM->z+m[2], 0);
        track->SetLineWidth(4);
        track->SetLineColor(kRed);

        MainWindow::ShowTracks();
      }
}

void MainWindow::on_tabWidget_currentChanged(int index)
{  
   if (index == 3)
     {
       if (PMs->count() == 0) ui->fIndPM->setEnabled(false);
       else ui->fIndPM->setEnabled(true);
       MainWindow::on_sbIndPMnumber_valueChanged(ui->sbIndPMnumber->value());
     }  
}

void MainWindow::on_pbUpdateToFullCustom_clicked()
{
  if (TriggerForbidden) return;
  int ipm = ui->sbIndPMnumber->value();
  int ul, index;
  Detector->findPM(ipm, ul, index);
  if (ipm<0) return; //protection: something is wrong

  PosAngTypeStructure *p = &Detector->PMarrays[ul].PositionsAnglesTypes[index];

  //protection agains click_in-click_out trigger causing warning about leaving regular array
  //and we want to avoid problems due to rounding of doubles
  bool flagMod=false;
  QString gui, data;
  gui = ui->ledIndPMz->text();
  gui.setNum(gui.toDouble(), 'g', 4); //logically the same e.g. 15 and 15.0
  data.setNum(p->z, 'g', 4); // format should match format used in MainWindow::on_pbIndPMshowInfo_clicked()
  if (gui != data) flagMod = true;
  gui = ui->ledIndPMtheta->text();
  gui.setNum(gui.toDouble(), 'g', 4); //logically the same e.g. 15 and 15.0
  data.setNum(p->theta, 'g', 4); // format should match format used in MainWindow::on_pbIndPMshowInfo_clicked()
  if (gui != data) flagMod = true;
  gui = ui->ledIndPMphi->text();
  gui.setNum(gui.toDouble(), 'g', 4); //logically the same e.g. 15 and 15.0
  data.setNum(p->phi, 'g', 4); // format should match format used in MainWindow::on_pbIndPMshowInfo_clicked()
  if (gui != data) flagMod = true;

  if (!flagMod) return; //nothing was actually modified
  if (Detector->PMarrays[ul].Regularity < 2)
    {
      TriggerForbidden = true;
      int ret = QMessageBox::question(this, "Array type change confirmation",
                                      "This modification will require to change PM Array mode\n"
                                      "to 'Fully custom'.\n"
                                      "Do you confirm the change?",
                                      QMessageBox::Yes | QMessageBox::No);
      TriggerForbidden = false;
      if (ret == QMessageBox::No)
        {
          MainWindow::on_pbIndPMshowInfo_clicked();
          return;
        }
    }
  //modification was autorized
  p->z = ui->ledIndPMz->text().toDouble();
  p->theta = ui->ledIndPMtheta->text().toDouble();
  p->phi = ui->ledIndPMphi->text().toDouble();
  if (Detector->PMarrays[ul].Regularity != 2) Detector->PMarrays[ul].Regularity = 2;

  MainWindow::ReconstructDetector();
  MainWindow::on_pbIndPMshowInfo_clicked();
  MainWindow::on_pbShowPMsArrayRegularData_clicked();
}

void MainWindow::on_pbUpdateToFixedZ_clicked()
{
  if (TriggerForbidden) return;
  int ipm = ui->sbIndPMnumber->value();
  int ul, index;
  Detector->findPM(ipm, ul, index);
  if (ipm<0) return; //protection: something is wrong

  PosAngTypeStructure *p = &Detector->PMarrays[ul].PositionsAnglesTypes[index];

  //protection agains click_in-click_out trigger causing warning about leaving regular array
  //and we want to avoid problems due to rounding of doubles
  bool flagMod=false;
  QString gui, data;
  gui = ui->ledIndPMx->text();
  gui.setNum(gui.toDouble(), 'g', 4); //logically the same e.g. 15 and 15.0
  data.setNum(p->x, 'g', 4); // format should match format used in MainWindow::on_pbIndPMshowInfo_clicked()
  if (gui != data) flagMod = true;
  gui = ui->ledIndPMy->text();
  gui.setNum(gui.toDouble(), 'g', 4);
  data.setNum(p->y, 'g', 4);
  if (gui != data) flagMod = true;
  gui = ui->ledIndPMpsi->text();
  gui.setNum(gui.toDouble(), 'g', 4);
  data.setNum(p->psi, 'g', 4);
  if (gui != data) flagMod = true;
  if (ui->cobPMtypeInExplorers->currentIndex() != p->type) flagMod = true;

  if (!flagMod) return; //nothing was actually modified
  if (Detector->PMarrays[ul].Regularity == 0)
    {
      TriggerForbidden = true;
      int ret = QMessageBox::question(this, "Array type change confirmation",
                                      "This modification will require to change PM Array mode\n"
                                      "to 'Custom with auto-Z'.\n"
                                      "Do you confirm the change?",
                                      QMessageBox::Yes | QMessageBox::No);
      TriggerForbidden = false;
      if (ret == QMessageBox::No)
        {
          MainWindow::on_pbIndPMshowInfo_clicked();
          return;
        }
    }
  //modification was autorized
  p->x = ui->ledIndPMx->text().toDouble();
  p->y = ui->ledIndPMy->text().toDouble();
  p->psi = ui->ledIndPMpsi->text().toDouble();  
  p->type = ui->cobPMtypeInExplorers->currentIndex();
  if (Detector->PMarrays[ul].Regularity == 0) Detector->PMarrays[ul].Regularity = 1; //if it is 1 or 2 - leave as is

  MainWindow::ReconstructDetector(true);
  MainWindow::on_pbIndPMshowInfo_clicked();
  MainWindow::on_pbShowPMsArrayRegularData_clicked();
}

void MainWindow::on_pbIndPmRemove_clicked()
{
  QMessageBox msgBox(this);
  msgBox.setWindowTitle("Confirmation");
  msgBox.setText("Remove this PM?");
  msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
  msgBox.setDefaultButton(QMessageBox::Yes);
  if (msgBox.exec() == QMessageBox::No) return;

  int ipm = ui->sbIndPMnumber->value();
  int ul, index;
  Detector->findPM(ipm, ul, index);
  if (index < 0) return;
  if (Detector->PMarrays[ul].Regularity == 0) Detector->PMarrays[ul].Regularity = 1;
  Detector->PMarrays[ul].PositionsAnglesTypes.remove(index);

  MainWindow::ReconstructDetector();

  ipm--;
  if (ipm<0) ipm =0;
  ui->sbIndPMnumber->setValue(ipm);
  MainWindow::ShowPMcount();
  MainWindow::on_pbIndPMshowInfo_clicked();
  MainWindow::on_pbShowPMsArrayRegularData_clicked();
  ReconstructDetector(false);
}

void MainWindow::on_pbIndShowType_clicked()
{
  ui->tabWidget->setCurrentIndex(1);
  ui->sbPMtype->setValue(ui->cobPMtypeInExplorers->currentIndex());
}

void MainWindow::on_ledIndEffectiveDE_editingFinished()
{
    double val = ui->ledIndEffectiveDE->text().toDouble();
    PMs->setPDEeffective(ui->sbIndPMnumber->value(), val);
    ReconstructDetector(true);
    MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndRestoreEffectiveDE_clicked()
{
    PMs->setPDEeffective(ui->sbIndPMnumber->value(), -1);
    ReconstructDetector(true);
    MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndShowDE_clicked()
{  
  int ipm = ui->sbIndPMnumber->value();
  int typ = ui->cobPMtypeInExplorers->currentIndex();

  //ShowGraphWindow();
  if (PMs->isPDEwaveOverriden(ipm))
    {
      if (PMs->isSiPM(ipm))
        GraphWindow->ShowGraph(PMs->getPDE_lambda(ipm), PMs->getPDE(ipm), kRed, "Wavelength, nm", "Photon Detection Efficiency");
      else
        GraphWindow->ShowGraph(PMs->getPDE_lambda(ipm), PMs->getPDE(ipm), kRed, "Wavelength, nm", "Quantum Efficiency");
    }
  else
    {
      if (PMs->isSiPM(ipm))
        GraphWindow->ShowGraph(&PMs->getType(typ)->PDE_lambda, &PMs->getType(typ)->PDE, kRed, "Wavelength, nm", "Photon Detection Efficiency");
      else
        GraphWindow->ShowGraph(&PMs->getType(typ)->PDE_lambda, &PMs->getType(typ)->PDE, kRed, "Wavelength, nm", "Quantum Efficiency");
    }  
}

void MainWindow::on_pbIndRestoreDE_clicked()
{
    QVector<double> x;
    x.resize(0);
    int ipm = ui->sbIndPMnumber->value();
    PMs->setPDEwave(ipm, &x, &x);
    PMs->setPDEbinned(ipm, &x);
    ReconstructDetector(true);

    MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndLoadDE_clicked()
{
  int ipm = ui->sbIndPMnumber->value();

  QString fileName;
  if (PMs->isSiPM(ipm))
     fileName = QFileDialog::getOpenFileName(this, "Load photon detection efficiency vs wavelength", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  else
     fileName = QFileDialog::getOpenFileName(this, "Load quantum efficiency vs wavelength", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");

  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector<double> x,y;
  LoadDoubleVectorsFromFile(fileName, &x, &y);

  for (int i=0; i<y.size(); i++)
    if (y[i]<0 || y[i]>1)
      {
        message("Quantum Efficiency and Photon Detection Probability should be in the range from 0 to 1", this);
        return;
      }

  PMs->setPDEwave(ipm, &x, &y);
  PMs->RebinPDEsForPM(ipm);
  ReconstructDetector(true);

  MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndShowDEbinned_clicked()
{
  if (!ui->cbWaveResolved->isChecked())
    {      
      message("First activate wavelength resolved simulation option", this);
      return;
    }
  //force rebinning
  on_cbWaveResolved_toggled(ui->cbWaveResolved->isChecked());

  int ipm = ui->sbIndPMnumber->value();
  int typ = ui->cobPMtypeInExplorers->currentIndex();
  QVector<double> x;
  x.resize(0);
  for (int i=0; i<WaveNodes; i++) x.append(WaveFrom + WaveStep*i);

  //MainWindow::ShowGraphWindow();
  if (PMs->getPDEbinned(ipm)->size() > 0)
    GraphWindow->ShowGraph(&x, PMs->getPDEbinned(ipm), kRed, "Wavelength, nm", "PDE");
  else
    GraphWindow->ShowGraph(&x, &PMs->getType(typ)->PDEbinned, kRed, "Wavelength, nm", "PDE");
}

void MainWindow::on_pbAddPM_clicked()
{
  int ul = ui->cobUpperLowerPMs->currentIndex();
  int ityp = ui->sbPMtype->value();
  Detector->PMarrays[ul].PositionsAnglesTypes.append(PosAngTypeStructure(0,0,0, 0,0,0, ityp));
  MainWindow::ReconstructDetector();
  int ipm = (ul == 0) ?  Detector->PMarrays[0].PositionsAnglesTypes.size()-1 :
                         Detector->PMarrays[0].PositionsAnglesTypes.size() + Detector->PMarrays[1].PositionsAnglesTypes.size()-1;
  ui->sbIndPMnumber->setValue(ipm);
  ui->tabWidget->setCurrentIndex(3);
}

void MainWindow::on_pbIndLoadAngular_clicked()
{
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load angular response (0 - 90 degrees)", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector<double> x,y;
  LoadDoubleVectorsFromFile(fileName, &x, &y);

  if (x[0] !=0 || x[x.size()-1] != 90)
      {
        message("Data should start at 0 and end at 90 degrees!", this);
        return;
      }

  for (int i=0; i<x.size(); i++)
      if (y[i] < 0)
       {
         message("Data should be positive!", this);
         return;
       }

  if (y[0] <1e-10)
    {
      message("Response for normal incidence cannot be 0 (due to the auto-scale it to 1)!", this);
      return;
    }

  double norm = y[0];
  if (norm != 1)
    {
      for (int i=0; i<x.size(); i++) y[i] = y[i]/norm;
      message("Data were scaled to have response of unity at normal incidence", this);
    }

  int ipm = ui->sbIndPMnumber->value();
  PMs->setAngular(ipm, &x, &y);
  PMs->setAngularN1(ipm, ui->ledIndMediumRefrIndex->text().toDouble());

  PMs->RecalculateAngularForPM(ipm);
  ReconstructDetector(true);

  MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndRestoreAngular_clicked()
{
  QVector<double> x;
  x.resize(0);
  int ipm = ui->sbIndPMnumber->value();
  PMs->setAngular(ipm, &x, &x);
  PMs->setAngularBinned(ipm, &x);
  ReconstructDetector(true);

  MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndShowAngular_clicked()
{  
  int ipm = ui->sbIndPMnumber->value();
  int typ = ui->cobPMtypeInExplorers->currentIndex();
  //ShowGraphWindow();
  if (PMs->isAngularOverriden(ipm))
        GraphWindow->ShowGraph(PMs->getAngularSensitivity_lambda(ipm), PMs->getAngularSensitivity(ipm), kRed, "Angle, degrees", "Response");
  else
        GraphWindow->ShowGraph(&PMs->getType(typ)->AngularSensitivity_lambda, &PMs->getType(typ)->AngularSensitivity, kRed, "Angle, degrees", "Response");
}

void MainWindow::on_pbIndShowEffectiveAngular_clicked()
{
  if (!ui->cbAngularSensitive->isChecked())
    {
      message("First activate angular resolved simulation option", this);
      return;
    }

  int ipm = ui->sbIndPMnumber->value();
  int typ = ui->cobPMtypeInExplorers->currentIndex();

  //force rebinning
  on_cbAngularSensitive_toggled(ui->cbAngularSensitive->isChecked());

  QVector<double> x;
  x.resize(0);
  int CosBins = ui->sbCosBins->value();
  for (int i=0; i<CosBins; i++) x.append(1.0/(CosBins-1)*i);

  //MainWindow::ShowGraphWindow();
  if (PMs->getAngularSensitivityCosRefracted(ipm)->size() > 0)
    GraphWindow->ShowGraph(&x, PMs->getAngularSensitivityCosRefracted(ipm), kRed, "Cosine of refracted beam", "Response");
  else
    GraphWindow->ShowGraph(&x, &PMs->getType(typ)->AngularSensitivityCosRefracted, kRed, "Cosine of refracted beam", "Response");
}

void MainWindow::on_ledIndMediumRefrIndex_editingFinished()
{
    int ipm = ui->sbIndPMnumber->value();
    PMs->setAngularN1(ipm, ui->ledIndMediumRefrIndex->text().toDouble());
    PMs->RecalculateAngularForPM(ipm);
    ReconstructDetector(true);

    MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndLoadArea_clicked()
{
    QString fileName;
    fileName = QFileDialog::getOpenFileName(this, "Load area response", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
    if (fileName.isEmpty()) return;
    GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

    int ipm = ui->sbIndPMnumber->value();
    QVector< QVector <double> > tmp;
    double xStep, yStep;
    int error = MainWindow::LoadAreaResponse(fileName, &tmp, &xStep, &yStep);

    if (error == 0) PMs->setArea(ipm, &tmp, xStep, yStep);
    //if (error == 1) message("Error reading the file - cannot open!", this);
    //if (error == 2) message("Error reading the file - wrong format!", this);
    ReconstructDetector(true);

    MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndRestoreArea_clicked()
{
    int ipm = ui->sbIndPMnumber->value();
    QVector<QVector<double> > tmp;
    tmp.resize(0);

    PMs->setArea(ipm, &tmp, 123.0, 123.0); //strange step deliberately
    ReconstructDetector(true);

    MainWindow::on_pbIndPMshowInfo_clicked();
}

void MainWindow::on_pbIndShowArea_clicked()
{
    int ipm = ui->sbIndPMnumber->value();
    int typ = ui->sbPMtype->value();

    int xNum;
    int yNum;
    double xStep;
    double yStep;
    //PMpropertiesStruct tp = *PMs->getTypeProperties(typ);

    if (PMs->isAreaOverriden(ipm))
      {
        xNum = PMs->getAreaSensitivity(ipm)->size();
        yNum = PMs->getAreaSensitivity(ipm)[0].size();
        xStep = PMs->getAreaStepX(ipm);
        yStep = PMs->getAreaStepY(ipm);
      }
    else
      {
        if (PMs->getType(typ)->AreaSensitivity.isEmpty()) return; //then nothing to show
        xNum = PMs->getType(typ)->AreaSensitivity.size();
        yNum = PMs->getType(typ)->AreaSensitivity[0].size();
        xStep=PMs->getType(typ)->AreaStepX;
        yStep=PMs->getType(typ)->AreaStepY;
      }
//    qDebug()<<xNum<<xStep<<yNum<<yStep;

    if (hist2D) delete hist2D;
    hist2D = new TH2D("T-AreaSens","Response, %",xNum, -0.5*xNum*xStep, +0.5*xNum*xStep, yNum, -0.5*yNum*yStep, +0.5*yNum*yStep);

    for (int iX=0; iX<xNum; iX++)
     for (int iY=0; iY<yNum; iY++)
     {
        if (PMs->isAreaOverriden(ipm)) hist2D->SetCellContent(iX+1, iY+1, (PMs->getAreaSensitivity(ipm))->at(iX)[iY]*100.0);
        else hist2D->SetCellContent(iX+1, iY+1, PMs->getType(typ)->AreaSensitivity[iX][iY]*100.0);
     }
    hist2D->SetXTitle("X, mm");
    hist2D->SetYTitle("Y, mm");
    GraphWindow->Draw(hist2D, "colz");
}

void MainWindow::on_cbGunAllowMultipleEvents_toggled(bool checked)
{
    ui->fGunMultipleEvents->setEnabled(checked);
}

void MainWindow::writeLoadExpDataConfigToJson(QJsonObject &json)
{
  QJsonObject js;
  bool fPre = ui->cbPMsignalPreProcessing->isChecked();
  js["Preprocessing"] = fPre;
  if (fPre)
    {
      QJsonArray ar;
      for (int ipm=0; ipm<PMs->count(); ipm++)
        {
          QJsonArray el;
          el.append(PMs->at(ipm).PreprocessingAdd);
          el.append(PMs->at(ipm).PreprocessingMultiply);
          ar.append(el);
        }
      js["AddMulti"] = ar;

      QJsonObject enj;
      bool fEn = ui->cbLoadedDataHasEnergy->isChecked();
      enj["Activated"] = fEn;
      if (fEn)
        {
          enj["Channel"] = ui->sbLoadedEnergyChannelNumber->value();
          enj["Add"] = EventsDataHub->PreprocessingSettings.LoadEnAdd;
          enj["Multi"] = EventsDataHub->PreprocessingSettings.LoadEnMulti;
        }
      js["LoadedEnergy"] = enj;

      QJsonObject posj;
      bool fPos = ui->cbLoadedDataHasPosition->isChecked();
      posj["Activated"] = fPos;
      if (fPos) posj["Channel"] = ui->sbLoadASCIIpositionXchannel->value();
      js["LoadedPosition"] = posj;

      QJsonObject zposj;
      bool fZPos = ui->cbLoadedDataHasZPosition->isChecked();
      zposj["Activated"] = fZPos;
      if (fZPos) zposj["Channel"] = ui->sbLoadASCIIpositionZchannel->value();
      js["LoadedZPosition"] = zposj;

      QJsonObject ij;
      bool fIg = ui->cbIgnoreEvent->isChecked();
      ij["Activated"] = fIg;
      if (fIg)
        {
          ij["Min"] = ui->ledIgnoreThresholdMin->text().toDouble();
          ij["Max"] = ui->ledIgnoreThresholdMax->text().toDouble();
        }
      js["IgnoreThresholds"] = ij;

      js["ConvertToPhotoelectrons"] = ui->cbPrePro_BacktrackElectronics->isChecked();

      js["LoadFirst"] = ( ui->cbLimitNumberEvents->isChecked() ) ? ui->sbFirstNevents->value() : -1;

      if (!ui->leManifestFile->text().isEmpty()) js["ManifestFile"] = ui->leManifestFile->text();
    }
  json["LoadExpDataConfig"] = js;

  SaveJsonToFile(json, "LoadExpDataConfig.json");
}

bool MainWindow::readLoadExpDataConfigFromJson(QJsonObject &json)
{
  PreprocessingSettingsClass *set = &EventsDataHub->PreprocessingSettings;
  set->readFromJson(json, Detector->PMs, ""); //"" - skip processing of manifest

  ui->cbPMsignalPreProcessing->setChecked(set->fActive);
  if (ui->sbPreprocessigPMnumber->value() == 0) on_sbPreprocessigPMnumber_valueChanged(0);
  else ui->sbPreprocessigPMnumber->setValue(0);

  if (set->fActive)
    {
      ui->cbLoadedDataHasEnergy->setChecked(set->fHaveLoadedEnergy);
      if (set->fHaveLoadedEnergy) ui->sbLoadedEnergyChannelNumber->setValue(set->EnergyChannel);

      ui->cbLoadedDataHasPosition->setChecked(set->fHavePosition);
      if (set->fHavePosition) ui->sbLoadASCIIpositionXchannel->setValue(set->PositionXChannel);

      ui->cbLoadedDataHasZPosition->setChecked(set->fHaveZPosition);
      if (set->fHaveZPosition) ui->sbLoadASCIIpositionZchannel->setValue(set->PositionZChannel);

      ui->cbIgnoreEvent->setChecked(set->fIgnoreThresholds);
       if (set->fIgnoreThresholds)
         {
           ui->ledIgnoreThresholdMin->setText( QString::number(set->ThresholdMin) );
           ui->ledIgnoreThresholdMax->setText( QString::number(set->ThresholdMax) );
         }

      ui->cbPrePro_BacktrackElectronics->setChecked(set->fToPhotoelectrons);

      ui->cbLimitNumberEvents->setChecked(set->fLimitNumber);
      if (set->fLimitNumber) ui->sbFirstNevents->setValue(set->LimitMax);

      if (set->fManifest) ui->leManifestFile->setText(set->ManifestFile);
      else ui->leManifestFile->setText("");
    }
  return true;
}

void MainWindow::on_pbLoadPMsignals_clicked()
{
  QStringList fileNames = QFileDialog::getOpenFileNames(this, "Load events from ascii file(s)", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");  
  if (fileNames.isEmpty()) return;
  this->activateWindow();
  this->raise();
  GlobSet->LastOpenDir = QFileInfo(fileNames.first()).absolutePath();

  if (LoadedEventFiles.isEmpty()) MainWindow::ClearData(); // first load - make sure there is no data (e.g. sim events)
  else
    {
#ifdef ANTS_FLANN
   ReconstructionManager->KNNmodule->Filter.clear();
#endif
    }

  QString SnumFiles = QString::number(fileNames.size());
  ui->labFilesProgress->setText("0 / "+SnumFiles);
  for (int i=0; i<fileNames.size(); i++)
    {
      ui->labFilesProgress->setText(QString::number(i+1)+" / "+SnumFiles);
      int loadedEvents = MainWindow::LoadPMsignals(fileNames[i]);           
      if (loadedEvents>0)
        {
          //qDebug()<<"Loaded"<<loadedEvents<<"events from file"<<fileNames[i];
          ui->leoLoadedEvents->setText(QString::number(loadedEvents));
          LoadedEventFiles.append(fileNames[i]);
          QListWidgetItem* item = new QListWidgetItem(fileNames[i]);
          item->setFlags (item->flags () | Qt::ItemIsEditable);
          ui->lwLoadedEventsFiles->addItem(item);
        }
      else ui->leoLoadedEvents->setText("0");
    }

  if (!EventsDataHub->Events.isEmpty())
    {
      ui->cbPMsignalPreProcessing->setEnabled(true);
      on_cbPMsignalPreProcessing_toggled(ui->cbPMsignalPreProcessing->isChecked());
      ui->pbReloadExpData->setEnabled(true);     
      Owindow->RefreshData();
      Rwindow->CheckAvailableEventData();
    }

  ui->leoTotalLoadedEvents->setText(QString::number(EventsDataHub->Events.size()));

  EventsDataHub->squeeze();
}

void MainWindow::on_pbReloadExpData_clicked()
{
  ui->fReloadRequired->setVisible(false);

  if (LoadedEventFiles.isEmpty())
    {
      message("File list is empty!", this);
      MainWindow::DeleteLoadedEvents();
      return;
    }
  MainWindow::DeleteLoadedEvents(true); //keeping the FileList intact!

  QString SnumFiles = QString::number(LoadedEventFiles.size());
  ui->labFilesProgress->setText("0 / "+SnumFiles);
  for (int i=0; i<LoadedEventFiles.size(); i++)
    {
      ui->labFilesProgress->setText(QString::number(i+1)+" / "+SnumFiles);
      int loadedEvents = MainWindow::LoadPMsignals(LoadedEventFiles[i]);
      //if (loadedEvents>0) qDebug()<<"Loaded"<<loadedEvents<<"events from file"<<LoadedEventFiles[i];
      ui->leoLoadedEvents->setText(QString::number(loadedEvents));
    }

  if (!EventsDataHub->Events.isEmpty())
    {      
      Owindow->RefreshData();
      Rwindow->CheckAvailableEventData();
    }

  ui->leoTotalLoadedEvents->setText(QString::number(EventsDataHub->Events.size()));

  EventsDataHub->squeeze();
}

void MainWindow::LoadEventsListContextMenu(const QPoint &pos)
{
    QMenu myMenu;
    myMenu.addSeparator();
    myMenu.addAction("Clone");
    myMenu.addSeparator();
    myMenu.addAction("Remove");
    myMenu.addSeparator();

    QPoint globalPos = ui->lwLoadedEventsFiles->mapToGlobal(pos);
    QAction* selectedItem = myMenu.exec(globalPos);

    if (selectedItem)
    {
        int row = ui->lwLoadedEventsFiles->currentRow();

        if (selectedItem->iconText() == "Clone")
         {
            //clone the line
            if (row>-1)
              {
                QListWidgetItem* newitem = ui->lwLoadedEventsFiles->item(row)->clone();
                ui->lwLoadedEventsFiles->insertItem(row, newitem);
                LoadedEventFiles.insert(row, LoadedEventFiles[row]);
                ui->fReloadRequired->setVisible(true);
              }
         }
        else if (selectedItem->iconText() == "Remove")
         {
            //remove
            if (row>-1)
              {
                delete ui->lwLoadedEventsFiles->item(row);
                LoadedEventFiles.removeAt(row);
                ui->fReloadRequired->setVisible(true);
              }
         }
    }
}


void MainWindow::clearPreprocessingData()
{
    ui->cbPMsignalPreProcessing->setChecked(false);
   // ui->cbLimitNumberEvents->setChecked(false);
    ui->cbPrePro_BacktrackElectronics->setChecked(false);
}

void MainWindow::on_pbDeleteLoadedEvents_clicked()
{
  //EventsDataHub->clear(); //not needed!
  MainWindow::DeleteLoadedEvents(false);
  if (GeometryWindow->isVisible()) MainWindow::ShowGeometry(false);
}

void MainWindow::DeleteLoadedEvents(bool KeepFileList)
{    
    EventsDataHub->Events.resize(0);
    EventsDataHub->TimedEvents.resize(0);

    ui->leoLoadedEvents->setText("");
    ui->leoTotalLoadedEvents->setText("");
    EventsDataHub->fLoadedEventsHaveEnergyInfo = false;
    ui->leoTotalLoadedEvents->setText("");
    ui->leoLoadedEvents->setText("");

    ui->leoTotalLoadedEvents->setText("");
    ui->leoLoadedEvents->setText("");
    ui->lwLoadedSims->clear();

    if (!KeepFileList)
      {
        LoadedEventFiles.clear();
        ui->lwLoadedEventsFiles->clear();
        ui->pbReloadExpData->setEnabled(false);
      }
    //else we only clear loaded data, "Reload" button was pressed!
    EventsDataHub->clearScan();

    EventsDataHub->clearManifest();

#ifdef ANTS_FLANN
   ReconstructionManager->KNNmodule->Filter.clear();
#endif

    if (Detector->GeoManager) Detector->GeoManager->ClearTracks();    
    clearGeoMarkers();
    Rwindow->CheckAvailableEventData(); //will clear the reconstruction
    Owindow->RefreshData();
    ui->fReloadRequired->setVisible(false);
}

void MainWindow::NavigatorWindowClosed()
{
  //qDebug()<<"  Navigator deleted";
  WindowNavigator = 0;
}

void MainWindow::on_pbPreprocessingLoad_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this,
                                                  "Load preprocessing data (ignores first column with PM number)", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt)");

  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  MainWindow::LoadPreprocessingAddMulti(fileName);
  ui->fReloadRequired->setVisible(true);
}

void MainWindow::on_pbPreprocessingSave_clicked()
{
  QString fileName = QFileDialog::getSaveFileName(this,
                                                  "Save preprocessing data", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt)");

  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  MainWindow::SavePreprocessingAddMulti(fileName);
}

void MainWindow::on_cbPMsignalPreProcessing_toggled(bool checked)
{
    ui->fPreprocessing->setEnabled(checked);
}

void MainWindow::on_sbPreprocessigPMnumber_valueChanged(int arg1)
{
  if ( (arg1>PMs->count()) || (arg1 == PMs->count() && !EventsDataHub->fLoadedEventsHaveEnergyInfo) )
    {
      if (arg1 == 0) return;
      ui->sbPreprocessigPMnumber->setValue(0);
      return;
    }  
  QString strA, strM;
  if (arg1 == PMs->count())
    {
      strA.setNum(EventsDataHub->PreprocessingSettings.LoadEnAdd);
      strM.setNum(EventsDataHub->PreprocessingSettings.LoadEnMulti);
    }
  else
    {
      strA.setNum(PMs->at(arg1).PreprocessingAdd);
      strM.setNum(PMs->at(arg1).PreprocessingMultiply);
    }
  ui->ledPreprocessingAdd->setText(strA);
  ui->lePreprocessingMultiply->setText(strM);
}

void MainWindow::on_cobPMshape_currentIndexChanged(int index)
{
  if (index == 1) ui->labSizeDiameterPM->setText("Diameter:");
  else ui->labSizeDiameterPM->setText("Size:");

  if (index == 0) ui->fRectangularPM->setVisible(true);
  else ui->fRectangularPM->setVisible(false);
}

void MainWindow::on_cobPMampGainModel_currentIndexChanged(int index)
{
  ui->swElSignalModel->setCurrentIndex(index);

  MainWindow::on_pbElUpdatePMampGainProperties_clicked();
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_pbElUpdatePMampGainProperties_clicked()
{
   int ipm = ui->sbElPMnumber->value();
   PMs->setSPePHSmode(ipm, ui->cobPMampGainModel->currentIndex());
   PMs->setAverageSignalPerPhotoelectron(ipm, ui->ledAverageSigPhotEl->text().toDouble());
   PMs->setSPePHSsigma(ipm, ui->ledElsigma->text().toDouble());
   PMs->setSPePHSshape(ipm, ui->ledElShape->text().toDouble());
}

void MainWindow::on_pbElGainLoadDistr_clicked()
{
  int ipm = ui->sbElPMnumber->value();
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load Single Photoelectron Pulse Height Spectrum", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector<double> x, y;
  bool error = MainWindow::LoadSPePHSfile(fileName, &x, &y);
  if (error == 0)
    {
      Detector->PMs->setElChanSPePHS(ipm, &x, &y);
      Detector->PMs->preparePHS(ipm);
    }
  else return;

  MainWindow::on_pbUpdateElectronics_clicked(); //reconstruct detector and update indication
  //MainWindow::on_pbElUpdateIndication_clicked(); //indication
}

void MainWindow::on_pbElGainShowDistr_clicked()
{  
  int ipm = ui->sbElPMnumber->value(); 
  PMs->preparePHS(ipm);
  GraphWindow->Draw(PMs->getSPePHShist(ipm));
}

void MainWindow::on_pbElTestGenerator_clicked()
{
  int ipm = ui->sbElPMnumber->value();  
  if (hist1) delete hist1;
  hist1 = new TH1D("tmpHistElTgen", "test", 100, 0, 0);
  for (int i=0; i<ui->sbElTestGeneratorEvents->value(); i++) hist1->Fill(PMs->GenerateSignalFromOnePhotoelectron(ipm)); 
  GraphWindow->Draw(hist1);
}

void MainWindow::on_sbElPMnumber_valueChanged(int arg1)
{
  if (arg1 == 0   &&   PMs->count() ==0) return;
  if (arg1 > PMs->count()-1)
    {
      if (PMs->count() == 0)
        {
          ui->sbElPMnumber->setValue(0);
          return;
        }
      ui->sbElPMnumber->setValue(PMs->count()-1);
      return;
    }
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_pbElUpdateIndication_clicked()
{
  //   qDebug() << "----Electronics update -----------";
  ui->cbEnableSPePHS->setChecked(Detector->PMs->isDoPHS());
  ui->cbEnableElNoise->setChecked(Detector->PMs->isDoElNoise());
  ui->cbEnableADC->setChecked(Detector->PMs->isDoADC());

  int ipm = ui->sbElPMnumber->value(); 
  int NumPMs = PMs->count();
  if (ipm>NumPMs-1 && ipm != 0)
    {
      ui->sbElPMnumber->setValue(0);
      return; //update on_change
    }
  if (NumPMs != 0)
    {
      ui->twElectronics->setEnabled(true);

      ui->leoElType->setText(PMs->getType(PMs->at(ipm).type)->name);
      QString str, str1;
      str.setNum(PMs->getAverageSignalPerPhotoelectron(ipm));
      ui->ledAverageSigPhotEl->setText(str);
      str.setNum(PMs->getSPePHSsigma(ipm));
      ui->ledElsigma->setText(str);
      str.setNum(PMs->getSPePHSshape(ipm));
      ui->ledElShape->setText(str);
      if (PMs->getSPePHShist(ipm)) ui->pbElGainShowDistr->setEnabled(true);
      else ui->pbElGainShowDistr->setEnabled(false);

      str.setNum(PMs->getElNoiseSigma(ipm));
      ui->ledElNoiseSigma->setText(str);

      str.setNum(PMs->getADCmax(ipm));
      ui->ledADCmax->setText(str);
      ui->sbADCbits->setValue(PMs->getADCbits(ipm));

      str.setNum(PMs->getADClevels(ipm)+1);
      str = "levels: " + str;
      str1.setNum(PMs->getADCstep(ipm));
      str += "  signal/level: "+str1;
      ui->leoADCInfo->setText(str);

      int mode = PMs->getSPePHSmode(ipm); //should be the last update to avoid data corruption - it will trigger update values!
      if (mode == 3) ui->ledAverageSigPhotEl->setReadOnly(true);
      else ui->ledAverageSigPhotEl->setReadOnly(false);
      ui->cobPMampGainModel->setCurrentIndex(mode);
    }
  else ui->twElectronics->setEnabled(false);

  ui->ledTimeOfOneMeasurement->setText( QString::number(Detector->PMs->getMeasurementTime()) );
}

void MainWindow::on_pbElCopyGainData_clicked()
{
   int mode = ui->cobElCopyMode->currentIndex();
   if (mode == 0) return;
   int selector = ui->twElectronics->currentIndex(); //0 -SPePHS, 1 - ElNoise, 2 - ADC

   int ipm =  ui->sbElPMnumber->value();
   int typ = PMs->at(ipm).type;
   bool flagSelective = false;
   if (mode == 1) flagSelective = true;

   for (int i=0; i<PMs->count(); i++)
     {
       if (i==ipm) continue;
       if (flagSelective)
         if (typ != PMs->at(i).type) continue;      
       if (selector == 0) PMs->CopySPePHSdata(ipm, i);
       else if (selector == 1) PMs->CopyElNoiseData(ipm, i);
       else PMs->CopyADCdata(ipm, i);
     }

   MainWindow::on_pbUpdateElectronics_clicked();
}

void MainWindow::on_sbADCbits_editingFinished()
{
  int ipm =  ui->sbElPMnumber->value();
  PMs->setADC(ipm, ui->ledADCmax->text().toDouble(), ui->sbADCbits->value());
  MainWindow::on_pbUpdateElectronics_clicked();
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_ledADCmax_editingFinished()
{
  int ipm =  ui->sbElPMnumber->value();
  PMs->setADC(ipm, ui->ledADCmax->text().toDouble(), ui->sbADCbits->value());
  MainWindow::on_pbUpdateElectronics_clicked();
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_cbEnableSPePHS_toggled(bool checked)
{
    ui->fElGains->setEnabled(checked);    
    if (checked) ui->twElectronics->setTabText(0, "Signal from 1 photoelectron  :On ");
    else ui->twElectronics->setTabText(0, "Signal from 1 photoelectron  :Off");   
}

void MainWindow::on_cbEnableADC_toggled(bool checked)
{  
    ui->fADC->setEnabled(checked);    
    if (checked) ui->twElectronics->setTabText(2, "Digitalization  :On ");
    else ui->twElectronics->setTabText(2, "Digitalization  :Off");
}

void MainWindow::on_pbScanDistrLoad_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load custom distribution", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  MainWindow::LoadScanPhotonDistribution(fileName); ///see MainWindowDiskIO.cpp
}

void MainWindow::on_pbScanDistrShow_clicked()
{
  if (!histScan) return;
  GraphWindow->Draw(histScan);
}

void MainWindow::on_pbScanDistrDelete_clicked()
{
  if (histScan)
    {
      delete histScan;
      histScan = 0;
    }
  ui->pbScanDistrShow->setEnabled(false);
  ui->pbScanDistrDelete->setEnabled(false);
}

void MainWindow::on_cobScanNumPhotonsMode_currentIndexChanged(int index)
{
  if (index > 0) ui->pbShowNumPhotSpectrum->setEnabled(true);
  else ui->pbShowNumPhotSpectrum->setEnabled(false);
}

void MainWindow::on_pbShowNumPhotSpectrum_clicked()
{
  if (EventsDataHub->Scan.isEmpty()) return;

  if (hist1) delete hist1;
  hist1 = new TH1D("hist1ShowPhSpectr", "", 100, 0,0);
  for (int iPoint=0; iPoint<EventsDataHub->Scan.size(); iPoint++)
    hist1->Fill(EventsDataHub->Scan[iPoint]->Points[0].energy);

  GraphWindow->Draw(hist1);
}

void MainWindow::on_pbUpdateScanFloodTabWidget_clicked()
{
  MainWindow::PointSource_UpdateTabWidget();
}

void MainWindow::on_pbInitializeScanFloodNoise_clicked()
{
  MainWindow::PointSource_InitTabWidget();
}

void MainWindow::on_twSingleScan_currentChanged(int index)
{  
  if (index == 0) ui->cbScanFloodLimitToPrimaryScint->setVisible(false);
  else ui->cbScanFloodLimitToPrimaryScint->setVisible(true);
}

void MainWindow::on_pbTestCompton_clicked()
{
   MainWindow::TestCompton(); ///MainWindowTests.cpp
}

void MainWindow::on_pbExportDeposition_clicked()
{
  if (EnergyVector.isEmpty())
    {
      message("No data to save!", this);
      return;
    }
  QFileDialog *fileDialog = new QFileDialog;
  fileDialog->setDefaultSuffix("dat");
  QString fileName = fileDialog->getSaveFileName(this,
                                                 "Save energy deposition data", GlobSet->LastOpenDir, "Data files (*.dat);;text files (*.txt);;All files (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QFileInfo file(fileName);
  if(file.suffix().isEmpty()) fileName += ".dat";
  qDebug()<<fileName;

  QFile outputFile(fileName);
  outputFile.open(QIODevice::WriteOnly);

  if(!outputFile.isOpen())
    {
          qDebug() << "- Error, unable to open" << fileName << "for output";
          message("Unable to open file " +fileName+ " for writing!", this);
          return;
     }   
   MainWindow::ExportDeposition(outputFile);
   outputFile.close();
}

void MainWindow::on_pbImportDeposition_clicked()
{
  QString fileName;
  fileName = QFileDialog::getOpenFileName(this, "Load energy deposition data", GlobSet->LastOpenDir, "Data files (*.dat);;text files (*.txt);;All files (*.*)");
  if (fileName.isEmpty()) return;

  QFile file(fileName);
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  if(!file.open(QIODevice::ReadOnly | QFile::Text))
      {          
          message("Error while opening deposition file "+fileName+"\n"+file.errorString(), this);
          return;
      }
  MainWindow::ImportDeposition(file); //MainWindowDiskIO

  file.close();
}

void MainWindow::on_pbTestsInterpolation_clicked()
{
  /*
   int order = ui->sbTestsInterpolationDegree->value() + 1;

   QVector<double> x,y;
   x.resize(0);
   y.resize(0);

   for (int i=0; i<10; i++)
     {
       double xx = TMath::Pi()*i*0.2;
       x.append(xx);
       y.append(sin(xx));
     }

   ShowGraphWindow();
   graphRW->ShowGraph(&x, &y, kBlue, "x", "y", 20, 1, 1, 0);

   TVectorD xx;
   TVectorD yy;
   xx.ResizeTo(100);
   yy.ResizeTo(100);

   //if (order == 2) //linear
   //  {
       for (int i=0; i<100; i++)
         {
           double XX = TMath::Pi()*i*0.02;
           xx[i] = XX;
           double YY;
           if (XX<x.last()) YY = InteractionValue(XX, &x, &y, order-1);
           else YY = y.last();
           yy[i] = YY;
         }
   //  }
   */
   /*
   else
     {  //lagrange
       for (int i=0; i<100; i++)
         {
           double XX = TMath::Pi()*i*0.02;
           xx[i] = XX;
           double YY = PolyInterpolation(XX, &x, &y, order);
           yy[i] = YY;
         }
     }
   */
/*
   TGraph* graph1 = new TGraph(xx,yy);
   graph1->SetLineColor(kRed);
   graph1->SetLineWidth(1);
   graph1->SetTitle("Integral");
   graph1->Draw("SAME L");

   graphRW->Update();
*/
}

void MainWindow::on_ledElNoiseSigma_editingFinished()
{
  int ipm =  ui->sbElPMnumber->value();
  PMs->setElNoiseSigma(ipm, ui->ledElNoiseSigma->text().toDouble());
  MainWindow::on_pbUpdateElectronics_clicked();
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_cbEnableElNoise_toggled(bool checked)
{
  ui->fElNoise->setEnabled(checked);  
  if (checked) ui->twElectronics->setTabText(1, "Electronic noise  :On ");
  else ui->twElectronics->setTabText(1, "Electronic noise  :Off");
}

void MainWindow::on_cobSecScintillationGenType_currentIndexChanged(int index)
{
  if (index == 3) ui->fSecondaryScintLoadProfile->setVisible(true);
  else ui->fSecondaryScintLoadProfile->setVisible(false);

  //PhotonGenerator->setSecScintDirectionType(index);
}

void MainWindow::on_pbSecScintShowProfile_clicked()
{
  //if (!PhotonGenerator->isSecScintDistribution()) return;
  if (!histSecScint) return;
  //GraphWindow->Draw(PhotonGenerator->getSecScintThetaDistribution());
  GraphWindow->Draw(histSecScint);
}

void MainWindow::on_pbSecScintLoadProfile_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load custom distribution", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  MainWindow::LoadSecScintTheta(fileName);
}

void MainWindow::LoadSecScintTheta(QString fileName)
{
  QVector<double> x, y;
  int error = LoadDoubleVectorsFromFile(fileName, &x, &y);
  if (error>0)
    {
      qDebug()<<"Error reading custom Theta distribution file";
      return;
    }

  //PhotonGenerator->deleteSecScintThetaDistribution();
  if (histSecScint) delete histSecScint;
  int size = x.size();
  double* xx = new double [size];
  for (int i = 0; i<size; i++) xx[i]=x[i];//*3.1415926535/180.;
  histSecScint = new TH1D("SecScintTheta","SecScint: Theta", size-1, xx);

  for (int j = 1; j<size+1; j++)  histSecScint->SetBinContent(j, y[j-1]);
  //PhotonGenerator->setSecScintThetaDistribution(histSecScint);

  ui->pbSecScintShowProfile->setEnabled(true);
  ui->pbSecScintDeleteProfile->setEnabled(true);
}

void MainWindow::on_pbSecScintDeleteProfile_clicked()
{
  //PhotonGenerator->deleteSecScintThetaDistribution();
  if (histSecScint) delete histSecScint;
  histSecScint = 0;

  ui->pbSecScintShowProfile->setEnabled(false);
  ui->pbSecScintDeleteProfile->setEnabled(false);
  ui->cobSecScintillationGenType->setCurrentIndex(0);
}

void MainWindow::on_lwMaterials_currentRowChanged(int currentRow)
{
  if (currentRow<0) return;  
  ui->cobMaterialForOverrides->setCurrentIndex(currentRow);
  MainWindow::on_pbRefreshOverrides_clicked();
}

void MainWindow::on_lwMaterials_doubleClicked(const QModelIndex &index)
{
  ui->lwMaterials->setCurrentRow(index.row());
  MIwindow->showNormal();
  MIwindow->raise();
  MIwindow->activateWindow();
  MIwindow->SetMaterial(index.row());
}

void MainWindow::on_pbConfigureAddOns_clicked()
{
  DAwindow->setForbidUpdateValues(false);
  DAwindow->show();
  DAwindow->raise();
  DAwindow->activateWindow();
}

void MainWindow::on_pbLoadSimTree_clicked()
{  
  QStringList fileNames = QFileDialog::getOpenFileNames(this, "Load/Append simulation data from Root tree", GlobSet->LastOpenDir, "Root files (*.root)");
  if (fileNames.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileNames.first()).absolutePath();

  if (ui->lwLoadedSims->count() == 0) MainWindow::ClearData(); //clear ascii loaded data  or sim if present
  else
    {
#ifdef ANTS_FLANN
   ReconstructionManager->KNNmodule->Filter.clear();
#endif
    }

  for (int i=0; i<fileNames.size(); i++) MainWindow::LoadSimulationDataFromTree(fileNames[i]);
  Rwindow->CheckAvailableEventData();
}

void MainWindow::timerTimeout()
{
    //qDebug() << "root in";
 //   RootApp->StartIdleing();
    //gSystem->InnerLoop(); - crashes!
    gSystem->ProcessEvents();
//    RootApp->StopIdleing();
    //qDebug() << "root out";
}

void MainWindow::on_ledPreprocessingAdd_editingFinished()
{
  int channel = ui->sbPreprocessigPMnumber->value();
  if ( (channel>PMs->count()) || (channel == PMs->count() && !EventsDataHub->fLoadedEventsHaveEnergyInfo) )
    {
      if (channel == 0) return;
      ui->sbPreprocessigPMnumber->setValue(0);
      return;
    }
  if (channel = PMs->count()) EventsDataHub->PreprocessingSettings.LoadEnAdd = ui->ledPreprocessingAdd->text().toDouble();
  else PMs->at(channel).PreprocessingAdd = ui->ledPreprocessingAdd->text().toDouble();
}


void MainWindow::on_lePreprocessingMultiply_editingFinished()
{
  int channel = ui->sbPreprocessigPMnumber->value();
  if ( (channel>PMs->count()) || (channel == PMs->count() && !EventsDataHub->fLoadedEventsHaveEnergyInfo) )
    {
      if (channel == 0) return;
      ui->sbPreprocessigPMnumber->setValue(0);
      return;
    }

  QString text = ui->lePreprocessingMultiply->text();
  //parse for a simple ratio ("double"/"double")
  QStringList fields = text.split("/", QString::SkipEmptyParts);

  double finalValue = 1.0;
  if (fields.size() == 0 )
    {
      ui->lePreprocessingMultiply->setText("1");
      message("A value or ratio expected! Resetting to 1");
    }
  else if (fields.size() == 1)
    {
      bool ok;
      finalValue = text.toDouble(&ok);
      if (ok)
        {
          if (channel == PMs->count()) EventsDataHub->PreprocessingSettings.LoadEnMulti = ui->lePreprocessingMultiply->text().toDouble();
          else PMs->at(channel).PreprocessingMultiply = ui->lePreprocessingMultiply->text().toDouble();
          return; //== done ==
        }
      else
        {
          ui->lePreprocessingMultiply->setText("1");
          message("A value or ratio expected! Resetting to 1");
          finalValue = 1.0;
        }      
    }
  else if (fields.size() == 2)
    {
       //processing ratio
       bool ok1, ok2;
       double top, bottom;
       top = fields[0].toDouble(&ok1);
       bottom = fields[1].toDouble(&ok2);
       if (!ok1 || !ok2 )
         {
           ui->lePreprocessingMultiply->setText("1");
           message("A value or ratio expected! Resetting to 1");
         }
       else if (bottom == 0)
         {
           ui->lePreprocessingMultiply->setText("1");
           message("Cannot divide by 0! Resetting to 1");
         }
       else finalValue = top / bottom;
    }

  ui->lePreprocessingMultiply->setText(QString::number(finalValue)); //to avoid rounding problems
  on_lePreprocessingMultiply_editingFinished(); //running again
}

void MainWindow::on_extractPedestals_clicked()
{
   if (EventsDataHub->Events.isEmpty())
     {
       message("Data are not loaded!", this);
       return;
     }
   if (ui->cbLoadedDataHasEnergy->isChecked() && !EventsDataHub->fLoadedEventsHaveEnergyInfo)
     {
       message("select the energy channel and reload the data!");
       return;
     }

   int ibound = PMs->count();
   if (ui->cbLoadedDataHasEnergy->isChecked()) ibound++;

   //calculating average for each channel
   QVector<double> averageSignal(ibound, 0);
   for (int iev=0; iev < EventsDataHub->Events.count(); iev++)
     for (int ipm=0; ipm<ibound; ipm++)
         averageSignal[ipm] += EventsDataHub->Events[iev][ipm];

   QString str, line;
   for (int ipm=0; ipm<ibound; ipm++)
     {
       averageSignal[ipm] /= EventsDataHub->Events.size();
       str = QString::number(averageSignal[ipm], 'g', 4);
       line += str + " ";
     }
   Owindow->OutText("\nPedestals extracted:\n"+line+"\n");

   for (int ipm=0; ipm<PMs->count(); ipm++) PMs->at(ipm).PreprocessingAdd = -averageSignal[ipm];
   if (ui->cbLoadedDataHasEnergy->isChecked()) EventsDataHub->PreprocessingSettings.LoadEnAdd = -averageSignal.last();
   MainWindow::on_sbPreprocessigPMnumber_valueChanged(ui->sbPreprocessigPMnumber->value());
   ui->fReloadRequired->setVisible(true);
}

void MainWindow::on_sbLoadedEnergyChannelNumber_editingFinished()
{
    int i = ui->sbLoadedEnergyChannelNumber->value();

    if (i < PMs->count())
      {
        ui->sbLoadedEnergyChannelNumber->setValue(PMs->count());
        message("Channel number should be >= number of PMs", this);
        return;
      }
}

void MainWindow::on_cbUseSameSizes_toggled(bool checked)
{
  ui->pbCheckChangeSizes->setEnabled(!checked);
  ui->fSimpleSize->setEnabled(checked);

  MainWindow::on_pbRebuildDetector_clicked();
}

void MainWindow::on_cobPMtypes_currentIndexChanged(int index)
{
    if (DoNotUpdateGeometry) return;
    ui->sbPMtype->setValue(index);
}

void MainWindow::on_pbShowMaterialInPMtypes_clicked()
{
  int mat = ui->cobMatPM->currentIndex();
  MIwindow->show();
  MIwindow->raise();
  MIwindow->activateWindow();
  MIwindow->SetMaterial(mat);

  ui->lwMaterials->setCurrentRow(mat);
}

void MainWindow::on_cobPMtypeInArrays_currentIndexChanged(int index)
{
  if (DoNotUpdateGeometry) return;
  ui->sbPMtypeForGroup->setValue(index);
}

void MainWindow::on_pbSetPMtype_clicked()
{
  QList<int> ToSet;
  bool ok = ExtractNumbersFromQString(ui->lePMnumberForTypeSet->text(), &ToSet);
  if (!ok)
    {
      message("Input error in PM numbers!", this);
      return;
    }

  int counter = 0;
  int itype = ui->cobPMtypeInArraysForCustom->currentIndex();
  int ul = ui->cobUpperLowerPMs->currentIndex();
  for (int i=0; i<ToSet.size(); i++)
    {
      int ipm = ToSet[i];
      if (ipm<0 || ipm > PMs->count()-1) continue;
      if (PMs->at(ipm).upperLower != ul) continue;

      PMs->at(ipm).type = itype;
      counter++;
    }

  message("Type has been changed for "+QString::number(counter)+" PMs", this);

  if (counter>0) MainWindow::ReconstructDetector();
}

void MainWindow::on_pbViewChangeRelQEfactors_clicked()
{
  MainWindow::ViewChangeRelFactors("QE");
}

void MainWindow::on_pbViewChangeRelELfactors_clicked()
{
  MainWindow::ViewChangeRelFactors("EL");
}

void MainWindow::ViewChangeRelFactors(QString options)
{
  QDialog* dialog = new QDialog(this);
  dialog->resize(350,380);
  dialog->move(this->x()+100, this->y()+100);

  if (options == "QE") dialog->setWindowTitle("Relative QE/PDE factors");
  else if (options == "EL") dialog->setWindowTitle("Relative factors of electronic channels");

  QPushButton *okButton = new QPushButton("Confirm");
  connect(okButton,SIGNAL(clicked()),dialog,SLOT(accept()));
  QPushButton *cancelButton = new QPushButton("Cancel");
  connect(cancelButton,SIGNAL(clicked()),dialog,SLOT(reject()));
  QHBoxLayout *buttonsLayout = new QHBoxLayout;
  buttonsLayout->addStretch(1);
  buttonsLayout->addWidget(okButton);
  buttonsLayout->addStretch(1);
  buttonsLayout->addWidget(cancelButton);
  buttonsLayout->addStretch(1);

  QTableWidget *tw = new QTableWidget();
  tw->clearContents();
  //tw->setShowGrid(false);
  int rows = PMs->count();
  tw->setRowCount(rows);

  int columns = 1;
  tw->setColumnCount(columns);
  if (options == "QE") tw->setHorizontalHeaderItem(0, new QTableWidgetItem("Relative QE/PDE"));
  else if (options == "EL") tw->setHorizontalHeaderItem(0, new QTableWidgetItem("Relative factors"));

  for (int i=0; i<rows; i++)
    {
      tw->setVerticalHeaderItem(i, new QTableWidgetItem("PM#"+QString::number(i)));
      if (options == "QE") tw->setItem(i, 0, new QTableWidgetItem(QString::number(PMs->at(i).relQE_PDE)));
      else if (options == "EL") tw->setItem(i, 0, new QTableWidgetItem(QString::number(PMs->getAverageSignalPerPhotoelectron(i))));
    }

  tw->setItemDelegate(new TableDoubleDelegateClass(tw)); //accept only doubles

  //tw->resizeColumnsToContents();
  //tw->resizeRowsToContents();
  //tw->setColumnWidth(0, 56);
  //tw->setColumnWidth(1, 56);

  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(tw);
  mainLayout->addLayout(buttonsLayout);

  dialog->setLayout(mainLayout);
  int result = dialog->exec();
  //qDebug()<<"dialog reports:"<<result;

  if (result == 1)
    {
      Detector->PMs->setDoPHS( true );
      if (options == "QE")
        {
          //updating data
          for (int i=0; i<rows; i++) PMs->at(i).relQE_PDE = tw->item(i, 0)->text().toDouble();

          MainWindow::CalculateIndividualQEPDE();
          ReconstructDetector(true);
        }
      else if (options == "EL")
        {
          //updating data
          for (int i=0; i<rows; i++) PMs->at(i).relElStrength = tw->item(i, 0)->text().toDouble();

          MainWindow::CalculateElChannelsStrength();
          ui->cbEnableSPePHS->setChecked(true);
          ReconstructDetector(true);
          MainWindow::on_pbElUpdateIndication_clicked();
        }
    }

  delete dialog;  
}

void MainWindow::CalculateIndividualQEPDE()
{
  for (int ipm = 0; ipm<PMs->count(); ipm++)
    {
      double factor = PMs->at(ipm).relQE_PDE;
      int itype = PMs->at(ipm).type;

      //scalar value
      double fromType = PMs->getType(itype)->effectivePDE;
      PMs->setPDEeffective(ipm, fromType*factor);

      //Wavelength resolved data
      QVector<double> tmp = PMs->getType(itype)->PDE;
      QVector<double> tmp_lambda = PMs->getType(itype)->PDE_lambda;
      if (!tmp.isEmpty())
        {
          for (int i=0; i<tmp.size(); i++) tmp[i] *= factor;

          PMs->setPDEwave(ipm, &tmp_lambda, &tmp);
          PMs->RebinPDEsForPM(ipm);
        }
    }
}

void MainWindow::CalculateElChannelsStrength()
{
  for (int ipm = 0; ipm<PMs->count(); ipm++)
    {
      double factor = PMs->at(ipm).relElStrength;

      MainWindow::ScaleSPePHS(ipm, factor);
    }
}

void MainWindow::ScaleSPePHS(int ipm, double factor)
{
  double NowAverage = PMs->getAverageSignalPerPhotoelectron(ipm);
  if (NowAverage == factor) return; //nothing to change

  if (fabs(factor) > 1e-20) factor /= NowAverage; else factor = 0;

  int mode = PMs->getSPePHSmode(ipm); //0 - use average value; 1 - normal distr; 2 - Gamma distr; 3 - custom distribution

  if (mode < 3) PMs->setAverageSignalPerPhotoelectron(ipm, NowAverage * factor);
  else if (mode == 3)
    {
      //custom SPePHS - have to adjust the distribution
      int ipm = ui->sbElPMnumber->value();
      QVector<double> x = *PMs->getSPePHS_x(ipm);
      QVector<double> y = *PMs->getSPePHS(ipm);

      for (int ix=0; ix<x.size(); ix++) x[ix] *= factor;

      PMs->setElChanSPePHS(ipm, &x, &y);
    }
}

void MainWindow::on_pbLoadRelQEfactors_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load relative QE / PDE factors", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  qDebug()<<fileName;

  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  Detector->PMs->setDoPHS( true );

  QVector<double> x;
  int ok = LoadDoubleVectorsFromFile(fileName, &x);

  if (ok != 0) return;
  if (x.size() != PMs->count())
    {
      message("Wrong number of PMs!", this);
      return;
    }

  for (int i=0; i<x.size(); i++) PMs->at(i).relQE_PDE = x[i];
  MainWindow::CalculateIndividualQEPDE();
  ReconstructDetector(true);
}

void MainWindow::on_pbLoadRelELfactors_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load relative strength of electronic channels", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*)");
  qDebug()<<fileName;

  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  Detector->PMs->setDoPHS( true );

  QVector<double> x;
  int ok = LoadDoubleVectorsFromFile(fileName, &x);

  if (ok != 0) return;
  if (x.size() != PMs->count())
    {
      message("Wrong number of PMs!", this);
      return;
    }

  for (int i=0; i<x.size(); i++) PMs->at(i).relElStrength = x[i];
  MainWindow::CalculateElChannelsStrength();

  //ui->cbEnableSPePHS->setChecked(true);
  ReconstructDetector(true);
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_pbRandomScaleELaverages_clicked()
{
  double min = ui->ledELavScaleMin->text().toDouble();
  double max = ui->ledELavScaleMax->text().toDouble();
  if (min >= max) return;

  //ui->cbEnableSPePHS->setChecked(true);
  Detector->PMs->setDoPHS( true );

  for (int ipm = 0; ipm<PMs->count(); ipm++)
    {
      double factor = Detector->RandGen->Rndm();
      factor = min + (max-min)*factor;

      MainWindow::ScaleSPePHS(ipm, factor);
    }

  ReconstructDetector(true);
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_pbSetELaveragesToUnity_clicked()
{
  //ui->cbEnableSPePHS->setChecked(true);
  Detector->PMs->setDoPHS( true );

  for (int ipm = 0; ipm<PMs->count(); ipm++) MainWindow::ScaleSPePHS(ipm, 1.0);

  ReconstructDetector(true);
  MainWindow::on_pbElUpdateIndication_clicked();
}

void MainWindow::on_pbShowRelGains_clicked()
{
  Owindow->show();
  Owindow->raise();
  Owindow->activateWindow();
  Owindow->SetTab(0);

  double max = -1e20;
  for (int ipm = 0; ipm<PMs->count(); ipm++)
    {
      double QE = PMs->getPDEeffective(ipm);
      if (QE == -1) QE = PMs->getType( PMs->at(ipm).type )->effectivePDE;
      double AvSig = 1.0;
      if (ui->cbEnableSPePHS->isChecked()) AvSig =  PMs->getAverageSignalPerPhotoelectron(ipm);
      double relStr = QE * AvSig;
      if (relStr > max) max = relStr;
    }
  if (fabs(max)<1.0e-20) max = 1.0;

  QVector<QString> tmp(0);
  Owindow->OutText("");
  Owindow->OutText("PM  Relative_QE   Average_signal_per_photoelectron   Relative_gain");
  for (int ipm = 0; ipm<PMs->count(); ipm++)
    {
      double QE = PMs->getPDEeffective(ipm);
      if (QE == -1) QE = PMs->getType( PMs->at(ipm).type )->effectivePDE;
      QString str = "PM#" + QString::number(ipm) +"> "+ QString::number(QE, 'g', 3);

      double AvSig = 1.0;
      if (ui->cbEnableSPePHS->isChecked()) AvSig =  PMs->getAverageSignalPerPhotoelectron(ipm);
      str += "  " + QString::number(AvSig, 'g', 3);

      double relStr = QE * AvSig / max;
      str += "  " + QString::number(relStr, 'g', 3);
      Owindow->OutText(str);
      tmp.append( QString::number(relStr, 'g', 3) );
    }

  MainWindow::ShowTextOnPMs(tmp, kRed);
}

void MainWindow::on_pbSaveResults_clicked()
{
    if (ui->cbSaveAsText->isChecked()) MainWindow::SaveSimulationDataAsText();
    else MainWindow::SaveSimulationDataTree();

    fSimDataNotSaved = false;
}

void MainWindow::SaveSimulationDataTree()
{
  if (EventsDataHub->Events.isEmpty())
    {
      message("No data to save!", this);
      return;
    }
  QString fileName = QFileDialog::getSaveFileName(this, "Save simulation data as Root Tree", GlobSet->LastOpenDir, "Root files (*.root)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  if(QFileInfo(fileName).suffix().isEmpty()) fileName += ".root";

  bool ok = EventsDataHub->saveSimulationAsTree(fileName);
  if (!ok) message("Error writing to file!", this);
}

void MainWindow::SaveSimulationDataAsText()
{
  if (EventsDataHub->Events.isEmpty())
    {
      message("No data to save!", this);
      return;
    }
  QString fileName = QFileDialog::getSaveFileName(this, "Save simulation data as text file", GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();
  if(QFileInfo(fileName).suffix().isEmpty()) fileName += ".dat";

  bool ok = EventsDataHub->saveSimulationAsText(fileName);
  if (!ok) message("Error writing to file!", this);
}

void MainWindow::on_pbRemoveMaterial_clicked()
{
  int imat = ui->lwMaterials->currentRow();
  if (imat == -1)
    {
      message("Select a material to remove!", this);
      return;
    }
  TGeoVolume *vol;
  //check first that this material is in use
  TObjArray* list = Detector->GeoManager->GetListOfVolumes();
  int size = list->GetEntries();
  //qDebug() << "Num volumes:"<<size;
  bool found = false;
  for (int i=0; i<size; i++)
    {
      vol = (TGeoVolume*)list->At(i);
      //qDebug() << "index"<<i<<vol->GetName();
      //qDebug() <<  "   mat name:"<<vol->GetMaterial()->GetName()<< vol->GetMaterial()->GetIndex();
      if (!vol) break;
      if (imat == vol->GetMaterial()->GetIndex())
        {
          found = true;
          break;
        }
    }
  if (found)
    {
      message("Cannot remove this material - it is in use (volume: "+QString(vol->GetName())+")", this);
      return;
    }

  //ask for confirmation
  switch( QMessageBox::information( this, "Confirmation", "Delete material " + (*MaterialCollection)[imat]->name + "?","Remove", "Cancel",0, 1 ) )
    {
     case 0: //OK received;
        break;
     default:
      return;
    }
  qDebug()<<"Removing!";
  MaterialCollection->DeleteMaterial(imat);
  qDebug()<<"material collection - done";

  //update detector elements  
   //PM types
  for (int itype=0; itype<PMs->countPMtypes(); itype++)
    {
      //int tmat = PMs->getTypeProperties(itype)->getMaterialIndex();
      int tmat = PMs->getType(itype)->MaterialIndex;
      //if (tmat > imat) PMs->getTypeProperties(itype)->setMaterialIndex(tmat-1);
      if (tmat > imat) PMs->getType(itype)->MaterialIndex = tmat-1;
    }
   //Add objects
  for (int iobj=0; iobj<Detector->AdditionalGeoObjects.size(); iobj++)
    {
      int tmat = Detector->AdditionalGeoObjects[iobj].Material;
      if (tmat > imat) Detector->AdditionalGeoObjects[iobj].Material--;
    }
   //masks
  for (int im=0; im<Detector->Masks.size(); im++)
    {
      MaskClass& m = Detector->Masks[im];
      if (m.MaterialBulk  > imat) m.MaterialBulk--;
      if (m.MaterialHoles > imat) m.MaterialHoles--;
    }
   //Lightguides
  for (int i=0; i<2; i++)
    {
      if (Detector->Lightguides[i].BulkMaterial > imat) Detector->Lightguides[i].BulkMaterial--;
      if (Detector->Lightguides[i].OpeningsMaterial > imat) Detector->Lightguides[i].OpeningsMaterial--;
    }
   //Dummy PMs use normal PM types - no need to update

  //now shift all materials
  MainWindow::RemoveMaterialFromCobs(imat); //also recreate sandwich and then builds detector
  MIwindow->SetMaterial(0);
  //ReconstructDetector();

  //updating materials indication on mainwindow
  MainWindow::UpdateMaterialListEdit();
  ui->cobMaterialForOverrides->setCurrentIndex(0);
  MainWindow::on_pbRefreshOverrides_clicked(); //overrides are ready, just refresh viz
}

void MainWindow::on_lwLoadedEventsFiles_itemChanged(QListWidgetItem *item)
{
  int row = ui->lwLoadedEventsFiles->currentRow();
  LoadedEventFiles[row] = item->text();
  ui->fReloadRequired->setVisible(true);
}

void MainWindow::LRF_ModuleReadySlot(bool ready)
{
//   qDebug()<<ready;
   ui->fLRFs->setEnabled(ready);
   if (!ready) ui->cbLRFs->setChecked(false);
}

void MainWindow::on_cbLRFs_toggled(bool checked)
{
   int i=0;
   if (checked) i=1;
   ui->swLRFs->setCurrentIndex(i);

   if (checked) ui->cbScanFloodAddNoise->setChecked(false);
   ui->cbScanFloodAddNoise->setEnabled(!checked);
}

void MainWindow::on_pbClearAdd_clicked()
{
  int i = QMessageBox::question( this, "Warning!", "Clear 'Add' values for all channels?", "Yes", "Cancel", 0, 1 );
  if (i == 1) return;
  for (int i=0; i<PMs->count(); i++) PMs->at(i).PreprocessingAdd = 0;
  EventsDataHub->PreprocessingSettings.LoadEnAdd = 0;
  ui->sbPreprocessigPMnumber->setValue(0);
  MainWindow::on_sbPreprocessigPMnumber_valueChanged(0);
}

void MainWindow::on_pbClearMulti_clicked()
{
  int i = QMessageBox::question( this, "Warning!", "Clear 'Multiply'' values for all channels?", "Yes", "Cancel", 0, 1 );
  if (i == 1) return;
  for (int i=0; i<PMs->count(); i++) PMs->at(i).PreprocessingMultiply = 1.0;
  EventsDataHub->PreprocessingSettings.LoadEnMulti = 1.0;
  ui->sbPreprocessigPMnumber->setValue(0);
  MainWindow::on_sbPreprocessigPMnumber_valueChanged(0);
}

void MainWindow::on_actionCredits_triggered()
{
  Credits cr(this);
  cr.exec();
}

void MainWindow::on_actionVersion_triggered()
{
  int versionNumber = ANTS2_VERSION;
  int minVer = ANTS2_MINOR;
  QString miv = QString::number(minVer);
  if (miv.length() == 1) miv = "0"+miv;
  int majVer = ANTS2_MAJOR;
  QString mav = QString::number(majVer);
  QString qv = QT_VERSION_STR;

  QString out = "ANTS2 version:  " + mav + "." + miv + "\n"
                "Build number:  " + QString::number(versionNumber)+"\n"
                "Build date/time:  "+QString::fromLocal8Bit(BUILDDATE)+"\n"
                "Qt version:  " + qv + "\n"
                "\n"
                "Compilation options:"
                "\nCUDA (gpu):  "
#ifdef __USE_ANTS_CUDA__
  "on"
#else
  "off"
#endif
                "\nFANN (neural networks):  "
#ifdef ANTS_FANN
  "on"
#else
  "off"
#endif
                "\nFLANN (kNN search):  "
#ifdef ANTS_FLANN
  "on"
#else
  "off"
#endif
                "";

  message(out, this);
}

void MainWindow::on_actionLicence_triggered()
{
  message("\nANTS2 is distributed under GNU Lesser General Public License version 3\n"
          "For details see http://www.fsf.org/", this);
}

void MainWindow::on_pnShowHideAdvanced_toggled(bool checked)
{
   if (checked) ui->swAdVancedSim->setCurrentIndex(1);
           else ui->swAdVancedSim->setCurrentIndex(0);
}

void MainWindow::on_pbYellow_clicked()
{
   bool fYellow = false;

   if (!ui->cbRandomDir->isChecked())
     {
       fYellow = true;
       ui->twAdvSimOpt->tabBar()->setTabIcon(0, Rwindow->YellowIcon);
     }
   else ui->twAdvSimOpt->tabBar()->setTabIcon(0, QIcon());

   if (ui->cbNumberOfRuns->isChecked())
     {
       fYellow = true;
       ui->twAdvSimOpt->tabBar()->setTabIcon(1, Rwindow->YellowIcon);
     }
   else ui->twAdvSimOpt->tabBar()->setTabIcon(1, QIcon());

   if (ui->cbScanFloodAddNoise->isChecked() && ui->leoScanFloodNoiseProbability->text()!="0")
     {
       fYellow = true;
       ui->twAdvSimOpt->tabBar()->setTabIcon(2, Rwindow->YellowIcon);
     }
   else ui->twAdvSimOpt->tabBar()->setTabIcon(2, QIcon());

   if (ui->cbLRFs->isChecked())
     {
       fYellow = true;
       ui->twAdvSimOpt->tabBar()->setTabIcon(3, Rwindow->YellowIcon);
     }
   else ui->twAdvSimOpt->tabBar()->setTabIcon(3, QIcon()); 

   ui->labAdvancedOn->setVisible(fYellow);
}

void MainWindow::on_pbReconstruction_clicked()
{
   Rwindow->showNormal();
   Rwindow->raise();
   Rwindow->activateWindow();
}

void MainWindow::on_pbReconstruction_2_clicked()
{
  Rwindow->showNormal();
  Rwindow->raise();
  Rwindow->activateWindow();
}

void MainWindow::on_cbPointSourceBuildTracks_toggled(bool checked)
{
  ui->cbGunPhotonTracks->setChecked(checked);
  ui->cbBuilPhotonTrackstester->setChecked(checked);
}
void MainWindow::on_cbGunPhotonTracks_toggled(bool checked)
{
  ui->cbPointSourceBuildTracks->setChecked(checked);
  ui->cbBuilPhotonTrackstester->setChecked(checked);
}
void MainWindow::on_cbBuilPhotonTrackstester_toggled(bool checked)
{
  ui->cbGunPhotonTracks->setChecked(checked);
  ui->cbPointSourceBuildTracks->setChecked(checked);
}

void MainWindow::on_sbNumberThreads_valueChanged(int arg1)
{
    ui->fMultiDisabled->setVisible( arg1==0 );
}

void MainWindow::on_pbSimulate_clicked()
{
  ui->tabwidMain->setCurrentIndex(1);
  ui->twSourcePhotonsParticles->setCurrentIndex(0);

  fSimDataNotSaved = false; // to disable the warning
  QJsonObject json;
  MainWindow::writeSimSettingsToJson(json);
  startSimulation(json);
}

void MainWindow::on_pbParticleSourcesSimulate_clicked()
{
  ui->tabwidMain->setCurrentIndex(1);
  ui->twSourcePhotonsParticles->setCurrentIndex(1);

  fSimDataNotSaved = false; // to disable the warning
  //watchdog on particle sources, can be transferred later to check-upwindow
  if (ParticleSources.size() == 0)
    {
      message("No particle sources defined!", this);
      return;
    }

  for (int i = 0; i<ParticleSources.size(); i++)
    {
      int error = ParticleSources.CheckSource(i);
      if (error == 0) continue;

      message("Error in source "+ParticleSources[i].name +":\n\n"+ParticleSources.getErrorString(error), this);
      return;
    }

  //Detector->remakeLastDetector();
  QJsonObject json;
  MainWindow::writeSimSettingsToJson(json);
  startSimulation(json);
}

void MainWindow::startSimulation(QJsonObject &json)
{    
    //============  prepare  gui  ============
    MainWindow::ClearData(); //clear GUI related things
    WindowNavigator->BusyOn(); //go busy mode, most of gui controls disabled
    ui->pbStopScan->setEnabled(true);
    //Owindow->OutText("Starting simulation");

    //========== prepare simulator ==========
    SimManager->setup(json, ui->sbNumberThreads->value());

    //============ RUN SIMULATION ============
    simThread.start();
    simTimerGuiUpdate.start();    
}

void MainWindow::simulationFinished()
{
    //qDebug()<<"Simulation finished: returned to mainwindow";
    simTimerGuiUpdate.stop();
    //qDebug()<<"Update timer stopped";
    QThread::usleep(5000);
    simThread.quit();
    //qDebug()<<"SimulationManager thread stopped (quit)";

    QJsonObject jsTmp;
    Rwindow->writePMrelatedInfoToJson(jsTmp);
    Detector->remakeLastDetector();
    Rwindow->readPMrelatedInfoFromJson(jsTmp);//passives, gains, pm groups etc - otherwise lost

    MIwindow->UpdateSelection();

    ui->pbStopScan->setEnabled(false);
    if (!SimManager->wasSuccessful())
    {        
        //qDebug() << "Sim manager reported fail!";
        ui->leEventsPerSec->setText("n.a.");
        QString report = SimManager->getErrorMessages();
        if (report != "Simulation stopped by user")
          message(report, this);
        ClearData();
        if (GeometryWindow->isVisible())
            MainWindow::ShowGeometry(false);        
        WindowNavigator->BusyOff(false);
        return;
    }

    //--- Retrieve results ---
    //    qDebug() << "Events recorded:"<< EventsDataHub->Events.size();
    QVector<Simulator *> simulators = SimManager->getWorkers();
    Simulator *lastSimulator = simulators.last();// SimManager->getLastWorker();
    PointSourceSimulator *lastPointSrcSimulator = dynamic_cast< PointSourceSimulator *>(lastSimulator);
    ParticleSourceSimulator *lastPartSrcSimulator = dynamic_cast< ParticleSourceSimulator *>(lastSimulator);
    //    qDebug() << "pointers:" << lastPointSrcSimulator << lastPartSrcSimulator;
    bool showTracks = false;
    if(lastPointSrcSimulator) //we have a PointSourceSimulator
    {        
        showTracks = ui->cbPointSourceBuildTracks->isChecked();
        //this->DotsTGeo.clear();
        clearGeoMarkers();
        for(int i = 0; i < simulators.count(); i++)
        {
            const PointSourceSimulator *simulator = dynamic_cast<const PointSourceSimulator *>(simulators[i]);
            //this->DotsTGeo << *simulator->getDotsTGeo();
            const QVector<DotsTGeoStruct>* dots = simulator->getDotsTGeo();
            GeoMarkerClass* marks = new GeoMarkerClass("Nodes", 6, 2, kBlack);
            for (int i=0; i<dots->size(); i++)
              marks->SetNextPoint(dots->at(i).r[0], dots->at(i).r[1], dots->at(i).r[2]);            
            GeoMarkers.append(marks);
        }
        if (ui->twSingleScan->currentIndex() == 0)
        {
            //info on last photon
            const One_Photon* ph = lastPointSrcSimulator->getLastPhotonOnStart();
            QString str;
            QTextStream strBuilder(&str);
            strBuilder<<"  Position xyz[mm]: "<<ph->r[0]<<"  "<<ph->r[1]<<"  "<<ph->r[2]<<endl;
            strBuilder<<"  Vector ikj: "<<ph->v[0]<<"  "<<ph->v[1]<<"  "<<ph->v[2]<<endl;
            if (ui->cbWaveResolved->isChecked())
                strBuilder<<"  Wavelength "<<ph->wavelength<<" nm (index = "<<ph->WaveIndex<<")"<<endl;
            if (ui->cbTimeResolved->isChecked())
                strBuilder<<"  Emission time: "<<ph->time<<endl;
            Owindow->OutText("Last Photon info:\n"+str);

            if (EventsDataHub->Events.size() == 1)
                Owindow->SiPMpixels = lastPointSrcSimulator->getLastEvent()->SiPMpixels;
        }
        int numRuns = (ui->cbNumberOfRuns->isChecked()) ? ui->sbNumberOfRuns->value() : 1;
        EventsDataHub->ScanNumberOfRuns = numRuns;
    }
    if(lastPartSrcSimulator) //we have a ParticleSourceSimulator
    {
        showTracks = ui->cbGunParticleTracks->isChecked() || ui->cbGunPhotonTracks->isChecked();
        EnergyVector = lastPartSrcSimulator->getEnergyVector();
        lastPartSrcSimulator->ClearEnergyVector(); // to avoid clearing the energy vector cells
        if (ui->cbDoLogsAndStatistics->isChecked())
        {
            Owindow->LogEventHistory(); //report data saved in history
            MainWindow::ShowGeneratedPhotonsLog(); //report photon data
        }
    }

    //tracks?
    if (showTracks)
      {
        if (fDemoMode)
          {
            //clear old if there
            for (int i=0; i<TrackInfo.size(); i++)
              delete TrackInfo[i];
            TrackInfo.clear();
            //new ones
            for(int iSim = 0; iSim<simulators.count(); iSim++)
              {
                TrackInfo += simulators[iSim]->tracks;
                simulators[iSim]->tracks.clear();
              }
          }
        else
          {
            int numTracks = 0;
            for(int iSim = 0; iSim<simulators.count() && numTracks<1000; iSim++)
              {
                //   qDebug() << "Sim:"<<iSim<<"Tracks collected:"<<simulators[iSim]->tracks.size();
                for (int iTr=0; iTr<simulators[iSim]->tracks.size() && numTracks<1000; iTr++)
                 {
                   TrackHolderClass* th = simulators[iSim]->tracks.at(iTr);
                   TGeoTrack* track = new TGeoTrack(1, th->UserIndex);
                   track->SetLineColor(th->Color);
                   track->SetLineWidth(th->Width);
                   for (int iNode=0; iNode<th->Nodes.size(); iNode++)
                     track->AddPoint(th->Nodes[iNode].R[0], th->Nodes[iNode].R[1], th->Nodes[iNode].R[2], th->Nodes[iNode].Time);

                   if (track->GetNpoints()>1)
                     {
                       numTracks++;
                       Detector->GeoManager->AddTrack(track);
                     }

                   delete th;
                 }
                simulators[iSim]->tracks.clear();
              }
          }
      }

    //This is part of a hack. Check this function declaration in .h for more details.
    SimManager->clearWorkers();

    //    qDebug() << "Data are dumped according to sim type";
    //GeometryWindow->resetRootMouseHandler();
    if (GeometryWindow->isVisible())
    {
        MainWindow::ShowGeometry(false);
        if (showTracks) MainWindow::ShowTracks();
        //   qDebug() << "Geometry visualized";
    }
    Rwindow->CheckAvailableEventData();
    //   qDebug()  << "Checked the available data, default Recon data created, basic filters applied";
    Owindow->SetCurrentEvent(0);

#ifdef advanced_photon_tracing
    DetectionStatisticsClass* d = EventsDataHub->DetStat;
    int sum = d->Absorbed + d->OverrideLoss + d->HitPM + d->HitDummy + d->Escaped + d->LossOnGrid + d->TracingSkipped + d->MaxCyclesReached;
    QString s = "=====================:\n";
    s += "Photon loss statistics:\n";
    s += "Absorbed: "+QString::number(d->Absorbed)+"\n"+
        "OverrideLoss: "+QString::number(d->OverrideLoss)+"\n"+
        "HitPM: "+QString::number(d->HitPM)+"\n"+
        "HitDummy: "+QString::number(d->HitDummy)+"\n"+
        "Escaped: "+QString::number(d->Escaped)+"\n"+
        "LossOnGrid: "+QString::number(d->LossOnGrid)+"\n"+
        "TracingSkipped: "+QString::number(d->TracingSkipped)+"\n"+
        "MaxTracingCyclesReached: "+QString::number(d->MaxCyclesReached)+"\n"+
        "------\n"+
        "Total: "+QString::number(sum)+"\n"+ "=====================";
    Owindow->OutText(s);
#endif

    WindowNavigator->BusyOff(false);
    fSimDataNotSaved = true;

    //qDebug() << "---Chain started from simulation has finished---";
}

ParticleSourceSimulator *MainWindow::setupParticleTestSimulation(GeneralSimSettings &simSettings)
{
    //============ prepare config ============
    QJsonObject json;
    SimGeneralConfigToJson(json);
    SimParticleSourcesConfigToJson(json);    
    json["Mode"] = "StackSim";
    json["DoGuiUpdate"] = true;
    SaveJsonToFile(json, "ThisSimConfig.json");    
    simSettings.readFromJson(json);
    simSettings.fLogsStat = true; //force to report logs

    //============  prepare  gui  ============
    WindowNavigator->BusyOn(); //go busy mode, most of gui controls disabled
    qApp->processEvents();

    //========== prepare simulator ==========
    ParticleSourceSimulator *pss = new ParticleSourceSimulator(Detector, "TestSimulator");

    pss->setSimSettings(&simSettings);
    //pss->setupStandalone(json);
    pss->setup(json);
    pss->setRngSeed(Detector->RandGen->Rndm()*1000000);
    return pss;
}

void MainWindow::on_pbTrackStack_clicked()
{
    fSimDataNotSaved = false; // to disable the warning
    MainWindow::ClearData();
    GeneralSimSettings simSettings;
    ParticleSourceSimulator *pss = setupParticleTestSimulation(simSettings);

    //============ run stack =========
    bool fOK = pss->standaloneTrackStack(&ParticleStack);
    qDebug() << "Standalone tracker reported:"<<fOK<<pss->getErrorString();

    //============   gui update   ============
    WindowNavigator->BusyOff();
    ui->pbStopScan->setEnabled(false);
    if (!fOK) message(pss->getErrorString(), this);
    else
    {
        //--- Retrieve results ---
        EnergyVector = pss->getEnergyVector();
        pss->ClearEnergyVector(); //disconnected this copy so delete of the simulator does not kill the vector
        qDebug() << "-------------En vector size:"<<EnergyVector.size();

        //track handling
        if (ui->cbBuildParticleTrackstester->isChecked())
          {
            int numTracks = 0;
            qDebug() << "Tracks collected:"<<pss->tracks.size();
            for (int iTr=0; iTr<pss->tracks.size() && numTracks<1000; iTr++)
              {
                TrackHolderClass* th = pss->tracks.at(iTr);
                TGeoTrack* track = new TGeoTrack(1, th->UserIndex);
                track->SetLineColor(th->Color);
                track->SetLineWidth(th->Width);
                for (int iNode=0; iNode<th->Nodes.size(); iNode++)
                  track->AddPoint(th->Nodes[iNode].R[0], th->Nodes[iNode].R[1], th->Nodes[iNode].R[2], th->Nodes[iNode].Time);

                if (track->GetNpoints()>1)
                  {
                    numTracks++;
                    Detector->GeoManager->AddTrack(track);
                  }
                delete th;
              }
            pss->tracks.clear();
          }
        //if tracks are visible, show them
        if (GeometryWindow->isVisible())
        {
            ShowGeometry();
            if (ui->cbBuildParticleTrackstester->isChecked()) MainWindow::ShowTracks();
        }
        //report data saved in history
        pss->appendToDataHub(EventsDataHub);
        qDebug() << "Event history imported";
        //Owindow->LogEventHistory();

        ui->pbExportDeposition->setEnabled(true);
        ui->pbGenerateLight->setEnabled(true);
        Owindow->SetCurrentEvent(0);
        //Owindow->RefreshData();
    }
    delete pss;
}

void MainWindow::on_pbGenerateLight_clicked()
{
    fSimDataNotSaved = false; // to disable the warning
    MainWindow::ClearData();
    GeneralSimSettings simSettings;
    ParticleSourceSimulator *pss = setupParticleTestSimulation(simSettings);
    Detector->PMs->configure(&simSettings); //also configures accelerators!!!
    bool fOK = pss->standaloneGenerateLight(&EnergyVector);

    //============   gui update   ============
    WindowNavigator->BusyOff();
    ui->pbStopScan->setEnabled(false);
    if (!fOK) message(pss->getErrorString(), this);
    else
    {
        //---- Retrieve results ----
        //EnergyVector = SPaS.getEnergyVector();  no need for energyVector in this case!
        //EventHistory = SPaS.getEventHistory();
        //EventsDataHub->GeneratedPhotonsHistory = SPaS.getGeneratedPhotonsHistory();
        //GenHistory is in EventsDataHub

        if (GeometryWindow->isVisible())
        {
            ShowGeometry();
            if (ui->cbBuildParticleTrackstester->isChecked()) MainWindow::ShowTracks();
        }
        Owindow->SetCurrentEvent(0);
        pss->appendToDataHub(EventsDataHub);
        MainWindow::ShowGeneratedPhotonsLog();
        Rwindow->CheckAvailableEventData();
    }
    delete pss;
}

void MainWindow::RefreshPhotSimOnTimer(int Progress, double msPerEv)
{
  ui->prScan->setValue(Progress);
  WindowNavigator->setProgress(Progress);
  ui->leEventsPerSec->setText( (msPerEv==0) ? "n.a." : QString::number(msPerEv, 'g', 4));

  qApp->processEvents();
  if (ui->pbStopScan->isChecked())
    {
      emit StopRequested();
      return;
    }
}

void MainWindow::on_pbGDML_clicked()
{
  MainWindow::on_pbConfigureAddOns_clicked();
  DAwindow->SetTab(6);
}

void MainWindow::startupDetector()
{
  if (QFile(GlobSet->ExamplesDir + "/StartupDetector.json").exists())
    ELwindow->LoadAllConfig(GlobSet->ExamplesDir + "/StartupDetector.json");
  else
    {
      message("Startup example NOT found, creating dummy detector", this);
      //startup detector file not found, to avoid crash initializing make-shift detector
        //Generates ParticleCollection
      //Detector->ParticleCollection.clear();
      Detector->clearParticleCollection();
      ParticleDescriptor *a1 = new ParticleDescriptor("gamma", _gamma_, 0, 0);
      Detector->ParticleCollection.append(a1);
      a1 = new ParticleDescriptor("neutron", _neutron_, 0, 1);
      Detector->ParticleCollection.append(a1);
      a1 = new ParticleDescriptor("alpha", _charged_, 2, 4);
      Detector->ParticleCollection.append(a1);
      MainWindow::ListActiveParticles();
        //Creates MaterialCollection
      if (MaterialCollection->size() > 0) MaterialCollection->Clear();
      MaterialCollection->AddNewMaterial();
      MaterialCollection->UpdateMaterial(MaterialCollection->size()-1, "Air", 1.2041e-3, 0, 1, 0, 5, 100, 2, 5, 1, 0, 0, 0);
      MainWindow::on_pbRefreshMaterials_clicked();
      MainWindow::on_pbRefreshOverrides_clicked();

      MIwindow->SetMaterial(0);
      MIwindow->SetParticleSelection(0);
      //ui->tabwidMain->setCurrentIndex(0);

      AddDefaultPMtype();
      //create detector geometry and visualize
      qDebug()<<"->Pre-building detector";
      Detector->PMarrays[0].fActive = ui->cbUPM->isChecked(); Detector->PMarrays[1].fActive = ui->cbLPM->isChecked();
      Detector->PMarrays[1].fUseRings = true;
      MainWindow::on_pbShowPMsArrayRegularData_clicked();
      MainWindow::on_pbUpdateSandwich_clicked();
    }
}

void MainWindow::on_pbSavePMtype_clicked()
{
  QString starter = (GlobSet->LibPMtypes.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibPMtypes;
  starter += "/PMtype_"+ui->lePMtypeName->text();

  QFileDialog fileDialog;
  QString fileName = fileDialog.getSaveFileName(this, "Save PM type properties", starter, "Json files (*.json);; All files (*.*)");
  if (fileName.isEmpty()) return;
  QFileInfo file(fileName);
  if (file.suffix().isEmpty()) fileName += ".json";

  int index = ui->sbPMtype->value();
  QJsonObject json;
  PMs->getType(index)->writeToJson(json); //here material is written as index!
    //adding material
  int imat = PMs->getType(index)->MaterialIndex;
  QJsonObject jsmat;
  MaterialCollection->writeMaterialToJson(imat, jsmat);
  json["Material"] = jsmat;
  SaveJsonToFile(json, fileName);
}

void MainWindow::on_pbLoadPMtype_clicked()
{
  QString starter = (GlobSet->LibPMtypes.isEmpty()) ? GlobSet->LastOpenDir : GlobSet->LibPMtypes;
  QFileDialog fileDialog;
  QString fileName = fileDialog.getOpenFileName(this, "Load PM type properties", starter, "Json files (*.json);; All files (*.*)");
  if (fileName.isEmpty()) return;

  QJsonObject json;
  LoadJsonFromFile(json, fileName);
  if (json.isEmpty())
    {
      message("Frong file format: Json is empty!");
      return;
    }
  if (!json.contains("Material"))
    {
      message("Frong json format!");
      return;
    }
  PMtypeClass* typ = new PMtypeClass();
  typ->readFromJson(json);
    //adding material
  QJsonObject jsmat = json["Material"].toObject();
  MaterialCollection->AddNewMaterial(jsmat);
  QString matname = (*MaterialCollection)[MaterialCollection->size()-1]->name;
  //GUI updates
    //Particles
  MainWindow::ListActiveParticles();
    //Materials
  MainWindow::on_pbRefreshMaterials_clicked();
  MIwindow->UpdateActiveMaterials(); //refresh indication on Material Inspector window
  MIwindow->SetMaterialSelection(0);
  message("PM type added and a new material "+matname+" registered", this);

  int itype = ui->sbPMtype->value();
  PMs->replaceType(itype, typ);

  PMs->RecalculateAngularForType(itype);
  PMs->RebinPDEsForType(itype);
  MainWindow::on_pbRefreshPMproperties_clicked(); //to refresh indication (also buttons enable/disable)
  MainWindow::ReconstructDetector(); //rebuild detector
}

void MainWindow::on_pbLoadNodes_clicked()
{
  QFileDialog fileDialog;
  QString fileName = fileDialog.getOpenFileName(this, "Load custom nodes", GlobSet->LastOpenDir, "Text files (*.txt);; All files (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  QVector<double> x, y, z;
  int err = LoadDoubleVectorsFromFile(fileName, &x, &y, &z);
  if (err == 0)
    {
      CustomScanNodes.clear();
      for (int i=0; i<x.size(); i++)
          CustomScanNodes.append(QVector3D(x[i], y[i], z[i]));
      ui->lScriptNodes->setText( QString::number(CustomScanNodes.size()) );
    }
}

void MainWindow::on_pbShowNodes_clicked()
{
   Detector->GeoManager->ClearTracks();
   //DotsTGeo.clear();
   //for (int i=0; i<CustomScanNodes.size(); i++)
   //  DotsTGeo.append(DotsTGeoStruct(CustomScanNodes[i].x(), CustomScanNodes[i].y(), CustomScanNodes[i].z(), kBlack));
   clearGeoMarkers();
   GeoMarkerClass* marks = new GeoMarkerClass("Nodes", 6, 2, kBlack);
   for (int i=0; i<CustomScanNodes.size(); i++)
     marks->SetNextPoint(CustomScanNodes[i].x(), CustomScanNodes[i].y(), CustomScanNodes[i].z());   
   GeoMarkers.append(marks);
   ShowGeometry();
}

void MainWindow::on_pbRunNodeScript_clicked()
{
  extractGeometryOfScriptWindow();
  if (GenScriptWindow) delete GenScriptWindow;
  GenScriptWindow = new GenericScriptWindowClass();
  recallGeometryOfScriptWindow();

  //configure the script window and engine
  NodesScriptInterface = new InterfaceToNodesScript(); //deleted by the GenScriptWindow  
  GenScriptWindow->SetInterfaceObject(NodesScriptInterface);
  //QStringList coms;
  //coms << "node";
  //GenScriptWindow->SetCustomCommands(coms);
  QString HelpText = "  Available commands:\n\n"
                     " node(x, y, z)\n"
                     " Math. followed by standard function (e.g. sin(x) )\n";
  GenScriptWindow->SetShowEvaluationResult(false); //do not show "undefined"
  GenScriptWindow->SetExample("for (var i=0; i<5; i++) node(i*10, (i-2)*20, 0)\nnode(40,-20,10)");

  GenScriptWindow->SetTitle("Custom nodes");

  GenScriptWindow->SetScript(&NodesScript);

  GenScriptWindow->SetStarterDir(GlobSet->LibScripts);

  //define what to do on evaluation success
  connect(GenScriptWindow, SIGNAL(success(QString)), this, SLOT(NodesScriptSuccess()));
  //if needed. connect signals of the interface object with the required slots of any ANTS2 objects
  GenScriptWindow->show();
}

void MainWindow::NodesScriptSuccess()
{
  CustomScanNodes = NodesScriptInterface->nodes;
  NodesScriptInterface->nodes.clear();
  ui->lScriptNodes->setText( QString::number(CustomScanNodes.size()) );
  MainWindow::on_pbShowNodes_clicked();
}

void MainWindow::on_sbNumberThreads_editingFinished()
{
   //if (ui->sbNumberThreads->value() == 0)
   //qDebug() << "*** Number of simulation threads changes, remaking detector";
  QJsonObject jsTmp;
  Rwindow->writePMrelatedInfoToJson(jsTmp);
  Detector->remakeLastDetector();
  Rwindow->readPMrelatedInfoFromJson(jsTmp);//passives, gains, pm groups etc - otherwise lost
}

void MainWindow::on_cobMatPointSource_activated(int index)
{
  if (!ui->cbWaveResolved->isChecked()) return;

  if ( ui->cobScintTypePointSource->currentIndex() == 0 )
    { //primary scint
      if ( (*MaterialCollection)[index]->PrimarySpectrum_lambda.isEmpty() ) message("This material has no primary scintillation spectrum defined!", this);
    }
  else
    { //secondary
      if ( (*MaterialCollection)[index]->SecondarySpectrum_lambda.isEmpty() ) message("This material has no secondary scintillation spectrum defined!", this);
    }
}

void MainWindow::on_ledSourceActivity_textChanged(const QString &/*arg1*/)
{
  double val = ui->ledSourceActivity->text().toDouble();
  QSize size(ui->lSourceActive->height(), ui->lSourceActive->height());
  QIcon wIcon = createColorCircleIcon(size, Qt::yellow);
  if (val == 0) ui->lSourceActive->setPixmap(wIcon.pixmap(16,16));
  else          ui->lSourceActive->setPixmap(QIcon().pixmap(16,16));
}

void MainWindow::on_actionOpen_settings_triggered()
{   
   GlobSetWindow->showNormal();
   GlobSetWindow->activateWindow();
}

void MainWindow::on_actionSave_Load_windows_status_on_Exit_Init_toggled(bool arg1)
{
   GlobSet->SaveLoadWindows = arg1;
}

void MainWindow::on_pbUpdateElectronics_clicked()
{
   Detector->PMs->setDoPHS( ui->cbEnableSPePHS->isChecked() );
   Detector->PMs->setDoElNoise( ui->cbEnableElNoise->isChecked() );
   Detector->PMs->setDoADC( ui->cbEnableADC->isChecked() );
   Detector->PMs->setMeasurementTime( ui->ledTimeOfOneMeasurement->text().toDouble() );

   ReconstructDetector(true);
}

void MainWindow::on_dsbNewProbLoss_editingFinished()
{
  double pLoss = ui->dsbNewProbLoss->value();
  double pRef = ui->dsbNewProbReflection->value();
  double pDiff = ui->dsbNewProbDiffuse->value();

  if (pLoss+pRef+pDiff > 100.0)
    {
      ui->dsbNewProbLoss->setValue(100.0-pRef-pDiff);
      message("Sum cannot exceed 100%", this);
    }

  QString str;
  double prob = 100.0 -pLoss -pRef -pDiff;
  str.setNum(prob, 'f', 1);
  ui->leNewFresnelProbability->setText(str);

  MainWindow::on_pbOverride_clicked();
}

void MainWindow::on_dsbNewProbReflection_editingFinished()
{
  double pLoss = ui->dsbNewProbLoss->value();
  double pRef = ui->dsbNewProbReflection->value();
  double pDiff = ui->dsbNewProbDiffuse->value();

  if (pLoss+pRef+pDiff > 100.0)
    {
      ui->dsbNewProbReflection->setValue(100.0-pLoss-pDiff);
      message("Sum cannot exceed 100%", this);
    }

  QString str;
  double prob = 100.0 -pLoss -pRef -pDiff;
  str.setNum(prob, 'f', 1);
  ui->leNewFresnelProbability->setText(str);

  MainWindow::on_pbOverride_clicked();
}

void MainWindow::on_dsbNewProbDiffuse_editingFinished()
{
  double pLoss = ui->dsbNewProbLoss->value();
  double pRef = ui->dsbNewProbReflection->value();
  double pDiff = ui->dsbNewProbDiffuse->value();

  if (pLoss+pRef+pDiff > 100.0)
  {
      ui->dsbNewProbDiffuse->setValue(100.0-pLoss-pRef);
      message("Sum cannot exceed 100%", this);
  }

  QString str;
  double prob = 100.0 -pLoss -pRef -pDiff;
  str.setNum(prob, 'f', 1);
  ui->leNewFresnelProbability->setText(str);

  MainWindow::on_pbOverride_clicked();
}

void MainWindow::on_pbOverlay_clicked()
{
  if (EventsDataHub->isEmpty())
    {
      message("There are no data!", this);
      return;
    }

  QString fileName = QFileDialog::getOpenFileName(this, "Overlay simulation data with data from an ASCII file", GlobSet->LastOpenDir, "All file (*.*)");
  if (fileName.isEmpty()) return;
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  WindowNavigator->BusyOn();
  bool ok = EventsDataHub->overlayAsciiFile(fileName, ui->cbApplyAddMultiplyPreprocess->isChecked(), Detector->PMs);
  if (!ok)
      message("Overlay error: "+EventsDataHub->ErrorString, this);
  EventsDataHub->clearReconstruction();
  Owindow->RefreshData();
  ui->lwLoadedSims->clear();
  WindowNavigator->BusyOff(false);
}

void MainWindow::on_pbLoadManifestFile_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load manifest file", GlobSet->LastOpenDir, "Text files (*.txt);;All files (*)");
  if (fileName.isEmpty()) return;
  //this->activateWindow();
  //this->raise();
  GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  ui->leManifestFile->setText(fileName);
  ui->cbLoadedDataHasPosition->setChecked(false);
  ui->fReloadRequired->setVisible(true);
}

void MainWindow::on_pbDeleteManifestFile_clicked()
{
  if (ui->leManifestFile->text().isEmpty()) return;
  ui->leManifestFile->setText("");
  ui->fReloadRequired->setVisible(true);
}

void MainWindow::on_pbManifestFileHelp_clicked()
{
    QString str = "ASCII file giving true positions for experimental data\n\n"
                  "Format: file_name   X_position   Y_position \n"
                  "    or: file_name   X_position   Y_position   Events_to_load\n"
                  "---------------\n"
                  "Command lines (define the handling of the data files located after the command):\n"
                  "#type hole   -> set round irradiated area\n"
                  "#type slit   -> set slit shaped irradiated area\n"
                  "#X0 double    -> double = offset of zero in X direction\n"
                  "#Y0 double    -> double = offset of zero in Y direction\n"
                  "#dX double   -> double = step in X direction in mm\n"
                  "#dY double   -> double = step in Y direction in mm\n"
                  "hole-type specific:\n"
                  "#Diameter double   -> double = diameter of irradiated area in mm\n"
                  "slit-type specific:\n"
                  "#Angle double   -> double = angle of slit with X axis in degrees\n"
                  "#Width double   -> double = slit width in mm\n"
                  "#Length double  -> double = slit length in mm\n"
                  "---------------\n"
                  "Comment line should start from \\\\ \n"
                  "Empty lines allowed.\n";
    message(str, this);
}

void MainWindow::on_pbLoadAppendFiles_clicked()
{ 
   if (ui->cobLoadDataType->currentIndex() == 0)
     MainWindow::on_pbLoadPMsignals_clicked();
   else
     MainWindow::on_pbLoadSimTree_clicked();
}

void MainWindow::on_sbLoadASCIIpositionXchannel_valueChanged(int arg1)
{
   ui->sbLoadASCIIpositionYchannel->setValue(arg1+1);
}

void MainWindow::on_cbLoadedDataHasPosition_toggled(bool checked)
{
    if (checked) ui->leManifestFile->setText("");
}

void MainWindow::on_cbLoadedDataHasZPosition_clicked(bool checked)
{
    if (checked) ui->leManifestFile->setText("");
}

void MainWindow::on_pbPDEFromWavelength_clicked()
{
    //int prMat = Detector->Sandwich.Layers[Detector->Sandwich.ZeroLayer].Material; connot do it this way, geometry could be loaded from a GDML file!

    //finding the material index of PrScint
    TObjArray* vols = Detector->GeoManager->GetListOfVolumes();
    TGeoVolume* PrScint = (TGeoVolume*)vols->FindObject("PrScint");
    if (!PrScint)
      {
        message("Primary scintillator not found!", this);
        return;
      }
    //finding material
    QString matName = PrScint->GetMaterial()->GetName();
    qDebug() << "PrScint has material with name:"<<matName;
    int iMat = Detector->MaterialCollection->FindMaterial(matName);
    if (iMat == -1 )
      {
        message("Material of PrScint volume not found!", this);
        return;
      }
    qDebug() << "PrScint material index is "<< iMat;

    if ((*Detector->MaterialCollection)[iMat]->PrimarySpectrum.isEmpty())
      {
        message("Primary scintillation emission spectrum is not defined for the material of the primary scintillator ('PrScint') volume!\n"
                "Check material "+matName, this);
        return;
      }
    if (PMs->getType(ui->sbPMtype->value())->PDE.isEmpty())
      {
        message("Wavelength-resolved data are not defined for this PM type!", this);
        return;
      }

    //  const QVector<double> &prSpectrum = (*Detector->MaterialCollection)[prMat]->PrimarySpectrum; cannot do it this way - the data can have different binning!
    //  const QVector<double> &pmTypePDE = PMs->getType(ui->sbPMtype->value())->PDE;

    qDebug() << "Converting data to standart wavelength: From To Nodes"<<WaveFrom<<WaveTo<<WaveNodes;
    QVector<double> prSpectrum;
    ConvertToStandardWavelengthes(&(*Detector->MaterialCollection)[iMat]->PrimarySpectrum_lambda,
                                  &(*Detector->MaterialCollection)[iMat]->PrimarySpectrum,
                                  WaveFrom, WaveStep, WaveNodes,
                                  &prSpectrum);
    QVector<double> pmTypePDE;
    ConvertToStandardWavelengthes(&PMs->getType(ui->sbPMtype->value())->PDE_lambda,
                                  &PMs->getType(ui->sbPMtype->value())->PDE,
                                  WaveFrom, WaveStep, WaveNodes,
                                  &pmTypePDE);

    double weightedSum = 0, sum = 0;
    for(int i = 0; i < prSpectrum.size(); i++)
      {
        weightedSum += prSpectrum[i] * pmTypePDE[i];
        sum += prSpectrum[i];
      }

    if (sum == 0)
      {
        message("Normalisation problem: sum = 0", this);
        return;
      }

    ui->ledPDE->setText(QString::number(weightedSum/sum, 'g', 4));
    on_pbUpdatePMproperties_clicked(); //update detector
}

void MainWindow::on_cobLoadDataType_customContextMenuRequested(const QPoint &/*pos*/)
{
    if (ui->cobLoadDataType->currentIndex() == 0) ui->cobLoadDataType->setCurrentIndex(1);
    else ui->cobLoadDataType->setCurrentIndex(0);
}

void MainWindow::on_pbManuscriptExtractNames_clicked()
{
   QString manifest = ui->leManifestFile->text();
   if (manifest.isEmpty()) return;

   MainWindow::DeleteLoadedEvents();

   QFile file(manifest);
   if(!file.open(QIODevice::ReadOnly | QFile::Text))
     {
       qWarning() << "||| Could not open manifest file!";
       return;
     }
   QString path = QFileInfo(file).absolutePath();
   qDebug() << "Path to manifest file:"<<path;

   QTextStream in(&file);
   QRegExp rx("(\\ |\\t)");

   while(!in.atEnd())
    {
      QString line = in.readLine().trimmed();
      if ( line.startsWith("//") || line.startsWith("#")) continue; //this is a comment or control, skip
      QStringList fields = line.split(rx, QString::SkipEmptyParts);
      if (fields.isEmpty()) continue; //empty line
      if (fields.size()<3)
        {
          qWarning() << "Bad format (too short) of manifest item:"<<line;
          return;
        }

      QString fn = path + "/" + fields[0];
      LoadedEventFiles.append(fn);
      ui->lwLoadedEventsFiles->addItem(fn);
     }

   ui->fReloadRequired->setVisible(true);
   ui->pbReloadExpData->setEnabled(!LoadedEventFiles.isEmpty());
}

void MainWindow::setPrScintLabelText(QString str)
{
  ui->label_265->setText(str);
}

void MainWindow::on_lwOverrides_itemClicked(QListWidgetItem* /*item*/)
{
  int row = ui->lwOverrides->currentRow();
  //if (ui->lwOverrides->currentItem()->text() == "Not defined") return;

  int MatFrom = ui->cobMaterialForOverrides->currentIndex();
  int Mats = Detector->MaterialCollection->size();
  int counter = 0;
  for (int iMat=0; iMat<Mats; iMat++)
    {
      OpticalOverrideClass* ov = (*Detector->MaterialCollection)[MatFrom]->OpticalOverrides[iMat];
      if (!ov) continue;
      if (counter == row)
        {
          ui->cobMaterialTo->setCurrentIndex(iMat);
          MainWindow::on_pbRefreshOverrides_clicked();
          return;
        }
      counter++;
    }
}

void MainWindow::setFloodZposition(double Z)
{
  ui->ledScanFloodZ->setText(QString::number(Z));
}
