#ifndef MAINWINDOW_H
#define MAINWINDOW_H

//ANTS
#include "classes.h"
#include "materialcollectionclass.h"
#include "particlesourcesclass.h"
#include "functions.h"
#include "eventsdataclass.h"
#include "detectorsandwichclass.h"

//Qt
#include <QMainWindow>
#include <QVector>
#include <QStandardItemModel>
#include <QListWidgetItem>
#include <QMessageBox>

//ROOT
#include "TGeoManager.h"
#include "TGraph.h"
#include "TVector.h"
#include "TRandom2.h"
#include "TH1D.h"
#include "TH1I.h"

//MODULES
class GeometryWindowClass;
class GraphWindowClass;
class RasterWindow;
class LRFwindow;
class ReconstructionWindow;
class SensorLRFs;
class ExamplesWindow;
class CheckUpWindowClass;
class DetectorAddOnsWindow;
class PrimaryParticleTracker;
class S1_Generator;
class S2_Generator;
class pms;
class ParticleSourcesClass;
class Photon_Tracker;
class Photon_Generator;
class ReconstructionManagerClass;
class MaterialInspectorWindow;
class OutputWindow;
class QComboBox;
class TGeoVolume;
class T1HD;
class TH2D;
class TTree;
class TMultiGraph;
class WindowNavigatorClass;
class GeometryWindowAddOn;
class GlobalSettingsClass;
class GlobalSettingsWindowClass;

#ifdef ANTS_FANN
class NeuralNetworksWindow;
#endif

class GainEvaluatorWindowClass;
class TApplication;
class Viewer2DarrayObject;
class GenericScriptWindowClass;
class DetectorClass;
class SimulationManager;
class ParticleSourceSimulator;
class PMtypeClass;
class TmpObjHubClass;

namespace Ui {
class MainWindow;
}

class SandwichLayerGUIclass : public BaseLayerClass
{
public:
  QString Name;

  QCheckBox* CBactive;
  QLineEdit* LEheight;
  QComboBox* COBmaterial;

  SandwichLayerGUIclass(){Shape = 0; Sides = 6; Angle = 0;}
  SandwichLayerGUIclass(QString Name, QCheckBox* CBactive, QLineEdit* LEheight, QComboBox* COBmaterial)
    : Name(Name), CBactive(CBactive), LEheight(LEheight), COBmaterial(COBmaterial) {Shape = 0; Sides = 6; Angle = 0;}

  void updateData();
  void updateGui();

  void writeToJson(QJsonObject &json);
};

class InterfaceToPMscript : public QObject
{
  Q_OBJECT
public:  
  QJsonArray arr; //output is placed here
public slots:
  void pm(double x, double y) {QJsonArray el; el<<x<<y; arr.append(el);}
  void PM(double x, double y, double z, int type) {QJsonArray el; el<<x<<y<<z<<type; arr.append(el);}
  void PM(double x, double y, double z, int type, double phi, double theta, double psi) {QJsonArray el; el<<x<<y<<z<<type<<phi<<theta<<psi; arr.append(el);}
};

class InterfaceToNodesScript : public QObject
{
  Q_OBJECT
public:
  QVector<QVector3D> nodes; //output is placed here
public slots:
  void node(double x, double y, double z) {nodes.append( QVector3D(x,y,z));}
};

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(DetectorClass *Detector,
                        EventsDataClass *EventsDataHub,
                        TApplication *RootApp,
                        ReconstructionManagerClass *ReconstructionManager,
                        TmpObjHubClass *TmpHub);
    ~MainWindow();

    //ROOT
    TApplication *RootApp;

    //ANTS2 windows
    GraphWindowClass *GraphWindow;
    GeometryWindowClass *GeometryWindow;   
    OutputWindow *Owindow;
    LRFwindow *lrfwindow;
    ReconstructionWindow *Rwindow;
    MaterialInspectorWindow *MIwindow;
    WindowNavigatorClass *WindowNavigator;
    ExamplesWindow* ELwindow;
    DetectorAddOnsWindow* DAwindow;
    CheckUpWindowClass* CheckUpWindow;
    GainEvaluatorWindowClass* GainWindow;
    GenericScriptWindowClass* GenScriptWindow;
    GlobalSettingsWindowClass* GlobSetWindow;
#ifdef ANTS_FANN
    NeuralNetworksWindow* NNwindow;
#endif

    //ANTS2 modules    
    pms* PMs;    
    //TRandom2 *RandGen;  //migrated to Detector
    SensorLRFs *SensLRF;  
    ReconstructionManagerClass *ReconstructionManager;    
    SimulationManager *SimManager;

    //Data hub
    EventsDataClass *EventsDataHub;

    //Detector-related properties
    DetectorClass *Detector;
    MaterialCollectionClass* MaterialCollection; //just the pointer, detector has the same

    //Temporary objects (more will migrate there later)
    TmpObjHubClass *TmpHub;

    //local data, just for GUI
    QVector<EnergyDepositionCell*> EnergyVector;
    QVector<GeoMarkerClass*> GeoMarkers;
    QVector<ParticleOnStack*> ParticleStack;

    //int ScanNumberOfRuns;
    TMultiGraph *TMPmultiGraph;

    InterfaceToPMscript *PMscriptInterface;

    QVector<QVector3D> CustomScanNodes;
    InterfaceToNodesScript *NodesScriptInterface;
    QString NodesScript;

    //global settings
    GlobalSettingsClass* GlobSet; //global settings

    //critical - updates
    void PMnumberMightHaveChanged();    ///see MainWindowDetectorConstructor.cpp

    //busy triggered on start of user action that can take long time and user should not be able
    //to change any settins
    void onBusyOn();
    void onBusyOff();

    void startRootUpdate();  // starts timer which in regular intervals process Root events
    void stopRootUpdate();   // stops timer which in regular intervals process Root events

    void ClearData(); //clear status, disconnect reconstruction

    //detector constructor
    void ReconstructDetector(bool fKeepData = false);

    void ListActiveParticles();

    void ShowGraphWindow();
    void UpdateMaterialListEdit();

    void ShowPMnumbers(); ///see MainWindowTracks.cpp
    void ShowTextOnPMs(QVector<QString> strData, Color_t color); ///see MainWindowTracks.cpp
    void setPrScintLabelText(QString str);

    void UpdateTestWavelengthProperties(); //if material properties were updated, need to update indication in the Test tab

    void ShowTracks();
    void ShowGeometry(bool ActivateWindow = true, bool SAME = true, bool ColorUpdateAllowed = true);    
    void NavigatorWindowClosed();

    int getNormalWindowStatus(QMainWindow* win);  ///MainWindowMenu.cpp
    void setNormalWindowStatus(QMainWindow *win, int status);///MainWindowMenu.cpp
    void setQWindowStatus(QWindow* win, int status);///MainWindowMenu.cpp

    void writeDetectorToJson(QJsonObject &json); //GDML is NOT here
    bool readDetectorFromJson(QJsonObject &json);
    void writeSimSettingsToJson(QJsonObject &json, bool fVerbose = false);  //true - save point and source settings
    bool readSimSettingsFromJson(QJsonObject &json);
    void writeDetectorLayersToJson(QJsonObject &json);
    void writeWorldFixedToJson(QJsonObject &json);

    //save data to file - public due to batch mode usage
    void LoadSimulationDataFromTree(QString fileName);
    int LoadPMsignals(QString fileName);
    //
    void ExportDeposition(QFile &outputFile);         ///see mainWindowDiskIO
    void ImportDeposition(QFile &file);                ///see mainWindowDiskIO

    //configuration from outside
    void setShowTop(bool flag) {ShowTop = flag;}
    void setColorByMaterial(bool flag) {ColorByMaterial = flag;}
    void SetProgress(int val);

    //gains
    void SetMultipliersUsingGains(bool fIncludeChPerPhEl, QVector<double> *ChPerPhEl);

    //config setting (updated bu the Examples window ELwindow)
    bool ShowExamplesOnStart;

    //public flags
    bool DoNotUpdateGeometry;  //if GUI is in bulk-update, we do not detector geometry be updated on each line
    bool GeometryDrawDisabled; //no drawing of th geometry or tracks
    bool fDemoMode;

    QVector<TrackHolderClass*> TrackInfo; //for GlobScript only, content is deleted on closing GlobScript interface object

    bool isWavelengthResolved() const;
    double WaveFrom, WaveTo, WaveStep;
    int WaveNodes;

    QVector<QString> NoiseTypeDescriptions; ///see MainWindowTools/MainWindowPhotonSource

    //hitsograms
    TH1D *hist1, *hist2;
    TH2D *hist2D;
    TH1D *histSecScint;

    int ScriptWinX, ScriptWinY, ScriptWinW, ScriptWinH;
    void recallGeometryOfScriptWindow();
    void extractGeometryOfScriptWindow();

    void TestCompton(); ///MainWindowTests.cpp

    void ExportGeometryImage(QString filename);

    void LoadSecScintTheta(QString fileName);

    int PMArrayType(int ul);
    void SetPMarrayType(int ul, int itype);

    void LoadDummyPMs(QString DFile); ///MainWindowDiskIO.cpp

    void ShowGeoMarkers(); //Show dots on ALREADY PREPARED geometry window!

    void LoadAdditionalObjects(QString File); ///MainWindowDiskIO.cpp

    void SaveGains(QString fileName);
    bool LoadGains(QString fileName);

    //handling of material COBs
    void AddMaterialToCOBs(QString s);
    void RemoveMaterialFromCobs(int imat);
    void UpdateMaterialNameInCobs(int imat);
    void CheckPresenseOfSecScintillator();
    void DeleteLoadedEvents(bool KeepFileList = false);   
    void AddLineToGeometry(QPointF &start, QPointF &end, Color_t color = 1, int width = 1);
    void AddPolygonfToGeometry(QPolygonF &poly, Color_t color, int width);

    //void SavePreprocessingSettings(QString fileName);
    //void LoadPreprocessingSettings(QString fileName);
    void SavePreprocessingAddMulti(QString fileName);
    void LoadPreprocessingAddMulti(QString filename);
public slots:
    void LoadEventsListContextMenu(const QPoint &pos);

    void LRF_ModuleReadySlot(bool ready);

    //here because of the batcher, can be moved to private slots afterwards
    void on_pbSimulate_clicked();
    void on_pbTrackStack_clicked();
    void on_pbGenerateLight_clicked();
    void on_pbParticleSourcesSimulate_clicked();    
    void RefreshPhotSimOnTimer(int Progress, double msPerEv);
    void PMscriptSuccess();
    void NodesScriptSuccess();
    void onGDMLstatusChage(bool fGDMLactivated);
    void updateLoaded(int events, int progress);
    void on_pbSingleSourceShow_clicked();
    void on_pbClearAllStack_clicked();
    void on_pbRefreshStack_clicked();

private slots:
    void on_pbAddParticleToStack_clicked();

    void on_pbRemoveFromStack_clicked();

    void on_sbTimeBins_valueChanged(int arg1);

    void on_ledTimeFrom_editingFinished();

    void on_ledTimeTo_editingFinished();

    void on_pbRefreshMaterials_clicked();

    void on_cbXbyYarray_stateChanged(int arg1);

    void on_cbRingsArray_stateChanged(int arg1);

    void on_pbRefreshOverrides_clicked();

    void on_pbOverride_clicked();

    void on_pbStartMaterialInspector_clicked();

    void on_pbAddparticleToActive_clicked();

    void on_cobParticleToInspect_currentIndexChanged(int index);

    void on_pushButton_4_clicked();

    void on_pushButton_5_clicked();

    void on_pushButton_7_clicked();

    void on_leParticleName_editingFinished();

    void on_pbRefreshParticles_clicked();

    void on_cbSecondAxis_toggled(bool checked);

    void on_cbThirdAxis_toggled(bool checked);  

    void on_cobPMdeviceType_currentIndexChanged(int index);

    void on_cbSS_toggled(bool checked); /// *** obsolete?

    void on_cbUPM_toggled(bool checked);

    void on_cbLPM_toggled(bool checked);

    void on_pbRefreshPMArrayData_clicked();

    void on_pbShowPMsArrayRegularData_clicked();

    void on_pbRefreshPMproperties_clicked();

    void on_pbUpdatePMproperties_clicked();

    void on_sbPMtype_valueChanged(int arg1);

    void on_sbPMtypeForGroup_valueChanged(int arg1);

    void on_cbXbyYarray_clicked();

    void on_cbRingsArray_clicked();

    void on_cobUpperLowerPMs_currentIndexChanged(int index);

    void on_pbRemoveThisPMtype_clicked();

    void on_pbAddNewPMtype_clicked();

    void on_cbAreaSensitive_toggled(bool checked);

    void on_cbAngularSensitive_toggled(bool checked);

    void on_cbTimeResolved_toggled(bool checked);

    void on_pbSavePMtype_clicked();

    void on_pbLoadPMtype_clicked();

    void on_pbPMtypeShowCurrent_clicked();  

    void on_cbWaveResolved_toggled(bool checked);

    void on_ledWaveFrom_editingFinished();

    void on_ledWaveTo_editingFinished();

    void on_ledWaveStep_editingFinished();

    void on_pbUpdateTestWavelengthProperties_clicked();

    void on_pbTestShowPrimary_clicked();

    void on_pbTestShowSecondary_clicked();

    void on_cobMaterialForWaveTests_currentIndexChanged(int index);

    void on_pbTestGeneratorPrimary_clicked();

    void on_pbTestGeneratorSecondary_clicked();

    void on_pbTestShowRefrIndex_clicked();

    void on_pbTestShowAbs_clicked();

    void on_pbShowThisMatInfo_clicked();

    void on_sbWaveIndexPointSource_valueChanged(int arg1);

    void on_pbShowPDE_clicked();

    void on_pbLoadPDE_clicked();

    void on_pbDeletePDE_clicked();

    void on_pbShowPDEbinned_clicked();

    void on_pbPMtypeLoadAngular_clicked();

    void on_pbPMtypeShowAngular_clicked();

    void on_pbPMtypeDeleteAngular_clicked();

    void on_pbPMtypeShowEffectiveAngular_clicked();

    void on_sbCosBins_valueChanged(int arg1);

    void on_cobGunSourceType_currentIndexChanged(int index);

    void on_pbGunShowSource_clicked();

    void on_pbGunTest_clicked();

    void on_pbGunRefreshparticles_clicked();

    void on_pbGunAddNew_clicked();

    void on_pbGunRemove_clicked();

    void on_cobGunParticle_activated(int index);

    void on_ledGunEnergy_editingFinished();

    void on_ledGunParticleWeight_editingFinished();

    void on_pbGunLoadSpectrum_clicked();

    void on_pbGunShowSpectrum_clicked();

    void on_pbGunDeleteSpectrum_clicked();

    void on_ledGunAverageNumPartperEvent_editingFinished();

    void on_ledMediumRefrIndex_editingFinished();

    void on_cobTubeBox_currentIndexChanged(int index);

    void on_pbPMtypeLoadArea_clicked();

    void on_pbPMtypeShowArea_clicked();

    void on_pbPMtypeDeleteArea_clicked();

    void on_cobPMarrayRegularity_currentIndexChanged(int index); 

    void on_sbIndPMnumber_valueChanged(int arg1);

    void on_pbIndPMshowInfo_clicked();

    void on_tabWidget_currentChanged(int index);

    void on_pbUpdateToFullCustom_clicked();

    void on_pbUpdateToFixedZ_clicked();

    void on_pbIndPmRemove_clicked();

    void on_pbIndShowType_clicked();

    void on_ledIndEffectiveDE_editingFinished();

    void on_pbIndRestoreEffectiveDE_clicked();

    void on_pbIndShowDE_clicked();

    void on_pbIndRestoreDE_clicked();

    void on_pbIndLoadDE_clicked();

    void on_pbIndShowDEbinned_clicked();

    void on_pbAddPM_clicked();

    void on_pbIndLoadAngular_clicked();

    void on_pbIndRestoreAngular_clicked();

    void on_pbIndShowAngular_clicked();

    void on_pbIndShowEffectiveAngular_clicked();

    void on_ledIndMediumRefrIndex_editingFinished();

    void on_pbIndLoadArea_clicked();

    void on_pbIndRestoreArea_clicked();

    void on_pbIndShowArea_clicked();

    void on_cbGunAllowMultipleEvents_toggled(bool checked);

    void on_pbLoadPMsignals_clicked();

    void on_pbDeleteLoadedEvents_clicked();

    void on_pbPreprocessingLoad_clicked();

    void on_pbPreprocessingSave_clicked();

    void on_cbPMsignalPreProcessing_toggled(bool checked);

    void on_sbPreprocessigPMnumber_valueChanged(int arg1);

    void on_ledPreprocessingAdd_editingFinished();

    void on_pbReloadExpData_clicked();   

    void on_cobPMshape_currentIndexChanged(int index);

    void on_pbElGainLoadDistr_clicked();

    void on_pbElGainShowDistr_clicked();

    void on_cobPMampGainModel_currentIndexChanged(int index);

    void on_pbElUpdatePMampGainProperties_clicked();

    void on_pbElTestGenerator_clicked();

    void on_sbElPMnumber_valueChanged(int arg1);

    void on_pbElUpdateIndication_clicked();

    void on_pbElCopyGainData_clicked();

    void on_sbADCbits_editingFinished();

    void on_ledADCmax_editingFinished();

    void on_cbEnableSPePHS_toggled(bool checked);

    void on_cbEnableADC_toggled(bool checked);

    void on_pbScanDistrLoad_clicked();

    void on_pbScanDistrShow_clicked();

    void on_pbScanDistrDelete_clicked();

    void on_cobScanNumPhotonsMode_currentIndexChanged(int index);

    void on_pbShowNumPhotSpectrum_clicked();   

    void on_pbUpdateScanFloodTabWidget_clicked();

    void on_pbInitializeScanFloodNoise_clicked();

    void on_tabwidScanFlood_cellChanged(int row, int column);

    void on_actionWindow_navigator_triggered();

    void on_actionGeometry_triggered();

    void on_actionReconstructor_triggered();

    void on_actionOutput_triggered();

    void on_actionLRF_triggered();

    void on_actionGraph_triggered();

    void on_actionReset_position_of_windows_triggered();

    void on_actionSave_position_and_stratus_of_all_windows_triggered();

    void on_actionLoad_positions_and_status_of_all_windows_triggered();

    void on_actionMaterial_inspector_window_triggered();

    void on_twSingleScan_currentChanged(int index);

    void on_pbTestCompton_clicked();

    void on_pbExportDeposition_clicked();

    void on_pbImportDeposition_clicked();

    void on_pbTestsInterpolation_clicked();

    void on_ledElNoiseSigma_editingFinished();

    void on_cbEnableElNoise_toggled(bool checked);

    void on_actionExamples_triggered();

    void on_cobSecScintillationGenType_currentIndexChanged(int index);

    void on_pbSecScintShowProfile_clicked();

    void on_pbSecScintLoadProfile_clicked();

    void on_pbSecScintDeleteProfile_clicked();

    void on_lwMaterials_currentRowChanged(int currentRow);

    void on_lwMaterials_doubleClicked(const QModelIndex &index);

    void on_lwParticles_currentRowChanged(int currentRow);

    void on_pbReconstruction_clicked();

    void on_pbConfigureAddOns_clicked();    

    void on_pbLoadSimTree_clicked();

    void on_pbRemoveSource_clicked();

    void on_pbAddSource_clicked();

    void on_pbUpdateSources_clicked();

    void on_pbUpdateSourcesIndication_clicked();

protected:
    void closeEvent(QCloseEvent *);    
    bool event(QEvent *event);

private:
    Ui::MainWindow *ui;
    QTimer *RootUpdateTimer; //root update timer
    QMessageBox *msBox; //box to be used to confirm discard or save sim data on data clear; 0 if not activated

    //flags
    bool TriggerForbidden;
    bool BulkUpdate;

    ParticleDescriptor tmpParticle;
    TH1I* histScan;

public:
    bool ShutDown; //when exiting ANTS2 by closing the main window
    ParticleSourcesClass ParticleSources;

    QVector<SandwichLayerGUIclass> SandwichLayers;
    bool fSimDataNotSaved;

    void SimGeneralConfigToJson(QJsonObject &jsonMaster);                              //Save to JSON general options of simulation
    void SimPointSourcesConfigToJson(QJsonObject &jsonMaster, bool fVerbose = false);  //Save to JSON config for PointSources simulation
    void SimParticleSourcesConfigToJson(QJsonObject &json);     //Save to JSON config for ParticleSources simulation

    void updatePMArrayDataIndication();

    void writeLoadExpDataConfigToJson(QJsonObject &json);
    bool readLoadExpDataConfigFromJson(QJsonObject &json);
    void clearGeoMarkers();
    void setFontSizeAllWindows(int size);
    void writeExtraGuiToJson(QJsonObject &json);
    void readExtraGuiFromJson(QJsonObject &json);
    void SaveSimulationDataTree();
    void SaveSimulationDataAsText();

    void setFloodZposition(double Z);    
private:
    void startupDetector();  //on ANTS start load/create detector
    void PointSource_UpdateTabWidget();
    void PointSource_InitTabWidget();
    void PointSource_ReadTabWidget();
    void ShowGeneratedPhotonsLog();
    void VisualizeTmpParticle();
    void CheckSetMaterial(const QString name, QComboBox* cob, QVector<QString>* vec);
    void ToggleUpperLowerPMs();
    void PopulatePMarray(int ul, double z, int istart);
    void AddDefaultPMtype();
    void CorrectWaveTo();
    void RefreshAngularButtons();
    void RefreshAreaButtons();
    void ShowPMcount();
    void SavePhElToSignalData(QString fileName);
    void LoadPhElToSignalData(QString fileName);
    void LoadScanPhotonDistribution(QString fileName);

    int LoadAreaResponse(QString fileName, QVector<QVector<double> >* tmp, double* xStep, double* yStep);       ///see MainWindowDiskIO.cpp
    int LoadSPePHSfile(QString fileName, QVector<double>* SPePHS_x, QVector<double>* SPePHS);                   ///see MainWindowDiskIO.cpp    
    QStringList LoadedEventFiles;

    void ShowSource(int isource, bool clear = true); ///mainWindowParticleSimulation.cpp

    PMtypeClass *tmpPMtype;  /// *** will be gone after cleanup

    QString CheckerScript;

    QVector<ScanFloodStructure> ScanFloodNoise;
    bool NoiseTableLocked;
    double ScanFloodNoiseProbability;

    QString PreprocessingFileName;

    int PreviousNumberPMs;
    bool ShowTop;
    bool ColorByMaterial;

    bool populateTable; //for SimLoadConfig - compatability check

    void clearPreprocessingData();
    void updateCOBsWithPMtypeNames();
    void CalculateIndividualQEPDE();
    void ViewChangeRelFactors(QString options);
    void CalculateElChannelsStrength();
    void ScaleSPePHS(int ipm, double factor);
   // template <typename TN> int LoadSimulationDataFromTreeHelper(TTree *T);
    void RemoveParticle(int Id);

private slots:
    void timerTimeout(); //timer-based update of Root events
    void on_cbIndividualParticle_clicked(bool checked);
    void on_ledLinkingProbability_editingFinished();
    void on_pbShowCheckUpWindow_clicked();
    void on_lePreprocessingMultiply_editingFinished();
    void on_extractPedestals_clicked();   
    void on_sbLoadedEnergyChannelNumber_editingFinished();
    void on_pbPositionScript_clicked();
    void on_cbUseSameSizes_toggled(bool checked);
    void on_pbCheckChangeSizes_clicked();   
    void on_cobPMtypes_currentIndexChanged(int index);
    void on_pbShowMaterialInPMtypes_clicked();
    void on_cobPMtypeInArrays_currentIndexChanged(int index);
    void on_pbLoadPMcenters_clicked();
    void on_pbSavePMcenters_clicked();
    void on_pbSetPMtype_clicked();    
    void on_pbViewChangeRelQEfactors_clicked();
    void on_pbLoadRelQEfactors_clicked();
    void on_pbViewChangeRelELfactors_clicked();
    void on_pbLoadRelELfactors_clicked();
    void on_pbRandomScaleELaverages_clicked();
    void on_pbSetELaveragesToUnity_clicked();
    void on_pbShowRelGains_clicked();
    void on_actionSave_configuration_triggered();
    void on_actionLoad_configuration_triggered();
    void on_actionSave_detector_constructor_triggered();
    void on_actionLoad_detector_settings_triggered();
    void on_actionSave_simulation_settings_triggered();
    void on_actionLoad_simulation_settings_triggered();
    void on_actionQuicksave_triggered();
    void on_actionQuickload_triggered();
    void on_pbRemoveParticle_clicked();
    void on_ledSourceActivity_editingFinished();
    void on_lwGunParticles_currentRowChanged(int currentRow);
    void on_pbSaveParticleSource_clicked();
    void on_pbLoadParticleSource_clicked();

    void on_ledGunSpread_editingFinished();
    void on_pbSaveResults_clicked();
    void on_pbRemoveMaterial_clicked();
    void on_lwLoadedEventsFiles_itemChanged(QListWidgetItem *item);
    void on_pobTest_clicked();
    void on_pbCheckDerivatives_clicked();    
    void on_actionGain_evaluation_triggered();
    void on_leiParticleLinkedTo_editingFinished();
    void on_cbLRFs_toggled(bool checked);  
    void on_pbClearAdd_clicked();
    void on_pbClearMulti_clicked();
    void on_actionCredits_triggered();
    void on_actionVersion_triggered();
    void on_actionLicence_triggered();
    void on_pnShowHideAdvanced_toggled(bool checked);
    void on_pbYellow_clicked();
    void on_pbReconstruction_2_clicked();

private slots:
    void on_cbPointSourceBuildTracks_toggled(bool checked);
    void on_cbGunPhotonTracks_toggled(bool checked);
    void on_cbBuilPhotonTrackstester_toggled(bool checked);

    /************************* Simulation *************************/
private:
    QThread simThread;
    QTimer simTimerGuiUpdate;
    void startSimulation(QJsonObject &json);
    ParticleSourceSimulator *setupParticleTestSimulation(GeneralSimSettings &simSettings);
signals:
    void StopRequested(); //to be used to request stop
private slots:
    void on_sbNumberThreads_valueChanged(int arg1);
    void simulationFinished();
    /**************************************************************/

    void on_pbUpdateSandwich_clicked();

    void on_pbGDML_clicked();

    void on_cobParticleSource_currentIndexChanged(int index);

    void on_cobParticleSource_editTextChanged(const QString &arg1);

    void on_pbLoadNodes_clicked();

    void on_pbShowNodes_clicked();

    void on_pbRunNodeScript_clicked();

    void on_sbNumberThreads_editingFinished();

    void on_cobPMdeviceType_activated(const QString &arg1);

    void on_cobMatPointSource_activated(int index);

    void on_pbShowColorCoding_pressed();

    void on_pbShowColorCoding_released();

    void on_ledSourceActivity_textChanged(const QString &arg1);

    void on_actionOpen_settings_triggered();

    void on_actionSave_Load_windows_status_on_Exit_Init_toggled(bool arg1);

    void on_pbShowEnergyDeposition_clicked();

    void on_pbShowSelected_clicked();

    void on_cobWhatToShow_currentIndexChanged(int index);

    void on_pbUpdateElectronics_clicked();

    void on_dsbNewProbLoss_editingFinished();

    void on_dsbNewProbReflection_editingFinished();

    void on_dsbNewProbDiffuse_editingFinished();

    void on_pbOverlay_clicked();

    void on_pbScalePDE_clicked();

    void on_pbLoadManifestFile_clicked();

    void on_pbDeleteManifestFile_clicked();

    void on_pbManifestFileHelp_clicked();

    void on_leRootIn_editingFinished();

    void on_pbLoadAppendFiles_clicked();

    void on_sbLoadASCIIpositionXchannel_valueChanged(int arg1);

    void on_cbLoadedDataHasPosition_toggled(bool checked);

    void on_pbPDEFromWavelength_clicked();

    void on_cobLoadDataType_customContextMenuRequested(const QPoint &pos);

    void on_pbManuscriptExtractNames_clicked();

    void on_pbShowDetailedLog_clicked();

    void on_pbCheckerScript_clicked();

    void on_pbCSMtestmany_clicked();

    void on_pbST_showTracks_clicked();

    void on_pbST_AngleCos_clicked();

    void on_lwOverrides_itemClicked(QListWidgetItem *item);

    void on_pbST_uniform_clicked();

    void on_pbST_RvsAngle_clicked();

    void on_pbTestFit_clicked();

    void on_pbST_ReflectionVsParameter_clicked();

    void on_pbST_VsParameterHelp_clicked();

    void on_cobST_ShowWhatRef_activated(int index);

    void on_pbSF_SvsL_clicked();

    void on_pbSF_help_clicked();

    void on_pbTestFit_simplistic_clicked();

    void on_cbLoadedDataHasZPosition_clicked(bool checked);

    void on_actionGlobal_script_triggered();

public slots:
    void on_cobSF_chi2Vs_activated(int index);
    void on_pbRebuildDetector_clicked();

private:
    void initDetectorSandwich();
    void SourceUpdateThisParticleIndication();

};

#endif // MAINWINDOW_H
