#ifndef MATERIALINSPECTORWINDOW_H
#define MATERIALINSPECTORWINDOW_H

#include <QMainWindow>
//#include "mainwindow.h"
#include "classes.h"
#include "materialcollectionclass.h"

class MainWindow;
class TGraph;
class DetectorClass;

namespace Ui {
class MaterialInspectorWindow;
}

class MaterialInspectorWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MaterialInspectorWindow(QWidget* parent, MainWindow *mw, DetectorClass *detector);
    ~MaterialInspectorWindow();

    //MaterialStructure tmpMaterial;
    //bool LogLogInterpolation;

    void UpdateActiveMaterials();
    void VisualizeTmpMaterial();
    void UpdateSelection();
    void SetParticleSelection(int index);
    void SetMaterial(int index);
    void ShowTotalInteraction();

    void AddMatToCobs(QString str);
    void RemoveMaterial(int imat);
    void SetMaterialSelection(int i); //can be used to refresh: -1 then 0
    void setLogLog(bool flag);

    void ConvertToStandardWavelengthes(QVector<double>* sp_x, QVector<double>* sp_y, QVector<double>* y);

    void UpdateMaterialInCobs(int imat);
protected:
    bool event(QEvent * e);

private slots:
    void on_pbAddToLibrary_clicked();

    void on_pbLoadFromLibrary_clicked();

    void on_pbAddToActive_clicked();  

    void on_pbShowThisScenarioCrossSection_clicked();

    void on_cobActiveMaterials_currentIndexChanged(int index);

    void on_cobParticle_currentIndexChanged(int index);

    void on_pbShowTotalInteraction_clicked();

    void on_cobTerminationScenarios_currentIndexChanged(int index);

    void on_pbUpdateTmpMaterial_clicked();

    void on_pbLoadDeDr_clicked();

    void on_pbLoadThisScenarioCrossSection_clicked();

    void on_pbAddNewTerminationScenario_clicked();

    void on_pbAddNewSecondary_clicked();

    void on_ledPrimaryYield_editingFinished();

    void on_ledBranching_editingFinished();

    void on_pbShowStoppingPower_clicked();

    void on_pbImportStoppingPowerFromTrim_clicked();

    void on_pbImportXCOM_clicked();

    void on_pbLoadTotalInteractionCoefficient_clicked();

    void on_pbNeutronClear_clicked();



    void on_pbCleanTmpMaterialMatParticles_clicked();



    void on_pbCleanThisParticleMatParticleData_clicked();

    void on_pbLoadPrimSpectrum_clicked();

    void on_pbShowPrimSpectrum_clicked();

    void on_pbDeletePrimSpectrum_clicked();



    void on_pbLoadSecSpectrum_clicked();

    void on_pbShowSecSpectrum_clicked();

    void on_pbDeleteSecSpectrum_clicked();

    void on_pbLoadNlambda_clicked();

    void on_pbShowNlambda_clicked();

    void on_pbDeleteNlambda_clicked();

    void on_pbLoadABSlambda_clicked();

    void on_pbShowABSlambda_clicked();

    void on_pbDeleteABSlambda_clicked();

    void on_pbWasModified_clicked();

    void on_pbGammaDiagnosticsPhComptRation_clicked();

    void on_ledGammaDiagnosticsEnergy_editingFinished();

    void on_pbShowThisScenarioCrossSectionInterpolated_clicked();

//    void on_sbInterpolationDegree_valueChanged(int arg1);

    void on_cbTrackingAllowed_toggled(bool checked);

    void on_leName_editingFinished();

    void on_leName_textChanged(const QString &arg1);

    void on_cbTransparentMaterial_toggled(bool checked);

    void on_pbComments_clicked();

    void on_ledRayleighWave_editingFinished();

    void on_ledRayleigh_textChanged(const QString &arg1);

    void on_ledRayleigh_editingFinished();

    void on_pbRemoveRayleigh_clicked();

    void on_pbShowUsage_clicked();

    void on_ledMFPenergy_editingFinished();

    void on_pbNistPage_clicked();

    void on_pbUpdateIndication_clicked();

    void on_pbRemoveSecondary_clicked();

    void on_lwGeneratedParticlesEnergies_currentRowChanged(int currentRow);

    void on_ledInitialEnergy_editingFinished();

    void on_cobSecondaryParticleToAdd_activated(int index);

    void on_pbRename_clicked();

    void on_cbLogLogInterpolation_clicked(bool checked);

    void on_ledMFPenergy_2_editingFinished();

private:
    Ui::MaterialInspectorWindow *ui;
    MainWindow* MW;
    DetectorClass* Detector;

    TGraph* graphOver;
    bool DoNotDefocus;
    bool flagDisreguardChange;
    int LastSelectedParticle;
    bool DoNotUpdateOnCOBchange;

    void UpdateWaveButtons();
    //void UpdateTransparencyLabel();

//    int InterpolationDegree;

};

#endif // MATERIALINSPECTORWINDOW_H
