//ANTS2
#include "outputwindow.h"
#include "ui_outputwindow.h"
#include "mainwindow.h"
#include "graphwindowclass.h"
#include "pms.h"
#include "windownavigatorclass.h"
#include "guiutils.h"
#include "materialcollectionclass.h"
#include "classes.h"
#include "reconstructionwindow.h"
#include "eventsdataclass.h"
#include "detectorclass.h"
#include "dynamicpassiveshandler.h"
#include "globalsettingsclass.h"

//ROOT
#include "TGraph2D.h"
#include "TH2C.h"
#include "TH1D.h"
#include "TMath.h"

//Qt
#include <QtGui>
#include <QGraphicsItem>
#include <QString>

OutputWindow::OutputWindow(QWidget *parent, MainWindow *mw, EventsDataClass *eventsDataHub) :
    QMainWindow(parent),
    ui(new Ui::OutputWindow)
{
    MW = mw;
    EventsDataHub = eventsDataHub;
    GVscale = 10.0;
    ui->setupUi(this);

    this->setWindowTitle("Output");
    this->setMinimumWidth(380);
    this->setMinimumHeight(150);

    Qt::WindowFlags windowFlags = (Qt::Window | Qt::CustomizeWindowHint);
    windowFlags |= Qt::WindowCloseButtonHint;
    this->setWindowFlags( windowFlags );

    modelPMhits = 0;
    hist = 0;
    LastOpenGraph = 0;

    WavelengthSpectrum = 0;
    AngularSpectrum = 0;

    ui->pbSiPMpixels->setEnabled(false);
    ui->sbTimeBin->setEnabled(false);
    ui->pbRefreshViz->setVisible(false);
    ui->tabPhStatistics->setEnabled(false);

    //Graphics view
    scaleScene = new QGraphicsScene(this);
    ui->gvScale->setScene(scaleScene);

    scene = new QGraphicsScene(this);
    //qDebug() << "This scene pointer:"<<scene;
    gvOut = new myQGraphicsView(ui->tabViz);
    gvOut->setGeometry(0,0,325,325);
    gvOut->setScene(scene);

    gvOut->setDragMode(QGraphicsView::ScrollHandDrag); //if zoomed, can use hand to center needed area
    gvOut->setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
    gvOut->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    gvOut->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    gvOut->setRenderHints(QPainter::Antialiasing);  

    ui->tabwinDiagnose->setCurrentIndex(0);
}

OutputWindow::~OutputWindow()
{
  clearGrItems();
  delete ui;  
}

void OutputWindow::PMnumChanged()
{
  ui->sbPMnumberToShowTime->setValue(0);
  SiPMpixels.clear();
}

void OutputWindow::SetCurrentEvent(int iev)
{  
  if (iev == ui->sbEvent->value()) OutputWindow::on_pbRefreshViz_clicked();
  else ui->sbEvent->setValue(iev); //update on_change
}

void OutputWindow::on_pbShowPMtime_clicked()
{  
    //if (!EventsDataHub->isTimed()) return;
    if (!EventsDataHub->LastSimSet.fTimeResolved) return;
    if (EventsDataHub->TimedEvents.isEmpty()) return;

    int CurrentEvent = ui->sbEvent->value();
    if (CurrentEvent>EventsDataHub->TimedEvents.size()-1)
      {
        message("Invalid event number!", this);
        return;
      }

    int bins = EventsDataHub->LastSimSet.TimeBins;
    double from = EventsDataHub->LastSimSet.TimeFrom;
    double to   = EventsDataHub->LastSimSet.TimeTo;
//    double step = (to-from)/bins;

    int ipm = ui->sbPMnumberToShowTime->value();

//    QVector<double> x;
//    x.resize(0);
//    for (int i=0; i<bins; i++)
//      {
//        double xx = from + (0.5 + i)*step;
//        x.append(xx);
//      }

//    QVector<double> y;
//    y.resize(0);

//    for (int i=0; i<bins; i++) y.append( EventsDataHub->TimedEvents[CurrentEvent][i][ipm]);

//    MW->GraphWindow->ShowGraph(&x, &y, kRed, "Time, ns", "Signal");
//    LastOpenGraph = 1;

    if (MW->hist1) delete MW->hist1;
    MW->hist1 = new TH1D("Time_spec_ev", "Time of detection", bins, from, to);
    for (int i=0; i<bins; i++)
      MW->hist1->SetBinContent(i+1, EventsDataHub->TimedEvents[CurrentEvent][i][ipm]);
    MW->hist1->GetXaxis()->SetTitle("Time, ns");
    MW->hist1->GetYaxis()->SetTitle("Signal");
    MW->GraphWindow->Draw(MW->hist1);
}

void OutputWindow::RefreshPMhitsTable()
{
    bool fHaveData = !EventsDataHub->isEmpty();    
    int CurrentEvent = ui->sbEvent->value();
    if (fHaveData && CurrentEvent>EventsDataHub->Events.size()-1) fHaveData = false; //protection

    int columns = MW->PMs->count() + 1; //+sum over all PMs  
    int rows = 0;    
    if (fHaveData && EventsDataHub->isTimed())
        rows = EventsDataHub->TimedEvents[0].size();

    if (modelPMhits)
      {
        ui->tvPMhits->setModel(0);
        delete modelPMhits;
      }

    modelPMhits = new QStandardItemModel(rows+1,columns,this);

    ui->tvPMhits->horizontalHeader()->setVisible(true);
    ui->tvPMhits->verticalHeader()->setVisible(true);

    //qDebug()<<"rows: "<<rows<<"columns: "<<columns;

    QString tmpStr;
    //prepare headers
      //horizontal
    modelPMhits->setHorizontalHeaderItem(0, new QStandardItem("sum(PMs)"));
    for (int i=1; i<columns;i++)
      {
        tmpStr.setNum(i-1);
        tmpStr = "PM# "+tmpStr;
        modelPMhits->setHorizontalHeaderItem(i, new QStandardItem(tmpStr));
      }
      //vertical
    if (rows>0) modelPMhits->setVerticalHeaderItem(0, new QStandardItem("sum(t)"));
    else modelPMhits->setVerticalHeaderItem(0, new QStandardItem(""));
    for (int i=0; i<rows;++i)
      {
        tmpStr.setNum(i);
        modelPMhits->setVerticalHeaderItem(i+1, new QStandardItem(tmpStr));
      }

    //filling with data
    for (int iColumn=0; iColumn<columns; iColumn++)
      {        
        if (!fHaveData) tmpStr = "";
        else
          {
            if (iColumn == 0)
              {
                //top is sum
                double sum = 0;               
                for (int ipm=0; ipm<MW->PMs->count(); ipm++) sum += EventsDataHub->Events[CurrentEvent][ipm];

                tmpStr.setNum(sum, 'g', 6);
              }
            else
              {
                //if (Events->isEmpty()) tmpStr = "-";
                tmpStr.setNum( EventsDataHub->Events[CurrentEvent][iColumn-1], 'g', 6);
              }
          }
        QStandardItem* item = new QStandardItem(tmpStr);
        modelPMhits->setItem(0,iColumn,item);

        //the rest is time-resolved info
        for (int j=0; j<rows; j++)
          {
            if (!fHaveData) tmpStr = "";
            else
              {
                if (iColumn == 0)
                  {
                    double sum=0;                    
                    for (int ipm=0; ipm<MW->PMs->count(); ipm++) sum += EventsDataHub->TimedEvents[CurrentEvent][j][ipm];
                    tmpStr.setNum(sum, 'g', 6);
                  }
                else tmpStr.setNum( EventsDataHub->TimedEvents[CurrentEvent][j][iColumn-1], 'g', 6);
              }

           item = new QStandardItem(tmpStr);
           modelPMhits->setItem(j+1,iColumn,item);
          }
      }
    ui->tvPMhits->setModel(modelPMhits);
}

void OutputWindow::clearGrItems()
{
  //qDebug() << "Item cleanup. This scene:"<<scene;
  for (int i=0; i<grItems.size(); i++)
    {
      if (grItems[i]->scene() != scene)
        qDebug() << i << grItems[i] << grItems[i]->scene();
      //QGraphicsTextItem *gr = dynamic_cast<QGraphicsTextItem*>(grItems[i]);
      //if (gr) qDebug() << i << gr->document()->toHtml();
      scene->removeItem(grItems[i]);
      delete grItems[i];
    }
  grItems.clear();
}

void OutputWindow::on_sbPMnumberToShowTime_valueChanged(int arg1)
{
    if (arg1 > MW->PMs->count()-1) ui->sbPMnumberToShowTime->setValue(0);

    if (LastOpenGraph == 1) OutputWindow::on_pbShowPMtime_clicked();
    if (LastOpenGraph == 2) OutputWindow::on_pbSiPMpixels_clicked();
}

void OutputWindow::OutTextClear()
{
    ui->teOut->clear();
}

void OutputWindow::OutText(QString str)
{
   ui->teOut->append(str);
}

void OutputWindow::SetTab(int index)
{
    if (index <0) return;
    if (index > ui->tabwinDiagnose->count()-1) return;
    ui->tabwinDiagnose->setCurrentIndex(index);
}

void OutputWindow::on_pbSiPMpixels_clicked()
{
    if (SiPMpixels.isEmpty())
      {
        message("Pixels are shown only for photon source simulation with one event!");
        return;
      }

    int ipm = ui->sbPMnumberToShowTime->value();
    if (!MW->PMs->isSiPM(ipm))
      {
        MW->GraphWindow->close();
        return;
      }

    int iTime = ui->sbTimeBin->value();
    if (iTime > EventsDataHub->getTimeBins()-1)
    {
      ui->sbTimeBin->setValue(0);
      MW->GraphWindow->close();
      return;
    }

    int binsX = MW->PMs->getPixelsX(ipm);
    int binsY = MW->PMs->getPixelsY(ipm);
//    qDebug()<<binsX<<binsY;

    if (hist) delete hist;
    hist = new TH2C("histOutW","x vs y",binsX,0,binsX, binsY, -binsY,0);

    PMtypeClass *typ = MW->PMs->getTypeForPM(ipm);//  PMtypeProperties[PMs[ipm].type];
    for (int iX=0; iX<binsX; iX++)
     for (int iY=0; iY<binsY; iY++)
      {         
        if (SiPMpixels[ipm].testBit((iTime*typ->PixelsY + iY)*typ->PixelsX + iX)) hist->Fill(iX, -iY);
        //if (MW->PMs->SiPMpixel(ipm, iTime, iX, iY)) hist->Fill(iX, -iY);
      }

    MW->GraphWindow->Draw(hist, "col");
    LastOpenGraph = 2;
}

void OutputWindow::on_sbTimeBin_valueChanged(int arg1)
{
   if (SiPMpixels.isEmpty()) return;
   if (EventsDataHub->TimedEvents.isEmpty()) return;
   if (arg1 > EventsDataHub->TimedEvents[0].size()-1) ui->sbTimeBin->setValue(0);

   OutputWindow::on_pbSiPMpixels_clicked();
}

void OutputWindow::LogEventHistory()
{  
  if (EventsDataHub->EventHistory.size() == 0)
    {
     message("No data available!", this);
     return;
    }

  OutText("----------------------------------");
  double TotalEnergyDeposited=0;

  for (int i=0; i<EventsDataHub->EventHistory.size(); i++)
    {
      QString tmpStr, tmpStr1;
      tmpStr.setNum(i);

      int Id = EventsDataHub->EventHistory[i]->ParticleId;
      tmpStr1.setNum(Id);
      tmpStr += "> " + MW->Detector->ParticleCollection[Id]->ParticleName + " (id: "+tmpStr1+")";

      for (int m=0; m < EventsDataHub->EventHistory[i]->Deposition.size(); m++)
      {
          int MatId = EventsDataHub->EventHistory[i]->Deposition[m].MaterialId;
          tmpStr += " " + (*MW->MaterialCollection)[MatId]->name +":";
          tmpStr1.setNum(EventsDataHub->EventHistory[i]->Deposition[m].Distance);
          tmpStr += tmpStr1+"mm:";
          tmpStr1.setNum(EventsDataHub->EventHistory[i]->Deposition[m].DepositedEnergy);
          tmpStr += tmpStr1+"keV;";
          TotalEnergyDeposited += EventsDataHub->EventHistory[i]->Deposition[m].DepositedEnergy;
      }

      tmpStr += " Termination:";
      int ter = EventsDataHub->EventHistory[i]->Termination;
      if (ter == 1) tmpStr += "escaped";
      else if (ter == 2) tmpStr += "stopped";
      else if (ter == 3) tmpStr += "photoelectric";
      else if (ter == 4) tmpStr += "compton";
      else if (ter == 5) tmpStr += "capture";
      else if (ter == 7) tmpStr += "created outside the defined geometry";
      else if (ter == 8) tmpStr += "found untrackable material";

      OutText(tmpStr);
  }

  OutText("");
  QString tmpStr;
  tmpStr.setNum(TotalEnergyDeposited);
  tmpStr = "Total energy deposited: "+tmpStr + " keV";
  OutText(tmpStr);
  OutText("----------------------------------");

//  SetTab(0);
}

void OutputWindow::resizeEvent(QResizeEvent *)
{
  if (!this->isVisible()) return;
  int Height = this->height();
  int Width = this->width();

  ui->fSelector->move(6, Height - (385 - 356));
  ui->tabwinDiagnose->resize(Width - (524-521), Height - (385 - 351));
  ui->teOut->resize(Width - (524 - 511), Height - (385-321));
  int NewHeight = Height - (385-150);
  if (NewHeight < 0 ) NewHeight = 0;
  ui->tvPMhits->resize(Width - (524 - 513), ui->tabwinDiagnose->height()-28 - ui->fPMhits->height());//(385-294));
  ui->fPMhits->move(0, ui->tvPMhits->height());// (385 - 297));

  int GVsize = ui->tabwinDiagnose->height()-28;//325;
  gvOut->resize(GVsize, GVsize);
  ui->gvScale->move(GVsize-325+330, 20);
  ui->fGVScale->move(GVsize-325+330, 33);
  ui->fGVpanel->move(GVsize-325+384, 0);
  OutputWindow::ResetViewport();
}

void OutputWindow::ResetViewport()
{
  if (MW->PMs->count() == 0) return;

  //calculating viewing area
  double Xmin =  1e10;
  double Xmax = -1e10;
  double Ymin =  1e10;
  double Ymax = -1e10;
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      double x = MW->PMs->at(ipm).x;
      double y = MW->PMs->at(ipm).y;
      int type = MW->PMs->at(ipm).type;
      double size = MW->PMs->getType(type)->SizeX;

      if (x-size < Xmin) Xmin = x-size;
      if (x+size > Xmax) Xmax = x+size;
      if (y-size < Ymin) Ymin = y-size;
      if (y+size > Ymax) Ymax = y+size;
    }

  double Xdelta = Xmax-Xmin;
  double Ydelta = Ymax-Ymin;

  scene->setSceneRect((Xmin - 0.5*Xdelta)*GVscale, (Ymin - 0.5*Ydelta)*GVscale, (Xmax-Xmin + 1.0*Xdelta)*GVscale, (Ymax-Ymin + 1.0*Ydelta)*GVscale);
  gvOut->fitInView( (Xmin - 0.01*Xdelta)*GVscale, (Ymin - 0.01*Ydelta)*GVscale, (Xmax-Xmin + 0.02*Xdelta)*GVscale, (Ymax-Ymin + 0.02*Ydelta)*GVscale, Qt::KeepAspectRatio);
}

void OutputWindow::InitWindow()
{
  bool shown = this->isVisible();
  this->showNormal();
  SetTab(2);
  RefreshData();
  OutputWindow::resizeEvent(0);
  SetTab(0);
  if (!shown) this->hide();
}

void OutputWindow::on_pbRefreshViz_clicked()
{
  OutputWindow::RefreshData();
}

void OutputWindow::RefreshData()
{
  bool fHaveData = !EventsDataHub->isEmpty();

  if (!fHaveData) ui->labTotal->setText("-");
  else ui->labTotal->setText(QString::number(EventsDataHub->Events.size()));

  //check current event
  int CurrentEvent = ui->sbEvent->value();
  if (CurrentEvent > EventsDataHub->Events.size()-1)
    {
      int iev = EventsDataHub->Events.size()-1;
      if (iev < 0) iev = 0;
      CurrentEvent = iev;
      ui->sbEvent->setValue(iev);
    }
  //bool fHaveEnVector = !MW->EnergyVector.isEmpty();
  //bool fHaveEventsLog = !EventsDataHub->EventHistory.isEmpty();
  bool fHaveSiPMpixData = !SiPMpixels.isEmpty();
  bool fHaveStat = !EventsDataHub->isStatEmpty(); //was crashing in debug mode on win
  bool fTimeResolved = !EventsDataHub->TimedEvents.isEmpty();

  //ui->tabEnergyDeposition->setEnabled(fHaveEnVector || fHaveEventsLog);
  ui->pbSiPMpixels->setEnabled(fHaveSiPMpixData);
  ui->tabPhStatistics->setEnabled(fHaveStat);
  ui->sbTimeBin->setEnabled(fTimeResolved);
  ui->pbShowPMtime->setEnabled(fTimeResolved);

  //dynamic passives for indication
  DynamicPassivesHandler *Passives = new DynamicPassivesHandler(MW->Detector->PMs, EventsDataHub);
  (EventsDataHub->isEmpty()) ? Passives->init(0) : Passives->init(&EventsDataHub->RecSettings); //just to copy static passives
  if (EventsDataHub->fReconstructionDataReady)
    if (EventsDataHub->RecSettings.fUseDynamicPassives) Passives->calculateDynamicPassives(CurrentEvent, EventsDataHub->ReconstructionData.at(CurrentEvent));

   //updating PMsignal table
   OutputWindow::RefreshPMhitsTable();
    //qDebug()<<"table updated";

    //updating viz
    //qDebug() << "------------crash point before-------";
    //qDebug() << "Items:"<<grItems.size();
    //clearGrItems();
    scene->clear();  //it crashes in Qt 5.4.1 due to QGraphicsTextItems
    //qDebug() << "------------crash point after -------";

    double MaxSignal = 0.0;
    if (!fHaveData) MaxSignal = 1.0;
    else
      {
        for (int i=0; i<MW->PMs->count(); i++)
         //if (!MW->PMs->at(i).isStaticPassive())  //normall passives
         if (Passives->isActive(i))
          if ( EventsDataHub->Events[CurrentEvent][i] > MaxSignal) MaxSignal = EventsDataHub->Events[CurrentEvent][i];
      }
    if (MaxSignal<=1.0e-25) MaxSignal = 1.0;
    //qDebug()<<"MaxSignal="<<MaxSignal<<"  selector="<<selector;

    updateSignalLabels(MaxSignal);
    addPMitems(fHaveData, CurrentEvent, MaxSignal, Passives); //add icons with PMs to the scene
    if (ui->cbShowPMsignals->isChecked())
      addTextitems(fHaveData, CurrentEvent, MaxSignal, Passives); //add icons with signal text to the scene
    updateSignalScale();
    //gvOut->update();

    delete Passives;
}

void OutputWindow::addPMitems(bool fHaveData, int CurrentEvent, double MaxSignal, DynamicPassivesHandler *Passives)
{
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      //pen
      QPen pen(Qt::black);
      int size = 6.0 * MW->PMs->SizeX(ipm) / 30.0;
      pen.setWidth(size);

      //brush
      QBrush brush(Qt::white);

      double sig = 0;
      if (fHaveData) sig = EventsDataHub->Events[CurrentEvent][ipm];

      if (sig > 0)
        {
          QColor color = QColor(0,0,0);
          int level = 255.0*sig/MaxSignal;
          if (level>255) level = 255;
//            qDebug()<<"Level = "<<level;

          if (level <64) color.setRgb(0, level*4,255);
          else if (level < 128) color.setRgb(0,255,255 - (level-64)*4);
          else if (level < 192) color.setRgb((level-128)*4,255,0);
          else color.setRgb(255,255 -(level-192)*4,0);

          brush.setColor(color);
        }
      else
        {
          if (sig == 0) brush.setColor(Qt::white);
          else brush.setColor(Qt::black);
        }

      const pm &PM = MW->PMs->at(ipm);
      if (Passives->isPassive(ipm)) brush.setColor(Qt::black);

      QGraphicsItem* tmp;
      if (MW->PMs->getType(PM.type)->Shape == 0)
        {
          double sizex = MW->PMs->getType(PM.type)->SizeX*GVscale;
          double sizey = MW->PMs->getType(PM.type)->SizeY*GVscale;
          tmp = scene->addRect(-0.5*sizex, -0.5*sizey, sizex, sizey, pen, brush);
        }
      else if (MW->PMs->getType(PM.type)->Shape == 1)
        {
          double diameter = MW->PMs->getType(PM.type)->SizeX*GVscale;
          tmp = scene->addEllipse( -0.5*diameter, -0.5*diameter, diameter, diameter, pen, brush);
        }
      else
        {
          double radius = 0.5*MW->PMs->getType(PM.type)->SizeX*GVscale;
          QPolygon polygon;
          for (int j=0; j<7; j++)
            {
              double angle = 3.1415926535/3.0 * j + 3.1415926535/2.0;
              double x = radius*cos(angle);
              double y = radius*sin(angle);
              polygon << QPoint(x, y);
            }
          tmp = scene->addPolygon(polygon, pen, brush);
        }

      if (ui->cbViewFromBelow->isChecked()) tmp->setZValue(-PM.z);
      else tmp->setZValue(PM.z);

      tmp->setRotation(PM.psi);
      tmp->setTransform(QTransform().translate(PM.x*GVscale, -PM.y*GVscale)); //minus!!!!

      //grItems.append(tmp);
    }
}

void OutputWindow::addTextitems(bool fHaveData, int CurrentEvent, double MaxSignal, DynamicPassivesHandler *Passives)
{
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      //QGraphicsTextItem *io = scene->addText("dummy");

      const pm &PM = MW->PMs->at(ipm);
      double size = 0.5*MW->PMs->getType( PM.type )->SizeX;
      //io->setTextWidth(40);


      double sig = 0;
      if (fHaveData) sig = EventsDataHub->Events[CurrentEvent][ipm];
      //qDebug()<<"sig="<<sig;
      QString text = QString::number(sig, 'g', 6);
      //text = "<CENTER>" + text + "</CENTER>";

      //color correction for dark blue
      QGraphicsSimpleTextItem *io = scene->addSimpleText(text);

      if (sig != 0 && sig < 0.15*MaxSignal || Passives->isPassive(ipm))
      //  text = "<font color=\"white\">" + text + "</font>";
        io->setBrush(Qt::white);
      //if (Passives->isPassive(ipm)) text = "<font color=\"white\">" + text + "</font>";

      //io->setHtml(text);
      //double x = ( PM.x - 0.75*size  )*GVscale;
      //qDebug() << "text"<<text<<io->boundingRect().width() ;
      int wid = (io->boundingRect().width()-6)/6;
      double x = ( PM.x -wid*size*0.125 -0.1*size )*GVscale;
      double y = (-PM.y - 0.3*size  ) *GVscale; //minus y to invert the scale!!!
      io->setPos(x, y);
      io->setScale(0.4*size);

      if (ui->cbViewFromBelow->isChecked()) io->setZValue(-PM.z+0.001);
      else io->setZValue(PM.z+0.001);

      //grItems.append(io);
    }
}

void OutputWindow::updateSignalScale()
{
  scaleScene->clear();
  for (int i=0; i<16; i++)
    {
       int level = 8+i*16;
       QBrush brush(Qt::white);
       QColor color = QColor(0,0,0);

       if (level <64) color.setRgb(0, level*4,255);
       else if (level < 128) color.setRgb(0,255,255 - (level-64)*4);
       else if (level < 192) color.setRgb((level-128)*4,255,0);
       else color.setRgb(255,255 -(level-192)*4,0);

       brush.setColor(color);

       QPen pen(Qt::white);
       scaleScene->addRect( 0, 16.0*(-i+7.5), 35, 16, pen, brush);
    }
}

void OutputWindow::updateSignalLabels(double MaxSignal)
{
  QString str;
  str.setNum(MaxSignal, 'g', 4);
  ui->labHitsMax->setText(str);

  double lowmid = 0.25*MaxSignal;
  if (lowmid>0)
    {
      str.setNum(lowmid, 'g', 4);
      ui->labLowMid->setText(str);
    }
  else ui->labLowMid->setText("");

  double mid = 0.5*MaxSignal;
  if (mid > 0 && mid !=lowmid)
    {
      str.setNum(mid, 'g', 4);
      ui->labMid->setText(str);
    }
  else ui->labMid->setText("");

  double highmid = 0.75*MaxSignal;
  if (highmid > 0 && highmid !=mid)
    {
      str.setNum(highmid, 'g', 4);
      ui->labHighMid->setText(str);
    }
  else ui->labHighMid->setText("");
}

void OutputWindow::on_tabwinDiagnose_currentChanged(int index)
{
  //if (index<2) ui->fSelector->setEnabled(false);
  //else ui->fSelector->setEnabled(true);
  if (index == 0) ui->swTools->setCurrentIndex(0);
  if (index == 1) ui->swTools->setCurrentIndex(1);
  if (index == 2 || index == 3) ui->swTools->setCurrentIndex(2);
  if (index == 4) ui->swTools->setCurrentIndex(1);

  if (index == 3) OutputWindow::on_pbRefreshViz_clicked();
  gvOut->update();
}

bool OutputWindow::event(QEvent *event)
{
  if (!MW->WindowNavigator) return QMainWindow::event(event);

  if (event->type() == QEvent::Hide) MW->WindowNavigator->HideWindowTriggered("out");
  if (event->type() == QEvent::Show) MW->WindowNavigator->ShowWindowTriggered("out");

  return QMainWindow::event(event);
}

void OutputWindow::on_pbWaveSpectrum_clicked()
{
  if (!EventsDataHub->LastSimSet.fWaveResolved) return;
  TH1I* spec = EventsDataHub->DetStat->getWaveSpectrum();
  if (!spec)
    {
      message("Data not found!", this);
      return;
    }

  //converting to wavelength
  if (WavelengthSpectrum) delete WavelengthSpectrum;
  int nBins = spec->GetNbinsX();
  WavelengthSpectrum = new TH1D("WaveSpectrumOutput","Wavelength spectrum", MW->WaveNodes, -1, -2);
  for (int i=1; i<nBins+1; i++) //0 - underflow, n+1 - overflow
    {
      int iWave = spec->GetBinCenter(i);
      double wavelength = 0; //to spot photons without wavelength
      if (iWave > -1) wavelength = MW->WaveFrom + iWave*MW->WaveStep;
      WavelengthSpectrum->Fill(wavelength, spec->GetBinContent(i));
      //qDebug()<<i<<spec->GetBinCenter(i)<<spec->GetBinContent(i);
    }

  MW->GraphWindow->Draw(WavelengthSpectrum);
}

void OutputWindow::on_pbTimeSpectrum_clicked()
{
  //if (!MW->TimeResolved) return;
  if (!EventsDataHub->isTimed()) return;

  TH1D* spec = EventsDataHub->DetStat->getTimeSpectrum();
  if (!spec)
    {
      message("Data not found!", this);
      return;
    }

  MW->GraphWindow->Draw(spec);
}

void OutputWindow::on_pbAngleSpectrum_clicked()
{
  //if (!MW->AngularSensitivity) return;
  if (!EventsDataHub->LastSimSet.fAngResolved) return;

  TH1I* spec = EventsDataHub->DetStat->getCosAngleSpectrum();
  if (!spec)
    {
      message("Data not found!", this);
      return;
    }

  //converting to degrees
  if (AngularSpectrum) delete AngularSpectrum;
  int nBins = spec->GetNbinsX();
  AngularSpectrum = new TH1D("AngleSpectrumOutput","Angular distribution", 200, -1, -2);

  for (int i=1; i<nBins+1; i++) //0 - underflow, n+1 - overflow
    {
      double iCosAngle = spec->GetBinCenter(i);
      double Angle = TMath::ACos(iCosAngle)*180./3.1415926535;
      AngularSpectrum->Fill(Angle, spec->GetBinContent(i));
//      qDebug()<<i<<iCosAngle<<spec->GetBinContent(i)<<"----"<<Angle;
    }

  MW->GraphWindow->Draw(AngularSpectrum);
}

void OutputWindow::on_pbNumTransitionsSpectrum_clicked()
{  
   TH1I* spec = EventsDataHub->DetStat->getTransitionSpectrum();

   if (!spec)
     {
       message("Data not found!", this);
       return;
     }

   MW->GraphWindow->Draw(spec);
}

void OutputWindow::on_sbEvent_valueChanged(int arg1)
{
  if (arg1 == 0 && EventsDataHub->Events.isEmpty()) return;
  if (EventsDataHub->Events.isEmpty()) //protection
    {      
      ui->sbEvent->setValue(0);
      return;
    }

  int maxEvent = EventsDataHub->Events.size()-1;
  if (arg1 > maxEvent)
    {
      ui->sbEvent->setValue(maxEvent);
      return; //already triggered "on change" = this procedure
    } 
  OutputWindow::on_pbRefreshViz_clicked();
}

void OutputWindow::on_pbResetViewport_clicked()
{
    OutputWindow::RefreshData();
    OutputWindow::ResetViewport();
}

void OutputWindow::on_pbClearText_clicked()
{
    ui->teOut->clear();
}

void OutputWindow::textLogOneEvent(int iev, ReconstructionStruct* result)
{
  int precision = MW->GlobSet->TextLogPrecision;
  QString str = "Event# " + QString::number(iev);

  if (!EventsDataHub->isScanEmpty())
      if ( !EventsDataHub->Scan[iev]->GoodEvent )
        str += "       'Bad' event: " + MW->NoiseTypeDescriptions[EventsDataHub->Scan[iev]->EventType].replace("\n", " ");

  MW->Owindow->OutText(str);

  QString str2;
  if (!EventsDataHub->isScanEmpty())
    {
      int iPoints = EventsDataHub->Scan[iev]->Points.size();
      for (int iP=0; iP<iPoints; iP++)
        {
          str2.setNum(EventsDataHub->Scan[iev]->Points[iP].r[0], 'g', precision);
          str = "->   ";
          str +="<font color=\"blue\">Actual position:</font>  X = "+str2+" mm  Y = ";
          str2.setNum(EventsDataHub->Scan[iev]->Points[iP].r[1], 'g', precision);
          str +=str2+" mm   Z = ";
          str2.setNum(EventsDataHub->Scan[iev]->Points[iP].r[2], 'g', precision);
          str +=str2+" mm";
          str2.setNum(EventsDataHub->Scan[iev]->Points[iP].energy);
              str += "   photons/energy = "+str2;

          MW->Owindow->OutText(str);
        }
      MW->Owindow->OutText("   -----");
    }

  if (EventsDataHub->RecSettings.ReconstructionAlgorithm != 0)
    {
      QString str = " CoG reconstgruction: X= "
          +QString::number(result->xCoG, 'g', precision)
          + "   Y= "
          + QString::number(result->yCoG, 'g', precision)
          + "   Z= "
          + QString::number(result->zCoG, 'g', precision)
          + "   Max signal PMT:"
          + QString::number(result->iPMwithMaxSignal);

      MW->Owindow->OutText(str);
    }

  if (!result->ReconstructionOK)
    {
      MW->Owindow->OutText("   Position reconstruction failed!");
      return;
    }

  int Points = result->Points.size();
  for (int iP=0; iP<Points; iP++)
    {
      str2.setNum(result->Points[iP].r[0], 'g', precision);
      str = "->   ";
      str += "<font color=\"red\">Reconstruction:</font>  X = "+str2+" mm  Y = ";
      str2.setNum(result->Points[iP].r[1], 'g', precision);
      str +=str2+" mm   Z = ";
      str2.setNum(result->Points[iP].r[2], 'g', precision);
      str +=str2+" mm";
      //if (ui->cbReconstructEnergy->isChecked())
      if ( MW->Rwindow->isReconstructEnergy() )
        {
          str2.setNum(result->Points[iP].energy);
          str += "   Energy = "+str2;
        }
      MW->Owindow->OutText(str);
    }
  //if (result->chi2 != 0)
  {
      str2.setNum(result->chi2);
      str = "-> Chi2 = "+str2;
      MW->Owindow->OutText(str);
  }

#ifdef ANTS_FLANN
  if (result->FilterkNN > 0)
    {
      //str = "-> kNN module reports distance to the neighbour(s): " + QString::number(MW->Reconstructor->getNNaverageDistance(iev));
      //MW->Owindow->OutText(str);
      str = ">   kNN filter: ";
      if (result->FilterkNN == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
    }
#endif

  //filter status
  if (result->FilterCutOff > 0)
  {
      str=">   Individual cut-offs filter: ";
      if (result->FilterCutOff == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
  }
  if (result->FilterSumCutOff > 0)
  {
      str=">   Sum signal cut-offs filter: ";
      if (result->FilterSumCutOff == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
  }
  if (result->FilterLoadedEnergy > 0)
  {
      str = ">   Loaded energy filter: ";
      if (result->FilterLoadedEnergy == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
  }
  if (result->FilterEnergy > 0)
  {
      str=">   Energy filter: ";
      if (result->FilterEnergy == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
  }
  if (result->FilterChi2 > 0)
  {
      str=">   Chi2 filter: ";
      if (result->FilterChi2 == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
  }
  if (result->FilterPosition > 0)
    {
      str=">   Position filter: ";
      if (result->FilterPosition == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
    }
  if (result->FilterCorrelation > 0)
    {
      str=">   Correlation filters: ";
      if (result->FilterCorrelation == 1) str += "<font color=\"green\">Pass</font>";
      else str += "<font color=\"red\">Fail</font>";
      MW->Owindow->OutText(str);
    }

  MW->Owindow->OutText("========");
  MW->Owindow->OutText("");
}

void OutputWindow::on_pbAverageSignals_clicked()
{
  if (EventsDataHub->isEmpty()) return;

  bool fGains = ui->cbTakeGainIntoAccount->isChecked();
  bool fGood = ui->cbOnlyGood->isChecked();

  if (MW->hist1) delete MW->hist1;
  MW->hist1 = new TH1D("MaxSigDist","Distribution of max signal", MW->GlobSet->BinsX, 0,0);

  for (int iEvent=0; iEvent<EventsDataHub->Events.size(); iEvent++)
    {
      if (fGood && !EventsDataHub->ReconstructionData.at(iEvent)->GoodEvent) continue;

      float max = -1e10;
      for (int ipm=0; ipm<MW->Detector->PMs->count(); ipm++)
        {
          if (MW->Detector->PMs->isStaticPassive(ipm)) continue;

          float sig = EventsDataHub->Events[iEvent].at(ipm);
          if (fGains) sig /= MW->Detector->PMs->getGain(ipm);

          if (sig>max) max=sig;
        }

      MW->hist1->Fill(max);
    }

  MW->GraphWindow->Draw(MW->hist1);
}
