#ifndef OUTPUTWINDOW_H
#define OUTPUTWINDOW_H

#include <QMainWindow>
#include <QGraphicsScene>

class MainWindow;
class myQGraphicsView;
class TH2C;
class QStandardItemModel;
class TH1D;
struct ReconstructionStruct;
class EventsDataClass;
class DynamicPassivesHandler;

namespace Ui {
class OutputWindow;
}

class OutputWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit OutputWindow(QWidget* parent, MainWindow *mw, EventsDataClass *eventsDataHub);
    ~OutputWindow();

    QVector< QBitArray > SiPMpixels; //[PM#] [time] [pixY] [pixX] - to report only!

    void PMnumChanged();
    void SetCurrentEvent(int iev);

    void OutTextClear();
    void OutText(QString str);

    void SetTab(int index);

    void RefreshData();
    void ResetViewport(); //auto-size and center viewing area
    void InitWindow();

    void LogEventHistory();

    void textLogOneEvent(int iev, ReconstructionStruct *result);

protected:
    void resizeEvent(QResizeEvent *event);
    
private slots:
    void on_pbShowPMtime_clicked();  

    void on_sbPMnumberToShowTime_valueChanged(int arg1);

    void on_pbSiPMpixels_clicked();

    void on_sbTimeBin_valueChanged(int arg1);

    void on_pbRefreshViz_clicked();

    void on_tabwinDiagnose_currentChanged(int index);

    void on_pbWaveSpectrum_clicked();

    void on_pbTimeSpectrum_clicked();

    void on_pbAngleSpectrum_clicked();

    void on_pbNumTransitionsSpectrum_clicked();

    void on_sbEvent_valueChanged(int arg1);

    void on_pbResetViewport_clicked();

    void on_pbClearText_clicked();

    void on_pbAverageSignals_clicked();

protected:
    bool event(QEvent *event);   


private:
    Ui::OutputWindow *ui;
    MainWindow* MW;
    EventsDataClass *EventsDataHub;

    QGraphicsScene *scene;
    QGraphicsScene *scaleScene;

    myQGraphicsView *gvOut;

    QStandardItemModel *modelPMhits;
    TH2C *hist;

    TH1D* WavelengthSpectrum;
    TH1D* AngularSpectrum;

    void RefreshPMhitsTable();

    int LastOpenGraph;
    double GVscale;

    QList<QGraphicsItem*> grItems;
    void clearGrItems();
    void updateSignalLabels(double MaxSignal);
    void addPMitems(bool fHaveData, int CurrentEvent, double MaxSignal, DynamicPassivesHandler *Passives);
    void addTextitems(bool fHaveData, int CurrentEvent, double MaxSignal, DynamicPassivesHandler *Passives);
    void updateSignalScale();
};

#endif // OUTPUTWINDOW_H
