#ifndef RECONSTRUCTIONWINDOW_H
#define RECONSTRUCTIONWINDOW_H

#include <QMainWindow>
#include "CorrelationFilters.h"

namespace Ui {
  class ReconstructionWindow;
}

class MainWindow;

class TTree;
class TH1D;
class TH2D;
class TLine;
class TEllipse;
class TPolyLine;
class TMultiGraph;
class Viewer2DarrayObject;
class QComboBox;
class EventsDataClass;
struct CorrelationFilterStructure;

class linkCustomClass
{
public:
  QSpinBox* Qbins;
  int bins;
  QLineEdit* Qfrom;
  double from;
  QLineEdit* Qto;
  double to;
  linkCustomClass() {} //not used
  linkCustomClass(QSpinBox* Qbins, QLineEdit* Qfrom, QLineEdit* Qto) : Qbins(Qbins), Qfrom(Qfrom), Qto(Qto) {bins=100; from = 0; to = 0;}
  void updateBins();
  void updateRanges();
};

struct TreeViewStruct
{
    QString what;
    QString cuts;
    QString options;

    int Bins[3];
    double From[3];
    double To[3];
    bool fCustomBins, fCustomRanges;

    TreeViewStruct() {}
    TreeViewStruct(QString What, QString Cuts, QString Options) {what = What; cuts = Cuts; options = Options;}
    void fillCustom(bool fCustomBins, bool fCustomRanges, QVector<linkCustomClass> &cus);
};

class ReconstructionWindow : public QMainWindow
{
  Q_OBJECT
  
public:
  explicit ReconstructionWindow(QWidget *parent, MainWindow *mw, EventsDataClass *eventsDataHub);
  ~ReconstructionWindow();

  MainWindow *MW; //public to be visible by gain evaluator
  QVector<CorrelationFilterStructure*> CorrelationFilters;
  CorrelationFilterStructure tmpCorrFilter;

  QString LastDistributionShown;
  QIcon RedIcon;
  QIcon YellowIcon;

  QString CurrentScriptText;

  void CheckAvailableEventData();  //when new data is available or removed
  void UpdateStatusAllEvents();

  //config reconstruction module
  void InitWindow(); //configures properties defined in the interface

  void PMnumChanged();

  void ShowEnergySpectrum();  
  void ShowChi2Spectrum();

  void ResetStop(); //reset stop button
  void SetProgress(int val);

  void onBusyOn();
  void onBusyOff();

  void UpdateSimVizData(int iev);
  void UpdatePMgroupIndication();

  void showCorrelation(const char *title, const char *xTitle, const char *yTitle, QVector<double> *x, QVector<double> *y);

  void HideCutShapes(); //hide correlation cut shapes

  void ShowReconstructionPositions();
  void DotActualPositions();

  void SaveIndividualCutOffs(QString fileName);
  void LoadIndividualCutOffs(QString fileName);

  void ClearLineMarkers();

  void ShowGainWindow();
  void VisualizeScan(int iev);

  bool GetCenterShift(int ipm, bool fDraw = false);
  void EvaluateShiftBrute(int ipm, double *dx, double *dy);
  double getSuggestedZ() const;
  double isReconstructEnergy() const;


  void writeToJson(QJsonObject &json, bool fVerbose = false); //all settings
  void writeReconOptionsToJson(QJsonObject &jsonMaster, bool fVerbose = false);
  void writeFilterSettingsToJson(QJsonObject &jsonMaster);
  void writePMrelatedInfoToJson(QJsonObject &jsonMaster);

  bool readFromJson(QJsonObject &json); //all settings
  bool readReconSettingsFromJson(QJsonObject &jsonMaster);
  bool readFilterSettingsFromJson(QJsonObject &jsonMaster);
  bool readPMrelatedInfoFromJson(QJsonObject &jsonMaster);
  void writeMiscGUIsettingsToJson(QJsonObject &json);
  void readMiscGUIsettingsFromJson(QJsonObject &json);
  void updateFiltersGui();

public slots:
  void ShowStatistics();
  void LRF_ModuleReadySlot(bool ready);
  void onSelectionChange(QVector<int> selectionArray);

  //can be triggered by script
  void on_pbReconstructAll_clicked();
  void DoPlotXY(int i);
  void DoBlur(double sigma);  
  void DoCenterShift(int ipm);

  int getReconstructionAlgorithm();
  bool isCoGignoreLow();
  double getCoGignoreThreshold();
  void on_pbUpdateFilters_clicked();

private slots:
  void PaintPMs();

  void on_pbShowLRFwindow_clicked();

  void on_pbReconstructOneEvent_clicked();

  void on_pbReconstructTrack_clicked();

  void on_cobReconstructionAlgorithm_currentIndexChanged(int index);

  void on_pbRefreshDataIndication_clicked();

  void on_sbEventNumberInspect_valueChanged(int arg1);

  void on_sbPMsignalIndividual_valueChanged(int arg1);

  void on_pbCutOffsLoad_clicked();

  void on_pbCutOffsSave_clicked();

  void on_ledFilterIndividualMin_editingFinished();

  void on_ledFilterIndividualMax_editingFinished();

  void on_pbShowSumSignal_clicked();

  void on_pbShowIndividualSpectrum_clicked();

  void on_pbShowPMtable_clicked();  

  void on_pbGoToNextEvent_clicked();

  void on_pbShowReconstructionPositions_clicked();

  void on_pbShowStatistics_clicked();

  void on_pbUpdateStatusAllEvents_clicked();

  void on_pbShowMeasuredVsExpected_clicked();

  void on_pbAnalyzeResolution_clicked();

  void on_pbPointShowDistribution_clicked();

  void on_sbAnalysisEventNumber_valueChanged(int arg1);

  void on_pbShowSigmaVsArea_clicked();

  void on_pbSaveScanTree_clicked();

  void on_pbConfigureNN_clicked();

  void on_pbLoadLRFs_clicked();

  void on_pbSaveLRFs_clicked();

  void on_pbEnergySpectrum1_clicked();

  void on_pbEnergySpectrum2_clicked();

  void on_pbChi2Spectrum1_clicked();

  void on_pbChi2Spectrum2_clicked();

  void on_sbZshift_valueChanged(int arg1);

  void on_cobHowToAverageZ_currentIndexChanged(int index);

  void on_pbShowSpatialFilter_clicked();

  void on_pbSpF_UpdateTable_clicked();

  void on_pbSpF_AddLine_clicked();

  void on_pbSpF_RemoveLine_clicked();

  void on_tabwidSpF_cellChanged(int row, int column);

  void on_pbSpF_AddLineAfter_clicked();

  void on_pbGoToNextNoise_clicked();

  void on_pbGoToNextNoiseFoundGood_clicked();

  void on_pbStopReconstruction_toggled(bool checked);

  void on_pbCutOffApplyToAll_clicked();

  void on_cbRecType_currentIndexChanged(int index);

  void on_pbAddToActive_clicked();

  void on_pbAddToPassive_clicked();

  void on_pbShowPassivePMs_clicked();

  void on_pbAllPMsActive_clicked();

  void on_pbSaveReconstructionAsText_clicked();

  void on_pbSaveReconstructionAsRootTree_clicked();

  void on_pbMainWindow_clicked();

  void on_pbShowLog_clicked();

  void on_ledDynamicPassiveThreshold_editingFinished();

  void on_pbDoAutogroup_clicked();

  void on_pbUpdatePMgroupIndication_clicked();

  void on_pbUpdateGainsIndication_clicked();

  void on_leiPMnumberForGains_editingFinished();

  void on_ledGain_editingFinished();

  void on_pbShowAllGroups_clicked();

  void on_pbShowAllGainsForGroup_clicked();

  void on_pbGainsToUnity_clicked();

  void on_pbFindNextPMinGains_clicked();

  void on_pbSaveGains_clicked();

  void on_pbLoadGains_clicked();

  void on_twOptions_currentChanged(int index);

  void on_pbClearGroups_clicked();

  void on_pbAssignPMtoGroup_clicked();

  void on_pbDefineNewGroup_clicked();

  void on_pbShowVsXY_clicked();

  void on_cbPlotVsActualPosition_toggled(bool checked);

  void on_cb3Dreconstruction_toggled(bool checked);

  void on_pbShowCorr_clicked();

  void on_pbDefSumCutOffs_clicked();

  void on_pbDefSumCutOffs2_clicked();

  void on_pbDefCutOffMin_clicked();

  void on_pbDefCutOffMax_clicked();

  void on_pbCutOffsNextPM_clicked();

  void on_pbDefEnMin_clicked();

  void on_pbDefEnMax_clicked();

  void on_pbDefChi2Min_clicked();

  void on_pbDefChi2Max_clicked();

  void on_cbCorrAutoBin_toggled(bool checked);

  void on_cbCorrAutoSize_toggled(bool checked);

  void on_leCorrList1_editingFinished();

  void on_leCorrList2_editingFinished();

  void on_cobCorr1_currentIndexChanged(int index);

  void on_cobCorr2_currentIndexChanged(int index);

  void on_pbCorrUpdateTMP_clicked();

  void on_pbCorrExtractLine_clicked();

  void on_pbCorrRemove_clicked();

  void on_pbCorrNew_clicked();

  void on_pbCorrAccept_clicked();

  void on_cbCorrActive_toggled(bool checked);

  void on_sbCorrFNumber_valueChanged(int arg1);

  void on_pbCorrExtractEllipse_clicked();

  void on_cbActivateCorrelationFilters_toggled(bool checked);

  void on_pbDefLoadEnMin_clicked();

  void on_pbDefLoadEnMax_clicked();

  void on_pbLoadEnergySpectrum_clicked();

  void on_pbShowCorrShowRangeOnGraph_clicked();

  void on_pbCorr_AddLine_clicked();

  void on_ppCorr_AddLineAfter_clicked();

  void on_pbCorr_RemoveLine_clicked();

  void on_tabwidCorrPolygon_cellChanged(int row, int column);

  void on_pbCorrExtractPolygon_clicked();

  void on_pbEvaluateGains_clicked();

  void on_pbSetPresprocessingForGains_clicked();

  void on_pbPurgeEvents_clicked();

  void on_cbCoGapplyStretch_toggled(bool checked);

  void on_pbTreeView_clicked();

  void on_pbTreeViewHelpWhat_clicked();

  void on_pbTreeViewHelpCuts_clicked();

  void on_pbTreeViewHelpOptions_clicked();

  void on_sbTreeViewHistory_valueChanged(int arg1);

  void on_ledPassiveRange_editingFinished();

  void on_cobCUDAoffsetOption_currentIndexChanged(int index);

  void on_pbEvaluateZfromDistribution_clicked();

  void on_pbBlurReconstructedXY_clicked();

  void on_pbNNshowSpectrum_clicked();

  void on_pbDefNNMin_clicked();

  void on_pbDefNNMax_clicked();

  void on_pbShowMap_clicked();

  void on_cobMapZ_currentIndexChanged(int index);

  void on_pbTogglePassives_clicked();

  void on_pbGainsMultiply_clicked();

  void on_cobPMGroupForDynPassives_currentIndexChanged(const QString &arg1);

  void on_cobGroupInGains_currentIndexChanged(const QString &arg1);

  void on_cobShowWhat_currentIndexChanged(int index);

  void on_pbExportData_clicked();

  void on_pbCenterControlEvaluate_clicked();

  void on_pbMakePictureVsXY_clicked();

  void on_pbEvaluateAndShift_clicked();

  void on_pbShiftAll_clicked();

  void on_pbShiftAllWithDis_clicked();

  void on_pbEvaluateGrid_clicked();

  void on_pbEvAllBruteThenShift_clicked();

  void on_pbSensorGainsToPMgains_clicked();

  void on_cbCustomBinning_toggled(bool checked);

  void on_cbCustomRanges_toggled(bool checked);

  void on_cbForceCoGgiveZof_toggled(bool checked);

  void on_cobInRecShape_currentIndexChanged(int index);

  void on_pbCopyFromRec_clicked();

  void on_pbAnalyzeChanPerPhEl_clicked();

  void on_pbChanPerPhElShow_clicked();

  void on_sbChanPerPhElPM_valueChanged(int arg1);

  void on_pbExtractChansPerPhEl_clicked();

  void on_pbGotoPreprocessAdjust_clicked();

  void on_pbResetKNNfilter_clicked();

  void on_pbCalibrateX_clicked();

  void on_pbCalibrateY_clicked();

  void on_pbCalibrateXY_clicked();

  void on_cobKNNcalibration_activated(int index);

  void on_pbKNNupdate_clicked();

  void on_sbKNNuseNeighboursInRec_editingFinished();

  void on_cobSpFXY_currentIndexChanged(int index);

  void on_pbScriptFilter_clicked();

  void on_cbReconstructEnergy_toggled(bool checked);

  void on_cobZ_currentIndexChanged(int index);

  void on_cbLimitNumberEvents_toggled(bool checked);

  void on_sbXbins_editingFinished();

  void on_cbXYsymmetric_toggled(bool checked);

  void on_ledXto_editingFinished();

protected:
    bool event(QEvent *event);

private:
  Ui::ReconstructionWindow *ui;

  //Events data hub
  EventsDataClass* EventsDataHub;

  TH1D* tmp1Dhist;
  TH2D* tmpHist;
  QStandardItemModel *modelSpF_TV;
  TH2D* histCorr;
  TMultiGraph* multigrEE;

  QVector<TreeViewStruct> TreeViewHistory;
  QVector<TLine*> Markers;

  QList<TObject*> tmpTObjects;

  Viewer2DarrayObject* vo;
  QDialog* dialog;

  //bool ReconstructionOK;
  //int FilterCutOff; //all filters: 0 - not activated    1 - pass   2 - fail
  //int FilterSumCutOff;
  bool TableLocked;
  bool StopRecon; //stop button flag

  int ShiftZoldValue;
  bool ForbidUpdate;  
  bool TMPignore;

  double lastChi2;

  TH2D* centHist;
  Double_t funcParams[4];

  QPolygonF polygon; //polygon for custom spatial filter

  QVector<TH1D*> sigmaHists;
  QVector<double> ChPerPhEl;

  QString FilterScript;

  void DrawPolygon(double z);  
  void PolygonToTable();
  void TableToPolygon();

  void VisualizeEnergyVector(int eventId);
  void VisualizeScan();

  void AnalyzeScanNode();
  bool BuildResolutionTree();

  bool ShowVsXY(QString strIn);

  void MakeActivePassive(QString input, QString mode);

  void ReconstructTrack();

  bool startXextraction();
  bool start2DLineExtraction();
  bool start2DEllipseExtraction();
  bool start2DBoxExtraction();  
  bool start2DPolygonExtraction();
    //all use this:
  bool Extraction();

  void updateRedStatusOfRecOptions();

  void DrawCuts();
  TLine* CorrCutLine;
  TEllipse* CorrCutEllipse;
  TPolyLine* CorrCutPolygon;
  void FillUiFromTmpCorrFilter();

  void PrepareMarker(double x0, double y0, double len, Color_t color);
  void UpdatePMGroupCOBs();
  void UpdateOneGroupCOB(QComboBox *cob);
  int GetGroupNumber(QComboBox *cob);

  void ShowIndividualSpectrum(bool focus); //true - window is focussed  

  void ClearTmpTObjects();

  double extractChPerPHEl(int ipm);
public slots:
  void RefreshOnTimer(int Progress, double usPerEv); //update timer for reconstruction progress

  void onKNNreadyXchanged(bool ready, int events);
  void onKNNreadyYchanged(bool ready, int events);

signals:

  void cbReconstructEnergyChanged(bool changed);
  void cb3DreconstructionChanged(bool changed);
  void StopRequested(); //to be used to request stop

public:
  bool isReconstructEnergy();
  bool isReconstructZ();
  double getSuggestedEnergy();
  double getSuggestedZ();

  void ReconstructAll(bool fShow = true);

  //for script mode:
  void SetShowFirst(bool fOn, int number = -1);
  void SetShowActual(bool fOn, bool fActualOnTop = false);
};

#endif // RECONSTRUCTIONWINDOW_H
