#include "detectionstatisticsclass.h"

//Root
#include "TH1I.h"
#include "TH1D.h"

#include <QtGui>

DetectionStatisticsClass::DetectionStatisticsClass(const TString nameID)
{
    WaveSpectrum = 0;
    TimeSpectrum = 0;
    CosAngleSpectrum = 0;
    TransitionSpectrum = 0;

    NameID = nameID;
    initialize(101);
}

DetectionStatisticsClass::~DetectionStatisticsClass()
{
    if (WaveSpectrum) delete WaveSpectrum;
    if (TimeSpectrum) delete TimeSpectrum;
    if (CosAngleSpectrum) delete CosAngleSpectrum;
    if (TransitionSpectrum) delete TransitionSpectrum;
}

void DetectionStatisticsClass::initialize(int nBins)
{    
    if (nBins == 0) nBins = numBins;
    else numBins = nBins;

    if (WaveSpectrum) delete WaveSpectrum;
    WaveSpectrum = new TH1I("iWaveSpectrum"+NameID,"WaveIndex spectrum",nBins,0,-1);
    if (TimeSpectrum) delete TimeSpectrum;
    TimeSpectrum = new TH1D("TimeSpectrum"+NameID, "Time spectrum",nBins,0,-1);
    if (CosAngleSpectrum) delete CosAngleSpectrum;
    CosAngleSpectrum = new TH1I("CosAngleSpectrum"+NameID, "cosAngle spectrum", nBins, 0,-1);
    if (TransitionSpectrum) delete TransitionSpectrum;
    TransitionSpectrum = new TH1I("TransitionsSpectrum"+NameID, "Transitions", nBins, 0,-1);

    Absorbed = OverrideLoss = HitPM = HitDummy = Escaped = LossOnGrid = TracingSkipped = MaxCyclesReached = 0;
}

bool DetectionStatisticsClass::isEmpty()
{
  if (!TransitionSpectrum) return true;
  return (TransitionSpectrum->GetEntries() > 0) ? false : true;
}

void DetectionStatisticsClass::registerWave(int iWave)
{
    WaveSpectrum->Fill(iWave);
}

void DetectionStatisticsClass::registerTime(double Time)
{
    TimeSpectrum->Fill(Time);
}

void DetectionStatisticsClass::registerAngle(double CosAngle)
{
    CosAngleSpectrum->Fill(CosAngle);
}

void DetectionStatisticsClass::registerNumTrans(int NumTransitions)
{
  TransitionSpectrum->Fill(NumTransitions);
}

void DetectionStatisticsClass::AppendPhotonLossStat(const DetectionStatisticsClass* from)
{
  Absorbed += from->Absorbed;
  OverrideLoss += from->OverrideLoss;
  HitPM += from->HitPM;
  HitDummy += from->HitDummy;
  Escaped += from->Escaped;
  LossOnGrid += from->LossOnGrid;
  TracingSkipped += from->TracingSkipped;
  MaxCyclesReached += from->MaxCyclesReached;
}

