#ifndef DETECTORCLASS_H
#define DETECTORCLASS_H

#include "classes.h"
#include "detectorsandwichclass.h"


#include <QVector>

class TGeoManager;
class SensorLRFs;
class pms;
class MaterialCollectionClass;
class TGeoVolume;
class TRandom2;

struct PosAngTypeStructure
{
  double x,y,z,phi,theta,psi;
  int type;
  PosAngTypeStructure(){}
  PosAngTypeStructure(double x, double y, double z, double phi, double theta, double psi, int type)
    : x(x), y(y), z(z), phi(phi), theta(theta), psi(psi), type(type){}

  QJsonArray writeToJArrayAutoZ();
  void readFromJArrayAutoZ(QJsonArray &arr);
  QJsonArray writeToJArrayFull();
  void readFromJArrayFull(QJsonArray &arr);
};

class PMarrayDataClass
{
public:
  bool   fActive;
  int    Regularity;   //0-regular, 1-auto_z, 2-custom
  int    PMtype;
  int    Packing;
  double CenterToCenter;
  bool   fUseRings;
  int    NumX;
  int    NumY;
  int    NumRings;
 QString PositioningScript;

  QVector<PosAngTypeStructure> PositionsAnglesTypes; //stores PM data, which is used to build pmts - both in GeoManager and PMs module

  PMarrayDataClass() {fActive = false;Regularity=0;PMtype=0;Packing=0;CenterToCenter=80.0;fUseRings=false;NumX=2;NumY=2;NumRings=1;PositioningScript="";}
  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);
};

struct PMdummyStructure
{
  int PMtype; //index of the PM type
  int UpperLower; //Belongs to a PM plane: 0- upper, 1 - lower
  double r[3]; //PMs center coordinates
  double Angle[3]; //Phi, Theta, Psi;

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);
};

struct LightguidePropertiesStructure
{
  bool fActive;
  bool fAutoshape;
  int Shape;
  int Sides;
  double SizeX, SizeY;
  double Phi;
  double Thickness;
  int BulkMaterial;

  int OpeningsShape;
  double OpeningsSize;
  bool fCone;
  double ConeFactor;  
  int OpeningsMaterial;
  double Angle;

  LightguidePropertiesStructure(){fActive = false; fAutoshape = true; Shape=1; Sides=6; SizeX=100; SizeY=100; Phi=0; Thickness = 1.; BulkMaterial = 0;
                                  OpeningsShape = 1; OpeningsSize = 10.; fCone = false; ConeFactor = 1.0; OpeningsMaterial = 0; Angle = 0;}

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);  

  //temporary
  double AutoSizeX, AutoSizeY, Z;  //calculated and used during detector construction phase
};

class AdditionalGeoObjectClass
{
public:
  QString Name;
  bool fActivated; //false - not added to GeoManager
  QString Container;
  int Material;
  double R[3]; //position of the center (relative to Top)
  double Angles[3]; //orientation (relative to Top)
  int Shape;  //1 - box
  double Size[3];
  int Sides;
  double Vertices[8][2]; //data for TGeoArb8 class
  double Arb8height;

  void init(QString name);

  AdditionalGeoObjectClass() {init("undefined");}
  AdditionalGeoObjectClass(QString Name){init(Name);}
  AdditionalGeoObjectClass(QString Name, int shape, double Lx, double Ly, double Lz, int iMat, QString container, double x, double y, double z, double phi, double theta, double psi)
  {init(Name); Shape=shape; Size[0]=Lx;Size[1]=Ly;Size[2]=Lz;Material=iMat;Container=container;R[0]=x;R[1]=y;R[2]=z;Angles[0]=phi;Angles[1]=theta;Angles[2]=psi;}

  bool isOverlapping(const AdditionalGeoObjectClass* other);

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);
};

struct MaskElementStructure
{
  MaskElementStructure(){;}
  MaskElementStructure(double dx, double dy, double radius) : Type(1), DX(dx), DY(dy), Radius(radius){}
  MaskElementStructure(double dx, double dy, double sizex, double sizey, double angle = 0) : Type(0), DX(dx), DY(dy), SizeX(sizex), SizeY(sizey), Angle(angle){}

  int Type; //0-box, 1-tube
  double DX, DY; //offset from center
  double Radius;
  double SizeX, SizeY;
  double Angle; //for box, rotation angle

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);
};

class MaskClass
{
public:
  bool fActive;
  int Shape;
  double Size;
  double Thickness;
  QString Container;
  double OriginX;
  double OriginY;
  double OriginZ;
  double Phi;
  double Theta;
  double Psi;
  int MaterialBulk;
  int MaterialHoles;
  QVector<MaskElementStructure> MaskElements;     // holes (or bars) in the phantom
  QString MaskScript; //script for phantom holes

  MaskClass();

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);
};

class MaskCollectionClass
{
public:
  bool isEmpty() {return Masks.isEmpty();}
  int size() {return Masks.size();}
  MaskClass& operator[](int i) {return Masks[i];}
  void append(MaskClass& Mask);
  void remove(int index);

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);

private:
  QVector<MaskClass> Masks;
};

struct OpticalGridStructure
{
  bool fActive;     // on/off
  double Diameter;  // wire diameter (mm)
  double Pitch;     // step in wire positioning (mm)
  int Type;         // 0 - parallel wires, 1 - crossed wires
  double Abs;       // abs probability on the wire *** for now is fixed at 1
  QString Container;// grid is in this volume
  double Z;         // z position in local coordinates
  int Shape;        // 0 - rectangular, 1 - round, 3 - polygon
  int Sides;        // number of sides for polygon
  double SizeX;     // size X of the grid (also diameter and size for polygon)
  double SizeY;     // size Y of the grid (only for box)
  double Phi;       // rotation of the polygon around vertical axis
  double WireAngle; // orientation of the wire (local), if crossed type, second wire is +90degrees

  OpticalGridStructure(){}
  OpticalGridStructure(int Type, double WireAngle, double Diameter, double Pitch, double Abs)
    : Type(Type), WireAngle(WireAngle), Diameter(Diameter), Pitch(Pitch), Abs(Abs) {}
  void PositionGrid(QString container, double z, int shape, double sizeX, double sizeY, double phi, int sides = 6 )
    {Container = container;  Z = z; Shape = shape; SizeX = sizeX; SizeY = sizeY; Phi = phi; Sides = sides;}

  void writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);
};

//-----------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------

class DetectorClass
{
public:  
  TGeoManager* GeoManager;             // detector geometry, navigator and visualizator
  pms* PMs;                            // all info on PMs 
  SensorLRFs* SensLRF;                 // detector response  
  TGeoVolume* top;                     // world in GeoManager
  TRandom2* RandGen;                   // random generator

  QVector<PMarrayDataClass> PMarrays;  //upper(0) and lower(1) PM array data
  QVector<PMdummyStructure> PMdummies; // all info on PM dummies
  QVector<LightguidePropertiesStructure> Lightguides; //Lightguides
  QVector<OpticalGridStructure> Grids; // grids (affect only photon tracing)
  QVector<AdditionalGeoObjectClass> AdditionalGeoObjects; //additional object which detector geometry contains
  bool fAdditionalGeoObjectsEnabled;   // master switch to disable/enable all while keeping enable/disable status of individual objects
  QString AddObjPositioningScript;
  MaskCollectionClass Masks;
  bool fPrScintCont;
  DetectorSandwichClass Sandwich;

  MaterialCollectionClass* MaterialCollection;     // collections of defined materials
  QVector<ParticleDescriptor*> ParticleCollection; // collections of defined particles
  void clearParticleCollection();

  bool fSecScintPresent;

  QString ErrorString;

  QJsonObject LastConfig;

  DetectorClass();
  ~DetectorClass();

  bool makeDetector(QJsonObject &json);
  bool remakeLastDetector();

  bool makeSandwichDetector(QJsonObject &json);
  bool importGDML(QString gdml);
  bool isGDMLempty() const {return GDML.isEmpty();}
  void clearGDML() {GDML = "";}
  void checkSecScintPresent();
  void colorVolumes(int scheme, int id = 0);
  int pmCount() const; // Raimundo: so it's not necessary to #include<pms.h> just to clear LRFs
  void findPM(int ipm, int &ul, int &index);

  //write to Json - can be used from outside
  void writePMarraysToJson(QJsonObject &json);
  void writeDummyPMsToJson(QJsonObject &json);
  void writeLightguidesToJson(QJsonObject &json);
  void writeAddGeoObjectsToJson(QJsonObject &json);
  void writeGridsToJson(QJsonObject &json);
  void writeParticleCollectionToJson(QJsonObject &json);
  void writeGDMLtoJson(QJsonObject &json);

private:
  //reads
  bool readPMarraysFromJson(QJsonObject &json);
  bool readDummyPMsFromJson(QJsonObject &json);
  bool readLightguidesFromJson(QJsonObject &json);
  bool readAddGeoObjectsFromJson(QJsonObject &json);
  bool readGridsFromJson(QJsonObject &json);
  bool readParticleCollectionFromJson(QJsonObject &json);

  QString GDML;
  bool processGDML(); //check validity, discard if bad and return to sandwich  
};

#endif // DETECTORCLASS_H
