#ifndef DETECTORSANDWICHCLASS_H
#define DETECTORSANDWICHCLASS_H

#include <QVector>
#include <QJsonObject>
#include <QJsonArray>

#include "TMath.h"
#include "TString.h"

class DetectorClass;
class pms;
class TGeoMedium;
class TGeoVolume;
class PMarrayDataClass;

class BaseLayerClass
{
public:
  bool fActive;
  Double_t Height;
  Double_t Size1;
  Double_t Size2;
  int Material;
  int Shape;
  int Sides;
  Double_t Angle;
};

class SandwichLayerClass : public BaseLayerClass
{
public:
  TString Name;  
  TGeoMedium* Medium;  
  Double_t Z; //calculated by the DetectorSandwichClass

  void updateWorldSize(double &XYm);
};

class DetectorSandwichClass
{
public:
  DetectorSandwichClass(DetectorClass* detector);
  ~DetectorSandwichClass();

  DetectorClass* Detector;

  QVector<SandwichLayerClass> Layers;
  int ZeroLayer; //PrimaryScintillatorIndex
  int Shape; //0-box, 1-tube, 2-polygon
  Double_t Angle; //default angle
  int Sides; //default sides
  Double_t DefaultSizeX, DefaultSizeY;
  bool fUseDefaultSizes;
  Double_t UpperEdge, LowerEdge; //used to calculate Z positions of PMs
  int TopMaterial;
  QString ErrorString;
  double widthPMs, widthDummyPMs;
  double WorldSizeXY, WorldSizeZ; //size of Top volume
  bool fWorldSizeFixed;  //fixed and defined by GUI

  void readFromJson(QJsonObject &json);
  void constructDetector();
private:
  TGeoVolume *generateVolume(const char *Name, TGeoMedium *Medium, int Shape, Double_t SizeX, Double_t SizeY, Double_t SizeZ, int Sides);
  void calculateLayersZpositions();
  void populatePMs();
  void positionPMs();
  void calculatePmsXY(int ul);
  void positionDummies();
  void positionLightGuides();
  void positionAddObjects();
  void positionMasks();
  void positionGrids();

  void updateWorldSize(double &XYm, double &Zm);
};

#endif // DETECTORSANDWICHCLASS_H
