#ifndef DYNAMICPASSIVESHANDLER_H
#define DYNAMICPASSIVESHANDLER_H

#include <QVector>

class pms;
class EventsDataClass;
struct ReconstructionStruct;
class ReconstructionSettings;

class DynamicPassivesHandler
{
public:
    DynamicPassivesHandler(pms *Pms, EventsDataClass *eventsDataHub);
    void init(ReconstructionSettings *RecSet); //configure, clean passives, copy static passives from PMs
    void calculateDynamicPassives(int ievent, const ReconstructionStruct *rec); //RecSet.fuseDynamicPassives knows to do it or not

    inline bool isActive(int ipm)  const {return StatePMs[ipm] == 0;}
    inline bool isPassive(int ipm) const {return StatePMs[ipm] != 0;}
    inline bool isStaticActive(int ipm)  const {return !(StatePMs[ipm] & 1);}
    inline bool isStaticPassive(int ipm) const {return   StatePMs[ipm] & 1;}
    inline bool isDynamicActive(int ipm)  const {return !(StatePMs[ipm] & 2);}
    inline bool isDynamicPassive(int ipm) const {return   StatePMs[ipm] & 2;}

    int countActives() const;

    inline void setDynamicPassive(int ipm) {StatePMs[ipm] |= 2;}
    inline void clearDynamicPassive(int ipm) {StatePMs[ipm] &= ~(2);}
    void clearDynamicPassives();
private:
    pms* PMs;
    EventsDataClass* EventsDataHub;
    QVector<char> StatePMs;
    int numPMs;

    //int PassiveType; //0 - by threshold; 1 - by distance
    bool fByThreshold;
    bool fByDistance;

    int CenterOption; //for PassiveType by distance: 0 - CoG, 1 - maximum signal PM
    bool fLocalData; //if false, ask PMs module for threshold/max_distance
    double Threashold, MaxDistance2;

    //for gui
    bool fForceLocal;
    double Threshold;
    double MaxDistance;
};

#endif // DYNAMICPASSIVESHANDLER_H
