#ifndef ONEEVENTCLASS_H
#define ONEEVENTCLASS_H

#include "classes.h"

#include <QVector>
#include <QBitArray>

#include "Math/Random.h"   // rnd - to make dynamic and move to cpp
#include "Math/GSLRndmEngines.h"

class pms;
class TRandom2;
class DetectionStatisticsClass;
class GeneralSimSettings;

class OneEventClass
{
public:
  OneEventClass(pms* Pms, TRandom2* randGen, DetectionStatisticsClass* detStat);

  //Pm hits info
  QVector<QVector<int> > TimedPMhits;      // [time][pm]
  QVector<QVector<float> > TimedPMsignals; // convrted to signal  [timeBin][PMnumber]
  QVector<int> PMhitsTotal; // PM hits [pm]
  QVector<float> PMsignals; // converted to signal [pm]
  QVector< QBitArray > SiPMpixels; //[PM#] [time] [pixY] [pixX]

  DetectionStatisticsClass* DetStat;

  //configure
  void configure(const GeneralSimSettings *SimSet);

  //hits processing
  void clearHits();
  bool isHitsEmpty();
  void CheckPMThit(int ipm, double time, int WaveIndex, double x, double y, double cosAngle, int Transitions, double rnd);
  void CheckSiPMhit(int ipm, double time, int WaveIndex, double x, double y, double cosAngle, int Transitions, double rnd);
  void HitsToSignal();  //convert hits of PMs to signal using "electronics" settings
  void addHits(int ipm, int hits) {PMhitsTotal[ipm] += hits;}
  void addTimedHits(int itime, int ipm, int hits) {TimedPMhits[itime][ipm] += hits;}
  void CollectStatistics(int WaveIndex, double time, double cosAngle, int Transitions);
  int  TimeToBin(double time);

private:
  pms* PMs;
  TRandom2* RandGen;  
  ROOT::Math::Random<ROOT::Math::GSLRandomEngine> rnd; // to make dynamic and move to cpp

  //settings
  const GeneralSimSettings *SimSet;
  int numPMs;

  void registerSiPMhit(int ipm, int iTime, int binX, int binY);
  void AddDarkCounts();
};

#endif // ONEEVENTCLASS_H
