#include "photon_generator.h"
#include "materialcollectionclass.h"

#include <QtGui>
#include "TRandom2.h"
#include "TMath.h"

Photon_Generator::Photon_Generator(TRandom2* RandomGenerator, MaterialCollectionClass *materialCollection, QObject *parent) :
    QObject(parent)
{
   RandGen = RandomGenerator;
   MaterialCollection = materialCollection;
/*
   WavelengthResolved = false;
   TimeResolved = false;
   SecScintDirectionType = 0;
   SecScintThetaDistribution = 0;
*/
}

/*
void Photon_Generator::configure(bool fWave, bool fTime, double waveFrom, double waveStep, int secScintType, TH1D *secThetaDistr)
{
   WavelengthResolved = fWave;
   TimeResolved = fTime;
   WaveFrom = waveFrom;
   WaveStep = waveStep;
   SecScintDirectionType = secScintType;
   SecScintThetaDistribution = secThetaDistr;
}
*/

Photon_Generator::~Photon_Generator()
{
   //Photon_Generator::deleteSecScintThetaDistribution();
}

void Photon_Generator::GenerateDirectionPrimary(One_Photon* Photon)
{    
    //Sphere function of Root:
    double a=0,b=0,r2=1;
    while (r2 > 0.25) {
          a  = RandGen->Rndm() - 0.5;
          b  = RandGen->Rndm() - 0.5;
          r2 =  a*a + b*b;
       }
    Photon->v[2] = ( -1. + 8.0 * r2 );
    double scale = 8.0 * TMath::Sqrt(0.25 - r2);
    Photon->v[0] = a*scale;
    Photon->v[1] = b*scale;
}

void Photon_Generator::GenerateDirectionSecondary(One_Photon *Photon)
{
  /* OBSOLETE
  if (SimSet->SecScintGenMode == 3)
      {
        //custom direction     
        double theta = SimSet->SecScintThetaHist->GetRandom();
        double phi = RandGen->Rndm()*2.*3.1415926535;

        Photon->v[2] = cos(theta);
        double proj = sin(theta);
        Photon->v[1] = proj*cos(phi);
        Photon->v[0] = proj*sin(phi);

        return;
      }
  */

  //Sphere function of Root:
  double a=0,b=0,r2=1;
  while (r2 > 0.25) {
        a  = RandGen->Rndm() - 0.5;
        b  = RandGen->Rndm() - 0.5;
        r2 =  a*a + b*b;
     }
  Photon->v[2] = ( -1. + 8.0 * r2 );
  double scale = 8.0 * TMath::Sqrt(0.25 - r2);
  Photon->v[0] = a*scale;
  Photon->v[1] = b*scale;

  if (SimSet->SecScintGenMode == 0) return;
  if (SimSet->SecScintGenMode == 1)
    {
      if (Photon->v[2]<0) //Photon->v[2] = -Photon->v[2];
        Photon->SkipThisPhoton = true;
      return;
    }
  if (Photon->v[2]>0) //Photon->v[2] = -Photon->v[2];
    Photon->SkipThisPhoton = true;
}

void Photon_Generator::GenerateWaveTime(One_Photon* Photon, int materialId)
{
  MaterialStructure* Material = (*MaterialCollection)[materialId];
//  qDebug()<<"name:"<<Material->name;
//  qDebug()<<"WaveFrom:"<<SimSet->WaveFrom<<"Wave step:"<<SimSet->WaveStep;
//  qDebug()<<"time"<<Photon->time<<"prim decay time:"<<Material->PriScintDecayTime<<"sec decay time:"<<Material->SecScintDecayTime;
  if (Photon->scint_type == 1)
    {
//      qDebug()<<Photon->time;
      if (SimSet->fTimeResolved) Photon->time += RandGen->Exp(  Material->PriScintDecayTime );
//      qDebug()<<"-->"<<Photon->time;
      if (SimSet->fWaveResolved && Material->PrimarySpectrumHist)
         {
             Photon->wavelength = Material->PrimarySpectrumHist->GetRandom();
             Photon->WaveIndex = (Photon->wavelength - SimSet->WaveFrom)/SimSet->WaveStep;
//             qDebug()<<"prim! lambda "<<Photon->wavelength<<" index:"<<Photon->WaveIndex;
         }
      else Photon->WaveIndex= -1;
    }
  else
    {
      //secondary
      if (SimSet->fTimeResolved) Photon->time += RandGen->Exp(  Material->SecScintDecayTime );

      if (SimSet->fWaveResolved && Material->SecondarySpectrumHist)
        {
            Photon->wavelength = Material->SecondarySpectrumHist->GetRandom();
            Photon->WaveIndex = (Photon->wavelength - SimSet->WaveFrom)/SimSet->WaveStep;
//            qDebug()<<"sec! lambda "<<Photon->wavelength<<" index:"<<Photon->WaveIndex;
        }
      else Photon->WaveIndex= -1;
    }
//  qDebug()<<"Final time"<<Photon->time;
}



/*
void Photon_Generator::deleteSecScintThetaDistribution()
{
  if (!SecScintThetaDistribution) return;
  delete SecScintThetaDistribution;
  SecScintThetaDistribution = 0;
}
*/
