#ifndef PHOTON_TRACKER_H
#define PHOTON_TRACKER_H
//ANTS2
#include "classes.h"
#include "pmtypeclass.h"
#include "oneeventclass.h"
#include "detectorclass.h"
//Qt
#include <QObject>
//ROOT
#include "TMath.h"
#include "TRandom2.h"

struct One_Photon;
class TGeoManager;
struct MaterialStructure;
class pms;
class MaterialCollectionClass;
class OneEventClass;
class GeneralSimSettings;
class TGeoNavigator;

class Photon_Tracker : public QObject
{
    Q_OBJECT
public:
    explicit Photon_Tracker(TGeoManager* geoManager, TRandom2* RandomGenerator, MaterialCollectionClass* materialCollection, pms* Pms, const QVector<OpticalGridStructure>* Grids, QObject *parent = 0);
    void UpdateGeoManager(TGeoManager* NewGeoManager) {GeoManager = NewGeoManager;}//will be obsolete with new simulation system soon

    void TrackPhoton(One_Photon* Photon);   
    void configure(const GeneralSimSettings *simSet, OneEventClass* oneEvent, bool fBuildTracks, QVector<TrackHolderClass *> *tracks);
private:    
     //modules
     TRandom2* RandGen;
     TGeoManager* GeoManager;
     TGeoNavigator *navigator;
     MaterialCollectionClass* MaterialCollection;
     pms* PMs;
     QVector<PMtypeClass>* PMtypeProperties;
     const QVector<OpticalGridStructure>* grids;
     OneEventClass* OneEvent; //PM signals for this event are collected here
     QVector<TrackHolderClass*>* Tracks;

     double K[3];  //current direction vector
     Double_t* N; //normal vector to the surface
     bool flagHaveNormal;

     bool BuildTracks;
     const GeneralSimSettings* SimSet;

     bool Refraction(Double_t nn);
     void PerformReflection();
     void RandomDir();
};

#endif // PHOTON_TRACKER_H
