#ifndef PHSCATCLAUDIOMODEL_H
#define PHSCATCLAUDIOMODEL_H

#include "opticaloverrideclass.h"

//Qt
#include <QJsonObject>

//Root
#include "TVector3.h"
#include "TRandom2.h"

enum HeightDistrEnum {empirical, gaussian, exponential};
enum SlopeDistrEnum {trowbridgereitz, cooktorrance, bivariatecauchy};

class PhScatClaudioModel : public OpticalOverrideClass //abstract class!
{
public:  
  // constructor
  PhScatClaudioModel(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : OpticalOverrideClass(MatCollection, MatFrom, MatTo) {}
  // main method:
  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave) = 0;
    //unitary vectors! iWave - photon wave index, -1 if no wave-resolved

  virtual void printConfiguration(int iWave);
  virtual QString getType() const = 0;
  virtual QString getReportLine();

  // save/load config
  bool writeToJson(QJsonObject &json);
  bool readFromJson(QJsonObject &json);

  // interface properties
  double sigma_alpha;                     // r.m.s. (or similar) of the slope distribution
  double sigma_h;                         // r.m.s (or similar) of the height distribution
  double albedo;                          // multiple diffuse albedo
  HeightDistrEnum HeightDistribution;     // model for heights distribution
  SlopeDistrEnum SlopeDistribution;       // model for distribution of slopes

protected:
  double SpikeIntensity(int iWave, double costi);
  double Fresnel(double E1_perp, double E1_parl, double cosinc, double Rinda1, double Rinda2);

  virtual double GnFunc(double cost) = 0; // shadowing functions
  virtual double SlopeAngle(double random_num) = 0;

};

class PhScatClaudioModelV2 : public PhScatClaudioModel
{
public:
  PhScatClaudioModelV2(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : PhScatClaudioModel(MatCollection, MatFrom, MatTo) {}
  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave);
  virtual QString getType() const {return "Claudio_Model_V2";}
  virtual QString getReportLine();

protected:
  virtual double GnFunc(double cost);
  virtual double SlopeAngle(double random_num);
};

class PhScatClaudioModelV2d1 : public PhScatClaudioModelV2
{
public:
  PhScatClaudioModelV2d1(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : PhScatClaudioModelV2(MatCollection, MatFrom, MatTo) {}
  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave);
  virtual QString getType() const {return "Claudio_Model_V2d1";}
  virtual QString getReportLine();
};

class PhScatClaudioModelV2d2 : public PhScatClaudioModelV2
{
public:
  PhScatClaudioModelV2d2(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : PhScatClaudioModelV2(MatCollection, MatFrom, MatTo) {}
  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave);
  virtual QString getType() const {return "Claudio_Model_V2d2";}
  virtual QString getReportLine();
};

class PhScatClaudioModelV1 : public PhScatClaudioModel
{
public:
  PhScatClaudioModelV1(MaterialCollectionClass* MatCollection, int MatFrom, int MatTo)
    : PhScatClaudioModel(MatCollection, MatFrom, MatTo) {}
  virtual OpticalOverrideResultEnum calculate(TRandom2* RandGen, double* PhotonDir, const double* NormalVector, int iWave);
  virtual QString getType() const {return "Claudio_Model_V1";}
  virtual QString getReportLine();

protected:
  virtual double GnFunc(double cost);
  virtual double SlopeAngle(double random_num);
};


#endif // PHSCATCLAUDIOMODEL_H
