#ifndef PMS_H
#define PMS_H
//#include "classes.h"
#include "pmtypeclass.h"

#include <QVector>
#include <QString>
#include <QTextStream>

#include "Math/Random.h"
#include "Math/GSLRndmEngines.h"

class TRandom2;
struct MaterialStructure;
class TH1D;
class MaterialCollectionClass;
class GeneralSimSettings;
struct PosAngTypeStructure;

struct PMgroupStructure
{
   double PassiveThreshold;
   double PassiveRange;
   double PassiveRange2;
   QString PMgroupDescription;
};

struct pm
{
    explicit pm();
    pm(double xx, double yy, double zz, double Psi, int typ);

    void setCoords(const double *const xyz) { x = xyz[0]; y = xyz[1]; z = xyz[2]; }
    void saveCoords(QTextStream &file) const { file << x << " " << y << " " << z << "\r\n"; }
    void saveCoords2D(QTextStream &file) const { file << x << " " << y << "\r\n"; }
    void setAngles(const double *const phithepsi) { phi = phithepsi[0]; theta = phithepsi[1]; psi = phithepsi[2]; }
    void saveAngles(QTextStream &file) const { file << phi << " " << theta << " " << psi << "\r\n"; }

    inline bool isStaticPassive() const { return passive & 1; }
    //inline bool isDynamicPassive() const { return passive & 2; }
    inline void setStaticPassive() {passive = 1;}
    //inline void setPassiveOverThreshold() { passive |= 2;}
    //inline void clearPassiveOverThreshold() { passive &= ~(2);}
    inline bool isTMPPassive() const { return passive & 4; }
    inline void setTMPpassve() { passive |= 4;}
    inline void clearTMPpassive() { passive &= ~(4);}

    double x, y, z;
    double phi, theta, psi;
    double relGain; //reconstructed relative gain
    double relQE_PDE;
    double relElStrength;
    int type, group;
    unsigned char passive; //0 - active PM, any other value - passive PM
    int upperLower; //0- upper, 1 - lower
    double cutOffMin;  //for individual PM signal filter, used by Reconstructor
    double cutOffMax;
    double PreprocessingAdd;
    double PreprocessingMultiply;

};

class pms : public QObject
{
   Q_OBJECT
public:
    explicit pms(MaterialCollectionClass* materialCollection);
    ~pms();

    //json
    void writePMtypesToJson(QJsonObject &json);
    bool readPMtypesFromJson(QJsonObject &json);
    void writeInividualOverridesToJson(QJsonObject &json);
    bool readInividualOverridesFromJson(QJsonObject &json);
    void writeElectronicsToJson(QJsonObject &json);
    bool readElectronicsFromJson(QJsonObject &json);

    void configure(GeneralSimSettings* SimSet);
      //recalculate PDE for selected wave binning
    void RebinPDEs(); //triggered by changing between true and false for WavelenegthResolved on the MainWindow
    void RebinPDEsForType(int typ);
    void RebinPDEsForPM(int ipm);
      //recalculate angular for selected angle binning
    void RecalculateAngular();
    void RecalculateAngularForType(int typ);
    void RecalculateAngularForPM(int ipm);
      //prepare pulse height spectra hists
    void preparePHS(int ipm);
    void preparePHSs();

      // Single PM signal using photoelectron spectrum
    double GenerateSignalFromOnePhotoelectron(int ipm);
      // used during simulation:
    double getActualPDE(int ipm, int WaveIndex); //returns partial probability to be detected (wave-resolved PDE)
    double getActualAngularResponse(int ipm, double cosAngle); //returns partial probability to be detected (vs angular response - cos of refracted beam)
    double getActualAreaResponse(int ipm, double x, double y); //returns partial probability to be detected (vs area response - x, y in local coordinates of the PM)

    int count() const {return numPMs;} //returns number of PMs
    int countActives(); //returns number of active PMs
    int countStaticPassives(); //returns number of static passives
    void clear();
    void resetOverrides();
    void add(int upperlower, double xx, double yy, double zz, double Psi, int typ);
    void add(int upperlower, PosAngTypeStructure* pat);
    void insert(int ipm, int upperlower, double xx, double yy, double zz, double Psi, int typ);
    void remove(int ipm); //removes PM number i

    //New memory layout stuff
    bool saveCoords(const QString &filename);
    bool saveAngles(const QString &filename);
    bool saveTypes(const QString &filename);
    bool saveUpperLower(const QString &filename);
    inline pm &at(int ipm) { return PMs[ipm]; }

    //accelerator
    double getMaxQE() const {return MaxQE;}
    double getMaxQEvsWave(int iWave) const {return MaxQEvsWave.at(iWave);}

    //individual PMs
    inline double X(int ipm) const {return PMs[ipm].x;}
    inline double Y(int ipm) const {return PMs[ipm].y;}
    inline double Z(int ipm) const {return PMs[ipm].z;}
    bool isActive(int ipm) {return PMs[ipm].passive == 0;}
    bool isStaticPassive(int ipm) {return PMs[ipm].passive & 1; }
    double getGain(int ipm) {return PMs[ipm].relGain;}
    int getPixelsX(int ipm){return PMtypes[PMs[ipm].type]->PixelsX;}
    int getPixelsY(int ipm){return PMtypes[PMs[ipm].type]->PixelsY;}
    double SizeX(int ipm) const {return PMtypes[PMs[ipm].type]->SizeX;}
    double SizeY(int ipm) const {return PMtypes[PMs[ipm].type]->SizeY;}
    bool isSiPM(int ipm) {return PMtypes[PMs[ipm].type]->SiPM;}

    void setSeed(int seed);

    //PDE
    bool        isPDEwaveOverriden(int i) const {return (!PDE[i].isEmpty());}
    bool        isPDEwaveOverriden() const;
    bool        isPDEeffectiveOverriden() const;
    void        writePDEeffectiveToJson(QJsonObject &json);
    bool        readPDEeffectiveFromJson(QJsonObject &json);
    void        writePDEwaveToJson(QJsonObject &json);
    bool        readPDEwaveFromJson(QJsonObject &json);
    void        setPDEeffective(int ipm, double val) {effectivePDE[ipm] = val;}
    double      getPDEeffective(int ipm) {return effectivePDE[ipm];}
    void        setAllEffectivePDE(QVector<double>* vec) {effectivePDE = *vec;} //will be gone sone
    void        setPDEwave(int ipm, QVector<double>* x, QVector<double>* y) {PDE_lambda[ipm] = *x; PDE[ipm] = *y;}
    void        setPDEbinned(int ipm, QVector<double>* vec) {PDEbinned[ipm] = *vec;}
    const QVector<double>* getPDE(int ipm) {return &PDE[ipm];}
    const QVector<double>* getPDE_lambda(int ipm) {return &PDE_lambda[ipm];}
    const QVector<double>* getPDEbinned(int ipm) {return &PDEbinned[ipm];}
    const QVector<double>* getAllEffectivePDE() {return &effectivePDE;}

    //Angular response
    bool        isAngularOverriden(int i) const {return !AngularSensitivity[i].isEmpty();}
    bool        isAngularOverriden() const;
    void        writeAngularToJson(QJsonObject &json);
    bool        readAngularFromJson(QJsonObject &json);
    void        setAngular(int ipm, QVector<double>* x, QVector<double>* y) {AngularSensitivity_lambda[ipm] = *x; AngularSensitivity[ipm] = *y;}
    void        setAngularN1(int ipm, double n1) {AngularN1[ipm] = n1;}
    double      getAngularN1(int ipm) {return AngularN1[ipm];}
    void        setAngularBinned(int ipm, QVector<double>* vec) {AngularSensitivityCosRefracted[ipm] = *vec;}
    const QVector<double>* getAngularSensitivity(int ipm) {return &AngularSensitivity[ipm];}
    const QVector<double>* getAngularSensitivity_lambda(int ipm) {return &AngularSensitivity_lambda[ipm];}
    const QVector<double>* getAngularSensitivityCosRefracted(int ipm) {return &AngularSensitivityCosRefracted[ipm];}

    //Area response
    bool        isAreaOverriden(int i) const {return !AreaSensitivity[i].isEmpty();}
    bool        isAreaOverriden() const;
    void        writeAreaToJson(QJsonObject &json);
    bool        readAreaFromJson(QJsonObject &json);
    void        setArea(int ipm, QVector<QVector <double> > *vec, double xStep, double yStep) {AreaSensitivity[ipm] = *vec; AreaStepX[ipm] = xStep; AreaStepY[ipm] = yStep;}
    double      getAreaStepX(int ipm) {return AreaStepX[ipm];}
    double      getAreaStepY(int ipm) {return AreaStepY[ipm];}
    const QVector<QVector<double> >* getAreaSensitivity(int ipm) {return &AreaSensitivity[ipm];}

    //type properties    
    PMtypeClass* getType(int i) {return PMtypes.at(i);}
    PMtypeClass* getTypeForPM(int ipm) {return PMtypes.at(PMs[ipm].type);}
    bool removePMtype(int itype);
    void appendNewPMtype(PMtypeClass* tp);
    int countPMtypes(){return PMtypes.size();}
    void clearPMtypes();
    void replaceType(int itype, PMtypeClass *newType) {delete PMtypes[itype]; PMtypes[itype] = newType;}
    void updateTypePDE(int typ, QVector<double> *x, QVector<double> *y) {PMtypes[typ]->PDE_lambda = *x; PMtypes[typ]->PDE = *y;}
    void scaleTypePDE(int typ, double factor);
    void updateTypeAngular(int typ, QVector<double> *x, QVector<double> *y) {PMtypes[typ]->AngularSensitivity_lambda = *x; PMtypes[typ]->AngularSensitivity = *y;}
    void updateTypeAngularN1(int typ, double val) {PMtypes[typ]->n1 = val;}
    void updateTypeArea(int typ, QVector<QVector <double> > *vec, double xStep, double yStep){PMtypes[typ]->AreaSensitivity = *vec; PMtypes[typ]->AreaStepX = xStep; PMtypes[typ]->AreaStepY = yStep;}
    void clearTypeArea(int typ){PMtypes[typ]->AreaSensitivity.resize(0);}


    void setAllPMampGains(QVector<double>* vec) {AverageSignalPerPhotoelectron = *vec;}
    void clearSPePHS(int ipm);
    void setAverageSignalPerPhotoelectron(int ipm, double val) {AverageSignalPerPhotoelectron[ipm] = val;}
    void setSPePHSmode(int ipm, int mode);
    void setSPePHSsigma(int ipm, double sigma) {SPePHSsigma[ipm] = sigma;}
    void setSPePHSshape(int ipm, double shape) {SPePHSshape[ipm] = shape;}
    void setElChanSPePHS(int ipm, QVector<double> *x, QVector<double> *y);
    void setElNoiseSigma(int ipm, double val) {ElNoiseSigma[ipm] = val;}
    void setADC(int ipm, double max, int bits);

    void CopySPePHSdata(int ipmFrom, int ipmTo);
    void CopyElNoiseData(int ipmFrom, int ipmTo);
    void CopyADCdata(int ipmFrom, int ipmTo);


    const QVector<double>* getSPePHS_x(int ipm) {return &SPePHS_x[ipm];}
    const QVector<double>* getSPePHS(int ipm) {return &SPePHS[ipm];}
    double getAverageSignalPerPhotoelectron(int ipm) {return AverageSignalPerPhotoelectron[ipm];}
    int getSPePHSmode(int ipm) {return SPePHSmode[ipm];}
    double getSPePHSsigma(int ipm) {return SPePHSsigma[ipm];}
    double getSPePHSshape(int ipm) {return SPePHSshape[ipm];}
    TH1D* getSPePHShist(int ipm) {return SPePHShist[ipm];}
    double getElNoiseSigma(int ipm) {return ElNoiseSigma[ipm];}
    const QVector<double>* getAllElNoiseSigma() {return &ElNoiseSigma;}
    double getADCmax(int ipm) {return ADCmax[ipm];}
    const QVector<double>* getAllADCmax() {return &ADCmax;}
    int getADCbits(int ipm) {return ADCbits[ipm];}
    const QVector<double>* getAllADCbits() {return &ADCbits;}
    int getADClevels(int ipm) {return ADClevels[ipm];}
    double getADCstep(int ipm) {return ADCstep[ipm];}

    //config
    void SetWave(bool wavelengthResolved, double waveFrom, double waveStep, int waveNodes);
    void setWavelengthResolved(bool flag) {WavelengthResolved = flag;}
    void setAngularResolved(bool flag) {AngularResolved = flag;}
    void setAreaResolved(bool flag){AreaResolved = flag;}
    void setWaveFrom(double from) {WaveFrom = from;}
    void setWaveStep(double step) {WaveStep = step;}
    void setWaveNodes(int nodes) {WaveNodes = nodes;}
    void setCosBins(int bins) {CosBins = bins;}
    void setDoPHS(bool flag) {fDoPHS = flag;}
    void setDoElNoise(bool flag) {fDoElNoise = flag;}
    void setDoADC(bool flag) {fDoADC = flag;}
    void setMeasurementTime(double time) {MeasurementTime = time;}

    bool isDoPHS() {return fDoPHS;}
    bool isDoElNoise() {return fDoElNoise;}
    bool isDoADC() {return fDoADC;}
    double getMeasurementTime() {return MeasurementTime;}

        //pmgroups
    int countPMgroups() {return PMgroupsVector.size();}
    QString getGroupDescription(int iGroup) {return PMgroupsVector[iGroup].PMgroupDescription;}
    int getGroupByDescription(QString desc);
    int definePMgroup(QString desc);
    void clearPMgroups();
    void setPassiveThreshold(int igroup, double val) {PMgroupsVector[igroup].PassiveThreshold = val;}
    double getPassiveThreshold(int igroup) {return PMgroupsVector[igroup].PassiveThreshold;}
    double getPassiveThresholdForPM(int ipm) {return PMgroupsVector[ PMs[ipm].group ].PassiveThreshold * PMs[ipm].relGain;}
    void setPassiveRange(int igroup, double val) {PMgroupsVector[igroup].PassiveRange = val; PMgroupsVector[igroup].PassiveRange2 = val*val;}
    double getPassiveRange(int igroup) {return PMgroupsVector[igroup].PassiveRange;}
    double getPassiveRange2(int igroup) {return PMgroupsVector[igroup].PassiveRange2;}
    double getPassiveRangeForPM(int ipm) {return PMgroupsVector[ PMs[ipm].group ].PassiveRange;}
    double getPassiveRange2ForPM(int ipm) {return PMgroupsVector[ PMs[ipm].group ].PassiveRange2;}
    bool isPMgroup0empty();
    QVector<QPair<double, int> > getPMsSortedByR() const;

public slots:
    void SetPmPassiveState(int ipm, bool state);


private:
    TRandom2* RandGen;
    MaterialCollectionClass* MaterialCollection;
    ROOT::Math::Random<ROOT::Math::GSLRandomEngine> rnd;

    int numPMs; //number of PMs

    //  ----====-----
    //when NEW vector properties are added to PMs -> "clear" "insert" and "remove" methodes have to be updated!
    //

    //individual PM properties
    QVector<pm> PMs;
    QVector<double> tmpGains;

    double MaxQE; //accelerator, not wave-resolved value
    QVector<double> MaxQEvsWave; //vs wavelength
    void calculateMaxQEs(); // first vector has to be rebinned according to acxtual wave properties (see configure())

    //overrides over base PM class:
    QVector<QVector<double> > PDE; //Quantun efficiency & Collection efficiency for PMTs; Photon Detection Efficiency for SiPMs
    QVector<QVector<double> > PDE_lambda;
    QVector<QVector<double> > PDEbinned;
    QVector<double> effectivePDE;
    QVector<QVector<double> > AngularSensitivity;
    QVector<QVector<double> > AngularSensitivity_lambda;
    QVector<double> AngularN1; //refractive index of the medium where PM was positioned to measure the angular response
    QVector<QVector<double> > AngularSensitivityCosRefracted; //Response vs cos of refracted beam, binned from 0 to 1 in CosBins bins
    QVector<QVector< QVector <double> > > AreaSensitivity;
    QVector<double> AreaStepX;
    QVector<double> AreaStepY;

    QVector<int> SPePHSmode; //0 - use average value; 1 - normal distr; 2 - Gamma distr; 3 - custom distribution
    QVector<double> AverageSignalPerPhotoelectron;
    QVector<double> SPePHSsigma;
    QVector<double> SPePHSshape; //For Gamma
    QVector<QVector<double> > SPePHS_x; //custom distribution
    QVector<QVector<double> > SPePHS;   //custom distribution
    QVector<TH1D* > SPePHShist;

    QVector<double> ElNoiseSigma;

    double MeasurementTime;  // measurement time to calculate dark counts for SiPMs

    QVector<double> ADCmax;
    QVector<double> ADCbits;
    QVector<double> ADCstep;
    QVector<double> ADClevels;


    //int PMgroups;
    //QVector<QString> PMgroupDescription;
    QVector<PMgroupStructure> PMgroupsVector;

    //flags for the current simulation mode
    bool WavelengthResolved;
    bool AngularResolved;
    bool AreaResolved;
    bool fDoPHS;
    bool fDoElNoise;
    bool fDoADC;
    //wave info
    double WaveFrom, WaveStep;
    int WaveNodes;
    //number of bins for angular response
    int CosBins;

    //defined base PM types
    QVector<PMtypeClass*> PMtypes;

    QString ErrorString;
    void writePHSsettingsToJson(int ipm, QJsonObject &json);
    bool readPHSsettingsFromJson(int ipm, QJsonObject &json);
    void writeRelQE_PDE(QJsonObject &json);
    void readRelQE_PDE(QJsonObject &json);
};

#endif // PMS_H
