#ifndef PRIMARYPARTICLETRACKER_H
#define PRIMARYPARTICLETRACKER_H

#include "classes.h"

#include <QObject>
#include <QVector>

#include "TH1D.h"

class TGeoManager;
class TRandom2;
class ParticleDescriptor;
struct MaterialStructure;
class ParticleOnStack;
struct EnergyDepositionCell;
struct EventHistoryStructure;
class MaterialCollectionClass;
class TVirtualGeoTrack;
class GeneralSimSettings;

class PrimaryParticleTracker : public QObject
{
    Q_OBJECT
public:
    explicit PrimaryParticleTracker(TGeoManager* geoManager,
                                    TRandom2* RandomGenerator,
                                    const QVector<ParticleDescriptor*>* particleCollection,
                                    MaterialCollectionClass* materialCollection,
                                    QVector<ParticleOnStack*>* particleStack,
                                    QVector<EnergyDepositionCell*>* energyVector,
                                    QVector<EventHistoryStructure*>* eventHistory,
                                    QObject *parent = 0);
    //main usage
    bool TrackParticlesInStack(int eventId = 0);

    //configure
    void setBuildTracks(bool opt){ BuildTracks = opt;}
    void setRemoveTracksIfNoEnergyDepo(bool opt){ RemoveTracksIfNoEnergyDepo = opt;}
      //configure all
    void configure(const GeneralSimSettings *simSet,                 
                   bool fbuildTrackes,
                   QVector<TrackHolderClass*> *tracks,
                   bool fremoveEmptyTracks = true);


    void UpdateGeoManager(TGeoManager* NewGeoManager) {GeoManager = NewGeoManager;} //will be absolete
    void resetCounter(){counter = -1;}  //will be absolute
    void setAddColorIndex(int val) {AddColorIndex = val;}

signals:
    
public slots:

private:   
    TGeoManager* GeoManager;
    TRandom2* RandGen;
    const QVector<ParticleDescriptor*>* ParticleCollection;
    MaterialCollectionClass* MaterialCollection;
    QVector<ParticleOnStack*>* ParticleStack;  
    QVector<EnergyDepositionCell*>* EnergyVector;
    QVector<EventHistoryStructure*>* EventHistory;

//    int InterpolationDegree;
//    bool LogLogInterpolation;

    const GeneralSimSettings* SimSet;
    /*
    double AbsMinStep;
    double AbsMaxStep;
    double RecFraction;
    double Safety;
    double NegligibleEnergy;
    */
    bool BuildTracks;
    bool RemoveTracksIfNoEnergyDepo;
    QVector<TrackHolderClass*> TrackCandidates;
    QVector<TrackHolderClass*> *Tracks;

    int Id;
    double energy, time;
    double r[3]; //staring coordinates
    double v[3]; //staring cosines
    double rCur[3]; //current position

    int counter; //particle number -can be reset from outside

    int AddColorIndex;
};

#endif // PRIMARYPARTICLETRACKER_H
