#ifndef RECONSTRUCTIONMANAGERCLASS_H
#define RECONSTRUCTIONMANAGERCLASS_H

/////////////////////////////////////
// Performs reconstruction: CoG, MG and RootMini - multithread; CUDA and ANN - single thread;
// CoG - preparation phase for all algorithms (multithread)
// Then recons with selected algorithm
// Chi2 calculation (multithread)
// Basic filtering  (multithread)
// Advanced filters: correlation, PrimScint, kNN filters - single thread!
/////////////////////////////////////

#include <QObject>
#include "reconstructionsettings.h"
#include <QPolygonF>

#include "Math/Functor.h"

class EventsDataClass;
class DetectorClass;
struct CorrelationFilterStructure;
class DynamicPassivesHandler;
class ProcessorClass;
class NNmoduleClass;
#ifdef ANTS_FANN
class NeuralNetworksModule;
#endif

class ReconstructionManagerClass : public QObject
{
  Q_OBJECT
  friend class ProcessorClass;
  friend class CoGReconstructorClass;
  friend class CGonCPUreconstructorClass;
  friend class RootMinReconstructorClass;
  friend class RootMinDoubleReconstructorClass;
  friend class Chi2calculatorClass;
  friend class EventFilterClass;

public:
  ReconstructionManagerClass(EventsDataClass *eventsDataHub, DetectorClass *detector);
  ~ReconstructionManagerClass();

  bool reconstructAll(QJsonObject &json);
  void filterEvents(QJsonObject &jsonFilters);

  QString getErrorString() {return ErrorString;}
  double getUsPerEvent() {return usPerEvent;}

  //kNN module, can be directly accessed from outside
#ifdef ANTS_FLANN
  NNmoduleClass* KNNmodule;
#endif
  //Artificail neural network module, can be accessed directly
#ifdef ANTS_FANN
  NeuralNetworksModule *ANNmodule;
#endif

private:
  DetectorClass *Detector;
  EventsDataClass *EventsDataHub;  
  ReconstructionSettings RecSet;

  bool run(QList<ProcessorClass*> reconstructorList);

  //filters - threaded
    //event number filter
  bool fEventNumberFilter;
  int EventNumberFilterMin, EventNumberFilterMax;
    //sum cutoff
  bool fSumCutOffFilter;
  bool fSumCutUsesGains;
  bool fCutOffsForPassivePMs;
  double SumCutOffMin, SumCutOffMax;
    //individual cut-off filters
  bool fCutOffFilter;
    //rec energy filter
  bool fEnergyFilter;
  double EnergyFilterMin, EnergyFilterMax;
    //loaded energy filter
  bool LoadedEnergyFilter;
  double LoadedEnergyFilterMin, LoadedEnergyFilterMax;
    //chi2 filter
  bool fChi2Filter;
  double Chi2FilterMin, Chi2FilterMax;
    //spatial: primary
  bool fSpF_PrScint;
    //spatial: custom
  bool fSpF_custom;
  int SpF_RecOrScan;
  int SpF_shape;
  double SpF_SizeX, SpF_SizeY;
  double SpF_Diameter;
  double SpF_Side, SpF_Angle;
  double SpF_X0, SpF_Y0;
  int SpF_CutOutsideInside;
  bool fSpF_allZ;
  double SpF_Zfrom, SpF_Zto;
  double SpF_halfSizeX, SpF_halfSizeY, SpF_radius2; // calculated in this module
  QPolygonF SpF_polygon; // calculated in this module

  //filters - after thread data merge
    //correlationFilter
  bool fCorrelationFilters;
  QVector <CorrelationFilterStructure* > CorrelationFilters; //link to filters configured at ReconstructionWindow
    //kNN filter
  bool fKNNfilter;
  double KNNfilterMin, KNNfilterMax;
  int KNNfilterAverageOver;

  //misc
  QString ErrorString;
  bool fStopRequested;
  int numEvents;
  double usPerEvent;

  bool fillSettingsAndVerify(QJsonObject &json);
  bool configureFilters(QJsonObject &json);
  void distributeWork(int Algorithm, QList<ProcessorClass*> &todo);

  void singleThreadEventFilters(); //i house use to process PrimScint spatioal, correlation and kNN filters which HAS to be one thread
public slots:
  void requestStop() {fStopRequested = true;}

signals:
  void UpdateReady(int Progress, double MsPerEv);
};

#endif // RECONSTRUCTIONMANAGERCLASS_H
