#include "s1_generator.h"
#include "materialcollectionclass.h"

#include <QtGui>

#include "photon_generator.h"
#include "photon_tracker.h"

S1_Generator::S1_Generator(Photon_Generator* photonGenerator, Photon_Tracker* photonTracker, MaterialCollectionClass* materialCollection, QVector<EnergyDepositionCell*>* energyVector, QVector<GeneratedPhotonsHistoryStructure>* PhotonsHistory, QObject *parent) :
    QObject(parent)
{       
    PhotonGenerator = photonGenerator;
    PhotonTracker = photonTracker;
    EnergyVector = energyVector;
    MaterialCollection = materialCollection;
    GeneratedPhotonsHistory = PhotonsHistory;

    DoTextLog = false;
}

bool S1_Generator::Generate() //uses MW->EnergyVector as the input parameter
{
    if (EnergyVector->isEmpty()) return true; //no data

    //what is the first particle to work with?
 //   int ThisId = EnergyVector->at(0)->ParticleId;
    int MatId = EnergyVector->at(0)->MaterialId;
   // double time = EnergyVector->t;
    double PhotonYield;

    double Remainer = 0.0;
    double Photons;
    int NumPhotons;

    //text log-related inits
    int TextLogPhotons = 0;
    double TextLogEnergy = 0.0;
    int LastEvent = EnergyVector->at(0)->eventId;
    int LastIndex = EnergyVector->at(0)->index;
    int LastParticle = EnergyVector->at(0)->ParticleId;
    int LastMaterial = EnergyVector->at(0)->MaterialId;

    for (int i = 0; i < EnergyVector->size(); i++)
      {
        //checking are we still tracking the same particle?
        int ThisId = EnergyVector->at(i)->ParticleId;
        int ThisEvent = EnergyVector->at(i)->eventId;
        int ThisIndex = EnergyVector->at(i)->index;

        if (LastEvent != ThisEvent || LastIndex != ThisIndex)
          {
            //Id = ThisId;
            //reset remainer
            Remainer = 0;            
          }

        MatId = EnergyVector->at(i)->MaterialId;
        PhotonYield = (*MaterialCollection)[MatId]->MatParticle[ThisId].PhYield;

        Photons = EnergyVector->at(i)->dE * PhotonYield + Remainer;
        NumPhotons = (int)Photons;
        Remainer   = Photons - (double)NumPhotons;

        //generate photons
//        qDebug()<<"Generate photons: "<<NumPhotons ;

        One_Photon Photon;
        Photon.r[0] = (*EnergyVector)[i]->r[0];
        Photon.r[1] = (*EnergyVector)[i]->r[1];
        Photon.r[2] = (*EnergyVector)[i]->r[2];
        Photon.scint_type = 1;
//        Photon.time = (*EnergyVector)[i]->time;
//        qDebug() << "Photons:"<<NumPhotons;
        for (int ii=0; ii<NumPhotons; ii++)
          {
            Photon.time = (*EnergyVector)[i]->time;  //will be changed if there is time dependence
            PhotonGenerator->GenerateDirectionPrimary(&Photon);
//            qDebug() << MatId;
            PhotonGenerator->GenerateWaveTime(&Photon, MatId);
//            qDebug() << "r, v, wave"<<Photon.r[0]<<Photon.r[1]<<Photon.r[2]<<Photon.v[0]<<Photon.v[1]<<Photon.v[2]<<Photon.WaveIndex;
            PhotonTracker->TrackPhoton(&Photon);
          }
//        qDebug() << "All photons tracked";
        //Text log
        if (DoTextLog)
          {            
            if (LastEvent != ThisEvent || LastIndex != ThisIndex || MatId != LastMaterial)
              {
                //changed - saving previous data
                GeneratedPhotonsHistoryStructure tmp;// = {};
                tmp.Energy = TextLogEnergy;
                tmp.MaterialId = LastMaterial;
                tmp.ParticleId = LastParticle;
                tmp.PrimaryPhotons = TextLogPhotons;
                tmp.SecondaryPhotons = 0;
                tmp.event = LastEvent;
                tmp.index = LastIndex;
                GeneratedPhotonsHistory->append(tmp);

                //updating current data
                LastEvent = ThisEvent;
                LastIndex = ThisIndex;
                LastMaterial = MatId;
                LastParticle = ThisId;
                TextLogPhotons = NumPhotons;
                TextLogEnergy = EnergyVector->at(i)->dE;
              }
            else
              {
                //continue previous
                TextLogPhotons += NumPhotons;
                TextLogEnergy += EnergyVector->at(i)->dE;
              }
          }    
      }

    if (DoTextLog)
      { //last point save
        GeneratedPhotonsHistoryStructure tmp;// = {};
        tmp.Energy = TextLogEnergy;
        tmp.MaterialId = LastMaterial;
        tmp.ParticleId = LastParticle;
        tmp.PrimaryPhotons = TextLogPhotons;
        tmp.SecondaryPhotons = 0;
        tmp.event = LastEvent;
        tmp.index = LastIndex;
        GeneratedPhotonsHistory->append(tmp);
        //qDebug() << "History:----"<<tmp.PrimaryPhotons;
      }

     return true;
}
