#include "scriptfilterclass.h"

InterfaceToScriptFilter::InterfaceToScriptFilter(EventsDataClass *EventsDataHub, pms *PMs)
  : EventsDataHub(EventsDataHub), PMs(PMs)
{
  H["setGood"] = "Set event # i as passed the filter";
  H["setFail"] = "Set event # i as failed the filter";
  H["eventId"] = "Returns event index (index !=i after purges)";
  H["x"] = "Returns reconstructed x position";
  H["y"] = "Returns reconstructed y position";
  H["z"] = "Returns reconstructed z position";
  H["rho"] = "Returns distance from reconstructed xy position to the center of PM number i";
  H["energy"] = "Returns reconstructed energy";
  H["chi2"] = "Returns reconstructed chi2";
  H["chi2"] = "Returns reconstructed chi2";
  H["good"] = "Returns 'good' status";
  H["recOK"] = "Returns reconstruction success 'recOK' status";
  H["points"] = "Returns number of points reconstructed for this event";
  H["xCoG"] = "Returns x recontructed by CoG";
  H["yCoG"] = "Returns y recontructed by CoG";
  H["zCoG"] = "Returns z recontructed by CoG";
  H["maxPM"] = "Returns index of PM which has the strongest signal (takes into account gains)";
  H["signal"] = "Returns signal in PM number ipm";
  H["xScan"] = "Returns scan/simulated x position of event i";
  H["yScan"] = "Returns scan/simulated y position of event i";
  H["zScan"] = "Returns scan/simulated z position of event i";
  H["eScan"] = "Returns scan/simulated energy of event i";
  H["pmX"] = "Returns X position of PM number ipm";
  H["pmY"] = "Returns Y position of PM number ipm";
  H["pmZ"] = "Returns Z position of PM number ipm";
  H["countEvents"] = "Returns the number of events in the current dataset";
  H["countGoodEvents"] = "Returns the number of events with 'good' status";
  H["countPMs"] = "Returns the number of PMs in this detector";
  H["isScanEmpty"] = "Returns true if scan/simulated true positions are available";
}

double InterfaceToScriptFilter::points(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->Points.size();
}

double InterfaceToScriptFilter::energy(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->Points[0].energy;
}

double InterfaceToScriptFilter::chi2(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->chi2;
}

double InterfaceToScriptFilter::x(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->Points[0].r[0];
}

double InterfaceToScriptFilter::y(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->Points[0].r[1];
}

double InterfaceToScriptFilter::z(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->Points[0].r[2];
}

double InterfaceToScriptFilter::xScan(int i)
{
  if (i<0 || i > EventsDataHub->Scan.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->Scan.at(i)->Points[0].r[0];
}

double InterfaceToScriptFilter::yScan(int i)
{
  if (i<0 || i > EventsDataHub->Scan.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->Scan.at(i)->Points[0].r[1];
}

double InterfaceToScriptFilter::zScan(int i)
{
  if (i<0 || i > EventsDataHub->Scan.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->Scan.at(i)->Points[0].r[2];
}

double InterfaceToScriptFilter::eScan(int i)
{
  if (i<0 || i > EventsDataHub->Scan.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->Scan.at(i)->Points[0].energy;
}

bool InterfaceToScriptFilter::good(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->GoodEvent;
}

bool InterfaceToScriptFilter::recOK(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->ReconstructionOK;
}

double InterfaceToScriptFilter::xCoG(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->xCoG;
}

double InterfaceToScriptFilter::yCoG(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->yCoG;
}

double InterfaceToScriptFilter::zCoG(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->zCoG;
}

double InterfaceToScriptFilter::maxPM(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->iPMwithMaxSignal;
}

int InterfaceToScriptFilter::eventId(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->ReconstructionData.at(i)->EventId;
}

double InterfaceToScriptFilter::pmX(int ipm)
{
  if (ipm<0 || ipm > PMs->count()-1) return std::numeric_limits<double>::quiet_NaN();
  return PMs->X(ipm);
}

double InterfaceToScriptFilter::pmY(int ipm)
{
  if (ipm<0 || ipm > PMs->count()-1) return std::numeric_limits<double>::quiet_NaN();
  return PMs->Y(ipm);
}

double InterfaceToScriptFilter::pmZ(int ipm)
{
  if (ipm<0 || ipm > PMs->count()-1) return std::numeric_limits<double>::quiet_NaN();
  return PMs->Z(ipm);
}

double InterfaceToScriptFilter::signal(int i, int ipm)
{
  if (i<0 || i > EventsDataHub->Events.size()-1) return std::numeric_limits<double>::quiet_NaN();
  if (ipm<0 || ipm > PMs->count()-1) return std::numeric_limits<double>::quiet_NaN();
  return EventsDataHub->Events.at(i).at(ipm);
}

double InterfaceToScriptFilter::rho(int i, int ipm)
{
  if (i<0 || i > EventsDataHub->Events.size()-1) return std::numeric_limits<double>::quiet_NaN();
  if (ipm<0 || ipm > PMs->count()-1) return std::numeric_limits<double>::quiet_NaN();

  double dx2 = PMs->X(ipm)-EventsDataHub->ReconstructionData.at(i)->Points[0].r[0];
  dx2 *= dx2;
  double dy2 = PMs->Y(ipm)-EventsDataHub->ReconstructionData.at(i)->Points[0].r[1];
  dy2 *= dy2;

  return sqrt(dx2 + dy2);
}

void InterfaceToScriptFilter::setGood(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return;
  EventsDataHub->ReconstructionData.at(i)->GoodEvent = true;
}

void InterfaceToScriptFilter::setFail(int i)
{
  if (i<0 || i > EventsDataHub->ReconstructionData.size()-1) return;
  EventsDataHub->ReconstructionData.at(i)->GoodEvent = false;
}
