#ifndef SCRIPTFILTERCLASS_H
#define SCRIPTFILTERCLASS_H

#include <QObject>
#include "eventsdataclass.h"
#include "scriptinterfaces.h"
#include "pms.h"

class InterfaceToScriptFilter : public BaseInterfaceClass
{
  Q_OBJECT

public:
  InterfaceToScriptFilter(EventsDataClass* EventsDataHub, pms* PMs);

public slots:
  //requests by event number
  double points(int i);
  double energy(int i);
  double chi2(int i);
  double x(int i);
  double y(int i);
  double z(int i);
  bool   good(int i);
  bool   recOK(int i);
  double maxPM(int i);
  int eventId(int i);
    //CoG
  double xCoG(int i);
  double yCoG(int i);
  double zCoG(int i);
    //scan
  double xScan(int i);
  double yScan(int i);
  double zScan(int i);
  double eScan(int i);

  //requests by PM number
  double pmX(int ipm);
  double pmY(int ipm);
  double pmZ(int ipm);

  //requests by event and PM number
  double signal(int i, int ipm);
  double rho(int i, int ipm);

  //settings requests
  int countEvents()  {return EventsDataHub->ReconstructionData.size();}
  int countGoodEvents()  {return EventsDataHub->countGoodEvents();}
  int countPMs()     {return PMs->count();}
  bool isScanEmpty() {return EventsDataHub->isScanEmpty();}

  //set GoodEvents status
  void setGood(int i);
  void setFail(int i);

private:
  EventsDataClass* EventsDataHub;
  pms* PMs;
};


#endif // SCRIPTFILTERCLASS_H
