#ifndef MAINSCRIPTINTERFACE_H
#define MAINSCRIPTINTERFACE_H

#include <QObject>
#include <QVariant>

class QScriptEngine;

class TRandom2;

class ScriptMode;
class ConfigScriptInterface;
class GenericScriptWindowClass;

class DetectorClass;
class EventsDataClass;
class ReconstructionManagerClass;
class SimulationManager;

class MainScriptInterface : public QObject
{
Q_OBJECT
public:
    MainScriptInterface(ScriptMode *batch, QScriptEngine *engine);

    //We are doing it wrong in GenericScriptWindow. We need more global objects
    //BatchScriptInterface(Batcher *batch, GenericScriptWindowClass *window);
    virtual ~MainScriptInterface();

    QJsonObject &getConfig();
public slots:
    QByteArray loadFile(QString filename);
    void print(QVariant msg);
    void simulate(int threadCount);
    int loadSimulation(QString fileName);
    bool saveSimulation(QString fileName);
    void clearEvents();
    bool reconstruct();
    bool saveReconstruction(QString fileName);
    void filterEvents();
    bool makeLRFs(QVariant lrfcreationconfig);
    bool saveLRFs(QString fileName, int iter);
    bool loadLRFs(QVariant lrfconfig);

private:
    void construct(ScriptMode *batch);
    void setupDetector();
    void onNewSimlationData();
    bool generateScintillation();

    DetectorClass *detector;
    EventsDataClass *eventsHub;
    ReconstructionManagerClass *reconMan;
    SimulationManager *simMan;
    ConfigScriptInterface *configInterface;
};

#endif
