#include "scriptmode.h"
#include "mainscriptinterface.h"
#include "configscriptinterface.h"

#include "detectorclass.h"
#include "eventsdataclass.h"
#include "reconstructionmanagerclass.h"
#include "simulationmanager.h"
#include "sensorlrfs.h"
#include "genericscriptwindowclass.h"
#include "mainwindow.h"

#include <TTree.h>
#include <TFile.h>

#include <QFile>
#include <QScriptEngine>

ScriptMode *batcher = 0;

static bool runScript(QScriptEngine *engine, const QString &script)
{

  //syntax check
  QScriptSyntaxCheckResult check = QScriptEngine::checkSyntax(script);
  if(check.state() != QScriptSyntaxCheckResult::Valid)
  {
    int lineNum = check.errorLineNumber();
    qDebug()<<"Syntax error at line"<<lineNum;
    return false;
  }

  QScriptValue scriptreturn = engine->evaluate(script);
  if(engine->hasUncaughtException())
  {
    //Script has uncaught exception
    QString message = engine->uncaughtException().toString();
    qDebug() << "Error message:" << message;
    QString backtrace = engine->uncaughtExceptionBacktrace().join('\n');
    qDebug() << "backtrace:" << backtrace;
    return false;
  }

  //success
  QString result = scriptreturn.toString();
  qDebug() << "Script returned:" << result;

  engine->collectGarbage();
  return true;
}

ScriptMode::ScriptMode(DetectorClass *detector, EventsDataClass *eventsHub, ReconstructionManagerClass *reconMan)
{
    this->detector = detector;
    this->eventsHub = eventsHub;
    this->reconMan = reconMan;
    this->simMan = new SimulationManager(detector, eventsHub);
}

ScriptMode::~ScriptMode()
{
  delete simMan;
}

void ScriptMode::run(const QStringList &args)
{
  QString script;
  if(!args.count() || args[0] == "-")
  {
    QFile *scriptfile = new QFile;
    if(scriptfile->open(stdin, QIODevice::ReadOnly))
      script = scriptfile->readAll();
    delete scriptfile;
  }
  else
  {
    QFile *scriptfile = new QFile(args[0]);
    if(scriptfile->open(QIODevice::ReadOnly))
      script = scriptfile->readAll();
    delete scriptfile;
  }

  QScriptEngine *engine = new QScriptEngine();
  MainScriptInterface interfacer(this, engine); (void)interfacer;
  runScript(engine, script);

  delete engine;
}

void ScriptMode::attachToWindow(GenericScriptWindowClass *scriptWindow)
{
  qDebug()<<"ScriptWindow is not supported for batch mode. We need to fix ScriptWindow, Andrei!";
  return;

  //Auto-links to window, then window deletes it
  //new MainScriptInterface(this, scriptWindow);
  QStringList script;
  script<<"basepath = '/path/to/workspace/'";
  script<<"Config.loadFile(basepath+'config.json'')";
  script<<"Config.cd('SimulationConfig')";
  script<<"Config.set('Mode', 'PointSim')";
  script<<"Config.cd('PointSourcesConfig/CustomNodesOptions')";
  script<<"var nodes = eval('new Array([0,0,0])')";
  script<<"Config.set('.', { Nodes : nodes, NumberNodes : nodes.length })";
  script<<"Config.set('../ControlOptions/MultipleRunsNumber', 1000)";
  script<<"Config.cd('../ControlOptions')";
  script<<"Config.set('MultipleRuns', true)";
  script<<"simulate(8)";
  script<<"saveSimulation(basepath+'1pointsim.root')";
  scriptWindow->SetExample(script.join('\n'));
}
