//ANTS2
#include "reconstructionwindow.h"
#include "ui_reconstructionwindow.h"
#include "mainwindow.h"
#include "geometrywindowclass.h"
#include "graphwindowclass.h"
#include "pms.h"
#include "outputwindow.h"
#include "sensorlrfs.h"
#include "lrfsliced3d.h"
#include "lrfwindow.h"
#include "CorrelationFilters.h"
#include "classes.h"
#include "windownavigatorclass.h"
#include "guiutils.h"
#include "functions.h"
#include "gainevaluatorwindowclass.h"
#include "viewer2darrayobject.h"
#include "genericscriptwindowclass.h"
#include "eventsdataclass.h"
#include "dynamicpassiveshandler.h"
#include "reconstructionmanagerclass.h"
#include "globalsettingsclass.h"

#ifdef __USE_ANTS_CUDA__
#include "cudamanagerclass.h"
#endif

#ifdef ANTS_FANN
#include "neuralnetworksmodule.h"
#include "neuralnetworkswindow.h"
#endif

//Qt
#include <QFileDialog>
#include <QMessageBox>
#include <QVBoxLayout>
#include <QDialog>
#include <QTextEdit>

//Root
#include "TMath.h"
#include "TVirtualGeoTrack.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TF2.h"
#include "TFile.h"
#include "TLine.h"
#include "TEllipse.h"
#include "TBox.h"
#include "TSystem.h"
#include "TPolyLine.h"
#include "TMultiGraph.h"
#include "TColor.h"
#include "TFitResultPtr.h"
#include "TFitResult.h"

static TH1* tmpHist1D;
static TH2* tmpHist2D;
static TH3* tmpHist3D;

ReconstructionWindow::~ReconstructionWindow()
{
  if (vo) delete vo;
  vo = 0;
  if (dialog)
    {
      disconnect(vo, SIGNAL(PMselectionChanged(QVector<int>)), this, SLOT(onSelectionChange(QVector<int>)));
      delete dialog;
      dialog = 0;
    }

  delete ui;
  if (histCorr) delete histCorr;
  if (CorrCutLine) delete CorrCutLine;
  if (CorrCutEllipse) delete CorrCutEllipse;
  if (CorrCutPolygon) delete CorrCutPolygon;
  if (multigrEE) delete multigrEE;
  if (centHist) delete centHist;
  if (tmp1Dhist) delete tmp1Dhist;
  if (tmpHist) delete tmpHist;

  for (int i=0; i<CorrelationFilters.size(); i++)
      delete CorrelationFilters[i];
  CorrelationFilters.clear();

  ReconstructionWindow::ClearTmpTObjects();

  if (MW->GainWindow)
    {
      //qDebug()<<"  -<Deleting gain evaluation window";
      MW->GainWindow->hide();
      delete MW->GainWindow->parent();
      //delete MW->GainWindow;
      MW->GainWindow = 0;
      //qDebug() << "  --<Deleted";
    }
  ReconstructionWindow::ClearLineMarkers();
  for (int ipm=0; ipm<sigmaHists.size(); ipm++) delete sigmaHists[ipm];
  sigmaHists.clear();
  //qDebug() << " -<Destructor of Reconstruction window finished";
}

void ReconstructionWindow::UpdateStatusAllEvents()
{
  //   qDebug() << "UpdateStatus all events triggered";
  if (EventsDataHub->isEmpty()) return;

  TMPignore = true;
  MW->WindowNavigator->BusyOn();
  QJsonObject json;
  writeFilterSettingsToJson(json);
  json["NumThreads"] = ui->sbRecNumThreads->value();
  SaveJsonToFile(json, "filt.json");
  MW->ReconstructionManager->filterEvents(json);
  //   qDebug() << "  Found good events:"<<EventsDataHub->countGoodEvents();

  ReconstructionWindow::ShowStatistics();
  MW->WindowNavigator->BusyOff(false);
  TMPignore = false;
  //   qDebug() << "Update done";
}

void ReconstructionWindow::ClearLineMarkers()
{
  if (Markers.size()>0)
    {
      for (int i=0; i<Markers.size(); i++) delete Markers[i];
    }

  Markers.clear();
}

void ReconstructionWindow::InitWindow()
{
  ReconstructionWindow::on_pbUpdateFilters_clicked();

  ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked(); //pms module is created after this window, so need to update indication of groups
  ReconstructionWindow::on_pbUpdateGainsIndication_clicked();

  ReconstructionWindow::on_pbCorrUpdateTMP_clicked(); //update correlation filter designer
  tmpCorrFilter.Active = true;
  CorrelationFilterStructure* NewFilter = new CorrelationFilterStructure();
  NewFilter->copyFrom(&tmpCorrFilter);
  NewFilter->Active = true;
  CorrelationFilters.append(NewFilter);  
}

void ReconstructionWindow::on_pbShowLRFwindow_clicked()
{  
  MW->lrfwindow->showNormal();
  MW->lrfwindow->raise();
  MW->lrfwindow->activateWindow();  
}

void ReconstructionWindow::on_cobReconstructionAlgorithm_currentIndexChanged(int index)
{
    ui->swReconstructionAlgorithm->setCurrentIndex(index);
    ReconstructionWindow::updateRedStatusOfRecOptions(); //update warning indicator in tabWidget
    if (index == 1 || index == 2) ui->fDynPassive->setVisible(true);
    else ui->fDynPassive->setVisible(false);  
    ui->cobMultipleOption->setEnabled(index == 2);
}

void ReconstructionWindow::on_pbRefreshDataIndication_clicked()
{   
   QString str;

   if ( EventsDataHub->isEmpty() )
     {
       str = "";
       ui->leoEventsTotal->setText(str);
     }
   else str.setNum(EventsDataHub->Events.size());
   ui->leoEventsTotal->setText(str);

   if (EventsDataHub->TimedEvents.isEmpty() )
     {
       str = "";
       ui->leoTimeBins->setText(str);
       ui->leoTimeBins->setEnabled(false);
       ui->labTimeBins->setEnabled(false);
     }
   else
     {
       str.setNum(EventsDataHub->getTimeBins());
       ui->leoTimeBins->setEnabled(true);
       ui->labTimeBins->setEnabled(true);
     }
   ui->leoTimeBins->setText(str);  
}

void ReconstructionWindow::on_sbEventNumberInspect_valueChanged(int arg1)
{
   if (ForbidUpdate) return;
   if (EventsDataHub->isEmpty()) return;

   int totNumEvents = EventsDataHub->Events.size();
   if ( totNumEvents == 0) return;
   if (arg1 > totNumEvents-1)
     {
       ui->sbEventNumberInspect->setValue(totNumEvents-1);
       return;
     }

   MW->Owindow->SetCurrentEvent(arg1);
   MW->Owindow->RefreshData();

   if (MW->GraphWindow->isVisible())
     {
       if (LastDistributionShown == "ShowExpectedVsMeasured")
         if (MW->SensLRF->isAllLRFsDefined()) ReconstructionWindow::on_pbShowMeasuredVsExpected_clicked();
       if (LastDistributionShown == "MapEvent") ReconstructionWindow::on_pbShowMap_clicked();
     }

   ReconstructionWindow::on_pbReconstructOneEvent_clicked(); //no actual reconstruction
}

void ReconstructionWindow::on_pbReconstructOneEvent_clicked()
{
  if (EventsDataHub->isEmpty())
    {
      message("Data are empty!", this);
      ui->twData->setCurrentIndex(0);
      return;
    }
  if (!EventsDataHub->isReconstructionDataReady())
    {
      message("Reconstruction was NOT yet performed!", this);
      ui->bsAnalyzeScan->setCurrentIndex(0);
      return;
    }
  int iev = ui->sbEventNumberInspect->value();
  if (iev > EventsDataHub->Events.size()-1)
    {
      qDebug()<<"Bad event number in reconstruct one event!";
      return;
    }  

  ReconstructionStruct* result = EventsDataHub->ReconstructionData[iev];
  MW->Owindow->textLogOneEvent(iev, result);
  MW->Owindow->SetCurrentEvent(ui->sbEventNumberInspect->value());
  MW->Owindow->RefreshData();

  if (MW->GeometryWindow->isVisible())
    {
      //MW->DotsTGeo.resize(0);
      MW->clearGeoMarkers();
      MW->Detector->GeoManager->ClearTracks();

      //if (EventsDataHub->fSimulatedData) ReconstructionWindow::UpdateSimVizData(iev);
      if (!EventsDataHub->isScanEmpty()) ReconstructionWindow::UpdateSimVizData(iev);
      if (result->ReconstructionOK)
        {
//          for (int i=0; i<result->Points.size(); i++)
//            MW->DotsTGeo.append(DotsTGeoStruct(result->Points[i].r, kRed));
          GeoMarkerClass* marks = new GeoMarkerClass("Recon", 6, 2, kRed);
          for (int i=0; i<result->Points.size(); i++)
            marks->SetNextPoint(result->Points[i].r[0], result->Points[i].r[1], result->Points[i].r[2]);         
          MW->GeoMarkers.append(marks);
        }
      MW->ShowGeometry(false);  //to clear view
      MW->ShowTracks(); //has to use ShowTracks since if there is continuos energy deposition - tracks are used for inidication
      //MW->ShowGeoMarkers();
    }
}

void ReconstructionWindow::UpdateSimVizData(int iev)
{
  if (EventsDataHub->isScanEmpty()) VisualizeEnergyVector(iev);
  else VisualizeScan(iev);
}

void ReconstructionWindow::VisualizeScan(int iev)
{
  GeoMarkerClass* marks;
  bool fAppend;
  if (MW->GeoMarkers.isEmpty() || MW->GeoMarkers.last()->Type != "Scan")
    { //previous marker was not scan, so making new one
      marks = new GeoMarkerClass("Scan", 6, 2, kBlue);
      fAppend = true;
    }
  else
    { //reusing old scan marker, just add the new point. This way its is MUCH faster to remove markers if one has ~100k o them.
      marks = MW->GeoMarkers.last();
      fAppend = false;
    }

  //scan object must exist
  //two modes to vizualize:
  //secondary scintillation
  if ( EventsDataHub->Scan[iev]->ScintType == 2)
    {
      for (int j=0; j<EventsDataHub->Scan[iev]->Points.size(); j++)
        {
          marks->SetNextPoint(EventsDataHub->Scan[iev]->Points[j].r[0], EventsDataHub->Scan[iev]->Points[j].r[1], EventsDataHub->Scan[iev]->Points[j].r[2]);
          marks->SetNextPoint(EventsDataHub->Scan[iev]->Points[j].r[0], EventsDataHub->Scan[iev]->Points[j].r[1], EventsDataHub->Scan[iev]->zStop);
          //showing track
          Int_t track_index = MW->Detector->GeoManager->AddTrack(10,55); //  Here track_index is the index of the newly created track in the array of primaries. One can get the pointer of this track and make it known as current track by the manager class:
          TVirtualGeoTrack *track = MW->Detector->GeoManager->GetTrack(track_index);
          track->SetLineColor(kBlue);
          track->AddPoint( EventsDataHub->Scan[iev]->Points[j].r[0], EventsDataHub->Scan[iev]->Points[j].r[1], EventsDataHub->Scan[iev]->Points[j].r[2], 0);
          track->AddPoint( EventsDataHub->Scan[iev]->Points[j].r[0], EventsDataHub->Scan[iev]->Points[j].r[1], EventsDataHub->Scan[iev]->zStop, 0);
        }
    }
  else if ( EventsDataHub->Scan[iev]->ScintType == 1) //primary scintillation
    {
      for (int j=0; j<EventsDataHub->Scan[iev]->Points.size(); j++)
        marks->SetNextPoint(EventsDataHub->Scan[iev]->Points[j].r[0], EventsDataHub->Scan[iev]->Points[j].r[1], EventsDataHub->Scan[iev]->Points[j].r[2]);
    }
  else
    {
      //exp data scan
      //qDebug() << EventsDataHub->SimulatedScan[iev]->Points[0].r[0] << EventsDataHub->SimulatedScan[iev]->Points[0].r[1] << ui->ledSuggestedZ->text().toDouble();
      marks->SetNextPoint(EventsDataHub->Scan[iev]->Points[0].r[0], EventsDataHub->Scan[iev]->Points[0].r[1], ui->ledSuggestedZ->text().toDouble());
    }

  if (fAppend) MW->GeoMarkers.append(marks);
}

void ReconstructionWindow::on_pbShowStatistics_clicked()
{
  ReconstructionWindow::ShowStatistics();
}

void ReconstructionWindow::ShowStatistics()
{
  //qDebug()<<"   Show statistics triggered in Recon window";
  int GoodEvents;
  double AvChi2, AvDeviation;
  EventsDataHub->prepareStatisticsForEvents(MW->Detector->SensLRF->isAllLRFsDefined(), GoodEvents, AvChi2, AvDeviation);

  ui->leoEventsPassingAllFilters->setText("");
  ui->leoAverageChi2->setText("");
  ui->leoAverageDeviation->setText("");
  if (GoodEvents == 0) return;

  ui->leoEventsPassingAllFilters->setText( QString::number(GoodEvents));
  if (AvChi2 != -1)
    {     
      ui->leoAverageChi2->setText( QString::number(AvChi2) );
      lastChi2 = AvChi2;  /// *** to remove!
    } 
  if (AvDeviation != -1)  ui->leoAverageDeviation->setText(QString::number(AvDeviation));
}

void ReconstructionWindow::on_pbShowReconstructionPositions_clicked()
{
    MW->GeometryWindow->show();
    MW->GeometryWindow->raise();
    MW->GeometryWindow->activateWindow();
    ReconstructionWindow::ShowReconstructionPositions();
}

void ReconstructionWindow::ShowReconstructionPositions()
{  
  int numEvents = EventsDataHub->ReconstructionData.size();
  if (numEvents==0) return;
  if (!EventsDataHub->isReconstructionDataReady()) return;

  //preparing geometry window
  MW->Detector->GeoManager->ClearTracks(); //tracks could be used for indication (if continuous deposition)

  //if sim data, show it
  //MW->DotsTGeo.resize(0);
  MW->clearGeoMarkers();

  //showing actual (if enabled) - if NOT on top
  if (!ui->cbActualOnTop->isChecked()) ReconstructionWindow::DotActualPositions();

  //showing reconstruction
  int UpperLimit = numEvents;
  if (ui->cbLimitNumberEvents->isChecked()) UpperLimit = ui->sbLimitNumberEvents->value();

  GeoMarkerClass* marks = new GeoMarkerClass("Recon", 6, 2, kRed);
  int goodCounter = 0;
  for (int iev=0; iev<numEvents; iev++)
    {
      if (goodCounter >= UpperLimit) break;
      if (EventsDataHub->ReconstructionData[iev]->GoodEvent)
        {
          for (int iP = 0; iP<EventsDataHub->ReconstructionData[iev]->Points.size(); iP++)
            //MW->DotsTGeo.append(DotsTGeoStruct(EventsDataHub->ReconstructionData[iev]->Points[iP].r, kRed));
            marks->SetNextPoint(EventsDataHub->ReconstructionData[iev]->Points[iP].r[0], EventsDataHub->ReconstructionData[iev]->Points[iP].r[1], EventsDataHub->ReconstructionData[iev]->Points[iP].r[2]);

          goodCounter++;
          if (goodCounter >= UpperLimit) break;
        }
    }

  MW->GeoMarkers.append(marks);

  //showing actual (if enabled) - if on top
  if (ui->cbActualOnTop->isChecked()) ReconstructionWindow::DotActualPositions();

  MW->ShowGeometry(false);
  MW->ShowTracks();
  //MW->ShowGeoMarkers();
}

void ReconstructionWindow::DotActualPositions()
{
  //if (EventsDataHub->fSimulatedData)
  if (!EventsDataHub->isScanEmpty())
   if (ui->cbShowActualPosition->isChecked())
    {
      if (!EventsDataHub->isScanEmpty()) ReconstructionWindow::VisualizeScan();
      else
        {
          qDebug()<<"Vizualizing EnergyVector data...";
          int numEvents = EventsDataHub->ReconstructionData.size();
          if (numEvents==0) return;
          int UpperLimit = numEvents;
          if (ui->cbLimitNumberEvents->isChecked())
            if (ui->sbLimitNumberEvents->value()<UpperLimit) UpperLimit = ui->sbLimitNumberEvents->value();
          for (int iev = 0; iev<UpperLimit; iev++)
            if (EventsDataHub->ReconstructionData[iev]->GoodEvent)
              ReconstructionWindow::VisualizeEnergyVector(iev);
        }
    }
}

void ReconstructionWindow::VisualizeEnergyVector(int eventId)
{
  if (MW->EnergyVector.isEmpty()) return;

//  qDebug()<<"EnergyVector contains "<<MW->EnergyVector.size()<<" cells";

  QVector<EnergyDepositionCell*> EV;
  EV.resize(0);
  for (int i=0; i<MW->EnergyVector.size(); i++)
    if (MW->EnergyVector.at(i)->eventId == eventId) EV.append(MW->EnergyVector.at(i));

//  qDebug()<<"This event ( "<< eventId <<" ) has "<<EV.size()<<" associated cells";

  if (EV.size() == 0) return;

  GeoMarkerClass* marks = new GeoMarkerClass("Scan", 6, 2, kBlue);

  int PreviousIndex = -1;
  bool flagTrackStarted = false;
  for (int ip=0; ip<EV.size(); ip++)
    {
      int ThisIndex = EV[ip]->index;
      if (ThisIndex == PreviousIndex)
        {
          //continuing the track
          if (!flagTrackStarted)
            {
              //previous was the first point of the track
//              qDebug()<<"  continuous deposition start detected";
              Int_t track_index = MW->Detector->GeoManager->AddTrack(10,22); //  Here track_index is the index of the newly created track in the array of primaries. One can get the pointer of this track and make it known as current track by the manager class:
              TVirtualGeoTrack *track = MW->Detector->GeoManager->GetTrack(track_index);
              MW->Detector->GeoManager->SetCurrentTrack(track);
              track->SetLineColor(kBlue);
              MW->Detector->GeoManager->GetCurrentTrack()->AddPoint(EV[ip-1]->r[0], EV[ip-1]->r[1], EV[ip-1]->r[2], 0);
            }
          flagTrackStarted = true; //nothing to do until track ended
        }
      else
        {
          //another particle starts here
          //track teminated last cell?
          if (flagTrackStarted)
            {
              //finishing the track of the previous particle
//              qDebug()<<"  continuous deposition end detected";
              MW->Detector->GeoManager->GetCurrentTrack()->AddPoint(EV[ip-1]->r[0], EV[ip-1]->r[1], EV[ip-1]->r[2], 0);
              //MW->DotsTGeo.append(DotsTGeoStruct(EV[ip-1]->r, kBlue));
              marks->SetNextPoint(EV[ip-1]->r[0], EV[ip-1]->r[1], EV[ip-1]->r[2]);
              flagTrackStarted = false;
            }
          //drawing this point - it is shown for both the track (start point) or individual
          //MW->DotsTGeo.append(DotsTGeoStruct(EV[ip]->r, kBlue));
          marks->SetNextPoint(EV[ip]->r[0], EV[ip]->r[1], EV[ip]->r[2]);

//          qDebug()<<EV[ip]->r[0]<<EV[ip]->r[1]<<EV[ip]->r[2];
        }
      PreviousIndex = ThisIndex;
    }

  //if finished, but track is not closed yet:
  if (flagTrackStarted)
    {
//      qDebug()<<"  continuous deposition end detected";
      MW->Detector->GeoManager->GetCurrentTrack()->AddPoint(EV.last()->r[0], EV.last()->r[1], EV.last()->r[2], 0);
      //MW->DotsTGeo.append(DotsTGeoStruct(EV.last()->r, kBlue));
      marks->SetNextPoint(EV.last()->r[0], EV.last()->r[1], EV.last()->r[2]);

//     qDebug()<<EV.last()->r[0]<< EV.last()->r[1]<< EV.last()->r[2];
    }

  MW->GeoMarkers.append(marks);
}

void ReconstructionWindow::VisualizeScan()
{  
  double ShownPointsLimit = ui->sbLimitNumberEvents->value();
  bool limitPoints = ui->cbLimitNumberEvents->isChecked();
  int ShownPoints = 0;
  for (int i=0; i<EventsDataHub->Scan.size(); i++)
   if (EventsDataHub->ReconstructionData[i]->GoodEvent)
    {
       if (limitPoints)
         if (ShownPoints == ShownPointsLimit) break;
       ReconstructionWindow::VisualizeScan(i);
       ShownPoints++;
    }  
}

void ReconstructionWindow::PMnumChanged()
{   
   ui->sbPMsignalIndividual->setValue(0);   /// *** obsolete ?
}

void ReconstructionWindow::on_sbPMsignalIndividual_valueChanged(int arg1)
{
  if (arg1 == -1)
    {
      if (MW->PMs->count() == 0) ui->sbPMsignalIndividual->setValue(0);
      else ui->sbPMsignalIndividual->setValue(MW->PMs->count()-1);
      return; //update on_change
    }

  if (arg1 > MW->PMs->count()-1)
    {
      if (arg1 == 0) return;
      ui->sbPMsignalIndividual->setValue(0);
      return;
    }

  QString str;
  str.setNum(MW->PMs->at(arg1).cutOffMin);
  ui->ledFilterIndividualMin->setText(str);
  str.setNum(MW->PMs->at(arg1).cutOffMax);
  ui->ledFilterIndividualMax->setText(str);

  if (EventsDataHub->isEmpty()) return;
  if (MW->GraphWindow->isVisible()) ReconstructionWindow::ShowIndividualSpectrum(false);
}

void ReconstructionWindow::on_pbCutOffsLoad_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this,
                                                  "Load individual cut-offs (ignores first column with PM number)", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt)");

  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  ReconstructionWindow::LoadIndividualCutOffs(fileName);
//  if (MW->GraphWindow) MW->GraphWindow->hide();
  ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::LoadIndividualCutOffs(QString fileName)
{
  QVector<double> ii, min, max;
  LoadDoubleVectorsFromFile(fileName, &ii, &min, &max);

  if (min.size() < MW->PMs->count())
    {
      message("Error: data do not exist for all PMs!", this);
      return;
    }

  //MW->Reconstructor->setCutOffMin(&min);
  //MW->Reconstructor->setCutOffMax(&max);
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      MW->PMs->at(ipm).cutOffMin = min[ipm];
      MW->PMs->at(ipm).cutOffMax = max[ipm];
    }
  ReconstructionWindow::on_sbPMsignalIndividual_valueChanged(ui->sbPMsignalIndividual->value());  
}

void ReconstructionWindow::on_pbCutOffsSave_clicked()
{
  QString fileName = QFileDialog::getSaveFileName(this,
                                                  "Save individual cut-offs", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt)");

  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  ReconstructionWindow::SaveIndividualCutOffs(fileName);
}

void ReconstructionWindow::SaveIndividualCutOffs(QString fileName)
{
  QVector<double> ii, min, max;
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      ii.append(ipm);
      min.append( MW->PMs->at(ipm).cutOffMin );
      max.append( MW->PMs->at(ipm).cutOffMax );
    }
  SaveDoubleVectorsToFile(fileName, &ii, &min, &max);
}

void ReconstructionWindow::on_ledFilterIndividualMin_editingFinished()
{
   MW->PMs->at(ui->sbPMsignalIndividual->value()).cutOffMin = ui->ledFilterIndividualMin->text().toDouble();
   if (!TMPignore) ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::on_ledFilterIndividualMax_editingFinished()
{
   MW->PMs->at(ui->sbPMsignalIndividual->value()).cutOffMax = ui->ledFilterIndividualMax->text().toDouble();
   if (!TMPignore) ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::on_pbShowSumSignal_clicked()
{
  int totNumEvents = EventsDataHub->Events.size();
  if ( totNumEvents == 0) return;

  if (MW->hist1) delete MW->hist1;
  MW->hist1 = new TH1D("haSumSign","Sum signal",MW->GlobSet->BinsX, 0,0);
  MW->hist1->SetXTitle("Sum signal");

  int counter = 0;
  const QVector<float>* event;
  double SumCutOffMin = ui->ledFilterSumMin->text().toDouble();
  double SumCutOffMax = ui->ledFilterSumMax->text().toDouble();

  bool AccountForPassive = ui->cbPassivePMsTakenAccount->isChecked();

  for (int iev = 0; iev < totNumEvents; iev++)
    {
      if (!EventsDataHub->ReconstructionData[iev]->GoodEvent) continue;

      double sum = 0;    
      event = EventsDataHub->getEvent(iev);
      for (int ipm = 0; ipm < MW->PMs->count(); ipm++)
        if (!MW->PMs->at(ipm).passive || AccountForPassive)
        {
          double sig = event->at(ipm);     
          if (ui->cbGainsConsideredInFiltering->isChecked()) sig /= MW->PMs->at(ipm).relGain;
          sum += sig;
        }  
      MW->hist1->Fill(sum);
      counter++;
    }

  if (counter > 0)
    {      
      MW->GraphWindow->Draw(MW->hist1);
        //showing range      
      double minY = 0;     
      double maxY = MW->GraphWindow->getCanvasMaxY();
      TLine* L1 = new TLine(SumCutOffMin, minY, SumCutOffMin, maxY);
      L1->SetLineColor(kGreen);
      L1->Draw();
      TLine* L2 = new TLine(SumCutOffMax, minY, SumCutOffMax, maxY);
      L2->SetLineColor(kGreen);
      L2->Draw();     
      MW->GraphWindow->UpdateRootCanvas();
    }
  else message("There are no events which pass all the activated filters!", this);
}

void ReconstructionWindow::on_pbShowIndividualSpectrum_clicked()
{
   ReconstructionWindow::ShowIndividualSpectrum(true) ;
}

void ReconstructionWindow::ShowIndividualSpectrum(bool focus)
{
  if (EventsDataHub->isEmpty()) return;

  int totNumEvents = EventsDataHub->Events.size();
  if ( totNumEvents == 0) return;

  if (MW->hist1) delete MW->hist1;
  int thisipm = ui->sbPMsignalIndividual->value();
  QString nameStr = "PM"+QString::number(thisipm)+" signal";
  QByteArray byteArray = nameStr.toUtf8();
  const char* name = byteArray.constData();

  double min = 0;
  double max = 0;
  if (ui->cbFilterIndividualSignals->isChecked())
    {
      min = MW->PMs->at(thisipm).cutOffMin;
      max = MW->PMs->at(thisipm).cutOffMax;
    }

  MW->hist1 = new TH1D("haIndSpect", name, MW->GlobSet->BinsX, min,max);
  MW->hist1->SetXTitle("Signal");

  int counter = 0;
  const QVector<float>* event;
  for (int iev = 0; iev < totNumEvents; iev++)
    {
      if (!EventsDataHub->ReconstructionData[iev]->GoodEvent) continue;
      event = EventsDataHub->getEvent(iev);
      double sig = event->at(thisipm);
      MW->hist1->Fill(sig);
      counter++;
    }

  if (counter > 0)
    {     
      if (focus) MW->GraphWindow->Draw(MW->hist1);
      else MW->GraphWindow->DrawWithoutFocus(MW->hist1);
       //showing range
      double minY = 0;     
      double maxY = MW->GraphWindow->getCanvasMaxY();
      TLine* L1 = new TLine(MW->PMs->at(thisipm).cutOffMin, minY, MW->PMs->at(thisipm).cutOffMin, maxY);
      L1->SetLineColor(kGreen);
      L1->Draw();
      TLine* L2 = new TLine(MW->PMs->at(thisipm).cutOffMax, minY, MW->PMs->at(thisipm).cutOffMax, maxY);
      L2->SetLineColor(kGreen);
      L2->Draw();      
      MW->GraphWindow->UpdateRootCanvas();
    }
  else message("There are no events which pass all the activated filters!", this);
}

void ReconstructionWindow::ClearTmpTObjects()
{
  for (int i=0; i<tmpTObjects.size(); i++) delete tmpTObjects[i];
  tmpTObjects.clear();
}

void ReconstructionWindow::on_pbShowPMtable_clicked()
{   
    MW->Owindow->SetTab(2);
    MW->Owindow->showNormal();
    MW->Owindow->raise();
    MW->Owindow->activateWindow();
}

void ReconstructionWindow::on_pbGoToNextEvent_clicked()
{
    if (EventsDataHub->isEmpty()) return;
    //if (MW->Reconstructor->sizeOfEvents() == 0) return;

    //if (!MW->Reconstructor->isReconstructionDataReady())
    if (!EventsDataHub->isReconstructionDataReady())
      {
        message("To use this feature, first perform \"Reconstruct all events\"", this);
        return;
      }

    QString selector = ui->cobGoToSelector->currentText();

    for (int iev = ui->sbEventNumberInspect->value()+1; iev < EventsDataHub->Events.size(); iev++)
      {              
        const ReconstructionStruct* result = EventsDataHub->ReconstructionData[iev];
 //       qDebug()<<iev<<"good?"<<result->GoodEvent<<"RecOK?"<<result->ReconstructionOK<<"CutOff?"<<result->FilterCutOff<<"SumCutOff?"<<result->FilterSumCutOff<<"EnFilt?"<<result->FilterEnergy<<"Chi2Filt?"<<result->FilterChi2;

        //good event
        if (result->GoodEvent && selector == "good event")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
        //bad event
        if (!result->GoodEvent && selector == "bad event")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }

        //failed reconstruction
        if (!result->ReconstructionOK && selector == "failed reconstruction")
         {
             ui->sbEventNumberInspect->setValue(iev);
             return;
         }
        //filter - cutoff
        if (result->FilterCutOff==2 && selector == "failed cut-off")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
        //filter - sum cutoff
        if (result->FilterSumCutOff==2 && selector == "failed sum cut-off")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
        //filter - energy
        if (result->FilterEnergy==2 && selector == "failed energy filter")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
        //filter - chi2
        if (result->FilterChi2==2 && selector == "failed chi2 filter")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
        //filter - position
        if (result->FilterPosition==2 && selector == "failed position filter")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
        //filter - kNN
        if (result->FilterkNN==2 && selector == "failed kNN filter")
         {
           ui->sbEventNumberInspect->setValue(iev);
           return;
         }
      }
    MW->Owindow->OutText("No more events according to the selected criterium!");
}

void ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked()
{    
  if (EventsDataHub->isEmpty()) return;
  ReconstructionWindow::UpdateStatusAllEvents();  
}

void ReconstructionWindow::on_pbShowMeasuredVsExpected_clicked()
{
    if (EventsDataHub->isEmpty())
      {
        message("Data are not selected!", this);
        ui->twData->setCurrentIndex(0);
        return;
      }
    if (!MW->SensLRF->isAllLRFsDefined())
      {
        message("LRFs are not defined!", this);
        ui->twOptions->setCurrentIndex(1);
        return;
      }
    //redo reconstruction:
    int iev = ui->sbEventNumberInspect->value();
    if (iev > EventsDataHub->Events.size()-1)
      {
        qDebug()<<"Bad event number in reconstruct one event!";
        return;
      }
    ReconstructionStruct* rec = EventsDataHub->ReconstructionData[iev];
    if (!rec->ReconstructionOK)
      {
        message("Reconstruction failed, no data to show!", this);
        return;
      }

    DynamicPassivesHandler *Passives = new DynamicPassivesHandler(MW->Detector->PMs, EventsDataHub);
    Passives->init(&EventsDataHub->RecSettings);
    if (EventsDataHub->RecSettings.fUseDynamicPassives) Passives->calculateDynamicPassives(iev, rec);

    QVector<double> xA, yA, xdP, ydP;
    for (int ipm = 0; ipm<MW->PMs->count(); ipm++)
      {
        //if (!MW->PMs->at(ipm).passive)
        if (Passives->isActive(ipm))
          {
            //active PM
            double ExpValue = 0;
            for (int iP=0; iP<rec->Points.size(); iP++)
            ExpValue += MW->SensLRF->getLRF(ipm, rec->Points[iP].r)*rec->Points[iP].energy;

            xA.append(ExpValue);          
            yA.append(EventsDataHub->Events[iev][ipm]);
          }
        //else if (MW->PMs->at(ipm).isDynamicPassive())
        else if (Passives->isDynamicPassive(ipm))
          {
            //dynamic passive PM
            double ExpValue = 0;
            for (int iP=0; iP<rec->Points.size(); iP++)
            ExpValue += MW->SensLRF->getLRF(ipm, rec->Points[iP].r)*rec->Points[iP].energy;

            xdP.append(ExpValue);          
            ydP.append(EventsDataHub->Events[iev][ipm]);
          }
      }

    if (xdP.isEmpty()) MW->GraphWindow->ShowGraph(&xA, &yA, kRed, "Expected (from LRF)", "PM signal", 20, 1, 0, 0);
    else
      {
        TGraph* actives = MW->GraphWindow->ShowGraph(&xA, &yA, kRed, "Expected (from LRF)", "PM signal", 20, 1, 0, 0, "", true);
        TGraph* passives = MW->GraphWindow->ShowGraph(&xdP, &ydP, kGray, "Expected (from LRF)", "PM signal", 20, 1, 0, 0, "", true);

        if (multigrEE) delete multigrEE;
        multigrEE = new TMultiGraph();
        multigrEE->Add(passives, "P"); //on delete, multigraph automatically deletes member graphs
        multigrEE->Add(actives, "P");
        multigrEE->SetTitle("Measured vs Expected PM signals;Expected signal (from LRF);PM signal");

        MW->GraphWindow->Draw(multigrEE, "A");
      }

    LastDistributionShown = "ShowExpectedVsMeasured";
    delete Passives;
}

void ReconstructionWindow::on_pbConfigureNN_clicked()
{
#ifdef ANTS_FANN
   MW->NNwindow->showNormal();
   MW->NNwindow->activateWindow();
#else
   message("Neural networks were disabled in .pro", this);
#endif
}

void ReconstructionWindow::LRF_ModuleReadySlot(bool ready)
{ 
//   qDebug()<<ready;
    QString status;
    if (ready) status = "Ready";
    else status = "Not Ready";

    ui->label_ready->setText(status);
    if (ready)
       {
        ui->label_ready->setStyleSheet("QLabel { color : green; }");
        QIcon no;
        ui->twOptions->tabBar()->setTabIcon(1, no);
       }
    else
      {
        ui->label_ready->setStyleSheet("QLabel { color : red; }");
        ui->twOptions->tabBar()->setTabIcon(1, RedIcon);
      }

    ReconstructionWindow::updateRedStatusOfRecOptions();
}

void ReconstructionWindow::updateRedStatusOfRecOptions()
{
  bool showRed = false;

  int recAlg = ui->cobReconstructionAlgorithm->currentIndex();
  if (MW->SensLRF)
     if (!MW->SensLRF->isAllLRFsDefined())
       if (recAlg !=0 && recAlg !=3) showRed = true; //CoG and network do not need LRFs!

  if (showRed)
    {
      ui->label_ready->setStyleSheet("QLabel { color : red; }");
      ui->twOptions->tabBar()->setTabIcon(1, RedIcon);
      ui->tabWidget->tabBar()->setTabIcon(1, RedIcon);
    }
  else
    {
      QIcon no;
      ui->tabWidget->tabBar()->setTabIcon(1, no);
    }
}

void ReconstructionWindow::on_pbLoadLRFs_clicked()
{
    MW->lrfwindow->LoadLRFDialog(this);
}

void ReconstructionWindow::on_pbSaveLRFs_clicked()
{
    MW->lrfwindow->SaveLRFDialog(this);
}

void ReconstructionWindow::updateFiltersGui()
{
  //Update UI
  ui->fEventNumberFilter->setEnabled(ui->cbFilterEventNumber->isChecked());
  ui->fEnergyFilter->setEnabled(ui->cbActivateEnergyFilter->isChecked());
  ui->fLoadedEnergyFilter->setEnabled(ui->cbActivateLoadedEnergyFilter->isChecked());
  ui->fChi2Filter->setEnabled(ui->cbActivateChi2Filter->isChecked());
  ui->fCustomSpatFilter->setEnabled(ui->cbSpFcustom->isChecked());
  ui->fSpFz->setEnabled(!ui->cbSpFallZ->isChecked());
  ui->fFilterSumSignal->setEnabled(ui->cbFilterSumSignal->isChecked());
  ui->fFilterIndividualSignal->setEnabled(ui->cbFilterIndividualSignals->isChecked());
    //warning icons
  bool masterWarningFlag = false;
  QIcon no;
  if (ui->cbFilterEventNumber->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(0, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(0, no);
  if (ui->cbFilterSumSignal->isChecked() || ui->cbFilterIndividualSignals->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(1, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(1, no);
  if (ui->cbActivateEnergyFilter->isChecked() || ui->cbActivateLoadedEnergyFilter->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(2, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(2, no);
  if (ui->cbActivateChi2Filter->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(3, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(3, no);
  if (ui->cbSpFlimitToPrimScint->isChecked() || ui->cbSpFcustom->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(4, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(4, no);
  if (ui->cbActivateCorrelationFilters->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(5, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(5, no);
  if (ui->cbActivateNNfilter->isChecked())
    {
      ui->twData->tabBar()->setTabIcon(6, YellowIcon);
      masterWarningFlag = true;
    }
  else ui->twData->tabBar()->setTabIcon(6, no);

  if (masterWarningFlag) ui->tabWidget->tabBar()->setTabIcon(0, YellowIcon);
  else ui->tabWidget->tabBar()->setTabIcon(0, no);

    //building polygon if ui->cobSpFXY->currentIndex() != 3 (custom), otherwise reading the table
  int mode = ui->cobSpFXY->currentIndex();
  ui->fTabPoly->setVisible(mode != 1);
  double x0 = ui->ledSpF_X0->text().toDouble();
  double y0 = ui->ledSpF_Y0->text().toDouble();
  switch (mode)
    {
     case 0: //square
      {
        polygon.clear();
//          qDebug()<<"creating polygon: square";
      double xs = ui->ledSpFsizeX->text().toDouble()*0.5;
      double ys = ui->ledSpFsizeY->text().toDouble()*0.5;      
      polygon << QPointF(x0+xs , y0+ys) << QPointF(x0+xs, y0-ys) << QPointF(x0-xs, y0-ys) << QPointF(x0-xs, y0+ys) << QPointF(x0+xs, y0+ys);
//        qDebug()<<"points:"<<polygon.size();
      break;
      }
     case 1: //round
      {
        polygon.clear();
       double radius = ui->ledSpFdiameter->text().toDouble()*0.5;
       for (int i=0; i<51; i++)
         {
           double angle = 3.1415926536*i/25.0;
           polygon << QPointF(x0+radius*sin(angle), y0+radius*cos(angle));
         }
       break;
      }
     case 2: //hexagon
      {
        polygon.clear();
       double side = ui->ledSpFside->text().toDouble();
       double angle0 = ui->ledSpFangle->text().toDouble();
       for (int i=0; i<6; i++)
         {
           double angle = angle0 + 60.0*i;
           angle = angle/180.0*3.1415926536;
           polygon << QPointF(x0+side*sin(angle), y0+side*cos(angle));
         }
       polygon << QPointF(x0+side*sin(angle0/180.0*3.1415926536), y0+side*cos(angle0/180.0*3.1415926536)); //to close it
      break;
      }
     case 3:
      {
        //reading table and creating polygon
        ReconstructionWindow::TableToPolygon();
      }
    }

  //writing the table
  ReconstructionWindow::on_pbSpF_UpdateTable_clicked();  
}

void ReconstructionWindow::on_pbUpdateFilters_clicked()
{
  //   qDebug() << "UpdateFilterButton pressed";
  ReconstructionWindow::updateFiltersGui();
  if (ui->cbSpFcustom->isChecked())
    if (MW->GeometryWindow->isVisible()) ReconstructionWindow::on_pbShowSpatialFilter_clicked();
  if (TMPignore) return; // in this case only GUI update
  EventsDataHub->clearReconstructionTree();
  ReconstructionWindow::UpdateStatusAllEvents();
  this->activateWindow();
}

void ReconstructionWindow::on_pbEnergySpectrum1_clicked()
{
    ReconstructionWindow::ShowEnergySpectrum();
}

void ReconstructionWindow::on_pbEnergySpectrum2_clicked()
{
    ReconstructionWindow::ShowEnergySpectrum();   
}

void ReconstructionWindow::ShowEnergySpectrum()
{
    if (EventsDataHub->ReconstructionData.isEmpty())
    {
        message("Reconstruction data are empty!", this);
        ui->bsAnalyzeScan->setCurrentIndex(1);
        return;
    }

    if (MW->hist1) delete MW->hist1;
    MW->hist1 = new TH1D("hist1ShEnSp","Energy spectrum", MW->GlobSet->BinsX, 0, 0);

    for (int i=0; i<EventsDataHub->ReconstructionData.size(); i++)
        if (EventsDataHub->ReconstructionData[i]->GoodEvent)
          for (int iP=0; iP<EventsDataHub->ReconstructionData[i]->Points.size(); iP++)
            MW->hist1->Fill(EventsDataHub->ReconstructionData[i]->Points[iP].energy);

    MW->hist1->SetXTitle("Energy");
    MW->GraphWindow->Draw(MW->hist1);
    //showing range
    double minY = 0;
    //double maxY = MW->graphRW->getCanvasMaxY();
    double maxY = MW->GraphWindow->getCanvasMaxY();
    double filtMin = ui->ledEnergyMin->text().toDouble();
    double filtMax = ui->ledEnergyMax->text().toDouble();
    TLine* L1 = new TLine(filtMin, minY, filtMin, maxY);
    L1->SetLineColor(kGreen);
    L1->Draw();
    TLine* L2 = new TLine(filtMax, minY, filtMax, maxY);
    L2->SetLineColor(kGreen);
    L2->Draw();
    MW->GraphWindow->UpdateRootCanvas();
    LastDistributionShown = "EnergySpectrum";
}

void ReconstructionWindow::on_pbChi2Spectrum1_clicked()
{  
  ReconstructionWindow::ShowChi2Spectrum();
}

void ReconstructionWindow::on_pbChi2Spectrum2_clicked()
{
    ReconstructionWindow::ShowChi2Spectrum();
}

void ReconstructionWindow::ShowChi2Spectrum()
{
  if (EventsDataHub->ReconstructionData.isEmpty())
    {
      message("Reconstruction data are empty!", this);
      ui->bsAnalyzeScan->setCurrentIndex(1);
      return;
    }
  if (MW->hist1) delete MW->hist1;
  MW->hist1 = new TH1D("hist1Chi2Dis","Chi2 distribution", MW->GlobSet->BinsX, 0, 0);
  MW->hist1->SetXTitle("Chi2");

  for (int i=0; i<EventsDataHub->ReconstructionData.size(); i++)
      if (EventsDataHub->ReconstructionData[i]->GoodEvent)
          MW->hist1->Fill(EventsDataHub->ReconstructionData[i]->chi2);

  MW->GraphWindow->Draw(MW->hist1);
  double minY = 0;
  double maxY = MW->GraphWindow->getCanvasMaxY();
  double filtMin = ui->ledChi2Min->text().toDouble();
  double filtMax = ui->ledChi2Max->text().toDouble();
  TLine* L1 = new TLine(filtMin, minY, filtMin, maxY);
  L1->SetLineColor(kGreen);
  L1->Draw();
  TLine* L2 = new TLine(filtMax, minY, filtMax, maxY);
  L2->SetLineColor(kGreen);
  L2->Draw();
  MW->GraphWindow->UpdateRootCanvas();
  LastDistributionShown = "Chi2Spectrum";
}

void ReconstructionWindow::ResetStop()
{
  ui->prReconstruction->setValue(0); //reset simulation progress bar
  MW->WindowNavigator->setProgress(0);
  ui->pbStopReconstruction->setChecked(false);
  ui->pbStopReconstruction->setEnabled(false);
}

void ReconstructionWindow::SetProgress(int val)
{
  ui->prReconstruction->setValue(val);
}

void ReconstructionWindow::onBusyOn()
{
  //qDebug() << "Busy ON!";
  ui->twData->setEnabled(false);
  ui->twOptions->setEnabled(false);
  ui->bsAnalyzeScan->setEnabled(false);
}

void ReconstructionWindow::onBusyOff()
{
  //qDebug() << "Busy OFF!";
  ui->twData->setEnabled(true);
  ui->twOptions->setEnabled(true);
  ui->bsAnalyzeScan->setEnabled(true);
  ui->pbStopReconstruction->setEnabled(false);
  ui->pbStopReconstruction->setChecked(false);
}

void ReconstructionWindow::on_sbZshift_valueChanged(int arg1)
{
  QString str;
  int delta = arg1-ShiftZoldValue;

  str.setNum(ui->ledZfrom->text().toDouble()+delta);
  ui->ledZfrom->setText(str);
  str.setNum(ui->ledZto->text().toDouble()+delta);
  ui->ledZto->setText(str);

  ShiftZoldValue = arg1;
}

void ReconstructionWindow::on_cobHowToAverageZ_currentIndexChanged(int index)
{
  ui->fAverageZ->setVisible((bool)index);
}

bool ReconstructionWindow::event(QEvent *event)
{
  if (!MW->WindowNavigator) return QMainWindow::event(event);

  if (event->type() == QEvent::Hide)
    {      
      MW->WindowNavigator->HideWindowTriggered("recon");
      return true;
    }
  if (event->type() == QEvent::Show)
    {
      MW->WindowNavigator->ShowWindowTriggered("recon");
    }

  return QMainWindow::event(event);
}

bool ReconstructionWindow::ShowVsXY(QString strIn) //false - error
{  
  if (strIn == "Delta rec-actual")
    {
      if (EventsDataHub->isScanEmpty())
        {
          message("Can be only used if scan data are present!", this);
          return false;
        }
    }

  int ipmSelected = ui->sbShowXYpmNumber->value();
  if (strIn == "PM signal")
    {
       if (ipmSelected > MW->PMs->count()-1)
       {
           message("PM number is out of range!", this);
           return false;
       }
    }

  if (!MW->SensLRF->isAllLRFsDefined())
    if (strIn != "Event density"  && strIn != "Sum PM signal" && strIn != "Delta rec-actual XY")
    {
      message("LRFs are not defined!", this);
      ui->twOptions->setCurrentIndex(2);
      return false;
    }

  if (EventsDataHub->ReconstructionData.isEmpty())
  {
    message("Data not ready: run reconstruction first!", this);
    return false;
  }

  if (EventsDataHub->isScanEmpty()) ui->cbPlotVsActualPosition->setChecked(false);
  bool AccountForPassive = ui->cbPassivePMsTakenAccount->isChecked();

  //init
  double Zmin = ui->ledZfrom->text().toDouble();
  double Zmax = ui->ledZto->text().toDouble();
  bool UseScan = ui->cbPlotVsActualPosition->isChecked();
  bool fShowPMs = ui->cbShowPMs->isChecked();
  bool fShowManifest = ui->cbShowManifestItems->isChecked();

  if (MW->hist2D) delete MW->hist2D;
  MW->hist2D = new TH2D("hist2d","",ui->sbXbins->value(), ui->ledXfrom->text().toDouble(), ui->ledXto->text().toDouble(),
                                    ui->sbYbins->value(), ui->ledYfrom->text().toDouble(), ui->ledYto->text().toDouble());


  //hist2D will hold the sum of energy,
  //while histNum will have number of events contributed to each cell
  TH2D* histNum = new TH2D("histNum","h2",ui->sbXbins->value(), ui->ledXfrom->text().toDouble(), ui->ledXto->text().toDouble(),
                                        ui->sbYbins->value(), ui->ledYfrom->text().toDouble(), ui->ledYto->text().toDouble());

//  qDebug() << "Events:" <<  EventsDataHub->ReconstructionData.size();
  for (int iev = 0; iev < EventsDataHub->ReconstructionData.size(); iev++)
   {
    if (EventsDataHub->ReconstructionData[iev]->Points.size() == 1)
      {
 //      qDebug()<<"iev"<<iev<<"Points"<<EventsDataHub->ReconstructionData[iev]->Points.size()<<"coords:"<<EventsDataHub->ReconstructionData[iev]->Points[0].r[0]<<EventsDataHub->ReconstructionData[iev]->Points[0].r[1]<<EventsDataHub->ReconstructionData[iev]->Points[0].r[2];
        if (! EventsDataHub->ReconstructionData[iev]->GoodEvent) continue;

        if (ui->cobHowToAverageZ->currentIndex() == 1)
          {//have to filter by Z
            double Z;
            if (UseScan) Z = EventsDataHub->Scan[iev]->Points[0].r[2];
            else Z = EventsDataHub->ReconstructionData[iev]->Points[0].r[2];

            if (Z<Zmin || Z>Zmax) continue;
          }

        double val;
        if (strIn == "Energy") val = EventsDataHub->ReconstructionData[iev]->Points[0].energy;
        else if (strIn == "Chi2") val = EventsDataHub->ReconstructionData[iev]->chi2;
        else if (strIn == "Event density") val = 1;
        else if (strIn == "Sum PM signal")
          {
            val = 0;
            for (int ipm = 0; ipm<MW->PMs->count(); ipm++)
              if (!MW->PMs->at(ipm).passive || AccountForPassive)
                {
                  double delta = EventsDataHub->Events[iev][ipm];
                  if (ui->cbGainsConsideredInFiltering->isChecked()) delta /= MW->PMs->at(ipm).relGain;
                  val += delta;
                }
          }
        else if (strIn == "PM signal")
          {
              val = EventsDataHub->Events[iev][ipmSelected];
          }
        else if (strIn == "Delta rec-actual XY")
          {
            double deltaX = EventsDataHub->ReconstructionData[iev]->Points[0].r[0] - EventsDataHub->Scan[iev]->Points[0].r[0]; deltaX *= deltaX;
            double deltaY = EventsDataHub->ReconstructionData[iev]->Points[0].r[1] - EventsDataHub->Scan[iev]->Points[0].r[1]; deltaY *= deltaY;
            //double deltaZ = EventsDataHub->ReconstructionData[iev]->Points[0].r[2] - (*MW->Reconstructor->Scan)[iev]->Points[0].r[2]; deltaZ *= deltaZ;
            //val = TMath::Sqrt(deltaX + deltaY + deltaZ);
            val = TMath::Sqrt(deltaX + deltaY);
          }
        else
          {
            message("Error: unknow distribution requested!", this);
            return false;
            //val = 0;
          }

        if (UseScan)
          {
            MW->hist2D->Fill( EventsDataHub->Scan[iev]->Points[0].r[0], EventsDataHub->Scan[iev]->Points[0].r[1], val);
               histNum->Fill( EventsDataHub->Scan[iev]->Points[0].r[0], EventsDataHub->Scan[iev]->Points[0].r[1]);
          }
        else
          {
            MW->hist2D->Fill(EventsDataHub->ReconstructionData[iev]->Points[0].r[0], EventsDataHub->ReconstructionData[iev]->Points[0].r[1], val);
                histNum->Fill(EventsDataHub->ReconstructionData[iev]->Points[0].r[0], EventsDataHub->ReconstructionData[iev]->Points[0].r[1]);                
          }
      }

   }

  //calculating average
  if (strIn != "Event density") *MW->hist2D = *MW->hist2D / *histNum;

  //title and axes
  if (strIn == "Event density") MW->hist2D->SetTitle("Event density vs XY");
  if (strIn == "Energy") MW->hist2D->SetTitle("Event energy vs XY");
  else if (strIn == "Chi2") MW->hist2D->SetTitle("Chi2 vs XY");
  else if (strIn == "Sum PM signal") MW->hist2D->SetTitle("Sum PM signal vs XY");
  else if (strIn == "Delta rec-actual XY") MW->hist2D->SetTitle("Delta reconstructed-actual vs XY");
  else if (strIn == "PM signal") MW->hist2D->SetTitle("PM signal vs XY");

  MW->hist2D->GetXaxis()->SetTitle("X, mm");
  MW->hist2D->GetYaxis()->SetTitle("Y, mm");

  double max = MW->hist2D->GetMaximum();
  double min = MW->hist2D->GetMinimum();
  double min0 = MW->hist2D->GetMinimum(0);  //except 0
  //qDebug()<<"min/max"<<min << max << "min0"<<min0;
  if (ui->cbSuppress0bins->isChecked()) MW->hist2D->GetZaxis()->SetRangeUser(min0, max);
  else
    {
      if (min >= 0) MW->hist2D->GetZaxis()->SetRangeUser(-0.01, max);
      else          MW->hist2D->GetZaxis()->SetRangeUser(min, max);
    }

  //drawing
  MW->GraphWindow->Draw(MW->hist2D, "colz", !(fShowPMs || fShowManifest) );
  delete histNum;

  //show additional objects
  if (fShowPMs || fShowManifest) ReconstructionWindow::ClearTmpTObjects();
  if (fShowPMs)
    {      
      for (int ipm=0; ipm<MW->PMs->count(); ipm++)
        {
          double x = MW->PMs->X(ipm);
          double y = MW->PMs->Y(ipm);
          double r1 = 0.5*MW->PMs->getTypeForPM(ipm)->SizeX;
          double r2 = 0.5*MW->PMs->getTypeForPM(ipm)->SizeY;
          int shape = MW->PMs->getTypeForPM(ipm)->Shape; // 0 - circ, 1 - rect, 2 - hexa

          switch (shape)
            {
            case 0:
              {
                TBox* e = new TBox(x-r1, y-r2, x+r1, y+r2);
                e->SetFillStyle(0);
                MW->GraphWindow->Draw(e, "same", false);
                tmpTObjects.append(e);
                break;
              }
            case 1:
              {
                TEllipse* e = new TEllipse(x, y, r1, r1);
                e->SetFillStyle(0);
                MW->GraphWindow->Draw(e, "same", false);
                tmpTObjects.append(e);
                break;
              }
            case 2:
              {
                ///
                double xx[7];
                double yy[7];
                for (int j=0; j<7; j++)
                  {
                    double angle = 3.1415926535/3.0 * j + 3.1415926535/2.0;
                    xx[j] = x + r1*cos(angle);
                    yy[j] = y + r1*sin(angle);
                  }
                TPolyLine *e = new TPolyLine(7, xx, yy);

             //   TEllipse* e = new TEllipse(x, y, r1, r1);
                e->SetFillStyle(0);
                MW->GraphWindow->Draw(e, "same", false);
                tmpTObjects.append(e);
              }
            }
        }      
    }

  if (fShowManifest)
    {
      for (int iobj=0; iobj<EventsDataHub->Manifest.size(); iobj++)
        {
          TObject* ob = EventsDataHub->Manifest[iobj]->makeTObject();
          MW->GraphWindow->Draw(ob, "same", false);
          tmpTObjects.append(ob);
        }
    }

  if (fShowPMs || fShowManifest)
    {
      MW->GraphWindow->UpdateRootCanvas();
      MW->GraphWindow->UpdateControls();
    }

  return true;
}

void ReconstructionWindow::on_pbShowSpatialFilter_clicked()
{  
    MW->Detector->GeoManager->ClearTracks();
    MW->GeometryWindow->SetAsActiveRootWindow();

    if (ui->cbSpFallZ->isChecked()) ReconstructionWindow::DrawPolygon(0);
    else
      {
        ReconstructionWindow::DrawPolygon(ui->ledSpFfromZ->text().toDouble());
        ReconstructionWindow::DrawPolygon(ui->ledSpFtoZ->text().toDouble());
      }
    MW->ShowTracks();
}

void ReconstructionWindow::DrawPolygon(double z)
{
  Int_t track_index = MW->Detector->GeoManager->AddTrack(11,22); //  user id = 11 for filter
  TVirtualGeoTrack *track = MW->Detector->GeoManager->GetTrack(track_index);
  track->SetLineColor(kBlue);
  track->SetLineWidth(3);

  for (int i=0; i<polygon.size(); i++)
    {
      double x = polygon[i].x();
      double y = polygon[i].y();
//      qDebug()<<x<<y;
      track->AddPoint(x, y, z, 0);
    }
//  qDebug()<<"track added";
}

void ReconstructionWindow::on_pbSpF_UpdateTable_clicked()
{
   ReconstructionWindow::PolygonToTable();
}

void ReconstructionWindow::on_pbSpF_AddLine_clicked()
{ //add line before
  int selected = ui->tabwidSpF->currentRow();
//  qDebug()<<selected;

  polygon.insert(selected, QPointF(0.0, 0.0));
  if (selected == 0)
    {
      //have to replace the last node (hidden) too
      polygon.replace(polygon.size()-1, polygon.first());
    }
  ReconstructionWindow::PolygonToTable();

  if (ui->cobSpFXY->currentIndex() != 3) ui->cobSpFXY->setCurrentIndex(3);
  else ReconstructionWindow::on_pbUpdateFilters_clicked();
}

void ReconstructionWindow::on_pbSpF_AddLineAfter_clicked()
{ //add line after
  int selected = ui->tabwidSpF->currentRow();
//  qDebug()<<selected;

  polygon.insert(selected+1, QPointF(0.0, 0.0));
  ReconstructionWindow::PolygonToTable();

  if (ui->cobSpFXY->currentIndex() != 3) ui->cobSpFXY->setCurrentIndex(3);
  else ReconstructionWindow::on_pbUpdateFilters_clicked();
}

void ReconstructionWindow::on_pbSpF_RemoveLine_clicked()
{
    int selected = ui->tabwidSpF->currentRow();
//    qDebug()<<selected;
    if (selected == -1)
      {
        message("select a row first!", this);
        return;
      }
    if (ui->tabwidSpF->rowCount()<4)
      {
        message("Minimum is three rows!", this);
        return;
      }

    polygon.remove(selected);
    if (selected == 0)
      {
        //have to change the last point of polygon too (it is not shown in the table)
        polygon.remove(polygon.size()-1);
        polygon.append(polygon.first());
      }
    ReconstructionWindow::PolygonToTable();

    if (ui->cobSpFXY->currentIndex() != 3) ui->cobSpFXY->setCurrentIndex(3);
    else ReconstructionWindow::on_pbUpdateFilters_clicked();
}

void ReconstructionWindow::on_tabwidSpF_cellChanged(int row, int column)
{
   if (TMPignore) return;
   if (TableLocked) return; //do not update during filling with data!

   bool ok;
   QString str = ui->tabwidSpF->item(row,column)->text();
   double newVal = str.toDouble(&ok);
   qDebug()<<str<<newVal;
   if (!ok)
     {
       message("Input error!", this);
       ReconstructionWindow::PolygonToTable();
       return;
     }   

   if (ui->cobSpFXY->currentIndex() != 3) ui->cobSpFXY->setCurrentIndex(3);
   else ReconstructionWindow::on_pbUpdateFilters_clicked();
}

void ReconstructionWindow::PolygonToTable()
{
  //table inits
  TableLocked = true;
  ui->tabwidSpF->clearContents();    
  ui->tabwidSpF->setShowGrid(false);
  int rows = polygon.size()-1;
  if (polygon.isEmpty() || ui->cobSpFXY->currentIndex() == 1) rows = 0;
  ui->tabwidSpF->setRowCount(rows);
  ui->tabwidSpF->setSelectionBehavior(QAbstractItemView::SelectRows);
  ui->tabwidSpF->setSelectionMode(QAbstractItemView::SingleSelection);

  for (int j=0; j<ui->tabwidSpF->rowCount(); j++)
    {
      QString str;
      str.setNum(polygon[j].x(), 'f', 1);
      ui->tabwidSpF->setItem(j, 0, new QTableWidgetItem(str));
      str.setNum(polygon[j].y(), 'f', 1);
      ui->tabwidSpF->setItem(j, 1, new QTableWidgetItem(str));
    }

  ui->tabwidSpF->resizeColumnsToContents();
  ui->tabwidSpF->resizeRowsToContents();
  //ui->tabwidSpF->setColumnWidth(0, 56);
  //ui->tabwidSpF->setColumnWidth(1, 56);
  TableLocked = false;
}

void ReconstructionWindow::TableToPolygon()
{
  polygon.resize(0);
  //reading table and filling the polygon
  int rows = ui->tabwidSpF->rowCount();
  for (int i=0; i<rows; i++)
    {
      double valX = ui->tabwidSpF->item(i,0)->text().toDouble();
      double valY = ui->tabwidSpF->item(i,1)->text().toDouble();
      polygon<<QPointF(valX, valY);
      //qDebug()<<valX<<valY;
    }
  polygon<<polygon.first();
}


void ReconstructionWindow::on_pbGoToNextNoise_clicked()
{
  if (EventsDataHub->isEmpty()) return;
  if (EventsDataHub->isScanEmpty()) return;

  for (int iev = ui->sbEventNumberInspect->value()+1; iev < EventsDataHub->Events.size(); iev++)
    {
      if ( !EventsDataHub->Scan[iev]->GoodEvent)
        {
          ui->sbEventNumberInspect->setValue(iev);
          return;
        }
    }
  MW->Owindow->OutText("There are no more noise events after this index!");
}

void ReconstructionWindow::on_pbGoToNextNoiseFoundGood_clicked()
{
  if (EventsDataHub->isEmpty()) return;
  if (EventsDataHub->isScanEmpty()) return;

  if (EventsDataHub->ReconstructionData.isEmpty())
    {
      message("Run reconstruction of all events first!", this);
      ui->bsAnalyzeScan->setCurrentIndex(1);
      return;
    }

  for (int iev = ui->sbEventNumberInspect->value()+1; iev < EventsDataHub->Events.size(); iev++)
    {
      if ( !EventsDataHub->Scan[iev]->GoodEvent)
        {
          if (EventsDataHub->ReconstructionData[iev]->GoodEvent)
            {
              ui->sbEventNumberInspect->setValue(iev);
              return;
            }
        }
    }
  MW->Owindow->OutText("There are no more noise events which pass all filters after this index!");
}

void ReconstructionWindow::on_pbStopReconstruction_toggled(bool checked)
{
  if (checked)
    {
      StopRecon = true;
      ui->pbStopReconstruction->setText("stopping...");
      emit StopRequested();
    }
  else
    {
      StopRecon = false;
      ui->pbStopReconstruction->setText("stop");
    }
}

void ReconstructionWindow::on_pbCutOffApplyToAll_clicked()
{
  for (int i=0; i<MW->PMs->count(); i++)
    {
     //MW->Reconstructor->setCutOffMin(i, ui->ledFilterIndividualMin->text().toDouble());
     MW->PMs->at(i).cutOffMin = ui->ledFilterIndividualMin->text().toDouble();
     //MW->Reconstructor->setCutOffMax(i, ui->ledFilterIndividualMax->text().toDouble());
     MW->PMs->at(i).cutOffMax = ui->ledFilterIndividualMax->text().toDouble();
    }

  ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::on_pbReconstructTrack_clicked()
{
  ReconstructionWindow::ReconstructTrack();  /// ReconstructionWindowTools/Reconstruction_Track.cpp
}

void ReconstructionWindow::on_cbRecType_currentIndexChanged(int index)
{
    ui->swPointVsTracks->setCurrentIndex(index);
}

void ReconstructionWindow::on_pbAddToActive_clicked()
{
  ReconstructionWindow::MakeActivePassive(ui->leActivePassivePMs->text(), "active");
}

void ReconstructionWindow::on_pbAddToPassive_clicked()
{
  ReconstructionWindow::MakeActivePassive(ui->leActivePassivePMs->text(), "passive");
}

void ReconstructionWindow::MakeActivePassive(QString input, QString mode)
{
  QList<int> ToAdd;
  bool ok = ExtractNumbersFromQString(input, &ToAdd);
  if (!ok)
    {
      message("Input error!", this);
      return;
    }

  int iMaxPM = MW->PMs->count()-1;
  for (int i=0; i<ToAdd.size(); i++)
    if (ToAdd[i] >iMaxPM )
      {
        message("Range error!", this);
        return;
      }

  //no error
  if (mode == "active") for (int i=0; i<ToAdd.size(); i++) MW->PMs->at(ToAdd[i]).passive = 0;
  else if (mode == "passive") for (int i=0; i<ToAdd.size(); i++) MW->PMs->at(ToAdd[i]).passive = 1;
  else
     {
      message("Unknown mode!", this);
      return;
     }

//  for (int i=0; i<ToAdd.size(); i++) qDebug()<<ToAdd[i];

  //MW->ShowGeometry();
  ReconstructionWindow::on_pbShowPassivePMs_clicked();
}

void ReconstructionWindow::on_pbShowPassivePMs_clicked()
{
    MW->Detector->GeoManager->ClearTracks();
    MW->ShowGeometry();
    //clear dots: to do

    //cycle by PMs and if passive, add tracks for indication
    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
        const pm &PM = MW->PMs->at(ipm);
        if (PM.isStaticPassive())
        {
            int typ = PM.type;
            double length = MW->PMs->getType(typ)->SizeX*0.25;
            Int_t track_index = MW->Detector->GeoManager->AddTrack(1,22);
            TVirtualGeoTrack *track = MW->Detector->GeoManager->GetTrack(track_index);
            track->AddPoint(PM.x-length, PM.y-length, PM.z-length, 0);
            track->AddPoint(PM.x+length, PM.y+length, PM.z+length, 0);
            track->AddPoint(PM.x, PM.y, PM.z, 0);
            track->AddPoint(PM.x-length, PM.y-length, PM.z+length, 0);
            track->AddPoint(PM.x+length, PM.y+length, PM.z-length, 0);
            track->AddPoint(PM.x, PM.y, PM.z, 0);
            track->AddPoint(PM.x-length, PM.y+length, PM.z-length, 0);
            track->AddPoint(PM.x+length, PM.y-length, PM.z+length, 0);
            track->AddPoint(PM.x, PM.y, PM.z, 0);
            track->AddPoint(PM.x-length, PM.y+length, PM.z+length, 0);
            track->AddPoint(PM.x+length, PM.y-length, PM.z-length, 0);
            track->AddPoint(PM.x, PM.y, PM.z, 0);

            track->SetLineWidth(3);
            track->SetLineColor(30);
        }
    }

    MW->ShowTracks();

    QString str, str1;
    for (int ipm = 0; ipm<MW->PMs->count(); ipm++)
      if (MW->PMs->at(ipm).passive)
        {
          if (str != "") str += ", ";
          str1.setNum(ipm);
          str += str1;
        }
    if (str == "") str = "none";
    MW->Owindow->OutText("Currently these PMs are marked as 'Passive PMs': "+str);
    //MW->Owindow->SetTab(0);
}

void ReconstructionWindow::on_pbAllPMsActive_clicked()
{
  for (int ipm = 0; ipm<MW->PMs->count(); ipm++) MW->PMs->at(ipm).passive = 0;
  ReconstructionWindow::on_pbShowPassivePMs_clicked();
  MW->Owindow->RefreshData();
}

void ReconstructionWindow::on_pbMainWindow_clicked()
{
    //this->hide();
    MW->showNormal();
    MW->raise();
    MW->activateWindow();
}

void ReconstructionWindow::on_pbShowLog_clicked()
{
    MW->Owindow->SetTab(0);
    MW->Owindow->raise();
    MW->Owindow->showNormal();
    MW->Owindow->activateWindow();
}

void ReconstructionWindow::on_ledDynamicPassiveThreshold_editingFinished()
{
  double threshold = ui->ledDynamicPassiveThreshold->text().toDouble();
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobPMGroupForDynPassives);
  MW->PMs->setPassiveThreshold(igroup, threshold);
}

void ReconstructionWindow::on_pbDoAutogroup_clicked()
{
  //depends on the current PM array type and whether or not GDMl was loaded
  int PMtypes = MW->PMs->countPMtypes();
  int numPMs = MW->PMs->count();

  MW->PMs->clearPMgroups();  //clears gains too!

  bool RegularArray = true;
  int ThisGroup = 1; // 0 is default group for unassigned PMs

  //if (MW->GeoManagerImported) RegularArray = false;
  if (!MW->Detector->isGDMLempty()) RegularArray = false;
  else RegularArray = true;
  //else if (MW->PMArrayType() > 1) RegularArray = false;   ///***new*** - to do!!!

  if (RegularArray)
    {
      //upper and lower PM planes are definitely defined
      for (int iul = 0; iul <2; iul++) //iul = 0 - upper, 1 - lower
        {

          for (int ityp = 0; ityp<PMtypes; ityp++)
            {
              int counter = 0; //found pms of this type/position
              for (int ipm = 0; ipm<numPMs; ipm++)
                if (MW->PMs->at(ipm).type == ityp && MW->PMs->at(ipm).upperLower == iul)
                  {
                    counter++;
                    MW->PMs->at(ipm).group = ThisGroup; //careful - group is defined below!
                  }
//              qDebug()<<"----"<<iul<<ityp<<counter;

              if (counter >0)
                {
                  //PMs found for this location / type
                  QString str;
                  if (iul == 0) str = "Upper PMs, ";
                  else str = "Lower PMs, ";
                  str += MW->PMs->getType(ityp)->name + " PM type";
    //              str += "  ("+QString::number(ityp) + ")";
                  int check = MW->PMs->definePMgroup(str);
                  if (check != ThisGroup) qWarning("Error while assigning PM group number!");
                  ThisGroup++;
                }
            }
        }
    }
  else
    {
      //irregular array or GDML file was loaded
      QList<int> AllPMs;
      AllPMs.append(0);

      for (int ipm=1; ipm<MW->PMs->count(); ipm++)
        {
          for (int i=0; i<AllPMs.size(); i++)
            {
              if (MW->PMs->at(ipm).z > MW->PMs->at(i).z )
                {
                  AllPMs.insert(i, ipm);
                  break;
                }
              if (i == AllPMs.size())
                {
                  //it is the smallest Z
                  AllPMs.append(ipm);
                }
            }
        }

      //AllPMs contai list of all PMs sorted by decreasing Z
      //splitting in inddependent list for each PM type
      QVector< QList<int> > PMsByType(PMtypes);
      for (int ityp=0; ityp<PMtypes; ityp++)
        {
          for (int index = 0; index < AllPMs.size(); index++)
            {
                int ipm = AllPMs[index];
                if (MW->PMs->at(ipm).type == ityp) PMsByType[ityp].append(ipm);
            }
        }

      int lastGroup = 0;
      //now all PMs are sorted by type and Z
      for (int ityp = 0; ityp<PMtypes; ityp++)
        {
          double lastZ = 1e10;
          for (int index = 0; index < PMsByType[ityp].size(); index++)
            {
              int ipm = PMsByType[ityp][index];
              int thisZ = MW->PMs->at(ipm).z;
              if (thisZ != lastZ)
                {
                  //new group
                  lastGroup = MW->PMs->definePMgroup("Z="+QString::number(thisZ)+" "+MW->PMs->getType(ityp)->name);
                }
              //registering group for this pm
              MW->PMs->at(ipm).group = lastGroup;
              lastZ = thisZ;
            }
        }
    }

   //updating group indication
   ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();
   //update linked indication for PM gains
   ReconstructionWindow::on_pbUpdateGainsIndication_clicked();
   //update for dynamicThreshold
   QString str = MW->PMs->getGroupDescription(0);
   ui->cobPMGroupForDynPassives->setCurrentText(str);
   ReconstructionWindow::on_cobPMGroupForDynPassives_currentIndexChanged(str);
   ReconstructionWindow::on_pbShowAllGroups_clicked();
}

void ReconstructionWindow::on_pbClearGroups_clicked()
{
  MW->PMs->clearPMgroups(); //clears gains too!
  //updating group indication
  ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();
  //update linked indication for PM gains
  ReconstructionWindow::on_pbUpdateGainsIndication_clicked();
  //update for dynamicThreshold
  QString str = MW->PMs->getGroupDescription(0);
  ui->cobPMGroupForDynPassives->setCurrentText(str);
  ReconstructionWindow::on_cobPMGroupForDynPassives_currentIndexChanged(str);
}

void ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked()
{
  ReconstructionWindow::UpdatePMgroupIndication();
}

void ReconstructionWindow::UpdatePMgroupIndication()
{
   ReconstructionWindow::UpdatePMGroupCOBs();

   ui->lwPMgroups->clear();
   int groups = MW->PMs->countPMgroups();

   if (groups < 2)
     {
       ui->fPassiveTresholdGroup->setVisible(false);
       ui->fGroupsInGains->setVisible(false);
     }
   else
     {
       ui->fPassiveTresholdGroup->setVisible(true);
       ui->fGroupsInGains->setVisible(true);
     }

//   bool Warning = false;
//   if (groups == 1) ui->lwPMgroups->addItem("-- No groups were defined --");
//   else
     {
       QString str;

       for (int ig = 0; ig<groups; ig++)
         {
           str = QString::number(ig)+"> ";
           str += MW->PMs->getGroupDescription(ig) + "  ";

           int counter = 0;
           for (int ipm = 0; ipm<MW->PMs->count(); ipm++)
               if (MW->PMs->at(ipm).group == ig) counter++;

           str += "( " + QString::number(counter) + " PMs)";
//           if (ig == 0 && counter == 0) ;
//           else
             {
               ui->lwPMgroups->addItem(str);
//               if (ig == 0)
//                 {
//                   ui->lwPMgroups->item(ui->lwPMgroups->count()-1)->setForeground(Qt::red);
//                   Warning = true;
//                 }
             }        
         }
     }    
}

void ReconstructionWindow::on_pbUpdateGainsIndication_clicked()
{
  if (!MW->PMs) return;  //on startup

  int numGroups = MW->PMs->countPMgroups();
  bool flag;
  if ( numGroups < 2) flag = false;
  else flag = true;

  ui->fGroupsInGains->setVisible(flag);
  ui->labForGroups->setVisible(flag);

  if (numGroups > 1) ui->pbShowAllGainsForGroup->setText("Show gains\nfor this group");
  else ui->pbShowAllGainsForGroup->setText("Show gains");

  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);

  //overrides
  int icurrent = -1;
  if (ui->leiPMnumberForGains->text() != "")
    {
      icurrent = ui->leiPMnumberForGains->text().toInt();
      if (MW->PMs->at(icurrent).group != igroup) icurrent = -1;
    }

  if (icurrent == -1)
    {
      bool found = false;
      int ifound = 0;
      for (int i=0; i<MW->PMs->count(); i++)
        if (MW->PMs->at(i).group == igroup)
          {
            found = true;
            ifound = i;
            break;
          }
      if (found)
        {
          ui->leiPMnumberForGains->setText(QString::number(ifound));
          ui->ledGain->setText(QString::number(MW->PMs->at(ifound).relGain));
        }
    }
}

void ReconstructionWindow::on_leiPMnumberForGains_editingFinished()
{
    if (ui->leiPMnumberForGains->text() == "")
      {
        ui->ledGain->setText("");
        return;
      }

    int ipm = ui->leiPMnumberForGains->text().toInt();

    if (ipm>MW->PMs->count()-1)
      {
        ui->leiPMnumberForGains->setText("");
        message("PM with this number does not exist in this detector!", this);
        return;
      }

    int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);
 //   int numGroups = MW->PMs->countPMgroups();
    //overrides
 //   if (numGroups == 1) igroup = 0;
 //   if (numGroups == 2) igroup = 1;
    if (MW->PMs->at(ipm).group != igroup)
      {
        ui->leiPMnumberForGains->setText("");
        message("This PM does not belong to PM group "+QString::number(igroup), this);
        return;
      }

    double gain = MW->PMs->at(ipm).relGain;
//    qDebug()<<"gain------:"<<gain;
    ui->ledGain->setText( QString::number(gain) );
}

void ReconstructionWindow::on_ledGain_editingFinished()
{
   if (ui->leiPMnumberForGains->text() == "") return;

   int ipm = ui->leiPMnumberForGains->text().toInt();
   int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);
//   int numGroups = MW->PMs->countPMgroups();
   //overrides
//   if (numGroups == 1) igroup = 0;
//   if (numGroups == 2) igroup = 1;

   if (ipm>MW->PMs->count()-1)
     {
       ui->leiPMnumberForGains->setText("");
       message("PM with this number does not exist in this detector!", this);
       return;
     }

   if (MW->PMs->at(ipm).group != igroup)
     {
       ui->leiPMnumberForGains->setText("");
       message("This PM does not belong to PM group "+QString::number(igroup), this);
       ui->leiPMnumberForGains->setText("");
       return;
     }

   double newgain = ui->ledGain->text().toDouble();
   if (newgain == 0)
     {
       double oldGain = MW->PMs->at(ipm).relGain;
       ui->ledGain->setText(QString::number(oldGain));
       message("Gain cannot be zero!", this);
       return;
     }

   MW->PMs->at(ipm).relGain = newgain;

//   qDebug()<<"new value check:"<<MW->PMs->getGain(ipm);

   ui->pbFindNextPMinGains->setFocus();
}

void ReconstructionWindow::on_pbShowAllGroups_clicked()
{
   QVector<QString> tmp(0);
   for (int i=0; i<MW->PMs->count(); i++) tmp.append( QString::number( MW->PMs->at(i).group ) );

   MW->ShowTextOnPMs(tmp, kRed);
}

void ReconstructionWindow::on_pbShowAllGainsForGroup_clicked()
{
  QVector<QString> tmp(0);
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);
//  int numGroups = MW->PMs->countPMgroups();
  //overrides
//  if (numGroups == 1) igroup = 0;
//  if (numGroups == 2) igroup = 1;

  for (int i=0; i<MW->PMs->count(); i++)
    {
      QString str;
      if (MW->PMs->at(i).group != igroup) str = "";
      else
        {
          double gain = MW->PMs->at(i).relGain;
          str = QString::number( gain,'g', 4);
        }

      tmp.append( str );
    }

  MW->ShowTextOnPMs(tmp, kBlue);
}

void ReconstructionWindow::on_pbGainsToUnity_clicked()
{
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);
 // int numGroups = MW->PMs->countPMgroups();
  //overrides
//  if (numGroups == 1) igroup = 0;
//  if (numGroups == 2) igroup = 1;

  double val = ui->ledGainsValue->text().toDouble();

  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    if (MW->PMs->at(ipm).group == igroup) MW->PMs->at(ipm).relGain = val;

  ui->ledGain->setText(ui->ledGainsValue->text());
  ReconstructionWindow::on_pbShowAllGainsForGroup_clicked();
}

void ReconstructionWindow::on_pbGainsMultiply_clicked()
{
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);
//  int numGroups = MW->PMs->countPMgroups();
  //overrides
//  if (numGroups == 1) igroup = 0;
//  if (numGroups == 2) igroup = 1;

  double val = ui->ledGainsValue->text().toDouble();

  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    if (MW->PMs->at(ipm).group == igroup) MW->PMs->at(ipm).relGain *= val;

  ui->ledGain->setText( QString::number(MW->PMs->at( ui->leiPMnumberForGains->text().toInt() ).relGain) );
  ReconstructionWindow::on_pbShowAllGainsForGroup_clicked();
}

void ReconstructionWindow::on_pbFindNextPMinGains_clicked()
{
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobGroupInGains);
//  int numGroups = MW->PMs->countPMgroups();
  //overrides
//  if (numGroups == 1) igroup = 0;
  //if (numGroups == 2) igroup = 1;

  int numPMs = MW->PMs->count();

   QString str = ui->leiPMnumberForGains->text();
   int ipmStart = numPMs-1; //to start from 0 if empty text
   if (str != "") ipmStart = str.toInt();

   int ipm = ipmStart;
   bool found = false;
   do
     {
       ipm++;
       if (ipm > numPMs-1) ipm = 0;

       if (MW->PMs->at(ipm).group == igroup) found = true;
     }
   while (!found && ipm != ipmStart);

   if (!found)
     {
       message("There are no PMs belonging to this group!", this);
       return;
     }

   ui->leiPMnumberForGains->setText( QString::number(ipm) );
   ui->ledGain->setText( QString::number( MW->PMs->at(ipm).relGain ) );
}

void ReconstructionWindow::on_pbSaveGains_clicked()
{
    QString fileName = QFileDialog::getSaveFileName(this,
                                                    "Save gains", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*.*)");
    if (fileName.isEmpty()) return;
    MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

    QFileInfo file(fileName);
    if(file.suffix().isEmpty()) fileName += ".dat";

    MW->SaveGains(fileName);
}

void ReconstructionWindow::on_pbLoadGains_clicked()
{
  QString fileName = QFileDialog::getOpenFileName(this, "Load gains", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt);;All files (*.*)");
//  qDebug()<<fileName;
  if (fileName.isEmpty()) return;
  MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

  MW->LoadGains(fileName);

  ReconstructionWindow::on_pbShowAllGainsForGroup_clicked();
}

void ReconstructionWindow::on_twOptions_currentChanged(int)
{
    if (!MW->PMs) return; //on start
    ReconstructionWindow::on_pbUpdateGainsIndication_clicked();
    ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();
}

void ReconstructionWindow::on_pbAssignPMtoGroup_clicked()
{
  int numPMs = MW->PMs->count();
  QList<int> ToAdd;
  bool ok = ExtractNumbersFromQString(ui->lePMsToAssign->text(), &ToAdd);
  if (!ok)
    {
      message("Input error!", this);
      return;
    }

  for (int i=0; i<ToAdd.size(); i++)
    {
      if (ToAdd[i]>numPMs-1)
        {
          message("PM number range error!", this);
          return;
        }
    }

//  int numGroups = MW->PMs->countPMgroups();
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobAssignToPMgroup);
/*
  if (igroup<0 || igroup>numGroups-1)
    {
      message("This groups is not defined anymore!", this);
      ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();
      return;
    }
*/

  for (int i=0; i<ToAdd.size(); i++) MW->PMs->at(ToAdd[i]).group = igroup;

  ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();

  //ui->cobAssignToPMgroup->setCurrentIndex(igroup-1);

  //ReconstructionWindow::on_pbShowAllGroups_clicked();
}

void ReconstructionWindow::on_pbDefineNewGroup_clicked()
{
    QString name = ui->leCustomGroupName->text();
    int groups = MW->PMs->countPMgroups();
    for (int i=0; i<groups; i++)
      {
        if (MW->PMs->getGroupDescription(i) == name)
          {
            message("Group with tis name was already defined!", this);
            return;
          }
      }
    MW->PMs->definePMgroup(name);
    //updating group indication
    ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();
    //update linked indication for PM gains
    ReconstructionWindow::on_pbUpdateGainsIndication_clicked();
}

void ReconstructionWindow::on_pbShowVsXY_clicked()
{
  if (EventsDataHub->ReconstructionData.isEmpty()) return; //just in case, empty should not happen at all
  if (!EventsDataHub->isReconstructionDataReady())
    {
      message("Data reconstruction was not performed yet!", this);
      return;
    }

  QString strIn = ui->cobShowWhat->currentText();
  if (strIn == "Delta rec-actual XY" && EventsDataHub->isScanEmpty())
    {
       message("There are no scan data!", this);
       return;
    }
  if (ui->cbPlotVsActualPosition->isChecked() && EventsDataHub->isScanEmpty())
    {
       qWarning()<<"Attempted to use scan data with loaded/experimental data! Un-checking control";
       ui->cbPlotVsActualPosition->setChecked(false);
    }
  ReconstructionWindow::ShowVsXY(strIn);
  LastDistributionShown = "ShowVsXY";
}

void ReconstructionWindow::DoPlotXY(int i)
{
    if (i<0 || i > ui->cobShowWhat->count()-1) return;
    ui->cobShowWhat->setCurrentIndex(i);
    ReconstructionWindow::on_pbShowVsXY_clicked();
}

void ReconstructionWindow::on_cbPlotVsActualPosition_toggled(bool)
{
    if (LastDistributionShown == "ShowVsXY") ReconstructionWindow::on_pbShowVsXY_clicked();
}

void ReconstructionWindow::on_cb3Dreconstruction_toggled(bool checked)
{
    if (checked)
      {
        ui->cbForceCoGgiveZof->setEnabled(true);
      }
    else
      {
        ui->cbForceCoGgiveZof->setEnabled(false);
        ui->cbForceCoGgiveZof->setChecked(true);
      }

    ui->fCoGStretchZ->setEnabled( ui->cb3Dreconstruction->isChecked() && !ui->cbForceCoGgiveZof->isChecked() );
    on_cobZ_currentIndexChanged(ui->cobZ->currentIndex()); //to update cob with Z strategy
}

bool ReconstructionWindow::startXextraction()
{
   MW->WindowNavigator->BusyOn();
   MW->GraphWindow->ExtractX();
   return ReconstructionWindow::Extraction();
}

bool ReconstructionWindow::start2DLineExtraction()
{
    MW->WindowNavigator->BusyOn();
    MW->GraphWindow->Extract2DLine();
    return ReconstructionWindow::Extraction();
}

bool ReconstructionWindow::start2DEllipseExtraction()
{
    MW->WindowNavigator->BusyOn();
    MW->GraphWindow->Extract2DEllipse();
    return ReconstructionWindow::Extraction();
}

void ReconstructionWindow::on_pbDefSumCutOffs_clicked()
{
   if (EventsDataHub->isEmpty()) return;
   int totNumEvents = EventsDataHub->Events.size();
   if ( totNumEvents == 0) return;

   ReconstructionWindow::on_pbShowSumSignal_clicked();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledFilterSumMin->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   //ReconstructionWindow::on_ledFilterSumMin_editingFinished();
   ReconstructionWindow::on_pbUpdateFilters_clicked();
   ReconstructionWindow::on_pbShowSumSignal_clicked();
}

void ReconstructionWindow::on_pbDefSumCutOffs2_clicked()
{   
   if (EventsDataHub->isEmpty()) return;
   int totNumEvents = EventsDataHub->Events.size();
   if ( totNumEvents == 0) return;

   ReconstructionWindow::on_pbShowSumSignal_clicked();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledFilterSumMax->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   //ReconstructionWindow::on_ledFilterSumMax_editingFinished();
   ReconstructionWindow::on_pbUpdateFilters_clicked();
   ReconstructionWindow::on_pbShowSumSignal_clicked();
}

void ReconstructionWindow::on_pbDefCutOffMin_clicked()
{   
   if (EventsDataHub->isEmpty()) return;
   int totNumEvents = EventsDataHub->Events.size();
   if ( totNumEvents == 0) return;
   ReconstructionWindow::on_pbShowIndividualSpectrum_clicked();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledFilterIndividualMin->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   ReconstructionWindow::on_ledFilterIndividualMin_editingFinished();
   ReconstructionWindow::on_pbShowIndividualSpectrum_clicked();
}

void ReconstructionWindow::on_pbDefCutOffMax_clicked()
{   
   if (EventsDataHub->isEmpty()) return;
   int totNumEvents = EventsDataHub->Events.size();
   if ( totNumEvents == 0) return;

   ReconstructionWindow::on_pbShowIndividualSpectrum_clicked();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledFilterIndividualMax->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   ReconstructionWindow::on_ledFilterIndividualMax_editingFinished();
   ReconstructionWindow::on_pbShowIndividualSpectrum_clicked();
}

void ReconstructionWindow::on_pbCutOffsNextPM_clicked()
{
   ui->sbPMsignalIndividual->setValue(ui->sbPMsignalIndividual->value()+1);
}

void ReconstructionWindow::on_pbDefEnMin_clicked()
{
   if (EventsDataHub->ReconstructionData.isEmpty())
   {
      message("Reconstruction data are empty!", this);
      ui->bsAnalyzeScan->setCurrentIndex(1);
      return;
   }

   ReconstructionWindow::ShowEnergySpectrum();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledEnergyMin->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   ReconstructionWindow::on_pbUpdateFilters_clicked();
   ReconstructionWindow::ShowEnergySpectrum();
}

void ReconstructionWindow::on_pbDefEnMax_clicked()
{
   if (EventsDataHub->ReconstructionData.isEmpty())
   {
     message("Reconstruction data are empty!", this);
     ui->bsAnalyzeScan->setCurrentIndex(1);
     return;
   }

   ReconstructionWindow::ShowEnergySpectrum();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledEnergyMax->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   ReconstructionWindow::on_pbUpdateFilters_clicked();
   ReconstructionWindow::ShowEnergySpectrum();
}

void ReconstructionWindow::on_pbDefChi2Min_clicked()
{
   if (EventsDataHub->ReconstructionData.isEmpty())
   {
     message("Reconstruction data are empty!", this);
     ui->bsAnalyzeScan->setCurrentIndex(1);
     return;
   }

   ReconstructionWindow::ShowChi2Spectrum();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledChi2Min->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   ReconstructionWindow::on_pbUpdateFilters_clicked();
   ReconstructionWindow::ShowChi2Spectrum();
}

void ReconstructionWindow::on_pbDefChi2Max_clicked()
{
   if (EventsDataHub->ReconstructionData.isEmpty())
   {
     message("Reconstruction data are empty!", this);
     ui->bsAnalyzeScan->setCurrentIndex(1);
     return;
   }

   ReconstructionWindow::ShowChi2Spectrum();

   bool ok = ReconstructionWindow::startXextraction();
   if (!ok) return;

   ui->ledChi2Max->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
   ReconstructionWindow::on_pbUpdateFilters_clicked();
   ReconstructionWindow::ShowChi2Spectrum();
}

void ReconstructionWindow::on_pbShowCorr_clicked()
{
    //are there data?
    if (EventsDataHub->isEmpty())
     {
        message("No events!", this);
        return;
     }

    const QString X = tmpCorrFilter.getCorrelationUnitX()->getType();
    const QString Y = tmpCorrFilter.getCorrelationUnitY()->getType();
//    qDebug()<<"--------------------"<<X<<Y;

    bool GoodEventsOnly = true;
    if (ui->cbCorrIgnoreAllFilters->isChecked()) GoodEventsOnly = false;

    //with some exceptions, reconstruction data should be present
    if ( !tmpCorrFilter.getCorrelationUnitX()->isRequireReconstruction() && !tmpCorrFilter.getCorrelationUnitY()->isRequireReconstruction() )
      {
        //for these X and Y we do not need reconstruction data if all filters are ignored
        if (EventsDataHub->ReconstructionData.isEmpty())
         {
           //forcing to ignore all filters - in this way we do not ask good/bad event and reconstruction is not needed
           GoodEventsOnly = false;
         }
      }
    else
      {
        if (EventsDataHub->ReconstructionData.isEmpty())
         {
           message("Reconstruction data are empty!", this);
           ui->bsAnalyzeScan->setCurrentIndex(1);
           return;
         }
      }

/*
    //debug
    QString str="X: ";
    for (int i=0; i<tmpCorrFilter.getCorrelationUnitX()->Channels.size(); i++) str+= QString::number(tmpCorrFilter.getCorrelationUnitX()->Channels[i])+" ";
    str+="\nY: ";
    for (int i=0; i<tmpCorrFilter.getCorrelationUnitY()->Channels.size(); i++) str+= QString::number(tmpCorrFilter.getCorrelationUnitY()->Channels[i])+" ";
    qDebug()<<str;
*/

    //=====MAIN CYCLE OVER ALL EVENTS=====
    QVector<double> x;
    QVector<double> y;

    qDebug()<<"Starting main cycle.  GoodEventsOnly="<<GoodEventsOnly;
    for (int iev=0; iev <EventsDataHub->Events.size(); iev++)
      {
        if (GoodEventsOnly)
          if (!EventsDataHub->ReconstructionData[iev]->GoodEvent) continue;

        //populating X
        x.append(tmpCorrFilter.getCorrelationUnitX()->getValue(iev));

        //populating Y
        y.append(tmpCorrFilter.getCorrelationUnitY()->getValue(iev));

      }
    //=====================================

    if (x.isEmpty())
      {
        message("There are no events which pass all the defined filters!", this);
        return;
      }

    ReconstructionWindow::showCorrelation("Correlation", tmpCorrFilter.getCorrelationUnitX()->getAxisTitle(), tmpCorrFilter.getCorrelationUnitY()->getAxisTitle(), &x, &y);
}

void ReconstructionWindow::showCorrelation(const char* histTitle, const char* xTitle, const char* yTitle, QVector<double>* x, QVector<double>* y)
{
  qDebug()<<"titles:"<<xTitle<<yTitle;
  if (histCorr) delete histCorr;

  int xbins = ui->sbCorrBinsX->value();
  int ybins = ui->sbCorrBinsY->value();
  double xmin = 0;
  double xmax = 0;
  double ymin = 0;
  double ymax = 0;
  if (!ui->cbCorrAutoSize->isChecked())
    {
      xmin = ui->ledCorrXmin->text().toDouble();
      xmax = ui->ledCorrXmax->text().toDouble();
      ymin = ui->ledCorrYmin->text().toDouble();
      ymax = ui->ledCorrYmax->text().toDouble();
    }

  histCorr = new TH2D("histCorrelation", histTitle, xbins, xmin, xmax,
                                                    ybins, ymin, ymax);


  qDebug()<<"setting titles";
  histCorr->GetXaxis()->SetTitle(xTitle);
  histCorr->GetYaxis()->SetTitle(yTitle);
  qDebug()<<"done!";

  if (ui->cbCorrAutoBin->isChecked()) histCorr->SetBit(TH2::kCanRebin);

  //filling histogam
  for (int i=0; i<x->size(); i++) histCorr->Fill(x->at(i), y->at(i));

  //drawing data 
  MW->GraphWindow->Draw(histCorr, "colz");
  LastDistributionShown = "Correlation";

  //drawing cuts if needed
  ReconstructionWindow::DrawCuts(); 
}

void ReconstructionWindow::DrawCuts()
{
   if (!ui->cbCorrShowCut->isChecked()) return;
   if (!ui->cbCorrShowCut->isEnabled()) return;

   if (!MW->GraphWindow) return;
   if (LastDistributionShown != "Correlation") return;

   qDebug()<<"Drawing cuts...";
   QList<double> tmp = tmpCorrFilter.getCut()->Data;

   qDebug()<<"CutType: "<<tmpCorrFilter.getCut()->getType();
   if (tmpCorrFilter.getCut()->getType() == "line")
         {
            //finding intersection points
              //trying left and right borders
            double point[2];
            QVector<double> PointX(0);
            QVector<double> PointY(0);

            double minX = MW->GraphWindow->getCanvasMinX();
            double maxX = MW->GraphWindow->getCanvasMaxX();
            double minY = MW->GraphWindow->getCanvasMinY();
            double maxY = MW->GraphWindow->getCanvasMaxY();

            qDebug()<<"Reported canvas limits: "<<minX<<maxX<<minY<<maxY;

            qDebug()<<"Data from cut:"<<tmp[0]<<tmp[1]<< tmp[2];

            bool OK  = IntersectionOfTwoLines(tmp[0], tmp[1], tmp[2], 1.0, 0, minX, point);
            if (OK)
              if (point[1]>=minY && point[1]<=maxY)
                {
                  PointX.append(point[0]);
                  PointY.append(point[1]);
                }

            OK  = IntersectionOfTwoLines(tmp[0], tmp[1], tmp[2], 0, 1.0, maxY, point);
            if (OK)
              if (point[0]>=minX && point[0]<=maxX)
                {
                  PointX.append(point[0]);
                  PointY.append(point[1]);
                }

            OK  = IntersectionOfTwoLines(tmp[0], tmp[1], tmp[2], 1.0, 0, maxX, point);
            if (OK)
              if (point[1]>=minY && point[1]<=maxY)
                {
                  PointX.append(point[0]);
                  PointY.append(point[1]);
                }

            OK  = IntersectionOfTwoLines(tmp[0], tmp[1], tmp[2], 0, 1.0, minY, point);
            if (OK)
              if (point[0]>=minX && point[0]<=maxX)
                {
                  PointX.append(point[0]);
                  PointY.append(point[1]);
                }

            ReconstructionWindow::HideCutShapes();
            CorrCutLine->SetX1(PointX[0]);
            CorrCutLine->SetY1(PointY[0]);
            CorrCutLine->SetX2(PointX[1]);
            CorrCutLine->SetY2(PointY[1]);
            CorrCutLine->SetLineColor(kRed);
            CorrCutLine->SetLineStyle(2);
            //CorrCutLine->Draw();
            MW->GraphWindow->Draw(CorrCutLine, "same");
         }

   if (tmpCorrFilter.getCut()->getType() == "ellipse")
         {
           ReconstructionWindow::HideCutShapes();
           CorrCutEllipse->SetX1(tmp[0]);
           CorrCutEllipse->SetY1(tmp[1]);
           CorrCutEllipse->SetR1(tmp[2]);
           CorrCutEllipse->SetR2(tmp[3]);
           CorrCutEllipse->SetTheta(tmp[4]);
           CorrCutEllipse->SetFillStyle(4000);
           CorrCutEllipse->SetLineColor(kRed);
           CorrCutEllipse->SetLineStyle(2);
           CorrCutEllipse->Draw();
         }

   if (tmpCorrFilter.getCut()->getType() == "polygon")
         {
//       qDebug()<<"-------------------";
//       qDebug()<<"tmpCorrFilter->DataLength:"<<tmpCorrFilter.getCut()->Data.length();
//       for (int i=0; i<tmpCorrFilter.getCut()->Data.length();i++) qDebug()<<tmpCorrFilter.getCut()->Data[i];

           ReconstructionWindow::HideCutShapes();

           CorrCutPolygon->SetPolyLine(0);
           for (int i=0; i<tmpCorrFilter.getCut()->Data.length()/2; i++)
             {
               CorrCutPolygon->SetPoint(i, tmpCorrFilter.getCut()->Data[i*2], tmpCorrFilter.getCut()->Data[i*2+1]);
//               qDebug()<<"Poly point: "<<tmpCorrFilter.getCut()->Data[i*2]<< tmpCorrFilter.getCut()->Data[i*2+1];
             }
           CorrCutPolygon->SetFillStyle(4000);
           CorrCutPolygon->SetLineColor(kRed);
           CorrCutPolygon->SetLineStyle(2);
           CorrCutPolygon->Draw();
         }

  //MW->graphRW->UpdateRootCanvas();
  MW->GraphWindow->UpdateRootCanvas();
}

void ReconstructionWindow::HideCutShapes()
{
    return;

    CorrCutLine->SetLineColor(0);
    CorrCutLine->Paint();
    CorrCutEllipse->SetLineColor(0);
    CorrCutEllipse->Paint();
    CorrCutPolygon->SetLineColor(0);
    CorrCutPolygon->Paint();
}

void ReconstructionWindow::on_cbCorrAutoBin_toggled(bool)
{
   // ui->fCorrBins->setEnabled(!checked);
}

void ReconstructionWindow::on_cbCorrAutoSize_toggled(bool checked)
{
    ui->fCorrSize->setEnabled(!checked);
}

void ReconstructionWindow::on_leCorrList1_editingFinished()
{
   if (TMPignore) return;
   QList<int> tmp;
   bool ok = ExtractNumbersFromQString(ui->leCorrList1->text(), &tmp);
   if (!ok)
    {
          TMPignore = true;
          message("Input error!", this);
          ui->leCorrList1->setFocus();
          ui->leCorrList1->selectAll();
          TMPignore = false;
          return;
    }

   for (int i=0; i<tmp.size(); i++)
        if (tmp[i]<0 || tmp[i]>MW->PMs->count()-1)
          {
            TMPignore = true;
            message("PM number is out of range", this);
            ui->leCorrList1->setFocus();
            ui->leCorrList1->selectAll();
            TMPignore = false;
            return;
          }

   ReconstructionWindow::on_pbCorrUpdateTMP_clicked();
}

void ReconstructionWindow::on_leCorrList2_editingFinished()
{
  qDebug()<<"-----------ignore?"<<TMPignore;
  if (TMPignore) return;

  QList<int> tmp;
  bool ok = ExtractNumbersFromQString(ui->leCorrList2->text(), &tmp);
  if (!ok)
   {
         TMPignore = true;
         message("Input error!", this);
         ui->leCorrList2->setFocus();
         ui->leCorrList2->selectAll();
         TMPignore = false;
         return;
   }

  for (int i=0; i<tmp.size(); i++)
       if (tmp[i]<0 || tmp[i]>MW->PMs->count()-1)
         {
           TMPignore = true;
           message("PM number is out of range", this);
           ui->leCorrList2->setFocus();
           ui->leCorrList2->selectAll();
           TMPignore = false;
           return;
         }

   ReconstructionWindow::on_pbCorrUpdateTMP_clicked();
}

void ReconstructionWindow::on_cobCorr1_currentIndexChanged(int index)
{
  int i;
  switch (index)
    {
    case 0:
      i=1;
      break;
    case 1:
      i=0;
      break;
    case 2:
      i=2;
      break;
    default:
      i=0;
    }

  ui->stwCorr1->setCurrentIndex(i);  //0 - empty, 1 - spin, 2 -lineEdit
}

void ReconstructionWindow::on_cobCorr2_currentIndexChanged(int index)
{
  int i;
  switch (index)
    {
    case 0:
      i=1;
      break;
    case 1:
      i=0;
      break;
    case 2:
      i=2;
      break;
    default:
      i=0;
    }

  ui->stwCorr2->setCurrentIndex(i);
}

void ReconstructionWindow::on_pbCorrUpdateTMP_clicked()
{
   if (TMPignore) return;

//   qDebug()<<"Updating tmpCorrFilter from UI";

   //ui
   ui->stwCorrCutType->setCurrentIndex(ui->cobCorrCutType->currentIndex());
   ui->pbCorrAccept->setIcon(RedIcon);

   //updating tmpCorrFilter
   CorrelationUnitGenericClass* p;
   //correlation units
     //X
   if (ui->cobCorr1->currentText() == "Signal: one channel")
     {
       CU_SingleChannel* tmp = new CU_SingleChannel();
       tmp->Channels << ui->sbCorrSingle1->value();
       p = tmp;
     }

   if (ui->cobCorr1->currentText() == "Signal: sum ALL channels")
     {
       CU_SumAllChannels* tmp = new CU_SumAllChannels();
       for (int i=0; i<MW->PMs->count(); i++) tmp->Channels << i;
       p = tmp;
     }

   if (ui->cobCorr1->currentText() == "Signal: sum channels")
     {
       CU_SumChannels* tmp = new CU_SumChannels();
       QList<int> list;
       ExtractNumbersFromQString(ui->leCorrList1->text(), &list);
       tmp->Channels = list;
       p = tmp;
     }

   if (ui->cobCorr1->currentText() == "Loaded energy")
     {
       CU_LoadedEnergy* tmp = new CU_LoadedEnergy();
       p = tmp;
     }

   if (ui->cobCorr1->currentText() == "Reconstructed energy")
     {
       CU_RecE* tmp = new CU_RecE();
       p = tmp;
     }

   if (ui->cobCorr1->currentText() == "Chi2")
     {
       CU_Chi2* tmp = new CU_Chi2();
       p = tmp;
     }

   if (ui->cobCorr1->currentText() == "Reconstructed X")
     {
       CU_RecX* tmp = new CU_RecX();
       p = tmp;
     }
   if (ui->cobCorr1->currentText() == "Reconstructed Y")
     {
       CU_RecY* tmp = new CU_RecY();
       p = tmp;
     }
   if (ui->cobCorr1->currentText() == "Reconstructed Z")
     {
       CU_RecZ* tmp = new CU_RecZ();
       p = tmp;//dynamic_cast<CorrelationUnitGenericStructure*> tmp;
     }

   //p->Events = &MW->Reconstructor->Events;
   p->EventsDataHub = MW->EventsDataHub;
   //p->Gains = MW->PMs->getGains();
   //p->PMs = &MW->PMs;
   p->Detector = MW->Detector;
   //p->ReconstructionData = &EventsDataHub->ReconstructionData;
   tmpCorrFilter.setCorrelationUnitX(p);



     //Y
   if (ui->cobCorr2->currentText() == "Signal: one channel")
     {
       CU_SingleChannel* tmp = new CU_SingleChannel();
       tmp->Channels << ui->sbCorrSingle2->value();
       p = tmp;
     }

   if (ui->cobCorr2->currentText() == "Signal: sum ALL channels")
     {
       CU_SumAllChannels* tmp = new CU_SumAllChannels();
       for (int i=0; i<MW->PMs->count(); i++) tmp->Channels << i;
       p = tmp;
     }

   if (ui->cobCorr2->currentText() == "Signal: sum channels")
     {
       CU_SumChannels* tmp = new CU_SumChannels();
       QList<int> list;
       ExtractNumbersFromQString(ui->leCorrList2->text(), &list);
       tmp->Channels = list;
       p = tmp;
     }

   if (ui->cobCorr2->currentText() == "Loaded energy")
     {
       CU_LoadedEnergy* tmp = new CU_LoadedEnergy();
       p = tmp;
     }

   if (ui->cobCorr2->currentText() == "Reconstructed energy")
     {
       CU_RecE* tmp = new CU_RecE();
       p = tmp;
     }

   if (ui->cobCorr2->currentText() == "Chi2")
     {
       CU_Chi2* tmp = new CU_Chi2();
       p = tmp;
     }

   if (ui->cobCorr2->currentText() == "Reconstructed X")
     {
       CU_RecX* tmp = new CU_RecX();
       p = tmp;
     }
   if (ui->cobCorr2->currentText() == "Reconstructed Y")
     {
       CU_RecY* tmp = new CU_RecY();
       p = tmp;
     }
   if (ui->cobCorr2->currentText() == "Reconstructed Z")
     {
       CU_RecZ* tmp = new CU_RecZ();
       p = tmp;
     }

   p->EventsDataHub = MW->EventsDataHub;
   //p->Gains = MW->PMs->getGains();
   //p->PMs = &MW->PMs;
   p->Detector = MW->Detector;
   tmpCorrFilter.setCorrelationUnitY(p);

   //cut
   if (ui->cobCorrCutType->currentText() == "Line")
     {
       Cut_Line* tmp = new Cut_Line();
       tmp->Data.clear();
       tmp->Data << ui->ledCorrLineA->text().toDouble() << ui->ledCorrLineB->text().toDouble() << ui->ledCorrLineC->text().toDouble();
       tmp->CutOption = ui->cobCorrLineCutOption->currentIndex();
       tmpCorrFilter.setCut(tmp);
     }

   if (ui->cobCorrCutType->currentText() == "Ellipse")
     {
       Cut_Ellipse* tmp = new Cut_Ellipse();
       tmp->Data.clear();
       tmp->Data << ui->ledCorrEllX->text().toDouble() << ui->ledCorrEllY->text().toDouble() << ui->ledCorrEllR1->text().toDouble() << ui->ledCorrEllR2->text().toDouble()<< ui->ledCorrEllAngle->text().toDouble();
       tmp->CutOption = ui->cobCorrEllCutOption->currentIndex();
       tmpCorrFilter.setCut(tmp);
     }

   if (ui->cobCorrCutType->currentText() == "Polygon")
     {
       Cut_Polygon* tmp = new Cut_Polygon();
       tmp->CutOption = ui->cobCorrPolygonCutOption->currentIndex();

       int rows = ui->tabwidCorrPolygon->rowCount();
       tmp->Data.clear();
       for (int i=0; i<rows; i++)
         {
           tmp->Data << ui->tabwidCorrPolygon->item(i,0)->text().toDouble() << ui->tabwidCorrPolygon->item(i,1)->text().toDouble();
 //          qDebug()<<"adding point to poly: "<<ui->tabwidCorrPolygon->item(i,0)->text().toDouble() << ui->tabwidCorrPolygon->item(i,1)->text().toDouble();
         }

       tmp->Data << ui->tabwidCorrPolygon->item(0,0)->text().toDouble() << ui->tabwidCorrPolygon->item(0,1)->text().toDouble(); //to have closed polygon
 //      qDebug()<<"adding last point to poly: "<<ui->tabwidCorrPolygon->item(0,0)->text().toDouble() << ui->tabwidCorrPolygon->item(0,1)->text().toDouble();
 //      qDebug()<<"Data length:"<<tmp->Data.length();
       for (int i=0; i<tmp->Data.length();i++) qDebug()<<tmp->Data[i];


       tmpCorrFilter.setCut(tmp);

 //      qDebug()<<"tmpCorrFilter->DataLength:"<<tmpCorrFilter.getCut()->Data.length();
 //      for (int i=0; i<tmpCorrFilter.getCut()->Data.length();i++) qDebug()<<tmpCorrFilter.getCut()->Data[i];
     }

    ReconstructionWindow::DrawCuts();  //safe to use - it checks for all necessary conditions

    //the rest of controls
    tmpCorrFilter.AutoBin = ui->cbCorrAutoBin->isChecked();
    tmpCorrFilter.BinX = ui->sbCorrBinsX->value();
    tmpCorrFilter.BinY = ui->sbCorrBinsY->value();
    tmpCorrFilter.AutoSize = ui->cbCorrAutoSize->isChecked();
    tmpCorrFilter.minX = ui->ledCorrXmin->text().toDouble();
    tmpCorrFilter.maxX = ui->ledCorrXmax->text().toDouble();
    tmpCorrFilter.minY = ui->ledCorrYmin->text().toDouble();
    tmpCorrFilter.maxY  = ui->ledCorrYmax->text().toDouble();
}

void ReconstructionWindow::on_pbCorrExtractLine_clicked()
{
    //are there data?
    if (EventsDataHub->isEmpty()) return;

    if ( tmpCorrFilter.getCorrelationUnitX()->isRequireReconstruction() || tmpCorrFilter.getCorrelationUnitY()->isRequireReconstruction() )
      if (EventsDataHub->ReconstructionData.isEmpty()) return;

    ReconstructionWindow::on_pbShowCorr_clicked();
    ReconstructionWindow::HideCutShapes();

    bool ok = ReconstructionWindow::start2DLineExtraction();
    if (!ok) return;

    ui->ledCorrLineA->setText(QString::number(MW->GraphWindow->extracted2DLineA(), 'g', 4));
    ui->ledCorrLineB->setText(QString::number(MW->GraphWindow->extracted2DLineB(), 'g', 4));
    ui->ledCorrLineC->setText(QString::number(MW->GraphWindow->extracted2DLineC(), 'g', 4));
    ReconstructionWindow::on_pbCorrUpdateTMP_clicked();

    ReconstructionWindow::on_pbUpdateFilters_clicked();
    ReconstructionWindow::on_pbShowCorr_clicked();
}

void ReconstructionWindow::on_sbCorrFNumber_valueChanged(int arg1)
{
//   qDebug()<<"-------------- ===Corr filter value changed! (total filters:"<<CorrelationFilters.size()<<")";
   if (arg1<0)
     {
       if (CorrelationFilters.isEmpty())
         {
           ui->sbCorrFNumber->setValue(0);
           return;
         }
       ui->sbCorrFNumber->setValue(CorrelationFilters.size()-1);
       return;
     }

   if (arg1 == 0  && CorrelationFilters.size() == 0) return; //protection just in case

   if (arg1 > CorrelationFilters.size()-1)
     {
       ui->sbCorrFNumber->setValue(0);
       return;
     }

   //updating tmpCorrFilter
//   qDebug()<<"Updating tmpCorrFilter from CorrFilters Qvector, index="<<arg1;
   tmpCorrFilter.copyFrom(CorrelationFilters[arg1]);   
   ReconstructionWindow::FillUiFromTmpCorrFilter();

   //updating "Active indication"
   ui->cbCorrActive->setChecked(CorrelationFilters[arg1]->Active);
//   qDebug()<<"active?----------"<<CorrelationFilters[arg1]->Active;

}

void ReconstructionWindow::FillUiFromTmpCorrFilter()
{
//   qDebug()<<"========== starting ui update using tmpCorrFilter object";
//   qDebug()<<"========== CorrUnit1 type:"<<tmpCorrFilter.getCorrelationUnitX()->getType();
//   qDebug()<<"========== CorrUnit2 type:"<<tmpCorrFilter.getCorrelationUnitY()->getType();
//   qDebug()<<"========== Cut type:"<<tmpCorrFilter.getCut()->getType();

   TMPignore = true; //to forbid cross-update

   ui->cbCorrAutoBin->setChecked(tmpCorrFilter.AutoBin);
   ui->sbCorrBinsX->setValue(tmpCorrFilter.BinX);
   ui->sbCorrBinsY->setValue(tmpCorrFilter.BinY);
   ui->cbCorrAutoSize->setChecked(tmpCorrFilter.AutoSize);
   ui->ledCorrXmin->setText(QString::number(tmpCorrFilter.minX));
   ui->ledCorrXmax->setText(QString::number(tmpCorrFilter.maxX));
   ui->ledCorrYmin->setText(QString::number(tmpCorrFilter.minY));
   ui->ledCorrYmax->setText(QString::number(tmpCorrFilter.maxY));

   int Xindex = tmpCorrFilter.getCorrelationUnitX()->getCOBindex();
   ui->cobCorr1->setCurrentIndex(Xindex);
   int Yindex = tmpCorrFilter.getCorrelationUnitY()->getCOBindex();
   ui->cobCorr2->setCurrentIndex(Yindex);

   if (Xindex == 0) ui->sbCorrSingle1->setValue(tmpCorrFilter.getCorrelationUnitX()->Channels[0]);
   if (Yindex == 0) ui->sbCorrSingle2->setValue(tmpCorrFilter.getCorrelationUnitY()->Channels[0]);

   if (Xindex == 2)
     {       
       QString str = "";
       for (int i=0; i<tmpCorrFilter.getCorrelationUnitX()->Channels.size(); i++)
           str += QString::number(tmpCorrFilter.getCorrelationUnitX()->Channels[i])+",";
       if (str.endsWith(',')) str.chop(1);
       ui->leCorrList1->setText(str);
     }
   if (Yindex == 2)
     {
       QString str = "";
       for (int i=0; i<tmpCorrFilter.getCorrelationUnitY()->Channels.size(); i++)
           str += QString::number(tmpCorrFilter.getCorrelationUnitY()->Channels[i])+",";
       if (str.endsWith(',')) str.chop(1);
       ui->leCorrList2->setText(str);
     }

   int CutIndex = tmpCorrFilter.getCut()->getCOBindex();
   ui->cobCorrCutType->setCurrentIndex(CutIndex);

   if (CutIndex == 0)
     {
        ui->cobCorrLineCutOption->setCurrentIndex(tmpCorrFilter.getCut()->CutOption);

        ui->ledCorrLineA->setText(QString::number(tmpCorrFilter.getCut()->Data[0]));
        ui->ledCorrLineB->setText(QString::number(tmpCorrFilter.getCut()->Data[1]));
        ui->ledCorrLineC->setText(QString::number(tmpCorrFilter.getCut()->Data[2]));
     }

   if (CutIndex == 1)
     {
        ui->cobCorrEllCutOption->setCurrentIndex(tmpCorrFilter.getCut()->CutOption);

        ui->ledCorrEllX->setText(QString::number(tmpCorrFilter.getCut()->Data[0]));
        ui->ledCorrEllY->setText(QString::number(tmpCorrFilter.getCut()->Data[1]));
        ui->ledCorrEllR1->setText(QString::number(tmpCorrFilter.getCut()->Data[2]));
        ui->ledCorrEllR2->setText(QString::number(tmpCorrFilter.getCut()->Data[3]));
        ui->ledCorrEllAngle->setText(QString::number(tmpCorrFilter.getCut()->Data[4]));
     }

   if (CutIndex == 2)
     {
       ui->cobCorrPolygonCutOption->setCurrentIndex(tmpCorrFilter.getCut()->CutOption);

       ui->tabwidCorrPolygon->clearContents();
       ui->tabwidCorrPolygon->setShowGrid(false);
       int rows = tmpCorrFilter.getCut()->Data.size()/2-1; //last 2 lines: they are the first 2 lines repeated to have closed polygon

       ui->tabwidCorrPolygon->setRowCount(rows);
//       ui->tabwidCorrPolygon->setSelectionBehavior(QAbstractItemView::SelectRows);
       ui->tabwidCorrPolygon->setSelectionMode(QAbstractItemView::SingleSelection);

       for (int j=0; j<ui->tabwidCorrPolygon->rowCount(); j++)
         {
           QString str;
           str.setNum(tmpCorrFilter.getCut()->Data[j*2], 'g', 4);
           ui->tabwidCorrPolygon->setItem(j, 0, new QTableWidgetItem(str));
           str.setNum(tmpCorrFilter.getCut()->Data[j*2+1], 'g', 4);
           ui->tabwidCorrPolygon->setItem(j, 1, new QTableWidgetItem(str));
         }

       ui->tabwidCorrPolygon->resizeColumnsToContents();
       ui->tabwidCorrPolygon->resizeRowsToContents();
       //ui->tabwidCorrPolygon->setColumnWidth(0, 56);
       //ui->tabwidCorrPolygon->setColumnWidth(1, 56);
     }

   TMPignore = false;
//   qDebug()<<"========== done";
}

void ReconstructionWindow::on_pbCorrNew_clicked()
{
   CorrelationFilterStructure* NewFilter = new CorrelationFilterStructure();
   NewFilter->copyFrom(&tmpCorrFilter);
//   qDebug()<<"---------------- copied";
   CorrelationFilters.append(NewFilter);  //cannot put tmpCorrFilter directly, will get problems on copy from update
//   qDebug()<<"---------------- appended!";

   ui->sbCorrFNumber->setValue(CorrelationFilters.size()-1);
//   qDebug()<<"---------------- ui updated";
   ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::on_pbCorrAccept_clicked()
{
   int i = ui->sbCorrFNumber->value();

   bool tmpFlag = CorrelationFilters[i]->Active;
   CorrelationFilters[i]->copyFrom(&tmpCorrFilter);
   CorrelationFilters[i]->Active = tmpFlag;

   ui->pbCorrAccept->setIcon(QIcon());

   ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();   
}

void ReconstructionWindow::on_cbCorrActive_toggled(bool checked)
{
    int i = ui->sbCorrFNumber->value();
    CorrelationFilters[i]->Active = checked;    
    ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::on_pbCorrRemove_clicked()
{
   int i = ui->sbCorrFNumber->value();
   if (CorrelationFilters.size() == 0) return; //cannot remove the last filter
   delete CorrelationFilters[i] ;
   CorrelationFilters.remove(i);
   if (i != 0)
     {
       i--;
       ui->sbCorrFNumber->setValue(i); //update on_change
     }
   else
     {
       ReconstructionWindow::on_sbCorrFNumber_valueChanged(0); //to update
     }
   ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
}

void ReconstructionWindow::on_pbCorrExtractEllipse_clicked()
{
  //are there data?
  if (EventsDataHub->isEmpty()) return;

  if ( tmpCorrFilter.getCorrelationUnitX()->isRequireReconstruction() || tmpCorrFilter.getCorrelationUnitY()->isRequireReconstruction() )
    if (EventsDataHub->ReconstructionData.isEmpty()) return;

  ReconstructionWindow::on_pbShowCorr_clicked();
  ReconstructionWindow::HideCutShapes();

  bool ok = ReconstructionWindow::start2DEllipseExtraction();
  if (!ok) return;

  ui->ledCorrEllX->setText(QString::number(MW->GraphWindow->extracted2DEllipseX(), 'g', 4));
  ui->ledCorrEllY->setText(QString::number(MW->GraphWindow->extracted2DEllipseY(), 'g', 4));
  ui->ledCorrEllR1->setText(QString::number(MW->GraphWindow->extracted2DEllipseR1(), 'g', 4));
  ui->ledCorrEllR2->setText(QString::number(MW->GraphWindow->extracted2DEllipseR2(), 'g', 4));
  ui->ledCorrEllAngle->setText(QString::number(MW->GraphWindow->extracted2DEllipseTheta(), 'g', 4));
  ReconstructionWindow::on_pbCorrUpdateTMP_clicked();

  ReconstructionWindow::on_pbUpdateFilters_clicked();
  ReconstructionWindow::on_pbShowCorr_clicked();
}

void ReconstructionWindow::on_cbActivateCorrelationFilters_toggled(bool checked)
{
  ui->fCorrCut->setEnabled(checked);
  ui->fCorrAddRemoe->setEnabled(checked);
  ui->cbCorrShowCut->setEnabled(checked);
  if (!checked) ui->cbCorrShowCut->setChecked(false);

  ReconstructionWindow::on_pbUpdateFilters_clicked();
}

void ReconstructionWindow::on_pbLoadEnergySpectrum_clicked()
{
  if (!EventsDataHub->fLoadedEventsHaveEnergyInfo)
  {
      message("Loaded data does not contain energy info!", this);
      return;
  }

  if (MW->hist1) delete MW->hist1;
  MW->hist1 = new TH1D("hist1ShLoEnSp","Loaded energy spectrum", MW->GlobSet->BinsX, 0, 0);
  MW->hist1->SetXTitle("Loaded energy");
  //check filter status only of reconstruction has been done
  bool DoFiltering = false;
  if (EventsDataHub->ReconstructionData.size() > 0) DoFiltering = true;

  int ienergy = MW->PMs->count(); //index of the energy channel
//  qDebug()<<"Energy channel="<<ienergy;

  for (int i=0; i < EventsDataHub->Events.size(); i++)
    {
       if (DoFiltering)
         if (!EventsDataHub->ReconstructionData[i]->GoodEvent) continue;

       MW->hist1->Fill(EventsDataHub->Events[i][ienergy]);
    }

  MW->GraphWindow->Draw(MW->hist1);
  //showing range
  double minY = 0;
  double maxY = MW->GraphWindow->getCanvasMaxY();
  double filtMin = ui->ledLoadEnergyMin->text().toDouble();
  double filtMax = ui->ledLoadEnergyMax->text().toDouble();
  TLine* L1 = new TLine(filtMin, minY, filtMin, maxY);
  L1->SetLineColor(kGreen);
  L1->Draw();
  TLine* L2 = new TLine(filtMax, minY, filtMax, maxY);
  L2->SetLineColor(kGreen);
  L2->Draw();
  MW->GraphWindow->UpdateRootCanvas();
  LastDistributionShown = "LoadedEnergySpectrum";
}

void ReconstructionWindow::on_pbDefLoadEnMin_clicked()
{
  if (EventsDataHub->Events.isEmpty())
  {
     message("No data are loaded!", this);
     return;
  }

  ReconstructionWindow::on_pbLoadEnergySpectrum_clicked();

  bool ok = ReconstructionWindow::startXextraction();
  if (!ok) return;

  ui->ledLoadEnergyMin->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
  ReconstructionWindow::on_pbUpdateFilters_clicked();
  ReconstructionWindow::on_pbLoadEnergySpectrum_clicked();
}

void ReconstructionWindow::on_pbDefLoadEnMax_clicked()
{
  if (EventsDataHub->Events.isEmpty())
  {
     message("No data are loaded!", this);
     return;
  }

  ReconstructionWindow::on_pbLoadEnergySpectrum_clicked();

  bool ok = ReconstructionWindow::startXextraction();
  if (!ok) return;

  ui->ledLoadEnergyMax->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
  ReconstructionWindow::on_pbUpdateFilters_clicked();
  ReconstructionWindow::on_pbLoadEnergySpectrum_clicked();
}

void ReconstructionWindow::on_pbShowCorrShowRangeOnGraph_clicked()
{
  if (EventsDataHub->isEmpty()) return;

  if ( tmpCorrFilter.getCorrelationUnitX()->isRequireReconstruction() || tmpCorrFilter.getCorrelationUnitY()->isRequireReconstruction() )
    if (EventsDataHub->ReconstructionData.isEmpty()) return;

  ReconstructionWindow::on_pbShowCorr_clicked();
  ReconstructionWindow::HideCutShapes();

  bool ok = ReconstructionWindow::start2DBoxExtraction();
  if (!ok) return;

  ui->ledCorrXmin->setText(QString::number(MW->GraphWindow->extractedX1(), 'g', 4));
  ui->ledCorrXmax->setText(QString::number(MW->GraphWindow->extractedX2(), 'g', 4));
  ui->ledCorrYmin->setText(QString::number(MW->GraphWindow->extractedY1(), 'g', 4));
  ui->ledCorrYmax->setText(QString::number(MW->GraphWindow->extractedY2(), 'g', 4));

  TMPignore=true;
  ui->cbCorrAutoSize->setChecked(false);
  ui->cbCorrAutoBin->setChecked(false);
  TMPignore=false;

  ReconstructionWindow::on_pbCorrUpdateTMP_clicked();

  ReconstructionWindow::on_pbShowCorr_clicked();
}

bool ReconstructionWindow::start2DBoxExtraction()
{
    MW->WindowNavigator->BusyOn();
    MW->GraphWindow->Extract2DBox();

    return ReconstructionWindow::Extraction();
}

void ReconstructionWindow::on_pbCorr_AddLine_clicked()
{
  //add line before
    int selected = ui->tabwidCorrPolygon->currentRow();
    qDebug()<<selected;

    tmpCorrFilter.getCut()->Data.insert(selected*2, 0);
    tmpCorrFilter.getCut()->Data.insert(selected*2, 0);
    if (selected == 0)
      {
        //have to replace the last node (hidden) too
        tmpCorrFilter.getCut()->Data.replace(tmpCorrFilter.getCut()->Data.size()-1, 0);
        tmpCorrFilter.getCut()->Data.replace(tmpCorrFilter.getCut()->Data.size()-2, 0);
      }
    ReconstructionWindow::on_pbCorrUpdateTMP_clicked();
}

void ReconstructionWindow::on_ppCorr_AddLineAfter_clicked()
{
  //add line after
   int selected = ui->tabwidCorrPolygon->currentRow();
   qDebug()<<selected;

   tmpCorrFilter.getCut()->Data.insert(selected*2, 0);
   tmpCorrFilter.getCut()->Data.insert(selected*2, 0);

   ReconstructionWindow::on_pbCorrUpdateTMP_clicked();
}

void ReconstructionWindow::on_pbCorr_RemoveLine_clicked()
{
   int selected = ui->tabwidCorrPolygon->currentRow();
//    qDebug()<<selected;
   if (selected == -1)
    {
      message("select a row first!", this);
      return;
    }
  if (ui->tabwidCorrPolygon->rowCount()<4)
    {
      message("Minimum is three rows!", this);
      return;
    }

  tmpCorrFilter.getCut()->Data.removeAt(selected*2);
  tmpCorrFilter.getCut()->Data.removeAt(selected*2);

  if (selected == 0)
    {
      //have to change the last point of polygon too (it is not shown in the table)
      tmpCorrFilter.getCut()->Data[tmpCorrFilter.getCut()->Data.size()-2] = tmpCorrFilter.getCut()->Data[0];
      tmpCorrFilter.getCut()->Data[tmpCorrFilter.getCut()->Data.size()-1] = tmpCorrFilter.getCut()->Data[1];
    }
  ReconstructionWindow::on_pbCorrUpdateTMP_clicked();
}

void ReconstructionWindow::on_tabwidCorrPolygon_cellChanged(int row, int column)
{
  if (TMPignore) return; //do not update during filling with data!

  bool ok;
  QString str = ui->tabwidCorrPolygon->item(row,column)->text();
  double newVal = str.toDouble(&ok);
  qDebug()<<str<<newVal;
  if (!ok)
    {
      message("Input error!", this);
      ReconstructionWindow::FillUiFromTmpCorrFilter();
      return;
    }
  else ReconstructionWindow::on_pbCorrUpdateTMP_clicked();
}

void ReconstructionWindow::on_pbCorrExtractPolygon_clicked()
{
  if (EventsDataHub->isEmpty()) return;

  if ( tmpCorrFilter.getCorrelationUnitX()->isRequireReconstruction() || tmpCorrFilter.getCorrelationUnitY()->isRequireReconstruction() )
    if (EventsDataHub->ReconstructionData.isEmpty()) return;

  ReconstructionWindow::on_pbShowCorr_clicked();
  ReconstructionWindow::HideCutShapes();

  bool ok = ReconstructionWindow::start2DPolygonExtraction();
  if (!ok) return;

  //get data from graph and put it to tmpCorrFilter
 // for (int i=0; i<MW->graphRW->extractedPolygon.size()/2; i++)
//    qDebug()<<"----------------polygon point: "<<MW->graphRW->extractedPolygon[i]<<MW->graphRW->extractedPolygon[i+1];
  tmpCorrFilter.getCut()->Data = MW->GraphWindow->extractedPolygon();
  ui->pbCorrAccept->setIcon(RedIcon); //sice tmpCorrFilter changed!

  //update UI
  ReconstructionWindow::FillUiFromTmpCorrFilter();

  ReconstructionWindow::on_pbUpdateFilters_clicked();
  ReconstructionWindow::on_pbShowCorr_clicked();
}

bool ReconstructionWindow::start2DPolygonExtraction()
{
    MW->WindowNavigator->BusyOn();
    MW->GraphWindow->Extract2DPolygon();

    return ReconstructionWindow::Extraction();
}

bool ReconstructionWindow::Extraction()
{
  do
    {
      qApp->processEvents();
      if (MW->GraphWindow->IsExtractionCanceled()) break;
    }
  while (!MW->GraphWindow->IsExtractionComplete() );

  MW->WindowNavigator->BusyOff(false);

  return !MW->GraphWindow->IsExtractionCanceled();  //returns false = canceled
}

void ReconstructionWindow::on_pbEvaluateGains_clicked()
{
    ReconstructionWindow::ShowGainWindow();
}

void ReconstructionWindow::ShowGainWindow()
{
  if (!MW->GainWindow)
    {
      //first time accessed, creating
      //qDebug()<<"-->Creating GainWindow";
      QWidget* w = new QWidget();
      MW->GainWindow = new GainEvaluatorWindowClass(w, MW, EventsDataHub);
      //MW->GainWindow = new GainEvaluatorWindowClass(this, MW, EventsDataHub);

      // fix font size on Linux
      #ifdef Q_OS_WIN//Q_OS_LINUX
      #else
        SetWindowFont(MW->GainWindow, 8);
  //      GainWindow->SetWindowFont(8);
      #endif
      MW->GainWindow->show();
      MW->GainWindow->resize(MW->GainWindow->width()+1, MW->GainWindow->height());
      MW->GainWindow->resize(MW->GainWindow->width()-1, MW->GainWindow->height());
    }
  MW->GainWindow->show();
  MW->GainWindow->raise();
  MW->GainWindow->activateWindow();  
}

void ReconstructionWindow::on_pbSetPresprocessingForGains_clicked()
{
  MW->SetMultipliersUsingGains(ui->cbTakeIntoAccountChPerPhEl->isChecked(), &ChPerPhEl);

  //and set all gains to 1
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    MW->PMs->at(ipm).relGain = 1.0;
  ui->ledGain->setText("1");
}

void ReconstructionWindow::on_pbPurgeEvents_clicked()
{
    if (EventsDataHub->isEmpty()) return;
    int counter = 0;
    for (int iev = 0; iev < EventsDataHub->Events.size(); iev++)
      if (!EventsDataHub->ReconstructionData[iev]->GoodEvent) counter++;
    if (counter == 0)
      {
        message("There are no filtered events!", this);
        return;
      }
    QMessageBox msgBox(this);
    msgBox.setWindowTitle("Purge events confirmation");
    msgBox.setText("Purge " + QString::number(counter) + " events?\nSelection: Any event filter or reconstruction fail\nThere is no undo!");
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::Yes);
    if (msgBox.exec() == QMessageBox::No) return;

    MW->WindowNavigator->BusyOn();
    qApp->processEvents();
    bool DoTimed = false;   
    if (!EventsDataHub->TimedEvents.isEmpty()) DoTimed = true;
    bool DoScan = false;   
    if (!EventsDataHub->Scan.isEmpty()) DoScan = true;

    int NumEvents = EventsDataHub->Events.size();
    int percent = NumEvents / 100;
    if (percent < 1) percent = 1;
    ui->prReconstruction->setValue(0);
    MW->WindowNavigator->setProgress(0);

//    qDebug()<<"num events:"<<NumEvents<<DoTimed<<DoScan;

    int iposition = 0;
    for (int iev = 0; iev<NumEvents; iev++)
      {
//        qDebug()<<"iev iposition good"<<iev<<iposition<<EventsDataHub->ReconstructionData[iev]->GoodEvent;

        if (iev % percent == 0)
          {
            int prog = 100.0 * iev / NumEvents;
            ui->prReconstruction->setValue(prog);
            MW->WindowNavigator->setProgress(prog);
            qApp->processEvents();
          }

        //if bad event, continue
        if (!EventsDataHub->ReconstructionData[iev]->GoodEvent) continue;

        //found good event, moving to iposition if not != iev
        if (iev != iposition)
          {
            //move
            EventsDataHub->Events[iposition] = EventsDataHub->Events[iev];
            if (DoTimed) EventsDataHub->TimedEvents[iposition] = EventsDataHub->TimedEvents[iev];
            if (DoScan) EventsDataHub->Scan[iposition] = EventsDataHub->Scan[iev];
            EventsDataHub->ReconstructionData[iposition] = EventsDataHub->ReconstructionData[iev];
          }
        iposition++;
      }

    //resize and compacting
    EventsDataHub->Events.resize(iposition);
    EventsDataHub->Events.squeeze();
    if (DoTimed)
      {
        EventsDataHub->TimedEvents.resize(iposition);
        EventsDataHub->TimedEvents.squeeze();
      }
    if (DoScan)
      {
        EventsDataHub->Scan.resize(iposition);
        EventsDataHub->Scan.squeeze();
      }
    EventsDataHub->ReconstructionData.resize(iposition);
    EventsDataHub->ReconstructionData.squeeze();

    //CheckAvailableEventData();
    UpdateStatusAllEvents();

    ReconstructionWindow::on_pbRefreshDataIndication_clicked();
    ui->prReconstruction->setValue(100);
    MW->WindowNavigator->setProgress(100);
    MW->WindowNavigator->BusyOff(false);
}

void ReconstructionWindow::on_cbCoGapplyStretch_toggled(bool checked)
{
  ui->fCoGStretchXY->setEnabled(checked);
  ui->fCoGStretchZ->setEnabled(checked && ui->cb3Dreconstruction->isChecked());
}

void ReconstructionWindow::on_pbTreeView_clicked()
{   
  if (EventsDataHub->isEmpty())
    {
      message("Data are empty!", this);
      ui->twData->setCurrentIndex(0);
      return;
    }
  if (!EventsDataHub->isReconstructionDataReady())
    {
      message("Reconstruction was NOT yet performed!", this);
      ui->bsAnalyzeScan->setCurrentIndex(0);
      return;
    }
  QString what = ui->cobTreeViewWhat->currentText();
  if (what == "")
    {
      message("What to draw?", this);
      return;
    }

  QString cond = ui->cobTreeViewCuts->currentText();
  QString how = ui->cobTreeViewOptions->currentText();

  QStringList fields = what.split(":", QString::SkipEmptyParts);
  int num = fields.size();
  if (num>3)
    {
      message("Too many fields in \"What\"", this);
      return;
    }
  if (!EventsDataHub->ReconstructionTree)
    {      
      MW->WindowNavigator->BusyOn();
      EventsDataHub->createReconstructionTree(MW->Detector->PMs);
      MW->WindowNavigator->BusyOff(false);
    }
  EventsDataHub->ReconstructionTree->ResetBranchAddresses(); //if addresses are not resetted std::vectors cause crash on attempt to draw

  QByteArray baw;
  QVector<linkCustomClass> dims;
  dims.append(linkCustomClass(ui->sbBins1, ui->ledCustomFrom1, ui->ledCustomTo1));
  dims.append(linkCustomClass(ui->sbBins2, ui->ledCustomFrom2, ui->ledCustomTo2));
  dims.append(linkCustomClass(ui->sbBins3, ui->ledCustomFrom3, ui->ledCustomTo3));
  QString str = what+">>htemp(";
  for (int i=0; i<num; i++)
    {
      if (ui->cbCustomBinning->isChecked())dims[i].updateBins();
      if (ui->cbCustomRanges->isChecked()) dims[i].updateRanges(); //else 0
      str += QString::number(dims[i].bins)+","+QString::number(dims[i].from)+","+QString::number(dims[i].to)+",";
    }
  str.chop(1);
  str += ")";
  baw = str.toLocal8Bit();

  const char *What = baw.data();
   QByteArray bac = cond.toLocal8Bit();
   const char *Cond = (cond == "") ? "" : bac.data();
   QByteArray bah = how.toLocal8Bit();
   const char *How = (how == "") ? "" : bah.data();
   QString howAdj;
   if (how == "") howAdj = "goff";
   else howAdj = "goff,"+how;
   QByteArray baha = howAdj.toLocal8Bit();
   const char *HowAdj = baha.data();

   switch (num)
     {
       case 1:
       {
         EventsDataHub->ReconstructionTree->Draw(What, Cond, HowAdj);
         tmpHist1D = (TH1*)gDirectory->Get("htemp");
         if (!tmpHist1D)
           {
             message("Root has not generated any data!", this);
             return;
           }
         QByteArray tmp = fields[0].toLocal8Bit();
         tmpHist1D->GetXaxis()->SetTitle(tmp.data());
         int size = tmpHist1D->GetEntries();
         if (size>0) MW->GraphWindow->Draw(tmpHist1D, How);
         else
           {
             message("There is no data to show!", this);
             MW->GraphWindow->close();
             return;
           }
         break;
       }
     case 2:
       {         
         EventsDataHub->ReconstructionTree->Draw(What, Cond, HowAdj);
         tmpHist2D = (TH2*)gDirectory->Get("htemp");
         if (!tmpHist2D)
           {
             message("Root has not generated any data!", this);
             return;
           }
         QByteArray tmp1 = fields[0].toLocal8Bit();
         tmpHist2D->GetYaxis()->SetTitle(tmp1.data());
         QByteArray tmp2 = fields[1].toLocal8Bit();
         tmpHist2D->GetXaxis()->SetTitle(tmp2.data());

         int size = tmpHist2D->GetEntries();
         if (size>0) MW->GraphWindow->Draw(tmpHist2D, How);
         else
           {
             message("There is no data to show!", this);
             MW->GraphWindow->close();
             return;
           }
         break;
       }
     case 3:
       {         
         EventsDataHub->ReconstructionTree->Draw(What, Cond, HowAdj);
         tmpHist3D = (TH3*)gDirectory->Get("htemp");
         if (!tmpHist3D)
           {
             message("Root has not generated any data!", this);
             return;
           }
         QByteArray tmp1 = fields[0].toLocal8Bit();
         tmpHist3D->GetZaxis()->SetTitle(tmp1.data());
         QByteArray tmp2 = fields[1].toLocal8Bit();
         tmpHist3D->GetYaxis()->SetTitle(tmp2.data());
         QByteArray tmp3 = fields[2].toLocal8Bit();
         tmpHist3D->GetXaxis()->SetTitle(tmp3.data());

         int size = tmpHist3D->GetEntries();
         if (size>0) MW->GraphWindow->Draw(tmpHist3D, How);
         else
           {
             message("There is no data to show!", this);
             MW->GraphWindow->close();
             return;
           }
         break;
       }
     }

   //Updating history
   TreeViewStruct tv = TreeViewStruct(what, cond, how);
   tv.fillCustom(ui->cbCustomBinning->isChecked(), ui->cbCustomRanges->isChecked(), dims);
   TreeViewHistory.append( tv );
   TMPignore = true;
   ui->sbTreeViewHistory->setValue( TreeViewHistory.size()-1 );
   TMPignore = false;
   ui->cobTreeViewWhat->insertItem(0, what);
   for (int i = ui->cobTreeViewWhat->count()-1; i>0; i--)
       if (ui->cobTreeViewWhat->itemText(i) == what) ui->cobTreeViewWhat->removeItem(i);
   ui->cobTreeViewWhat->setCurrentIndex(0);

   ui->cobTreeViewCuts->insertItem(0, cond);
   for (int i=ui->cobTreeViewCuts->count()-1; i>0; i--)
       if (ui->cobTreeViewCuts->itemText(i) == cond) ui->cobTreeViewCuts->removeItem(i);
   ui->cobTreeViewCuts->setCurrentIndex(0);

   ui->cobTreeViewOptions->insertItem(0, how);
   for (int i=ui->cobTreeViewOptions->count()-1; i>0; i--)
       if (ui->cobTreeViewOptions->itemText(i) == how) ui->cobTreeViewOptions->removeItem(i);
   ui->cobTreeViewOptions->setCurrentIndex(0);
}

void ReconstructionWindow::on_pbTreeViewHelpWhat_clicked()
{
   QString str;

   str = "\tValid entries:\n\n "

         "\t Event data\n"
         " i\t\t event number\n"
         " signal[ip]\t signal of ip's PM\n"
         " ssum \t\t sum signal of all PMS (weighted by gains) \n"

         "\t Reconstruction results\n"
         " x[], y[], z[]\t reconstructed coordinates ([] - multiple events)\n"
         " energy[] \t reconstruted energy\n"
         " rho[ip] \t distance from xyz to the center of ip's PM\n"
         " chi2 \t\t chi2 of the reconstruction\n"
         " recOK \t\t reconstruction event successful = true \n"
         " good \t\t true is recOK and all filters true\n"

         " \t Event filters\n"
         " fcut \t\t cut-off for individual signals for each PM \n"
         " fsumcut \t sum signal \n"
         " fen \t\t event energy \n"
         " fchi \t\t reduced chi-square \n"
         " fsp \t\t spatial filter\n"

         " \tScan Data (if present)\n"
         " xScan[] \tactual coordinates \n"
         " yScan[]\n"
         " zScan[]\n"
         " zStop     \t For secondary scint, light is emitted in range zScan->zStop\n"
         " numPhotons[] number of photons this event \n"
         " ScintType  \t 0 - unknown, 1 - primary, 2 - secondary \n"
         " GoodEvent  \t false for noise events \n"
         " EventType  \t type of noise event \n"
/*
              "\n\n"
              "   The item to be plotted can contain up to three entries, one for each dimension, <br>"
              "   separated by a colon (e1:e2:e3), where e1, e2 and e3 are valid entries. <br>"
              "   Each configuration in this field is under the conditions defined by 'Cuts' and  <br>"
              "   'Options' configurations. <br>"
              "    \n"
*/
              "\n\tExamples:\n"
              "          x - This entry will draw an histogram of all x\n"
              "        x:y - This will draw x vs. y in a 2D plot. \n"
              "      x:y:z - To draw x vs. y vs. z in a 3D plot. ";

   message(str, this);
}

void ReconstructionWindow::on_pbTreeViewHelpCuts_clicked()
{
    QString str;

    str = "Any combination of logicaloperation on the valid entries (see help for 'Draw what')\n"
          "For example: energy>1.2 && chi2<2.1";

    message(str, this);
}

void ReconstructionWindow::on_pbTreeViewHelpOptions_clicked()
{
    QString str;

    str = "See CERN ROOT help for histogram drawing options.\n"
          "For example: 1D hist: \"APL\", 2D hist: \"lego\"";

    message(str, this);
}

void ReconstructionWindow::on_sbTreeViewHistory_valueChanged(int arg1)
{
  if (TMPignore) return;
  int maxEvN = TreeViewHistory.size();
  if (arg1 >= maxEvN)
  {
      ui->sbTreeViewHistory->setValue(maxEvN-1);
      return; //update is on_change
  }

  TreeViewStruct tv = TreeViewHistory[arg1];
  ui->cobTreeViewWhat->setCurrentText(tv.what);
  ui->cobTreeViewCuts->setCurrentText(tv.cuts);
  ui->cobTreeViewOptions->setCurrentText(tv.options);
  ui->cbCustomBinning->setChecked(tv.fCustomBins);
  ui->cbCustomRanges->setChecked(tv.fCustomRanges);
  if (tv.fCustomBins)
    {
      ui->sbBins1->setValue(tv.Bins[0]);
      ui->sbBins2->setValue(tv.Bins[1]);
      ui->sbBins3->setValue(tv.Bins[2]);
    }
  if (tv.fCustomRanges)
    {
      ui->ledCustomFrom1->setText(QString::number(tv.From[0]));
      ui->ledCustomFrom2->setText(QString::number(tv.From[1]));
      ui->ledCustomFrom3->setText(QString::number(tv.From[2]));
      ui->ledCustomTo1->setText(QString::number(tv.To[0]));
      ui->ledCustomTo2->setText(QString::number(tv.To[1]));
      ui->ledCustomTo3->setText(QString::number(tv.To[2]));
    }
}

void ReconstructionWindow::on_ledPassiveRange_editingFinished()
{
  double range = ui->ledPassiveRange->text().toDouble();
  int igroup = ReconstructionWindow::GetGroupNumber(ui->cobPMGroupForDynPassives);
  MW->PMs->setPassiveRange(igroup, range);
}

void ReconstructionWindow::on_cobCUDAoffsetOption_currentIndexChanged(int index)
{
   ui->fCUDAoffsets->setVisible(index == 2);
}

struct ZSortStruct
{
  int iev;
  int pmOverThreshold;
};

bool lessThanSorter(const ZSortStruct* s1, const ZSortStruct* s2)
 {
     return s1->pmOverThreshold < s2->pmOverThreshold;
 }

void ReconstructionWindow::on_pbEvaluateZfromDistribution_clicked()
{
   if (EventsDataHub->ReconstructionData.isEmpty()) return;
   TString type = (*MW->SensLRF)[0]->type();
   if ( type != "Sliced3D")
     {
       message("This algorithm works only with Sliced3D LRFs", this);
       return;
     }

   MW->WindowNavigator->BusyOn();
   qApp->processEvents();

   QList<ZSortStruct*> sorter;
   sorter.reserve(EventsDataHub->Events.size());

   for (int iev=0; iev < EventsDataHub->Events.size(); iev++)
     {
       ZSortStruct* tmp = new ZSortStruct();
       tmp->iev = iev;

       double ssig = 0;
       double maxsig = 0;
       for (int ipm=0; ipm<MW->PMs->count(); ipm++)
         {
           double sig = EventsDataHub->Events.at(iev)[ipm] / MW->PMs->at(ipm).relGain;
           ssig += sig;
           if (sig > maxsig) maxsig = sig;
         }
       double avSig = ssig / MW->PMs->count();

       int AboveThr = 0;
       double fraction = ui->ledZevaluatorThreshold->text().toDouble();
       for (int ipm=0; ipm<MW->PMs->count(); ipm++)
         {
           double sig = EventsDataHub->Events.at(iev)[ipm] / MW->PMs->at(ipm).relGain;
           double threshold = avSig + fraction*(maxsig - avSig);
           if (sig > threshold) AboveThr++;
         }
       tmp->pmOverThreshold = AboveThr;
       sorter.append(tmp);
     }

   //sorting
   qSort(sorter.begin(), sorter.end(), lessThanSorter);

//   for (int i=0; i<Reconstructor->Events->size(); i++)
//     qDebug()<<sorter[i]->iev<<sorter[i]->pmOverThreshold<<"    z="<<Reconstructor->Scan->at(sorter[i]->iev)->Points[0].r[2];

   //slices?
   LRFsliced3D *lrf = (LRFsliced3D*)(*MW->SensLRF)[0];
   int slices = lrf->getNintZ();

   int evPerSlice = EventsDataHub->Events.size()/slices;
   for (int i=0; i < EventsDataHub->Events.size(); i++)
     {
       int iz = slices-1 - i / evPerSlice;
       double z = lrf->getSliceMedianZ(iz);

       EventsDataHub->ReconstructionData[sorter[i]->iev]->Points[0].r[2] = z;
     }

   MW->WindowNavigator->BusyOff();
}

void ReconstructionWindow::on_pbBlurReconstructedXY_clicked()
{
  if (EventsDataHub->ReconstructionData.isEmpty()) return;

  MW->WindowNavigator->BusyOn();
  double delta = ui->ledBlurDelta->text().toDouble();

  for (int iev=0; iev<EventsDataHub->ReconstructionData.size(); iev++)
    for (int ip=0; ip<EventsDataHub->ReconstructionData[iev]->Points.size(); ip++)
    {
      EventsDataHub->ReconstructionData[iev]->Points[ip].r[0] += -delta + 2.0*delta*MW->Detector->RandGen->Rndm();
      EventsDataHub->ReconstructionData[iev]->Points[ip].r[1] += -delta + 2.0*delta*MW->Detector->RandGen->Rndm();
    }

   MW->WindowNavigator->BusyOff(false);
}

void ReconstructionWindow::on_pbDefNNMin_clicked()
{
#ifdef ANTS_FLANN 
  if (EventsDataHub->isEmpty())
  {
    message("There are no events!", this);
    return;
  }
  ReconstructionWindow::on_pbNNshowSpectrum_clicked();

  bool ok = ReconstructionWindow::startXextraction();
  if (!ok) return;
  ui->ledNNmin->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
  ReconstructionWindow::UpdateStatusAllEvents();
  ReconstructionWindow::on_pbNNshowSpectrum_clicked();
#endif
}

void ReconstructionWindow::on_pbDefNNMax_clicked()
{
#ifdef ANTS_FLANN  
  if (EventsDataHub->isEmpty())
  {
    message("There are no events!", this);
    return;
  }
  ReconstructionWindow::on_pbNNshowSpectrum_clicked();

  bool ok = ReconstructionWindow::startXextraction();
  if (!ok) return;
  ui->ledNNmax->setText(QString::number(MW->GraphWindow->extractedX(), 'g', 4));
  ReconstructionWindow::UpdateStatusAllEvents();
  ReconstructionWindow::on_pbNNshowSpectrum_clicked();
#endif
}

void ReconstructionWindow::on_pbShowMap_clicked()
{
  int iev = ui->sbEventNumberInspect->value();

  if (EventsDataHub->isEmpty()) return;
  if (EventsDataHub->ReconstructionData.isEmpty()) return;
  if (!MW->SensLRF->isAllLRFsDefined())
    {
      message("All LRFs have to be defined!", this);
      return;
    }
  if (!EventsDataHub->ReconstructionData[iev]->ReconstructionOK)
    {
       MW->GraphWindow->close();
       return;
    }
  int MapCenterOption = ui->cobMapCenter->currentIndex();
  if (MapCenterOption == 1 && EventsDataHub->isScanEmpty())
    {
      message("Scan does not exists: changing center option to 'CoG'", this);
      ui->cobMapCenter->setCurrentIndex(0);
      MapCenterOption = 0;
    }
  //if (MapCenterOption == 2 && !MW->Reconstructor->isReconstructionDataReady())
  if (MapCenterOption == 2 && !EventsDataHub->isReconstructionDataReady())
    {
      message("To center on reconstructed position, reconstruction of all events has to be performed first!", this);
      return;
    }
  int ZOption = ui->cobMapZ->currentIndex();
  //if (ZOption == 1 && !MW->Reconstructor->isReconstructionDataReady())
  if (ZOption == 1 && !EventsDataHub->isReconstructionDataReady())
    {
      message("To get reconstructed Z, reconstruction of all events has to be performed first!", this);
      return;
    }
  double z;
  if (ZOption == 0) z = ui->ledMapFixedZ->text().toDouble();
  else z = EventsDataHub->ReconstructionData[iev]->Points[0].r[2];

  //MW->Reconstructor->evaluateEvent(EventsDataHub->ReconstructionData[iev], iev); //updates dynamic passives
  //if (MW->Reconstructor->isUseDynamicPassives()) {} /// ***
  DynamicPassivesHandler *Passives = new DynamicPassivesHandler(MW->Detector->PMs, EventsDataHub);
  Passives->init(&EventsDataHub->RecSettings);
  if (EventsDataHub->RecSettings.fUseDynamicPassives)
    Passives->calculateDynamicPassives(iev, EventsDataHub->ReconstructionData.at(iev));

  MW->Owindow->RefreshData();

  double range = ui->ledMapRange->text().toDouble();
  int bins = ui->sbMapBins->value();
  bool RecEnergy = ui->cbReconstructEnergy->isChecked();
  double Energy = ui->ledSuggestedEnergy->text().toDouble();
  bool WeightedChi2calculation = EventsDataHub->RecSettings.fWeightedChi2calculation;//ui->cbWeightedChi2->isChecked();

  double delta = range/bins;
  double x0, y0;
  switch (MapCenterOption)
    {
    case 0: //CoG
      x0 = EventsDataHub->ReconstructionData[iev]->xCoG - 0.5*delta*(bins-1);
      y0 = EventsDataHub->ReconstructionData[iev]->yCoG - 0.5*delta*(bins-1);
      break;
    case 1: //Scan
      x0 = EventsDataHub->Scan[iev]->Points[0].r[0] - 0.5*delta*(bins-1);
      y0 = EventsDataHub->Scan[iev]->Points[0].r[1] - 0.5*delta*(bins-1);
      break;
    case 2: //reconstructed
      x0 = EventsDataHub->ReconstructionData[iev]->Points[0].r[0] - 0.5*delta*(bins-1);
      y0 = EventsDataHub->ReconstructionData[iev]->Points[0].r[1] - 0.5*delta*(bins-1);
      break;
    default:
      qWarning()<<"Unknown map start option!";
      return;
    }

  if (tmpHist) delete tmpHist;
  tmpHist = new TH2D("MapEvHist","",
                     bins, x0, x0+delta*bins,
                     bins, y0, y0+delta*bins);
  tmpHist->GetXaxis()->SetTitle("X, mm");
  tmpHist->GetYaxis()->SetTitle("Y, mm");

  MW->WindowNavigator->BusyOn();
  double Zmin=1e10;
  double Zmax=-1e10;
  for (int ix = 0; ix<bins; ix++)
    for (int iy = 0; iy<bins; iy++)
      {
        double x = x0 + delta*ix;
        double y = y0 + delta*iy;
        double val = 0; //chi2 or Ln(likelihood) in this point

        if (RecEnergy)
          {
            double sumSig = 0;
            double sumLRF = 0;
            for (int ipm=0; ipm<MW->PMs->count(); ipm++)
              ///if (!MW->PMs->at(ipm).passive)
              if (Passives->isActive(ipm))
                {
                  sumSig += EventsDataHub->Events[iev][ipm];
                  sumLRF += MW->SensLRF->getLRF(ipm, x, y, z);
                }
            if (sumLRF>0) Energy = sumSig/sumLRF;
          }

        for (int ipm=0; ipm<MW->PMs->count(); ipm++)
          ///if (!MW->PMs->at(ipm).passive)
          if (Passives->isActive(ipm))
            {
              //calculating LRF of this pm at this coordinates
              double LRFhere = MW->SensLRF->getLRF(ipm, x, y, z) * Energy;
              if (LRFhere == 0)
                {
                  //LRF is undefined here!
                  val = 0;
                  break;
                }

              //calculating the value of ci2 or LN(like) at this position
              if (ui->cobMapType->currentIndex() == 0)
                {
                  //chi2
                  double delta = (LRFhere - EventsDataHub->Events[iev][ipm]);

                  if (WeightedChi2calculation)
                    {
                      double sigma;
                      //sigma = LRFmodule->getSigma(ipm, X, Y) * sqrt(energy);
                      sigma = sqrt(LRFhere);

                      val += delta*delta/sigma/sigma;
                    }
                  else val += delta*delta;
                }
              else
                {
                  //ln max likelihood
                  val += EventsDataHub->Events[iev][ipm] * log(LRFhere) - LRFhere;
                }
            }

        if (val != 0)
          {
            tmpHist->Fill(x,y,val);
            if (val>Zmax) Zmax = val;
            if (val<Zmin) Zmin = val;
          }
      }
  MW->WindowNavigator->BusyOff(false);

  tmpHist->GetZaxis()->SetRangeUser(Zmin,Zmax);
  MW->GraphWindow->Draw(tmpHist, "colz");

  //showing markers
  ReconstructionWindow::ClearLineMarkers();
  double len = delta*0.06*bins; //marker length

  if (!EventsDataHub->isScanEmpty())
    {
      for (int ip=0; ip<EventsDataHub->Scan[iev]->Points.size(); ip++)
        {
          //triple whiter
          double x0 = EventsDataHub->Scan[iev]->Points[ip].r[0];
          double y0 = EventsDataHub->Scan[iev]->Points[ip].r[1];

          ReconstructionWindow::PrepareMarker(x0, y0, len, kBlue);
        }
    }

  if (EventsDataHub->ReconstructionData[iev]->ReconstructionOK)
    {
      for (int ip=0; ip<EventsDataHub->Scan[iev]->Points.size(); ip++)
        {
          //triple whiter
          double x0 = EventsDataHub->ReconstructionData[iev]->Points[ip].r[0];
          double y0 = EventsDataHub->ReconstructionData[iev]->Points[ip].r[1];

          ReconstructionWindow::PrepareMarker(x0, y0, len, kRed);
        }
    }

  for (int i=0; i<Markers.size(); i++) MW->GraphWindow->Draw(Markers[i], "same");
  MW->GraphWindow->UpdateRootCanvas();
  LastDistributionShown = "MapEvent";
  delete Passives;
}

void ReconstructionWindow::PrepareMarker(double x0, double y0, double len, Color_t color)
{
  TLine* l1h = new TLine(x0-len,y0,x0+len,y0);
  l1h->SetLineWidth(3);
  l1h->SetLineColor(kWhite);
  Markers.append(l1h);
  TLine* l1v = new TLine(x0,y0-len,x0,y0+len);
  l1v->SetLineWidth(3);
  l1v->SetLineColor(kWhite);
  Markers.append(l1v);

  TLine* l2h = new TLine(x0-len,y0,x0+len,y0);
  l2h->SetLineWidth(2);
  l2h->SetLineColor(color);
  Markers.append(l2h);
  TLine* l2v = new TLine(x0,y0-len,x0,y0+len);
  l2v->SetLineWidth(1);
  l2v->SetLineColor(color);
  Markers.append(l2v);
}

void ReconstructionWindow::on_cobMapZ_currentIndexChanged(int index)
{
    ui->ledMapFixedZ->setEnabled(!(bool)index);
}

void ReconstructionWindow::on_pbTogglePassives_clicked()
{
  if (dialog)
    {
      dialog->show();
      dialog->activateWindow();
      return;
    }

  dialog = new QDialog(this);
  dialog->setWindowTitle("Toggle active/passive PMs");
  dialog->resize(400,400);
  dialog->show();

  QPushButton *okButton = new QPushButton("Close");
  connect(okButton,SIGNAL(clicked()),dialog,SLOT(accept()));

  if (vo) delete vo;
  myQGraphicsView* gv = new myQGraphicsView(this);

  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(gv);
  mainLayout->addWidget(okButton);

  dialog->setLayout(mainLayout);

  vo = new Viewer2DarrayObject(gv, MW->PMs);
  gv->show();
  okButton->show();
  ReconstructionWindow::PaintPMs();

  vo->DrawAll();
  vo->ResetViewport();
  connect(vo, SIGNAL(PMselectionChanged(QVector<int>)), this, SLOT(onSelectionChange(QVector<int>)));

  dialog->exec();

  if (dialog) delete dialog;
  dialog = 0;
  if (vo) delete vo;
  vo = 0;
}

void ReconstructionWindow::onSelectionChange(QVector<int> selectionArray)
{
  //qDebug()<<"Selection change was triggered. PMs selected: "<<selectionArray;
//  qDebug()<<"If selected one PM, toggling PM passive";
  if (selectionArray.size() == 1)
    {
      int ipm = selectionArray.first();
      if (MW->PMs->at(ipm).passive == 0) MW->PMs->at(ipm).passive = 1;
      else MW->PMs->at(ipm).passive = 0;
    }

   ReconstructionWindow::PaintPMs();
   vo->DrawAll();
}

void ReconstructionWindow::PaintPMs()
{
  //MW->Reconstructor->clearAllDynamicPassives();
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      vo->SetText(ipm, QString::number(ipm));

      if (MW->PMs->at(ipm).passive != 0)
        {
          vo->SetBrushColor(ipm, Qt::black);
          vo->SetTextColor(ipm, Qt::white);
        }
      else
        {
          vo->SetBrushColor(ipm, Qt::white);
          vo->SetTextColor(ipm, Qt::black);
        }
    }
}

void ReconstructionWindow::UpdatePMGroupCOBs()
{
  UpdateOneGroupCOB(ui->cobAssignToPMgroup);
  UpdateOneGroupCOB(ui->cobGroupInGains);
  UpdateOneGroupCOB(ui->cobPMGroupForDynPassives);
}

void ReconstructionWindow::UpdateOneGroupCOB(QComboBox* cob)
{
  QString old = cob->currentText();
  cob->clear();
  bool oldFound = false;

  for (int igroup=0; igroup<MW->PMs->countPMgroups(); igroup++)
    {
      QString str = MW->PMs->getGroupDescription(igroup);
      cob->addItem(str);

      if (str == old)
        {
          cob->setCurrentText(str);
          oldFound = true;
        }
    }

  if (!oldFound) cob->setCurrentIndex(0);
}

int ReconstructionWindow::GetGroupNumber(QComboBox* cob)
{
  int igroup = MW->PMs->getGroupByDescription(cob->currentText());
  if (igroup > -1) return igroup;

  //else group not found
  cob->setCurrentText(MW->PMs->getGroupDescription(0)); //default group
  return 0;
}

void ReconstructionWindow::on_cobPMGroupForDynPassives_currentIndexChanged(const QString &arg1)
{
  if (!MW->PMs) return;

  int igroup = MW->PMs->getGroupByDescription(arg1);
  if (igroup == -1)
    {
      ui->cobPMGroupForDynPassives->setCurrentIndex(0);
      igroup = 0;
    }

  double threshold = MW->PMs->getPassiveThreshold(igroup);
  ui->ledDynamicPassiveThreshold->setText(QString::number(threshold));

  double range = MW->PMs->getPassiveRange(igroup);
  ui->ledPassiveRange->setText(QString::number(range));
}

void ReconstructionWindow::on_cobGroupInGains_currentIndexChanged(const QString &arg1)
{
  if (!MW->PMs) return;

  int igroup = MW->PMs->getGroupByDescription(arg1);
  if (igroup == -1)
    {
      ui->cobGroupInGains->setCurrentIndex(0);
      igroup = 0;
    }

  ui->leiPMnumberForGains->setText("");
}

void ReconstructionWindow::on_cobShowWhat_currentIndexChanged(int /*index*/)
{
    bool show = false;
    if (ui->cobShowWhat->currentText() == "PM signal") show = true;

    ui->fShowXYPmnumber->setVisible(show);
}

void ReconstructionWindow::DoBlur(double sigma)
{
   QString str = QString::number(sigma);
   ui->ledBlurDelta->setText(str);
   qApp->processEvents();
   ReconstructionWindow::on_pbBlurReconstructedXY_clicked();
}

void ReconstructionWindow::on_pbExportData_clicked()
{
    if (EventsDataHub->Events.isEmpty()) return;
    if (EventsDataHub->ReconstructionData.isEmpty()) return;

    QString fileName = QFileDialog::getSaveFileName(this, "Export data", MW->GlobSet->LastOpenDir, "Data files (*.dat);;Text files (*.txt)");
    if (fileName.isEmpty()) return;
    MW->GlobSet->LastOpenDir = QFileInfo(fileName).absolutePath();

    QFile outFile( fileName );
    outFile.open(QIODevice::WriteOnly);
    if(!outFile.isOpen())
      {
        message("Unable to open file " +fileName+ " for writing!", this);
        return;
      }
    QTextStream outStream(&outFile);

    for (int ievent=0; ievent < EventsDataHub->Events.size(); ievent++)
      if (EventsDataHub->ReconstructionData[ievent]->GoodEvent)
        {
          for (int ipm=0; ipm<MW->PMs->count(); ipm++)
            outStream << EventsDataHub->Events.at(ievent)[ipm] << " ";
          outStream << "\r\n";
        }
    outFile.close();
}

Double_t gauss(Double_t *x, Double_t *par)
{
   Double_t X = x[0];
   Double_t Y = x[1];
   Double_t x0 = par[0];
   Double_t y0 = par[1];
   Double_t w = par[2];
   Double_t Norm = par[3];

   Double_t r2 = (X-x0)*(X-x0) + (Y-y0)*(Y-y0);
   Double_t result = Norm * TMath::Exp(-r2/w);
   return result;
}

void ReconstructionWindow::on_pbCenterControlEvaluate_clicked()
{
  int ipm = ui->sbCenterControlPM->value();
  if (ipm < 0 || ipm > MW->PMs->count()-1)
    {
      message("Wrong PM number!", this);
      return;
    }

  //if (!MW->Reconstructor->isReconstructionDataReady())
  if (!EventsDataHub->isReconstructionDataReady())
    {
      message("To use this feature, first perform \"Reconstruct all events\"", this);
      return;
    }

  ReconstructionWindow::GetCenterShift(ipm, true);
}

bool ReconstructionWindow::GetCenterShift(int ipm, bool fDraw)
{
  if (ipm < 0 || ipm > MW->PMs->count()-1) return false;
  int bins = ui->sbCenterControlBins->value();
  double radius = ui->ledCenterControlRadius->text().toDouble();
  double radius2 = radius*radius;
  int MethodOption = ui->cobCenterOption->currentIndex(); // 0- Gaussian, 1 - centroid

  double sumX = 0;
  double sumY = 0;
  double sumSig = 0;
  int iEvents = 0;
  int x0 = MW->PMs->X(ipm);
  int y0 = MW->PMs->Y(ipm);

  if (centHist) delete centHist;
  centHist = new TH2D("centrHist","", bins, -radius, radius, bins, -radius, radius);
  TH2D* centHistNorm = new TH2D("centrHistNorm","", bins, -radius, radius, bins, -radius, radius);

  for (int iev = 0; iev < EventsDataHub->ReconstructionData.size(); iev++)
   {
    if (EventsDataHub->ReconstructionData[iev]->Points.size() == 1)
      {
        if (! EventsDataHub->ReconstructionData[iev]->GoodEvent) continue;
        double x = EventsDataHub->ReconstructionData[iev]->Points[0].r[0] - x0;
        double y = EventsDataHub->ReconstructionData[iev]->Points[0].r[1] - y0;
        double r2 = x*x + y*y;
        if (r2 > radius2) continue;

        iEvents++;
        double sig = EventsDataHub->Events.at(iev).at(ipm);
        sumSig += sig;
        sumX += sig*x;
        sumY += sig*y;

        centHist->Fill(x, y, sig);
        centHistNorm->Fill(x, y);
      }
    }

  if (sumSig <= 0)
    {
      qDebug() << "ipm="<<ipm<< "Error: sum signal = " << sumSig;
      delete centHistNorm;
      return false;
    }

  sumX /= sumSig;
  sumY /= sumSig;

  *centHist = *centHist / *centHistNorm;

  //Gauss fit
  funcParams[0] = 0;
  funcParams[1] = 0;  
  TF2 *Gauss = new TF2("f2",gauss,-radius,radius,-radius,radius, 4);
  Gauss->SetParameters(funcParams);
  centHist->Fit("f2");

  if (fDraw)
    {
      centHist->GetXaxis()->SetTitle("Delta from PM center X, mm");
      centHist->GetYaxis()->SetTitle("Delta from PM center Y, mm");

      MW->GraphWindow->Draw(centHist, "colz");
      MW->GraphWindow->Draw(Gauss, "same cont1");
    }

  if (MethodOption == 1)
    {
      //centroid
      funcParams[0] = sumX;
      funcParams[1] = sumY;
    }
  else
    {
      //gaussian
      funcParams[0] = Gauss->GetParameter(0);
      funcParams[1] = Gauss->GetParameter(1);
    }
  funcParams[2] = Gauss->GetParameter(2); //next will start from these data
  funcParams[3] = Gauss->GetParameter(3);

  qDebug() << "Events in radius, centroid x,y: "<< iEvents << sumX << sumY;
  qDebug() << "Gauss center x,y:" << Gauss->GetParameter(0) << Gauss->GetParameter(1);
  // x0 = par[0] y0 = par[1] w = par[2] Norm = par[3];

  delete centHistNorm;
  return true;
}


void ReconstructionWindow::on_pbEvaluateAndShift_clicked()
{
   QMessageBox::StandardButton reply;
   reply = QMessageBox::question(this, "Confirmation needed", "Shift sensor position (no undone)?",
                                  QMessageBox::Yes|QMessageBox::Cancel);
   if (reply == QMessageBox::Cancel) return;

   int ipm = ui->sbCenterControlPM->value();
   ReconstructionWindow::DoCenterShift(ipm);
}

void ReconstructionWindow::DoCenterShift(int ipm)
{
   //if (!MW->Reconstructor->isReconstructionDataReady())
   if (!EventsDataHub->isReconstructionDataReady())
     {
       message("To use this feature, first perform \"Reconstruct all events\"", this);
       return;
     }
   if (ipm < 0 || ipm > MW->PMs->count()-1)
     {
       message("Wrong PM number!", this);
       return;
     }

  ReconstructionWindow::GetCenterShift(ipm, false);
  double dx = funcParams[0]; //x shift from actual PM position
  double dy = funcParams[1];

  //shifting PM
  double r[3];
  r[0] = MW->PMs->X(ipm) + dx;
  r[1] = MW->PMs->Y(ipm) + dy;
  r[2] = MW->PMs->Z(ipm);
  MW->PMs->at(ipm).setCoords(r);
}

int ReconstructionWindow::getReconstructionAlgorithm()
{
  return ui->cobReconstructionAlgorithm->currentIndex();
}

bool ReconstructionWindow::isCoGignoreLow()
{
  return ui->cbCogIgnoreLow->isChecked();
}

double ReconstructionWindow::getCoGignoreThreshold()
{
  return ui->ledCoGignoreLevel->text().toDouble();
}

void ReconstructionWindow::on_pbMakePictureVsXY_clicked()
{
    if (!MW->hist2D)
      {
        message("Histogram is not ready!", this);
        return;
      }

    int binsX = MW->hist2D->GetNbinsX();// ui->sbXbins->value();
    int binsY = MW->hist2D->GetNbinsY();//ui->sbYbins->value();
    QImage image(binsX, binsY, QImage::Format_RGB16);
    QRgb value;

    double scale = ui->ledMakePictureScale->text().toDouble() / MW->hist2D->GetMaximum();
    for (int iy=0; iy<binsY; iy++)   //0 - underflow, bins+1 overflow
      for (int ix=0; ix<binsX; ix++)
        {
          double level = MW->hist2D->GetBinContent(ix+1, binsY-iy) * scale;
          int lev = level;
          if (lev > 255) lev = 255;
          value = qRgb(lev, lev, lev);
          image.setPixel(ix, iy, value);
        }

    image.save("showvsxy.png");
    QDesktopServices::openUrl(QUrl("file:showvsxy.png", QUrl::TolerantMode));
}

void ReconstructionWindow::on_pbShiftAll_clicked()
{
    QVector<double> dx, dy;

    //calculating all shifts
    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
        bool OK = ReconstructionWindow::GetCenterShift(ipm, false);
        if (OK)
        {
            dx.append(funcParams[0]); //x shift from actual PM position
            dy.append(funcParams[1]);
        }
        else
        {
            dx.append(0);
            dy.append(0);
        }
        qDebug() << "pm:"<<ipm<<dx.last()<<dy.last();
    }

    //shifting PM
    double r[3];
    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
        r[0] = MW->PMs->X(ipm) + dx[ipm];
        r[1] = MW->PMs->Y(ipm) + dy[ipm];
        r[2] = MW->PMs->Z(ipm);
        MW->PMs->at(ipm).setCoords(r);
    }
}

void ReconstructionWindow::on_pbShiftAllWithDis_clicked()
{
    QVector<double> dx, dy;

    //calculating all shifts
    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
        MW->PMs->at(ipm).setTMPpassve();
        ReconstructionWindow::on_pbReconstructAll_clicked();
        MW->PMs->at(ipm).clearTMPpassive();

        bool OK = ReconstructionWindow::GetCenterShift(ipm, false);
        if (OK)
        {
            dx.append(funcParams[0]); //x shift from actual PM position
            dy.append(funcParams[1]);
        }
        else
        {
            dx.append(0);
            dy.append(0);
        }
        qDebug() << "pm:"<<ipm<<dx.last()<<dy.last();
    }

    //shifting PM
    double r[3];
    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
        r[0] = MW->PMs->X(ipm) + dx[ipm];
        r[1] = MW->PMs->Y(ipm) + dy[ipm];
        r[2] = MW->PMs->Z(ipm);
        MW->PMs->at(ipm).setCoords(r);
    }
}

void ReconstructionWindow::on_pbEvaluateGrid_clicked()
{
   int ipm = ui->sbCenterControlPM->value();

   double dx, dy;
   ReconstructionWindow::EvaluateShiftBrute(ipm, &dx, &dy);
}

void ReconstructionWindow::EvaluateShiftBrute(int ipm, double* dx, double* dy)
{
  int steps = ui->sbCenterStepsLR->value();
  double step = ui->ledCenterStep->text().toDouble();

  double r0[3], r[3];
  r0[0] = MW->PMs->X(ipm);
  r0[1] = MW->PMs->Y(ipm);
  r0[2] = r[2] = MW->PMs->Z(ipm);
  QVector< QVector <double> > chi2;
  chi2.resize(steps*2+1);

  TH2D* h = new TH2D("centerTest","",steps*2+1,r0[0] - step*(steps+0.5),r0[0] + step*(steps+0.5),
                                     steps*2+1,r0[1] - step*(steps+0.5),r0[1] + step*(steps+0.5));
  double bestChi2=1e10;
  int best_ix=0, best_iy=0;

  for (int iy=-steps; iy<steps+1; iy++)
    for (int ix=-steps; ix<steps+1; ix++)
      {
        //shifting PM
        r[0] = r0[0] + ix*step;
        r[1] = r0[1] + iy*step;
        MW->PMs->at(ipm).setCoords(r);
        qDebug() << "New coords:"<<MW->PMs->X(ipm)<<MW->PMs->Y(ipm);

        ReconstructionWindow::on_pbReconstructAll_clicked();
        qDebug() << "----> chi2:"<<lastChi2;
        chi2[iy+steps].append(lastChi2);
        h->Fill(r[0],r[1],lastChi2);

        if (lastChi2 < bestChi2)
        {
            bestChi2 = lastChi2;
            best_ix = ix;
            best_iy = iy;
        }
      }

  MW->PMs->at(ipm).setCoords(r0); //returning back to original coordinates
  qDebug() << "---->Best Chi2 found at indexes:" << best_ix<<best_iy << "and chi2="<<bestChi2;
  *dx = best_ix*step;
  *dy = best_iy*step;

  MW->GraphWindow->Draw(h,"surf");
  qApp->processEvents();
}

double ReconstructionWindow::getSuggestedZ() const
{
  return ui->ledSuggestedZ->text().toDouble();
}

double ReconstructionWindow::isReconstructEnergy() const
{
  return ui->cbReconstructEnergy->isChecked();
}

void ReconstructionWindow::on_pbEvAllBruteThenShift_clicked()
{
    QVector <double> dX, dY;

    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
      {
        qDebug() << "-----------------------------\nPM#:" << ipm;
        double dx, dy;
        ReconstructionWindow::EvaluateShiftBrute(ipm, &dx, &dy);
        qDebug() << "shifts:"<<dX<<dY;
        dX.append(dx);
        dY.append(dy);
      }

    qDebug() << "Shifting all PMs";
    for (int ipm=0; ipm<MW->PMs->count(); ipm++)
      {
        double r[3];
        r[0] = MW->PMs->X(ipm) + dX[ipm];
        r[1] = MW->PMs->Y(ipm) + dY[ipm];
        r[2] = MW->PMs->Z(ipm);

        MW->PMs->at(ipm).setCoords(r);
      }
}

void ReconstructionWindow::on_pbSensorGainsToPMgains_clicked()
{
   if (!MW->SensLRF->isAllLRFsDefined())
     {
       message("LRFs are not defined!", this);
       return;
     }

   for (int ipm=0; ipm<MW->PMs->count(); ipm++)
     {
       double gain = MW->SensLRF->getSensor(ipm)->GetGain();
       MW->PMs->at(ipm).relGain = gain;
     }
   ReconstructionWindow::on_pbShowAllGainsForGroup_clicked();
}

void ReconstructionWindow::CheckAvailableEventData()
{
  EventsDataHub->clearReconstruction();
  ForbidUpdate = true; //--//
  ui->sbEventNumberInspect->setValue(0);
  ForbidUpdate = false; //--//

  if (EventsDataHub->Events.size() == 0)
    {
      ui->fNoData->setVisible(true);
      ReconstructionWindow::on_pbRefreshDataIndication_clicked();
      ReconstructionWindow::ShowStatistics();
      return;
    }

  ui->dLoadedEnergy->setEnabled(EventsDataHub->fLoadedEventsHaveEnergyInfo);
  if (!EventsDataHub->fLoadedEventsHaveEnergyInfo) ui->cbActivateLoadedEnergyFilter->setChecked(false);

  if (EventsDataHub->isScanEmpty()) ui->cbShowActualPosition->setChecked(false);
  ui->cbShowActualPosition->setEnabled(!EventsDataHub->isScanEmpty());

  ui->fAnalysisAreaScan->setEnabled(false);
  if (EventsDataHub->ScanNumberOfRuns>1) //means that scan is there (clearData sets it to 1)
    if (EventsDataHub->Scan.size() > EventsDataHub->ScanNumberOfRuns) //that is its not single point scan
      ui->fAnalysisAreaScan->setEnabled(true);

  bool fScanEmpty = EventsDataHub->isScanEmpty();
  if (fScanEmpty) ui->cbPlotVsActualPosition->setChecked(false);
  ui->cbPlotVsActualPosition->setEnabled(!fScanEmpty);

  bool fManifestEmpty = EventsDataHub->Manifest.isEmpty();
  if (fManifestEmpty) ui->cbShowManifestItems->setChecked(false);
  ui->cbShowManifestItems->setEnabled(!fManifestEmpty);

  //QIcon no;
  //ui->twData->tabBar()->setTabIcon(0, no);
  ui->fNoData->setVisible(false);
  MW->Owindow->SetCurrentEvent(0); //necessary?
  //do default population of EventsDataHub->ReconstructionData array and check all filters that can be checked before performing reconstruction
  ReconstructionWindow::on_pbUpdateStatusAllEvents_clicked();
  ReconstructionWindow::on_pbRefreshDataIndication_clicked();

  MW->WindowNavigator->setProgress(0);
  ui->prReconstruction->setValue(0);
}

void ReconstructionWindow::RefreshOnTimer(int Progress, double usPerEv)
{  
  ui->prReconstruction->setValue(Progress);
  MW->WindowNavigator->setProgress(Progress);
  if (usPerEv == 0) ui->leoMsPerEv->setText("n.a.");
  else ui->leoMsPerEv->setText(QString::number(usPerEv, 'g', 4));

  qApp->processEvents();
  if (ui->pbStopReconstruction->isChecked())
    {
      emit StopRequested();
      return;
    }
}

void ReconstructionWindow::writeReconOptionsToJson(QJsonObject &jsonMaster, bool fVerbose)
{
  QJsonObject RecJson;
  //general
  QJsonObject gjson;
  gjson["ReconstructEnergy"] = ui->cbReconstructEnergy->isChecked();
  gjson["InitialEnergy"] = ui->ledSuggestedEnergy->text().toDouble();
  bool fRecZ = ui->cb3Dreconstruction->isChecked();
  gjson["ReconstructZ"] = fRecZ;
  gjson["Zstrategy"] = ui->cobZ->currentIndex();
  gjson["InitialZ"] = ui->ledSuggestedZ->text().toDouble();
  gjson["IncludePassives"] = ui->cbIncludePassives->isChecked();
  gjson["WeightedChi2"] = ui->cbWeightedChi2->isChecked();
  gjson["NumThreads"] = ui->sbRecNumThreads->value();
  RecJson["General"] = gjson;

  //Algotithm
  QJsonObject ajson;
  int iAlg = ui->cobReconstructionAlgorithm->currentIndex(); //0-CoG, 1-CG cpu, 2-Root, 3-SNN, 4-CUDA
  ajson["Algorithm"] = iAlg;
    //always include cog
  QJsonObject cogjson;
  if (fRecZ) cogjson["ForceFixedZ"] = ui->cbForceCoGgiveZof->isChecked();
  bool fDoStretch = ui->cbCoGapplyStretch->isChecked();
  cogjson["DoStretch"] = fDoStretch;
  if (fDoStretch)
    {
      cogjson["StretchX"] = ui->ledCoGstretchX->text().toDouble();
      cogjson["StretchY"] = ui->ledCoGstretchY->text().toDouble();
      cogjson["StretchZ"] = ui->ledCoGstretchZ->text().toDouble();
    }
  bool fIgnore = ui->cbCogIgnoreLow->isChecked();
  cogjson["IgnoreLow"] = fIgnore;
  if (fIgnore) cogjson["IgnoreThreshold"] = ui->ledCoGignoreLevel->text().toDouble();
  bool fIgnoreFar = ui->cbCoGIgnoreFar->isChecked();
  cogjson["IgnoreFar"] = fIgnoreFar;
  if (fIgnoreFar) cogjson["IgnoreDistance"] = ui->ledCoGignoreThreshold->text().toDouble();
  ajson["CoGoptions"] = cogjson;
      //algorithm specific
  //0 (CoG) always included
  if (iAlg == 1 || fVerbose)
    { // CPU grids
      QJsonObject gcpuJson;
      gcpuJson["StartOption"] = ui->cobCGstartOption->currentIndex();
      gcpuJson["OptimizeWhat"] = ui->cobCGoptimizeWhat->currentIndex();
      gcpuJson["NodesXY"] = ui->sbCGnodes->value();
      gcpuJson["Iterations"] = ui->sbCGiter->value();
      gcpuJson["InitialStep"] = ui->ledCGstartStep->text().toDouble();
      gcpuJson["Reduction"] = ui->ledCGreduction->text().toDouble();
      bool fUseDynPass = ui->cbDynamicPassives->isChecked();
      gcpuJson["DynamicPassives"] = fUseDynPass;
      if (fUseDynPass) gcpuJson["PassiveType"] = ui->cobDynamicPassiveType->currentIndex();
      ajson["CPUgridsOptions"] = gcpuJson;
    }
  if (iAlg == 2 || fVerbose)
    { // Root minimizer
      QJsonObject rootJson;
      rootJson["StartOption"] = ui->cobLSstartingXY->currentIndex();
      rootJson["Minuit2Option"] = ui->cobMinuit2Option->currentIndex();
      rootJson["LSorLikelihood"] = ui->cobLSminimizeWhat->currentIndex();
      rootJson["StartStepX"] = ui->ledInitialStepX->text().toDouble();
      rootJson["StartStepY"] = ui->ledInitialStepY->text().toDouble();
      rootJson["StartStepZ"] = ui->ledInitialStepZ->text().toDouble();
      rootJson["StartStepEnergy"] = ui->ledInitialStepEnergy->text().toDouble();
      rootJson["MaxCalls"] = ui->sbLSmaxCalls->value();
      rootJson["LSsuppressConsole"] = ui->cbLSsuppressConsole->isChecked();
      bool fUseDynPass = ui->cbDynamicPassives->isChecked();
      rootJson["DynamicPassives"] = fUseDynPass;
      if (fUseDynPass) rootJson["PassiveType"] = ui->cobDynamicPassiveType->currentIndex();
      ajson["RootMinimizerOptions"] = rootJson;
    }
  if (iAlg == 3 || fVerbose)
    { // ANN
#ifdef ANTS_FANN
        QJsonObject fannJson;
        MW->NNwindow->saveJSON(fannJson);
        ajson["FANNsettings"] = fannJson;
#endif
    }
  if (iAlg == 4 || fVerbose)
    { // CUDA reconstruction options
      QJsonObject cudaJson;
      cudaJson["ThreadBlockXY"] = ui->sbCUDAthreadBlockSize->value();
      cudaJson["StartStep"] = ui->ledCUDAstartStep->text().toFloat();
      cudaJson["Iterations"] = ui->sbCUDAiterations->value();
      cudaJson["ScaleReduction"] = ui->ledCUDAscaleReductionFactor->text().toFloat();
      cudaJson["StartOption"] = ui->cobCUDAoffsetOption->currentIndex();
      cudaJson["StartX"] = ui->ledCUDAxoffset->text().toFloat();
      cudaJson["StartY"] = ui->ledCUDAyoffset->text().toFloat();
      cudaJson["OptimizeMLChi2"] = ui->cobCUDAminimizeWhat->currentIndex();
      cudaJson["Passives"] = ui->cbCUDApassives->isChecked();
      cudaJson["PassiveOption"] = ui->cobCUDApassiveOption->currentIndex();
      cudaJson["Threshold"] = ui->ledCUDAthreshold->text().toFloat();
      cudaJson["MaxDistance"] = ui->ledCUDAmaxDistance->text().toFloat();
      cudaJson["StarterZ"] = ui->ledSuggestedZ->text().toFloat();
      ajson["CGonCUDAsettings"] = cudaJson;
    }
  if (iAlg == 5 || fVerbose)
    { // kNN rec
#ifdef ANTS_FLANN
        QJsonObject kNNrecJson;
        kNNrecJson["numNeighbours"] = ui->sbKNNnumNeighbours->value();
        kNNrecJson["numTrees"] = ui->sbKNNnumTrees->value();
        kNNrecJson["calibScheme"] = ui->cobKNNcalibration->currentIndex(); //only gui
        kNNrecJson["useNeighbours"] = ui->sbKNNuseNeighboursInRec->value();
        kNNrecJson["weightMode"] = ui->cobKNNweighting->currentIndex();
        ajson["kNNrecSet"] = kNNrecJson;
#endif
    }
  RecJson["AlgorithmOptions"] = ajson;

    //Multiples
  QJsonObject multJson;
  multJson["Option"] = ui->cobMultipleOption->currentIndex();
  RecJson["MultipleEvents"] = multJson;  

    //Limit nodes
  if (ui->cbInRecActivate->isChecked())
    {
      QJsonObject lnJson;
      lnJson["Active"] = true;
      lnJson["Shape"] = ui->cobInRecShape->currentIndex();
      lnJson["Size1"] = ui->ledInRecSize1->text().toDouble();
      lnJson["Size2"] = ui->ledInRecSize2->text().toDouble();
      RecJson["LimitNodes"] = lnJson;
    }

  jsonMaster["ReconstructionOptions"] = RecJson;
}

bool ReconstructionWindow::readReconSettingsFromJson(QJsonObject &jsonMaster)
{
  if (!jsonMaster.contains("ReconstructionOptions"))
    {
      message("--- Json does not contain reconstruction options!");
      return false;
    }

  QJsonObject RecJson = jsonMaster["ReconstructionOptions"].toObject();

  //general
  QJsonObject gjson = RecJson["General"].toObject();
  JsonToCheckbox(gjson, "ReconstructEnergy", ui->cbReconstructEnergy);
  JsonToLineEdit(gjson, "InitialEnergy", ui->ledSuggestedEnergy);
  JsonToCheckbox(gjson, "ReconstructZ", ui->cb3Dreconstruction);
  JsonToLineEdit(gjson, "InitialZ", ui->ledSuggestedZ);
  ui->cobZ->setCurrentIndex(0); //compatibility
  JsonToComboBox(gjson, "Zstrategy", ui->cobZ);
  JsonToCheckbox(gjson, "IncludePassives", ui->cbIncludePassives);
  JsonToCheckbox(gjson, "WeightedChi2", ui->cbWeightedChi2);
  JsonToSpinBox(gjson, "NumThreads", ui->sbRecNumThreads);
  //Algotithm
  QJsonObject ajson = RecJson["AlgorithmOptions"].toObject();
  JsonToComboBox(ajson, "Algorithm", ui->cobReconstructionAlgorithm);//0-CoG, 1-CG cpu, 2-Root, 3-SNN, 4-CUDA
    //cog
  QJsonObject cogjson = ajson["CoGoptions"].toObject();
  JsonToCheckbox(cogjson, "ForceFixedZ", ui->cbForceCoGgiveZof);
  JsonToCheckbox(cogjson, "DoStretch", ui->cbCoGapplyStretch);
  JsonToLineEdit(cogjson, "StretchX", ui->ledCoGstretchX);
  JsonToLineEdit(cogjson, "StretchY", ui->ledCoGstretchY);
  JsonToLineEdit(cogjson, "StretchZ", ui->ledCoGstretchZ);
  JsonToCheckbox(cogjson, "IgnoreLow", ui->cbCogIgnoreLow);
  JsonToLineEdit(cogjson, "IgnoreThreshold", ui->ledCoGignoreLevel);
  ui->cbCoGIgnoreFar->setChecked(false); //compatibility
  JsonToCheckbox(cogjson, "IgnoreFar", ui->cbCoGIgnoreFar);
  JsonToLineEdit(cogjson, "IgnoreDistance", ui->ledCoGignoreThreshold);
    //MG
  QJsonObject gcpuJson = ajson["CPUgridsOptions"].toObject();
  JsonToComboBox(gcpuJson, "StartOption", ui->cobCGstartOption);
  JsonToComboBox(gcpuJson, "OptimizeWhat", ui->cobCGoptimizeWhat);
  JsonToSpinBox(gcpuJson, "NodesXY", ui->sbCGnodes);
  JsonToSpinBox(gcpuJson, "Iterations", ui->sbCGiter);
  JsonToLineEdit(gcpuJson, "InitialStep", ui->ledCGstartStep);
  JsonToLineEdit(gcpuJson, "Reduction", ui->ledCGreduction);
  JsonToCheckbox(gcpuJson, "DynamicPassives", ui->cbDynamicPassives);
  JsonToComboBox(gcpuJson, "PassiveType", ui->cobDynamicPassiveType);
    //Minuit
  QJsonObject rootJson = ajson["RootMinimizerOptions"].toObject();
  JsonToComboBox(rootJson, "StartOption", ui->cobLSstartingXY);
  JsonToComboBox(rootJson, "Minuit2Option", ui->cobMinuit2Option);
  JsonToComboBox(rootJson, "LSorLikelihood", ui->cobLSminimizeWhat);
  JsonToLineEdit(rootJson, "StartStepX", ui->ledInitialStepX);
  JsonToLineEdit(rootJson, "StartStepY", ui->ledInitialStepY);
  JsonToLineEdit(rootJson, "StartStepZ", ui->ledInitialStepZ);
  JsonToLineEdit(rootJson, "StartStepEnergy", ui->ledInitialStepEnergy);
  JsonToSpinBox(rootJson, "MaxCalls", ui->sbLSmaxCalls);
  JsonToCheckbox(rootJson, "LSsuppressConsole", ui->cbLSsuppressConsole);
  JsonToCheckbox(rootJson, "DynamicPassives", ui->cbDynamicPassives);
  JsonToComboBox(rootJson, "PassiveType", ui->cobDynamicPassiveType);
    //ANN
#ifdef ANTS_FANN
  QJsonObject fannJson = ajson["FANNsettings"].toObject();
  MW->NNwindow->saveJSON(fannJson);
#endif
    // CUDA
  QJsonObject cudaJson = ajson["CGonCUDAsettings"].toObject();
  JsonToSpinBox(cudaJson, "ThreadBlockXY", ui->sbCUDAthreadBlockSize);
  JsonToSpinBox(cudaJson, "Iterations", ui->sbCUDAiterations);
  JsonToLineEdit(cudaJson, "StartStep", ui->ledCUDAstartStep);
  JsonToLineEdit(cudaJson, "ScaleReduction", ui->ledCUDAscaleReductionFactor);
  JsonToLineEdit(cudaJson, "StartX", ui->ledCUDAxoffset);
  JsonToLineEdit(cudaJson, "StartY", ui->ledCUDAyoffset);
  JsonToLineEdit(cudaJson, "Threshold", ui->ledCUDAthreshold);
  JsonToLineEdit(cudaJson, "MaxDistance", ui->ledCUDAmaxDistance);
  JsonToComboBox(cudaJson, "StartOption", ui->cobCUDAoffsetOption);
  JsonToComboBox(cudaJson, "OptimizeMLChi2", ui->cobCUDAminimizeWhat);
  JsonToComboBox(cudaJson, "PassiveOption", ui->cobCUDApassiveOption);
  JsonToCheckbox(cudaJson, "Passives", ui->cbCUDApassives);
  //kNNrec
#ifdef ANTS_FLANN
  QJsonObject kNNrecJson = ajson["kNNrecSet"].toObject();
  JsonToSpinBox(kNNrecJson, "numNeighbours", ui->sbKNNnumNeighbours);
  JsonToSpinBox(kNNrecJson, "useNeighbours", ui->sbKNNuseNeighboursInRec);
  JsonToSpinBox(kNNrecJson, "numTrees", ui->sbKNNnumTrees);
  JsonToComboBox(kNNrecJson, "weightMode", ui->cobKNNweighting);
  JsonToComboBox(kNNrecJson, "calibScheme", ui->cobKNNcalibration); //only gui
#endif
    //Multiples
  QJsonObject multJson = RecJson["MultipleEvents"].toObject();
  JsonToComboBox(multJson, "Option", ui->cobMultipleOption);
    //Limit nodes
  //Limit nodes
  ui->cbInRecActivate->setChecked(false); //compatibility
  QJsonObject lnJson = RecJson["LimitNodes"].toObject();
  if (!lnJson.isEmpty())
  {
    JsonToCheckbox(lnJson, "Active", ui->cbInRecActivate);
    JsonToComboBox(lnJson, "Shape", ui->cobInRecShape);
    JsonToLineEdit(lnJson, "Size1", ui->ledInRecSize1);
    JsonToLineEdit(lnJson, "Size2", ui->ledInRecSize2);
  }

  return true;
}

void ReconstructionWindow::writeFilterSettingsToJson(QJsonObject &jsonMaster)
{
  QJsonObject FiltJson;

  //event number
  if (ui->cbFilterEventNumber->isChecked())
    {
      QJsonObject njson;
      njson["Min"] = ui->leiFromNE->text().toDouble();
      njson["Max"] = ui->leiToNE->text().toDouble();
      FiltJson["EventNumber"] = njson;
    }  

  //sum signal
  if (ui->cbFilterSumSignal->isChecked())
    {
      QJsonObject ssjson;
      ssjson["UseGains"] = ui->cbGainsConsideredInFiltering->isChecked();
      ssjson["UsePassives"] = ui->cbPassivePMsTakenAccount->isChecked();
      ssjson["Min"] = ui->ledFilterSumMin->text().toDouble();
      ssjson["Max"] = ui->ledFilterSumMax->text().toDouble();
      FiltJson["SumSignal"] = ssjson;
    }  

  //indi signal
  if (ui->cbFilterIndividualSignals->isChecked())
    {
      QJsonObject sjson; //cut offs are individual and are transferred through PMs json
      sjson["UsePassives"] = ui->cbPassivePMsTakenAccount->isChecked();
      FiltJson["IndividualPMSignal"] = sjson;
    }

  //reconstructed energy
  if (ui->cbActivateEnergyFilter->isChecked())
    {
      QJsonObject rejson;
      rejson["Min"] = ui->ledEnergyMin->text().toDouble();
      rejson["Max"] = ui->ledEnergyMax->text().toDouble();
      FiltJson["ReconstructedEnergy"] = rejson;
    }

  //loaded energy
  if (ui->cbActivateLoadedEnergyFilter->isChecked())
    {
      QJsonObject lejson;
      lejson["Min"] = ui->ledLoadEnergyMin->text().toDouble();
      lejson["Max"] = ui->ledLoadEnergyMax->text().toDouble();
      FiltJson["LoadedEnergy"] = lejson;
    } 

  //chi2 
  if (ui->cbActivateChi2Filter->isChecked())
    {
      QJsonObject cjson;
      cjson["Min"] = ui->ledChi2Min->text().toDouble();
      cjson["Max"] = ui->ledChi2Max->text().toDouble();
      FiltJson["Chi2"] = cjson;
    }  

  //spatial - primary scint
  if (ui->cbSpFlimitToPrimScint->isChecked())
    {
      QJsonObject spfPjson;
      FiltJson["SpatialToPrimary"] = spfPjson;
    }

  //spatial - custom
  if (ui->cbSpFcustom->isChecked())
    {
      QJsonObject spfCjson;
      spfCjson["RecOrScan"] = ui->cobSpFRecScan->currentIndex();
      int shape = ui->cobSpFXY->currentIndex();
      spfCjson["Shape"] = shape;
      switch (shape)
        {
        case 0: //square
          spfCjson["SizeX"] = ui->ledSpFsizeX->text().toDouble();
          spfCjson["SizeY"] = ui->ledSpFsizeY->text().toDouble();
          break;
        case 1: //round
          spfCjson["Diameter"] = ui->ledSpFdiameter->text().toDouble();
          break;
        case 2: //hexa
          spfCjson["Side"] = ui->ledSpFside->text().toDouble();
          spfCjson["Angle"] = ui->ledSpFangle->text().toDouble();
          break;
        case 3: //custom
          {
            QJsonArray polyarr;
            for (int i=0; i<polygon.size(); i++)
              {
                QJsonArray el;
                el.append( polygon[i].x() );
                el.append( polygon[i].y() );
                polyarr.append(el);
              }
            spfCjson["Polygon"] = polyarr;
            break;
          }
        default: qWarning() << "Unknown spatial filter shape!";
        }
      if (shape != 3)
        {
          spfCjson["X0"] = ui->ledSpF_X0->text().toDouble();
          spfCjson["Y0"] = ui->ledSpF_Y0->text().toDouble();
        }
      spfCjson["CutOutsideInside"] = ui->cobSpF_cutOutsideInside->currentIndex();
      bool allZ = ui->cbSpFallZ->isChecked();
      spfCjson["AllZ"] = allZ;
      if (!allZ)
        {
          spfCjson["Zfrom"] = ui->ledSpFfromZ->text().toDouble();
          spfCjson["Zto"] = ui->ledSpFtoZ->text().toDouble();
        }
      FiltJson["SpatialCustom"] = spfCjson;
    }

  //correlation
  if (ui->cbActivateCorrelationFilters->isChecked() && CorrelationFilters.size()>0)
    {
      QJsonObject corjson;
      QJsonArray jsonArray;
      for (int ifil=0; ifil<CorrelationFilters.size(); ifil++)
        {
          QJsonObject jsonCorrFilt;
          CorrelationFilters[ifil]->saveJSON(jsonCorrFilt);
          jsonArray.append(jsonCorrFilt);
        }
      corjson["Set"] = jsonArray;
      FiltJson["Correlation"] = corjson;
    }  

  //kNN
  if (ui->cbActivateNNfilter->isChecked())
    {
      QJsonObject knnjson;
      knnjson["Min"] = ui->ledNNmin->text().toDouble();
      knnjson["Max"] = ui->ledNNmax->text().toDouble();
      knnjson["AverageOver"] = ui->sbNNaverageOver->value();
      FiltJson["kNN"] = knnjson;
    }  

  //adding to master object
  jsonMaster["FilterOptions"] = FiltJson; 
}

bool ReconstructionWindow::readFilterSettingsFromJson(QJsonObject &jsonMaster)
{
  if (!jsonMaster.contains("FilterOptions"))
    {
      message("--- Json does not contain filter settings");
      return false;
    }

  bool fIgnore = TMPignore;
  TMPignore = true; //--//
  bool fOK = true;

  QJsonObject FiltJson = jsonMaster["FilterOptions"].toObject();

  bool flag = FiltJson.contains("EventNumber"); //event number
  ui->cbFilterEventNumber->setChecked(flag);
  if (flag)
    {
      QJsonObject njson = FiltJson["EventNumber"].toObject();
      JsonToLineEdit(njson, "Min", ui->leiFromNE);
      JsonToLineEdit(njson, "Max", ui->leiToNE);
    }

  flag = FiltJson.contains("SumSignal"); //sum signal
  ui->cbFilterSumSignal->setChecked(flag);
  if (flag)
    {
      QJsonObject ssjson = FiltJson["SumSignal"].toObject();
      JsonToCheckbox(ssjson, "UseGains", ui->cbGainsConsideredInFiltering);
      JsonToCheckbox(ssjson, "UsePassives", ui->cbPassivePMsTakenAccount);
      JsonToLineEdit(ssjson, "Min", ui->ledFilterSumMin);
      JsonToLineEdit(ssjson, "Max", ui->ledFilterSumMax);
    }

  flag = FiltJson.contains("IndividualPMSignal"); //indi signal
  ui->cbFilterIndividualSignals->setChecked(flag);
  if (flag)
    {
      QJsonObject sjson = FiltJson["IndividualPMSignal"].toObject();
      JsonToCheckbox(sjson, "UsePassives", ui->cbPassivePMsTakenAccount);
    }
  if (ui->sbPMsignalIndividual->value() == 0) on_sbPMsignalIndividual_valueChanged(0);
  else ui->sbPMsignalIndividual->setValue(0);

  flag = FiltJson.contains("ReconstructedEnergy"); //reconstructed energy
  ui->cbActivateEnergyFilter->setChecked(flag);
  if (flag)
    {
      QJsonObject rejson = FiltJson["ReconstructedEnergy"].toObject();
      JsonToLineEdit(rejson, "Min", ui->ledEnergyMin);
      JsonToLineEdit(rejson, "Max", ui->ledEnergyMax);
    }

  flag = FiltJson.contains("LoadedEnergy"); //loaded energy
  ui->cbActivateLoadedEnergyFilter->setChecked(flag);
  if (flag)
    {
      QJsonObject lejson = FiltJson["LoadedEnergy"].toObject();
      JsonToLineEdit(lejson, "Min", ui->ledLoadEnergyMin);
      JsonToLineEdit(lejson, "Max", ui->ledLoadEnergyMax);
    }

  flag = FiltJson.contains("Chi2"); //chi2
  ui->cbActivateChi2Filter->setChecked(flag);
  if (flag)
    {
      QJsonObject cjson = FiltJson["Chi2"].toObject();
      JsonToLineEdit(cjson, "Min", ui->ledChi2Min);
      JsonToLineEdit(cjson, "Max", ui->ledChi2Max);
    }

  flag = FiltJson.contains("SpatialToPrimary"); //spatial - primary scint
  ui->cbSpFlimitToPrimScint->setChecked(flag);

  flag = FiltJson.contains("SpatialCustom"); //spatial - custom
  ui->cbSpFcustom->setChecked(flag);
  if (flag)
    {      
      QJsonObject spfCjson = FiltJson["SpatialCustom"].toObject();
      ui->cobSpFRecScan->setCurrentIndex(0); //compatibility
      JsonToComboBox(spfCjson, "RecOrScan", ui->cobSpFRecScan);
      JsonToComboBox(spfCjson, "Shape", ui->cobSpFXY);
      JsonToLineEdit(spfCjson, "SizeX", ui->ledSpFsizeX);
      JsonToLineEdit(spfCjson, "SizeY", ui->ledSpFsizeY);
      JsonToLineEdit(spfCjson, "Diameter", ui->ledSpFdiameter);
      JsonToLineEdit(spfCjson, "Side", ui->ledSpFside);
      JsonToLineEdit(spfCjson, "Angle", ui->ledSpFangle);
      JsonToCheckbox(spfCjson, "AllZ", ui->cbSpFallZ);
      JsonToLineEdit(spfCjson, "Zfrom", ui->ledSpFfromZ);
      JsonToLineEdit(spfCjson, "Zto", ui->ledSpFtoZ);
      ui->ledSpF_X0->setText("0"); //compatibility
      JsonToLineEdit(spfCjson, "X0", ui->ledSpF_X0);
      ui->ledSpF_Y0->setText("0"); //compatibility
      JsonToLineEdit(spfCjson, "Y0", ui->ledSpF_Y0);
      ui->cobSpF_cutOutsideInside->setCurrentIndex(0); //compatibility
      JsonToComboBox(spfCjson, "CutOutsideInside", ui->cobSpF_cutOutsideInside);
      if (spfCjson.contains("Polygon"))
        {
          polygon.clear();
          QJsonArray polyarr = spfCjson["Polygon"].toArray();
          for (int i=0; i<polyarr.size(); i++)
            {
              double x = polyarr[i].toArray()[0].toDouble();
              double y = polyarr[i].toArray()[1].toDouble();
              polygon.append(QPointF(x, y));
            }
          if (polygon.size()<3)
            {
              message("Error loading polygon for custom spatial filter", this);
              fOK = false;
            }
        }          
    }

  flag = FiltJson.contains("Correlation"); //correlation
  ui->cbActivateCorrelationFilters->setChecked(flag);
  if (flag)
    {
      QJsonObject corjson = FiltJson["Correlation"].toObject();
      if (corjson.contains("Set"))
        {
          for (int i=0; i<CorrelationFilters.size(); i++) delete CorrelationFilters[i];
          CorrelationFilters.clear();
          QJsonArray jsonArray = corjson["Set"].toArray();
          if (jsonArray.isEmpty())
            {
              message("Correlation filters active, but json does not contain their config", this);
              fOK = false;
            }
          for (int i=0; i<jsonArray.size(); i++)
            {
              QJsonObject js = jsonArray[i].toObject();
              CorrelationFilterStructure* tmp = CorrelationFilterStructure::createFromJson(js, MW->Detector, EventsDataHub);
              if (tmp) CorrelationFilters.append(tmp);
              else
                {
                  message("Error in json config of correlation filter! Number is json array: "+QString::number(i), this);
                  fOK = false;
                }              
            }
        }
      if (ui->sbCorrFNumber->value() == 0) on_sbCorrFNumber_valueChanged(0);
      else ui->sbCorrFNumber->setValue(0);
      ui->pbCorrAccept->setIcon(QIcon());
    }

  //kNN
#ifdef ANTS_FLANN
  flag = FiltJson.contains("kNN");
  ui->cbActivateNNfilter->setChecked(flag);
  if (flag)
    {
      QJsonObject knnjson = FiltJson["kNN"].toObject();
      JsonToLineEdit(knnjson, "Min", ui->ledNNmin);
      JsonToLineEdit(knnjson, "Max", ui->ledNNmax);
      JsonToSpinBox(knnjson, "AverageOver", ui->sbNNaverageOver);
    }
#else
  ui->cbActivateNNfilter->setChecked(false);
#endif

  ReconstructionWindow::updateFiltersGui();
  TMPignore = fIgnore; //--//

  return fOK;
}

void ReconstructionWindow::writePMrelatedInfoToJson(QJsonObject &jsonMaster)
{
  QJsonObject pmJson;

  //min max signal cutoffs
  if (ui->cbFilterIndividualSignals->isChecked())
    {
      QJsonArray mima;
      for (int ipm=0; ipm<MW->PMs->count(); ipm++)
        {
          QJsonArray el;
          el.append(MW->PMs->at(ipm).cutOffMin);
          el.append(MW->PMs->at(ipm).cutOffMax);
          mima.append(el);
        }
      pmJson["CutOffs"] = mima;
    }

  //gathering info
  bool fAllGainsUnity = true;
  bool fAllActive = true;
  for (int ipm=0; ipm<MW->PMs->count(); ipm++)
    {
      if (MW->PMs->at(ipm).relGain != 1.0) fAllGainsUnity = false;
      if (MW->PMs->at(ipm).isStaticPassive()) fAllActive = false;
    }

   //gains
  QJsonArray gains;
  if (!fAllGainsUnity)
    {
      for (int ipm=0; ipm<MW->PMs->count(); ipm++) gains.append( MW->PMs->at(ipm).relGain );
      pmJson["Gains"] = gains;
    }


  //static passives
  if (!fAllActive)
    {
      QJsonArray passives;
      for (int ipm=0; ipm<MW->PMs->count(); ipm++) passives.append( MW->PMs->at(ipm).isStaticPassive() );
      pmJson["StaticPassives"] = passives;
    }

  //PM groups
  int numGroups = MW->PMs->countPMgroups();
  QJsonObject groups;
  groups["NumberGroups"] = numGroups;//0th group is default which is always present
  if (numGroups>1)
    {
      QJsonArray groupsArr;
      for (int igroup = 0; igroup<numGroups; igroup++)
          groupsArr.append(MW->PMs->getGroupDescription(igroup));
      groups["PMgroupDescriptions"] = groupsArr;

      QJsonArray groupAssignArr;
      for (int ipm=0; ipm<MW->PMs->count(); ipm++) groupAssignArr.append(MW->PMs->at(ipm).group);
      groups["PMgrouping"] = groupAssignArr;
    }
  if (ui->cbDynamicPassives->isChecked())
    {
      if (ui->cobDynamicPassiveType->currentIndex() == 0)
        {
          //threshold
          QJsonArray DynamicPassiveThresholds;
          for (int igroup = 0; igroup<numGroups; igroup++)
            DynamicPassiveThresholds.append(MW->PMs->getPassiveThreshold(igroup));
          groups["DynamicPassiveThreshold"] = DynamicPassiveThresholds;
        }
      else if (ui->cobDynamicPassiveType->currentIndex() == 1)
        {
          //range
          QJsonArray DynamicPassiveRanges;
          for (int igroup = 0; igroup<numGroups; igroup++)
            DynamicPassiveRanges.append(MW->PMs->getPassiveRange(igroup));
          groups["DynamicPassiveRanges"] = DynamicPassiveRanges;
        }
    }
  pmJson["Groups"] = groups;

  jsonMaster["PMrelatedSettings"] = pmJson;
}

bool ReconstructionWindow::readPMrelatedInfoFromJson(QJsonObject &jsonMaster)
{
  if (!jsonMaster.contains("PMrelatedSettings"))
    {
      message("--- Json does not contain PMrelated settings");
      return false;
    }

  QJsonObject pmJson = jsonMaster["PMrelatedSettings"].toObject();

  if (pmJson.contains("CutOffs"))
    {
      QJsonArray mima = pmJson["CutOffs"].toArray();
      if (mima.size() == MW->PMs->count())
        {
          for (int ipm=0; ipm<mima.size(); ipm++)
            {
              QJsonArray el = mima[ipm].toArray();
              double min = el[0].toDouble();
              double max = el[1].toDouble();
              MW->PMs->at(ipm).cutOffMin = min;
              MW->PMs->at(ipm).cutOffMax = max;
            }
        }
      else
        {
          message("Size mismatch: cuttof array and number of PMs", this);
          return false;
        }
    }

  if (pmJson.contains("Gains"))
    {
      QJsonArray gains = pmJson["Gains"].toArray();
      if (gains.size() == MW->PMs->count())
        {
          for (int ipm=0; ipm<gains.size(); ipm++)
              MW->PMs->at(ipm).relGain = gains[ipm].toDouble();
        }
      else
        {
          message("Size mismatch: gain array and number of PMs", this);
          return false;
        }
    }

  if (pmJson.contains("StaticPassives"))
    {
      QJsonArray passives = pmJson["StaticPassives"].toArray();
      if (passives.size() == MW->PMs->count())
        {
          for (int ipm=0; ipm<passives.size(); ipm++)
              if (passives[ipm].toBool()) MW->PMs->at(ipm).setStaticPassive();
        }
      else
        {
          message("Size mismatch: passives array and number of PMs", this);
          return false;
        }
    }

  //PM groups
  if (pmJson.contains("Groups"))
    {
      QJsonObject grJson = pmJson["Groups"].toObject();
      int numGroups = 1;
      parseJson(grJson, "NumberGroups", numGroups);
      if (numGroups == 1 && MW->PMs->countPMgroups() != 1)
        {
          //resetting to default
          MW->PMs->clearPMgroups(); //only one default group will be there, PMs are set to group 0
        }
      if (grJson.contains("PMgroupDescriptions"))
        {
          QJsonArray PMgroups = grJson["PMgroupDescriptions"].toArray();
          MW->PMs->clearPMgroups(); //only one default will be there
          for (int igroup=1; igroup<PMgroups.size(); igroup++)  //skipping the first default
              MW->PMs->definePMgroup(PMgroups[igroup].toString());
        }
      if (grJson.contains("PMgrouping"))
        {
          QJsonArray PMgrouping = grJson["PMgrouping"].toArray();
          if (PMgrouping.size() != MW->PMs->count())
              message("PM grouping array length is wrong!", this);
          else
            {
              for (int ipm=0; ipm<MW->PMs->count(); ipm++) MW->PMs->at(ipm).group = PMgrouping[ipm].toInt();
            }
        }

      if (grJson.contains("DynamicPassiveThreshold"))
        {
          QJsonArray DynamicPassiveThresholds = grJson["DynamicPassiveThreshold"].toArray();
          if (DynamicPassiveThresholds.size() != MW->PMs->countPMgroups())
              message("DynamicPassive threshold array length is wrong!", this);
          else
            {
              for (int igroup =0; igroup<MW->PMs->countPMgroups(); igroup++)
                MW->PMs->setPassiveThreshold(igroup, DynamicPassiveThresholds[igroup].toDouble() );
            }
        }

      if (grJson.contains("DynamicPassiveRanges"))
        {
          QJsonArray DynamicPassiveRanges = grJson["DynamicPassiveRanges"].toArray();
          if (DynamicPassiveRanges.size() != MW->PMs->countPMgroups())
              message("DynamicPassive ranges array length is wrong!", this);
          else
            {
              for (int igroup =0; igroup<MW->PMs->countPMgroups(); igroup++)
                MW->PMs->setPassiveRange(igroup, DynamicPassiveRanges[igroup].toDouble() );
            }
        }
    }

  //updating group indication
  ReconstructionWindow::on_pbUpdatePMgroupIndication_clicked();
  //update linked indication for PM gains
  ReconstructionWindow::on_pbUpdateGainsIndication_clicked();
  //update for dynamicThreshold
  QString str = MW->PMs->getGroupDescription(0);
  ui->cobPMGroupForDynPassives->setCurrentText(str);
  ReconstructionWindow::on_cobPMGroupForDynPassives_currentIndexChanged(str);

  return true;
}

/////////////////////////////////////////
void ReconstructionWindow::writeToJson(QJsonObject &json, bool fVerbose) //fVerbose - saving as config including LRFs, gui info and info on all algorithms
{
  QJsonObject js;
  ReconstructionWindow::writeReconOptionsToJson(js, fVerbose);
  ReconstructionWindow::writeFilterSettingsToJson(js);
  ReconstructionWindow::writePMrelatedInfoToJson(js);

  QJsonObject js1;
  js1["ReconstructionConfig"] = js;
  SaveJsonToFile(js, "ReconstructionConfig.json");

  //if saving configuration from gui, save additional info:
  if (fVerbose)
    {
      if (MW->SensLRF->isAllLRFsDefined())
        {
          QJsonObject LRFjson;
          int iCur = MW->SensLRF->getCurrentIterIndex();
          MW->SensLRF->saveIteration(iCur, LRFjson);
          js["ActiveLRF"] = LRFjson;
        }
        //Misc setting (scripts, PlotXY)
      MW->Rwindow->writeMiscGUIsettingsToJson(js);
    }
  json["ReconstructionConfig"] = js;  
}

bool ReconstructionWindow::readFromJson(QJsonObject &json)
{
  if (!json.contains("ReconstructionConfig"))
    {
      qDebug()<<"||| Json does not contain reconstruction settings";
      return false;
    }
  QJsonObject js = json["ReconstructionConfig"].toObject();
  //passives, gains, pm groups etc
  readPMrelatedInfoFromJson(js);
  //gen recons settings (algorithm)
  readReconSettingsFromJson(js);
  //event filter settings
  readFilterSettingsFromJson(js);  
  //scripts, PlotXY
  if (js.contains("GuiMisc")) readMiscGUIsettingsFromJson(js);
  //LRFs - has to be after LRF window load in order to configer the window according to loaded LRFs
  if (js.contains("ActiveLRF"))
    {
      QJsonObject lj = js["ActiveLRF"].toObject();
      MW->SensLRF->loadAll(lj);
      //update GUI
      MW->lrfwindow->updateIterationIndication();
    }
#ifdef ANTS_FANN
  if (js.contains("FANNsettings"))
    {
      QJsonObject fannJson = js["FANNsettings"].toObject();
      MW->NNwindow->loadJSON(fannJson);
    }
#endif
  return true;
}

void ReconstructionWindow::on_pbReconstructAll_clicked()
{
  ReconstructAll(true);
}

void ReconstructionWindow::ReconstructAll(bool fShow)
{
  MW->WindowNavigator->BusyOn();
  qApp->processEvents();

  QJsonObject json;
  ReconstructionWindow::writeToJson(json);

  ui->pbStopReconstruction->setEnabled(true);

  bool fOK = MW->ReconstructionManager->reconstructAll(json);
  if (!fOK)
  {
     QString err = MW->ReconstructionManager->getErrorString();
     if (err != "") message(err, this); //skip empty message for user interrupted
  }
  //else qDebug() << "Success!";

  if (fOK)
    {
       ui->prReconstruction->setValue(100);
       MW->WindowNavigator->setProgress(100);
       if (fShow && MW->GeometryWindow->isVisible()) ReconstructionWindow::ShowReconstructionPositions();
       MW->Owindow->RefreshData();
       ReconstructionWindow::ShowStatistics();
       double usPerEvent = MW->ReconstructionManager->getUsPerEvent();
       ui->leoMsPerEv->setText(QString::number(usPerEvent, 'g', 4));
    }
  else
    {
      CheckAvailableEventData(); //recon was already cleaned in manager
      ui->leoMsPerEv->setText("");
    }

  MW->WindowNavigator->BusyOff();

  MW->WindowNavigator->setProgress(100);
  ui->prReconstruction->setValue(100);
}

void ReconstructionWindow::SetShowFirst(bool fOn, int number)
{
  ui->cbLimitNumberEvents->setChecked(fOn);
  if (number != -1) ui->sbLimitNumberEvents->setValue(number);
}

void ReconstructionWindow::SetShowActual(bool fOn, bool fActualOnTop)
{
  ui->cbShowActualPosition->setChecked(fOn);
  ui->cbActualOnTop->setChecked(fActualOnTop);
}

void ReconstructionWindow::on_cbCustomBinning_toggled(bool checked)
{
   ui->fCustomBins->setEnabled(checked);
   if (!checked) ui->fCustomRanges->setEnabled(false);
}

void ReconstructionWindow::on_cbCustomRanges_toggled(bool checked)
{
   if (checked) ui->fCustomBins->setEnabled(true);
   ui->fCustomRanges->setEnabled(checked);
}

void linkCustomClass::updateBins() {bins = Qbins->value();}

void linkCustomClass::updateRanges() {from = Qfrom->text().toDouble(); to = Qto->text().toDouble();}

void TreeViewStruct::fillCustom(bool fCustomBins, bool fCustomRanges, QVector<linkCustomClass> &cus)
{
  this->fCustomBins = fCustomBins;
  this->fCustomRanges = fCustomRanges;
  if (cus.size()<3) return;

  for (int i=0; i<3; i++)
    {
      Bins[i]=cus[i].bins;
      From[i]=cus[i].from;
      To[i]=cus[i].to;
    }
}

void ReconstructionWindow::on_cbForceCoGgiveZof_toggled(bool /*checked*/)
{
   ui->fCoGStretchZ->setEnabled( ui->cb3Dreconstruction->isChecked() && !ui->cbForceCoGgiveZof->isChecked() );
}

bool ReconstructionWindow::isReconstructEnergy()
{
  return ui->cbReconstructEnergy->isChecked();
}

bool ReconstructionWindow::isReconstructZ()
{
  return ui->cb3Dreconstruction->isChecked();
}

double ReconstructionWindow::getSuggestedEnergy()
{
  return ui->ledSuggestedEnergy->text().toDouble();
}

double ReconstructionWindow::getSuggestedZ()
{
  return ui->ledSuggestedZ->text().toDouble();
}

void ReconstructionWindow::on_cobInRecShape_currentIndexChanged(int index)
{
   ui->fInRecSize2->setEnabled(index == 0);
   ui->lInRecX->setText( (index==0)?"Size X:":"Radius:" );
}

void ReconstructionWindow::on_pbCopyFromRec_clicked()
{
    ui->cbCustomBinning->setChecked(true);
    ui->cbCustomRanges->setChecked(true);

    ui->sbBins1->setValue(ui->sbXbins->value());
    ui->sbBins2->setValue(ui->sbYbins->value());

    ui->ledCustomFrom1->setText(ui->ledXfrom->text());
    ui->ledCustomTo1->setText(ui->ledXto->text());
    ui->ledCustomFrom2->setText(ui->ledYfrom->text());
    ui->ledCustomTo2->setText(ui->ledYto->text());
}

void ReconstructionWindow::on_pbAnalyzeChanPerPhEl_clicked()
{
  if (EventsDataHub->isEmpty()) return;
  if (!EventsDataHub->isReconstructionDataReady())
    {
      message("Reconstruction is not ready!", this);
      return;
    }
  if (!MW->Detector->SensLRF->isAllLRFsDefined())
    {
      message("LRFs are not ready!", this);
      return;
    }

  MW->WindowNavigator->setProgress(0);
  ui->prReconstruction->setValue(0);
  MW->WindowNavigator->BusyOn();
  qApp->processEvents();

  int numPMs = MW->Detector->PMs->count();
  for (int ipm=0; ipm<sigmaHists.size(); ipm++) delete sigmaHists[ipm];
  sigmaHists.clear();
  QVector<TH1D*> numberHists;
  //will be used to calculate sigma vs distance to PM center
  for (int ipm=0; ipm<numPMs; ipm++)
    {
      TString s = "sigma";
      s += ipm;
      TString n = "num";
      n += ipm;
      TH1D* tmp1 = new TH1D(s, "sigma", ui->pbBinsChansPerPhEl->value(), 0,0);
      sigmaHists.append(tmp1);
      TH1D* tmp2 = new TH1D(n, "number", ui->pbBinsChansPerPhEl->value(), 0,0);
      numberHists.append(tmp2);
    }

  //filling hists - go through all events, each will populate data all PMs
  for (int iev=0; iev<EventsDataHub->Events.size(); iev++)
    {
      if (iev%1000 == 0)
        {
          int ipr = 100.0*iev/EventsDataHub->Events.size();
          MW->WindowNavigator->setProgress(ipr);
          ui->prReconstruction->setValue(ipr);
        }
      if (!EventsDataHub->ReconstructionData[iev]->GoodEvent) continue; //respecting the filters

      double x = EventsDataHub->ReconstructionData[iev]->Points[0].r[0]; //only single events! !!!***
      double y = EventsDataHub->ReconstructionData[iev]->Points[0].r[1];
      double e = EventsDataHub->ReconstructionData[iev]->Points[0].energy;
      for (int ipm=0; ipm<numPMs; ipm++)
        {
           double x0 = MW->Detector->PMs->X(ipm);
           double y0 = MW->Detector->PMs->Y(ipm);
           double r = sqrt( (x-x0)*(x-x0) + (y-y0)*(y-y0) );
           if (r>15) continue;

           double AvSig = MW->Detector->SensLRF->getLRF(ipm, EventsDataHub->ReconstructionData[iev]->Points[0].r);
           double sig = EventsDataHub->Events[iev][ipm];
           double delta2 = sig/e - AvSig; //take energy into account
           delta2 *= delta2;

           sigmaHists[ipm]->Fill(AvSig, delta2);
           numberHists[ipm]->Fill(AvSig, 1.0);
        }
    }

  //calculating sigmas
  int MinEntries = 5;
  for (int ipm=0; ipm<numPMs; ipm++)
    {
      //qDebug() << sigmaHists[ipm]->GetXaxis()->GetNbins() << numberHists[ipm]->GetXaxis()->GetNbins();
      //*sigmaHists[ipm] = *sigmaHists[ipm] / *numberHists[ipm];
      for (int i=1; i<numberHists[ipm]->GetXaxis()->GetNbins()-1; i++) //ignoring under and over bins
        {
          if (numberHists[ipm]->GetBinContent(i) < MinEntries) sigmaHists[ipm]->SetBinContent(i, 0);
          else sigmaHists[ipm]->SetBinContent(i, sigmaHists[ipm]->GetBinContent(i)/numberHists[ipm]->GetBinContent(i));
        }
    }

  for (int ipm=0; ipm<numberHists.size(); ipm++) delete numberHists[ipm];
  numberHists.clear();

  MW->WindowNavigator->setProgress(100);
  ui->prReconstruction->setValue(100);
  MW->WindowNavigator->BusyOff(false);
}

void ReconstructionWindow::on_pbChanPerPhElShow_clicked()
{
  int ipm = ui->sbChanPerPhElPM->value();
  if (ipm> sigmaHists.size()-1)
    {
      message("Wrong PM number or data not ready!", this);
      return;
    }
  MW->GraphWindow->Draw(sigmaHists[ipm]);
}

void ReconstructionWindow::on_sbChanPerPhElPM_valueChanged(int arg1)
{
   if (arg1> MW->Detector->PMs->count()-1)
     {
       ui->sbChanPerPhElPM->setValue(0);
       return;
     }

   if (ChPerPhEl.isEmpty())
     {
       ui->ledChansPerPhEl->setText("");
       return;
     }
   ui->ledChansPerPhEl->setText( QString::number(ChPerPhEl[arg1], 'g', 3) );
   if (MW->GraphWindow->isVisible()) on_pbChanPerPhElShow_clicked();
}

void ReconstructionWindow::on_pbExtractChansPerPhEl_clicked()
{
  int ipm = ui->sbChanPerPhElPM->value();
  int numPMs = MW->Detector->PMs->count();

  if (sigmaHists.size()<numPMs)
    {
      message("There is no data - use 'Prepare data...' first!", this);
      return;
    }

  if (ChPerPhEl.size()<numPMs) ChPerPhEl.resize(numPMs);

  if (ui->cbExtractChPerPhElForAll->isChecked())
    {
      MW->Owindow->OutText("---------");
      MW->Owindow->OutText("#PM   Channels_per_pbhotoelectron");
      //doing for all PMs
      for (int ipm=0; ipm<numPMs; ipm++)
        {
          ChPerPhEl[ipm] = extractChPerPHEl(ipm);
          MW->Owindow->OutText(QString::number(ipm)+" "+QString::number(ChPerPhEl[ipm], 'g', 3));
        }
    }
  else
    {
      //doing only for one selected PM
      if (ipm > sigmaHists.size()-1)
        {
          message("Wrong PM number or data not ready!", this);
          return;
        }

      MW->GraphWindow->Draw(sigmaHists[ipm]);
      double ChanelsPerPhEl = extractChPerPHEl(ipm);
      MW->GraphWindow->UpdateRootCanvas();

      ChPerPhEl[ipm] = ChanelsPerPhEl;
    }

  if (ipm<ChPerPhEl.size()) ui->ledChansPerPhEl->setText( QString::number(ChPerPhEl[ipm], 'g', 3) );
  else ui->ledChansPerPhEl->setText("");
}

double ReconstructionWindow::extractChPerPHEl(int ipm)
{
  int numPMs = MW->Detector->PMs->count();
  if (ipm<0 || ipm>numPMs-1) return -1;

  double upperLim = ui->ledUpperLimitForChanPerPhEl->text().toDouble();
  TFitResultPtr fit = sigmaHists[ipm]->Fit("pol1", "S", "", 0, upperLim);
  double cutoff = fit->Value(0);
  double ChannelsPerPhEl = fit->Value(1);
  double ENF = ui->ledENFforChanPerPhEl->text().toDouble();
  if (ENF <= 0)
    {
      qWarning() << "ENF is zero or negative, ignoring!";
    }
  else ChannelsPerPhEl /= ENF;
  qDebug() << ipm << "> ChPerPhEl:"<<ChannelsPerPhEl<<" const:"<<cutoff;

  return ChannelsPerPhEl;
}

void ReconstructionWindow::on_pbGotoPreprocessAdjust_clicked()
{
    ui->tabWidget->setCurrentIndex(1);
    ui->twOptions->setCurrentIndex(3);
}

void ReconstructionWindow::on_cobSpFXY_currentIndexChanged(int index)
{
  ui->fSpF_Origin->setVisible(index!=3);
}

void ReconstructionWindow::on_cbReconstructEnergy_toggled(bool checked)
{
   ui->lReconstructEnergy->setText( checked ? "Initial energy:" : "Fixed energy:" );
}

void ReconstructionWindow::on_cobZ_currentIndexChanged(int index)
{
   QPalette p = ui->cobZ->palette();
   if (index == 0)
     {
       ui->ledSuggestedZ->setVisible(true);
       ui->label_8->setVisible(true);
       ui->cobZ->setGeometry(ui->cobZ->x(), ui->cobZ->y(), 106, ui->cobZ->height());
       // combo button
       p.setColor(QPalette::Button, Qt::white);
       p.setColor(QPalette::ButtonText, Qt::black);
       // combo menu
       //p.setColor(QPalette::Base, Qt::white);
       //p.setColor(QPalette::Text, Qt::red);
     }
   else
     {
       ui->ledSuggestedZ->setVisible(false);
       ui->label_8->setVisible(false);
       ui->cobZ->setGeometry(ui->cobZ->x(), ui->cobZ->y(), 161, ui->cobZ->height());
       p.setColor(QPalette::Button, Qt::white);
       p.setColor(QPalette::ButtonText, Qt::red);
     }
   ui->cobZ->setPalette(p);

   if (ui->cb3Dreconstruction->isChecked())
     {
       //3D - reconstruct Z
       ui->cobZ->setItemText(0, "Start from Z of:");
       ui->cobZ->setItemText(1, "Start from loaded/simulated Z");
     }
   else
     {
       //2D
       ui->cobZ->setItemText(0, "Set Z to:");
       ui->cobZ->setItemText(1, "Use loaded/simulated Z");
     }
}

void ReconstructionWindow::on_cbLimitNumberEvents_toggled(bool checked)
{
  if (checked) ui->lShowFirst->setText("<font color='red'>first</font>");
  else ui->lShowFirst->setText("first");
}

void ReconstructionWindow::on_sbXbins_editingFinished()
{
  if (ui->cbXYsymmetric->isChecked())
      ui->sbYbins->setValue(ui->sbXbins->value());
}

void ReconstructionWindow::on_cbXYsymmetric_toggled(bool checked)
{
  //disable is in signal/slot gui
  if (checked)
    {
      QString txt = ui->ledXto->text();
      ui->ledXfrom->setText("-"+txt);
      ui->ledYfrom->setText("-"+txt);
      ui->ledYto->setText(txt);
      ui->sbYbins->setValue(ui->sbXbins->value());
    }
}

void ReconstructionWindow::on_ledXto_editingFinished()
{
  if (ui->cbXYsymmetric->isChecked())
    {
      QString txt = ui->ledXto->text();
      ui->ledXfrom->setText("-"+txt);
      ui->ledYfrom->setText("-"+txt);
      ui->ledYto->setText(txt);
    }
}
